#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook
#
# This class is able to check whether a given cookbook file has any
# invalid entries.
#
# If an entry is not valid then this will be report to the user.
#
# The class expects the name of a .yml file, as string e. g. "htop.yml",
# as its main input.
#
# If an error is encountered via one of the various check-related
# methods, then :error should be returned.
#
# Usage example:
#
#   RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/checks_and_validations/check_for_invalid_entries_in_this_cookbook.rb'
# RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(ARGV)
# RBT.check_whether_this_yaml_file_has_two_url1_entries
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class CheckForInvalidEntriesInThisCookbook < RBT::Action # === RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook

  require 'rbt/aliases/aliases.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      check_these_entries = ARGV,
      run_already         = true,
      &block
    )
    reset
    set_commandline_arguments(
      check_these_entries
    )
    set_check_these_entries(
      check_these_entries
    )
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :do_not_run_yet
        run_already = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :main_counter
    # ======================================================================= #
    @internal_hash[:main_counter] = 1
    # ======================================================================= #
    # === :work_on_this_file
    #
    # This variable keeps track on which file we are currently working on.
    # ======================================================================= #
    @internal_hash[:work_on_this_file] = nil
    # ======================================================================= #
    # === :dataset
    #
    # This variable must be nil initially.
    # ======================================================================= #
    @internal_hash[:dataset] = nil
    # ======================================================================= #
    # === :first_line
    # ======================================================================= #
    @internal_hash[:first_line] = nil
  end

  # ========================================================================= #
  # === first_line?
  # ========================================================================= #
  def first_line?
    @internal_hash[:first_line]
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @internal_hash[:dataset]
  end

  # ========================================================================= #
  # === set_check_these_entries
  # ========================================================================= #
  def set_check_these_entries(i)
    @internal_hash[:check_these_entries] = [i].flatten.compact
    sanitize_the_main_entries
  end

  # ========================================================================= #
  # === all_fine
  # ========================================================================= #
  def all_fine
    mediumslateblue('      All fine. ')
  end

  # ========================================================================= #
  # === main_counter?
  # ========================================================================= #
  def main_counter?
    @internal_hash[:main_counter]
  end

  # ========================================================================= #
  # === return_the_main_counter_then_increment_it
  # ========================================================================= #
  def return_the_main_counter_then_increment_it
    result = "  #{rev}(#{royalblue(main_counter?)}#{rev})"
    @internal_hash[:main_counter] += 1
    return result
  end

  # ========================================================================= #
  # === check_whether_this_yaml_file_has_two_url1_entries
  #
  # This method will return a boolean value - true if the .yml file
  # has two url1 entries, and false otherwise.
  # ========================================================================= #
  def check_whether_this_yaml_file_has_two_url1_entries(
      i         = dataset?,
      this_file = nil
    )
    this_file = this_file.first if this_file.is_a? Array
    if this_file
      unless this_file.include? '/'
        this_file = File.absolute_path(this_file)
      end
    end
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         nil
      if File.exist? this_file
        i = @internal_hash[:dataset] = read_file_with_default_encoding(this_file)
      end
    end
    begin
      n_url1_entries = i.scan(/^ url1: /).size
    rescue ArgumentError => error
      e "#{rev}Error for: #{sfile(this_file)}"
      pp error
    end
    if n_url1_entries > 1
      opne swarn('There are too many')+
           steelblue(' url1: ')+
           swarn('entries in that file.')
      opne swarn('Please fix this issue, in the following file:')
      e
      e sfile("  #{this_file}")
      e
      return :error
    end
    return false
  end

  # ========================================================================= #
  # === check_these_entries?
  # ========================================================================= #
  def check_these_entries?
    @internal_hash[:check_these_entries]
  end


  # ========================================================================= #
  # === check_whether_there_is_more_than_one_program_name_and_program_version_entry
  # ========================================================================= #
  def check_whether_there_is_more_than_one_program_name_and_program_version_entry(
      i = dataset?
    )
    this_file = @internal_hash[:work_on_this_file]
    # ======================================================================= #
    # We can simply scan() here. The faulty entry may look like this:
    #
    #   program_name_and_program_version: mafft-7.467
    #   program_name_and_program_version: mafft-7.310
    #
    # ======================================================================= #
    scanned = i.scan(/ program_name_and_program_version: /)
    if scanned.size > 1
      opne swarn('There are too many')+
           steelblue(' program_name_and_program_version: ')+
           swarn('entries in that file.')
      opne swarn('Please fix this issue, in the following file:')
      e
      e sfile("  #{this_file}")
      e
      return :error
    end
  end

  # ========================================================================= #
  # === check_whether_all_left_entries_are_valid
  #
  # This method will check whether the left entries of the file are
  # all valid.
  # ========================================================================= #
  def check_whether_all_left_entries_are_valid
    # ======================================================================= #
    # Get a handle towards all valid cookbook-entries next. This will
    # include Strings such as "tags" or "url1" and so forth.
    # ======================================================================= #
    valid_entries = array_registered_and_valid_cookbook_entries?
    splitted = dataset?.split(N)
    these_entries_were_found = splitted.select {|entry|
      entry.start_with?(' ') and
     !entry.start_with?(' #') and
     !entry.start_with?(' - ') and
     !entry.start_with?('  ')
    }.map {|line|
      line.strip!
      if line.include? ':'
        line = line.split(':').first
      end
      line.strip!
      line
    }
    # ======================================================================= #
    # Next, check each of these entries:
    # ======================================================================= #
    these_entries_were_found.each {|check_this_entry|
      if valid_entries.include? check_this_entry
      else
        opne crimson('Not registered entry called `')+
             sfancy(check_this_entry)+
             crimson('`.')
      end
    }
  end

  # ========================================================================= #
  # === array_registered_and_valid_cookbook_entries?
  # ========================================================================= #
  def array_registered_and_valid_cookbook_entries?(
      i = file_specification_of_registered_cookbook_entries
    )
    YAML.load_file(i)
  end

  # ========================================================================= #
  # === check_if_the_first_entry_is_the_name_of_the_file
  # ========================================================================= #
  def check_if_the_first_entry_is_the_name_of_the_file(i)
    i = File.basename(i)
    name_of_the_file_without_extension = i.sub(/#{File.extname(i)}$/,'')
    if first_line? == name_of_the_file_without_extension
    else
      opnn { :no_colon}
      e
      e crimson('The name of the file is not the same as the filename.')
      e crimson('This should be corrected.')
      e
      e slateblue(first_line?)+
        lightgreen(' != ')+
        royalblue(name_of_the_file_without_extension)
      e
      return :error
    end
  end

  # ========================================================================= #
  # === sanitize_the_main_entries
  # ========================================================================= #
  def sanitize_the_main_entries
    case @internal_hash[:check_these_entries].first # case tag
    # ======================================================================= #
    # === --everything
    # ======================================================================= #
    when /^-?-?everything$/i
      @internal_hash[:check_these_entries] = available_programs?.map {|line| "#{line}.yml" }
    end
    # ======================================================================= #
    # Work through the .yml files next:
    # ======================================================================= #
    @internal_hash[:check_these_entries].map! {|entry|
      if entry.end_with? '.yml'
        unless File.exist? entry
          entry = return_location_to_this_programs_yaml_file(entry)
        end
      end
      entry
    }
  end

  # ========================================================================= #
  # === consider_to_exit_now
  # ========================================================================= #
  def consider_to_exit_now(i)
    case i
    # ======================================================================= #
    # === :error
    # ======================================================================= #
    when :error
      opne 'Please fix this error before we can continue.'
      exit
    end
  end

  # ========================================================================= #
  # === batch_process_the_entries
  # ========================================================================= #
  def batch_process_the_entries
    check_these_entries?.each {|this_cookbook_entry|
      if this_cookbook_entry.frozen?
        this_cookbook_entry = this_cookbook_entry.dup
      end
      unless this_cookbook_entry.end_with? '.yml'
        this_cookbook_entry << '.yml'
      end
      unless File.exist? this_cookbook_entry
        this_cookbook_entry = cookbooks_dir?+this_cookbook_entry
      end
      # ===================================================================== #
      # Read in the full dataset next.
      # ===================================================================== #
      @internal_hash[:work_on_this_file] = this_cookbook_entry
      @internal_hash[:dataset] = readlines_with_proper_encoding(this_cookbook_entry).reject {|line|
        line.start_with? '#'
      }.join(N)
      @internal_hash[:first_line] = @internal_hash[:dataset].split(N).first.strip.delete(':')
      if @internal_hash[:first_line].include? ' #'
        @internal_hash[:first_line] = @internal_hash[:first_line].split(' #').first.strip
      end
      # ===================================================================== #
      # We have to check for registered entries.
      # ===================================================================== #
      e "#{rev}Working on the file #{sfile(this_cookbook_entry)} #{rev}next."
      e
      e return_the_main_counter_then_increment_it+
        rev+' Checking whether '+
        steelblue('the first entry is the name of the file')+':'
      result = check_if_the_first_entry_is_the_name_of_the_file(this_cookbook_entry)
      consider_to_exit_now(result)
      e all_fine+steelblue(cheering_person)
      e return_the_main_counter_then_increment_it+
        rev+' Checking whether the yaml file has two url1 entries:'
      result = check_whether_this_yaml_file_has_two_url1_entries(dataset?, this_cookbook_entry)
      consider_to_exit_now(result)
      e "      #{mediumslateblue('All fine here. There are no two url1 entries in that file.')} "\
        "#{steelblue(cheering_person)}"
      e return_the_main_counter_then_increment_it+
        rev+' Checking whether all left entries are valid:'
      result = check_whether_all_left_entries_are_valid
      consider_to_exit_now(result)
      e all_fine+steelblue(cheering_person)
      e "#{return_the_main_counter_then_increment_it}"\
        "#{rev} Checking whether there is more than one "\
        "program_name_and_program_version entry:"
      result = check_whether_there_is_more_than_one_program_name_and_program_version_entry
      consider_to_exit_now(result)
      e "#{all_fine}#{steelblue(cheering_person)}"
      e
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    batch_process_the_entries
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::RawCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.does_this_yaml_file_have_two_url1_entries?
# =========================================================================== #
def self.does_this_yaml_file_have_two_url1_entries?(i)
  RBT.check_whether_this_yaml_file_has_two_url1_entries(i) == true
end

# =========================================================================== #
# === RBT.check_whether_this_yaml_file_has_two_url1_entries
#
# The argument to this method should be the local file path, e. g.
# "foobar.yml", as a String. Make sure that the file exists before
# calling this method here.
#
# Usage example:
#
#   RBT.check_whether_this_yaml_file_has_two_url1_entries('xcbutilerrors.yml')
#
# =========================================================================== #
def self.check_whether_this_yaml_file_has_two_url1_entries(
    i = ARGV
  )
  _ = RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(i) { :do_not_run_yet }
  return _.check_whether_this_yaml_file_has_two_url1_entries(:default, i)
end

# =========================================================================== #
# === RBT.check_for_invalid_entries_in_this_cookbook
#
# This method will also check for invalid entries in the given cookbook
# .yml file.
# =========================================================================== #
def self.check_for_invalid_entries_in_this_cookbook(
    i = ARGV,
    &block
  )
  return RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(i, :do_not_run_yet, &block)
end; self.instance_eval { alias check_for_invalid_entries_in_this_cookbook_file check_for_invalid_entries_in_this_cookbook } # === check_for_invalid_entries_in_this_cookbook_file

end

if __FILE__ == $PROGRAM_NAME
  require 'colours'
  require 'cliner'
  alias e puts
  RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(ARGV)
end # checkforinvalidentriesinthiscookbook
    # checkforinvalidentriesinthiscookbook htop
    # checkforinvalidentriesinthiscookbook mafft
    # checkforinvalidentriesinthiscookbook libebml.yml
    # checkforinvalidentriesinthiscookbook --everything
    # checkforinvalidentriesinthiscookbook xcbutilerrors