#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Actions::Configure
#
# This class will do all steps necessary for a configure-setup. This will
# work with GNU configure, cmake and ninja.
# =========================================================================== #
# require 'rbt/actions/individual_actions/configure/configure.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

module Actions

class Configure < RBT::Action # === RBT::Actions::Configure

  require 'rbt/compile_base/compile_base_module.rb'
  include RBT::CompileBaseModule

  require 'rbt/requires/require_sanitize_cookbook.rb'
  require 'rbt/toplevel_methods/configure_appdir_prefix.rb'
  require 'rbt/actions/individual_actions/infer_build_system/infer_build_system.rb'

  # ========================================================================= #
  # === REGEX_TO_CAPTURE_THE_PREFIX_FOR_CONFIGURE_BUILD_TYPES
  # ========================================================================= #
  REGEX_TO_CAPTURE_THE_PREFIX_FOR_CONFIGURE_BUILD_TYPES = /(--prefix=)([-A-Za-z\/]+)\s/

  # ========================================================================= #
  # === REGEX_TO_CAPTURE_THE_PREFIX_FOR_CMAKE_BUILD_TYPES
  # ========================================================================= #
  REGEX_TO_CAPTURE_THE_PREFIX_FOR_CMAKE_BUILD_TYPES = /-DCMAKE_INSTALL_PREFIX=([\/A-Za-z]+)/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset # Must come before set_commandline_arguments().
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_extra_verbose
      # ===================================================================== #
      when :be_extra_verbose
        @internal_hash[:be_extra_verbose] = true
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === :try_to_use_configure_options
    # ======================================================================= #
    @internal_hash[:try_to_use_configure_options] = true
    # ======================================================================= #
    # === :be_extra_verbose
    # ======================================================================= #
    @internal_hash[:be_extra_verbose] = false
    # ======================================================================= #
    # === :build_type_to_use
    # ======================================================================= #
    set_build_type_to_use(:configure)
  end

  # ========================================================================= #
  # === be_extra_verbose?
  # ========================================================================= #
  def be_extra_verbose?
    @internal_hash[:be_extra_verbose]
  end

  # ========================================================================= #
  # === build_type_to_use?
  # ========================================================================= #
  def build_type_to_use?
    @internal_hash[:build_type_to_use]
  end

  # ========================================================================= #
  # === set_build_type_to_use
  # ========================================================================= #
  def set_build_type_to_use(i)
    @internal_hash[:build_type_to_use] = i.to_sym # Always use Symblls here.
  end

  # ========================================================================= #
  # === build_type_is_configure?
  # ========================================================================= #
  def build_type_is_configure?
    build_type_to_use? == :configure
  end

  # ========================================================================= #
  # === build_type_is_cmake?
  # ========================================================================= #
  def build_type_is_cmake?
    build_type_to_use? == :cmake
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry|
        menu(entry)
      }
    else
      case i # case tag
      # ===================================================================== #
      # === action-configure --use-cmake
      # ===================================================================== #
      when /^-?-?use(-|_)?cmake$/
        set_build_type_to_use(:cmake)
      # ===================================================================== #
      # === action-configure --use-this-version=1.2.3 --statically
      # ===================================================================== #
      when /^-?-?statically$/,
           /^-?-?static$/
        do_compile_statically
      # ===================================================================== #
      # === action-configure --use-this-version=1.2.3
      # ===================================================================== #
      when /^-?-?use(-|_)?this(-|_)?version=(.+)$/ # === $3
        _ = $3.to_s
        e "#{rev}Modifying the prefix towards an "+
          steelblue('AppDir')+rev+' prefix next, using '+
          'the specified version '+steelblue(_)+'.'
        new_version = _
        new_prefix = RBT.return_appdir_prefix(@internal_hash[:program_name]+'-'+new_version)
        @internal_hash[:sys_command].sub!(
          REGEX_TO_CAPTURE_THE_PREFIX_FOR_CONFIGURE_BUILD_TYPES,
          '--prefix='+new_prefix.to_s+' '
        )
      # ===================================================================== #
      # === action-configure --appdir
      # ===================================================================== #
      when /^-?-?appdir/,
           /^-?-?ntrad/
        if build_type_is_configure?
          regex_to_use = REGEX_TO_CAPTURE_THE_PREFIX_FOR_CONFIGURE_BUILD_TYPES
        elsif build_type_is_cmake?
          regex_to_use = REGEX_TO_CAPTURE_THE_PREFIX_FOR_CMAKE_BUILD_TYPES
        end
        e rev+'Modifying the prefix towards an '+steelblue('AppDir')+
          "#{rev} prefix next."
        new_version = program_version?
        new_prefix = RBT.return_appdir_prefix(@internal_hash[:program_name]+'-'+new_version)
        if build_type_is_configure?
          @internal_hash[:sys_command].sub!(
            regex_to_use,
            "--prefix=#{new_prefix}"
          )
        elsif build_type_is_cmake?
          @internal_hash[:sys_command].sub!(
            regex_to_use,
            '-DCMAKE_INSTALL_PREFIX='+new_prefix.to_s
          )
        end
      end
    end
  end

  # ========================================================================= #
  # === do_compile_statically
  # ========================================================================= #
  def do_compile_statically(
      be_verbose = true
    )
    if be_verbose
      e "#{rev}Enabling static compilation mode next - at the least trying to."
    end
    ENV['CFLAGS'] = ENV['CFLAGS']+' -static -g' # This works as-is; determined in June 2023.
  end

  # ========================================================================= #
  # === try_to_use_configure_options?
  # ========================================================================= #
  def try_to_use_configure_options?
    @internal_hash[:try_to_use_configure_options]
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @internal_hash[:program_version]
  end

  # ========================================================================= #
  # === do_run_the_sys_command
  # ========================================================================= #
  def do_run_the_sys_command(
      i = @internal_hash[:sys_command] # This is our default here.
    )
    if be_extra_verbose?
      e "#{rev}The current working directory is `#{sdir(return_pwd)}#{rev}`."
    end
    coloured_and_padded_esystem(i) { :dodgerblue }
  end

  # ========================================================================= #
  # === automatically_try_to_determine_the_real_build_type
  # ========================================================================= #
  def automatically_try_to_determine_the_real_build_type
    build_system = RBT.infer_build_system
    case build_system
    # ======================================================================= #
    # === cmake
    # ======================================================================= #
    when 'cmake'
      set_build_type_to_use(:cmake)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    cwd = return_pwd
    work_on_this_program = File.basename(cwd)
    @internal_hash[:program_name]    = ProgramInformation.program_name(work_on_this_program)
    @internal_hash[:program_version] = ProgramInformation.program_version(work_on_this_program)
    cookbook = RBT.return_cookbook_for(@internal_hash[:program_name])
    automatically_try_to_determine_the_real_build_type
    case build_type_to_use?
    # ======================================================================= #
    # === :configure
    # ======================================================================= #
    when :configure,
         :default
      _ = './configure'.dup
      _ << ' --prefix=/usr/'
    # ======================================================================= #
    # === :cmake
    # ======================================================================= #
    when :cmake
      _ = 'cmake'.dup
      _ << ' -DCMAKE_INSTALL_PREFIX=/usr/'
    else
      ewarn "Unknown build type: #{steelblue(build_type_to_use?)}"
      exit
    end
    if try_to_use_configure_options?
      _ << " #{cookbook.configure_options?}"
      _.rstrip! # We have to do this in case the above returns an empty String.
    end
    # ======================================================================= #
    # Use a build directory in this event.
    # ======================================================================= #
    if cookbook.use_build_directory?
      name_of_the_build_directory = cookbook.name_of_the_build_directory?
      if name_of_the_build_directory.nil?
        name_of_the_build_directory = 'BUILD/'
      end
      mkdir(name_of_the_build_directory)
      cd(name_of_the_build_directory)
      if build_type_is_configure?
        _.prepend('.')
      elsif build_type_is_cmake?
        _ << (' ..')
      end
    end
    @internal_hash[:sys_command] = _
    menu(commandline_arguments?)
    do_run_the_sys_command(@internal_hash[:sys_command])
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Actions::Configure.new(ARGV) { :be_extra_verbose }
end # action_configure