#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::Blfs
#
# This class will display the BLFS entry, that is, the "Beyond Linux
# from Scratch" entry for a given program.
#
# Note that not every entry in the cookbooks will have such a BLFS entry;
# mostly just the more commonly available programs will have a BLFS entry.
#
# Usage example:
#
#   RBT::Action::Cookbooks::Blfs.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/blfs/blfs.rb'
# RBT::Action::Cookbooks::Blfs.new(ARGV)
# RBT.blfs(ARGV)
# x = RBT.return_all_blfs_entries; pp x;''
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class Blfs < RBT::Action # === RBT::Action::Cookbooks::Blfs

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    register_sigint
    reset # reset() must come before the other methods.
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_quiet_and_no_colours_and_do_not_tamper_with_the_xorg_buffer_and_do_not_open_in_the_browser
      # ===================================================================== #
      when :be_quiet_and_no_colours_and_do_not_tamper_with_the_xorg_buffer_and_do_not_open_in_the_browser
        set_do_not_be_verbose
        do_not_open_via_the_browser
        do_not_show_opnn
        disable_colours
        do_not_report_the_result
        do_not_make_use_of_the_xorg_buffer
      # ===================================================================== #
      # === :no_colours_be_quiet
      # ===================================================================== #
      when :no_colours_be_quiet
        disable_colours
        set_do_not_be_verbose
        do_not_show_opnn
        do_not_report_the_result
      # ===================================================================== #
      # === :be_quiet_and_no_colours_and_do_not_tamper_with_the_xorg_buffer
      # ===================================================================== #
      when :be_quiet_and_no_colours_and_do_not_tamper_with_the_xorg_buffer
        set_do_not_be_verbose
        do_not_show_opnn
        disable_colours
        do_not_report_the_result
        do_not_make_use_of_the_xorg_buffer
      # ===================================================================== #
      # === :be_quiet_and_do_not_tamper_with_the_xorg_buffer
      # ===================================================================== #
      when :be_quiet_and_do_not_tamper_with_the_xorg_buffer
        set_do_not_be_verbose
        do_not_show_opnn
        do_not_report_the_result
        do_not_make_use_of_the_xorg_buffer
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet
        set_be_quiet
      # ===================================================================== #
      # === :no_colours
      # ===================================================================== #
      when :no_colours,
           :disable_colours
        disable_colours
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :report_the_result
    # ======================================================================= #
    do_report_the_result
    # ======================================================================= #
    # === :work_on_these_programs
    #
    # This variable will be an Array of programs that will be
    # compiled/installed.
    # ======================================================================= #
    @internal_hash[:work_on_these_programs] = []
    # ======================================================================= #
    # === :program_name
    #
    # This variable must be nil initially.
    # ======================================================================= #
    @internal_hash[:program_name] = nil
    # ======================================================================= #
    # === :also_assign_the_xorg_buffer
    # ======================================================================= #
    @internal_hash[:also_assign_the_xorg_buffer] = true
    # ======================================================================= #
    # === :open_in_browser
    #
    # If this variable is true then this class will attempt to open 
    # ======================================================================= #
    @internal_hash[:open_in_browser] = true
    # ======================================================================= #
    # === :blfs_homepage
    #
    # This variable will keep a reference towards the BLFS homepage of
    # the corresponding program at hand.
    # ======================================================================= #
    @internal_hash[:blfs_homepage] = nil
    # ======================================================================= #
    # === :raw_cookbook
    #
    # We need an instance of raw-cookbooks, in order to find not only
    # the BLFS entry, but also make use of url1 and url2.
    # ======================================================================= #
    @internal_hash[:raw_cookbook] = action(:raw_cookbook) { :be_quiet_if_the_file_was_not_found }
    # ======================================================================= #
    # === :run_in_batch_mode
    #
    # If this is true then we will report all entries registered in the
    # RBT project.
    # ======================================================================= #
    @internal_hash[:run_in_batch_mode] = false
  end

  # ========================================================================= #
  # === sanitize_this_program_name
  #
  # This method will sanitize the program-name a little bit.
  # ========================================================================= #
  def sanitize_this_program_name(i)
    i = i.first if i.is_a? Array
    i = i.dup if i.frozen?
    i = i.to_s.dup.downcase # Downcase the input since as of September 2017.
    i = i[0, i.index('.')] if i.include? '.'
    i.chop! if i.end_with? '/'
    return i
  end

  # ========================================================================= #
  # === do_not_make_use_of_the_xorg_buffer
  # ========================================================================= #
  def do_not_make_use_of_the_xorg_buffer
    @internal_hash[:also_assign_the_xorg_buffer] = false
  end

  # ========================================================================= #
  # === run_in_batch_mode?
  # ========================================================================= #
  def run_in_batch_mode?
    @internal_hash[:run_in_batch_mode]
  end; alias in_batch_mode?          run_in_batch_mode? # === in_batch_mode?
       alias is_in_batchmode?        run_in_batch_mode? # === is_in_batchmode?
       alias do_we_run_in_batchmode? run_in_batch_mode? # === do_we_run_in_batchmode?

  # ========================================================================= #
  # === also_assign_the_xorg_buffer?
  # ========================================================================= #
  def also_assign_the_xorg_buffer?
    @internal_hash[:also_assign_the_xorg_buffer]
  end

  # ========================================================================= #
  # === program_name?
  #
  # This method will return the program we are searching for in this
  # class.
  #
  # In other words, this will be roughly equal to the input the user
  # provided to this class, although a bit sanitized (for instance,
  # a trailing '/' is chopped off).
  # ========================================================================= #
  def program_name?
    @internal_hash[:program_name]
  end; alias program?       program_name? # === program?
       alias which_program? program_name? # === which_program?

  # ========================================================================= #
  # === do_not_show_opnn
  # ========================================================================= #
  def do_not_show_opnn
    disable_opn
  end

  # ========================================================================= #
  # === report_the_result?
  # ========================================================================= #
  def report_the_result?
    @internal_hash[:report_the_result]
  end

  # ========================================================================= #
  # === do_report_the_result
  # ========================================================================= #
  def do_report_the_result
    @internal_hash[:report_the_result] = true
  end

  # ========================================================================= #
  # === do_not_report_the_result
  # ========================================================================= #
  def do_not_report_the_result
    @internal_hash[:report_the_result] = false
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    @internal_hash[:blfs_homepage] # Store the homepage here.
  end; alias blfs_homepage?    homepage? # === blfs_homepage?
       alias homepage          homepage? # === homepage
       alias remote_url?       homepage? # === remote_url?
       alias result_as_string? homepage? # === result_as_string?
       alias result?           homepage? # === result?

  # ========================================================================= #
  # === raw_cookbook?
  # ========================================================================= #
  def raw_cookbook?
    @internal_hash[:raw_cookbook]
  end

  # ========================================================================= #
  # === open_in_browser?
  # ========================================================================= #
  def open_in_browser?
    @internal_hash[:open_in_browser]
  end; alias shall_we_open_via_the_browser? open_in_browser? # === shall_we_open_via_the_browser?

  # ========================================================================= #
  # === set_work_on_these_programs
  #
  # This will actually assign to an Array. The method is not allowed to
  # modify the content of this Array.
  # ========================================================================= #
  def set_work_on_these_programs(
      i = non_hyphened_arguments?
    )
    i = [i].flatten.compact
    @internal_hash[:work_on_these_programs] = i
  end; alias set_seek_this_program           set_work_on_these_programs # === set_seek_this_program
       alias set_seek_these_programs         set_work_on_these_programs # === set_seek_these_programs
       alias set_seek_this_program_name      set_work_on_these_programs # === set_array_work_on_these_entries
       alias set_array_work_on_these_entries set_work_on_these_programs # === set_array_work_on_these_entries

  # ========================================================================= #
  # === work_on_these_programs?
  # ========================================================================= #
  def work_on_these_programs?
    @internal_hash[:work_on_these_programs]
  end; alias work_on_which_entries?       work_on_these_programs? # === work_on_which_entries?       
       alias array_work_on_these_entries? work_on_these_programs? # === array_work_on_these_entries?

  # ========================================================================= #
  # === do_run_in_batch_mode
  # ========================================================================= #
  def do_run_in_batch_mode
    @internal_hash[:run_in_batch_mode] = true
  end; alias run_in_batch_mode do_run_in_batch_mode # === run_in_batch_mode

  # ========================================================================= #
  # === do_not_open_in_the_browser
  # ========================================================================= #
  def do_not_open_in_the_browser
    @internal_hash[:open_in_browser] = false
  end; alias do_not_open_via_the_browser do_not_open_in_the_browser # === do_not_open_via_the_browser

  # ========================================================================= #
  # === report_the_program
  # ========================================================================= #
  def report_the_program(
      this_program = program_name?
    )
    opne lightblue(this_program)
  end

  # ========================================================================= #
  # === set_homepage
  # ========================================================================= #
  def set_homepage(i)
    if i and i.is_a? Array
      i = i.first
    end
    @internal_hash[:blfs_homepage] = i
  end; alias set_blfs_homepage set_homepage # === set_blfs_homepage

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    return raw_cookbook?.url2?
  end

  # ========================================================================= #
  # === report_that_no_blfs_entry_exists_for_the_program
  # ========================================================================= #
  def report_that_no_blfs_entry_exists_for_the_program(
      i    = program_name?,
      url2 = url2?
    )
    if be_verbose?
      if i and !i.empty? and !run_in_batch_mode?
        opne "#{rev}No #{teal('BLFS entry')} #{rev}is available "\
             "for `#{slategray(File.basename(i).delete_suffix('.yml'))}#{rev}`."
        if url2
          e
          opne "However had, as it may often be the case that you would like"
          opne "#{rev}to see any useful URL, the main download page "\
               "(#{steelblue('url2')}#{rev}) will be"
          opne "shown next:"
          e
          e "  #{steelblue(url2)}"
          e
        end
      end
    end
  end; alias report_no_entry_was_found_for_this_program report_that_no_blfs_entry_exists_for_the_program # === report_no_entry_was_found_for_this_program

  # ========================================================================= #
  # === report_this_BLFS_homepage
  # ========================================================================= #
  def report_this_BLFS_homepage(
      i                              = blfs_homepage?, # Defaults to @internal_hash[:blfs_homepage] as argument.
      also_assign_to_the_xorg_buffer = also_assign_the_xorg_buffer?
    )
    i = i.first if i.is_a? Array
    if be_verbose? and report_the_result?
      if run_in_batch_mode?
        e "#{lightgreen(program_name?)} #{orange(right_arrow?)} #{steelblue(i)}"
      else
        opnn { :no_trailing }
        e
        e
        e "#{rev}Opening"
        e
        e "  #{sfancy(homepage)}#{rev}"
        e
        e "#{rev}in the browser next."
      end
    end
    e
    if also_assign_to_the_xorg_buffer
      set_xorg_buffer(i)
    end
  end; alias report_homepage report_this_BLFS_homepage # === report_homepage

  # ========================================================================= #
  # === check_blfs_entry_for                                      (check tag)
  #
  # This method is normally called from within 
  # do_work_on_the_designated_programs().
  # ========================================================================= #
  def check_blfs_entry_for(
      this_program
    )
    set_blfs_homepage(nil) # Reset it here.
    set_program_name(nil)  # Reset this as well.
    _ = raw_cookbook?
    this_program = this_program.to_s.dup
    this_program.delete!('-') if this_program.include? '-'
    set_program_name(this_program) # Always keep a reference here.
    _result_of_the_search = _.try_to_find(which_program?) # This variable will normally be a Hash.
    if _.has_non_empty_BLFS_entry?
      set_blfs_homepage(_.blfs_homepage?)
      homepage = homepage? # Get a reference, for easier working with this variable.
      # report_the_program(this_program)
      if report_the_result?
        report_this_BLFS_homepage(homepage)
      end
      if homepage and !homepage.empty?
        if shall_we_open_via_the_browser? and !in_batch_mode?
          # ================================================================= #
          # Since as of January 2023, the homepage will be opened on my
          # home system whenever we report the BLFS-homepage. This will,
          # however had, NOT be the case when we run in batch-mode.
          # ================================================================= #
          open_in_browser(homepage)
        end
      end
    else
      report_that_no_blfs_entry_exists_for_the_program(this_program)
    end
    _.reset_the_internal_state # And reset RawCookbook again.
  end

  # ========================================================================= #
  # === do_work_on_the_designated_programs
  # ========================================================================= #
  def do_work_on_the_designated_programs(
      i = work_on_these_programs?
    )
    i.each {|this_program|
      check_blfs_entry_for(this_program)
    }
  end

  # ========================================================================= #
  # === all_blfs_entries
  # ========================================================================= #
  def all_blfs_entries
    RBT.return_all_blfs_entries
  end; alias all_programs_with_a_registered_BLFS_entry all_blfs_entries # === all_programs_with_a_registered_BLFS_entry

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= # 
  def menu(
      i = hyphened_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # (case tag)
      # ===================================================================== #
      # === Operate in batch-mode                                 (batch tag)
      #
      # Invocation example:
      #
      #   blfs --batch
      #
      # ===================================================================== #
      when /^-?-?batch/i,
           /^-?-?everything/i
        opne "#{rev}#{steelblue('Running in batch-mode, as instructed.')}"
        do_run_in_batch_mode
      end
    end
  end

  # ========================================================================= #
  # === set_program_name
  #
  # We seek this program through the method here.
  #
  # The method will also handle aliases since as of June 2020.
  # ========================================================================= #
  def set_program_name(
      i = first_argument?
    )
    if i
      i = sanitize_this_program_name(i)
      if RBT.is_included?(i, false)
        # ===================================================================== #
        # In this case all is fine. We'll assign the program-name past this
        # clause.
        # ===================================================================== #
      elsif RBT.is_included?(i, true)
        old_name = i.dup
        # ===================================================================== #
        # === Handle aliases next
        #
        # It is an alias, so find the real name.
        # ===================================================================== #
        i = RBT.find_cookbook_alias_for(i)
        opne "#{rev}This name (#{forestgreen(old_name)}"\
             "#{rev}) was not found, but a replacement name ("\
             "#{steelblue(i)}#{rev})"
        opne "#{rev}was found and will subsequently be used instead."
        e
      end
    end
    @internal_hash[:program_name] = i
  end; alias set_program set_program_name # === set_program
       alias set_input   set_program_name # === set_input

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # By default all non-hyphened commandline arguments will constitute
    # the array of programs this class will operate on. Note that
    # --batch will overrule this anyway, if provided on the commandline.
    # ======================================================================= #
    _ = non_hyphened_arguments?
    # ======================================================================= #
    # Next we enter menu():
    # ======================================================================= #
    menu
    if do_we_run_in_batchmode?
      _ = all_programs_with_a_registered_BLFS_entry
    end
    if _.empty? and !is_in_batchmode?
      opne "#{rev}Please supply at the least one argument to this class."
    else
      set_work_on_these_programs(_)
      do_work_on_the_designated_programs
    end
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::Blfs[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.blfs_instance
#
# This method will guarantee that a new instance of class Blfs is returned.
# =========================================================================== #
def self.blfs_instance(
    i = ARGV,
    &block
  )
  if block_given?
    return RBT::Action::Cookbooks::Blfs.new(i, &block)
  else
    return RBT::Action::Cookbooks::Blfs.new(i) # { :no_colours_be_quiet }
  end
end

# =========================================================================== #
# === RBT.array_all_BLFS_entries

# This method will return an Array of all BLFS entries. It will not hold
# any specific information about the programs that hold these BLFS
# entries.
# =========================================================================== #
def self.array_all_BLFS_entries
  array = []
  all_yaml_files = Dir["#{cookbook_directory?}*.yml"]
  content_of_all_yaml_files = all_yaml_files.map {|entry|
    "#{File.read(entry)}\n"
  }.join("\n")
  array = content_of_all_yaml_files.scan(
    /^ blfs:\n - (.+)$/ # Grab the relevant entries here.
  ).flatten
  return array
end

# =========================================================================== #
# === RBT.array_all_BLFS_entries_attributed
#
# This method will (silently) return an Array with all entries that
# have the BLFS tag.
#
# This method is similar to RBT.array_all_BLFS_entries, but it will also
# return all BLFS entries associated with the corresponding program name.
#
# This method used grep in the past, but will use Dir[] and .scan()
# since as of 21.05.2020 instead. This should make it work better on
# windows specifically, for instance.
#
# The result will look like this:
#
#   ["libpcap",
#    "https://www.linuxfromscratch.org/blfs/view/svn/basicnet/libpcap.html"],
#   ["xulrunner",
#    "https://www.linuxfromscratch.org/blfs/view/7.5/x/xulrunner.html"],
#
# Usage example:
#
#   x = RBT.return_all_blfs_entries; pp x; ''
#
# =========================================================================== #
def self.array_all_BLFS_entries_attributed
  array = []
  all_yaml_files = Dir["#{cookbook_directory?}*.yml"]
  content_of_all_yaml_files = all_yaml_files.map {|entry|
    "#{File.read(entry)}\n"
  }.join("\n")
  array = content_of_all_yaml_files.scan(
    /^(.+):\n^ blfs:\n - (.+)$/
  )
  return array
end; self.instance_eval { alias return_all_blfs_entries array_all_BLFS_entries_attributed } # === RBT.return_all_blfs_entries

# =========================================================================== #
# === RBT.n_BLFS_entries?
#
# This method will return a number, indicating how many BLFS entries are
# registered in the BLFS project.
# =========================================================================== #
def self.n_BLFS_entries?
  return array_all_BLFS_entries.size
end

# =========================================================================== #
# === RBT.return_blfs_page_of_this_program
#
# This method will either return the BLFS page of the given program,
# as a String - or nil otherwise.
#
# If a String is returned then we will assume that this is to the
# remote BLFS page entry of a (locally registered) program.
#
# Returns:
#
#   - either a string
#   - or nil
#
# Invocation examples:
#
#   x = RBT.return_blfs_page_of_this_program('gcc')
#   x = RBT.blfs(:openssl) # => http://www.linuxfromscratch.org/lfs/view/development/chapter06/openssl.html
#   x = RBT.blfs(:zvbi)    # => nil
#
# =========================================================================== #
def self.return_blfs_page_of_this_program(
    this_program = 'gcc'
  )
  _ = RBT.blfs_instance(this_program) {
    :be_quiet_and_no_colours_and_do_not_tamper_with_the_xorg_buffer_and_do_not_open_in_the_browser
  }
  result = _.remote_url?
  result = result.first if result.is_a? Array
  return result
end; self.instance_eval { alias return_blfs_entry_for return_blfs_page_of_this_program } # === RBT.return_blfs_entry_for
     self.instance_eval { alias blfs                  return_blfs_page_of_this_program } # === RBT.blfs

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e
  e 'Testing class RBT::Action::Cookbooks::Blfs next:'
  e
  _blfs_instance = RBT::Action::Cookbooks::Blfs.new(ARGV)
  array = RBT.array_all_BLFS_entries
  pp array
  pp array.size
  pp RBT.array_all_BLFS_entries_attributed
end # blfs ruby
    # blfs htop
    # blfs php