#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::RawCookbook
#
# This class essentially handles a raw cookbook instance. Such a raw
# cookbook may have special methods, in particular simplified
# getter-methods, but it may NEVER modify the original dataset.
#
# If you have a use case where you have to modify the original
# dataset, you should use a more feature-complete class, such
# as class RBT::Action::Cookbooks::SanitizeCookbook instead.
#
# Since as of April 2021, this class can load either the "raw", unmodified 
# yaml file, or the yaml file available from the expanded cookbooks. The 
# latter will be much faster, so if speed is of importance, consider
# using the expanded cookbook dataset rather than the "raw", unmodified
# yaml file. The raw dataset is more accurate and "up-to-date", though,
# as any changes done to it will be instantly available, whereas for
# the expanded cookbooks we first have to expand them into their final,
# and thus correct, format.
#
# Usage example:
#
#   RBT::Action::Cookbooks::RawCookbook.new(ARGV)
#
# =========================================================================== #
#   require 'rbt/actions/individual_actions/cookbooks/raw_cookbook/raw_cookbook.rb'
#
# or
#
#   require 'rbt/requires/require_the_raw_cookbook.rb'
#
# RBT::Action::Cookbooks::RawCookbook.new(ARGV)
# RBT.raw_cookbook(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class RawCookbook < RBT::Action # === RBT::Action::Cookbooks::RawCookbook

  # ========================================================================= #
  # === DIRECTORY_EXPANDED_COOKBOOKS
  #
  # Easier toplevel-constant to refer to the expanded_cookbooks/
  # directory.
  # ========================================================================= #
  DIRECTORY_EXPANDED_COOKBOOKS = RBT.directory_yaml_expanded_cookbooks

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    # ======================================================================= #
    # === Handle blocks given to this class next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      # ===================================================================== #
      # === :simple
      # ===================================================================== #
      when :simple,
           :simple_cookbook
        set_load_what(:simple_cookbook)
      # ===================================================================== #
      # === :use_expanded_dataset
      #
      # This variant is significantly faster than the :simple variant.
      # ===================================================================== #
      when :use_expanded_dataset,
           :use_the_expanded_dataset,
           :fast,
           :complex
        set_complex
      # ===================================================================== #
      # === :be_quiet_if_the_file_was_not_found
      # ===================================================================== #
      when :be_quiet_if_the_file_was_not_found
        @internal_hash[:be_quiet_if_the_file_was_not_found] = true
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    reset_the_internal_variables
    infer_the_namespace
    set_the_default_value_for_the_cookbook_variant
  end

  # ========================================================================= #
  # === set_the_default_value_for_the_cookbook_variant
  # ========================================================================= #
  def set_the_default_value_for_the_cookbook_variant
    # ======================================================================= #
    # === :load_simple_or_expanded_cookbook_variant
    #
    # The next variable will determine whether we will load from the
    # expanded cookbook-dataset or from a raw, simple .yml file. The
    # latter may contain less data. Ideally we should load from the
    # expanded_cookbook, if possible and available, as it is faster.
    #
    # The two allowed states for this variable are:
    #
    #   :simple_cookbook
    #   :expanded_cookbook
    #
    # ======================================================================= #
    # @internal_hash[:load_simple_or_expanded_cookbook_variant] = :simple_cookbook
    @internal_hash[:load_simple_or_expanded_cookbook_variant] = :expanded_cookbook
  end

  # ========================================================================= #
  # === reset_the_internal_variables
  # ========================================================================= #
  def reset_the_internal_variables
    # ======================================================================= #
    # === :shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file
    # ======================================================================= #
    @internal_hash[:shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file] = false
    if is_on_roebe?
      @internal_hash[:shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file] = true
    end
    # ======================================================================= #
    # === :is_the_program_included
    #
    # This variable will tell us whether the program is included or
    # whether it is not included. By default it is assumed that
    # the program is NOT included, hence the false value.
    # ======================================================================= #
    @internal_hash[:is_the_program_included] = false
    # ======================================================================= #
    # === :dataset_from_the_yaml_file
    #
    # We will store all data into the following variable. The data
    # comes from a .yml file typically.
    #
    # Initialize the variable to an empty Hash.
    # ======================================================================= #
    @internal_hash[:dataset_from_the_yaml_file] = {}
    # ======================================================================= #
    # === :be_quiet_if_the_file_was_not_found
    # ======================================================================= #
    @internal_hash[:be_quiet_if_the_file_was_not_found] = false
    # ======================================================================= #
    # === :absolute_path_to_the_cookbook_file
    #
    # This variable will store from which location the dataset was read.
    # ======================================================================= #
    @internal_hash[:absolute_path_to_the_cookbook_file] = nil
  end; alias reset_the_internal_state reset_the_internal_variables # === reset_the_internal_state

  # ========================================================================= #
  # === shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file?
  # ========================================================================= #
  def shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file?
    @internal_hash[:shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file]
  end

  # ========================================================================= #
  # === do_not_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file
  # ========================================================================= #
  def do_not_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file
    @internal_hash[:shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file] = false
  end; alias do_not_autogenerate do_not_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file # === do_not_autogenerate

  # ========================================================================= #
  # === set_complex
  # ========================================================================= #
  def set_complex
    reset_the_internal_variables
    set_load_what(:expanded_cookbook)
    load_up_the_dataset
  end

  # ========================================================================= #
  # === set_autogenerate_expanded_dataset
  # ========================================================================= #
  def set_autogenerate_expanded_dataset(i)
    @internal_hash[:shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file] = i
  end

  # ========================================================================= #
  # === set_simple
  # ========================================================================= #
  def set_simple
    reset_the_internal_variables
    set_load_what(:simple_cookbook)
    load_up_the_dataset
  end

  # ========================================================================= #
  # === set_absolute_path_to_the_cookbook_file
  # ========================================================================= #
  def set_absolute_path_to_the_cookbook_file(i)
    @internal_hash[:absolute_path_to_the_cookbook_file] = i
  end

  # ========================================================================= #
  # === be_quiet_if_the_file_was_not_found?
  # ========================================================================= #
  def be_quiet_if_the_file_was_not_found?
    @internal_hash[:be_quiet_if_the_file_was_not_found]
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e 'Available options:'
    e
    eparse '  --complex # use the expanded dataset rather '\
           'than the simple .yml file'
    e
  end

  # ========================================================================= #
  # === set_load_simple_or_expanded_cookbook_variant
  # ========================================================================= #
  def set_load_simple_or_expanded_cookbook_variant(
      i = :expanded_cookbook
    )
    @internal_hash[:load_simple_or_expanded_cookbook_variant] = i
  end; alias set_load_what set_load_simple_or_expanded_cookbook_variant # === set_load_what
       alias set_mode      set_load_simple_or_expanded_cookbook_variant # === set_mode

  # ========================================================================= #
  # === set_search_for_this_program
  # ========================================================================= #
  def set_search_for_this_program(
      i = first_argument?
    )
    case i
    # ======================================================================= #
    # === :default
    #
    # This means to obtain the first argument given to this class.
    # ======================================================================= #
    when :default,
         :first_argument
      i = first_argument?
    end
    @internal_hash[:search_for_this_program] = i
  end

  # ========================================================================= #
  # === search_for_this_program?
  # ========================================================================= #
  def search_for_this_program?
    @internal_hash[:search_for_this_program]
  end

  # ========================================================================= #
  # === the_program_is_not_included
  # ========================================================================= #
  def the_program_is_not_included
    @internal_hash[:is_the_program_included] = false
  end

  # ========================================================================= #
  # === the_program_is_included
  # ========================================================================= #
  def the_program_is_included
    @internal_hash[:is_the_program_included] = true
  end

  # ========================================================================= #
  # === is_the_program_included?
  # ========================================================================= #
  def is_the_program_included?
    @internal_hash[:is_the_program_included]
  end

  # ========================================================================= #
  # === directory_expanded_cookbooks?
  # ========================================================================= #
  def directory_expanded_cookbooks?
    DIRECTORY_EXPANDED_COOKBOOKS
  end

  # ========================================================================= #
  # === dataset?
  #
  # Query method over the main dataset - the most important dataset for
  # this class.
  # ========================================================================= #
  def dataset?
    @internal_hash[:dataset_from_the_yaml_file]
  end; alias raw_data? dataset? # === raw_data?
       alias hash?     dataset? # === hash?
       alias dataset   dataset? # === dataset

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === tags?
  # ========================================================================= #
  def tags?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === licence?
  # ========================================================================= #
  def licence?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === load_what?
  # ========================================================================= #
  def load_what?
    @internal_hash[:load_simple_or_expanded_cookbook_variant]
  end; alias mode? load_what? # === mode?

  # ========================================================================= #
  # === toggle_the_state                                         (toggle tag)
  # ========================================================================= #
  def toggle_the_state(
      i = load_what?
    )
    case i
    # ======================================================================= #
    # === :expanded_cookbook
    # ======================================================================= #
    when :expanded_cookbook
      @internal_hash[:load_simple_or_expanded_cookbook_variant] = :simple_cookbook
    else
      @internal_hash[:load_simple_or_expanded_cookbook_variant] = :expanded_cookbook
    end
  end; alias toggle toggle_the_state # === toggle

  # ========================================================================= #
  # === reset_the_main_hash
  # ========================================================================= #
  def reset_the_main_hash
    @internal_hash[:dataset_from_the_yaml_file] = {}
  end; alias clear    reset_the_main_hash # === clear
       alias unfreeze reset_the_main_hash # === unfreeze

  # ========================================================================= #
  # === extra_information?
  # ========================================================================= #
  def extra_information?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === libraries?
  # ========================================================================= #
  def libraries?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === required_deps_on?
  # ========================================================================= #
  def required_deps_on?
    obtain(__method__.to_s.delete('?'))
  end; alias deps? required_deps_on? # === deps?

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === required_deps_on?
  # ========================================================================= #
  def required_deps_on?
    obtain(__method__.to_s.delete('?'))
  end; alias deps?                     required_deps_on? # === deps?
       alias required_dependencies_on? required_deps_on? # === required_dependencies_on?

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === has_to_be_compiled?
  # ========================================================================= #
  def has_to_be_compiled?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === program_path?
  # ========================================================================= #
  def program_path?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === last_update?
  # ========================================================================= #
  def last_update?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_build_directory?
  # ========================================================================= #
  def use_build_directory?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === set_env_variables?
  # ========================================================================= #
  def set_env_variables?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === apply_patch?
  # ========================================================================= #
  def apply_patch?
    obtain(__method__.to_s.delete('?'))
  end; alias set_apply_patch? apply_patch? # === set_apply_patch?

  # ========================================================================= #
  # === headers?
  # ========================================================================= #
  def headers?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === installation_steps?
  # ========================================================================= #
  def installation_steps?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === run_configure?
  # ========================================================================= #
  def run_configure?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === pre_make_commands?
  # ========================================================================= #
  def pre_make_commands?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_autogen?
  # ========================================================================= #
  def use_autogen?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === short_description?
  # ========================================================================= #
  def short_description?
    obtain(__method__.to_s.delete('?'))
  end; alias short_desc? short_description? # === short_desc?

  # ========================================================================= #
  # === use_this_program_name_on_gobolinux?
  # ========================================================================= #
  def use_this_program_name_on_gobolinux?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === preinstall?
  # ========================================================================= #
  def preinstall?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_this_make_command?
  # ========================================================================= #
  def use_this_make_command?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === parameters_to_make?
  # ========================================================================= #
  def parameters_to_make?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === pre_configure_steps?
  # ========================================================================= #
  def pre_configure_steps?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === gir_files?
  # ========================================================================= #
  def gir_files?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === git_url?
  # ========================================================================= #
  def git_url?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === pkgconfig_files?
  # ========================================================================= #
  def pkgconfig_files?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === build_static?
  # ========================================================================= #
  def build_static?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === autosymlink_lib64?
  # ========================================================================= #
  def autosymlink_lib64?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === archive_type?
  # ========================================================================= #
  def archive_type?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === echo_yes?
  # ========================================================================= #
  def echo_yes?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === enable_shared?
  # ========================================================================= #
  def enable_shared?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === enable_static?
  # ========================================================================= #
  def enable_static?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === do_not_download?
  # ========================================================================= #
  def do_not_download?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === can_be_compiled_statically?
  # ========================================================================= #
  def can_be_compiled_statically?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === symlink_headers?
  # ========================================================================= #
  def symlink_headers?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === configure_command_to_use?
  # ========================================================================= #
  def configure_command_to_use?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === is_an_active_project?
  # ========================================================================= #
  def is_an_active_project?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === short_name?
  # ========================================================================= #
  def short_name?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === do_not_symlink?
  # ========================================================================= #
  def do_not_symlink?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === keep_extracted?
  # ========================================================================= #
  def keep_extracted?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === m4_files?
  # ========================================================================= #
  def m4_files?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === localstatedir?
  # ========================================================================= #
  def localstatedir?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === libexec?
  # ========================================================================= #
  def libexec?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === run_ldconfig?
  # ========================================================================= #
  def run_ldconfig?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === flatpak_url?
  # ========================================================================= #
  def flatpak_url?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === mirror?
  # ========================================================================= #
  def mirror?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === may_we_modify_the_configure_options?
  # ========================================================================= #
  def may_we_modify_the_configure_options?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === symlink_glib_schema?
  # ========================================================================= #
  def symlink_glib_schema?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === wikipedia?
  # ========================================================================= #
  def wikipedia?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_this_make_install_command?
  # ========================================================================= #
  def use_this_make_install_command?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === svn_url?
  # ========================================================================= #
  def svn_url?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_autoconf?
  # ========================================================================= #
  def use_autoconf?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_this_build_directory?
  # ========================================================================= #
  def use_this_build_directory?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === find                                                       (find tag)
  # ========================================================================= #
  def find(
      for_this_program = search_for_this_program?
    )
    reset_the_internal_variables
    load_up_the_dataset(for_this_program)
  end; alias try_to_find find # === try_to_find

  # ========================================================================= #
  # === has_key?
  #
  # Query method to determine whether a specific key is present in
  # this class.
  # ========================================================================= #
  def has_key?(i)
    dataset?.has_key?(i)
  end; alias key? has_key? # === key?

  # ========================================================================= #
  # === configure_base_dir?
  # ========================================================================= #
  def configure_base_dir?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_this_build_system?
  # ========================================================================= #
  def use_this_build_system?
    obtain(__method__.to_s.delete('?'))
  end; alias use_which_build_system? use_this_build_system? # === use_which_build_system?

  # ========================================================================= #
  # === program_name_and_program_version?
  # ========================================================================= #
  def program_name_and_program_version?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === archive_size?
  # ========================================================================= #
  def archive_size?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === copy_source?
  # ========================================================================= #
  def copy_source?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === modify_the_makefile?
  # ========================================================================= #
  def modify_the_makefile?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === optional_start_message?
  # ========================================================================= #
  def optional_start_message?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === override?
  # ========================================================================= #
  def override?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === pre_make_install_sed?
  # ========================================================================= #
  def pre_make_install_sed?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === rating?
  # ========================================================================= #
  def rating?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === recommended_deps_on?
  # ========================================================================= #
  def recommended_deps_on?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === ruby_premake_commands?
  # ========================================================================= #
  def ruby_premake_commands?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === run_make?
  # ========================================================================= #
  def run_make?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === run_make_check?
  # ========================================================================= #
  def run_make_check?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === sed?
  # ========================================================================= #
  def sed?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === sub_dir?
  # ========================================================================= #
  def sub_dir?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === url3?
  # ========================================================================= #
  def url3?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === url4?
  # ========================================================================= #
  def url4?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === url5?
  # ========================================================================= #
  def url5?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === configure_options_explained?
  # ========================================================================= #
  def configure_options_explained?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === meson_configure_options?
  # ========================================================================= #
  def meson_configure_options?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === cmake_configure_options?
  # ========================================================================= #
  def cmake_configure_options?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === configure_options?
  # ========================================================================= #
  def configure_options?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_glib_schema?
  # ========================================================================= #
  def use_glib_schema?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === use_this_program_name?
  # ========================================================================= #
  def use_this_program_name?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === md5sum?
  # ========================================================================= #
  def md5sum?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === manual_steps?
  # ========================================================================= #
  def manual_steps?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === maintainer?
  # ========================================================================= #
  def maintainer?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === extract_to?
  # ========================================================================= #
  def extract_to?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === github?
  # ========================================================================= #
  def github?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === base_dir?
  # ========================================================================= #
  def base_dir?
    obtain(__method__.to_s.delete('?'))
  end

  # ========================================================================= #
  # === obtain                                                   (obtain tag)
  # ========================================================================= #
  def obtain(i)
    result = nil
    dataset = dataset?
    if dataset
      string_key = i.to_s # Get our key, as a String.
      # ===================================================================== #
      # First, we try to query the main Hash via a string-key.
      # ===================================================================== #
      if dataset.has_key? string_key
        result = dataset[i]
      # ===================================================================== #
      # Next we try the same key, but as a Symbol.
      # ===================================================================== #
      elsif dataset.has_key? string_key.to_sym
        result = dataset[string_key.to_sym]
      else
        # e 'The key is not available: '+i.to_s
      end
      return result
    end
  end

  # ========================================================================= #
  # === absolute_path_to_the_cookbook_file?
  # ========================================================================= #
  def absolute_path_to_the_cookbook_file?
    @internal_hash[:absolute_path_to_the_cookbook_file]
  end; alias cookbook_file? absolute_path_to_the_cookbook_file? # === cookbook_file?
       alias yaml_file?     absolute_path_to_the_cookbook_file? # === yaml_file?
       alias file?          absolute_path_to_the_cookbook_file? # === file?

  # ========================================================================= #
  # === pretty_show_the_dataset
  # ========================================================================= #
  def pretty_show_the_dataset
    # pp the dataset: pp _.dataset?
    dataset = dataset?
    pp dataset
    e
    print Colours.rev
    Cliner.cliner
    e Colours::HtmlColoursMethods.steelblue('This dataset was read from this file: ')+
      Colours.sfile(cookbook_file?)
    e Colours::HtmlColoursMethods.steelblue('The mode is: '.ljust(38))+
      Colours::HtmlColoursMethods.olivedrab(':'+mode?.to_s)
    e Colours::HtmlColoursMethods.steelblue('base_dir: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(base_dir?.to_s)
    e Colours::HtmlColoursMethods.steelblue('program_name_and_program_version: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(program_name_and_program_version?.to_s)
    e Colours::HtmlColoursMethods.steelblue('program_path: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(program_path?.to_s)
    e Colours::HtmlColoursMethods.steelblue('The url1 entry is: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(url1?.to_s)
    if url2? and !url2?.empty?
      e Colours::HtmlColoursMethods.steelblue('The url2 entry is: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(url2?.to_s)
    end
    if homepage? and !homepage?.empty?
      e Colours::HtmlColoursMethods.steelblue('The homepage entry is: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(homepage?.to_s)
    end
    e Colours::HtmlColoursMethods.steelblue('apply_patch entry is: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(apply_patch?.to_s)
    if headers? and !headers?.empty?
      e Colours::HtmlColoursMethods.steelblue('headers entry is: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(headers?.to_s[0 .. 40]+'[...]')
    end
    # ======================================================================= #
    # === prefix
    # ======================================================================= #
    e Colours::HtmlColoursMethods.steelblue('prefix: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(prefix?.to_s)
    # ======================================================================= #
    # === use build directory
    # ======================================================================= #
    e Colours::HtmlColoursMethods.steelblue('use build directory: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(use_build_directory?.to_s)
    if gir_files? and !gir_files?.empty?
      e Colours::HtmlColoursMethods.steelblue('gir files: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(gir_files?.join(' ').to_s)
    end
    if pkgconfig_files? and !pkgconfig_files?.empty?
      e Colours::HtmlColoursMethods.steelblue('pkgconfig files: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(pkgconfig_files?.join(' ').to_s)
    end
    if symlink_pkgconfig_files? and !symlink_pkgconfig_files?.to_s.empty?
      e Colours::HtmlColoursMethods.steelblue('symlink pkgconfig files: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(symlink_pkgconfig_files?.to_s)
    end
    e Colours::HtmlColoursMethods.steelblue('symlink headers: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(symlink_headers?.to_s)
    # ======================================================================= #
    # === configure_command_to_use
    # ======================================================================= #
    if configure_command_to_use? and !configure_command_to_use?.empty?
      e Colours::HtmlColoursMethods.steelblue('configure_command_to_use: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(configure_command_to_use?.to_s)
    end
    unless flatpak_url?.to_s.empty?
      e Colours::HtmlColoursMethods.steelblue('flatpak_url: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(flatpak_url?.to_s)
    end
    e Colours::HtmlColoursMethods.steelblue('configure_base_dir: '.ljust(38))+
      Colours::HtmlColoursMethods.lightgreen(configure_base_dir?.to_s)
    # ======================================================================= #
    # === configure_options
    # ======================================================================= #
    if configure_options? and !configure_options?.empty?
      e Colours::HtmlColoursMethods.steelblue('configure_options: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(
          configure_options?.to_s.tr("\n",' ').strip.squeeze(' ')
        )
    end
    extra_information = extra_information?
    # ======================================================================= #
    # === extra_information
    # ======================================================================= #
    if extra_information and !extra_information.empty?
      content = extra_information.tr("\n",' ').strip.squeeze(' ')
      if content.size > 35
        content = content[0 .. 35]+' [...]'
      end
      e Colours::HtmlColoursMethods.steelblue('extra_information: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(
          content
        )
    end
    # ======================================================================= #
    # === meson_configure_options
    # ======================================================================= #
    if meson_configure_options? and !meson_configure_options?.empty?
      e Colours::HtmlColoursMethods.steelblue('meson_configure_options: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(
          meson_configure_options?.to_s.tr("\n",' ').strip.squeeze(' ')
        )
    end
    if use_this_build_system? and !use_this_build_system?.empty?
      e Colours::HtmlColoursMethods.steelblue('use_this_build_system: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(use_this_build_system?.to_s)
    end
    if modify_the_makefile? and !modify_the_makefile?.to_s.empty?
      e Colours::HtmlColoursMethods.steelblue('modify_the_makefile: '.ljust(38))+
        Colours::HtmlColoursMethods.lightgreen(modify_the_makefile?.to_s)
    end; ee rev
    Cliner.cliner
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = return_hyphened_commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === rawcookbook matedesktop --do-not-autogenerate
      # ===================================================================== #
      when /^-?-?do(-|_)?not(-|_)?autogenerate$/i
        do_not_autogenerate
      # ===================================================================== #
      # === rawcookbook gxine --simple
      # ===================================================================== #
      when /^-?-?simple$/i
        set_simple
      # ===================================================================== #
      # === rawcookbook gxine --complex
      # ===================================================================== #
      when /^-?-?complex$/i,
           /^-?-?expanded$/i
        set_complex
      # ===================================================================== #
      # === rawcookbook gxine --help 
      # ===================================================================== #
      when /help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === load_up_the_dataset
  #
  # This method will load the dataset from the .yml file.
  # ========================================================================= #
  def load_up_the_dataset(
      for_this_program = first_argument?
    )
    unless for_this_program.is_a? String
      for_this_program = for_this_program.to_s
    end
    # ======================================================================= #
    # End early here if the program is not included.
    # ======================================================================= #
    return unless ::RBT.is_this_program_included?(for_this_program)
    original_name = for_this_program.dup
    case load_what?
    # ======================================================================= #
    # === :simple_cookbook
    #
    # This refers to the unsanitized cookbook-dataset.
    # ======================================================================= #
    when :simple_cookbook,
         :default,
         :simple
      _ = RBT.cookbook_directory?+
          for_this_program.to_s
    # ======================================================================= #
    # === :expanded_cookbook
    #
    # This is also known as :complex.
    # ======================================================================= #
    when :expanded_cookbook,
         :expanded,
         :complex
      _ = ::RBT.directory_expanded_cookbooks?+
          for_this_program.to_s
    end
    _ = _.dup
    _ << '.yml' unless _.end_with?('.yml') or _.empty?
    _ = absolute_path(_) # Always try as absolute path.
    # ======================================================================= #
    # Since as of January 2024, if the file does NOT exist, and we are
    # on a roebe-system, we will automatically generate a new .yml file.
    # In February 2024 it was noticed that this can lead to an infinite
    # loop, so the code was modified to no longer have this issue.
    # ======================================================================= #
    if !File.exist?(_) and shall_we_try_to_autogenerate_the_dataset_for_an_expanded_yaml_file?
      # ===================================================================== #
      # Autogenerate it anew in this case:
      # ===================================================================== #
      do_autogenerate_a_new_expanded_cookbook_file_for(_)
    end
    # ======================================================================= #
    # The file exists, so we continue here.
    # ======================================================================= #
    if File.exist? _
      the_program_is_included
      set_absolute_path_to_the_cookbook_file(_)
      require 'rbt/actions/individual_actions/checks_and_validations/check_for_invalid_entries_in_this_cookbook.rb'
      if RBT.does_this_yaml_file_have_two_url1_entries?(cookbook_file?)
        opne 'Please fix the above issue first before we can continue here.'
        exit
      end
      # ===================================================================== #
      # Next, load the yaml file.
      # ===================================================================== #
      yaml_dataset = load_yaml(cookbook_file?)
      if dataset?.frozen?
        @internal_hash[:dataset_from_the_yaml_file] = @internal_hash[:dataset_from_the_yaml_file].dup
      end
      if yaml_dataset.nil?
        e "#{rev}No value found for the file #{sfile(cookbook_file?)}#{rev}."
      end
      @internal_hash[:dataset_from_the_yaml_file].update(yaml_dataset) if yaml_dataset
      if @internal_hash[:dataset_from_the_yaml_file].has_key?(original_name) and # Sync it down one level in this case.
        !@internal_hash[:dataset_from_the_yaml_file].has_value?(original_name)
        # =================================================================== #
        # Note that this will fail for "sed" because it may have an entry
        # called sed. :D
        # This explains the above ! check in use.
        # =================================================================== #
        @internal_hash[:dataset_from_the_yaml_file] = @internal_hash[:dataset_from_the_yaml_file][original_name]
      end
      @internal_hash[:dataset_from_the_yaml_file].freeze
    else
      the_program_is_not_included
      unless be_quiet_if_the_file_was_not_found?
        opnn; no_file_exists_at(_)
      end
    end
    return @internal_hash[:dataset_from_the_yaml_file] # Always return the dataset as-is.
  end; alias this_file=              load_up_the_dataset # === this_file=
       alias load_this_file          load_up_the_dataset # === load_this_file
       alias load_this_program       load_up_the_dataset # === load_this_program
       alias load_this_cookbook_file load_up_the_dataset # === load_this_cookbook_file

  # ========================================================================= #
  # === do_autogenerate_a_new_expanded_cookbook_file_for
  # ========================================================================= #
  def do_autogenerate_a_new_expanded_cookbook_file_for(i)
    opne "#{rev}The .yml file does not exist at #{sfile(i)}#{rev}. As we are"
    opne "#{rev}on a roebe-system, this .yml file will be generated now."
    target = File.basename(i).delete_suffix('.yml')
    # action(:expand_cookbooks, target) # This would lead to a loop.
    set_load_what(:simple_cookbook)
    load_up_the_dataset
    sanitize_cookbook = action(:sanitize_cookbook, target) { :do_not_instantiate_a_new_raw_cookbook }
    sanitize_cookbook.merge_in_this_dataset(dataset?)
    sanitize_cookbook.set_raw_cookbook(self)
    sanitize_cookbook.do_convert_the_dataset(
      dataset?['url1'],
      dataset?['url2']
    )
    RBT.store_this_expanded_dataset_into_that_yaml_file(
      sanitize_cookbook.dataset?,
      i
    ) 
  end

  # ========================================================================= #
  # === blfs?
  # ========================================================================= #
  def blfs?
    obtain(__method__.to_s.delete('?'))
  end; alias blfs_homepage? blfs? # === blfs_homepage?

  # ========================================================================= #
  # === has_non_empty_BLFS_entry?
  #
  # This method will return true if the :blfs entry is NOT empty. This
  # will include nil, an empty String as well as an empty Array.
  # ========================================================================= #
  def has_non_empty_BLFS_entry?
    _ = blfs?
    _ and !_.empty?
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu(
      return_hyphened_commandline_arguments
    )
    set_search_for_this_program(:first_argument) # Should come after menu().
    load_up_the_dataset
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::RawCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.raw_cookbook
#
# Ideally use this method to instantiate a new raw cookbook instance.
#
# Since as of February 2024 this is part of the RBT::Action namespace.
# =========================================================================== #
def self.raw_cookbook(
    i            = ARGV,
    options_hash = {},
    &block
  )
  raw_cookbook = RBT::Action::Cookbooks::RawCookbook.new(i, :do_not_run_yet, &block)

  if options_hash and !options_hash.empty?
    options_hash.each_pair {|key, value|
      case key
      # ===================================================================== #
      # === :autogenerate_expanded_dataset
      # ===================================================================== #
      when :autogenerate_expanded_dataset
        raw_cookbook.set_autogenerate_expanded_dataset(value)
      end
    }
  end
  raw_cookbook.run
  return raw_cookbook
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours'
  require 'cliner'
  alias e puts
  module Colours
    include HtmlColoursMethods
  end
  _ = RBT.raw_cookbook(ARGV)
  # e; pp _; e
  unless ARGV.empty?
    _.pretty_show_the_dataset
  end
end # rawcookbook htop
    # rawcookbook zathura
    # rawcookbook poppler
    # rawcookbook glibnetworking
    # rawcookbook gxine --complex