#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::ShowConfigurationOptions
#
# This small class will show configuration options for a specific program
# on the commandline.
#
# The output will usually be something like this:
#
#   "The configuration options for gcc are the following:
#
#    --build=x86_64-linux-gnu
#    --host=x86_64-linux-gnu
#    --target=x86_64-linux-gnu"
#
# Usage example:
#
#   RBT::Action::Cookbooks::ShowConfigurationOptions.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/show_configuration_options/show_configuration_options.rb'
# RBT.show_configuration_options(ARGV)
# RBT.return_configuration_options_for? 'php'
# RBT::Action::Cookbooks::ShowConfigurationOptions.new(ARGV)
# ruby_options = RBT.return_configuration_options_for? :ruby
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class ShowConfigurationOptions < RBT::Action # === RBT::Action::Cookbooks::ShowConfigurationOptions

  require 'rbt/toplevel_methods/show_last_configure_option.rb'

  # ========================================================================= #
  # === DEFAULT_PROGRAM_NAME
  #
  # This constant is no longer in use.
  # ========================================================================= #
  DEFAULT_PROGRAM_NAME = 'glibc'

  # ========================================================================= #
  # === VERSION_REGEX
  #
  # This constant is also no longer in use, but it may be necessary in
  # the future, so we'll keep it here for the time being.
  # ========================================================================= #
  VERSION_REGEX = /\d{1,2}\.?\d{1,3}\.?\d{1,3}/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    reset_the_internal_variables
    infer_the_namespace
    try_to_require_the_xorg_buffer
  end

  # ========================================================================= #
  # === reset_the_internal_variables
  # ========================================================================= #
  def reset_the_internal_variables
    # ======================================================================= #
    # === :use_xorg_buffer
    # ======================================================================= #
    @internal_hash[:use_xorg_buffer] = true
    # ======================================================================= #
    # === :work_on_this_program
    # ======================================================================= #
    @internal_hash[:work_on_this_program] = nil
    # ======================================================================= #
    # === :user_prefix
    #
    # This will initially be nil.
    # ======================================================================= #
    @internal_hash[:user_prefix] = nil
    # ======================================================================= #
    # === :are_we_in_a_build_directory
    #
    # This variable will set to true if the current working directory
    # is a BUILD/ or BUILD_DIRECTORY subdirectory.
    # ======================================================================= #
    @internal_hash[:are_we_in_a_build_directory] = false
    # ======================================================================= #
    # === :user_defined_program_version
    #
    # This can be defined by the user and will override any guesses
    # from the cookbook-dataset.
    # ======================================================================= #
    @internal_hash[:user_defined_program_version] = nil
    # ======================================================================= #
    # === :use_this_build_system
    # ======================================================================= #
    @internal_hash[:use_this_build_system] = nil
    # ======================================================================= #
    # === :assume_build_directory
    # ======================================================================= #
    @internal_hash[:assume_build_directory] = false
    # ======================================================================= #
    # === :show_the_command
    # ======================================================================= #
    @internal_hash[:show_the_command] = false
  end

  # ========================================================================= #
  # === show_the_command?
  # ========================================================================= #
  def show_the_command?
    @internal_hash[:show_the_command]
  end

  # ========================================================================= #
  # === assume_build_directory?
  # ========================================================================= #
  def assume_build_directory?
    @internal_hash[:assume_build_directory]
  end

  # ========================================================================= #
  # === use_xorg_buffer?
  # ========================================================================= #
  def use_xorg_buffer?
    @internal_hash[:use_xorg_buffer]
  end

  # ========================================================================= #
  # === set_user_prefix
  # ========================================================================= #
  def set_user_prefix(i)
    @internal_hash[:user_prefix] = i
  end; alias set_prefix set_user_prefix # === set_prefix

  # ========================================================================= #
  # === work_on_this_program?
  # ========================================================================= #
  def work_on_this_program?
    @internal_hash[:work_on_this_program]
  end; alias this_program?          work_on_this_program? # === this_program?
       alias program_name?          work_on_this_program? # === program_name?
       alias work_on_which_program? work_on_this_program? # === work_on_which_program?
       alias input?                 work_on_this_program? # === input?
       alias for_this_program?      work_on_this_program? # === for_this_program?

  # ========================================================================= #
  # === are_we_in_a_build_directory?
  # ========================================================================= #
  def are_we_in_a_build_directory?
    @internal_hash[:are_we_in_a_build_directory]
  end

  # ========================================================================= #
  # === we_are_in_a_build_directory
  # ========================================================================= #
  def we_are_in_a_build_directory
    @internal_hash[:are_we_in_a_build_directory] = true
  end

  # ========================================================================= #
  # === find_cookbook_alias_for
  # ========================================================================= #
  def find_cookbook_alias_for(i)
    ::RBT.find_cookbook_alias_for(i)
  end

  # ========================================================================= #
  # === feedback_help_then_exit                                    (help tag)
  #
  # This will show some help options, then exit.
  #
  # Invocation example:
  #
  #   sco --help
  #
  # ========================================================================= #
  def feedback_help_then_exit
    opnn { :no_trailing}
    e 
    e
    eparse '  XORG                     # Set the xorg buffer here as well, silently.'
    eparse '  PWD                      # Use the current working directory via PWD or --pwd.'
    eparse '  --ntrad                  # Make use of an AppDir prefix. Example: sco htop --ntrad'
    eparse '  --appdir                 # as ^^^ above'
    eparse '  --use-configure          # specifically use the configure options'
    eparse '  --use-extended-configure # as ^^^ above, but also show the leading ./configure string'
    eparse '  --use-meson              # specifically use meson and its configure options'
    eparse '  --usr                    # use the /usr/ prefix'
    e
    exit
  end; alias show_help feedback_help_then_exit # === show_help

  # ========================================================================= #
  # === found?
  # ========================================================================= #
  def found?(
      cookbook_dataset = cookbook_dataset?
    )
    cookbook_dataset and cookbook_dataset.found?
  end

  # ========================================================================= #
  # === cookbook_dataset?
  # ========================================================================= #
  def cookbook_dataset?
    @internal_hash[:cookbook_dataset]
  end; alias dataset? cookbook_dataset? # === dataset?
       alias dataset  cookbook_dataset? # === dataset
       alias data?    cookbook_dataset? # === data?
       alias data     cookbook_dataset? # === data

  # ========================================================================= #
  # === set_new_xorg_buffer
  # ========================================================================= #
  def set_new_xorg_buffer(
      i = data?.join(' ')
    ) 
    set_xorg_buffer(i) # Useful to set the xorg buffer.
  end

  # ========================================================================= #
  # === dataset_prefix?
  # ========================================================================= #
  def dataset_prefix?
    dataset.prefix?
  end

  # ========================================================================= #
  # === user_prefix?
  # ========================================================================= #
  def user_prefix?
    @internal_hash[:user_prefix]
  end; alias prefix? user_prefix? # === prefix?

  # ========================================================================= #
  # === user_defined_program_version?
  # ========================================================================= #
  def user_defined_program_version?
    @internal_hash[:user_defined_program_version]
  end; alias program_version? user_defined_program_version? # === program_version?

  # ========================================================================= #
  # === assumed_prefix?
  # ========================================================================= #
  def assumed_prefix?(
      user_prefix = user_prefix?
    )
    _ = dataset_prefix? # First get the default prefix as used by the cookbook-dataset.
    if user_prefix
      case user_prefix
      # ===================================================================== #
      # === :appdir_prefix
      # ===================================================================== #
      when :appdir_prefix
        # =================================================================== #
        # Next, we can either .split('-') on that directory name, or we
        # use ProgramInformation instead.'
        # =================================================================== #
        _ = return_appdir_prefix(
              work_on_which_program?+'-'+assumed_version?.to_s
            )
      else
        _ = user_prefix unless user_prefix.nil?
      end
    elsif user_prefix.nil? and !user_defined_program_version?.nil?
      # ===================================================================== #
      # This entry point is valid for "scopwd" usage cases.  
      # ===================================================================== #
      _ = return_appdir_prefix(
            work_on_which_program?+'-'+user_defined_program_version?.to_s
          )
    end
    return _
  end; alias prefix_to_use?              assumed_prefix? # === prefix_to_use?
       alias return_insert_prefix_to_use assumed_prefix? # === return_insert_prefix_to_use
       alias insert_prefix_to_use        assumed_prefix? # === insert_prefix_to_use

  # ========================================================================= #
  # === store_last_configuration_options
  #
  # Use this method to store the last configure options used in a specific
  # file, so that it can be read lateron.
  #
  # Note that the target directory must exist, for this method to properly
  # work.
  # ========================================================================= #
  def store_last_configuration_options(
      i,
      into = RBT.store_the_last_configuration_options_in_this_file
    )
    base_directory = File.dirname(into)
    if File.directory? base_directory
      # opne 'Storing into '+sfile(into)+'.' if be_verbose?
      write_what_into(
        i.strip, into
      )
    else
      opnn; no_directory_exists_at(base_directory)
    end
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      # ===================================================================== #
      # This means that the user provided no arguments, thus we will use
      # the current working directory as our assumed input.    
      # ===================================================================== #
      if i.empty?
        do_assume_that_the_user_wanted_to_work_with_the_current_working_directory
      else
        i.each {|entry| menu(entry) }
      end
    else
      case i # case tag
      # ===================================================================== #
      # === --use-extended-configure
      #
      # Usage example:
      #
      #   sco gtk+ --use-extended-configure
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?extended(-|_)?configure$/i,
           /^-?-?use(-|_)?extended(-|_)?configure(-|_)?options$/i,
           /^-?-?full$/i
        @internal_hash[:use_this_build_system] = :configure
        @internal_hash[:show_the_command] = true
      # ===================================================================== #
      # === --use-configure
      #
      # This entry point allows the user to make use of the configure
      # options by default, if said program does have any.
      # ===================================================================== #
      when /^-?-?use(-|_)?configure$/i,
           /^-?-?use(-|_)?configure(-|_)?options$/i
        @internal_hash[:use_this_build_system] = :configure
      # ===================================================================== #
      # === --assume-build-directory
      # ===================================================================== #
      when /^-?-?assume(-|_)?build(-|_)?directory$/i,
           /^-?-?assume$/i,
           '..',
           ' ..'
        @internal_hash[:assume_build_directory] = true
      # ===================================================================== #
      # === --use-meson
      #
      # This entry point allows the user to make use of meson and the
      # meson configure options.
      # ===================================================================== #
      when /^-?-?use(-|_)?meson$/i
        @internal_hash[:use_this_build_system] = :meson
      # ===================================================================== #
      # === --usr
      #
      # Use the /usr/ prefix here.
      # ===================================================================== #
      when /^-?-?usr$/i
        set_prefix('/usr/')
      # ===================================================================== #
      # === XORG
      # ===================================================================== #
      when /^-?-?XORG$/i,
           /^-?-?BUFFER$/i
        @internal_hash[:use_xorg_buffer] = true
      # ===================================================================== #
      # === appdir
      #
      # Usage example:
      #
      #  sco xorgserver --appdir
      #  sco gcc --ntrad
      #
      # ===================================================================== #
      when /-?-?appdir$/i,
           /-?-?appconf$/i,
           /-?-?ntrad$/i,
           /^-?-?app?dir$/i,
           /^app/i
        set_user_prefix(:appdir_prefix)
      # ===================================================================== #
      # === --pwd
      # ===================================================================== #
      when /-?-?pwd$/i,
           /-?-?BUILD$/i,
           ''
        do_assume_that_the_user_wanted_to_work_with_the_current_working_directory
      # ===================================================================== #
      # === sco --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
      else
        if i
          unless i.start_with?('--') # Only consider entries without leading --.
            set_work_on_this_program(i)
          end
        end
      end
    end
  end

  # ========================================================================= #
  # === set_work_on_this_program
  # ========================================================================= #
  def set_work_on_this_program(i)
    if i.is_a? Array
      i = i.first
    end
    i = i.dup if i
    if i and i.end_with?('.yml')
      i.delete_suffix!('.yml')
    end
    # ======================================================================= #
    # Last but not least, we query whether there is an alias program name
    # but ONLY if the Cookbooks project does NOT include the given input.
    # ======================================================================= #
    if RBT.is_included? i, :do_not_check_for_aliases
      result = i
    else # else pass it through our alias-filter there.
      result = find_cookbook_alias_for(i)
    end
    if result.nil? and # Try again in this case.
       RBT.include_this_program?("lib#{i}")
      result = "lib#{i}"
    end
    if result
      unless result == i
        opne 'Using the alias name `'+sfancy(result)+
             rev+'` for the input `'+simp(i)+rev+'` instead.'
        i = result
      end
    end
    @internal_hash[:work_on_this_program] = i
  end; alias set_for_this_program set_work_on_this_program # === set_for_this_program

  # ========================================================================= #
  # === use_this_build_system?
  #
  # This query method is a bit hackish. In theory we should rely on the
  # code showing us the real value, but there is presently (April 2021)
  # a bug, which explains the work around used here. At a later time
  # we may have to re-evaluate the code here, but for now this has
  # to suffice.
  #
  # Since as of September 2022 the user can ignore this, via the
  # --use-configure flag.
  # ========================================================================= #
  def use_this_build_system?(
      cookbook_dataset = cookbook_dataset?
    )
    _ = cookbook_dataset.use_this_build_system?
    case cookbook_dataset.raw_cookbook?.use_this_build_system?
    when 'meson'
      _ = 'meson'
    end
    if user_defined_use_this_build_system?
      _ = user_defined_use_this_build_system?
    end
    return _
  end; alias build_type? use_this_build_system? # === build_type?

  # ========================================================================= #
  # === user_defined_use_this_build_system?
  # ========================================================================= #
  def user_defined_use_this_build_system?
    @internal_hash[:use_this_build_system]
  end

  # ========================================================================= #
  # === cookbook_dataset_program_name?
  # ========================================================================= #
  def cookbook_dataset_program_name?
    cookbook_dataset.program_name?
  end

  # ========================================================================= #
  # === uses_meson?
  # ========================================================================= #
  def uses_meson?
    # ======================================================================= #
    # An alternative may be:
    #
    #   if cookbook_dataset?.configure_command?.include? 'meson'
    #
    # ======================================================================= #
    use_this_build_system?.to_s == 'meson'
  end

  # ========================================================================= #
  # === uses_cmake?
  # ========================================================================= #
  def uses_cmake?
    use_this_build_system? == 'cmake'
  end

  # ========================================================================= #
  # === return_proper_configure_command
  #
  # This method will honour the entry configure_command: in the respective
  # .yml file. We can return 'configure' or 'cmake' or 'scons'.
  #
  # Since as of December 2019, we will also check for the explicit
  # option for "use_this_build_system".
  # ========================================================================= #
  def return_proper_configure_command(
      cookbook_dataset = cookbook_dataset?
    )
    result = ''.dup
    use_this_build_system = use_this_build_system?
    case use_this_build_system.to_s
    # ======================================================================= #
    # === cmake
    #
    # This entry point will specifically use cmake.
    # ======================================================================= #
    when 'cmake'
      result << use_this_build_system
    # ======================================================================= #
    # === meson
    #
    # Same as above, but for meson rather than cmake.
    # ======================================================================= #
    when 'meson'
      result << 'meson'
      # if @internal_hash[:use_this_build_system] = :meson
      #   result << " #{cookbook_dataset.meson_configure_options?}"
      # end
    # ======================================================================= #
    # === configure
    # ======================================================================= #
    when 'configure'
      # result << "#{cookbook_dataset.configure_command?}"
    else # This else-clause  covers :infer_automatically.
      # result << "#{cookbook_dataset.configure_command?}" # ← This usually adds the String 'configure'.
    end
    case result
    when 'configure'
      result.prepend './' # For "./configure" syntax.
      result.prepend '.' if are_we_in_a_build_directory?
    end
    result << ' ' unless result.end_with? ' '
    return result
  end

  # ========================================================================= #
  # === do_assume_that_the_user_wanted_to_work_with_the_current_working_directory
  # ========================================================================= #
  def do_assume_that_the_user_wanted_to_work_with_the_current_working_directory
    pwd = return_pwd.dup
    if pwd.include? '-' # Must include at the least one '-'.
      # ===================================================================== #
      # We may be in a directory called BUILD/ here.
      # ===================================================================== #
      if pwd.include? 'BUILD'
        pwd.sub!(/BUILD_DIRECTORY/,'')
        pwd.sub!(/BUILD/,'')
        we_are_in_a_build_directory
      end
      _ = File.basename(pwd)
      if _.include? '-'
        set_work_on_this_program(
          ProgramInformation.return_program_name(_)
        )
        set_program_version(
          ProgramInformation.return_program_version(_)
        )
      else
        opne _+' can not be evaluated by ProgramInformation; '\
             'the input must include a '+tomato('-')+' token.'
      end
    else
      opne 'Can not work on '+pwd+' - the input must '\
           'contain at the least one - character.'
      set_work_on_this_program(nil)
    end
  end

  # ========================================================================= #
  # === set_user_defined_program_version
  # ========================================================================= #
  def set_user_defined_program_version(i)
    @internal_hash[:user_defined_program_version] = i
  end; alias set_program_version set_user_defined_program_version # === set_program_version

  # ========================================================================= #
  # === assumed_version?
  # ========================================================================= #
  def assumed_version?
    _ = dataset.program_version?
    if @internal_hash[:user_defined_program_version]
      _ = @internal_hash[:user_defined_program_version]
    end
    return _
  end; alias program_version? assumed_version? # === program_version?

  # ========================================================================= #
  # === configure_options?
  # ========================================================================= #
  def configure_options?
    dataset?.meson_or_cmake_or_configure_options?
  end

  # ========================================================================= #
  # === show_the_configure_options
  #
  # Commandline usage example:
  #
  #   sco gnutls
  #
  # ========================================================================= #
  def show_the_configure_options
    cwd = return_pwd
    # ======================================================================= #
    # Obtain the assumed program-version from the name of the current working
    # directory next. In April 2023 the subsequent code had to be changed -
    # if no "-" is part of the current working directory then we will
    # short circuit.
    # ======================================================================= #
    cwd = File.basename(cwd)
    if cwd.include? '-'
      assumed_version_from_the_cwd = ProgramInformation.return_version(cwd)
    else
      assumed_version_from_the_cwd = program_version?
    end
    opnn { :no_trailing }; e; e
    e "#{rev}These are the #{steelblue('raw')} #{rev}configure options for "\
      "#{sfancy(for_this_program?)}"
    e "#{rev}(the xorg buffer will also be modified):#{N}"
    cmd = '  '.dup
    if show_the_command?
      if assume_build_directory?
        cmd << '.'
      end
      cmd << './configure'
    end
    cmd << return_proper_configure_command.strip.squeeze(' ')
    # ======================================================================= #
    # Insert the prefix next:
    # ======================================================================= #
    cmd << " --prefix="
    assumed_prefix = assumed_prefix?.dup
    if assumed_prefix.include? programs_dir? # We know that this must be an AppDir.
      # ===================================================================== #
      # The following regex is not perfect - we may have to change that one
      # day, but for now it suffices (April 2023).
      # ===================================================================== #
      use_this_regex = /(\d{1,4}\.?\d{0,4}\.?\d{0,4})/ # See: https://rubular.com/r/2hnUQZpcRYwNxD
      assumed_prefix.sub!(use_this_regex, assumed_version_from_the_cwd)
    end
    # ======================================================================= #
    # We may have to re-write the prefix here, e. g. to sanitize from
    # binutils-2.40.0 towards binutils-14.04.2023.
    # ======================================================================= #
    cmd << assumed_prefix
    if uses_cmake?
      # ===================================================================== #
      # Modify the prefix a little bit here - see
      # https://rubular.com/r/EtjMy1X87lz7mH
      # ===================================================================== #
      cmd.sub!(/-prefix/, 'DCMAKE_INSTALL_PREFIX')
    end
    # ======================================================================= #
    # And the configure-options will be added next:
    # ======================================================================= #
    assumed_configure_options = dataset?.meson_or_cmake_or_configure_options?(build_type?.to_sym)
    if assumed_configure_options.nil?
      assumed_configure_options = dataset?.meson_or_cmake_or_configure_options?
    end
    cmd << " #{assumed_configure_options}"
    if uses_meson?
      # ===================================================================== #
      # meson always requires a build-directory, so let's use one:
      # ===================================================================== #
      cmd << ' BUILD_DIRECTORY/'
    end
    if assume_build_directory?
      cmd << ' ..'
    end
    # ======================================================================= #
    # Next we output the full commandline. We will NOT show the name of
    # the class here though.
    # ======================================================================= #
    e
    e mediumpurple(cmd)
    e
    # ======================================================================= #
    # === Handle xorg-specific parts next
    # ======================================================================= #
    if use_xorg_buffer?
      set_new_xorg_buffer(cmd)
      store_last_configuration_options(cmd)
      shell_file = work_on_which_program?.to_s+'.sh'
      write_what_into(cmd.strip, shell_file)
      e "#{rev}The above was assigned to the #{teal('xorg-buffer')} "\
        "#{rev}as well, so you can"
      e 'very easily copy-paste this. Additionally it was also stored'
      e 'into a shell file, at `'+sfile(shell_file)+rev+'`.'
    end
  end; alias show_configuration_options show_the_configure_options # === show_configuration_options
       alias report                     show_the_configure_options # === report

  # ========================================================================= #
  # === initialize_the_cookbook_dataset
  # ========================================================================= #
  def initialize_the_cookbook_dataset(
      i = work_on_which_program?
    )
    # ======================================================================= #
    # === :cookbook_dataset
    # ======================================================================= #
    @internal_hash[:cookbook_dataset] = action(:cookbook_for, i) # This uses :fast implicitely already.
  end; alias determine_cookbook initialize_the_cookbook_dataset # === determine_cookbook

  # ========================================================================= #
  # === notify_the_user_that_the_given_input_was_not_found
  # ========================================================================= #
  def notify_the_user_that_the_given_input_was_not_found(i = input?)
    opne "#{rev}The given input `#{sfancy(i)}#{rev}` was not found."
    opne 'This means that no program called that way could'
    opne 'be found to be registered.'
    opne 'Another reason why it failed may be a typo, so '
    opne 'make sure to check the above input.'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    initialize_the_cookbook_dataset
    # ======================================================================= #
    # Must check whether we have found the given input or whether we have
    # not.
    # ======================================================================= #
    if found?
      show_the_configure_options
    else
      notify_the_user_that_the_given_input_was_not_found(input?)
    end
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::ShowConfigurationOptions[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.return_configuration_options_for?
#
# This method will delegate towards class
# RBT::Action::Cookbooks::SanitizeCookbook.
#
# Symbols are valid input to this method, as well as Strings.
#
# Usage examples:
#
#   php_options  = RBT.return_configuration_options_for? 'php'
#   ruby_options = RBT.return_configuration_options_for? :ruby
#
# =========================================================================== #
def self.return_configuration_options_for?(
    this_program = ARGV
  )
  RBT.action(:SanitizeCookbook, this_program).configuration_options?
end; self.instance_eval { alias configure_options_for return_configuration_options_for? }    # === RBT.configure_options_for
     self.instance_eval { alias return_configure_options return_configuration_options_for? } # === RBT.return_configure_options

# =========================================================================== #
# === RBT.show_configuration_options
#
# This variant must report the result onto the commandline.
# =========================================================================== #
def self.show_configuration_options(
    i = ARGV,
    &block
  )
  RBT::Action::Cookbooks::ShowConfigurationOptions.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_configuration_options(ARGV)
end # sco ruby
    # sco gcc