#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::ReportXfceVersion
#
# This class will report the components of the xfce desktop environment
# available on the given host computer.
#
# Usage examples:
#
#   RBT::Action::ReportXfceVersion.new(ARGV)
#   RBT.report_xfce_version
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/information/report_xfce_version.rb'
# RBT.report_xfce_version
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class ReportXfceVersion < RBT::Action # === RBT::Action::ReportXfceVersion

  # ========================================================================= #
  # === N_LJUST
  # ========================================================================= #
  N_LJUST = 18

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true,
      &block
    )
    reset
    # ======================================================================= #
    # === Handle blocks given to this method next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_report
      # ===================================================================== #
      when :do_not_report,
           :do_not_report_anything
        @internal_hash[:do_report] = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :n_ljust
    # ======================================================================= #
    @internal_hash[:n_ljust] = N_LJUST
    # ======================================================================= #
    # === :string_to_display
    # ======================================================================= #
    @internal_hash[:string_to_display] = ''.dup
    # ======================================================================= #
    # === :do_report
    # ======================================================================= #
    @internal_hash[:do_report] = true
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(
      i           = '',
      add_newline = true
    )
    case add_newline
    when :no_newline
      add_newline = false
    end
    if add_newline
      i = i.dup if i.frozen?
      i << "\n"
    end
    @internal_hash[:string_to_display] << i
  end

  # ========================================================================= #
  # === string_to_display?
  # ========================================================================= #
  def string_to_display?
    @internal_hash[:string_to_display]
  end; alias string? string_to_display? # === string?
       alias result? string_to_display? # === result?

  # ========================================================================= #
  # === add_newline
  # ========================================================================= #
  def add_newline
    add
  end

  # ========================================================================= #
  # === left_padded
  # ========================================================================= #
  def left_padded
    "#{grey}  "
  end

  # ========================================================================= #
  # === report_pluma_version
  # ========================================================================= #
  def report_pluma_version
    add left_padded+'Pluma: '.ljust(n_ljust?)+slateblue(plumav?)
  end

  # ========================================================================= #
  # === n_ljust?
  # ========================================================================= #
  def n_ljust?
    @internal_hash[:n_ljust]
  end

  # ========================================================================= #
  # === return_pkgconfig_result_for
  # ========================================================================= #
  def return_pkgconfig_result_for(
      i = :libmatekbd
    )
    result = `pkg-config --modversion #{i} 2>&1`.strip
    if result.include?('No package ') and result.include?(' found')
      result = ''
    end
    return result
  end; alias pkgconfig_result return_pkgconfig_result_for # === pkgconfig_result

  # ========================================================================= #
  # === do_report?
  # ========================================================================= #
  def do_report?
    @internal_hash[:do_report]
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report
    if do_report?
      string_to_display?.split(N).each {|line|
        if line.include? ':'
          _ = line.split(':')
          e seagreen(_[0]+':')+steelblue(_[1])
        else
          e line
        end
      }
      e
    end
  end

  # ========================================================================= #
  # === determine_version_of
  # ========================================================================= #
  def determine_version_of(
      i = 'xfdesktop'
    )
    case i
    # ======================================================================= #
    # === exo
    # ======================================================================= #
    when 'exo'
      result = `pkg-config --modversion exo-2`.strip
    # ======================================================================= #
    # === libxfce4util
    # ======================================================================= #
    when 'libxfce4util'
      result = `pkg-config --modversion libxfce4util-1.0`.strip
    # ======================================================================= #
    # === libxfce4ui
    # ======================================================================= #
    when 'libxfce4ui'
      result = `pkg-config --modversion libxfce4ui-2`.strip
    # ======================================================================= #
    # === xfce4-panel
    # ======================================================================= #
    when /xfce4-?panel/
      result = `pkg-config --modversion libxfce4panel-2.0`.strip
    # ======================================================================= #
    # === garcon
    # ======================================================================= #
    when 'garcon'
      result = `pkg-config --modversion garcon-1`.strip
    else # This is the default entry point.
      result = `#{i} --version 2>&1`
    end
    if result.include? N
      result = result.split(N).first
    end
    # ======================================================================= #
    # Check for results such as:
    #
    #   "This is xfdesktop version 4.14.2, running on Xfce 4.15."
    #
    # or
    #
    #   "xfce4-session 4.14.1 (Xfce 4.15)"
    #
    # ======================================================================= #
    if result.include?(' version ')
      result = result.scan(
        /version (\d{1,2}\.?\d{1,2}\.?\d{1,2})/
      ).flatten.first
    elsif result.include?(' (')
      result = result.split(' (').first.split(' ').last
    end
    result
  end

  # ========================================================================= #
  # === add_garcon_version
  # ========================================================================= #
  def add_garcon_version
    add '  garcon: '.ljust(n_ljust?)+determine_version_of('garcon')
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    add grey, :no_newline
    add rev+'Reporting the installed versions of the '+
        royalblue('xfce-desktop components')+
        grey(':')+rev
    add_newline
    # ======================================================================= #
    # Next, add the programs that will be queried. These must all belong
    # to XFCE in one way or another.
    # ======================================================================= #
    n_ljust = n_ljust?
    add '  thunar: '.ljust(n_ljust)+pkgconfig_result('thunarx-3')
    add '  tumbler: '.ljust(n_ljust)+pkgconfig_result('tumbler-1')
    add '  xfconf: '.ljust(n_ljust)+pkgconfig_result('libxfconf-0')
    add '  xfdesktop: '.ljust(n_ljust)+determine_version_of('xfdesktop')
    add '  xfwm4: '.ljust(n_ljust)+determine_version_of('xfwm4')
    add '  xfce4-session: '.ljust(n_ljust)+determine_version_of('xfce4-session')
    add '  xfce4-settings: '.ljust(n_ljust)+determine_version_of('xfce4-settings-manager')
    add '  xfce4-panel: '.ljust(n_ljust)+determine_version_of('xfce4-panel')
    add '  libxfce4util: '.ljust(n_ljust)+determine_version_of('libxfce4util')
    add '  libxfce4ui: '.ljust(n_ljust)+determine_version_of('libxfce4ui')
    add '  exo: '.ljust(n_ljust)+determine_version_of('exo')
    add_garcon_version
    add_newline
    report
  end

  # ========================================================================= #
  # === RBT::Action::ReportXfceVersion[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.return_xfce_version_array                       (array tag)
#
# This variant will return a nested Array with the proper versions
# in use.
#
# Invocation example:
#
#   puts RBT.return_xfce_version_array
#
# =========================================================================== #
def self.return_xfce_version_array
  RBT::Action::ReportXfceVersion.new { :do_not_report_anything }.array?
end

# =========================================================================== #
# === RBT.return_xfce_version_string
#
# This method will return the String that contains the information which
# mate-desktop components are available. That way we can then embed
# this String in other programs.
#
# Usage example:
#
#   x = RBT.return_xfce_version_string
#
# =========================================================================== #
def self.return_xfce_version_string
  RBT::Action::ReportXfceVersion.new { :do_not_report_anything }.string?
end

# =========================================================================== #
# === RBT.report_xfce_version
# =========================================================================== #
def self.report_xfce_version
  RBT::Action::ReportXfceVersion.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.report_xfce_version
end # rbtaction report_xfce_version