#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::RemoveOutdatedArchives
#
# This class can traverse through the local directory structure in the
# source_directory? (where the local archives are kept) and then remove
# "outdated" archives.
#
# An outdated archive is, for example, this setup:
#
#   powerdevil-5.17.4.tar.xz
#   powerdevil-5.17.5.tar.xz
#
# Given these two entries, the first entry, referring to version 5.17.4,
# is "outdated". A more recent version exists, thus it is "outdated".
# The class here will then proceed to remove that archive, but not the
# other package.
#
# Thus, do not invoke this class unless you want to really delete old
# archives!
#
# By default the class scans only for .tar.xz entries. This is hardcoded
# right now, as I only keep entries in the .tar.xz format; an instance
# variable can be used to denote another archive entry.
#
# Usage example:
#
#   RBT::Action::RemoveOutdatedArchives.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/remove_outdated_archives/remove_outdated_archives.rb'
# RBT::Action::RemoveOutdatedArchives.new(ARGV)
# RBT.raw_cookbook(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class RemoveOutdatedArchives < RBT::Action # === RBT::Action::RemoveOutdatedArchives

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @look_only_for_these_archive_types
    #
    # The next variable determines which archive types we are looking
    # for.
    # ======================================================================= #
    @look_only_for_these_archive_types = '.tar.xz'
    # ======================================================================= #
    # === @array_work_on_these_programs
    #
    # This Array keeps track on the programs that will be handled by this
    # class. It will be empty on startup, so as to require the user to
    # tell this class which programs it ought to handle.
    # ======================================================================= #
    @array_work_on_these_programs = []
    # ======================================================================= #
    # === @n_files_were_removed
    # ======================================================================= #
    @n_files_were_removed = 0
    # ======================================================================= #
    # === @total_file_size
    # ======================================================================= #
    @total_file_size = 0
  end

  # ========================================================================= #
  # === format_type?
  # ========================================================================= #
  def format_type?
    @look_only_for_these_archive_types
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      if i.start_with?(':') or i.is_a?(Symbol) or i.start_with?('--')
        if i.is_a?(String) and i.include?(':')
          i = i.dup if i.frozen?
          i.delete!(':')
        end
        case i # case
        # =================================================================== #
        # === all_programs=n
        #
        # Use this entry with care.
        #
        # Invocation example:
        #
        #   oldbegone --all-programs=5
        #
        # =================================================================== #
        when /^-?-?all(_|-)?programs=(\d{1,2})$/i
          array = []
          n_programs_as_threshold = $2.to_i
          available_programs?.each {|entry|
            _ = Dir[src_dir?+entry+'/*']
            if _.size >= n_programs_as_threshold
              array << entry
            end
          }
          add(array) 
        # =================================================================== #
        # === all_programs
        #
        # Use this entry with care.
        #
        # Invocation example:
        #
        #   oldbegone --all-programs
        #
        # =================================================================== #
        when /^-?-?all(_|-)?programs$/i,
             :all_programs
          add available_programs?
        # =================================================================== #
        # === python_addons
        #
        # Invocation example:
        #
        #   oldbegone --python-addons
        #
        # =================================================================== #
        when /^-?-?python(_|-)?addons$/i,
             :python_addons,
             /^-?-?python$/i
          add RBT.return_python_addons
        # =================================================================== #
        # === xfce_desktop
        #
        # Invocation example:
        #
        #   oldbegone --xfce-desktop
        #
        # =================================================================== #
        when /^-?-?xfce(_|-)?desktop$/i,
             :xfce_desktop,
             /^-?-?xfce$/i
          add RBT.return_xfce_desktop_components
        # =================================================================== #
        # === mate_desktop
        #
        # Invocation example:
        #
        #   oldbegone --mate-desktop
        #
        # =================================================================== #
        when /^-?-?mate(_|-)?desktop$/i,
             :mate_desktop,
             /^-?-?mate$/i
          add RBT.return_mate_desktop_components
        # =================================================================== #
        # === kde_foundation
        #
        # Invocation example:
        #
        #   oldbegone --kde-foundation
        #
        # =================================================================== #
        when /^-?-?kde(_|-)?foundation$/i,
             :kde_foundation,
             :kde5_foundation
          add RBT.return_kde5_foundation
        # =================================================================== #
        # === kde_plasma
        #
        # Invocation example:
        #
        #   oldbegone --kde-plasma
        #
        # =================================================================== #
        when /^-?-?kde(_|-)?plasma$/i,
             :kde_plasma
          add RBT.return_kde5_plasma
        # =================================================================== #
        # === xorg_apps
        #
        # Invocation example:
        #
        #   oldbegone --xorg-apps
        #
        # =================================================================== #
        when /^-?-?xorg(_|-)?apps$/i,
             :xorg_apps
          add RBT.return_xorg_apps
        # =================================================================== #
        # === kde_applications
        #
        # Return all KDE5 applications next.
        #
        # Invocation example:
        #
        #   old_are_gone --kde_apps
        #
        # =================================================================== #
        when /^-?-?kde(_|-)?applications$/i,
             :kde_applications,
             /^-?-?kde(_|-)?apps$/i,
             :kde_apps
          add RBT.return_kde5_applications
        end
      end
    end
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  #
  # This method will simply append (aka add) to the main Array of this class.
  # ========================================================================= #
  def add(i)
    @array_work_on_these_programs << i
    @array_work_on_these_programs.flatten!
    @array_work_on_these_programs.compact!
  end

  # ========================================================================= #
  # === notify_the_user_as_to_how_many_files_were_removed
  # ========================================================================= #
  def notify_the_user_as_to_how_many_files_were_removed
    if @n_files_were_removed > 0
      opne 'A total of '+sfancy(@n_files_were_removed.to_s)+
           rev+' files were removed.'
      opne "#{rev}The total filesize of these archives was: "+
              sfancy(@total_file_size.to_s)+
              tomato('B')+' '+
              sfancy(
                (@total_file_size.to_f / (1000.0 * 1000.0)).round(1)
              )+
              tomato('MB')
    end
  end

  # ========================================================================= #
  # === work_on_which_programs?
  # ========================================================================= #
  def work_on_which_programs?
    @array_work_on_these_programs
  end

  # ========================================================================= #
  # === work_through_the_programs
  #
  # This is the method that will iterate over all available program.
  # ========================================================================= #
  def work_through_the_programs(
      i = work_on_which_programs?
    )
    if i.empty?
      opne "#{rev}No program was given to remove its outdated archive(s)."
    else
      i.each {|this_program|
        this_directory = "#{src_dir?}#{this_program}/"
        opne "#{rev}Working on the directory #{sdir(this_directory)} #{rev}next:"
        # =================================================================== #
        # Obtain all entries of that directory next, in a sorted manner. This
        # may yield wrong results though, such as:
        #
        #   kcontacts-5.114.0.tar.xz
        #   kcontacts-5.93.0.tar.xz
        #
        # Applying .max there yields the wrong result.
        # =================================================================== #
        use_this_regex = /\d{1,8}\.?\d{0,12}\.?\d{0,12}/ # See: https://rubular.com/r/ypXFhDrhYI4NEN
        files_in_that_directory = Dir[
          this_directory+'*'+@look_only_for_these_archive_types
        ].sort_by {|entry|
          entry = remove_archive_at_the_end(entry)
          extracted_number = entry.scan(use_this_regex).flatten
          if extracted_number.respond_to?(:first)
            extracted_number = extracted_number.first
          end
          Gem::Version.new(extracted_number) # Make use of Gem::Version to get the "real" version.
        }.reverse
        # =================================================================== #
        # Next, determine the highest entry that will be kept.
        # =================================================================== #
        highest_entry = files_in_that_directory[0] # .max
        # =================================================================== #
        # The next variable denotes which entries will be purged (aka
        # removed).
        # =================================================================== #
        array_of_entries_that_are_to_be_purged = files_in_that_directory.reject {|entry|
          entry.include? highest_entry
        }
        purge_these_entries(array_of_entries_that_are_to_be_purged)
      }
    end
  end

  # ========================================================================= #
  # === purge_these_entries                                       (purge tag)
  # ========================================================================= #
  def purge_these_entries(i)
    if i.is_a? Array
      i.flatten.each {|entry| purge_these_entries(entry) }
    else
      if File.file?(i) and i.end_with?(format_type?)
        opne "#{rev}The entry `#{sfile(i)}#{rev}` will be removed next."
        @n_files_were_removed += 1
        @total_file_size += File.size(i)
        remove_file(i)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    work_through_the_programs
    notify_the_user_as_to_how_many_files_were_removed
  end

  # ========================================================================= #
  # === RBT::Action::RemoveOutdatedArchives[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.remove_outdated_archives
# =========================================================================== #
def self.remove_outdated_archives(
    i = ARGV,
    &block
  )
  return RBT::Action::RemoveOutdatedArchives.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.remove_outdated_archives(ARGV)
end # removeoutdatedarchives :kde_plasma
    # removeoutdatedarchives :kde_foundation
    # removeoutdatedarchives :kde_applications