#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::SimpleAppdirConfigure
#
# This class can be used from the commandline to quickly use the
# guessed appdir prefix.
#
# Usage example:
#
#   RBT::Action::SimpleAppdirConfigure.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/simple_appdir_configure/simple_appdir_configure.rb'
# RBT::Action::SimpleAppdirConfigure.new(ARGV)
# RBT.simple_appdir_configure(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SimpleAppdirConfigure < RBT::Action # === RBT::Action::SimpleAppdirConfigure

  # ========================================================================= #
  # === BUILD_DIRECTORY
  # ========================================================================= #
  BUILD_DIRECTORY = 'BUILD_DIRECTORY/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end
  
  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @cmd
    # ======================================================================= #
    @cmd = './configure'.dup
    # ======================================================================= #
    # === :use_a_build_directory
    # ======================================================================= #
    @internal_hash[:use_a_build_directory] = false
  end

  # ========================================================================= #
  # === do_use_a_build_directory
  #
  # This method will enable the use of a build directory, as-is.
  # ========================================================================= #
  def do_use_a_build_directory
    @internal_hash[:use_a_build_directory] = true
  end

  # ========================================================================= #
  # === use_a_build_directory?
  # ========================================================================= #
  def use_a_build_directory?
    @internal_hash[:use_a_build_directory]
  end; alias use_build_directory? use_a_build_directory? # === use_build_directory?

  # ========================================================================= #
  # === padded_esystem
  # ========================================================================= #
  def padded_esystem(cmd)
    e
    e slateblue(cmd)
    e
    system cmd
  end

  # ========================================================================= #
  # === run_ninja_command
  # ========================================================================= #
  def run_ninja_command
    padded_esystem 'ninja'
  end

  # ========================================================================= #
  # === run_ninja_install_command
  # ========================================================================= #
  def run_ninja_install_command
    padded_esystem 'ninja install'
  end

  # ========================================================================= #
  # === make_use_of_meson
  # ========================================================================= #
  def make_use_of_meson
    @cmd = 'meson'.dup
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    opne "#{rev}These options are currently available:"
    e
    e steelblue('  --down          # pass one "." to the configure or cmake or meson script')
    e steelblue('  --builddir      # use a build directory')
    e steelblue('  --use-cmake')
    e steelblue('  --use-meson')
    e rev
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # (case tag)
      # ===================================================================== #
      # === sappconf --builddir
      # ===================================================================== #
      when /^-?-?build(_|-| )?dir$/i
        do_use_a_build_directory
      # ===================================================================== #
      # === sappconf --down
      #
      # This entry point may be for an argument such as:
      #
      #   ["/Depot/j/glibc-2.27/BUILD/", "--down"]
      #
      # The entry point will instantly modify this.
      # ===================================================================== #
      when /^-?-?down$/,
           :down
        @cmd.prepend('.') # from[0] = File.dirname(return_pwd)
      # ===================================================================== #
      # === sappconf --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      # ===================================================================== #
      # === sappconf --use-meson
      # ===================================================================== #
      when /^-?-?use(_|-| )?meson$/i,
           /^-?-?meson$/i
        remove_this_entry_from_the_commandline_arguments(i)
        make_use_of_meson
      # ===================================================================== #
      # === sappconf --use-cmake
      # ===================================================================== #
      when /^-?-?use(_|-| )?cmake$/i,
           /^-?-?cmake$/i
        @cmd = 'cmake'.dup
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    # ======================================================================= #
    # Establish the cmd to use early on.
    # ======================================================================= # 
    cmd = @cmd # ← This is the command that will be used.
    from = return_pwd # Default.
    _ = first_non_hyphen_argument?
    case _
    # ======================================================================= #
    # === :pwd
    # ======================================================================= #
    when :pwd
      _ = return_pwd
    end
    _ = _.to_s.dup # Work on a copy here, anyway.
    unless _.empty?
      from = _.dup
    end
    from = File.basename(from)
    if File.exist?(from) and File.file?(from)
      # ===================================================================== #
      # Extract the archive in this case.
      #
      # Usage example:
      #
      #   
      #
      # ===================================================================== #
      e "#{rev}Extracting the archive #{sdir(from)} #{rev}next."
      action(:extract_what_to, from)
      to = log_dir?+
           remove_archive_from_the_end(File.basename(from))+'/'
      opne "#{rev}Entering the directory "\
           "#{sdir(File.absolute_path(to))} #{rev}next."
      cd to
    end
    if File.exist?(from) and !File.directory?(from)
      opne swarn('Currently this code only works on already '\
           'extracted directories.')
      exit
    end
    append_these_configure_options = ''.dup
    commandline_arguments = commandline_arguments?
    commandline_arguments.reject! {|entry|
      entry == _ or
      entry.start_with?('--down') or
      entry.start_with?('--builddir')
    }
    these_arguments = hyphen_arguments(
      commandline_arguments
    ).join(' ')
    unless these_arguments.rstrip.empty?
      append_these_configure_options << these_arguments.rstrip
    end
    # ======================================================================= #
    # Obtain the program-name and program-version next:
    # ======================================================================= #
    program_name    = ProgramInformation.return_short_name(from).delete('-')
    program_version = ProgramInformation.return_version(from)
    # ======================================================================= #
    # === Determine the correct prefix next
    #
    # Next we must determine the proper prefix-target for PROGRAMS.
    # ======================================================================= #
    use_this_as_the_target_prefix = programs_directory?+
                                    program_name.capitalize+'/'+
                                    program_version+'/'
    # ======================================================================= #
    # We will pass this into RBT::CreateAppDirSkeleton next:
    # ======================================================================= #
    action(:create_appdir_skeleton, use_this_as_the_target_prefix)
    # ======================================================================= #
    # === Handle cmake next
    # ======================================================================= #
    if File.exist?('CMakeLists.txt') and !File.exist?('configure') and
      cmd.include?('configure')
      # ===================================================================== #
      # In this case there is no GNU configure script; only a cmake file
      # available, so use that.
      # ===================================================================== #
      cmd = 'cmake'
    # ======================================================================= #
    # Run the three sys-commands next, starting with "./configure". If
    # the working directory does NOT include a file called "configure",
    # but it DOES contain a file called 'autogen.sh' then we will first
    # invoke autogen before continuing.
    # ======================================================================= #
    elsif File.exist?('autogen.sh') and
         !File.exist?('configure') and
          cmd.include?('configure')
      e
      opne rev+'No '+sfile('configure')+rev+' file was found, but a file called'
      opne rev+sfile('autogen.sh')+
           "#{rev} was found. Thus, invoking autogen next."
      e
      padded_esystem 'sh autogen.sh'
    end
    cmd = cmd.dup if cmd.frozen?
    if cmd.include? 'cmake'
      # ===================================================================== #
      # Add the proper prefix.
      # ===================================================================== #
      cmd << ' -DCMAKE_INSTALL_PREFIX='+use_this_as_the_target_prefix
    else
      # ===================================================================== #
      # This part is valid for both GNU configure as well as meson.
      # ===================================================================== #
      cmd << ' --prefix='+use_this_as_the_target_prefix
    end
    unless cmd.include? append_these_configure_options
      cmd << " #{append_these_configure_options}" unless append_these_configure_options.empty?
    end
    if File.exist?('meson.build') and !File.exist?('configure')
      do_use_a_build_directory
      _ = "meson --prefix=#{use_this_as_the_target_prefix}".dup
      if append_these_configure_options and !append_these_configure_options.empty?
        _ << " #{append_these_configure_options}"
      end
      _ << " #{BUILD_DIRECTORY}" if use_a_build_directory?
      padded_esystem _
      cd BUILD_DIRECTORY if use_a_build_directory?
      run_ninja_command
      run_ninja_install_command
      return
    else
      # ===================================================================== #
      # Run the cmd via esystem() next:
      # ===================================================================== #
      if use_build_directory?
        mkdir(BUILD_DIRECTORY) unless File.directory?(BUILD_DIRECTORY)
        cd(BUILD_DIRECTORY) unless return_pwd.include?(BUILD_DIRECTORY)
      end
      padded_esystem cmd
      if cmd.start_with? 'meson'
        run_ninja_command
        run_ninja_install_command
      else # This clause is valid for GNU configure and cmake.
        cmd = 'make'
        padded_esystem(cmd)
        cmd = 'make install'
        padded_esystem(cmd)
      end
    end
  end

  # ========================================================================= #
  # === RBT::Action::SimpleAppdirConfigure[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.simple_appdir_configure
#
# This method can be used predominantly from a local directory, to
# invoke a "configure", "make" and "make install" run.
#
# Since as of 29.12.2019, the method also responds to --cmake as
# option.
#
# Example for this latter use case, from the commandline:
#
#   appconf --cmake
#
# If you use a build directory then you may wish to notify this method
# about it, via:
#
#   appconf --down
#
# Since as of February 2023, a new class was added, and the option
# "--use-meson" was added, to force the use of meson. The latter was
# necessary as a few programs come with support for both "configure"
# as well as "meson", so we need a commandline way to pick either
# one.
# =========================================================================== #
def self.simple_appdir_configure(i = ARGV)
  RBT::Action::SimpleAppdirConfigure.new(i)
end; self.instance_eval { alias simple_app_conf simple_appdir_configure } # === RBT.simple_app_conf

end

if __FILE__ == $PROGRAM_NAME
   RBT.simple_appdir_configure(ARGV)
end # simpleappdirconfigure htop
    # simpleappdirconfigure