#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/reset.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  require 'rbt/utility_scripts/build_directory.rb'

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # Note that the super() call will also define the variable
    # @internal_hash properly. Hence why it must come first,
    # as other definitions depend on it.
    # ======================================================================= #
    super()
    infer_the_namespace
    reset_the_invariant_dataset_to_the_internal_hash
    reset_user_definable_variables
    extract_to_default_directory
    disable_colours unless RBT.use_colours?
    set_program_path(nil) # This should come last within reset().
  end

  # ========================================================================= #
  # === populate_the_internal_hash_with_values_specific_to_this_class
  # ========================================================================= #
  def populate_the_internal_hash_with_values_specific_to_this_class
    reset_user_definable_variables
    reset_the_invariant_dataset_to_the_internal_hash
  end

  # ========================================================================= #
  # === reset_the_invariant_dataset_to_the_internal_hash      (invariant tag)
  #
  # The variants in this method are, by default, not amenable any user
  # input at all. They are thus only used internally by the class.
  #
  # This is thus the main difference to the other method used to fill
  # up @internal_hash.
  #
  # The values defined in the method here are mostly done to use default
  # values.
  #
  # Note that this may not be entirely true, so this method is a bit
  # arbitrary. Still, it is true for other internal entries, such as
  # :have_we_successfully_installed_the_program.
  # ========================================================================= #
  def reset_the_invariant_dataset_to_the_internal_hash
    # ======================================================================= #
    # === :make
    #
    # Instantiate a new RBT::Make object next, as our wrapper to 'make'.
    # ======================================================================= #
    @internal_hash[:make] = RBT::Make.new
    # ======================================================================= #
    # === :make_install_stage_was_a_success
    #
    # Whether the "make install" stage was a success. This is true by
    # default and will only become false if an error is encountered.
    # ======================================================================= #
    @internal_hash[:make_install_stage_was_a_success] = true
    # ======================================================================= #
    # === :has_the_program_been_installed_successfully
    #
    # By default, all programs will have been installed successfully. The
    # next variable keeps track of this.
    # ======================================================================= #
    @internal_hash[:has_the_program_been_installed_successfully] = true
    # ======================================================================= #
    # === :may_we_run_ninja
    #
    # By default we can always run ninja again.
    # ======================================================================= #
    @internal_hash[:may_we_run_ninja] = true
    # ======================================================================= #
    # === :may_we_initialize_a_new_cookbook_dataset
    #
    # The next variable must be true by default.
    # ======================================================================= #
    @internal_hash[:may_we_initialize_a_new_cookbook_dataset] = true
    # ======================================================================= #
    # === :pid
    # ======================================================================= #
    @internal_hash[:pid] = nil
    # ======================================================================= #
    # === use_gobolinux_symlinkprogram
    #
    # Next, determine whether we will make use of GoboLinux's SymlinkProgram.
    # This will be automatically changed if we are on GoboLinux. For other
    # systems we will prefer to use SymlinkProgram from RBT itself.
    # ======================================================================= #
    @internal_hash[:use_gobolinux_symlinkprogram] = false
    # ======================================================================= #
    # === :append_shell_error_code
    #
    # If this variable is set to true then we will append the shell
    # error code aka ' 2>&1'. This is true by default, so we will
    # append it.
    # ======================================================================= #
    @internal_hash[:append_shell_error_code] = true
    # ======================================================================= #
    # === :run_make_install
    #
    # By default, we will run "make install". This covers most programs
    # that are to be compiled from source.
    # ======================================================================= #
    @internal_hash[:run_make_install] = true
    # ======================================================================= #
    # === :try_to_recompile_upon_error
    # ======================================================================= #
    @internal_hash[:try_to_recompile_upon_error] = true
    # ======================================================================= #
    # === :dataset_from_the_file_cookbook_chained_programs
    #
    # This will contain a dataset that looks like this:
    #
    #   "lxqt"=> ["lxqtbuildtools",
    #             "libsysstat",
    #             "qtermwidget",
    #             "liblxqt",
    # ======================================================================= #
    @internal_hash[:dataset_from_the_file_cookbook_chained_programs] =
      YAML.load_file(FILE_COOKBOOK_CHAINED_PROGRAMS)
    # ======================================================================= #
    # === :have_we_successfully_installed_the_program
    #
    # The next variable keeps track over as to whether we have
    # successfully installed the program or whether we have not.
    #
    # By default we will assume this is not the case so, hence why the
    # value is false here.
    # ======================================================================= #
    @internal_hash[:have_we_successfully_installed_the_program] = false
    # ======================================================================= #
    # === :try_to_autofix
    # ======================================================================= #
    @internal_hash[:try_to_autofix] = true
    # ======================================================================= #
    # === :do_not_use_special_environment_flags
    #
    # Whether we are allowed to use special ENV flags. By default we are
    # allowed to use special flags, hence why the variable is set to false.
    # ======================================================================= #
    @internal_hash[:do_not_use_special_environment_flags] = false
    # ======================================================================= #
    # === :last_compiled_program
    # ======================================================================= #
    @internal_hash[:last_compiled_program] = nil  # Define it once at least.
    # ======================================================================= #
    # === :exit_after_a_rubygem_has_been_installed
    # ======================================================================= #
    @internal_hash[:exit_after_a_rubygem_has_been_installed] = false
    # ======================================================================= #
    # === :time_compilation
    #
    # Determine whether we shall time the compilation or whether we shall
    # not.
    # ======================================================================= #
    file_time_compilation = FILE_TIME_COMPILATION
    if File.exist? file_time_compilation
      @internal_hash[:time_compilation] = YAML.load_file(file_time_compilation)
      if @internal_hash[:time_compilation]
        @internal_hash[:array_time_compilation] = []
      end
    else
      opnn; no_file_exists_at(file_time_compilation)
    end
    # ======================================================================= #
    # === :prefix_object
    #
    # The :default_prefix Symbol equals to /usr/.
    # ======================================================================= #
    @internal_hash[:prefix_object] = action(:prefix, :default_prefix)
    # ======================================================================= #
    # The next method call can only happen after the prefix-object
    # was already initialized - see the line of code above this comment
    # section.
    # ======================================================================= #
    #set_user_prefix(nil)
    # ======================================================================= #
    # === :continue_after_make_step
    # ======================================================================= #
    @internal_hash[:continue_after_make_step] = true
    # ======================================================================= #
    # === :array_sys_command_string
    #
    # The next variable can keep track of sys-commands that were run.
    # ======================================================================= #
    @internal_hash[:array_sys_command_string] = []
    # ======================================================================= #
    # === :counter_called_traditional_or_non_traditional_compilation
    #
    # Next, how often we invoked traditional or non-traditional compilation.
    # ======================================================================= #
    @internal_hash[:counter_called_traditional_or_non_traditional_compilation] = 0
    # ======================================================================= #
    # === :full_configure_command
    #
    # We have to store the full configure command as well. It is not just
    # for configure but also for cmake, meson and so forth.
    # ======================================================================= #
    @internal_hash[:full_configure_command] = ''.dup
    # ======================================================================= #
    # === :original_start_directory
    # ======================================================================= #
    @internal_hash[:original_start_directory] = return_pwd
    # ======================================================================= #
    # === :is_an_abbreviation
    #
    # Whether the given input is an abbreviation or not. By default we
    # assume that the user has NOT used an abbreviated term for a real
    # program.
    # ======================================================================= #
    @internal_hash[:is_an_abbreviation] = false
    # ======================================================================= #
    # === :array_required_dependencies
    #
    # Register any reported dependencies of the target program at hand.
    # ======================================================================= #
    @internal_hash[:array_required_dependencies] = []
    # ======================================================================= #
    # === :run_make_command
    #
    # The next variable will be nil by default, so that we can grab the
    # value that the cookbook-dataset is using here.
    # ======================================================================= #
    @internal_hash[:run_make_command] = nil
    # ======================================================================= #
    # === :compile_these_programs
    # ======================================================================= #
    @internal_hash[:compile_these_programs] = []
    # ======================================================================= #
    # === :cookbook_aliases
    #
    # This entry is invariant and not amenable to the user modifying its
    # value.
    # ======================================================================= #
    @internal_hash[:cookbook_aliases] = action(:cookbook_aliases)
    # ======================================================================= #
    # === :simple_or_complex
    #
    # Determine the dataset to use from the cookbook dataset. This can
    # be :simple or :complex, although at a later time the alias name
    # :use_expanded_dataset is now in use, instead of :complex, as this
    # name is more explicit.
    # ======================================================================= #
    @internal_hash[:simple_or_complex] = :use_expanded_dataset # :simple
  end; alias initialize_default_values                      reset_the_invariant_dataset_to_the_internal_hash # === initialize_default_values
       alias add_the_invariant_dataset_to_the_internal_hash reset_the_invariant_dataset_to_the_internal_hash # === add_the_invariant_dataset_to_the_internal_hash

  # ========================================================================= #
  # === reset_user_definable_variables
  #
  # The variables in this method can be changed by the user lateron. They
  # are thus modifiable by the user via the commandline.
  #
  # In other words, the user can change these variables and thus toggle
  # the behaviour of RBT::Action::SoftwareManager. Thus, the user can overrule
  # any setting defined in this method here - this is important to 
  # retain for this method, hence why it is documented so extensively.
  #
  # Many variables in this method will be set to nil initially; that
  # way allows us to easily find out which values have been modified
  # via the commandline, and which ones have not.
  # ========================================================================= #
  def reset_user_definable_variables
    # ======================================================================= #
    # === :cookbook_dataset
    # ======================================================================= #
    @internal_hash[:cookbook_dataset] = nil
    # ======================================================================= #
    # === :install_only_binaries
    # ======================================================================= #
    @internal_hash[:install_only_binaries] = false
    # ======================================================================= #
    # === :skip_extracting
    #
    # Default value is nil for the next variable. If true then we will
    # NOT extract.
    # ======================================================================= #
    @internal_hash[:skip_extracting] = nil
    # ======================================================================= #
    # === :time_compilation
    #
    # Whether to time the compilation procedure or not:
    # ======================================================================= #
    @internal_hash[:time_compilation] = true
    # ======================================================================= #
    # === :honour_check_to_ensure_that_the_program_exists
    #
    # The next variable has to be kept for if we actually do want to
    # check whether a program is included or not. Normally we should
    # check, but there may be situations where you may build up your
    # own functionality - so we need to provide an option that can be
    # toggled by the user. This is why the next variable exists, and
    # is kept at true.
    # ======================================================================= #
    @internal_hash[:honour_check_to_ensure_that_the_program_exists] = true
    # ======================================================================= #
    # === :check_whether_the_given_input_is_a_file_and_does_exist
    #
    # The next variable, if true, instructs the class to check whether the
    # given input argument(s) do exist as a regular local file. If false
    # this check is skipped.
    # ======================================================================= #
    @internal_hash[:check_whether_the_given_input_is_a_file_and_does_exist] = true
    # ======================================================================= #
    # === :run_simulation
    #
    # Whether we will run in simulation-mode or not. Simulation-mode means
    # that we will not make any real lasting changes. This thus allows a
    # user to predict (and show a summary) what would happen if a
    # program would be installed/compiled.
    # ======================================================================= #
    @internal_hash[:run_simulation] = false
    # ======================================================================= #
    # === :exit_before_make_install
    #
    # Whether we will exit when we reach the "make install" stage.
    # ======================================================================= #
    @internal_hash[:exit_before_make_install] = false
    # ======================================================================= #
    # === :exit_before_make
    #
    # Whether we will exit when we reach the "make" stage.
    # ======================================================================= #
    @internal_hash[:exit_before_make] = false
    # ======================================================================= #
    # Only install the .h header files, if this variable is set to true.
    # ======================================================================= #
    @internal_hash[:install_only_the_headers] = false
    # ======================================================================= #
    # === :we_tried_to_compile_this_program_n_times
    # ======================================================================= #
    @internal_hash[:we_tried_to_compile_this_program_n_times] = 0
    # ======================================================================= #
    # === :run_configure_script
    #
    # Determine whether we will run the configure script or whether we
    # will not. This is nil initially, so that the user can overrule
    # this in any way he/she sees fit.
    # ======================================================================= #
    @internal_hash[:run_configure_script] = nil
    # ======================================================================= #
    # === :configure_base_directory
    # ======================================================================= #
    @internal_hash[:configure_base_directory] = nil
    # ======================================================================= #
    # === :use_this_build_system
    #
    # Determine which build system is to be used. By default it will be nil.
    # ======================================================================= #
    @internal_hash[:use_this_build_system] = nil
    # ======================================================================= #
    # === :build_directory
    # ======================================================================= #
    @internal_hash[:build_directory] = BuildDirectory.new { :do_not_run_yet }
    # ======================================================================= #
    # === :extract_where_to
    # ======================================================================= #
    @internal_hash[:extract_where_to] = RBT.log_dir?+'rbt/' # TEMP_DIR_FOR_RBT
    # ======================================================================= #
    # === :symlink_into_usr_lib_dir
    #
    # Determine whether we will symlink the content of lib/ into 
    # /usr/lib/. This setting is only honoured if we compile in an
    # appdir-like fashion.
    # ======================================================================= #
    @internal_hash[:symlink_into_usr_lib_dir] = false
    # ======================================================================= #
    # === :use_autoconf
    #
    # Use autoconf or not - by default we will not.
    # ======================================================================= #
    @internal_hash[:use_autoconf] = nil
    # ======================================================================= #
    # === :symlink_glib_schema
    # ======================================================================= #
    @internal_hash[:symlink_glib_schema] = nil
    # ======================================================================= #
    # === :parameters_to_make
    #
    # The next entry point allows us to pass options/parameters to the
    # "make" binary. The user can overrule this on the commandline.
    #
    # A value of nil is the initialize default state for this variable.
    # ======================================================================= #
    @internal_hash[:parameters_to_make] = nil
    # ======================================================================= #
    # === :rename_tab_title
    #
    # Whether to rename the tab title or whether we will not. This refers
    # to the KDE konsole typically.
    # ======================================================================= #
    if RBT.respond_to? :configuration?
      @internal_hash[:rename_tab_title] = RBT.configuration?.rename_tab_title
    else
      @internal_hash[:rename_tab_title] = true
    end
    # ======================================================================= #
    # === :cd_into_the_rbt_log_directory
    #
    # Whether this class will try to cd into the rbt-log directory.
    # ======================================================================= #
    @internal_hash[:cd_into_the_rbt_log_directory] = true
    # ======================================================================= #
    # === :try_to_determine_which_programs_have_to_be_installed
    # ======================================================================= #
    @internal_hash[:try_to_determine_which_programs_have_to_be_installed] = true
    # ======================================================================= #
    # === :make_use_of_cookbook_aliases
    # ======================================================================= #
    @internal_hash[:make_use_of_cookbook_aliases] = true
    # ======================================================================= #
    # === :run_postinstall_actions
    #
    # Determine whether we will run postinstallation steps.
    # ======================================================================= #
    @internal_hash[:run_postinstall_actions] = true
    # ======================================================================= #
    # === :create_appdir_skeleton
    #
    # Whether to create an AppDir skeleton by default.
    # ======================================================================= #
    @internal_hash[:create_appdir_skeleton] = true
    # ======================================================================= #
    # === :shall_we_compile_the_program
    #
    # The next variable can be used to determine whether we will compile
    # a particular program or whether we will not.
    # ======================================================================= #
    @internal_hash[:shall_we_compile_the_program] = true
    # ======================================================================= #
    # === :use_which_system_command
    #
    # There are only two valid settings for this variable:
    #
    #    :ioparser or 
    #    :system
    #
    # ======================================================================= #
    @internal_hash[:use_which_system_command] = :ioparser
    # ======================================================================= #
    # === :colourize_parser
    #
    # Add an instance of class RBT::ColourizeParser.
    # ======================================================================= #
    @internal_hash[:colourize_parser] = RBT::ColourizeParser.new
    # ======================================================================= #
    # === :strip_binaries
    #
    # By default we will not strip the binaries. The configuration
    # setting can change this value though.
    # ======================================================================= #
    @internal_hash[:strip_binaries] = false
    # ======================================================================= #
    # === :use_a_prefix
    #
    # This entry point allows us to use a prefix variable, e. g for
    # "./configure --prefix=/opt". By default this is enabled (set
    # to true), but a user may wish to disable this too.
    #
    # If you wish to query this value, use the method
    # .do_we_use_any_prefix?.
    # ======================================================================= #
    @internal_hash[:use_a_prefix] = true
    # ======================================================================= #
    # === :is_on_gobolinux
    #
    # Determine whether we are on GoboLinux or whether we are not.
    # ======================================================================= #
    @internal_hash[:is_on_gobolinux] = RBT.are_we_on_gobolinux?
    # ======================================================================= #
    # === :show_output_from_the_extracter_class
    #
    # If the following variable is set to true then we will show output
    # from class Extracter. This is the class that will handle
    # extracting of an archive. The reason why we have this binary
    # flag (toggle) is because sometimes we do not want to output
    # a LOT of information on the commandline. The user can toggle
    # this behaviour on the commandline.
    # ======================================================================= #
    @internal_hash[:show_output_from_the_extracter_class] = false
    # ======================================================================= #
    # === :use_porg
    #
    # Determine whether we will make use of porg.
    # ======================================================================= #
    @internal_hash[:use_porg] = configuration?.use_porg
    # ======================================================================= #
    # === :symlink_headers
    # ======================================================================= #
    @internal_hash[:symlink_headers] = nil
    # ======================================================================= #
    # === :check_for_this_licence
    #
    # If the following variable is not nil, and not false, then this
    # class will check for the given licence matching, before it will
    # compile the program at hand. So, for instance, the user passes the
    # commandline flag for GPLv3 licences. If the software at hand is
    # NOT GPLv3 licenced then it can not be compiled. This is an optional
    # setting - most users won't need this, but purists may want to have
    # such an option, so it has been enabled in September 2022.
    # ======================================================================= #
    @internal_hash[:check_for_this_licence] = nil
    # ======================================================================= #
    # === :can_we_continue
    #
    # The variable at :can_we_continue is reset during every iteration.
    # It will be true by default.
    # ======================================================================= #
    @internal_hash[:can_we_continue] = true
    # ======================================================================= #
    # === :original_stdout
    #
    # The next variable is a bit unclear - it used to exist to allow for
    # easy redirection, but may no longer be that important. We may have to
    # re-evaluate this at a later time (written at 12.04.2019).
    # ======================================================================= #
    @internal_hash[:original_stdout] = nil
    # ======================================================================= #
    # === :postinstallation
    # ======================================================================= #
    @internal_hash[:postinstallation] = nil
    # ======================================================================= #
    # === @internal_hash[:make_use_of_the_cookbook_aliases]
    #
    # If the following variable is set to true then the RBT project
    # will try to honour the cookbook aliases.
    # ======================================================================= #
    @internal_hash[:make_use_of_the_cookbook_aliases] = true
    # ======================================================================= #
    # === :sed
    #
    # Whether to apply a sed-operation. Will be nil by default. If the
    # user passes in some special option here, then we know that we
    # have to use this value to overrule any other value for sed.
    # ======================================================================= #
    @internal_hash[:sed] = nil
    # ======================================================================= #
    # === :keep_la_files
    #
    # The next flag determines whether we will delete .la files or whether
    # we will keep them.
    # ======================================================================= #
    @internal_hash[:keep_la_files] = nil
    # ======================================================================= #
    # === :keep_the_archive_extracted
    #
    # The keep-extracted setting will lateron be sanitized. The default
    # value will be nil.
    # ======================================================================= #
    @internal_hash[:keep_the_archive_extracted] = nil
    # ======================================================================= #
    # === :may_we_download
    # ======================================================================= #
    @internal_hash[:may_we_download] = RBT::TRY_TO_DOWNLOAD_IF_NOT_FOUND
    # ======================================================================= #
    # === :ignore_errors
    #
    # By default we will NOT ignore errors, thus stop when we have noticed
    # an error. This can be overruled by the user on the commandline,
    # through the following value.
    #
    # Since as of November 2022, a new option has been added - this
    # variable will now default to :infer. That way the user can
    # override it on the commandline, and we can call the method
    # cookbook_dataset_ignore_errors? anyway.
    # ======================================================================= #
    @internal_hash[:ignore_errors] = :infer # false
    # ======================================================================= #
    # === :extract_to_this_directory_as_specified_by_the_user
    #
    # The next variable can be used to allow the user to determine another
    # directory for where we will extract archives into. Initially set
    # to nil in order to allow distinction to the default extractable
    # directory that is to be used.
    # ======================================================================= #
    @internal_hash[:extract_to_this_directory_as_specified_by_the_user] = nil
    # ======================================================================= #
    # === :use_autoconf
    # ======================================================================= #
    @internal_hash[:use_autoconf] = false
    # ======================================================================= #
    # === :extract_the_archive
    #
    # By default we will extract the archive, hence the value must be
    # set to true.
    # ======================================================================= #
    @internal_hash[:extract_the_archive] = true
    # ======================================================================= #
    # === :do_not_use_special_flags
    #
    # The next variable, false on startup, tells RBT::Action::SoftwareManager
    # to retain all environment variables. Sometimes we may want to NOT
    # use the environment variables, which is why this flag exists.
    # ======================================================================= #
    @internal_hash[:do_not_use_special_flags] = false
    # ======================================================================= #
    # === :use_autogen
    #
    # Default value for use_autogen will be nil. The user is able to
    # override this from the commandline, which is why this variable
    # will be nil initially.
    # ======================================================================= #
    @internal_hash[:use_autogen] = nil
    # ======================================================================= #
    # === :compiler_to_use
    #
    # Specify which compiler is to be used. By default we will use GCC,
    # but clang may also be an option. It will be stored as a lowercased
    # Symbol, though, so :gcc would refer to GCC.
    # ======================================================================= #
    @internal_hash[:compiler_to_use] = :gcc
    # ======================================================================= #
    # === :use_build_directory
    #
    # Must initialize the build directory to a nil value next. It may
    # be set to a non-nil value lateron.
    # ======================================================================= #
    @internal_hash[:use_build_directory] = nil
    # ======================================================================= #
    # === :name_for_the_build_directory
    #
    # Use a default build directory. There is another entry that determines
    # whether we will use any build directory at all or not, so it is safe
    # to assign to the variable here - the entry called
    # @internal_hash[:use_build_directory].
    #
    # Since as of April 2021, we will only use the setter-method for this.
    # See the next line, right below the commented line.
    # ======================================================================= #
    set_name_for_the_build_directory(:use_the_default_name) # Set a default name for the build directory.
    # ======================================================================= #
    # === :build_static
    #
    # The next variable must be defined here. A value of nil specifies
    # that this has not been set via the commandline yet. A value of
    # nil here essentially means a default to false - aka we will not
    # try to compile a program in a static manner. 
    # ======================================================================= #
    @internal_hash[:build_static] = nil
    # ======================================================================= #
    # === :use_ccache
    #
    # The use_ccache entry has to be persistent.
    # ======================================================================= #
    @internal_hash[:use_ccache] = RBT.configuration?.use_ccache
    # ======================================================================= #
    # === :try_to_use_configure_options
    #
    # Determine whether we will try to use any declared configure options
    # from the cookbook-yaml file.
    #
    # The reason why this variable is set to nil initially is so that we
    # can distinguish the situation from when the user did specifically
    # want to disable this functionality, via the commandline. So the
    # nil value does not necessarily mean "false" in this context - it
    # just means that the user did not specify anything specifically.
    # ======================================================================= #
    @internal_hash[:try_to_use_configure_options] = nil
    # ======================================================================= #
    # === :extract_only
    #
    # If the next variable is set to true, we will exit after we have
    # extracted the archive. We will first initialize it to nil, though,
    # in order to distinguish between the default value and user-supplied
    # values.
    # ======================================================================= #
    @internal_hash[:extract_only] = nil
    # ======================================================================= #
    # === Show the flexbox
    #
    # Next determine whether we will show the flexbox or whether we will
    # not. This has to be set in the method here so that we do not
    # re-set it for each program that is to be compiled/installed.
    # ======================================================================= #
    @internal_hash[:show_the_flexbox] = true
    # ======================================================================= #
    # === :use_this_as_build_directory
    # ======================================================================= #
    @internal_hash[:use_this_as_build_directory] = 'BUILD_DIRECTORY'
    # ======================================================================= #
    # === :force_symlinks_when_appdir_prefix_is_used
    # ======================================================================= #
    if is_on_roebe?
      @internal_hash[:force_symlinks_when_appdir_prefix_is_used] = true
    else
      @internal_hash[:force_symlinks_when_appdir_prefix_is_used] = false
    end
    # ======================================================================= #
    # === :shall_we_use_an_appdir_prefix
    #
    # This variable must be set to true if we make use of an AppDir
    # prefix.
    # ======================================================================= #
    @internal_hash[:shall_we_use_an_appdir_prefix] = false
    # ======================================================================= #
    # === :use_meson_configure_options
    #
    # Whether to use meson-configure options or not.
    # ======================================================================= #
    @internal_hash[:use_meson_configure_options] = true
    # ======================================================================= #
    # === :symlink_gir_files
    #
    # This entry point controls whether we will symlink .gir files,
    # in principle.
    # ======================================================================= #
    @internal_hash[:symlink_gir_files] = configuration_setting_for_symlink_gir_files?
    # ======================================================================= #
    # === :register_the_last_compiled_program
    #
    # Whether to register the last compiled program or not.
    # ======================================================================= #
    @internal_hash[:register_the_last_compiled_program] = true
    # ======================================================================= #
    # === :use_this_specific_program_version
    #
    # Use another program version than the default one via this
    # variable.
    # ======================================================================= #
    @internal_hash[:use_this_specific_program_version] = nil
    # ======================================================================= #
    # === :append_this_to_the_configure_options
    #
    # This variant has been specifically added to allow for the --release
    # flag to work with cmake-based projects.
    # ======================================================================= #
    @internal_hash[:append_this_to_the_configure_options] = nil
    # ======================================================================= #
    # === :use_opn
    #
    # This entry point determines whether RBT::Action::SoftwareManager will make
    # use of opn or whether it will not. By default it will be true; if it
    # is false then we will not show opn in any other class that is called
    # by RBT::Action::SoftwareManagerRBT::Action::SoftwareManagerRBT::Action::SoftwareManager.
    # ======================================================================= #
    @internal_hash[:use_opn] = true
    # ======================================================================= #
    # === :user_prefix
    #
    # This prefix can be determined by the user.
    # ======================================================================= #
    @internal_hash[:user_prefix] = nil
    # ======================================================================= #
    # === :extract_only
    # ======================================================================= #
    @internal_hash[:extract_only] = false
    # ======================================================================= #
    # === :configure_options
    #
    # Whether we will use special configure options or not. By default
    # we will not, hence why we initialize the next setting to nil.
    # ======================================================================= #
    @internal_hash[:configure_options] = nil
    # ======================================================================= #
    # === :use_the_internal_sed_wrapper
    #
    # Next, determine whether to use the internal sed-wrapper. No
    # longer true by default since as of February 2018, but perhaps
    # we may re-enable it at some later moment in time.
    # ======================================================================= #
    @internal_hash[:use_the_internal_sed_wrapper] = false
    # ======================================================================= #
    # The configuration setting can change the above default false-value.
    # ======================================================================= #
    if config?
      @internal_hash[:strip_binaries] = config?.use_strip
    end
    # ======================================================================= #
    # The next variable will keep track of class SymlinkProgram. We
    # need to be able to modify its behaviour at "runtime".
    # ======================================================================= #
    set_symlink_program(nil)
    set_use_this_make_command(nil)
  end; alias add_the_default_dataset_to_the_internal_hash reset_user_definable_variables # === add_the_default_dataset_to_the_internal_hash

  # ========================================================================= #
  # === reset_compiled_n_times_counter
  # ========================================================================= #
  def reset_compiled_n_times_counter
    @internal_hash[:we_tried_to_compile_this_program_n_times] = 0
  end

  # ========================================================================= #
  # === reset_the_cookbook_dataset
  # ========================================================================= #
  def reset_the_cookbook_dataset
    @internal_hash[:cookbook_dataset] = nil
  end

end; end; end
