#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::SoftwareManager
#
# RBT::Action::SoftwareManager aims to replace the older Installer class.
# The key idea for SoftwareManager is to make all of its relevant
# activities a rbt-action. The class can be used to install or otherwise
# compile a program, hence its semi-generic name being "SoftwareManager".
#
# By default, the GNU "configure", "make" and "make install" steps are
# used by this class. 
#
# The first argument to this class should be the name of the program or
# the name of the programs that ought to be compiled. This may be
# converted into an Array, internally, so that this class can operate
# in a batch-oriented matter, compiling several programs in one go.
#
# Basic usage example:
#
#   RBT::Action::SoftwareManager.new(ARGV)
#   RBT::Action::SoftwareManager.new(:htop)
#   RBT::Action::SoftwareManager.new('ruby')
#   glib = RBT::Action::SoftwareManager.new 'glib'
#   sed = RBT::Action::SoftwareManager.new 'sed'
#   php  = RBT::Action::SoftwareManager.new 'php'
#
# A more "modern" use case, via actions, goes like this:
#
#   glibc = action(:SoftwareManager, :glibc)
#   glibc = action(:Installer, :glibc) # :Installer is now an alias.
#
# Commandline invocation examples:
#
#   ry wxwid
#   ry PYTHON gobo # ← input will be lowercased anyway
#   ry PYTHON trad
#   ry PYTHON --ntrad
#   ry bmp oldschool
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/software_manager.rb'
# RBT::Action::SoftwareManager.new(ARGV)
# RBT.software_manager(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  require 'rbt/compile_base/compile_base_module.rb'
  include RBT::CompileBaseModule

  require 'rbt/compile/constants.rb'
  include RBT::Compile::Constants

  unless Object.const_defined? :ToTarXz
    begin # Try to pull in gem to_tar_xz here.
      require 'totarxz'
    rescue LoadError; end
  end

  require 'rbt/actions/individual_actions/software_manager/initialize.rb'
  require 'rbt/actions/individual_actions/software_manager/extract_related_code.rb'
  require 'rbt/actions/individual_actions/software_manager/logic_related_code.rb'
  require 'rbt/actions/individual_actions/software_manager/menu.rb'
  require 'rbt/actions/individual_actions/software_manager/misc.rb'
  require 'rbt/actions/individual_actions/software_manager/query_related_methods.rb'
  require 'rbt/actions/individual_actions/software_manager/reset.rb'
  require 'rbt/toplevel_methods/opn.rb'

  begin
    require 'roebe/requires/require_kde_konsole.rb'
  rescue LoadError
    if ENV['IS_ROEBE'].to_s == '1'
      puts 'roebe/requires/require_kde_konsole.rb is unavailable '\
           '(should be part of the roebe gem).'
    end
  end # ^^^ it is optional, hence why the check occurs.

  begin
    require 'ftp_paradise/without_gui_bindings'
  rescue LoadError; end

end; end

# =========================================================================== #
# === RBT.software_manager
#
# This method must be guaranteed to always be a direct wrapper
# over class RBT::Action::SoftwareManager.
# =========================================================================== #
def self.software_manager(
    i = ARGV,
    &block
  )
  RBT::Action::SoftwareManager.new(i, &block)
end; self.instance_eval { alias installer software_manager } # === RBT.installer

end

if __FILE__ == $PROGRAM_NAME
  RBT::Action::SoftwareManager.new(ARGV)
end # SM htop --ntrad
    # ry htop@2.3.3
    # ry xorg_server
    # ry mate1
    # ry libarch
    # ry glib-2.74.0
    # ry glib-2.72.3.tar.xz
    # ry glib@2.72.3
    # ry lxqt
    # ry htop ntrad
    # ry ccache
    # ry ccache ntrad
