#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Statistics::ShowHowManyFilesAreTracked
#
# This class will simply report how many files are tracked by the RBT
# project.
#
# Example output (last updated July 2020):
#
#   3686 programs are registered.
#   5282 binaries are registered.
#   22657 .h header files are registered.
#   10979 library files (.so, a.) are registered.
#
# Update in February 2024:
#
#   3803 programs are registered.
#   5745 binaries are registered.
#   25264 .h header files are registered.
#   12319 .so / .a library files are registered.
#    127 .gir files are registered.
#     16 .m4 files are registered.
#
# Usage example:
#
#   RBT::Action::Statistics::ShowHowManyFilesAreTracked.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/statistics/show_how_many_files_are_tracked.rb'
# RBT::Action::Statistics::ShowHowManyFilesAreTracked.new(ARGV)
# RBT.show_how_many_files_are_tracked
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Statistics

class ShowHowManyFilesAreTracked < RBT::Action # === RBT::Action::Statistics::ShowHowManyFilesAreTracked

  require 'rbt/toplevel_methods/all_binaries.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks given
    # ======================================================================= #
    if block_given?
      case yield
      # ===================================================================== #
      # === :do_not_use_opn
      # ===================================================================== #
      when :do_not_use_opn
        @internal_hash[:use_opn] = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :use_opn
    # ======================================================================= #
    @internal_hash[:use_opn] = true
  end

  # ========================================================================= #
  # === report_how_many_header_files_are_registered
  # ========================================================================= #
  def report_how_many_header_files_are_registered
    i = 0
    # ======================================================================= #
    # We must load the files from the expanded cookbook directory.
    # ======================================================================= #
    target_directory = directory_expanded_cookbooks?
    use_this_key = :headers
    Dir["#{target_directory}*.yml"].each {|this_yaml_file|
      dataset = load_yaml(this_yaml_file)
      if dataset.has_key?(use_this_key)
        headers = dataset[use_this_key]
        unless headers.empty?
          i += headers.size
        end
      end
    }
    e "#{rev}There are #{steelblue(i.to_s.rjust(5))} "\
      "#{tomato('.h header files')} "\
      "#{rev}registered in the RBT project."
  end

  # ========================================================================= #
  # === report_how_many_gir_files_are_registered
  # ========================================================================= #
  def report_how_many_gir_files_are_registered
    i = 0
    # ======================================================================= #
    # We must load the files from the expanded cookbook directory.
    # ======================================================================= #
    target_directory = directory_expanded_cookbooks?
    use_this_key = :gir_files
    Dir["#{target_directory}*.yml"].each {|this_yaml_file|
      dataset = load_yaml(this_yaml_file)
      if dataset.has_key?(use_this_key)
        gir_files = dataset[use_this_key]
        unless gir_files.empty?
          i += gir_files.size
        end
      end
    }
    e rev+'There are '+steelblue(i.to_s.rjust(5))+
      rev+' '+steelblue('.gir files')+
      " #{rev}registered in the RBT project."
  end

  # ========================================================================= #
  # === report_how_many_m4_files_are_registered
  # ========================================================================= #
  def report_how_many_m4_files_are_registered
    i = 0
    # ======================================================================= #
    # We must load the files from the expanded cookbook directory.
    # ======================================================================= #
    target_directory = directory_expanded_cookbooks?
    use_this_key = :m4_files
    Dir["#{target_directory}*.yml"].each {|this_yaml_file|
      dataset = load_yaml(this_yaml_file)
      if dataset.has_key?(use_this_key)
        m4_files = dataset[use_this_key]
        unless m4_files.empty?
          i += m4_files.size
        end
      end
    }
    e rev+'There are '+steelblue(i.to_s.rjust(5))+
      rev+' '+steelblue('.m4 files')+rev+
      ' registered in the RBT project.'
  end

  # ========================================================================= #
  # === report_how_many_programs_are_registered
  # ========================================================================= #
  def report_how_many_programs_are_registered
    e rev+'There are '+steelblue(RBT.n_registered_programs?.to_s.rjust(5))+
      rev+' programs registered in total in the RBT project.'
  end

  # ========================================================================= #
  # === report_how_many_binaries_are_registered
  # ========================================================================= #
  def report_how_many_binaries_are_registered
    e rev+'There are '+steelblue(RBT.all_binaries.keys.uniq.size.to_s.rjust(5))+
      "#{rev} binaries registered in the RBT project."
  end

  # ========================================================================= #
  # === report_how_many_library_files_are_registered
  # ========================================================================= #
  def report_how_many_library_files_are_registered
    i = 0
    # ======================================================================= #
    # We must load the files from the expanded cookbook directory.
    # ======================================================================= #
    target_directory = directory_expanded_cookbooks?
    use_this_key = :libraries
    Dir["#{target_directory}*.yml"].each {|this_yaml_file|
      dataset = load_yaml(this_yaml_file)
      if dataset.has_key?(use_this_key)
        libraries = dataset[use_this_key]
        unless libraries.empty?
          i += libraries.size
        end
      end
    }
    e "#{rev}There are #{steelblue(i.to_s.rjust(5))} "\
      "#{tomato('.so / .a library files')} #{rev}"\
      "registered in the RBT project."
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if @internal_hash[:use_opn]
      opnn { :no_trailing }
      e
    end
    e
    report_how_many_programs_are_registered      # n programs
    report_how_many_binaries_are_registered      # n binaries
    report_how_many_header_files_are_registered  # n header files
    report_how_many_library_files_are_registered # n library files
    report_how_many_gir_files_are_registered     # n gir files
    report_how_many_m4_files_are_registered      # n m4 files
    e
  end

  # ========================================================================= #
  # === RBT::Action::Statistics::ShowHowManyFilesAreTracked[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.show_how_many_files_are_tracked
# =========================================================================== #
def self.show_how_many_files_are_tracked
  RBT::Action::Statistics::ShowHowManyFilesAreTracked.new { :do_not_use_opn }
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_how_many_files_are_tracked
end # showhowmanyfilesaretracked