#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::ToCurrent
#
# This class is able to create a new 'Current' symlink in the /Programs/
# hierarchy (or any other hierarchy, depending on where the programs
# base directory has been set).
#
# You can use a short input such as "python" but you can also pass
# a full path to this class, such as "/Programs/Python/3.6.2". The
# latter means that this version will become the new target of the
# Current symlink. I personally recommend to provide the full path
# since there will be less ambiguity that way.
#
# Usage examples:
#
#   RBT::Action::ToCurrent.new(ARGV)
#   RBT.to_current(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/to_current/to_current.rb'
# RBT.to_current(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class ToCurrent < RBT::Action # === RBT::Action::ToCurrent

  require 'rbt/utility_scripts/symlink_from_to_current/symlink_from_to_current.rb'

  # ========================================================================= #
  # === initialize
  #
  # The input to this method should be a String such as
  # "/Programs/Python/3.6.2/".
  # ========================================================================= #
  def initialize(
      program_version = nil,
      run_already     = true
    )
    reset
    set_symlink_this_program_version(program_version.dup)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_register
      # ===================================================================== #
      when :do_not_register
        @internal_hash[:also_do_register_into_the_yaml_database] = false
      else
        if yielded.is_a? Hash
          # ================================================================= #
          # === :use_this_symlink_program_instance
          # ================================================================= #
          if yielded.has_key? :use_this_symlink_program_instance
            set_symlink_program(
              yielded.delete(:use_this_symlink_program_instance)
            )
          end
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :also_do_register_into_the_yaml_database
    # ======================================================================= #
    @internal_hash[:also_do_register_into_the_yaml_database] = true
    # ======================================================================= #
    # === @symlink_program_to_use
    # ======================================================================= #
    @symlink_program_to_use = RBT::SymlinkFromToCurrent.new(nil, :do_not_run_yet)
    set_program_name # Must initialize it at the least once.
  end

  # ========================================================================= #
  # === set_symlink_program
  #
  # This method can be used to assign towards the symlink-program that
  # will be used. For example, class RBT::Compile can pass its own
  # modified variant into this method here, thus being able to control
  # what class RBT::ToCurrent is doing at the symlink-progam step.
  # ========================================================================= #
  def set_symlink_program(i)
    # ======================================================================= #
    # We really set to the class that we want to use here, so the
    # next line is correct.
    #
    # We default to class RBT::SymlinkFromToCurrent since as of
    # April 2021.
    # ======================================================================= #
    @symlink_program_to_use = i
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @symlink_this_program_version
  end

  # ========================================================================= #
  # === set_program_name
  #
  # The setter of the program name.
  # ========================================================================= #
  def set_program_name(i = nil)
    i = i.first if i.is_a? Array
    if i.nil?
      # ===================================================================== #
      # In this case, we will try to obtain the program name from
      # the target directory.
      # ===================================================================== #
      _ = return_pwd
      if _.include? programs_dir? and
         (_.count('/') > 1)
         i = _
      end
    end
    i = i.to_s.dup
    i = File.basename(i) if i.include? '/'
    i.capitalize!
    @program_name = i
  end

  # ========================================================================= #
  # === set_symlink_this_program_version
  #
  # This setter keeps track of the particular program version that
  # we will use. For example, if we have an input such as 
  # "/Programs/Python/3.6.2", then the program_version will be
  # 3.6.2.
  # ========================================================================= #
  def set_symlink_this_program_version(i)
    i = i.first if i.is_a? Array
    # ======================================================================= #
    # === Safeguard against nil-entries
    # ======================================================================= #
    if i.nil? # fetch random entry
      entries = Dir['*'] # Fetch all entries first.
      if entries.size > 1 # Try to find a number
        i = entries.select {|line| line =~ /\d+/ }.first
      else
        i = entries.first
      end
    end
    # ======================================================================= #
    # === Get rid of trailing '/' tokens.
    # ======================================================================= #
    if i.end_with? '/'
      i = i.dup if i.frozen?
      i.chop!
    end
    # ======================================================================= #
    # === Counter how many '/' are in the given input
    #
    # Example: /home/Programs/Xev/1.2.4
    # ======================================================================= #
    if i.include? programs_dir? and
       (i =~ /\d+/) and
       (i.count('/') > 1) # Has more than one '/' token.
      # ===================================================================== #
      # In this case, we assume that the user has input something
      # such as "/Programs/Python/3.6.2".
      # ===================================================================== #
      splitted = i.split('/').reject(&:empty?)
      # ===================================================================== #
      # Example for splitted:
      #
      #   ["home", "Programs", "Xev", "1.2.4"]
      #
      # ===================================================================== #
      use_this_program_name = splitted[-2]
      set_program_name(use_this_program_name)
      i = splitted[-1]
    end
    unless (i =~ /\d+/)
      # ===================================================================== #
      # Set the name of the program e. g. if we input only "python" here.
      # ===================================================================== #
      set_program_name(i) unless i.include? '/'
      i = infer_the_program_version # Must infer the version in this case.
    end
    @symlink_this_program_version = i
  end; alias set_program_version set_symlink_this_program_version # === set_program_version

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end

  # ========================================================================= #
  # === considering_deleting_the_current_symlink_if_it_exists
  # ========================================================================= #
  def considering_deleting_the_current_symlink_if_it_exists
    _ = current_target? # Get the symlink target.
    if File.symlink? _
      orev "There already is a symlink at `#{sfile(_)}#{rev}`. "\
           "It will be removed next."
      delete(_)
    end
  end

  # ========================================================================= #
  # === do_create_the_new_symlink
  # ========================================================================= #
  def do_create_the_new_symlink
    # ======================================================================= #
    # Next, create a new symlink.
    # ======================================================================= #
    _ = current_target? # Get the symlink target.
    orev "Creating a new symlink at `#{sfile(_)}#{rev}`."
    new_target = full_program_version_target?
    orev "This symlink will point towards `#{sfile(new_target)}#{rev}`."
    if File.directory? new_target
      symlink(program_version?, _)
      # ===================================================================== #
      # Consider invoking rnsymc as well. 
      # ===================================================================== #
      rnsymc(new_target)
    end
  end

  # ========================================================================= #
  # === current_target?
  #
  # This method will assemble the target residing under the /Programs
  # hierarchy.
  # ========================================================================= #
  def current_target?
    "#{programs_dir?}#{program_name?}/Current"
  end

  # ========================================================================= #
  # === infer_the_program_version
  # ========================================================================= #
  def infer_the_program_version
    _ = Dir[programs_dir?+program_name?+'/*'].select {|entry|
      File.directory? entry
    }.sort.reverse
    version = File.basename(_.first)
    return version
  end 

  # ========================================================================= #
  # === full_program_version_target?
  #
  # This method is assumed to point to a directory.
  # ========================================================================= #
  def full_program_version_target?
    "#{programs_dir?}#{program_name?.capitalize}/#{program_version?}/"
  end

  # ========================================================================= #
  # === rnsymc
  #
  # This method delegates towards class RBT::SymlinkFromToCurrent.
  #
  # The argument to this method will default to a String such as:
  #
  #   /home/Programs/Python/3.8.3/
  #
  # ========================================================================= #
  def rnsymc(
      i = full_program_version_target?
    )
    @symlink_program_to_use.set_use_this_program(i)
    if also_do_register_into_the_yaml_database?
      # Pass through in this case.
    else
      @symlink_program_to_use.do_not_register
    end
    @symlink_program_to_use.do_symlink
  end

  # ========================================================================= #
  # === also_do_register_into_the_yaml_database?
  # ========================================================================= #
  def also_do_register_into_the_yaml_database?
    @internal_hash[:also_do_register_into_the_yaml_database]
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    considering_deleting_the_current_symlink_if_it_exists
    do_create_the_new_symlink
  end

  # ========================================================================= #
  # === RBT::Action::ToCurrent[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.to_current
# =========================================================================== #
def self.to_current(
    i            = ARGV,
    options_hash = {},
    &block
  )
  RBT::Action::ToCurrent.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours'
  require 'cliner'
  alias e puts
  RBT.to_current(ARGV)
end # tocurrent python
    # tocurrent /home/Programs/Python/3.6.2