#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Base
#
# This is the base class of the RBT project, based on class LeanPrototype.
#
# It extends the code offered by class LeanPrototype a little bit.
#
# All classes that desire to have a common functionality within the
# RBT namespace should subclass from this class here.
#
# Not every class in the RBT project will need to subclass from this
# class, though, so you should first check whether the RBT::Base class
# is really needed.
#
# For example - the class Prefix does not really need to subclass from
# this class here, as it barely needs the variables defined herein.
#
# The most important part of this class is probably the @debug variable.
# Every subclass should have the @debug variable, which can be useful
# for finding out where a problem occurs within the RBT project.
# Since as of 2022 this is actually @internal_hash[:debug].
#
# Usage example:
#
#   RBT::Base.new
#
# =========================================================================== #
# require 'rbt/base/base.rb'
# =========================================================================== #
# === How to subclass from this class:
#
#   require 'rbt/base/base.rb'; class Foo < ::RBT::Base
#
# =========================================================================== #
require 'rbt/lean_prototype/lean_prototype.rb'

module RBT

class Base < RBT::LeanPrototype # === RBT::Base

  # ========================================================================= #
  # Add exceptions:
  # ========================================================================= #
  require 'rbt/errors/exceptions.rb'

  begin
    require 'roebe/requires/require_kde_konsole.rb'
  rescue LoadError; end
  # ========================================================================= #
  # === rename_kde_konsole_tab
  # ========================================================================= #
  def rename_kde_konsole_tab(
      use_this_as_the_new_title = ''
    )
    if Object.const_defined?(:Roebe) and
       Roebe.respond_to?(:rename_tab)
      Roebe.rename_tab(use_this_as_the_new_title)
    end
  end

  # ========================================================================= #
  # === repackage                                             (repackage tag)
  #
  # This method will repackage an archive format such as .tar.gz
  # into .tar.xz.
  # ========================================================================= #
  def repackage(
      i,
      use_this_for_the_opnn_namespace = 'RBT::Repackage'
    )
    begin
      require 'repackage'
      if File.exist? i
        opnn(
          namespace_to_use: use_this_for_the_opnn_namespace
        )
        e "#{rev}Trying to repackage `#{sfile(i)}#{rev}` next:"
        Repackage.new(i) {{
          run:                    :run_already,
          delete_code_of_conduct: delete_code_of_conduct?
        }}
      else
        # Not sure whether we should report to the user or not.
      end
    rescue LoadError
      e "#{rev}The gem called \"repackage\" is not available - please install it"
      e "in order to repackage the archive at hand here (`#{sfile(i)}`)."
    end
  end; alias repackage_this repackage # === repackage_this

  require 'rbt/toplevel_methods/fast_return_file_size_of_this_program.rb'
  # ========================================================================= #
  # === fast_return_file_size_of_this_program
  # ========================================================================= #
  def fast_return_file_size_of_this_program(i)
    RBT.fast_return_file_size_of_this_program(i)
  end

  # ========================================================================= #
  # === do_not_show_names
  # ========================================================================= #
  def do_not_show_names
    RBT.do_not_show_names
  end

  # ========================================================================= #
  # === is_a_64bit_system?
  #
  # This method will return true if the underlying system is a 64-bit
  # system.
  # ========================================================================= #
  def is_a_64bit_system?
    RUBY_PLATFORM.include? '_64' # Due to → "x86_64-linux".
  end; alias is_on_64_bit? is_a_64bit_system? # === is_on_64_bit?

  # ========================================================================= #
  # === return_compile_time_statistics
  # ========================================================================= #
  def return_compile_time_statistics
    _ = RBT.file_storing_the_time_it_took_to_compile_programs
    if File.exist? _
      return YAML.load_file(_)
    end
  end

  # ========================================================================= #
  # === delete_code_of_conduct?
  #
  # Query method over as to whether we will try to remove code-of-conduct
  # files. This is usually only valid for my home system, or when a
  # config-option has been set to true. Note that the default value is
  # false, so that other users can decide on their own whether they
  # want to retain code of conducts or not.
  # ========================================================================= #
  def delete_code_of_conduct?
    if is_on_roebe?
      true
    else # else try to check for the config file.
      if config?
        config?.automatically_delete_code_of_conduct_files_when_repackaging
      else
        false
      end
    end
  end

  # ========================================================================= #
  # === rcp
  #
  # This method performs a recursive copy action, through RBT.action().
  # ========================================================================= #
  def rcp(i, &block)
    action(:rcp, i, &block)
  end

  # ========================================================================= #
  # === program_exists?
  #
  # Use this method to find out whether the given program, as input to
  # this method, exists.
  #
  # Usage example:
  #
  #   if program_exists? :htop
  #
  # ========================================================================= #
  def program_exists?(
      this_program
    )
    does_program_exist = false # Defaults to false - the program is not installed.
    this_program = this_program.to_s.dup #. downcase # sanitize. downcase is BAD, it breaks stuff like "Esetroot"
    if this_program.include? ' ' # split it, and take the first word in this case.
      this_program = this_program.split(' ').first 
    end
    path_variable_file = "#{RUBY_SRC_DIR}rcfiles/lib/rcfiles/yaml/path_variable.yml"
    if File.exist? path_variable_filelib
      array_available_paths = load_yaml(path_variable_file)['path'].strip.split(' ')
      array_available_paths.each {|path|
        _ = "#{path}/#{this_program}"
        does_program_exist = true if File.exist?(_)
      }
    end
    # Hack - an exception for configure:
    does_program_exist = true if this_program.include? '/configure'
    return does_program_exist
  end

  require 'rbt/toplevel_methods/run_make_then_make_install.rb'
  # ========================================================================= #
  # === run_make_then_make_install
  # ========================================================================= #
  def run_make_then_make_install
    RBT.run_make_then_make_install
  end

  # ========================================================================= #
  # === use_yaml_database_for_tags?
  # ========================================================================= #
  def use_yaml_database_for_tags?
    RBT.use_yaml_database_for_tags?
  end

  # ========================================================================= #
  # === report_working_directory
  #
  # This method will simply report the current working directory
  # (pwd), without making use of opnn().
  # ========================================================================= #
  def report_working_directory
    e "#{rev}The current working directory is #{sdir(return_pwd)}#{rev}."
  end

  # ========================================================================= #
  # === directory_validation?
  # ========================================================================= #
  def directory_validation?
    RBT.directory_validation?
  end

  # ========================================================================= #
  # === system_tags?
  # ========================================================================= #
  def system_tags?
    RBT.system_tags?
  end

  # ========================================================================= #
  # === sanitize_url1_entry
  # ========================================================================= #
  def sanitize_url1_entry(
      i, optional_hash = {}
    )
    RBT.sanitize_url1_entry(i, optional_hash)
  end

  require 'rbt/utility_scripts/show_overview_over_available_utility_scripts.rb'
  # ========================================================================= #
  # === show_overview_over_available_utility_scripts
  # ========================================================================= #
  def show_overview_over_available_utility_scripts
    RBT.show_overview_over_available_utility_scripts
  end

  require 'rbt/toplevel_methods/determine_appdir_prefix_from_this_input.rb'
  # ========================================================================= #
  # === determine_appdir_prefix_from_this_input
  #
  # This method has to return a String, which constitutes the AppDir
  # prefix of the target program at hand.
  # ========================================================================= #
  def determine_appdir_prefix_from_this_input(
      i               = nil,
      program_version = nil
    )
    return RBT.determine_appdir_prefix_from_this_input(
      i, program_version
    )
  end; alias static_appdir_prefix_of? determine_appdir_prefix_from_this_input # === static_appdir_prefix_of?

  # ========================================================================= #
  # === is_ccache_available?
  #
  # This method queries as to whether ccache is available or whether
  # it is not.
  # ========================================================================= #
  def is_ccache_available?
    result = true
    begin
      check_for_ccache = `ccache 2>&1`
      if check_for_ccache.include? 'command not found'
        result = false
      end
    rescue Errno::ENOENT
      # ===================================================================== #
      # This clause can happen when there is no
      # /bin/sh symlink.
      # ===================================================================== #
      check_for_ccache = false
    end
    result
  end; alias ccache_is_available? is_ccache_available? # === ccache_is_available?

  # ========================================================================= #
  # === warn_and_exit
  #
  # This method will exit from the scripts.
  #
  # You should be careful when using this, because in some situations we
  # do not actually want to exit completely - for example when we
  # chain-compile something.
  # ========================================================================= #  
  def warn_and_exit(input, exit_mode = true)
    warn(input, exit_mode)
  end; alias warn_then_exit warn_and_exit # === warn_then_exit

  # ========================================================================= #
  # === consider_reporting_how_many_programs_are_registered
  # ========================================================================= #
  def consider_reporting_how_many_programs_are_registered(
      shall_we_report = false
    )
    if shall_we_report
      action(:report_the_registered_programs)
    end
  end

  # ========================================================================= #
  # === file_last_symlinked_program?
  # ========================================================================= #
  def file_last_symlinked_program?
    RBT.file_last_symlinked_program?
  end

  # ========================================================================= #
  # === gem_version
  #
  # This method is a wrapper over Gem::Version.new()
  #
  # The reason as to why this resides in a method is so that we can do some
  # input-sanitizing, and easier rescue, if this is necessary one day.
  # ========================================================================= #
  def gem_version(i)
    i = i.to_s.delete('vr')
    begin
      if i =~ /\d+/ # Must have at the least one number.
        return Gem::Version.new(i)
      end
    rescue ArgumentError => error
      opne 'An error occurred in gem_version()'
      pp error
    end
    nil # Indicate "failure" aka unable to parse this version.
  end

  # ========================================================================= #
  # === wget
  # ========================================================================= #
  def wget(i)
    esystem "wget #{i}"
  end

  # ========================================================================= #
  # === is_this_program_included?
  #
  # Use this method to query whether a program is included or whether it
  # is not.
  #
  # The second argument is, by default, nil. It can be a symbol such as
  # :include_abbreviations, in which case we will include abbreviations.
  #
  # Usage example:
  #
  #   is_this_program_included?(:htop, :include_abbreviations)
  #
  # ========================================================================= #
  def is_this_program_included?(
      i, optional_arguments = nil
    )
    RBT.is_this_program_included?(i, optional_arguments) # bl $RBT/toplevel_methods/available_programs.rb
  end; alias cookbooks_include_this_program? is_this_program_included? # === cookbooks_include_this_program?

  # ========================================================================= #
  # === copy_recursively
  # ========================================================================= #
  def copy_recursively(
      what,
      where_to   = '/usr/',
      be_verbose = false
    )
    ::RBT.copy_recursively(what, where_to, be_verbose)
  end

  # ========================================================================= #
  # === file_programs_version_url
  # ========================================================================= #
  def file_programs_version_url
    RBT.send(__method__)
  end

  # ========================================================================= #
  # === is_porg_available?
  #
  # This method is required to determine whether porg is available or not.
  # ========================================================================= #
  def is_porg_available?
    result = `porg 2>&1`.include? 'porg: command not found'
    return !result
  end; alias porg_is_available? is_porg_available? # === porg_is_available?

  # ========================================================================= #
  # === set_source_directory
  #
  # Set the source dir to ANY new location. Defaults to Dir.pwd.
  #
  # Example to test this method:
  #
  #   ry mantis --source_dir=/Depot/j
  #
  # ========================================================================= #
  def set_source_directory(
      i          = return_pwd,
      be_verbose = false
    )
    i << '/' unless i.end_with? '/'
    RBT.set_source_directory(i)
    if be_verbose?
      e "#{rev}Setting source directory to `#{sdir(i)}#{rev}` next." 
    end
  end; alias set_source_dir set_source_directory # === set_source_dir

  # ========================================================================= #
  # === make_command?
  # ========================================================================= #
  def make_command?
    RBT::Base.make_command?
  end

  # ========================================================================= #
  # === make_install_command?
  #
  # This is the default "make install" command for now.
  # ========================================================================= #
  def make_install_command?
    'make install'
  end

  # ========================================================================= #
  # === report_n_programs
  #
  # Usage example for the report functionality of this method:
  #
  #   report_n_programs
  #
  # ========================================================================= #
  def report_n_programs
    consider_reporting_how_many_programs_are_registered
  end

  # ========================================================================= #
  # === cd_back_to_the_current_working_directory
  #
  # This method can be used to ensure that we are back in the old
  # "current working directory", after invoking the given block.
  # ========================================================================= #
  def cd_back_to_the_current_working_directory
    working_directory = return_pwd
    if block_given?
      yield
    end
    cd(working_directory)
  end

  # ========================================================================= #
  # === cookbook_files?
  #
  # This method will return all available cookbook files in an Array.
  #
  # Usage example:
  #
  #   cookbook_files?(:show_full_path)
  #
  # ========================================================================= #
  def cookbook_files?(
      show_how = :show_only_name
    )
    result = Dir["#{individual_cookbooks_directory?}*yml"].sort
    case show_how
    when :show_only_name # In this case, show only the name.
      result.map! {|entry|
        File.basename(entry).gsub( File.extname(entry), '' )
      }
    when :show_full_path # In this case make no modifications.
    end
    return result
  end; alias available_cookbook_files? cookbook_files? # === available_cookbook_files?

  # ========================================================================= #
  # === return_n_random_characters
  #
  # This method will return n random characters, as Strings. It is used
  # specifically to "create" a random temporary directory, into which
  # archives can be extracted into.
  # ========================================================================= #
  def return_n_random_characters(
      i = 10
    )
    array = ('a'..'z').to_a
    _ = ''.dup
    i.times { _ << array.sample }
    _
  end

  # ========================================================================= #
  # === RBT::Base.registered_binaries?
  #
  # Tell us where our binaries are registered.
  # ========================================================================= #
  def self.registered_binaries?
    RBT.all_binaries?
  end

  # ========================================================================= #
  # === is_this_binary_registered?
  # ========================================================================= #
  def is_this_binary_registered?(i)
    unless RBT.respond_to?(:registered_binaries?)
      require 'rbt/registered/registered_binaries.rb'
    end
    array = RBT.registered_binaries?
    array.include?(i.to_s)
  end

  # ========================================================================= #
  # === cat                                                         (cat tag)
  # ========================================================================= #
  def cat(i)
    if File.exist? i
      return readlines_with_proper_encoding(i).join
    end
  end

  # ========================================================================= #
  # === symlink_all_files_from_this_directory_to_that_directory
  #
  # This method can be used to symlink two directories to one
  # another, e. g. from directory A onto directory B.
  # ========================================================================= #
  def symlink_all_files_from_this_directory_to_that_directory(
      this_dir,
      that_dir,
      delete_target_file = false
    )
    # ======================================================================= #
    # Select only files next:
    # ======================================================================= #
    _ = Dir[("#{this_dir}/*").squeeze('/')].select {|entry| is_file?(entry) }
    # ======================================================================= #
    # === Handle blocks next
    #
    # The block-variant can remove target-files. This is not the default,
    # so you have to specifically enable this if you want that
    # functionality.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :also_include_directories
      # ===================================================================== #
      when :also_include_directories,
           :include_directories
        _ = Dir[("#{this_dir}/*").squeeze('/')].select {|entry|
          is_file?(entry) or is_directory?(entry)
        }
      when :delete_target_file_if_it_exists
        delete_target_file = true
      end
    end
    _.each {|entry|
      # ===================================================================== #
      # Note that the following clause is NOT the default.
      # ===================================================================== #
      if delete_target_file
        _ = that_dir+File.basename(entry)
        if File.exist?(_) and File.file?(_)
          remove_file(_)
        end
      end
      symlink(entry, that_dir)
    }
  end; alias symlink_from_to symlink_all_files_from_this_directory_to_that_directory # === symlink_from_to

  # ========================================================================= #
  # === remove_symlink
  #
  # Simply remove a symlink. Use this method consistently when you
  # wish to remove any symlink.
  # ========================================================================= #
  def remove_symlink(
      i,
      be_verbose = false
    )
    case be_verbose
    # === :be_quiet
    when :be_quiet
      be_verbose = false
    # === :be_verbose
    when :be_verbose
      be_verbose = true
    end
    if i.is_a? Array
      i.each {|entry| remove_symlink(entry, be_verbose) }
    else
      if File.symlink?(i)
        if be_verbose
          e "#{rev}Now deleting the symlink `#{sfancy(i)}#{rev}`."
        end
        File.delete(i)
      end
    end
  end; alias delete_symlink      remove_symlink # === delete_symlink
       alias delete_this_symlink remove_symlink # === delete_this_symlink
       alias remove_this_symlink remove_symlink # === remove_this_symlink

  # ========================================================================= #
  # === new_cookbook_instance_for
  #
  # Use a slightly shorter wrapper to access class Cookbook.
  #
  # Note that you still have to require class Cookbook on your
  # own, prior to calling this method.
  #
  # The argument to this method should be the name of the program
  # whose cookbook-dataset you wish to access/manipulate. 
  # ========================================================================= #
  def new_cookbook_instance_for(name_of_the_program)
    action(:sanitize_cookbook, name_of_the_program)
  end

  require 'rbt/toplevel_methods/ftp.rb'
  # ========================================================================= #
  # === publish_list_of_all_programs_version
  # ========================================================================= #
  def publish_list_of_all_programs_version
    ::RBT.publish_list_of_all_programs_version
  end

  # ========================================================================= #
  # === upload_list_of_all_programs_version
  # ========================================================================= #
  def upload_list_of_all_programs_version
    ::RBT.upload_list_of_all_programs_version
  end

  # ========================================================================= #
  # Requiring the next .rb file by default allows us to quickly return
  # the version of a registered program, if necessary.
  # ========================================================================= #
  require 'rbt/requires/swift_version.rb'
  # ========================================================================= #
  # === return_version_of_this_program
  # ========================================================================= #
  def return_version_of_this_program(i)
    RBT.swift_return_version_of_this_program(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  base = RBT::Base.new
  e
  e 'Is it a 64-bit system? '+base.is_a_64bit_system?.to_s
  e base.return_weekday_based_on_utc
  e
end