#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForRemoteWebpages
#
# This is the base class from which all other .rb files in the same
# directory should subclass from. These classes have a common
# functionality in that they deal with updating programs from a
# remote source.
# =========================================================================== #
# require 'rbt/check_for_updates/base.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CheckForRemoteWebpages < Base # === RBT::Cookbooks::CheckForRemoteWebpages

  require 'open-uri'
  require 'optparse'
  require 'rbt/constants/remote_URLs.rb'
  require 'rbt/requires/require_class_update_entry.rb'

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @internal_hash
    # ======================================================================= #
    @internal_hash = {}
    # ======================================================================= #
    # === :use_wget
    # ======================================================================= #
    @internal_hash[:use_wget] = false
    # ======================================================================= #
    # === :use_this_download_mode
    # ======================================================================= #
    @internal_hash[:use_this_download_mode] = :ruby_wrapper_over_wget
    # ======================================================================= #
    # === :array_these_programs_were_updated
    #
    # The next Array can keep track of which programs were updated.
    # ======================================================================= #
    @internal_hash[:array_these_programs_were_updated] = []
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = nil # ← This variable will store the downloaded-dataset (from open-uri).
    # ======================================================================= #
    # === @downloaded_n_programs
    # ======================================================================= #
    @downloaded_n_programs = 0 # ← Keeps track of how many programs were batch-downloaded.
    do_use_colours
  end

  # ========================================================================= #
  # === array_these_programs_were_updated?
  # ========================================================================= #
  def array_these_programs_were_updated?
    @internal_hash[:array_these_programs_were_updated]
  end

  # ========================================================================= #
  # === that_remote_URL_should_be_at
  # ========================================================================= #
  def that_remote_URL_should_be_at(full_remote_url)
    opne "#{rev}That remote URL should be at: "\
         "#{sfancy(full_remote_url)}#{rev}"
  end

  # ========================================================================= #
  # === add_these_programs_were_updated
  # ========================================================================= #
  def add_these_programs_were_updated(i)
    @internal_hash[:array_these_programs_were_updated] << i
  end

  # ========================================================================= #
  # === do_use_wget
  # ========================================================================= #
  def do_use_wget
    @internal_hash[:use_wget] = true
  end

  # ========================================================================= #
  # === use_wget?
  # ========================================================================= #
  def use_wget?
    @internal_hash[:use_wget]
  end

  # ========================================================================= #
  # === commandline_size_query_was_issued?
  # ========================================================================= #
  def commandline_size_query_was_issued?
    commandline_arguments?.any? {|entry|
      entry.include? 'size'
    }
  end

  # ========================================================================= #
  # === distrowatch_url?
  # ========================================================================= #
  def distrowatch_url?
    'https://distrowatch.org/'
  end

  # ========================================================================= #
  # === correct_for_common_mistakes_in_program_names
  #
  # The BLFS folks, but also the slackware folks, use some wrong names.
  #
  # The following ad-hoc case/when menu will correct some of these
  # wrong names.
  #
  # The correct name is the one that comes from the official tarball
  # release. So for example, for "gtk+-3.93.0.tar.xz", the official
  # name is exactly "gtk+" and NOT "gtk+3", despite the BLFS team
  # thinking otherwise.
  # ========================================================================= #
  def correct_for_common_mistakes_in_program_names(i)
    case i # case tag, sanitize tag
    when 'node.js'
      i = 'node'
    when 'openjpeg2'
      i = 'openjpeg'
    when 'gtk+3'
      i = 'gtk+'
    end
    return i
  end

  # ========================================================================= #
  # === report_that_these_programs_were_updated
  # ========================================================================= #
  def report_that_these_programs_were_updated(array)
    unless array.empty?
      opne "#{rev}The following #{simp(array.size.to_s)} "\
           "#{rev}programs were updated:"
      array.each {|this_program|
        e "  #{sfancy(this_program)}"
      }
    end
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === notify_the_user_that_no_downloads_have_occurred
  # ========================================================================= #
  def notify_the_user_that_no_downloads_have_occurred
    opne "#{rev}No downloads occurred. This usually means"
    opne "that all local programs are up to date! "+
         yellow("\\o/")+rev
  end

  # ========================================================================= #
  # === return_local_version_of_this_program
  # ========================================================================= #
  def return_local_version_of_this_program(i)
    action(:SanitizeCookbook, i) { :fast }.program_version?
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === --use-wget
      # ===================================================================== #
      when /^-?-?use(-|_| )wget$/i
        do_use_wget
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e
    e "#{rev}  The option #{steelblue('--use-wget')}"\
      " can be used to force the use of system wget."
    e
  end

  # ========================================================================= #
  # === report_the_remote_webpage
  #
  # Standard method to report the remote webpage in use.
  # ========================================================================= #
  def report_the_remote_webpage(
      i = remote_webpage?
    )
    opne grey('Attempting to obtain data from the following ')+
         lightblue('remote URL')+
         (':')
    e
    e "  #{steelblue(i)}"
    e
  end

  # ========================================================================= #
  # === obtain_dataset_from_the_remote_webpage
  #
  # The argument to this method can be a String such as
  # "https://download.kde.org/stable/frameworks/5.102/".
  # ========================================================================= #
  def obtain_dataset_from_the_remote_webpage(
      url_to_the_remote_webpage = remote_webpage?
    )
    begin
      @dataset = URI.open(url_to_the_remote_webpage).read
    # ======================================================================= #
    # The next clause will be entered when we encountered some OpenSSL
    # related problem.
    # ======================================================================= #
    rescue OpenSSL::SSL::SSLError => error
      opne tomato('An error occurred, related to OpenSSL. Perhaps openssl')
      opne tomato('has not been installed correctly. Either way the')
      opne tomato('download mode will be changed.')
      ensure_download_mode_is_system_wget_from_now_on
      cd log_dir? # We are now in "/home/x/Temp/rbt/".
      local_file = "#{log_dir?}index.html"
      delete_file(local_file)
      esystem "wget #{url_to_the_remote_webpage}"
      @dataset = File.read(local_file)
    rescue OpenURI::HTTPError => error
      opnn; etomato('OpenURI::HTTPError was encountered. This usually happens when the')
      opne  tomato('URL is invalid (')+
            sfancy(url_to_the_remote_webpage)+
            tomato(')')
      opnn; etomato('Consider fixing the URL and trying again.')
      pp error
      exit
    end
  end; alias obtain_remote_dataset obtain_dataset_from_the_remote_webpage # === obtain_remote_dataset

  # ========================================================================= #
  # === ensure_download_mode_is_system_wget_from_now_on
  # ========================================================================= #
  def ensure_download_mode_is_system_wget_from_now_on
    @internal_hash[:use_this_download_mode] = :system_wget
  end

  # ========================================================================= #
  # === update_entry
  # ========================================================================= #
  def update_entry(full_remote_url)
    require 'rbt/requires/require_class_update_entry.rb'
    RBT::UpdateEntry.new(full_remote_url) {{
      do_not_ftp_upload: true,
      download_mode:     @internal_hash[:use_this_download_mode]     
    }}
  end

end; end; end