#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::UpdateKdeApplications
#
# Use this class to update the kde-applications.
#
# Usage example:
#
#   RBT::Cookbooks::UpdateKdeApplications.new(ARGV)
#
# =========================================================================== #
# require 'rbt/check_for_updates/update_kde_applications.rb'
# RBT::Cookbooks::UpdateKdeApplications.new(ARGV)
# RBT.update_kde_applications
# =========================================================================== #
require 'rbt/check_for_updates/base.rb'

module RBT

module Cookbooks

class UpdateKdeApplications < CheckForRemoteWebpages # === RBT::Cookbooks::UpdateKdeApplications

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_URL
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL = REMOTE_URL_FOR_KDE_APPLICATIONS

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === main_url?
  # ========================================================================= #
  def main_url?
    REMOTE_URL_FOR_KDE_APPLICATIONS
  end; alias remote_url?     main_url? # === remote_url?
       alias remote_webpage? main_url? # === remote_webpage?
       alias remote_website? main_url? # === remote_website?

  # ========================================================================= #
  # === remote_download_url?
  # ========================================================================= #
  def remote_download_url?
    REMOTE_DOWNLOAD_URL
  end

  # ========================================================================= #
  # === iterate_through_the_results_and_update_old_cookbooks_entries
  # ========================================================================= #
  def iterate_through_the_results_and_update_old_cookbooks_entries
    @scanned_results.each {|this_program| # ← That variable is e. g. 'plasma-vault-5.11.0'
      name_of_the_program, program_version =
        ::ProgramInformation.return_array_program_name_and_program_version(this_program)
      # ===================================================================== #
      # Next, we must compare it to the local dataset.
      # ===================================================================== #
      local_dataset = new_cookbook_instance_for(name_of_the_program)
      local_program_version = local_dataset.program_version?
      if local_program_version.nil?
        e tomato('Warning: no local program version was found for ')+
          steelblue(this_program)
        e "#{rev}Is this program registered?"
      end
      if local_program_version < program_version
        # =================================================================== #
        # In this case, we have to updated.
        # =================================================================== #
        @downloaded_n_programs += 1
        opne "#{rev}There is a more recent program version "\
             "available for `#{sfancy(name_of_the_program)}`."
        full_remote_url = "#{remote_download_url?}#{this_program}.tar.xz"
        that_remote_URL_should_be_at(full_remote_url)
        colourized_result = slateblue(
          "class RBT::UpdateEntry.new(#{full_remote_url})"
        )
        opne "#{rev}Now delegating towards #{colourized_result}#{rev}."
        update_entry(full_remote_url)
        # =================================================================== #
        # Keep track of which programs were updated:
        # =================================================================== #
        add_these_programs_were_updated(
          remove_archive_from_the_end(
            File.basename(full_remote_url)
          )
        )
      end
    }
    if @downloaded_n_programs == 0
      notify_the_user_that_no_downloads_have_occurred
    else
      report_that_these_programs_were_updated(
        array_these_programs_were_updated?
      )
    end
  end

  # ========================================================================= #
  # === decide_what_to_do_next
  #
  # The first variant can be invoked via:
  #
  #   KDE_Applications --size
  #
  # ========================================================================= #
  def decide_what_to_do_next
    if commandline_size_query_was_issued?
      opne "There are #{sfancy(@scanned_results.size)} remote "\
           "KDE Applications programs."
    else # else, enter the default "modus operandi".
      iterate_through_the_results_and_update_old_cookbooks_entries
    end
  end

  # ========================================================================= #
  # === scan_for_all_results
  #
  # Note that for this method to work, the regex must be correct.
  #
  # In November 2020, the old regex suddenly no longer worked, so
  # a link was put up towards rubular.com, in order to determine
  # how a regex has to be tweaked. We could also use the constant
  # called KDE_REGEX_FOR_REMOTE_ARCHIVES, though - but for the
  # time being we'll use the one hardcoded here.
  # ========================================================================= #
  def scan_for_all_results
    use_this_regex =
      # /<tr><td valign="top">&nbsp;<\/td><td><a href=".+.tar.xz">(.+).tar.xz<\/a>/ # <- This one until November 2020.
      /<tr><td valign="top"><a href="([0-9a-zA-Z\-.]+).tar.xz"><img src=/ # See: https://rubular.com/r/aE07UuDWTTPUNv
    @scanned_results = @dataset.scan(use_this_regex).flatten
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    report_the_remote_webpage
    obtain_remote_dataset
    scan_for_all_results
    decide_what_to_do_next
  end

  # ========================================================================= #
  # === RBT::Cookbooks::UpdateKdeApplications[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.update_kde_applications
# =========================================================================== #
def self.update_kde_applications
  RBT::Cookbooks::UpdateKdeApplications.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::UpdateKdeApplications.new(ARGV)
end # updatekdeapplications