#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::UpdateKdePortingAids
#
# This class can be used to update all KDE porting aids.
#
# The URL to these programs is something like this:
#
#   https://download.kde.org/stable/frameworks/5.53/portingAids/
#
# Usage example:
#
#   RBT::Cookbooks::UpdateKdePortingAids.new(ARGV)
#
# =========================================================================== #
# require 'rbt/check_for_updates/update_kde_porting_aids.rb'
# RBT::Cookbooks::UpdateKdePortingAids.new(ARGV)
# =========================================================================== #
require 'rbt/check_for_updates/base.rb'

module RBT

module Cookbooks

class UpdateKdePortingAids < CheckForRemoteWebpages # === RBT::Cookbooks::UpdateKdePortingAids

  # ========================================================================= #
  # === REMOTE_URL
  # ========================================================================= #
  REMOTE_URL = REMOTE_URL_FOR_KDE_PORTING_AIDS

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_URL
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL = REMOTE_URL

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_commandline(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === main_url?
  # ========================================================================= #
  def main_url?
    REMOTE_URL
  end; alias remote_url?     main_url? # === remote_url?
       alias remote_webpage? main_url? # === remote_webpage?
       alias remote_website? main_url? # === remote_website?

  # ========================================================================= #
  # === remote_download_url?
  # ========================================================================= #
  def remote_download_url?
    REMOTE_DOWNLOAD_URL
  end

  # ========================================================================= #
  # === decide_what_to_do_next
  #
  # The first variant can be invoked via:
  #
  #   KDE_PortingAids --size
  #
  # ========================================================================= #
  def decide_what_to_do_next
    if commandline_size_query_was_issued?
      opne "#{rev}There are #{sfancy(@scanned_results.size)} #{rev}"\
           "remote KDE PortingAids programs."
    else # else, enter the default "modus operandi".
      iterate_through_the_results_and_update_old_cookbooks_entries
    end
  end

  # ========================================================================= #
  # === iterate_through_the_results_and_update_old_cookbooks_entries
  # ========================================================================= #
  def iterate_through_the_results_and_update_old_cookbooks_entries
    @scanned_results.each {|this_program| # ← That variable is e. g. 'plasma-vault-5.11.0'
      name_of_the_program, program_version =
        ::ProgramInformation.return_array_program_name_and_program_version(this_program)
      # ===================================================================== #
      # Next, we must compare it to the local dataset.
      # ===================================================================== #
      local_dataset = new_cookbook_instance_for(name_of_the_program)
      local_program_version = local_dataset.program_version?
      if local_program_version < program_version
        # =================================================================== #
        # In this case, we have to updated.
        # =================================================================== #
        @downloaded_n_programs += 1
        opne "#{rev}There is a more recent program version "\
             "available for `#{sfancy(name_of_the_program)}`."
        full_remote_url = "#{remote_download_url?}#{this_program}.tar.xz"
        that_remote_URL_should_be_at(full_remote_url)
        colourized_result = slateblue(
          "class RBT::UpdateEntry.new(#{full_remote_url})"
        )
        opne "Now delegating towards #{colourized_result}."
        update_entry(full_remote_url)
        # =================================================================== #
        # Keep track of which programs were updated:
        # =================================================================== #
        add_these_programs_were_updated(
          remove_archive_from_the_end(
            File.basename(full_remote_url)
          )
        )
      end
    }
    if @downloaded_n_programs == 0
      notify_the_user_that_no_downloads_have_occurred
    else
      report_that_these_programs_were_updated(array_these_programs_were_updated?)
    end
  end

  # ========================================================================= #
  # === scan_for_all_results
  # ========================================================================= #
  def scan_for_all_results
    # ======================================================================= #
    # See this entry for the following regex:
    #
    #   https://rubular.com/r/FHa92f76Xe
    #
    # ======================================================================= #
    use_this_regex = KDE_REGEX_FOR_REMOTE_ARCHIVES
    @scanned_results = @dataset.scan(use_this_regex).flatten.map {|entry|
      entry.delete_suffix('.tar.xz')
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_the_remote_webpage
    obtain_remote_dataset
    scan_for_all_results
    decide_what_to_do_next
  end

  # ========================================================================= #
  # === RBT::Cookbooks::UpdateKdePortingAids[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.update_kde_porting_aids
# =========================================================================== #
def self.update_kde_porting_aids(i = ARGV)
  RBT::Cookbooks::UpdateKdePortingAids.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::UpdateKdePortingAids.new(ARGV)
end # updatekdeportingaids