#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::FindDirectoryEntriesWithoutCorrespondingYamlFile
#
# This class allows the user to find all sub-directories in the "archive
# directory" that do not have a corresponding .yml file. That way, the
# user can either delete these directories, or add a missing .yml entry.
#
# Usage example:
#
#   RBT::Cookbooks::FindDirectoryEntriesWithoutCorrespondingYamlFile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/checks_and_validations/find_directory_entries_without_corresponding_yaml_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class FindDirectoryEntriesWithoutCorrespondingYamlFile < Base # === RBT::Cookbooks::FindDirectoryEntriesWithoutCorrespondingYamlFile

  # ========================================================================= #
  # === ARRAY_UNIMPORTANT_DIRECTORY
  #
  # The entries in this constant will be filtered away.
  # ========================================================================= #
  ARRAY_UNIMPORTANT_DIRECTORY = %w(
    bios
    chrome
    flash
    fonts
    firmware
    gobolinux
    hardware
    wlan
    windows
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    archive_dir = archive_dir?
    array_missing_yaml_files = []
    if File.directory? archive_dir
      # ===================================================================== #
      # If the directory exists, obtain all subdirectories there.
      # ===================================================================== #
      all_subdirectories = Dir["#{archive_dir}*"].select {|entry| File.directory? entry }
      all_subdirectories.each {|entry|
        entry = File.basename(entry)
        possible_yaml_file_location = individual_cookbooks_dir?+entry+'.yml'
        if File.exist? possible_yaml_file_location
          # ================================================================ #
          # Then this is fine.
          # ================================================================ #
        else
          # ================================================================ #
          # Else store it so that we can report it a bit later.
          # ================================================================ #
          array_missing_yaml_files << possible_yaml_file_location
        end
      }
    else
      opne "#{rev}No directory called `#{sdir(archive_dir)}#{rev}` exists."
    end
    array_missing_yaml_files = filter_away_unimportant_directories(array_missing_yaml_files)
    unless array_missing_yaml_files.empty?
      opne tomato(array_missing_yaml_files.size.to_s)+
           rev+' yaml files are missing:'
      array_missing_yaml_files.each {|i|
        e "#{rev}No file exists at"
        e "  `#{sfile(i)}`"
      }
    end
  end

  # ========================================================================= #
  # === filter_away_unimportant_directories
  # ========================================================================= #
  def filter_away_unimportant_directories(i)
    i.reject {|entry|
      _ = File.basename(entry).delete_suffix('.yml')
      ARRAY_UNIMPORTANT_DIRECTORY.include? _
    }
  end

  # ========================================================================= #
  # === RBT::Cookbooks::FindDirectoryEntriesWithoutCorrespondingYamlFile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::FindDirectoryEntriesWithoutCorrespondingYamlFile.new(ARGV)
end # finddirectoryentrieswithoutcorrespondingyamlfile