#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ReportUselessDescriptions
#
# This class will first scan over all available cookbooks. It will query
# the description of said cookbooks entries as it scans over them.
#
# If this specific entry for the description is either empty or defaults
# to a generic entry, then this will be noted down, and usually reported
# on the commandline. This may then help the user, respectively me, to
# fix these problematic entries.
#
# A flag exists, called ALSO_OPEN_VIA_EDITOR. If this flag is set to true
# then this class here will additionally open that file via the local
# editor, after a short delay. Note that, internally, this will become
# an instance variable, which can be toggled on the commandline as well,
# such as via the flag --do-not-open.
#
# The main idea for this class is that it should help clean up incomplete
# or erroneous descriptions of programs. Additionally, some statistics can
# be reported as well, after information was gathered.
#
# Usage example:
#
#   RBT::Cookbooks::ReportUselessDescriptions.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/report_useless_descriptions.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ReportUselessDescriptions < Base # === RBT::Cookbooks::ReportUselessDescriptions

  # ========================================================================= #
  # === DEFAULT_DESCRIPTION_OF_THE_PROGRAM
  #
  # This is the generic description that was used in the past for the
  # RBT project. If a .yml file still contains this String then we
  # know that it is essentially an empty description, which should
  # be improved on.
  # ========================================================================= #
  DEFAULT_DESCRIPTION_OF_THE_PROGRAM =
    'Default description of the program.'

  # ========================================================================= #
  # === ALSO_OPEN_VIA_EDITOR
  #
  # If this variable is set to true then the problematic programs,
  # that is programs that do not have a useful description, will
  # be opened via the editor. This is mostly useful on my home
  # system, since I can sanitize such problematic programs before
  # publishing the whole gem.
  # ========================================================================= #
  ALSO_OPEN_VIA_EDITOR = true

  # ========================================================================= #
  # === SLEEP_FOR_THIS_AMOUNT
  #
  # This sleep delay is only useful if we open the .yml file via an
  # editor.
  # ========================================================================= #
  SLEEP_FOR_THIS_AMOUNT = 0.7

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    menu
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @raw_cookbook
    # ======================================================================= #
    @raw_cookbook = actions(:raw_cookbook) { :fast }
    # ======================================================================= #
    # === @array_these_programs_are_problematic
    #
    # Store the problematic programs in the following Array:
    # ======================================================================= #
    @array_these_programs_are_problematic = []
    # ======================================================================= #
    # === @also_open_via_editor
    # ======================================================================= #
    @also_open_via_editor = ALSO_OPEN_VIA_EDITOR
    # ======================================================================= #
    # === @use_this_as_the_key_for_the_description
    # ======================================================================= #
    @use_this_as_the_key_for_the_description = :description
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === report_useless_descriptions --do-not-open
      # ===================================================================== #
      when /^-?-?do(-|_)?not(-|_)?open$/
        do_not_open_files_in_the_editor
      end
    end
  end

  # ========================================================================= #
  # === do_not_open_files_in_the_editor
  # ========================================================================= #
  def do_not_open_files_in_the_editor
    @also_open_via_editor = false
  end

  # ========================================================================= #
  # === this_program_is_problematic
  # ========================================================================= #
  def this_program_is_problematic(this_program)
    @array_these_programs_are_problematic << this_program
  end

  # ======================================================================= #
  # === run_through_every_available_program
  # ======================================================================= #
  def run_through_every_available_program
    available_programs?.each {|this_program|
      @raw_cookbook.clear

      hash = @raw_cookbook.load_this_program(this_program)

      if hash.has_key? @use_this_as_the_key_for_the_description
        description = hash[@use_this_as_the_key_for_the_description]
      end
      if hash.nil? or hash.empty?
        this_program_is_problematic(this_program)
      else
        # =================================================================== #
        # Nil descriptions or empty descriptions will be reported.
        # =================================================================== #
        if description.nil? or description.strip.empty?
          this_program_is_problematic(this_program)
        # =================================================================== #
        # As will those programs that conform to a default, but
        # pretty useless, description.
        # =================================================================== #
        elsif description.strip == DEFAULT_DESCRIPTION_OF_THE_PROGRAM
          this_program_is_problematic(this_program)
        # else # Else all seems fine.
        end
      end
    }
  end; alias iterate_over_every_available_program run_through_every_available_program # === iterate_over_every_available_program

  # ========================================================================= #
  # === report_results                                           (report tag)
  # ========================================================================= #
  def report_results
    _ = @array_these_programs_are_problematic
    opne "A total of #{sfancy(_.size.to_s)} problematic "\
         "descriptions were found."
    if _.empty?
      opne "This is #{lightgreen('excellent')}! All programs "\
           "appear to have a useful description."
    else
      e
      _.each {|this_program|
        e "  - #{simp(this_program)}"
        if @also_open_via_editor
          target = File.basename(this_program.dup)
          target << '.yml' unless target.end_with? '.yml'
          unless target.start_with? RUBY_SRC_RBT_COOKBOOKS
            target.prepend(RUBY_SRC_RBT_COOKBOOKS)
          end 
          open_in_editor(target)
          sleep SLEEP_FOR_THIS_AMOUNT
        end
      }
      e
    end
  end; alias report report_results # === report

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_through_every_available_program
    report_results
  end

  # ========================================================================= #
  # === RBT::Cookbooks::ReportUselessDescriptions[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ReportUselessDescriptions.new(ARGV)
end # reportuseless
    # reportuseless --do-not-open