#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ValidateBlfsEntries
#
# This class will simply check every BLFS entry and if the entry does NOT
# start with "https" then it will report these entries at once.
#
# Usage example:
#
#   RBT::Cookbooks::ValidateBlfsEntries.new(ARGV)
#
# =========================================================================== #
# require 'rbt/checks_and_validations/validate_blfs_entries.rb'
# RBT.validate_blfs_entries
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ValidateBlfsEntries < RBT::Base # === RBT::Cookbooks::ValidateBlfsEntries

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @raw_cookbook
    # ======================================================================= #
    @raw_cookbook = action(:raw_cookbook)
    # ======================================================================= #
    # === @encountered_n_problems
    #
    # Keep track how many problems we encountered.
    # ======================================================================= #
    @encountered_n_problems = 0
    try_to_require_the_open_gem
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === validate_blfs_entries --help
      # ===================================================================== #
      when /^help$/
        show_help
      else
        opne 'Unregistered commandline argument: '+sfancy(i)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    unless commandline_arguments?.empty?
      menu
    end
    report_all_programs_that_do_not_have_a_proper_BLFS_entry
  end

  # ========================================================================= #
  # === report_all_programs_that_do_not_have_a_proper_BLFS_entry (report tag)
  # ========================================================================= #
  def report_all_programs_that_do_not_have_a_proper_BLFS_entry
    all_programs = all_programs?
    opne "#{rev}Going over all "\
         "#{steelblue(all_programs.size.to_s)} "\
         "#{rev}registered programs next, looking"
    opne "#{rev}for incorrect BLFS entries - this may take a while."
    all_programs.each {|this_program|
      @raw_cookbook.find(this_program) 
      dataset = @raw_cookbook.dataset
      if dataset.has_key? 'blfs'
        blfs_entry = dataset['blfs']
        if blfs_entry.is_a? Array
          blfs_entry = blfs_entry.first
        end
        if blfs_entry.nil?
          target = '/home/x/programming/ruby/src/rbt/lib/rbt/yaml/cookbooks/'+this_program+'.yml'
          e "#{rev}Odd BLFS entry (nil) for "+
            sfile(target)+rev
          e 'Proceeding nonetheless now.'
          # exit 
        end
        if blfs_entry and blfs_entry.start_with?('https') # Then all is fine.
        else
          @encountered_n_problems += 1
          e tomato(
              'The BLFS entry for '\
              '/home/x/programming/ruby/src/rbt/lib/rbt/yaml/cookbooks/'+this_program+'.yml '\
              'does not start with https.'
            )
          if is_on_roebe?
            Open.in_editor(
              '/home/x/programming/ruby/src/rbt/lib/rbt/yaml/cookbooks/'+this_program+'.yml'
            )
            sleep 1
          end
        end
      else # The next clause may be too verbose.
        # e 'No BLFS entry for `'+steelblue(this_program)+'`.'
      end
    }
    if @encountered_n_problems == 0
      do_notify_the_user_that_no_problems_were_encountered
    end
  end

  # ========================================================================= #
  # === do_notify_the_user_that_no_problems_were_encountered
  # ========================================================================= #
  def do_notify_the_user_that_no_problems_were_encountered
    opne "#{rev}No problems were encountered, thus indicating that all"
    opne "BLFS entries registered in the RBT project are correct. "+
         gold(cheerful_person)
  end

  # ========================================================================= #
  # === RBT::Cookbooks::ValidateBlfsEntries[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.validate_blfs_entries
# =========================================================================== #
def self.validate_blfs_entries(i = ARGV)
  RBT::Cookbooks::ValidateBlfsEntries.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ValidateBlfsEntries.new(ARGV)
end # validateblfsentries