#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile_base/compile_base_module.rb'
# include RBT::CompileBaseModule
# =========================================================================== #
module RBT

module CompileBaseModule # === RBT::CompileBaseModule

  require 'rbt/toplevel_methods/games.rb'
  require 'rbt/toplevel_methods/compile_these_programs.rb'
  require 'rbt/requires/require_the_prefix.rb'
  require 'rbt/utility_scripts/change_prefix.rb'
  require 'rbt/toplevel_methods/error_message.rb'
  require 'rbt/utility_scripts/symlink_headers.rb'
  require 'rbt/utility_scripts/fetch_all_m4_macros.rb'
  require 'rbt/errors/map_line_to_a_specific_error.rb'
  require 'rbt/misc/classes_for_individual_programs/change_lib64_to_lib.rb'
  require 'rbt/linux/linux_from_scratch/lfs/lfs.rb'
  
  require 'rbt/make/colour_make_install.rb' # This file also pulls in rbt/make/make.rb

  # ========================================================================= #
  # === FILE_AUTOGEN
  # ========================================================================= #
  FILE_AUTOGEN = 'autogen.sh'

  begin
    require 'rbt/toplevel_methods/error_message.rb'
  rescue LoadError => error
    pp error
    puts 'Not sure what to do with error_message.rb yet. Needs to be'
    puts 'reviewed for a later time.'
  end

  # ========================================================================= #
  # === reset_the_module
  # ========================================================================= #
  def reset_the_module
    # ======================================================================= #
    # === :silent_redirection
    #
    # The following instance-variable can be used to determine whether we
    # will silently redirect the output-stream or whether we will not.
    # ======================================================================= #
    @internal_hash[:silent_redirection] = false
  end

  # ========================================================================= #
  # === consider_removing_libtool_la_files
  #
  # To test this, try:
  #
  #   ry apr ntrad
  #
  # ========================================================================= #
  def consider_removing_libtool_la_files
    # ======================================================================= #
    # Must check for two conditions:
    #
    #   (a) the user specifically wants to delete libtool files, due
    #       to setting a configuration value
    #   (b) the program at hand uses an appdir-like prefix
    #
    # ======================================================================= #
    if delete_libtool_files?
      prefix_to_the_libtool_files = rds("#{prefix?.to_s}/lib/")
      action(:remove_libtool_files, prefix_to_the_libtool_files)
    end
  end

  # ========================================================================= #
  # === install_only_binaries?
  # ========================================================================= #
  def install_only_binaries?
    @internal_hash[:install_only_binaries]
  end

  # ========================================================================= #
  # === shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
  # ========================================================================= #
  def shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
    @internal_hash[:shall_we_symlink_the_header_files_into_the_usr_include_hierarchy]
  end

  # ========================================================================= #
  # === symlink_into_tags
  #
  # Use this method if you want to symlink into /System/Tags/*
  #
  # The first argument to this method is the name of the program
  # at hand.
  #
  # The second argument to this method is target_directory, which may
  # either be an AppDir, or another (non-AppDir) prefix.
  #
  # If there are no entries under the bin/ subdirectory, or not registered
  # as binary for that program, then this method will not symlink anything
  # at all.
  # ========================================================================= #
  def symlink_into_tags(
      compile_this_program = compile_which_program?,
      target_directory     = real_prefix?.to_s,
      dataset_to_use       = cookbook_dataset?,
      use_tags             = use_tags?
    )
    # ======================================================================= #
    # Handle tags only when we have specified so via the configuration
    # file "use_tags.yml". The following check ensures exactly that.
    # ======================================================================= #
    if use_tags
      # ===================================================================== #
      # So the next job is to obtain the tags.
      # ===================================================================== #
      tags = [dataset_to_use.tags?].flatten.compact
      if tags and !tags.empty? # Ok, we found tags here - and they are not empty.
        tags.each {|this_tag|
          # case this_tag # A bit sanitizing.
          # when 'game' # From game to games.
          #   this_tag = 'games'
          # end # <- No longer allowed since as of Feb 2017. Will be
          # removed past the middle of 2019 or so; perhaps a bit later
          # than that, but eventually it will be gone.
          # ================================================================= #
          # If you wish to CamelCase directory names, then the
          # following line will achieve precisely this. We will
          # CamelCase these tags.
          # ================================================================= #
          if this_tag.include? '_'
            this_tag = this_tag.split('_').map {|e| e.capitalize }.join
          end
          this_tag[0,1] = this_tag[0,1].upcase # Nicer to have capitalized directories.
          # ================================================================= #
          # We have to ensure that the relevant /System/Tags/* subdirectory does
          # actually exist. And if not, then we must create it.
          # ================================================================= #
          target_directory_for_the_system_tags = "#{system_tags?}#{this_tag}/"
          unless File.directory? target_directory_for_the_system_tags
            create_directory(target_directory_for_the_system_tags) # <- Create the subdirectory here.
          end
          # ================================================================= #
          # Next, obtain all binaries of the given program. This will include
          # both bin/ and sbin/ subdirectories. However had, the code here
          # will only work if we have compiled something as an AppDir. For
          # non-appdir programs we have to fetch the binaries that are
          # registered for that particular program.
          # ================================================================= #
          if uses_an_appdir_prefix?(target_directory)
            all_binaries = Dir["#{target_directory}bin/*"]
            if File.directory? "#{target_directory}sbin/"
              all_binaries << Dir["#{target_directory}sbin/*"] # <- Add sbin/ entries here.
              all_binaries.flatten!
            end
          else
            all_binaries = dataset_to_use.binaries?
            # =============================================================== #
            # And cd to the target directory, aka, the prefix.
            # =============================================================== #
            cd target_directory
          end
          unless all_binaries.empty?
            opne "We will next symlink the various tags into the `"\
                 "#{sdir(target_directory_for_the_system_tags)}#{rev}` "\
                 "hierarchy."
            all_binaries.each { |binary|
              # ============================================================= #
              # We must find the target next:
              # ============================================================= #
              _ = target_directory_for_the_system_tags+File.basename(binary)
              if File.exist? "bin/#{binary}"
                binary = return_pwd+'bin/'+binary
              elsif File.exist? 'sbin/'+binary
                binary = return_pwd+'sbin/'+binary
              end
              # ============================================================= #
              # We have to check if the target already exists. The current
              # behaviour (Oct 2018) is to delete the old target, if it is
              # a symlink. This will also get rid of symlinks that are
              # invalid - e. g. no longer point at a correct target.
              # ============================================================= #
              if File.symlink?(_)
                if is_on_roebe?
                  opne "Deleting the old symlink at `#{royalblue(_)}`."
                end
                delete_symlink(_) # <- Get rid of the old symlink finally.
              end
              symlink(
                from: binary,
                to:   _
              )
            }
          end
        }
      end
    end
  end

  # ========================================================================= #
  # === play_song
  #
  # Play a song when the compilation phase has finished. This can be disabled
  # via the configuration file, in the entry called play_song. Please call
  # CONFIG.play_song before calling this method. We will play the audio file
  # only if it exists, otherwise we will be silent.
  # ========================================================================= #
  def do_play_song(
      song_location =
        '/home/x/songs/YoonDohyun_Dolgo.mp3',
      use_this_player = 'mpv'
    )
    _ = "#{use_this_player} #{song_location} -endpos 4"
    esystem(_) if File.exist? song_location
  end; alias play_song do_play_song # === play_song

  # ========================================================================= #
  # === remove_compiling_is_in_progress
  #
  # We must get rid of special files. Right now only of a file called
  # "COMPILING_IS_IN_PROGRESS" - this file will be created when we
  # start compiling.
  # ========================================================================= #
  def remove_compiling_is_in_progress(
      this_file = 'COMPILING_IS_IN_PROGRESS.md'
    )
    remove_file(this_file) if File.exist? this_file
  end

  # ========================================================================= #
  # === return_files_from_the_profiles_directory
  #
  # This method will return all .md files from the RBT profiles directory.
  # ========================================================================= #
  def return_files_from_the_profiles_directory(
      target_directory = DIRECTORY_RBT_PROFILES
    )
    Dir["#{target_directory}*.md"]
  end

  # ========================================================================= #
  # === consider_symlinking_lib64_into_lib
  #
  # This method will symlink lib64/ into lib/. Only invoke this method
  # if you have an AppDir versioned directory though.
  # ========================================================================= #
  def consider_symlinking_lib64_into_lib(
      shall_we_symlink = cookbook_dataset_autosymlink_lib64?
    )
    if shall_we_symlink
      old_dir = Dir.pwd # Keep a reference to the old directory.
      cd prefix?.to_s # cd into the AppDir prefix.
      opne 'Next trying to symlink '+sdir('lib64/')+' into '+
           sdir('lib/')+' from the directory '+sdir(return_pwd)+':'
      symlink('lib64','lib')
      # ===================================================================== #
      # Next check whether the operation was a success, so that we can
      # inform the user about this.
      # ===================================================================== #
      if File.symlink?('lib') and
         File.directory?('lib')
        opne 'Success!'
      else
        opne swarn('Failed to symlink `')+sdir('lib64/')+
             swarn('` towards `')+sdir('lib/')+swarn('`.')
      end
      cd old_dir # And go back again.
    end
  end

  # ========================================================================= #
  # === cookbook_dataset_autosymlink_lib64?
  # ========================================================================= #
  def cookbook_dataset_autosymlink_lib64?
    cookbook_dataset?.autosymlink_lib64?
  end

  # ========================================================================= #
  # === create_architecture_file
  #
  # This method here will create a file called "Architecture".
  #
  # The first argument, `create_where`, should be the AppDir path to
  # the program in question, such as /Programs/Htop/Current.
  # ========================================================================= #
  def create_architecture_file(
      create_where                        = prefix?.to_s,
      can_we_create_the_architecture_file = is_on_gobolinux?
    )
    if can_we_create_the_architecture_file
      this_architecture = `uname -m`
      resources_directory = rds("#{create_where}/Resources/")
      # ===================================================================== #
      # Point to the new target that will hold the file Architecture:
      # ===================================================================== #
      file_location = rds("#{resources_directory}Architecture")
      mkdir(resources_directory) unless File.directory? resources_directory
      # ===================================================================== #
      # Remove the old file it if already exists.
      # ===================================================================== #
      remove_file(file_location) if File.exist? file_location
      write_what_into(this_architecture, file_location)
    end
  end

  # ========================================================================= #
  # === check_whether_we_will_install_only_programs_from_the_bin_subdirectory
  # ========================================================================= #
  def check_whether_we_will_install_only_programs_from_the_bin_subdirectory
    if install_only_binaries?
      opne 'We will only retain the '+sdir('bin/')+' subdirectory.'
      delete_these_directories = Dir[return_proper_prefix?.to_s+'*'].select {|entry|
        File.directory? entry
      }
      delete_these_directories.reject! {|entry| entry.end_with? 'bin' }
      unless delete_these_directories.empty?
        opne 'We will now delete several directories that we did find.'
        remove(delete_these_directories)
      end
    end
  end

  # ========================================================================= #
  # === consider_symlinking_header_files_into_the_usr_include_hierarchy
  #
  # This method can be called if you wish to symlink any .h files or
  # directories within the AppDir at hand, into the /usr/include/
  # hierarchy.
  # ========================================================================= #
  def consider_symlinking_header_files_into_the_usr_include_hierarchy(
      i                                                                = compile_which_program?,
      shall_we_symlink_the_header_files_into_the_usr_include_hierarchy =
        shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
    )
    if shall_we_symlink_the_header_files_into_the_usr_include_hierarchy
      # ===================================================================== #
      # Delegate into class RBT::SymlinkHeaders next.
      # ===================================================================== #
      opne "#{rev}Next attempting to symlink all #{royalblue('.h')} #{rev}files "\
           "into the #{sdir('/usr/include/')} #{rev}hierarchy."
      opne "#{rev}In order for this to work, #{royalblue('.h')}"\
           " #{rev}files have to exist in the AppDir directory."
      RBT::SymlinkHeaders.new(i) { :do_not_notify_if_there_were_no_header_files }
    end
  end

  # ========================================================================= #
  # === symlink_into_usr_lib_dir?
  # ========================================================================= #
  def symlink_into_usr_lib_dir?
    @internal_hash[:symlink_into_usr_lib_dir]
  end

  # ========================================================================= #
  # === consider_symlinking_cmake_modules
  #
  # Prior to entering this method, there is already a check whether we
  # are using an AppDir-like prefix. Only if this is the case, will
  # this method here be entered at all.
  #
  # This method will normally be called from the method
  # consider_performing_postinstall_actions().
  # ========================================================================= #
  def consider_symlinking_cmake_modules
    # ======================================================================= #
    # We must first check whether there are any cmake modules in the
    # first place.
    # ======================================================================= #
    target_cmake_dir = "#{prefix?}lib64/cmake/"
    if File.directory? target_cmake_dir
      # ===================================================================== #
      # But we must also check whether a cmake directory exists at a
      # /usr hierarchy.
      # ===================================================================== #
      if File.directory? '/usr/lib/cmake/'
        opne "#{rev}Symlinking from #{sdir(target_cmake_dir)} #{rev}into "\
             "#{sdir('/usr/lib/cmake/')}"
        # =================================================================== #
        # Symlink only directories.
        # =================================================================== #
        all_dirs = Dir["#{target_cmake_dir}*"].select {|entry| File.directory? entry }
        all_dirs.each {|this_directory|
          basename = File.basename(this_directory)
          target_in_usr_lib_cmake = "/usr/lib/cmake/#{basename}"
          # ================================================================= #
          # Only on roebe-like systems will we also remove the target
          # directory.
          # ================================================================= #
          if is_on_roebe?
            if File.exist?(target_in_usr_lib_cmake) and
               File.directory?(target_in_usr_lib_cmake) and
              !File.symlink?(target_in_usr_lib_cmake)
              remove_directory(target_in_usr_lib_cmake)
            end
          end
          symlink(this_directory, target_in_usr_lib_cmake)
        }
      end
    end
  end

  # ========================================================================= #
  # === cookbook_dataset_program_version?
  #
  # Obtain the version string of the program at hand through this
  # method. This method is quite reliable and can ideally be used
  # whenever we need to determine the version of a program.
  # ========================================================================= #
  def cookbook_dataset_program_version?
    if cookbook_dataset?
      return cookbook_dataset?.program_version?
    end
    nil
  end

  # ========================================================================= #
  # === no_error_has_occurred?
  # ========================================================================= #
  def no_error_has_occurred?
    !has_an_error_occurred?
  end

  # ========================================================================= #
  # === store_full_configure_command_into_resources_directory
  #
  # We will store the full configure command into:
  #
  #   /Programs/Name/Version/Resources/full_configure_command
  #
  # ========================================================================= #
  def store_full_configure_command_into_resources_directory
    what = full_configure_command?
    unless what.strip.empty?
      # ===================================================================== #
      # Designate the file into where to store this:
      # ===================================================================== #
      into = real_prefix?.to_s.dup
      into << '/Resources/full_configure_command.md'
      unless File.directory? File.dirname(into)
        mkdir(File.dirname(into))
      end
      # ===================================================================== #
      # Remove old files at that position first.
      # ===================================================================== #
      delete_file(into) if File.exist? into
      append_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === uses_appdir_prefix?
  #
  # This method will return true if the program at hand is compiled in
  # an AppDir-like fashion (e. g. having a prefix including the String
  # "/Programs/" directory), and false otherwise.
  # ========================================================================= # 
  def uses_appdir_prefix?(
      use_this_as_prefix           = prefix?.to_s,
      use_this_as_cookbook_dataset = cookbook_dataset?
    )
    if use_this_as_cookbook_dataset.nil?
      ensure_dataset_for_this_program(compile_which_program?)
    end
    use_this_as_prefix.to_s.include? programs_dir?
  end; alias is_installed_in_an_appdir_like_fashion? uses_appdir_prefix? # === is_installed_in_an_appdir_like_fashion?
       alias is_an_appdir_program?                   uses_appdir_prefix? # === is_an_appdir_program?
       alias is_appdir_compilation_style?            uses_appdir_prefix? # === is_appdir_compilation_style?
       alias is_an_appdir_installation?              uses_appdir_prefix? # === is_an_appdir_installation?
       alias uses_an_appdir_prefix?                  uses_appdir_prefix? # === uses_an_appdir_prefix?
       alias use_appdir_prefix?                      uses_appdir_prefix? # === use_appdir_prefix?
       alias appdir_prefix?                          uses_appdir_prefix? # === use_appdir_prefix?
       alias uses_appdir_approach?                   uses_appdir_prefix? # === uses_appdir_approach?
       alias did_we_use_an_appdir_prefix?            uses_appdir_prefix? # === did_we_use_an_appdir_prefix?

  # ========================================================================= #
  # === return_these_programs
  #
  # This method will essentially return the information stored in the file
  # 'chained_programs.yml'.
  # ========================================================================= # 
  def return_these_programs(i)
    return return_chained_programs[i.to_s]
  end

  # ========================================================================= #
  # === return_chained_programs
  # ========================================================================= #
  def return_chained_programs
    ::RBT.chained_programs?
  end

  # ========================================================================= #
  # === symlink_m4_macros
  #
  # The various .m4 macros usually reside within the share/ subdirectory.
  #
  # So in order to symlink them, we need to enter that directory.
  #
  # This method is called from do_postinstall_actions().
  # ========================================================================= #
  def symlink_m4_macros(
      i, do_run_fetch_all_m4_macros = do_run_fetch_all_m4_macros?
    )
    if do_run_fetch_all_m4_macros
      # ===================================================================== #
      # Delegate the job to class RBT::FetchAllM4Macros
      # ===================================================================== #
      RBT::FetchAllM4Macros.new(i) { :do_not_report_anything_if_there_were_no_matches }
    end
  end

  # ========================================================================= #
  # === has_an_error_occurred?
  # ========================================================================= #
  def has_an_error_occurred?
    RBT.has_an_error_occurred?
  end

  # ========================================================================= #
  # === use_gobolinux_symlinkprogram?
  # ========================================================================= #
  def use_gobolinux_symlinkprogram?
    @internal_hash[:use_gobolinux_symlinkprogram]
  end

  # ========================================================================= #
  # === host_cpu?
  # ========================================================================= #
  def host_cpu?
    ::RBT.host_cpu?
  end

  # ========================================================================= #
  # === symlink_program?
  # ========================================================================= #
  def symlink_program?
    @internal_hash[:symlink_program]
  end

  # ========================================================================= #
  # === create_a_new_current_symlink_here
  # ========================================================================= #
  def create_a_new_current_symlink_here(i)
    target = "#{File.dirname(i)}/Current"
    if File.symlink? target
      remove_symlink(target)
    end
    e "#{rev}Creating a symlink at #{slateblue(i)}#{rev}."
    symlink(i, target)
  end

  # ========================================================================= #
  # === register_program
  #
  # This method will register the program files into a yaml "database".
  #
  # The first argument to this method may be an AppDir (String) such
  # as '/home/Programs/Libfontenc/1.1.5/'.
  #
  # The second argument to this method can be used to determine whether
  # we will really register the program at hand.
  # ========================================================================= #
  def register_program(
      this_program                = compile_which_program?,
      can_we_register_the_program = RBT.register_program_files_into_yaml_database?
    )
    if can_we_register_the_program
      require 'rbt/utility_scripts/register_program_files_into_yaml_database.rb'
      if this_program.include?('/')
        # In this case assume a target such as
        # "/home/Programs/Libfontenc/1.1.5/"
        appdir_prefix_of_this_program = this_program.dup
      else # else we have to determine the AppDir on our own
        appdir_prefix_of_this_program = static_appdir_prefix_of?(this_program)
      end
      # ===================================================================== #
      # Next we will delegate towards class
      # RBT::RegisterProgramFilesIntoYamlDatabase, but this can be
      # controlled via the above flag.
      # ===================================================================== #
      RBT::RegisterProgramFilesIntoYamlDatabase.new(
        appdir_prefix_of_this_program
      ) # bl RegisterProgramFilesIntoYamlDatabase
    end
  end

  # ========================================================================= #
  # === do_invoke_symlink_program
  #
  # This will either use the RBT-internal class called RBT::SymlinkProgram;
  # or it will, on GoboLinux, make use of SymlinkProgram instead.
  # ========================================================================= #
  def do_invoke_symlink_program(
      i                        = compile_which_program?,
      use_this_program_version = program_version?
    )
    if use_gobolinux_symlinkprogram?
      # ================================================================= #
      # === Delegate onto GoboLinux next
      #
      # In this case, use the GoboLinux program called 'SymlinkProgram.'
      #
      # This program then requires two arguments:
      #
      #   (1) the program name, and
      #   (2) the program_version
      #
      # ================================================================= #
      # We should also check via 'NamingConventions'.
      # ================================================================= #
      _ = "SymlinkProgram #{compile_which_program?} #{use_this_program_version}"
      esystem _
    else
      symlink_program?.run
      # ================================================================= #
      # Since as of September 2022 we will use an action here. The
      # second argument must be the appdir-prefix to the corresponding
      # bin/ directory. We will, however had, also supply sbin/, for
      # compatibility reasons mostly - thus why an Array is passed.
      # ================================================================= #
      action(
        :symlink_into_the_usr_bin_hierarchy,
        [
          "#{prefix?}bin/",
          "#{prefix?}sbin/"
        ]
      )
    end
  end

  # ========================================================================= #
  # === symlink_program
  # ========================================================================= #
  def symlink_program(this_program)
    unless RBT.const_defined? :SymlinFromToCurrent
      begin
        require 'rbt/requires/require_symlink_this_program.rb'
      rescue LoadError
      end
    end
    RBT::SymlinkFromToCurrent.new(this_program)
  end

  # ========================================================================= #
  # === cflags?
  # ========================================================================= #
  def cflags?
    ENV['CFLAGS']
  end

  # ========================================================================= #
  # === return_build_type
  # ========================================================================= #
  def return_build_type
    return ::RBT.build_type?
  end

  # ========================================================================= #
  # === use_porg?
  #
  # Whether we will use porg for installation or whether we will not.
  #
  # Most users probably do not need/want this, so this is mostly for
  # my home system, but also those users who may wish to use porg.
  # ========================================================================= #
  def use_porg?
    RBT.configuration?.use_porg
  end

  # ========================================================================= #
  # === always_show_dependencies?
  #
  # If this setting is true then we will always show the dependencies
  # on the commandline.
  # ========================================================================= #
  def always_show_dependencies?
    RBT.configuration?.always_show_dependencies
  end

  # ========================================================================= #
  # === use_stow?
  # ========================================================================= #
  def use_stow?
    RBT.configuration?.use_stow
  end

  # ========================================================================= #
  # === email?
  # ========================================================================= #
  def email?
    RBT.configuration?.email.to_s
  end

  # ========================================================================= #
  # === use_ccache?
  #
  # Whether we will make use of ccache or whether we don't, by default.
  # ========================================================================= #
  def use_ccache?
    RBT.configuration?.use_ccache
  end

  # ========================================================================= #
  # === use_mrxvt?
  # ========================================================================= #
  def use_mrxvt?
    RBT.configuration?.use_mrxvt
  end

  # ========================================================================= #
  # === use_tags?
  # ========================================================================= #
  def use_tags?
    RBT.configuration?.use_tags
  end

  # ========================================================================= #
  # === play_song?
  #
  # Query over as to whether we will play a song or not, after compilation
  # has finished.
  # ========================================================================= #
  def play_song?
    RBT.configuration?.play_song
  end

  # ========================================================================= #
  # === recipes_dir?
  # ========================================================================= #
  def recipes_dir?
    RBT.configuration?.recipes_dir
  end

  # ========================================================================= #
  # === config?
  # ========================================================================= #
  def config?
    RBT.configuration?
  end

  # ========================================================================= #
  # === exit_on_unregistered_cookbook_entries?
  # ========================================================================= #
  def exit_on_unregistered_cookbook_entries?
    RBT.configuration?.exit_on_unregistered_cookbook_entries
  end

  # ========================================================================= #
  # === run_ldconfig?
  # ========================================================================= #
  def run_ldconfig?
    RBT.configuration?.run_ldconfig
  end; alias use_ldconfig? run_ldconfig? # === use_ldconfig?

  # ========================================================================= #
  # === capitalize_program_names?
  # ========================================================================= #
  def capitalize_program_names?
    RBT.configuration?.capitalize_program_names
  end; alias do_capitalize? capitalize_program_names? # === do_capitalize?

  # ========================================================================= #
  # === RBT::Base.make_command?
  # ========================================================================= #
  def self.make_command?
    RBT.configuration?.make_command # The Configuration object has our make command.
  end

  # ========================================================================= #
  # === editor_to_use?
  # ========================================================================= #
  def editor_to_use?
    _ = ENV['EDITOR']
    if _.nil? or _.empty?
      _ = 'bluefish'
    end
    return _
  end

  # ========================================================================= #
  # === report_to_developers
  #
  # Feedback information to the user so that he can report useful
  # information to the project lead.
  # ========================================================================= #
  def report_to_developers(
      this_email = RBT.configuration?.email
    )
    ewarn 'To report to the developer(s) of RBT, use this '\
          'email address:'
    opnn; efancy "  #{this_email}"
    opnwarn 'You can also use this URL:'
    opnn; efancy "  #{RBT_HOMEPAGE_URL}"
    if RBT_HOMEPAGE_URL.include? 'rbt'
      opnn; efancy '  https://rubygems.org/profiles/58718/'
    end
    opne
  end

  # ========================================================================= #
  # === cleanup_system_tag_directory
  # ========================================================================= #
  def cleanup_system_tag_directory
    # ======================================================================= #
    # Be quiet - and only report when a stray symlink is removed.
    # ======================================================================= #
    action(:CleanupSystemTags) { :be_quiet }
  end

  # ========================================================================= #
  # === update_kde_apps
  # ========================================================================= #
  def update_kde_apps
    action(:UpdateKdeApplications)
  end

  # ========================================================================= #
  # === validate_the_cookbook_aliases
  # ========================================================================= #
  def validate_the_cookbook_aliases
    action(:ValidateCookbookAliases) # bl $RUBY_COOKBOOKS/lib/cookbooks/validate_cookbook_aliases.rb
  end

  # ========================================================================= #
  # === installer_constant?
  # ========================================================================= #
  def installer_constant?
    RBT::Action::SoftwareManager
  end

end; end
