#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will hold the relevant remote URL entries for the classes
# found in this directory.
# =========================================================================== #
# require 'rbt/constants/remote_URLs.rb'
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT::DOWNLOAD_KDE_ORG
  # ========================================================================= #
  DOWNLOAD_KDE_ORG = 'https://download.kde.org/'

  # ========================================================================= #
  # === RBT::REMOTE_URL_FOR_KDE_APPLICATIONS
  #
  # The KDE applications include software such as "dolphin". They are now
  # called "KDE Gear" rather than "KDE Applications".
  #
  # The URL may look like this:
  #
  #   https://download.kde.org/stable/release-service/?C=M;O=D
  #
  # To batch-update these applications, do:
  #
  #   update_kde_applications
  #   updatekdeapplications
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_APPLICATIONS =
    "#{DOWNLOAD_KDE_ORG}stable/release-service/24.02.0/src/"

  # ========================================================================= #
  # === REMOTE_URL_FOR_KDE_FRAMEWORKS
  #
  # This URL should point to the latest KDE-frameworks release.
  #
  # Presently, (08 September 2018) the latest KDE-frameworks release is
  # 5.50.0 (the URL, oddly enough, does not include the trailing "0").
  # Subsequent releases neither have had a trailing "0".
  #
  # To batch-update from the commandline, try this:
  #
  #   update_kde_frameworks
  #   updatekdeframeworks
  #
  # ========================================================================= #
  # https://download.kde.org/stable/frameworks/5.115/
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_FRAMEWORKS =
    "#{DOWNLOAD_KDE_ORG}stable/frameworks/5.115/"

  # ========================================================================= #
  # === REMOTE_URL_FOR_KDE_PORTING_AIDS
  #
  # This constant must come after the constant REMOTE_URL_FOR_KDE_FRAMEWORKS
  # has been defined.
  #
  # To update these, try:
  #
  #   rbt --update_porting_aids
  #   update_porting_aids
  #
  # ========================================================================= #
  # https://download.kde.org/stable/frameworks/5.115/portingAids/
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_PORTING_AIDS =
    "#{REMOTE_URL_FOR_KDE_FRAMEWORKS}portingAids/"

  # ========================================================================= #
  # === REMOTE_URL_FOR_KDE_PLASMA                                (plasma tag)
  #
  # Always point to the latest KDE-Plasma (upstream) URL here with this
  # constant, such as https://www.kde.org/info/plasma-5.14.3.php.
  #
  # To invoke updating all KDE-plasma components from the commandline,
  # you may issue the following command:
  #
  #   update_kde_plasma
  #   updatekdeplasma
  #
  # ========================================================================= #
  REMOTE_URL_FOR_KDE_PLASMA =
    # "#{DOWNLOAD_KDE_ORG}stable/plasma/5.27.11/"
    "#{DOWNLOAD_KDE_ORG}stable/plasma/6.0.2/"

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_URL_FOR_KDE_PLASMA
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL_FOR_KDE_PLASMA =
    "#{DOWNLOAD_KDE_ORG}stable/plasma/"

  # ========================================================================= #
  # === REMOTE_PUB_MATE_DESKTOP_RELEASES
  # ========================================================================= #
  REMOTE_PUB_MATE_DESKTOP_RELEASES =
    'https://pub.mate-desktop.org/releases/'

  # ========================================================================= #
  # === REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION            (mate tag)
  #
  # This should be used as the base for the download-functionality in
  # the file check_for_mate_desktop_updates.rb
  #
  # This would correspond to an URL like:
  #
  #   https://pub.mate-desktop.org/releases/1.24/
  #   https://pub.mate-desktop.org/releases/1.26/
  #   https://pub.mate-desktop.org/releases/
  #
  # ========================================================================= #
  REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION =
    "#{REMOTE_PUB_MATE_DESKTOP_RELEASES}1.26/"

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT
  # ========================================================================= #
  REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT = 
    'https://downloads.lxqt.org/'

  # ========================================================================= #
  # === CURRENT_VERSION_OF_LXQT
  #
  # Use this constant to keep track as to which lxqt version is the
  # one that is currently tracked by RBT. It has to be manually
  # updated (for now).
  # ========================================================================= #
  CURRENT_VERSION_OF_LXQT = '0.14.1'

  # ========================================================================= #
  # === REMOTE_DOWNLOAD_URL_FOR_LXQT
  #
  # The following constant should have an URL such as:
  #   https://downloads.lxqt.org/0.14.1.html
  # ========================================================================= #
  REMOTE_DOWNLOAD_URL_FOR_LXQT =
    "#{REMOTE_DOWNLOAD_BASE_URL_FOR_LXQT}#{CURRENT_VERSION_OF_LXQT}.html"

  # ========================================================================= #
  # === REMOTE_SLACKWARE_HOMEPAGE
  # ========================================================================= #
  REMOTE_SLACKWARE_HOMEPAGE =
    'https://www.slackware.com/changelog/current.php?cpu=x86_64'

  # ========================================================================= #
  # === BASE_URL_TO_GNOME_SOURCES
  # ========================================================================= #
  BASE_URL_TO_GNOME_SOURCES =
    'https://ftp.gnome.org/pub/GNOME/sources/'

  # ========================================================================= #
  # === BLFS_CHANGELOG_WEBPAGE
  # ========================================================================= #
  BLFS_CHANGELOG_WEBPAGE =
    #'https://www.linuxfromscratch.org/blfs/view/cvs/introduction/changelog.html'
    'https://www.linuxfromscratch.org/blfs/view/svn/introduction/changelog.html'

  # ========================================================================= #
  # === REMOTE_URL_TO_THE_MATE_DESKTOP_COMPONENTS
  #
  # This constant specifies where we can find the (remote) source code
  # to the mate-desktop, via HTTP-file listing.
  # ========================================================================= #
  REMOTE_URL_TO_THE_MATE_DESKTOP_COMPONENTS =
    "#{REMOTE_PUB_MATE_DESKTOP_RELEASES_LATEST_VERSION}?C=M&O=D"

  # ========================================================================= #
  # === URL_TO_REMOTE_CHANGELOG
  #
  # Where to keep the remote changelog, if we make use of one that is.
  #
  # This is no longer in use as of February 2024 - but, for the time
  # being, we will retain it.
  # ========================================================================= #
  URL_TO_REMOTE_CHANGELOG =
    'https://shevy.bplaced.net/rbt_changelog.html'

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e
  e "  #{RBT::REMOTE_URL_FOR_KDE_APPLICATIONS}"
  e
end