#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckIfSourceDirectoriesExist
#
# This class will iterate through the various cookbook entries and count
# what we have got there, inside the directories where these cookbook
# reside.
#
# The class will then report which directories do not exist.
# =========================================================================== #
# require 'rbt/cookbooks/check_for_existing_source_directories.rb'
# RBT.check_if_source_directories_exist
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CheckIfSourceDirectoriesExist < Base # === RBT::Cookbooks::CheckIfSourceDirectoriesExist

  # ========================================================================= #
  # === PADDING
  # ========================================================================= #
  PADDING = 35

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_input = nil,
      run_already    = true
    )
    reset
    set_commandline_arguments(
      optional_input
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @array_targets
    # ======================================================================= #
    @array_targets = []
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_do_not_be_verbose # if true then we will also report directories that exist.
  end

  # ========================================================================= #
  # === show_help_then_exit
  # ========================================================================= #
  def show_help_then_exit
    show_help :then_exit
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help(
      optional_then_exit = false
    )
    begin
      require 'class_docu_shower'
    rescue LoadError; end
    if Object.const_defined? :ClassDocuShower
      ClassDocuShower.new(__FILE__)
    end
    exit if optional_then_exit
  end

  # ========================================================================= #
  # === check_commandline_arguments
  # ========================================================================= #
  def check_commandline_arguments
    case first_argument? # case tag
    # ======================================================================= #
    # === --help
    # ======================================================================= #
    when /^-?-?help$/i
      show_help_then_exit
    end
  end

  # ========================================================================= #
  # === register_target_does_not_exist
  # ========================================================================= #
  def register_target_does_not_exist(i)
    @array_targets << i
  end

  # ========================================================================= #
  # === report_missing_entries
  #
  # Report all missing entries here.
  # ========================================================================= #
  def report_missing_entries # report them all now.
    sanitize_targets
    # ======================================================================= #
    # Next display all found targets:
    # ======================================================================= #
    e
    @array_targets.sort.each {|entry|
      e "  #{sdir(entry.squeeze('/').ljust(PADDING))} #{rev}does "\
        "#{tomato('not')} #{rev}exist."
    }
    e
    unless @array_targets.empty?
      _ = @array_targets.size.to_s
      # ===================================================================== #
      # Always report non-existing directories.
      # ===================================================================== #
      opne "#{simp(_)} #{rev}non-existing directories were found."
    end
  end

  # ========================================================================= #
  # === sanitize_targets
  # ========================================================================= #
  def sanitize_targets
    @array_targets.map! {|entry| entry << '/' }
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    PADDING
  end

  # ========================================================================= #
  # === load_this_cookbook
  # ========================================================================= #
  def load_this_cookbook(i)
    action(:SanitizeCookbook, i) { :fast }
  end

  # ========================================================================= #
  # === check_each_entry
  # ========================================================================= #
  def check_each_entry
    available_programs?.each {|entry|
      target = source_directory?+File.basename(entry).sub(/\.yml$/,'')+'/'
      shall_we_check_this_target = false
      if File.exist? target # ok, the target exists, all may be fine, so we pass through.
        shall_we_check_this_target = true
      end
      # ===================================================================== #
      # Some entries in the yaml file have a "do_not_download"
      # entry. In that case we will ignore them. 
      # ===================================================================== #
      location = individual_cookbooks_directory?+File.basename(entry)
      if File.exist? location
        hash = load_this_cookbook(location) # in module_methods.rb
        unless hash['do_not_download'].nil?
        end
      end
      if shall_we_check_this_target
        if be_verbose?
          e 'Target '+sfile(target.ljust(padding?))+' does exist.'
        end
      else # does not exist, report
        register_target_does_not_exist(target)
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_commandline_arguments
    check_each_entry
    report_missing_entries
  end

end; end

# =========================================================================== #
# === RBT.check_if_source_directories_exist
# =========================================================================== #
def self.check_if_source_directories_exist
  RBT::Cookbooks::CheckIfSourceDirectoriesExist.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckIfSourceDirectoriesExist.new(ARGV)
end # cexist HELP
    # cexist