#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForInclusion
#
# This script scans over all entries in $SRC and checks
# whether they are included in the cookbook or not.
# =========================================================================== #
# require 'rbt/cookbooks/check_for_inclusion/check_for_inclusion.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CheckForInclusion < Base # === RBT::Cookbooks::CheckForInclusion

  # ========================================================================= #
  # === ARRAY_WITH_EXCEPTIONS
  #
  # The following Array has a list of exceptions. If you wish to skip
  # entries then simply add them here.
  # ========================================================================= #
  ARRAY_WITH_EXCEPTIONS = %w(
    /home/x/src/bios/
    /home/x/src/firmware/
    /home/x/src/firefox/
    /home/x/src/firefox/extensions/
    /home/x/src/flash/
    /home/x/src/fonts/
    /home/x/src/gobolinux/
    /home/x/src/hardware/
    /home/x/src/nvidialinux/
    /home/x/src/openjdk/
    /home/x/src/windows/
    /home/x/src/wlan/
  )

  # ========================================================================= #
  # === DEFAULT_FILE
  #
  # Where to store our results. A .md markdown file may seem appropriate.
  # ========================================================================= #
  DEFAULT_FILE = "#{RBT.depot_directory?}Information/UnidentifiedFiles.md"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @base_dir
    # ======================================================================= #
    @base_dir = source_directory?
    # ======================================================================= #
    # === @array_files_which_were_not_found
    # ======================================================================= #
    @array_files_which_were_not_found = []
    # ======================================================================= #
    # === @all_files
    # ======================================================================= #
    @all_files = []
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose
    # ======================================================================= #
    # === @query
    # ======================================================================= #
    @query = action(:SanitizeCookbook, :do_not_run_yet) { :fast }
  end

  # ========================================================================= #
  # === scan_for_all_files
  #
  # Scan for all files and fill up the array @all_files
  # ========================================================================= #
  def scan_for_all_files
    _ = @base_dir+'*/**'
    opne 'Scanning for all files in directory `'+sdir(_)+rev+
         '` now.'
    opne 'Be patient, this may take a while.'
    @all_files = Dir[_].sort
  end

  # ========================================================================= #
  # === check_each_file_whether_it_is_included
  #
  # Here we report only if @be_verbose is true. (3)
  # ========================================================================= #
  def check_each_file_whether_it_is_included
    @all_files.each { |file_long_name, file_short_name|
      unless ARRAY_WITH_EXCEPTIONS.empty?
        dirname = File.dirname(file_long_name)+'/'
        next if ARRAY_WITH_EXCEPTIONS.include?(dirname)
      end
      file_short_name = ProgramInformation.return_real_name(
        File.basename(file_long_name)
      )
      file_short_name.downcase! if file_short_name
      e "#{rev}Now #{steelblue('scanning')} #{rev}for the "\
        "program `#{sfile(file_short_name)}#{rev}`."
      # ===================================================================== #
      # If it is included then everything is fine.
      # ===================================================================== #
      if is_included?(file_short_name)
        # f '`'+file_long_name+'` is included.' if be_verbose? # Be silent here. 
      else # Else it is not included.
        if be_verbose?
          opnwarn '`'+simp(file_long_name)+
                      swarn('` is not included.')
        end
        @array_files_which_were_not_found << file_long_name
      end
    }
  end

  # ========================================================================= #
  # === is_included?
  #
  # Query whether the program is included or not.
  # ========================================================================= #
  def is_included?(i)
    i.delete!('_') if i.include? '_'
    is_included = @query.is_included?(i)
    return true if is_included
    false # default return value
  end

  # ========================================================================= #
  # === report_to_user_which_files_are_not_included
  # ========================================================================= #
  def report_to_user_which_files_are_not_included
    e sfancy('These files were not found:')
    e
    cliner {
      @array_files_which_were_not_found.each {|entry|
        e steelblue("  #{entry}")
      }
    }
    e
  end

  # ========================================================================= #
  # === store_results_to_file
  # ========================================================================= #
  def store_results_to_file(
      what     = @array_files_which_were_not_found,
      where_to = DEFAULT_FILE
    )
    remove_file(where_to) # Delete the old file first, before appending into a new one.
    @array_files_which_were_not_found.each { |f|
      append_what_into(f+N, where_to)
    }
    e 'Stored into:'
    e sfancy("  #{where_to}")
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    reset                                       # (1)
    scan_for_all_files                          # (2)
    check_each_file_whether_it_is_included      # (3)
    report_to_user_which_files_are_not_included # (4)
    store_results_to_file                       # (5)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForInclusion.new
end # checkforinclusion