#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ConvertDateFormat
#
# This class can convert a Date, such as "14 Mar 2023" into "14 March 2023",
# in a given cookbook .yml file.
# =========================================================================== #
# require 'rbt/cookbooks/utility_scripts/convert_date_format.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ConvertDateFormat < Base # === RBT::Cookbooks::ConvertDateFormat

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                   (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ===================================================================== #
    # === @base_directory_for_the_date_converted_yaml_files
    # ===================================================================== #
    @base_directory_for_the_date_converted_yaml_files = rbt_log_dir?+
                                                        'date_converted_yaml_files/'
  end

  # ========================================================================= #
  # === do_process_this_file
  # ========================================================================= #
  def do_process_this_file(i)
    _ = default_read(i)
    if _.include? 'last_update:'
      last_update_entry = _.scan(/ last_update: (.*)$/)
      if last_update_entry.is_a? Array
        last_update_entry = last_update_entry.flatten.first
      end
      if is_this_date_already_in_the_correct_format?(last_update_entry, i)
        opne "#{rev}All is fine - nothing has to be changed. "+
             steelblue('\o/')
      else
        do_change_this_file(i, last_update_entry)
      end
    else
      e "#{rev}No last_update entry exists in the "\
        "file at #{sfile(i)}#{rev}."
    end
  end

  # ========================================================================= #
  # === is_this_date_already_in_the_correct_format?
  # ========================================================================= #
  def is_this_date_already_in_the_correct_format?(
      i, optional_this_file = nil
    )
    split_char_to_be_used = ' '
    if i.include?('.') and !i.include?(' ')
      split_char_to_be_used = '.'
    end
    splitted = i.split(split_char_to_be_used)
    month_entry = splitted[1]
    if month_entry =~ /^\d{1,2}$/ # Only numbers.
      opne tomato('Only numbers were returned for the entry '+
           optional_this_file.to_s+
           '. Changing this to')
      opne tomato('an incorrect date deliberately so.')
      month_entry = return_month_based_on_this_number(month_entry)
    end
    months?.include?(month_entry)
  end

  # ========================================================================= #
  # === do_change_this_file
  #
  # The first argument to this method is the name of the locally existing
  # file.
  #
  # When this method is called then we can be certain that there already
  # was a prior check, determining that this file HAS to be modified.
  # ========================================================================= #
  def do_change_this_file(
      i, last_update_entry
    )
    # ===================================================================== #
    # First, we have to determine the correct date.
    # ===================================================================== #
    months = months?
    new_date = last_update_entry.sub(//, '') # From "Mar" to "March", for instance.

    split_char_to_be_used = ' '
    if last_update_entry.include?('.') and !last_update_entry.include?(' ')
      split_char_to_be_used = '.'
    end
    splitted = last_update_entry.split(split_char_to_be_used)
    month_entry = splitted[1]
    new_month = months.select {|entry|
      entry.start_with?(month_entry)
    }.first
    if new_month.nil?
      opne "#{rev}No new month entry was found for "\
           "the file #{sfile(i)}#{rev}."
      opne 'Please fix the .yml file.'
      exit
    end
    new_date = splitted[0]+' '+new_month+' '+splitted[2]
    dataset = default_read(i)
    dataset.gsub!(/ last_update: (.+)$/, " last_update: #{new_date}")
    # ======================================================================= #
    # First, create a backup of the file:
    # ======================================================================= #
    target = @base_directory_for_the_date_converted_yaml_files+
             "backup_of_#{File.basename(i)}"
    opne "#{rev}Creating a backup of the file at #{sfile(i)}"
    opne "#{rev}towards #{sfile(target)}"
    copy_file(i, target)
    opne "#{rev}Now modifying the file #{sfile(i)}#{rev}."
    opne "#{rev}The new date is #{tomato(new_date)}#{rev}."
    # ======================================================================= #
    # Now we can modify the original file as-is:
    # ======================================================================= #
    write_what_into(dataset, i)
  end; private :do_change_this_file

  # ========================================================================= #
  # === create_the_log_directory_for_this_class
  # ========================================================================= #
  def create_the_log_directory_for_this_class(
      i = @base_directory_for_the_date_converted_yaml_files
    )
    mkdir_p(i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_the_log_directory_for_this_class
    _ = commandline_arguments?
    # ======================================================================= #
    # Operate in a batch-centric manner next:
    # ======================================================================= #
    _.each {|this_file|
      if this_file and File.exist?(this_file)
        do_process_this_file(this_file)
      else
        opnn; no_file_exists_at(this_file)
      end
    }
  end

  # ========================================================================= #
  # === RBT::Cookbooks::ConvertDateFormat[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.convert_date_format
# =========================================================================== #
def self.convert_date_format(i = ARGV)
  RBT::Cookbooks::ConvertDateFormat.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ConvertDateFormat.new(ARGV)
end # convertdateformat /home/x/programming/ruby/src/rbt/lib/rbt/yaml/cookbooks/htop.yml