#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::DisplayCookbookDataset
#
# This class can show the dataset associated to a specific program at
# hand.
# =========================================================================== #
# require 'rbt/cookbooks/display_cookbook_dataset/class.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class DisplayCookbookDataset < Base # === RBT::Cookbooks::DisplayCookbookDataset

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === REPLACE_PLUS_WITH_LONG_NAME
  # ========================================================================= #
  REPLACE_PLUS_WITH_LONG_NAME =
     RBT::Action::Cookbooks::SanitizeCookbook

  # ========================================================================= #
  # === RBT::Cookbooks::DisplayCookbookDataset::LEFT_PADDING
  # ========================================================================= #
  LEFT_PADDING = 30

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program = ARGV,
      run_already  = true
    )
    reset # Defined in reset.rb
    set_program(this_program)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :do_not_show_md5sum
        set_show_md5sum false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === set_show_md5sum
  # ========================================================================= #
  def set_show_md5sum(i)
    @internal_hash[:show_md5sum] = i
  end

  # ========================================================================= #
  # === show_md5sum?
  # ========================================================================= #
  def show_md5sum?
    @internal_hash[:show_md5sum]
  end

  # ========================================================================= #
  # === use_only_symbols_for_the_main_hash?
  # ========================================================================= #
  def use_only_symbols_for_the_main_hash?
    false # Hardcoded for now. We keep only Strings.
  end

  # ========================================================================= #
  # === main_program_entries?
  #
  # Query method over ARRAY_MAIN_PROGRAM_ENTRIES.
  # ========================================================================= #
  def main_program_entries?
    self.class.main_program_entries?
  end

  # ========================================================================= #
  # === set_program
  # ========================================================================= #  
  def set_program(i)
    i = i.first if i.is_a? Array
    @program = i
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end

  # ========================================================================= #
  # === determine_dataset
  # ========================================================================= #
  def determine_dataset(i = program?)
    @dataset = action(:SanitizeCookbook, i) { :fast }
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === display_this_option
  # ========================================================================= #
  def display_this_option(i = :short_name)
    result = ''.dup
    result << case i
    when :prefix
      'Prefix: '
    when :base_dir
      'Base dir: '
    when :short_name,
         :real_short_name
      'Short name: '
    end
    result = result.ljust(30)
    name_of_the_method = "#{i}?"
    result << sfancy(
      @dataset.send(name_of_the_method.to_sym).to_s
    )
    e "#{result}#{rev}"
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :show_md5sum
    # ======================================================================= #
    @internal_hash[:show_md5sum]    = show_md5sum?
    # ======================================================================= #
    # === :this_program
    # ======================================================================= #
    @internal_hash[:this_program]   = nil # Initialize it.
    # ======================================================================= #
    # === :programs_dir
    # ======================================================================= #
    @internal_hash[:programs_dir]   = programs_directory?
    # ======================================================================= #
    # === :base_dir
    # ======================================================================= #
    @internal_hash[:base_dir]       = source_directory? # Keep a reference to "/home/x/src/" or to any other value.
    # ======================================================================= #
    # === :extra_commands
    # ======================================================================= #
    @internal_hash[:extra_commands] = []
    @internal_hash[:found_result]   = false # We have not yet searched, so this defaults to false.
    @internal_hash[:may_we_exit]    = false
    @internal_hash[:wrap_result]    = true # If true then we will wrap the description of a given program.
    # ======================================================================= #
    # === :replace_plus_with_long_name
    #
    # We can replace '+' with 'plus' depending on whether the constant
    # REPLACE_PLUS_WITH_LONG_NAME tells us to do so. The program gtk+
    # is an example for this - it would become "gtkplus". If you do not
    # want to have '+' as directory names under /Programs/, then you
    # can enable this via that constant.
    # ======================================================================= #
    @internal_hash[:replace_plus_with_long_name] = REPLACE_PLUS_WITH_LONG_NAME
    reset_yaml_dataset
  end; alias clear reset # === clear

  # ========================================================================= #
  # === reset_yaml_dataset
  # ========================================================================= #
  def reset_yaml_dataset
    @yaml_dataset = {}
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    determine_dataset
    opne "#{rev}Displaying information about the "\
         "program #{sfancy(@program)}#{rev}."
    dataset = dataset?
    ljust_value = 30
    # ======================================================================= #
    # Display important values.
    # ======================================================================= #
    display_this_option :prefix; ee rev
    e 'Program path: '.ljust(ljust_value)+sfancy(dataset.program_path?.to_s)+rev
    e 'Dependencies on: '.ljust(ljust_value)+sfancy(dataset.deps_on?.join(',').rstrip.to_s)+rev
    display_this_option :base_dir
    display_this_option :short_name
    display_this_option :real_short_name
    e 'Archive size: '.ljust(ljust_value)+sfancy(dataset.archive_size?.to_s)+rev
    if show_md5sum?
      e 'md5sum: '.ljust(ljust_value)+sfancy(dataset.md5sum?.to_s)+rev
    end
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::DisplayCookbookDataset.new(ARGV)
end # scookie2 vim