#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::FindDuplicateBinaries
#
# This class will check the /Programs/ hierarchy respectively the 
# /System/ hierarchy for all binaries - or wherever the user hat
# designeted his or her PROGRAMS directory to reside at.
#
# Whenever we find a match that also exists at /usr/bin/, we will
# collect it. Then, we tell the user which binary files are
# duplicates.
#
# If both the /usr/bin/ target is the same as /System/Index/bin/,
# then evidently this would result a garbage value.
#
# Usage example:
#
#   RBT::Cookbooks::FindDuplicateBinaries.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/find_duplicate_binaries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class FindDuplicateBinaries < Base # === RBT::Cookbooks::FindDuplicateBinaries

  # ========================================================================= #
  # === DELETE_DUPLICATE_FILES
  #
  # If this constant is true then we will also delete the duplicate
  # binaries. This can be toggled by the user on the commandline.
  #
  # By default, we will NOT delete the duplicate entries and this will
  # also not be changed. The user specifically has to enable it on
  # his/her own.
  # ========================================================================= #
  DELETE_DUPLICATE_FILES = false

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @delete_duplicate_files
    # ======================================================================= #
    @delete_duplicate_files = DELETE_DUPLICATE_FILES
  end

  # ========================================================================= #
  # === delete_duplicates?
  # ========================================================================= #
  def delete_duplicates?
    @delete_duplicate_files
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === grab_files_from_usr_bin
  # ========================================================================= #
  def grab_files_from_usr_bin
    @usr_bin = Dir["#{ubin_dir}*"]
  end

  # ========================================================================= #
  # === ubin_dir
  # ========================================================================= #
  def ubin_dir
    '/usr/bin/'
  end; alias ubin?     ubin_dir # === ubin?
       alias ubin_dir? ubin_dir # === ubin_dir?

  # ========================================================================= #
  # === found_duplicates?
  # ========================================================================= #
  def found_duplicates?
    !@duplicate_files.empty?
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(
      i = @input
    )
    i = i.first if i.is_a? Array
    case i # case tag
    # ======================================================================= #
    # === duplicate_binaries --help
    # ======================================================================= #
    when /^-?-?help$/i
      show_help_options
      exit
    # ======================================================================= #
    # === duplicate_binaries --delete
    # ======================================================================= #
    when /^-?-?delete$/i,
         /^-?-?purge$/i,
         '--toggle'
      @delete_duplicate_files = true # Toggle.
    end
  end

  # ========================================================================= #
  # === show_help_options                                          (help tag)
  # ========================================================================= #
  def show_help_options
    e 'Currently we support only one option:'
    e
    ecomment '   --delete # delete duplicate files found in '\
             'the '+sdir(ubin?)+
             comment(' hierarchy') # finddup --help
    e
  end

  # ========================================================================= #
  # === sysdir                                                   (sysdir tag)
  #
  # This method will refer to "/System/Index/bin" normally - at the least
  # on a GoboLinux-like system.
  # ========================================================================= #
  def sysdir?
    sysbin_directory?
  end; alias sysbin?     sysdir? # === sysbin?
       alias sysdir      sysdir? # === sydir
       alias main_dir?   sysdir? # === main_dir?
       alias sysbin_dir? sysdir? # === sysbin_dir?

  # ========================================================================= #
  # === do_delete_duplicate_files
  #
  # This will delete only files in the /usr/bin/ hierarchy. 
  # ========================================================================= #
  def do_delete_duplicate_files
    if delete_duplicates? and !@duplicate_files.empty?
      opne 'Now deleting duplicate files from the '+sdir(ubin?)+
           rev+' directory.'
      @duplicate_files.each {|entry|
        _ = sysbin?+File.basename(entry)
        if File.exist? _ # Extra safeguard here.
          e 'The file path at '+
            sfile(ubin?+entry)+
            ' is a duplicate '\
            'of `'+sfile(File.readlink(_))+'`.'
          e "Thus we can delete `#{sfile(ubin?+entry)}`."
          delete(ubin?+entry) # Delete it finally.
        end
      }
    end
  end

  # ========================================================================= #
  # === grab_files_from_system_index_bin
  #
  # This will typically obtain all entries from /System/Index/bin/.
  # ========================================================================= #
  def grab_files_from_system_index_bin(i = sysbin_directory?)
    @system_executables = Dir["#{i}*"]
  end

  # ========================================================================= #
  # === detect_duplicate_files
  #
  # This will detect all duplicate files.
  # ========================================================================= #
  def detect_duplicate_files
    @duplicate_files = [] # Empty at the beginning.
    ubin_array   = @usr_bin.map {|entry| File.basename(entry) }.sort
    sysbin_array = @system_executables.map {|entry| File.basename(entry) }.sort
    counter = 0
    # ======================================================================= #
    # Next, compare the ubin-array with the sysbin-array:
    # ======================================================================= #
    ubin_array.each {|entry|
      if sysbin_array.include? entry
        counter += 1
        _ = ("(#{simp(counter.to_s)}) ").rjust(20)
        e _+'Yes, '+slateblue(entry)+' is included. It is thus '+
          steelblue('a duplicate')+'.'
        @duplicate_files << entry
      end
    }
    @duplicate_files.sort! # And keep them sorted alphabetically.
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do_next
  # ========================================================================= #
  def report_to_the_user_what_we_will_do_next
    opne 'We will compare all entries from '+
         sdir(sysdir)+' with '\
         'entries from `'+
         sdir(ubin_dir)+'`.'
    opne 'Duplicate entries will be reported.'
  end; private :report_to_the_user_what_we_will_do_next

  # ========================================================================= #
  # === report_duplicate_files
  #
  # This method will report whether we have duplicate files.
  # ========================================================================= #
  def report_duplicate_files
    if found_duplicates?
      opne 'We have found these '+simp(@duplicate_files.size.to_s)+' '\
           'duplicates.'
      opne "They originate from the directory `#{sdir(ubin_dir?)}` and"
      opne 'are compared to the directory `'+sdir(sysbin_dir?)+'.'
      # ======================================================================= #
      # I am not sure why the code below exists; I assume I may have
      # wanted to show which entries are duplicates. But it is quite
      # spammy, so it was disabled on 25.05.2020. It may be re-enabled
      # at a later time, but then perhaps via a method call, and in
      # a slightly different formatted output, including colours.
      # But it may also be removed entirely instead - we will see.
      #   long_string = word_wrap(
      #     @duplicate_files.join(' ').strip, 62
      #   )
      #   long_string.chop! if long_string.end_with? ',' # Get rid of traling ','.
      #   e "  #{long_string.strip}"
      # ======================================================================= #
    else
      opne 'We did not find any duplicate entries. Thus, we will '\
           'do no further action.'
    end
  end

  # ========================================================================= #
  # === are_the_two_target_directories_the_same_directory?
  # ========================================================================= #
  def are_the_two_target_directories_the_same_directory?
    sysdir == ubin_dir
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu # Check for the commandline-arguments first.
    report_to_the_user_what_we_will_do_next
    if are_the_two_target_directories_the_same_directory?
      opne 'The two directories are the same, so we can not '\
           'compare them -'
      opne 'it would make no sense.'
    else
      grab_files_from_system_index_bin
      grab_files_from_usr_bin
      detect_duplicate_files
      report_duplicate_files
      do_delete_duplicate_files if delete_duplicates?
    end
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::FindDuplicateBinaries.new(ARGV)
end # findduplicatebinaries