#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::LocalCheckForLatest
#
# This script reports to the user which packages are up-to-date and which
# ones are not. Te user can then use this information to remove the
# outdated program-versions, for instance.
#
# To invoke it, do:
#
#   latest
#
# =========================================================================== #
# require 'rbt/cookbooks/local_check_for_latest/local_check_for_latest.rb'
# RBT.local_check_for_latest
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class LocalCheckForLatest < Base # === RBT::Cookbooks::LocalCheckForLatest

  # ========================================================================= #
  # === STORE_HERE
  #
  # Specify where to store the results.
  # ========================================================================= #
  STORE_HERE = '/Depot/Information/LocalCheckForLatest.md'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  #
  # Initialize our variables here.
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @store_where
    # ======================================================================= #
    @store_where = STORE_HERE
    # ======================================================================= #
    # === @source_directory
    # ======================================================================= #
    @source_directory = "#{source_directory?}*/**"
    # ======================================================================= #
    # === @save_this_string_to_file
    # ======================================================================= #
    @save_this_string_to_file = ''.dup # we save this string to the file.
    # ======================================================================= #
    # === @cookbook_dataset
    # ======================================================================= #
    @cookbook_dataset = action(:SanitizeCookbook, :do_not_run_yet) { :fast }
  end

  # ========================================================================= #
  # === all_files?
  # ========================================================================= #
  def all_files?
    @all_files
  end

  # ========================================================================= #
  # === iterate_over_available_programs
  #
  # When we have stored all available programs in the array
  # @available_programs we can continue to iterate through it.
  # ========================================================================= #
  def iterate_over_available_programs
    @available_programs.each { |program|
      cliner {
        @save_this_string_to_file << N+N
        _ = "   ---> #{program}"
        @save_this_string_to_file << _+N
        opne _
        all_files?.each { |long, short|
          if program == short
            if is_latest?(long)
              _ = long+' this is the latest version locally. You should keep it.'
              # Explanation: We don't store what we should keep, we only store
              # what we should remove, as suggestion to the user.
              # @save_this_string_to_file << _+N
              # Disabled the above APR 2011.
              e _
            else
              _ = "#{long} is quite old. You could consider deleting it."
              @save_this_string_to_file << "#{_}#{N}"
              opnwarn _
            end
          end
        }
      }
    }
  end

  # ========================================================================= #
  # === get_all_local_files
  # ========================================================================= #
  def get_all_local_files
    @all_files = Dir.glob(@source_directory).sort
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    @store_where
  end

  # ========================================================================= #
  # === save_data_to_file
  #
  # Remove the file, then save the data into @store_where.
  # ========================================================================= #
  def save_data_to_file
    opne 'Storing data into:'
    e "  #{sfile(store_where?)}"
    remove_file(store_where?)
    write_what_into(@save_this_string_to_file, store_where?)
  end

  # ========================================================================= #
  # === get_avilable_programs
  # ========================================================================= #
  def get_available_programs
    @available_programs = available_programs?
  end

  # ========================================================================= #
  # === is_latest?
  #
  # The input to this method is something like:
  #
  #   /home/x/src/zsh/zsh-4.3.10.tar.bz2
  #
  # ========================================================================= #
  def is_latest?(input)
    short_name = ProgramInformation.return_name(input).downcase
    @cookbook_dataset.seek(short_name)
    _ = @cookbook_dataset.is_included?
    # The above is by default always true. Hence we must set it to false
    # in case ! .full_path == input
    the_path = @cookbook_dataset.full_path
    e the_path
    _ = false unless the_path == input
    return _
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    reset
    get_available_programs # From the cookbook yaml file.
    get_all_local_files
    iterate_over_available_programs
    save_data_to_file
  end

end; end

# =========================================================================== #
# === RBT.local_check_for_latest
# =========================================================================== #
def self.local_check_for_latest
  RBT::Cookbooks::LocalCheckForLatest.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::LocalCheckForLatest.new
end # latest