#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Url
#
# Just a small helper-class to output the URLs of a program in question.
# =========================================================================== #
# require 'rbt/cookbooks/url/url.rb'
# RBT.return_url1_of_this_program(:php)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class Url < Base # === RBT::Cookbooks::Url

  require 'rbt/aliases/aliases.rb'

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_find_these_programs(i)
    # ======================================================================= #
    # === Handle blocks given to this method
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet
        set_be_quiet
      # ===================================================================== #
      # === :show_the_homepage_as_well
      # ===================================================================== #
      when :show_the_homepage_as_well
        @show_the_homepage = true
      # ===================================================================== #
      # === :show_the_homepage_as_well_and_the_git_URL_should_it_exist
      # ===================================================================== #
      when :show_the_homepage_as_well_and_the_git_URL_should_it_exist
        @show_the_homepage = true
        @show_the_git_URL_as_well_should_it_exist = true
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @show_the_homepage
    # ======================================================================= #
    @show_the_homepage = false
    # ======================================================================= #
    # === @show_the_git_URL_as_well_should_it_exist
    # ======================================================================= #
    @show_the_git_URL_as_well_should_it_exist = false
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose # Explicitely set it here.
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = nil
    # ======================================================================= #
    # === :search_for
    # ======================================================================= #
    @internal_hash[:search_for] = nil
    # ======================================================================= #
    # === @show_leading_url_entry
    #
    # If this variable is set to true then we will show the leading
    # url1: part.
    # ======================================================================= #
    @show_leading_url_entry = true
    try_to_require_the_xorg_buffer
  end

  # ========================================================================= #
  # === set_find_these_programs
  #
  # Use this method to designate which program we wish to find the URL for.
  # ========================================================================= #
  def set_find_these_programs(i)
    i = [i].flatten.compact
    i.map! {|entry|
      entry = entry.to_s.strip
      entry.delete!('-') if entry.include? '-' # Since 05.11.2011
      entry.delete!('/') if entry.include? '/'
      entry.sub!(/\.yml$/,'') if entry.end_with? '.yml'
      case entry # case tag
      when /^PWD$/i
        entry = File.basename(return_pwd)
      end unless File.exist? entry
      entry.downcase! # Since as of 24.09.2014.
      # ======================================================================= #
      # Check for a Cookbook-alias next:
      # ======================================================================= #
      if RBT.is_an_alias?(entry)
        entry = RBT.find_cookbook_alias_for(entry)
      end unless RBT.has_this_program?(entry) # This check is necessary so that "ruby" won't become rubysqlite.
      # ======================================================================= #
      # If the input was not found, then we also check for 'lib' as prefix.
      #
      # This will correct commands such as "url memcached" into
      # "url libmemcached".
      # ======================================================================= #
      unless RBT.does_include? entry
        # ===================================================================== #
        # Check a +lib variant here.
        # ===================================================================== #
        if RBT.does_include? "lib#{entry}"
          entry.prepend('lib')
        # ===================================================================== #
        # Check a -lib variant here.
        # ===================================================================== #
        elsif RBT.does_include? entry.sub(/^lib/,'')
          entry.sub!(/^lib/,'')
        end
      end
      if entry.empty? and
         is_on_roebe? and
         return_pwd.include?(source_directory?)
        entry = File.basename(return_pwd)
      end
      entry
    }
    @find_these_programs = i
  end; alias search_for_url set_find_these_programs # === search_for_url

  # ========================================================================= #
  # === search_for?
  # ========================================================================= #
  def search_for?
    @internal_hash[:search_for]
  end; alias find? search_for? # === find?

  # ========================================================================= #
  # === array_urls?
  #
  # This method will return the URLs in Array format.
  # ========================================================================= #
  def array_urls?
    [url1, url2]
  end; alias result array_urls? # === result

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    @dataset.url1?
  end; alias return_main_url url1? # === return_main_url
       alias url1            url1? # === url1

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    @dataset.url2?
  end; alias return_second_url url2? # === return_second_url
       alias url               url2? # === url
       alias url?              url2? # === url?
       alias url2              url2? # === url2

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    @dataset.homepage?
  end; alias homepage homepage? # === homepage

  # ========================================================================= #
  # === git_URL?
  # ========================================================================= #
  def git_URL?
    @dataset.git_URL?
  end; alias git_url? git_URL? # === git_url?

  # ========================================================================= #
  # === consider_reporting_url2
  #
  # We will try to report the second URL, aka url2.
  #
  # As of December 2017, if url2 does not exist BUT a homepage
  # url exists, we will use that one instead.
  # ========================================================================= #
  def consider_reporting_url2
    if url2? # ← Prevent nil entries here.
      if !url2?.empty?
        if be_verbose?
          result = ''.dup
          if @show_leading_url_entry
            if use_colours?
              result << skyblue('url2: ')
            else
              result << 'url2: '
            end
          end
          if use_colours?
            result << yellowgreen(url2?.to_s)
          else
            result << url2?.to_s
          end
          e result.strip
          use_this_as_xorg_buffer(url2?)
        end
      end
    end
  end

  # ========================================================================= #
  # === tell_the_user_what_we_seek
  #
  # This method notifies the user for which particular program this
  # class will try to obtain the associated homepage.
  # ========================================================================= #
  def tell_the_user_what_we_seek(
      be_verbose = be_verbose?
    )
    if be_verbose
      opne "#{rev}Seeking URLs to `#{sfancy(search_for?)}#{rev}` next."
    end
  end

  # ========================================================================= #
  # === determine_dataset
  #
  # This method will also set the @dataset variable.
  # ========================================================================= #
  def determine_dataset(
      search_for = search_for?
    )
    if RBT.does_include?(search_for) # In this case, we can search for it.
      @dataset = action(:SanitizeCookbook, search_for) { :fast }
    else # Else it was not included.
      opne "The search_term `#{sfancy(search_for)}#{rev}` is not included."
      exit
    end
  end

  # ========================================================================= #
  # === set_search_for
  # ========================================================================= #
  def set_search_for(i)
    @internal_hash[:search_for] = i
  end

  # ========================================================================= #
  # === use_this_as_xorg_buffer
  #
  # This method has to account for "xsel" existing or not.
  # ========================================================================= #
  def use_this_as_xorg_buffer(i)
    set_xorg_buffer(i)
  end

  # ========================================================================= #
  # === feedback_urls
  # ========================================================================= #
  def feedback_urls
    if url1?.nil? or url1?.empty?
      e "url.rb: We did not find `#{search_for?}`." if be_verbose?
    else
      use_this_as_xorg_buffer(url1?)
      result = ''.dup
      if @show_leading_url_entry
        if be_verbose?
          if use_colours?
            result << skyblue('url1: ')
          else
            result << 'url1: '
          end
        else
          result << 'url1: '
        end
      end
      if use_colours?
        result << yellowgreen(url1?.to_s)
      else
        result << url1?.to_s
      end
      e result if be_verbose?
      if be_verbose?
        consider_reporting_url2 # Delegate to this method.
      end
      # ===================================================================== #
      # === Show the homepage as well
      #
      # This allows us to use the homepage entry.
      # ===================================================================== #
      if @show_the_homepage and
         homepage and
        !homepage.empty?
        if use_colours?
          e "#{skyblue('homepage: ')}"+
            yellowgreen(homepage?.to_s)
        else
          e "#homepage: #{homepage?}"
        end
      end
      if @show_the_git_URL_as_well_should_it_exist and
         git_URL? and
        !git_URL?.empty?
        if use_colours?
          e "#{skyblue('git:      ')}"+
            yellowgreen(git_URL?.to_s)
        else
          e "#git: #{git_URL?}"
        end
      end
    end
    e
  end

  # ========================================================================= #
  # === do_work_on_all_found_programs
  # ========================================================================= #
  def do_work_on_all_found_programs
    @find_these_programs.each {|entry|
      set_search_for(entry)
      tell_the_user_what_we_seek
      determine_dataset
      feedback_urls
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_work_on_all_found_programs
  end

end; end

# =========================================================================== #
# === RBT.return_url1_of_this_program
#
# This method will return the url1: entry of the given program at hand.
# =========================================================================== #
def self.return_url1_of_this_program(i = 'ruby')
  RBT::Cookbooks::Url.new(i) { :be_quiet }.url1?
end; self.instance_eval { alias return_url1_of return_url1_of_this_program } # === RBT.return_url1_of

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::Url.new(ARGV)
  # puts RBT.return_url1_of_this_program('ruby')
end # url ruby
    # url poppler