#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateSnapcraftFile
#
# This class can be used to create a snapcraft file. A snapcraft file
# describes how to create a "snap" package.
#
# If you want to look at already existing snapcraft.yml files,
# have a look here:
#
#   https://github.com/ubuntu/snap-tutorials-code/blob/master/snap-python-app/final/snap/snapcraft.yaml
#
# For the specification, see this file:
#
#   https://docs.snapcraft.io/snapcraft-yaml-reference/4276
#
# Usage example:
#
#   RBT::CreateSnapcraftFile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/generate_files/create_snapcraft_file.rb'
# RBT::CreateSnapcraftFile.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreateSnapcraftFile < RBT::Base # === RBT::CreateSnapcraftFile

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      program_name = nil,
      run_already  = true
    )
    reset
    set_program_name(
      program_name
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @snapcraft_file_content
    #
    # The following instance variable, a String, is the one that is used
    # to store all content that will be written into the snapcraft yaml
    # file. It will be used to store the file in the current directory.
    # ======================================================================= #
    @snapcraft_file_content = ''.dup
    # ======================================================================= #
    # === :store_into_this_file
    #
    # Denote into which file we will store that content.
    # ======================================================================= #
    @internal_hash[:store_into_this_file] = 'snapcraft.yaml' 
  end

  # ========================================================================= #
  # === set_store_into_this_file
  # ========================================================================= #
  def set_store_into_this_file(i)
    i = i.dup if i.frozen?
    i << '.yaml' unless i.end_with? '.yaml'
    @internal_hash[:store_into_this_file] = i
  end

  # ========================================================================= #
  # === set_program_name
  #
  # Denote onto which program we will work on.
  # ========================================================================= #
  def set_program_name(i = '')
    if i.nil?
      opne 'Please provide a valid name for a program to this class.'
      exit
    end
    i = [i].flatten.compact
    if i.first.is_a?(String) and i.first.include?(',')
      i = i.map {|entry|
        if entry.include? ','
          entry = entry.split(',')
        end
        entry
      }.flatten.compact
    end
    @program_name = i
  end

  # ========================================================================= #
  # === into?
  #
  # Where to store into - a file.
  # ========================================================================= #
  def into?
    @internal_hash[:store_into_this_file]
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  #
  # Add onto the main string through this method.
  # ========================================================================= #
  def add(
      i           = '',
      add_newline = true
    )
    @snapcraft_file_content << i
    @snapcraft_file_content << N if add_newline
  end

  # ========================================================================= #
  # === add_newline
  #
  # This method will simply add a newline to the snapcraft file.
  # ========================================================================= #
  def add_newline
    add
  end

  # ========================================================================= #
  # === what?
  # ========================================================================= #
  def what?
    @snapcraft_file_content
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end; alias programs? program_name? # === programs?

  # ========================================================================= #
  # === try_to_find_corresponding_program
  #
  # This method attempts to find a corresponding entry from the individual
  # yaml files.
  # ========================================================================= #
  def try_to_find_corresponding_program(
      program_name = program_name?
    )
    @dataset = action(:SanitizeCookbook, program_name) { :fast }
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @dataset.program_version?
  end

  # ========================================================================= #
  # === add_name
  # ========================================================================= #
  def add_name
    add('name: '+short_name?)
  end

  # ========================================================================= #
  # === short_name?
  #
  # Tap into @dataset to find out the short name of the program at
  # hand.
  # ========================================================================= #
  def short_name?
    @dataset.short_name?
  end

  # ========================================================================= #
  # === add_grade                                                (grade tag)
  #
  # The grade can be 'devel' or 'stable'. Only stable will be used
  # for release into candidate/stable channels.
  # ========================================================================= #
  def add_grade(
      use_this_grade = 'stable'
    )
    add("grade: #{use_this_grade}")
  end

  # ========================================================================= #
  # === add_confinement
  #
  # Confinement describes the level of confinement that can be applied
  # to the given app.
  #
  # Can be either 'devmode' or 'strict'.
  #
  # Unconfined applications, specified with 'devmode', can only be
  # released to the hidden "edge" channel.
  # ========================================================================= #
  def add_confinement(
      use_this_confinement = 'strict'
    )
    add('confinement: '+use_this_confinement)
  end

  # ========================================================================= #
  # === add_program_version                                     (version tag)
  # ========================================================================= #
  def add_program_version
    add("version: '#{program_version?}'")
  end

  # ========================================================================= #
  # === add_program_summary
  # ========================================================================= #
  def add_program_summary
    add('summary: |'+N+' '+short_desc?)
  end

  # ========================================================================= #
  # === add_program_licence
  # ========================================================================= #
  def add_program_licence
    if licence?
      add("license: #{licence?}")
    end
  end

  # ========================================================================= #
  # === short_desc?
  # ========================================================================= #
  def short_desc?
    _ = @dataset.short_desc?
    if _.size > 66
      _ = _[0 .. 60] # Truncate it in this case.
    end
    return _
  end

  # ========================================================================= #
  # === licence?
  # ========================================================================= #
  def licence?
    @dataset.licence?
  end

  # ========================================================================= #
  # === remote_url?
  # ========================================================================= #
  def remote_url?
    @dataset.remote_url?
  end

  # ========================================================================= #
  # === name_of_this_individual_program?
  # ========================================================================= #
  def name_of_this_individual_program?
    @name_of_this_individual_program
  end

  # ========================================================================= #
  # === set_name_of_this_individual_program
  # ========================================================================= #
  def set_name_of_this_individual_program(i)
    @name_of_this_individual_program = i
  end

  # ========================================================================= #
  # === add_parts                                                 (parts tag)
  #
  # This method denotes the necessary parts (components) of our program
  # at hand.
  # ========================================================================= #
  def add_parts
    heredoc_variable = <<-EOF
parts:
 #{name_of_this_individual_program?}:
  # See 'snapcraft plugins' for more information.
  plugin: #{plugin_type?}
  source: #{remote_url?}
EOF
    add(heredoc_variable)
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    @dataset.binaries?
  end

  # ========================================================================= #
  # === plugin_type?
  #
  # This is currently hardcoded.
  # ========================================================================= #
  def plugin_type?
    'autotools'
  end

  # ========================================================================= #
  # === add_the_proper_base
  #
  # This will add a String such as:
  #
  #   'base: core18'
  #
  # The base keyword declares which base snap to use with your
  # project. A base snap is a special kind of snap that provides
  # a run-time environment alongside a minimal set of libraries
  # that are common to most applications.
  #
  # For more documentation see:
  #
  #   https://snapcraft.io/docs/ruby-applications
  #
  # ========================================================================= #
  def add_the_proper_base
    add('base: core18')
    add_newline
  end

  # ========================================================================= #
  # === add_program_description                                    (desc tag)
  # ========================================================================= #
  def add_program_description
    description_to_use = "description: |#{N}".dup
    _ = @dataset.description?
    _.split(N).each {|line|
      description_to_use << ' '+line+N
    }
    add(description_to_use.strip.chomp)
  end

  # ========================================================================= #
  # === notify_the_user_that_we_will_create_a_new_snapcraft_file_next
  # ========================================================================= #
  def notify_the_user_that_we_will_create_a_new_snapcraft_file_next(
      _ = programs?
    )
    result = 'Creating a snapcraft file next for the program'.dup
    if _.is_a?(Array) and (_.size > 1)
      result << 's'
    end
    if _.is_a? Array
      _ = _.join(', ').strip
    end
    result << " called #{royalblue(_)}."
    opne result
  end

  # ========================================================================= #
  # === generate_the_snapcraft_file
  # ========================================================================= #
  def generate_the_snapcraft_file(
      into = into?
    )
    what = what?
    remove_file(into) if File.exist? into
    opne "Now saving the content into the file `#{sfile(into)}`."
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === add_apps                                                   (apps tag)
  #
  # This refers essentially to the binaries of a program.
  # ========================================================================= #
  def add_apps
    _ = "apps:\n".dup
    binaries?.each {|this_binary|
      _ << " #{this_binary}:\n"
      _ << "  command: bin/#{this_binary}\n"
    }
    add(_)
  end

  # ========================================================================= #
  # === add_everything
  # ========================================================================= #
  def add_everything
    add_name
    add_program_version
    add_program_summary
    add_program_licence
    add_program_description
    add_the_proper_base
    add_grade
    add_confinement
    add_newline
    add_apps
    add_parts
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    programs?.each {|this_program|
      reset
      try_to_find_corresponding_program(this_program) # <- Must come first as it sets the @dataset variable.
      set_name_of_this_individual_program(this_program)
      set_store_into_this_file(
        this_program+'-'+program_version?+'-snapcraft'
      )
      notify_the_user_that_we_will_create_a_new_snapcraft_file_next(
        this_program
      )
      add_everything
      generate_the_snapcraft_file
    }
  end

  # ========================================================================= #
  # === RBT::CreateSnapcraftFile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.create_snapcraft_file
#
# Simpler toplevel module-method for creating a snapcraft file.
# =========================================================================== #
def self.create_snapcraft_file(i)
  RBT::CreateSnapcraftFile.new(i)
end; self.instance_eval { alias snapcraft_file            create_snapcraft_file } # === RBT.snapcraft_file
     self.instance_eval { alias create_snapcraft_file_for create_snapcraft_file } # === RBT.create_snapcraft_file_for

end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateSnapcraftFile.new(ARGV)
end # createsnapcraftfile m4
    # createsnapcraftfile hello
    # createsnapcraftfile php,ruby,m4