#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Libtool
#
# Usage example:
#
#   RBT::Libtool.new
#
# =========================================================================== #
# require 'rbt/libtool/libtool.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Libtool < Base # === RBT::Libtool

  # =========================================================================== #
  # Next, require .rb files that belong to class RBT::Libtool.
  # =========================================================================== #
  require 'rbt/libtool/constants.rb'
  require 'rbt/libtool/help.rb'
  require 'rbt/libtool/reset.rb'
  require 'rbt/libtool/run.rb'
  require 'rbt/libtool/search_in_these_directories.rb'
  require 'rbt/libtool/remove_libtool_files/remove_libtool_files.rb'
  require 'rbt/libtool/class_methods/use_this_directory.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    case i
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      # In this case, do not run anything.
    else # This else clause here is the default clause.
      set_commandline_arguments(i)
      run if run_already
    end
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # The input to this should ideally be the whole commandline, as String.
  #
  # Reason being that we can apply a regex on the whole line.
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === Add search-directories
      #
      # This entry point will add new directories to the main list of
      # search directories for class RBT::Libtool.
      #
      # Invocation example:
      #
      #   rubylibtool --search-in-this-directory=/opt/lib
      #   rubylibtool --add=/home/Programs/ /usr/lib64/libglib-2.0.la
      #   rubylibtool --add=PROGRAMS /usr/lib64/libpng16.la
      #
      # ===================================================================== #
      when /(_|-)?(_|-)?search(_|-)?in(_|-)?this(_|-)?directory=(.+)/i, # === $6
           /(_|-)?(_|-)?add=(.+)/i                                      # === $3
        _ = $3.to_s.dup
        _ = $6.to_s.dup if $6
        append_this_directory_to_the_list_of_directories_to_search_for(
          _
        ) { :be_verbose }
      # ===================================================================== #
      # === Remove a faulty .la entry that was stored in a local .md file
      #
      # This entry point can be used to simply remove the last stored .la
      # file.
      #
      # Invocation example:
      #
      #   rubylibtool --remove-from-stored-file
      #
      # ===================================================================== #
      when /^-?-?remove(_|-)?from(_|-)?stored(_|-)?file$/i,
           /^-?-?remove(_|-)?from(_|-)?file$/i
        remove_the_last_faulty_libtool_entry_stored_in_a_file
      # ===================================================================== #
      # === Remove a single, faulty libtool entry from every .la file
      #
      # This method will modify offending .la files. The input argument
      # will be removed from these files.
      #
      # Trigger the following via:
      #
      #   rubylibtool --remove-this-entry=/usr/lib/libpng16.la
      #   rubylibtool --remove-this-entry=/Programs/Sqliteautoconf/3110000/lib/libsqlite3.la
      #
      # ===================================================================== #
      when /-?-?remove-?this-?entry=(.+)/,
           /-?-?remove=(.+)/,
           /-?-?purge=(.+)/,
           /-?-?purge (.+)/,
           /-?-?purge-?this-?entry=(.+)/,
           /-?-?purge-?this=(.+)/,
           /-?-?purge-?entry=(.+)/,
           /-?-?fix-?this-?entry=(.+)/
        remove_this_entry($1.to_s.dup)
        exit
      # ===================================================================== #
      # === Show the help section
      # ===================================================================== #
      when /-?-?help/i
        show_help
        exit
      # ===================================================================== #
      # === rubylibtool --search-in-which-directories?
      # ===================================================================== #
      when /(_|-)?(_|-)?search(_|-)?in(_|-)?which(_|-)?directories\??/
        notify_the_user_which_directories_will_be_searched
      # ===================================================================== #
      # === Designate another directory to use
      #
      # Specific usage example:
      #
      #   rubylibtool --from-this-directory=PWD --purge=/usr/lib/libstdc++.la
      #
      # ===================================================================== #
      when /--from-this-directory=(.+)/
        use_this_directory($1.to_s.dup)
      # ===================================================================== #
      # === rubylibtool --autocorrect
      #
      # The idea about autocorrecting the .la files is to convert
      # /Programs/ entries into /usr/ entries.
      # ===================================================================== #
      when /-?-?autofix/, /-?-?autocorrect/
        autofix_every_la_file
      # ===================================================================== #
      # === Show every .la file on this machine
      #
      # Invocation example:
      #   roebelibtool --show-every-la-file
      # ===================================================================== #
      when /-?-?show(_|-)?every(_|-)?la(_|-)?file/,
           /-?-?all$/,'ALL'
        show_every_la_file
      # ===================================================================== #
      # === roebe_libtool --stats
      # ===================================================================== #
      when /^-?-?stats$/i,
           /^-?-?statistics$/i,
           /^-?-?report$/i
        show_statistics
        exit
      else # else nothing was found.
        if i.end_with?('.la')
          remove_this_entry(i)
        end
        # opnn; e 'Unsure which action to perform. Pass in "--help" for the '\
        #         'available options.'
      end
    end
  end

  # ========================================================================= #
  # === log_these_files
  # ========================================================================= #
  def log_these_files
    @array_log_these_files
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i)
    @commandline_arguments = [i].flatten.compact
    sanitize_commandline_arguments
    menu(@commandline_arguments)
  end

  # ========================================================================= #
  # === sanitize_commandline_arguments
  # ========================================================================= #
  def sanitize_commandline_arguments
    @commandline_arguments.map! {|entry|
      if entry and entry.end_with?(':')
        entry.chop! # Trailing ':' seem pointless.
      end
      entry
    }
  end

  # ========================================================================= #
  # === ensure_that_we_have_found_la_files
  # ========================================================================= #
  def ensure_that_we_have_found_la_files
    if @array_all_libtool_files.empty?
      obtain_all_the_la_files
    end
  end

  # ========================================================================= #
  # === append_this_directory_to_the_list_of_directories_to_search_for
  # ========================================================================= #
  def append_this_directory_to_the_list_of_directories_to_search_for(i)
    case i
    when 'PROGRAMS'
      i = program_dir?
    end unless File.exist?(i)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    unless @search_in_these_directories.include? i
      if block_given?
        yielded = yield
        case yielded
        # =================================================================== #
        # === :be_verbose
        # =================================================================== #
        when :be_verbose
          opne "Adding the directory `#{sdir(i)}"\
               "` to the search path of this class."
        end
      end
      @search_in_these_directories << i
    end
  end

  # ========================================================================= #
  # === notify_the_user_which_directories_will_be_searched
  # ========================================================================= #
  def notify_the_user_which_directories_will_be_searched
    opne 'The following directories will be searched:'
    e "  #{sfancy(@search_in_these_directories.join(', '))}"
  end

  # ========================================================================= #
  # === search_in_these_directories?
  # ========================================================================= #
  def search_in_these_directories?
    @search_in_these_directories
  end; alias target_directories?          search_in_these_directories? # === target_directories?
       alias search_in_which_directories? search_in_these_directories? # === search_in_which_directories?

  # ========================================================================= #
  # === obtain_all_the_la_files
  # ========================================================================= #
  def obtain_all_the_la_files(
      i = @search_in_these_directories
    )
    i.each {|this_dir|
      these_la_files = Dir["#{this_dir}**/**.la"]
      @array_all_libtool_files << these_la_files
      @array_all_libtool_files.flatten!
    }
    return @array_all_libtool_files
  end; alias obtain_all_libtool_files obtain_all_the_la_files # === obtain_all_libtool_files
       alias every_la_file            obtain_all_the_la_files # === every_la_file

  # ========================================================================= #
  # === clear_array_log_these_files
  # ========================================================================= #
  def clear_array_log_these_files
    @array_log_these_files = []
  end

  # ========================================================================= #
  # === remove_this_entry
  #
  # Use this method to remove a specific entry within all .la files.
  # ========================================================================= #
  def remove_this_entry(i)
    clear_array_log_these_files # Reset the Array first.
    if i.end_with?(':') and !File.exist?(i)
      i.chop!
    end
    opne 'The following entry will be removed from every '\
         'encountered .la file on this computer:'
    e
    e "  #{seagreen(i)}"
    e
    if ::RBT::Libtool.use_this_directory?
      la_files = Dir["#{RBT::Libtool.use_this_directory?}**/**.la"]
    else
      la_files = obtain_all_libtool_files
    end
    notify_the_user_which_directories_will_be_searched
    # ======================================================================= #
    # Next, reject those who do not contain the given search term "i".
    # We will also log these files.
    # ======================================================================= #
    la_files.select! {|file|
      # ===================================================================== #
      # Must first check whether it is a symlink that exists.
      # ===================================================================== #
      if File.symlink?(file) and !File.exist?(File.readlink(file))
        symlink_target = File.readlink(file)
        opne 'No file exists at '+sfile(file)+rev+
             ' (a symlink pointing '\
             'at '+sfile(symlink_target)+')'
      end
      if File.exist? file
        dataset = File.read(file)
        dataset.include?(i)
      else
        opne "No file could be found at `#{sfile(file)}#{rev}`."
      end
    }
    if la_files.empty?
      opne "We did not find any .la file that includes "\
           "the entry `#{simp(i)}#{rev}`."
    else
      la_files.each {|file|
        opne "#{rev}Working on `#{sfile(file)}#{rev}` next."
        dataset = File.readlines(file)
        _ = ''.dup
        dataset.each {|line|
          if line.include?('dependency_libs') and line.include?(i)
            line.gsub!(/#{Regexp.quote(i)}/,'')
            @array_log_these_files << i
          end
          _ << line
        }
        # =================================================================== #
        # Write the new content into that .la file.
        # =================================================================== #
        write_what_into(_, file)
      }
      unless @array_log_these_files.empty?
        what = N+@array_log_these_files.join(N)+('-' * 40)+N
        # ===================================================================== #
        # Store the modified files in a file:
        # ===================================================================== #
        into = "#{rbt_log_directory?}libtool_modified_these_files.md"
        opne "#{rev}Keeping a backup of which libtool files were"
        opne "modified, at `#{sfile(into)}#{rev}`."
        append_what_into(what, into)
      end
    end
  end

  # ========================================================================= #
  # === remove_the_last_faulty_libtool_entry_stored_in_a_file
  #
  # Invocation example:
  #
  #   rubylibtool --remove-from-file
  #
  # ========================================================================= #
  def remove_the_last_faulty_libtool_entry_stored_in_a_file
    _ = "#{rbt_log_directory?}libtool/last_faulty_libtool_file.md"
    # ======================================================================= #
    # (1) Read the entry from the .md file, if it exists
    # ======================================================================= #
    if File.exist? _
      this_is_a_faulty_libtool_entry = File.read(_).strip
      # ===================================================================== #
      # (2) pass this into remove_this_entry()
      # ===================================================================== #
      remove_this_entry(this_is_a_faulty_libtool_entry)
    else
      opnn; no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === report_how_many_la_files_were_found
  # ========================================================================= #
  def report_how_many_la_files_were_found
    opne "There are a total of #{sfancy(n_la_files?.to_s)}"\
         " .la files on this system."
    opne 'The directories '+sdir(target_directories?.join(', '))+
         ' were searched.'
  end
  
  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === show_every_la_file
  #
  # Invocation example:
  #   roebelibtool --show-every-la-file
  # ========================================================================= #
  def show_every_la_file
    ensure_that_we_have_found_la_files
    opne 'Every .la file on this computer will be shown next'
    e
    @array_all_libtool_files.each_with_index {|this_libtool_file, index| index += 1
      index = (index.to_s+')').ljust(6)
      e "  #{sfancy(index)}#{sfile(this_libtool_file)}"
    }; e
  end

  # ========================================================================= #
  # === Libtool.autofix_every_la_file
  #
  # Use this method to fix every .la file on your system.
  #
  # Invocation example:
  #
  #   rubylibtool --autofix
  #
  # ========================================================================= #
  def self.autofix_every_la_file
    ensure_that_we_have_found_la_files
    if Libtool.faulty_la_files?.empty?
      # ===================================================================== #
      # In this case, fill up the Array first.
      # ===================================================================== #
      Libtool.fill_up_array_that_holds_faulty_la_files
    end
    Libtool.faulty_la_files?.each {|file|
      opne "Now working on the #{orange('.la')} file `#{sfile(file)}`."
      dataset = File.readlines(file)
      new_file_content = ''
      dataset.each {|line|
        if line.include?('dependency_libs') and 
           line.include? PROGRAMS_DIRECTORY.chop # .chop because we have a trailing '/'.
          splitted = line.split(' ')
          splitted.map! {|entry|
            if entry.include? PROGRAMS_DIRECTORY.chop
              entry = rds(entry)
              # ============================================================= #
              #   /usr//Programs/Libx11/1.6.3/lib/libX11.la
              # or
              #   /usr//usr//usr//Programs/Atspi2core/2.18.2/lib/libatspi.la
              # really strange
              # ============================================================= #
              regex = REGEX_TO_DISCOVER_PROGRAMS_ENTRIES_IN_DEPENDENCY_LIBS
              entry =~ regex
              match = $1.to_s.dup
              entry.sub!(/#{match}/, '/usr/')
            end
            entry
          }
          line = splitted.join(' ')+N
        end
        new_file_content << line
      }
      what = new_file_content
      into = file
      opne "Now storing into `#{sfile(into)}`."
      write_what_into(what, into)
    }
  end; self.instance_eval { alias fix_libtool_files autofix_every_la_file } # === RBT.fix_libtool_files

  # ========================================================================= #
  # === n_la_files?
  # ========================================================================= #
  def n_la_files?
    @array_all_libtool_files.size
  end

  # ========================================================================= #
  # === show_statistics
  # ========================================================================= #
  def show_statistics
    ensure_that_we_have_found_la_files
    report_how_many_la_files_were_found
  end

  # ========================================================================= #
  # === RBT::Libtool[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.fix_libtool_la_file_from_saved_file
# =========================================================================== #
def self.fix_libtool_la_file_from_saved_file
  RBT::Libtool.new.remove_the_last_faulty_libtool_entry_stored_in_a_file
end

# =========================================================================== #
# === RBT.libtool
#
# Easier module-level method towards Libtool.
# =========================================================================== #
def self.libtool(i = ARGV)
  RBT::Libtool.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Libtool.new(ARGV)
end # rbtlibtool --remove-this-entry=/usr/lib/libgobject-2.0.la
    # rbtlibtool --remove-this-entry=/usr/lib/libpng16.la