#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Libtool::RemoveLibtoolFiles
#
# This class can remove all .la files from a target directory.
#
# The reason why this is a standalone class is primarily because I
# needed to test (and use) its functionality from the commandline,
# and it was simpler to just create a class for that purpose.
#
# The first argument to it should be the path of the target directory
# that you wish to purge from libtool .la files.
# =========================================================================== #
# require 'rbt/libtool/remove_libtool_files/remove_libtool_files.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Libtool < Base

class RemoveLibtoolFiles < Base # === RBT::Libtool::RemoveLibtoolFiles

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      from_this_directory = return_pwd,
      run_already         = true
    )
    reset
    set_commandline_arguments(
      from_this_directory
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @all_libtool_files
    # ======================================================================= #
    @all_libtool_files = [] # ← Initialize them to an empty Array.
  end

  # ========================================================================= #
  # === handle_commandline_arguments_with_hyphens
  # ========================================================================= #
  def handle_commandline_arguments_with_hyphens
    _ = commandline_arguments?
    if _.any? {|entry| entry.start_with? '--' }
      selection = _.select {|entry| entry.start_with? '--' }
      menu(selection)
    end
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Currently this class has no special option.'
    e
    e 'The first argument to it should be the target directory. All'
    e '.la files residing in that directory will be removed then, so'
    e 'make sure to invoke this class ONLY if you really wish to get'
    e 'rid of all libtool .la files there.'
  end

  # ========================================================================= #
  # === determine_the_main_directory_automatically
  # ========================================================================= #
  def determine_the_main_directory_automatically
    set_use_this_directory(
      first_commandline_argument?
    )
  end

  # ========================================================================= #
  # === set_use_this_directory
  #
  # This setter defines which directory we will be using.
  # ========================================================================= #
  def set_use_this_directory(i)
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    i = return_pwd if i.empty? # <- Use a sane default here.
    i << '/' unless i.end_with? '/'
    @internal_hash[:use_this_directory] = i
  end

  # ========================================================================= #
  # === use_this_directory?
  # ========================================================================= #
  def use_this_directory?
    @internal_hash[:use_this_directory]
  end; alias prefix_to_the_libtool_files? use_this_directory? # === prefix_to_the_libtool_files?

  # ========================================================================= #
  # === determine_which_libtool_files_are_to_be_purged
  # ========================================================================= #
  def determine_which_libtool_files_are_to_be_purged
    @all_libtool_files = Dir["#{use_this_directory?}*.la"]
  end

  # ========================================================================= #
  # === try_to_purge_the_libtool_files
  # ========================================================================= #
  def try_to_purge_the_libtool_files
    # ======================================================================= #
    # If we have found at the least one .la file, delete it/them.
    # ======================================================================= #
    all_libtool_files = @all_libtool_files
    unless all_libtool_files.empty?
      result = powderblue(all_libtool_files.size.to_s)+"#{rev} .la file"
      result << 's' if all_libtool_files.size > 1
      if all_libtool_files.size > 1
        result << ' were'
      else
        result << ' was'
      end
      result << " found at `#{sdir(prefix_to_the_libtool_files?)}`."
      opne result
      opne "#{rev}The configure flag #{royalblue('delete_libtool_files')}#{rev}"\
           " has been set to true, so"
      opne "#{rev}these #{slateblue('.la files')}#{rev} will be removed next."
      remove_these_files(all_libtool_files)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    handle_commandline_arguments_with_hyphens
    determine_the_main_directory_automatically
    determine_which_libtool_files_are_to_be_purged
    try_to_purge_the_libtool_files
  end

  # ========================================================================= #
  # === RBT::RemoveLibtoolFiles.from
  # ========================================================================= #
  def self.from(this_directory)
    RBT::Libtool::RemoveLibtoolFiles.new(this_directory)
  end

end; end

# =========================================================================== #
# === RBT.remove_libtool_files_from
#
# Easier top-level API for removing libtool files from a certain
# directory (and subdirectories).
#
# The input-argument should be a locally existing directory which should
# contain .la files.
# =========================================================================== #
def self.remove_libtool_files_from(
    this_directory = ''
  )
  RBT::Libtool::RemoveLibtoolFiles.new(this_directory) # <- Just delegate to the class.
end; self.instance_eval { alias remove_libtool_files remove_libtool_files_from } # === RBT.remove_libtool_files 

end

if __FILE__ == $PROGRAM_NAME
  RBT.remove_libtool_files_from(ARGV)
end # removelibtoolfiles /home/Programs/Gcc/8.2.0/lib64/