#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredGirFiles
#
# The task of this class is simple:
#
#   Iterate over all .gir files and collect them, and then
#   store them into a yaml file.
#
# =========================================================================== #
# require 'rbt/registered/registered_gir_files.rb'
# RBT::Cookbooks::RegisteredGirFiles.new
# =========================================================================== #
module RBT

require 'rbt/registered/registered.rb'

# =========================================================================== #
# === FILE_REGISTERED_GIR_FILES
# =========================================================================== #
FILE_REGISTERED_GIR_FILES =
  "#{RBT.yaml_directory_for_the_registered_yaml_files?}gir_files.yml"

# =========================================================================== #
# === RBT.file_registered_gir_files
# =========================================================================== #
def self.file_registered_gir_files
  FILE_REGISTERED_GIR_FILES
end

module Cookbooks

class RegisteredGirFiles < RBT::Cookbooks::Registered

  # ========================================================================= #
  # === LOCATION_REGISTERED_GIR_FILES
  # ========================================================================= #
  LOCATION_REGISTERED_GIR_FILES = RBT.file_registered_gir_files

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash_to_be_stored
    # ======================================================================= #
    @hash_to_be_stored = {} # This is the hash that will be stored.
    # ======================================================================= #
    # === @query
    # ======================================================================= #
    @query = nil
    @this_program = nil
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opne 'Obtaining Information about available .gir files next.'
    opne 'This may take a while.'
    opne 'Please be patient.'
  end

  # ========================================================================= #
  # === location_gir_files?
  # ========================================================================= #
  def location_gir_files?
    RBT.file_registered_gir_files
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    available_programs?.each { |program_name|
      @query = action(:SanitizeCookbook, program_name) { :fast }
      data = @query.return_dataset
      gir_files = data['gir_files'] # This is an Array.
      if gir_files.nil?
        # Simply ignore these nil values then.
      else
        unless gir_files.empty?
          if SHOW_PROGRAM_NAME # <- This constant is defined in the file registered.rb
            e program_name+
              steelblue(
                ' ['+
                gir_files.join(',').strip+
                ']'
              )
          end
          gir_files.each {|entry|
            @hash_to_be_stored[entry] = program_name
          }
        end
      end
    }
    save_yaml_file
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # .gir files (in hash format). Right now, we need to manually
  # update this yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    what = YAML.dump(@hash_to_be_stored)
    into = location_gir_files?
    e sfancy('Storing yaml dataset in ')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if is_on_roebe? # Also store at home next.
      into = "#{ruby_src_dir_for_the_registered_yaml_files?}"\
             "#{File.basename(into)}"
      also_storing_into
      e
      e "  #{sfile(into)}"
      e
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    i = i.to_s
    opne "Searching for #{sfancy(i)} now:"
    hash = load_yaml(location_registered_gir_files?)
    if hash.keys.include? i
      opne "The .gir file called `#{sfancy(i)}` was found."
      it_is_part_of_the_package(hash[i])
    else
      opne p+' is '+swarn('NOT')+' included. We could not '\
           'find any program'
      opne 'including this program.'
    end
  end

end; end

# =========================================================================== #
# === RBT.registered_gir_files?
# =========================================================================== #
def self.registered_gir_files?
  RBT::Cookbooks::RegisteredGirFiles.new
end

# =========================================================================== #
# === RBT.is_this_a_registered_gir_file?
# =========================================================================== #
def self.is_this_a_registered_gir_file?(i = 'Gtk+-3.0.gir')
  dataset = YAML.load_file(file_registered_gir_files)
  dataset.has_key?(i)
end; self.instance_eval { alias is_this_gir_file_included? is_this_a_registered_gir_file? } # === RBT.is_this_gir_file_included?

# =========================================================================== #
# === RBT.return_the_associated_program_of_this_gir_file
#
# Invocation example:
#
#   RBT.return_the_associated_program_of_this_gir_file('xau.pc') # => "libxau"
#
# =========================================================================== #
def self.return_the_associated_program_of_this_gir_file(i)
  YAML.load_file(file_registered_gir_files)[i]
end; self.instance_eval { alias return_the_associated_program_for_this_gir_file return_the_associated_program_of_this_gir_file } # === RBT.return_the_associated_program_for_this_gir_file

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredGirFiles.new(:do_not_run_yet)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # reggirfiles