#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredHeaders
#
# The task of this class is simple:
#
#   Iterate over all headers and collect them, storing them into a
#   yaml file.
#
#   If however had an argument is passed, then we will try to find
#   out to which program this header file belongs to. (This is
#   already done by FindHeaders, so we delegate to it.)
# =========================================================================== #
# require 'rbt/registered/registered_headers.rb'
# RBT::Cookbooks::RegisteredHeaders.new
# RBT.all_headers?
# =========================================================================== #
module RBT

module Cookbooks

require 'rbt/project/project.rb'
require 'rbt/registered/registered.rb'
require 'rbt/cookbooks/find_headers.rb'

# =========================================================================== #
# === LOCATION_REGISTERED_HEADERS
#
# cat $COOKBOOKS/yaml/registered_headers.yml
# =========================================================================== #
LOCATION_REGISTERED_HEADERS = RBT.file_registered_headers

# =========================================================================== #
# === YAML_FILE_REGISTERED_HEADERS
# =========================================================================== #
if File.exist? LOCATION_REGISTERED_HEADERS
  begin
    YAML_FILE_REGISTERED_HEADERS = YAML.load_file(LOCATION_REGISTERED_HEADERS)
  rescue Psych::SyntaxError => error
    YAML_FILE_REGISTERED_HEADERS = {}
    puts "Problem for the file #{LOCATION_REGISTERED_HEADERS}"
    pp error
  end
else
  YAML_FILE_REGISTERED_HEADERS = {} # Otherwise, initialize as empty Hash.
end

# =========================================================================== #
# === ALL_HEADERS
# =========================================================================== #
ALL_HEADERS = YAML_FILE_REGISTERED_HEADERS

# =========================================================================== #
# === RBT::Cookbooks.all_headers?
# =========================================================================== #
def self.all_headers?
  ALL_HEADERS
end

# =========================================================================== #
# === all_headers?
# =========================================================================== #
def all_headers?
  Cookbooks.all_headers?
end

class RegisteredHeaders < RBT::Cookbooks::Registered # === RBT::Cookbooks::RegisteredHeaders

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  #
  # Whether to show the program name as well, as we iterate over our
  # dataset.
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash_to_be_stored
    # ======================================================================= #
    @hash_to_be_stored = {} # This is the hash that will be stored.
    # ======================================================================= #
    # === @try_to_use_the_expanded_cookbook_dataset_if_available
    #
    # This option should be significantly faster, so we will try to
    # make use of it. The drawback is that we may have outdated
    # data, if we did not update the expanded dataset for a long
    # time - but the speed advantage is a big reason in keeping
    # this set to true.
    # ======================================================================= #
    @try_to_use_the_expanded_cookbook_dataset_if_available = true
  end

  # ========================================================================= #
  # === return_headers_of_this_program
  # ========================================================================= #
  def return_headers_of_this_program(i)
    use_this_key = :headers
    if @try_to_use_the_expanded_cookbook_dataset_if_available and
       does_this_expanded_cookbook_file_exist_for_this_program?(i)
      i = load_dataset_from_this_expanded_cookbook(i)
      headers = i[use_this_key]
    else
      headers = action(:SanitizeCookbook, i) { :fast }.headers?
    end
    return headers
  end

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(i)
    i = i.to_s
    opne "Searching for #{simp(i)} now:"
    hash = load_yaml(RBT.file_registered_headers)
    if hash.has_key?(i)
      # This entry point means that we header was directly found.
      opne 'The header called `'+sfancy(i)+rev+'` was found.'
      it_is_part_of_the_package(hash[i])
    elsif hash.keys.map {|entry| File.basename(entry) }.include? i
      selection = hash.select {|key, value| key.include? i }
      this_program hash[selection.keys.first]
      opne 'Yes, is a sub-header and belongs to the '\
           'program `'+simp(this_program)+rev+'`.'
      return this_program
    else
      opne i+' is '+swarn('NOT')+' included. We could not find any program'
      opne 'including this header.'
    end
  end

  # ========================================================================= #
  # === RegisteredBinaries.location?
  # ========================================================================= #
  def self.location?
    opn; e LOCATION_REGISTERED_HEADERS
  end

  # ========================================================================= #
  # === iterate_over_the_available_programs
  # ========================================================================= #
  def iterate_over_the_available_programs
    available_programs?.each {|program_name|
      headers = return_headers_of_this_program(program_name)
      unless headers.empty?
        if SHOW_PROGRAM_NAME
          opne "#{rev}The program called #{sfancy(program_name)} #{rev}has "\
               "some header files (#{headers.size} of them)."
        end
        headers.each {|entry|
          if entry.include? ','
            entry.split(',').each {|inner_entry|
              @hash_to_be_stored[inner_entry.dup] = program_name.dup
            }
          else
            @hash_to_be_stored[entry] = program_name.dup
          end
        }
      end
    }
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opne 'Obtaining Information about available headers next.'
    opne 'This may take a while.'
    opne 'Please be patient.'
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # binaries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file(
      what = YAML.dump(@hash_to_be_stored),
      into = LOCATION_REGISTERED_HEADERS
    )
    e sfancy('Storing yaml dataset in')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if on_roebe? # ← Store on my home dir as well.
      into = "#{ruby_src_dir_for_the_registered_yaml_files?}#{File.basename(into)}"
      e sfancy('Storing the yaml dataset also in')
      e "  #{sfile(into)}"
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    iterate_over_the_available_programs
    save_yaml_file
  end

end; end

# =========================================================================== #
# === RBT.all_headers?
# =========================================================================== #
def self.all_headers?
  RBT::Cookbooks.all_headers?
end

# =========================================================================== #
# === RBT.is_this_header_included?
#
# Usage example:
#
#   RBT.is_this_header_included?('libusb.h') # => false
#
# =========================================================================== #
def self.is_this_header_included?(i)
  all_headers?.has_key?(i)
end

# =========================================================================== #
# === RBT.is_this_partial_header_included?
#
# This method is a bit more "forgiving" than the method
# RBT.is_this_header_included.
#
# Usage example:
#
#   RBT.is_this_partial_header_included?('libusb.h') # => true
#
# =========================================================================== #
def self.is_this_partial_header_included?(i)
  result = all_headers?.transform_keys {|entry|
    File.basename(entry)
  }
  result.has_key?(i)
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredHeaders.new(false)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # regheaders
    # regheaders SMlib.h