#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateShellScriptContainingTheseProgramsVersions
#
# Usage example:
#
#   RBT::CreateShellScriptContainingTheseProgramsVersions.new(ARGV)
#
# =========================================================================== #
# require 'rbt/shell/create_shell_script_containing_these_program_versions.rb'
# RBT::CreateShellScriptContainingTheseProgramsVersions.new('--everything')
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreateShellScriptContainingTheseProgramsVersions < Base # === RBT::CreateShellScriptContainingTheseProgramsVersions

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @_
    #
    # We will store into this String.
    # ======================================================================= #
    @_ = ''.dup
    # ======================================================================= #
    # === @array_work_on_these_programs
    # ======================================================================= #
    @array_work_on_these_programs = []
  end

  # ========================================================================= #
  # === return_all_programs
  # ========================================================================= #
  def return_all_programs
    available_programs?
  end

  # ========================================================================= #
  # === set_work_on_these_programs
  # ========================================================================= #
  def set_work_on_these_programs(i)
    i = [i].flatten.compact # Work on an Array past that point here.
    i.each {|entry|
      unless @array_work_on_these_programs.include? entry
        @array_work_on_these_programs << entry
      end
    }
  end; alias add_work_on_these_programs set_work_on_these_programs # === add_work_on_these_programs

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e 'No help options are presently documented.'
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === shell_script_program_versions --everything
      # ===================================================================== #
      when /^-?-?everything$/
        set_work_on_these_programs(return_all_programs)
      # ===================================================================== #
      # === --lfs
      # ===================================================================== #
      when /^-?-?lfs$/
        set_work_on_these_programs(return_all_programs)
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      else # else tag
        add_work_on_these_programs(i)
      end
    end
  end

  # ========================================================================= #
  # === determine_all_programs_versions
  # ========================================================================= #
  def determine_all_programs_versions(
      i = @array_work_on_these_programs
    )
    # ======================================================================= #
    # Obtain a handle to a file such as:
    #
    #   "/usr/lib/ruby/site_ruby/3.0.0/rbt/yaml/programs_version/available_programs_versions.md"
    #
    # ======================================================================= #
    _ = RBT.file_available_programs_versions
    if File.exist? _
      dataset = File.readlines(_).reject {|line| line.strip.empty? }
    end
    if is_on_roebe? and i.empty?
      opne rev+'Assuming that, as input, '+steelblue('--everything')+
           rev+' was meant.'
      i << return_all_programs
    end
    if i.empty?
      opne 'No program was passed. Thus, as no dataset has been given,'
      opne 'no file will be created. Please supply the programs that'
      opne 'you wish to generate shell-variables for (or use '+
           steelblue('--lfs')+')'
    else
      i.flatten.compact.each {|this_program|
        selection = dataset.select {|line|
          line.start_with? "#{this_program} "
        }
        first_line = selection.first
        # ===================================================================== #
        # Next, skip entries we can not find:
        # ===================================================================== #
        next unless first_line
        # program_version = RBT.swift_return_version_of_this_program(
        #                     this_program.to_s.delete('_').to_sym
        #                   )
        program_version = first_line.split(' ')[1].to_s.strip    
        # ===================================================================== #
        # === export MY_GAWKV="gawk-5.0.1"
        #
        # But also:
        #
        #   export MY_GAWK="${MY_GAWKV}.tar.xz"
        #
        # We need to remove '+' chars, though, as these are not valid for
        # the shell.
        # ===================================================================== #
        add(
          "export #{this_program.delete('+').upcase}V=\"#{this_program.downcase}-#{program_version}\"\n"
        )
        add(
          "export #{this_program.delete('+').upcase}=\"\$\{#{this_program.upcase}V\}.tar.xz\"\n"
        )
        add :newline
      }
      save_our_findings_into_a_file
    end
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(i)
    case i
    # ======================================================================= #
    # === :newline
    # ======================================================================= #
    when :newline
      i = "\n"
    end
    @_ << i
  end

  # ========================================================================= #
  # === save_our_findings_into_a_file
  # ========================================================================= #
  def save_our_findings_into_a_file
    into = 'shell_script_containing_the_program_versions.sh'
    e "#{rev}Now storing into `#{sfile(into)}#{rev}`."
    write_what_into(@_, into)
    if is_on_roebe?
      into = '/home/x/programming/ruby/src/rbt/lib/rbt/shell/'+
             File.basename(into)
      e "#{rev}Now storing into `#{sfile(into)}#{rev}`."
      write_what_into(@_, into)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    determine_all_programs_versions
  end

  # ========================================================================= #
  # === RBT::CreateShellScriptContainingTheseProgramsVersions[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.create_shell_script_containing_these_program_versions
# =========================================================================== #
def self.create_shell_script_containing_these_program_versions
  RBT::CreateShellScriptContainingTheseProgramsVersions.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateShellScriptContainingTheseProgramsVersions.new(ARGV)
end # createshellscriptcontainingtheseprogramversions htop ruby php
    # createshellscriptcontainingtheseprogramversions --lfs
    # createshellscriptcontainingtheseprogramversions --everything