#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/report_total_size_of_all_archives.rb'
# RBT.report_total_size_of_all_archives
# RBT.return_archive_type
# RBT.remove_file_extension
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/available_programs.rb'
  require 'rbt/configuration/configuration.rb'
  require 'rbt/constants/constants.rb'
  require 'rbt/colours/colours.rb'

  # ========================================================================= #
  # === RBT.is_an_archive?
  #
  # This method will return true if the given input is considered to be
  # an archive; and false otherwise.
  # ========================================================================= #
  def self.is_an_archive?(i)
    if i.is_a? Array
      i.map {|entry| is_an_archive?(entry) }
    else
      ARRAY_REGISTERED_ARCHIVES.any? {|entry|
        entry = entry.to_s unless entry.is_a?(String)
        i and i.end_with?(entry)
      }
    end
  end

  # ========================================================================= #
  # === RBT.return_total_size_of_all_archives
  # ========================================================================= #
  def self.return_total_size_of_all_archives(
      use_this_key = :program_path
    )
    total_size = 0
    available_programs = available_programs?
    available_programs.each {|this_program|
      possible_target = "#{RBT.directory_expanded_cookbooks?}"\
                        "#{this_program}.yml"
      if File.exist? possible_target
        dataset = YAML.load_file(possible_target)
        program_path = dataset[use_this_key]
      else
        # =================================================================== #
        # The expanded-cookbooks file does not exist, so use
        # Cookbooks::Cookbook instead.
        # =================================================================== #
        require 'rbt/requires/require_the_cookbook_class.rb'
        program_path = ::RBT::Cookbooks::SanitizeCookbook.new(this_program).program_path?
      end
      if program_path and File.exist?(program_path)
        total_size += File.size(program_path)
      end
    }
    return total_size
  end

  # ========================================================================= #
  # === RBT.report_total_size_of_all_archives
  #
  # This method can be used to report the total size of all locally
  # available archives.
  #
  # Invocation example from the commandline:
  #
  #   rbt --report_total_size_of_all_archives
  #
  # ========================================================================= #
  def self.report_total_size_of_all_archives
    total_size = RBT.return_total_size_of_all_archives
    n_kilobytes = total_size  / 1024.0
    n_megabytes = n_kilobytes / 1024.0
    n_gigabytes = n_megabytes / 1024.0
    # ======================================================================= #
    # Finally, report it here.
    # ======================================================================= #
    e "#{rev}The total file size of all #{sfancy(available_programs.size.to_s)}"\
      " #{rev}programs is #{total_size.to_s} bytes."
    e steelblue(
        "  (#{n_kilobytes.round(2)} KB; "
      )+
      tomato(
        "#{n_megabytes.round(2)} MB"
      )+
      "#{rev}; "+
      crimson(
        n_gigabytes.round(2).to_s+' GB'
      )+"#{rev})"
  end

  # ========================================================================= #
  # === RBT.return_archive_type
  #
  # This method will feedback the archive type in use, for the given
  # input at hand.
  #
  # It is used in class RBT::UpdateEntry, for example.
  #
  # The default will be, as can be seen from the default argument,
  # .tar.xz. Note that this assumption is not always correct, e. g.
  # if the user does not repackage a given archive.
  #
  # Note that the simple archives, such as .gem or .zip, could be
  # detected automatically, but entries such as .tar.gz are harder
  # to detect, hence why we will use a hardcoded case menu here.
  #
  # Usage example:
  #
  #   RBT.return_archive_type('htop-1.2.0.tar.xz') # => '.tar.xz'
  #
  # ========================================================================= #
  def self.return_archive_type(
      i = '.tar.xz'
    )
    if i.is_a? Array
      i = i.first
    end
    # ======================================================================= #
    # === .tar.xz
    # ======================================================================= #
    if    i.end_with? '.tar.xz'
      '.tar.xz'
    # ======================================================================= #
    # === .xz
    # ======================================================================= #
    elsif i.end_with? '.xz'
      '.tar.xz'
    # ======================================================================= #
    # === .tar.gz
    # ======================================================================= #
    elsif i.end_with? '.tar.gz'
      '.tar.gz'
    # ======================================================================= #
    # === .tar.bz2
    # ======================================================================= #
    elsif i.end_with? '.tar.bz2'
      '.tar.bz2'
    # ======================================================================= #
    # === .tar.lz
    # ======================================================================= #
    elsif i.end_with? '.tar.lz'
      '.tar.lz'
    # ======================================================================= #
    # === .gem
    # ======================================================================= #
    elsif i.end_with? '.gem'
      '.gem'
    # ======================================================================= #
    # === .zip
    # ======================================================================= #
    elsif i.end_with? '.zip'
      '.zip'
    # ======================================================================= #
    # === .jar
    # ======================================================================= #
    elsif i.end_with? '.jar'
      '.jar'
    # ======================================================================= #
    # === .phar
    # ======================================================================= #
    elsif i.end_with? '.phar'
      '.phar'
    # ======================================================================= #
    # === .js
    # ======================================================================= #
    elsif i.end_with? '.js'
      '.js'
    else # else try to make an "educated guess" - via File.extname().
      # ===================================================================== #
      # But this can also return faulty results. For example, if the input
      # is "27.01.2019" then it does not make any sense to return ".2019" as
      # an archive type, since that is not an archive type. This is the
      # reason why specifically numbers are excluded in the following
      # check.
      # ===================================================================== #
      extname = File.extname(i)
      if extname.delete('.') =~ /^\d+$/ # If only numbers.
        extname = ''.dup
      end
      extname
    end
  end; self.instance_eval { alias archive_type?              return_archive_type } # === RBT.archive_type?
       self.instance_eval { alias archive_type_of?           return_archive_type } # === RBT.archive_type_of?
       self.instance_eval { alias archive_type_of            return_archive_type } # === RBT.archive_type_of
       self.instance_eval { alias determine_the_archive_type return_archive_type } # === RBT.determine_the_archive_type

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  if ARGV.empty?
    e 'Please provide at the least one argument.'
  else
    e Colours.rev+'The archive type for this given input is:'
    e
    e '  '+sfancy(RBT.return_archive_type(ARGV))
    e
  end
  e
  RBT.report_total_size_of_all_archives
  e
end # atype libdrm-2.38.tar.bz2