#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/available_programs.rb'
# RBT.store_available_programs_into_a_yaml_file
# RBT.store_abbreviations
# =========================================================================== #
module RBT

  # ========================================================================= #
  # Require some rbt/toplevel_methods/ next.
  # ========================================================================= #
  require 'rbt/constants/constants.rb'
  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/roebe.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'

  # ========================================================================= #
  # === RBT.store_available_programs_into_a_yaml_file
  #
  # This method will, upon being invoked, store the available programs
  # in a yaml file. This is only useful on my home system, though -
  # regular users can just use the generated .yml file as-is.
  # ========================================================================= #
  def self.store_available_programs_into_a_yaml_file(
      array = ::RBT.array_available_programs
    )
    require 'yaml'
    dataset = YAML.dump(array)
    file = "#{RUBY_SRC_DIR_RBT_YAML}available_programs/available_programs.yml"
    e "#{rev}Storing into `#{file}#{rev}`."
    write_what_into(dataset, file)
  end; self.instance_eval { alias update_available_programs store_available_programs_into_a_yaml_file } # === RBT.update_available_programs

  # ========================================================================= #
  # === RBT.store_abbreviations
  #
  # This method will store the abbreviations to the programs in a file.
  #
  # To invoke this method on the commandline do:
  #
  #   rbt --generate-abbreviations
  #
  # ========================================================================= #
  def self.store_abbreviations(
      into = RBT.file_abbreviations
    )
    what = return_all_abbreviations_of_this_array
    what = YAML.dump(what)
    e "#{::Colours.rev}Storing into the file `#{::Colours.sfile(into)}#{::Colours.rev}`."
    write_what_into(what, into)
    if is_on_roebe?
      # ===================================================================== #
      # Also store this in my home directory, the respective yaml
      # directory.
      # ===================================================================== #
      into = RUBY_SRC_DIR_RBT_YAML+
             File.basename(into.delete_suffix('.yml'))+'/'+
             File.basename(into)
      e "#{::Colours.rev}Also storing into the file "\
        "`#{::Colours.sfile(into)}#{::Colours.rev}`."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === RBT.test_this_alias
  #
  # This method here is mostly used by a .rb file under the rbt/bin/
  # hierarchy.
  # ========================================================================= #
  def self.test_this_alias(i)
    require 'rbt/aliases/aliases.rb'
    e "#{rev}This program will test whether the given input is:"
    e
    e '  - a registered cookbook program       (1)'
    e '  - a registered cookbook alias         (2)'
    e '  - a registered cookbook abbreviation  (3)'
    e
    e 'in precisely this order.'
    e
    i = [i].flatten.compact
    i.each {|entry|
      if RBT.is_a_registered_cookbook_program?(entry)
        e 'Yes, '+RBT.sfancy(entry)+rev+' is a registered cookbook program.'
      else
        e 'No, '+RBT.sfancy(entry)+rev+' is NOT a registered cookbook program.'
      end
      if RBT.is_an_alias?(entry)
        e 'Yes, '+RBT.sfancy(entry)+rev+' is an alias.'
      else
        e 'No, '+RBT.sfancy(entry)+rev+' is NOT an alias.'
      end
      if RBT.is_an_abbreviation?(entry)
        e 'Yes, '+RBT.sfancy(entry)+rev+' is an abbreviation.'
      else
        e 'No, '+RBT.sfancy(entry)+rev+' is NOT an abbreviation.'
      end
    }
  end

  # ========================================================================= #
  # === RBT.is_an_abbreviation?
  #
  # Usage example:
  #
  #   RBT.is_an_abbreviation? 'libxshmfe' # => true
  #   RBT.is_an_abbreviation? 'x11'       # => true
  #   RBT.is_an_abbreviation? 'libx11'    # => false
  #
  # ========================================================================= #
  def self.is_an_abbreviation?(i)
    RBT.hash_aliases_to_the_available_programs?.has_key? i
  end; self.instance_eval { alias is_this_a_registered_abbreviation? is_an_abbreviation? } # === RBT.is_this_a_registered_abbreviation

  # ========================================================================= #
  # === RBT.hash_aliases_to_the_available_programs?
  #
  # Query method over the available aliases to the given programs. This
  # will include the abbreviations, too.
  #
  # Usage example:
  #
  #   RBT.hash_aliases_to_the_available_programs?['libxshmfenc'] # => "libxshmfence"
  #
  # ========================================================================= #
  def self.hash_aliases_to_the_available_programs?
    @hash_aliases_to_the_available_programs
  end; self.instance_eval { alias hash_aliases_to_the_available_programs hash_aliases_to_the_available_programs? } # === RBT.hash_aliases_to_the_available_programs
       self.instance_eval { alias abbreviations?                         hash_aliases_to_the_available_programs? } # === RBT.abbreviations?

  # ========================================================================= #
  # === RBT.is_this_program_included?
  #  
  # This method can be used to determine whether the given input argument
  # to this method is a registered program; aka whether this program is
  # included as part of the RBT project. A boolean will thus be returned,
  # true indicating inclusion, and false indicating that the input is
  # NOT a registered program.
  #
  # Any input that is assumed to be a program, and included (thus returning
  # true via this method), is additionally assumed to have an associated
  # .yml dataset that describes how this program can be installed.
  #
  # If the second argument is given, and is the Symbol :include_abbreviations
  # or :with_abbreviations, or any similar symbol, then the abbreviations
  # will also be considered.
  #
  # Usage examples:
  #
  #   RBT.is_this_program_included? 'gawk'
  #   RBT.is_this_program_included? 'awk'
  #   RBT.is_this_program_included? 'wwk', :include_abbreviations
  #
  # ========================================================================= #
  def self.is_this_program_included?(
      this_program,
      optional_argument = nil # <- Can be :include_abbreviations
    )
    result = RBT.determine_available_programs.include?(this_program)
    case optional_argument
    # ======================================================================= #
    # === :do_not_check_for_aliases
    # ======================================================================= #
    when :do_not_check_for_aliases
      # This clause will not check for aliases.
    # ======================================================================= #
    # === :include_abbreviations
    #
    # This is the default value - see the symbol :default.
    # ======================================================================= #
    when :include_abbreviations,
         :with_abbreviations,
         :with_aliases,
         :default
      # ===================================================================== #
      # Keep a reference to the Hash that stores the aliases to the
      # available programs at hand.
      # ===================================================================== #
      abbreviations = RBT.hash_aliases_to_the_available_programs
      if abbreviations.has_key? this_program
        result = true
      end
    end
    result
  end; self.instance_eval { alias does_include?                     is_this_program_included? } # === RBT.does_include?
       self.instance_eval { alias is_registered?                    is_this_program_included? } # === RBT.is_registered?
       self.instance_eval { alias is_available?                     is_this_program_included? } # === RBT.is_available?
       self.instance_eval { alias has_this_program?                 is_this_program_included? } # === RBT.has_this_program?
       self.instance_eval { alias has_this_program_registered?      is_this_program_included? } # === RBT.has_this_program_registered?
       self.instance_eval { alias is_this_program_registered?       is_this_program_included? } # === RBT.is_this_program_registered?
       self.instance_eval { alias includes_this_program?            is_this_program_included? } # === RBT.includes_this_program?
       self.instance_eval { alias program_is_included?              is_this_program_included? } # === RBT.program_is_included?
       self.instance_eval { alias is_a_registered_cookbook_program? is_this_program_included? } # === RBT.is_a_registered_cookbook_program?
       self.instance_eval { alias is_this_a_registered_program?     is_this_program_included? } # === RBT.is_this_a_registered_program?
       self.instance_eval { alias is_a_registered_program?          is_this_program_included? } # === RBT.is_a_registered_program?
       self.instance_eval { alias registered_program?               is_this_program_included? } # === RBT.registered_program?

  # ========================================================================= #
  # === RBT.return_all_abbreviations_of_this_array
  #
  # This method will calculate all abbreviations for a given program.
  #
  # It will NOT include the self-name, e. g. to avoid redundant
  # 'php' => 'php' entries.
  #
  # For lib-entries, such as "libX11", entries such as "X11" will also
  # work in principle, but this conflicts with e. g. "x11perf". So we
  # have to keep this in mind.
  #
  # To store these abbreviations from the commandline, issue the
  # following instruction:
  #
  #   rbt --store
  #
  # ========================================================================= # 
  def self.return_all_abbreviations_of_this_array(
      i = determine_available_programs
    )
    require 'rbt/requires/require_raw_yaml.rb'
    require 'abbrev'
    # ======================================================================= #
    # The next constant refers to a location such as:
    #
    #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/abbrevations_to_the_programs.yml
    #
    # ======================================================================= #
    file_abbreviations = RBT.file_abbreviations
    if File.exist?(file_abbreviations) and !File.empty?(file_abbreviations)
      # ===================================================================== #
      # If the abbreviations do exist, we will load up this file.
      # ===================================================================== #
      result = YAML.load_file(file_abbreviations)
    else
      # ===================================================================== #
      # Else we will have to re-create the file, or rather the content
      # of said file next.
      # ===================================================================== #
      result = {} # <- We will store into this Hash.
      # ===================================================================== #
      # Next, iterate over the registered programs:
      # ===================================================================== #
      i.each {|entry|
        # =================================================================== #
        # Obtain all abbreviations for this specific entry.
        # =================================================================== #
        abbreviations = Abbrev.abbrev([entry])
        # =================================================================== #
        # Next, if entry starts with 'lib', we will enable more aliases
        # to this name.
        # =================================================================== #
        if entry.start_with? 'lib'
          suffix = entry[3 .. -1]
          more_abbreviations = {}
          loop {
            more_abbreviations[suffix] = entry
            suffix.chop! # Chop off the last character, then continue.
            break if suffix.size < 1
          }
          more_abbreviations.update(abbreviations)
          abbreviations = more_abbreviations
        end
        # =================================================================== #
        # Now that the abbreviations have been entered, we will again
        # overwrite the result-hash with the core programs.
        # =================================================================== #
        abbreviations.reject! {|key, value|
          # ================================================================= #
          # Two conditions are possible here:
          #
          #   key == value means that this is a self-alias
          #     or
          #   it has already been defined
          #
          # ================================================================= #
          (key == value) or i.include?(key)
        }
        result.update(abbreviations)
      }
    end
    if File.exist? FILE_COOKBOOK_ALIASES
      # ===================================================================== #
      # If the file "cookbook_aliases.yml" exists, use that file to merge
      # it into the above abbreviations-hash.
      # ===================================================================== #
      result.update(
        YAML.load_file(FILE_COOKBOOK_ALIASES)
      ) if result
    end
    # ======================================================================= #
    # Merge in the file "windows_cookbook.yml".
    # ======================================================================= #
    if File.exist?(FILE_WINDOWS_COOKBOOK) and
       File.exist?(FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK) and
       File.read(FILE_TRY_TO_USE_THE_FILE_WINDOWS_COOKBOOK).strip.start_with?("t")
      array = YAML.load_file(FILE_WINDOWS_COOKBOOK).keys
      array = Abbrev.abbrev(array)
      result.update(array)
    end
    result
  end

  # ========================================================================= #
  # === RBT.feedback_available_programs
  #
  # This method will report to the user, in a more verbose manner, which
  # specific programs are registered in the RBT-project.
  # ========================================================================= #
  def self.feedback_available_programs
    result = ''.dup # Build up the return String.
    opn; e "Feedbacking all available programs next,"\
           " #{available_programs.size} of them."
    available_programs?.each_with_index {|entry, index|
      index += 1
      result << "(#{simp(index.to_s.rjust(3))}) #{sfancy(entry)}#{N}"
    }
    e result
  end; self.instance_eval { alias feedback_help feedback_available_programs } # === RBT.feedback_available_programs

  # ========================================================================= #
  # === RBT.determine_available_programs
  #
  # This method will determine how many .yml files exist in the
  # cookbook directory at hand.
  #
  # Always keep the result (alphabetically) sorted.
  #
  # Note that if we use a sqlite database then we could use this
  # instead.
  # ========================================================================= #
  def self.determine_available_programs(
      load_which_dataset = :default
    )
    case load_which_dataset
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :raw_cookbook_directory
      target = "#{RBT.cookbook_directory?}*.yml"
      Dir[target].map {|entry|
        File.basename(entry.sub(/\.yml$/,''))
      }.sort
    end
  end

  # ========================================================================= #
  # === @hash_aliases_to_the_available_programs
  #
  # Initialize this hash at once - we may re-use it at a later time.
  # ========================================================================= #
  @hash_aliases_to_the_available_programs = 
    return_all_abbreviations_of_this_array

  # ========================================================================= #
  # === RBT.available_programs?
  #
  # This method will return an Array of all available (registered or
  # otherwise made known) programs.
  #
  # By default it will return a list of programs from the given .yml files.
  #
  # The optional argument allows us to also use abbreviations, if the
  # argument is the Symbol called :include_abbreviations,
  # :include_aliases or :with_aliases. (These symbols are aliases
  # and do the same.)
  #
  # The reason why this is not enabled by default is primarily because
  # many other files are only interested in the directly registered
  # programs.
  #
  # The aliases will at first appear in a Hash, with key->value
  # pairs such as:
  #
  #   "gnomewe"=>"gnomeweather"
  #
  # Obviously we are here only interested in the keys, which is
  # why the code below will use the method .keys().
  #
  # Keep in mind that whenever you wish to compile one such program,
  # you need to check to which the program may point at.
  # ========================================================================= #
  def self.available_programs?(
      extra_arguments = nil
    )
    array = determine_available_programs
    case extra_arguments
    # ======================================================================= #
    # === :include_abbreviations
    # ======================================================================= #
    when :include_abbreviations,
         :include_aliases,
         :with_aliases,
         :with_abbreviations
      keys = @hash_aliases_to_the_available_programs.keys
      array << keys
    else
      if extra_arguments.is_a? Symbol
        e "Warning - not registered Symbol called :#{extra_arguments.to_s}"
      end
    end
    # ======================================================================= #
    # The resulting Array that will be returned from this method will
    # always be sorted and contain only unique entries:
    # ======================================================================= #
    array = array.flatten.uniq.sort
    array # Return the Array finally.
  end; self.instance_eval { alias available_programs              available_programs? } # === RBT.available_programs
       self.instance_eval { alias available_programs?             available_programs? } # === RBT.available_programs?
       self.instance_eval { alias cookbooks_include_this_program? available_programs? } # === RBT.cookbooks_include_this_program?
       self.instance_eval { alias available_cookbooks?            available_programs? } # === RBT.available_cookbooks?
       self.instance_eval { alias available_cookbooks             available_programs? } # === RBT.available_cookbooks
       self.instance_eval { alias get_cookbooks                   available_programs? } # === RBT.get_cookbooks
       self.instance_eval { alias include_this_program?           available_programs? } # === RBT.include_this_program?
       self.instance_eval { alias array_available_programs        available_programs? } # === RBT.array_available_programs
       self.instance_eval { alias all_programs?                   available_programs? } # === RBT.all_programs?
       self.instance_eval { alias programs?                       available_programs? } # === RBT.programs?

  # ========================================================================= #
  # === RBT.n_programs_available?
  #
  # Determine how many programs are registered in the RBT project.
  # ========================================================================= #
  def self.n_programs_available?
    RBT.available_programs?.size
  end; self.instance_eval { alias n_programs_available   n_programs_available? } # === RBT.n_programs_available
       self.instance_eval { alias n_programs?            n_programs_available? } # === RBT.n_programs?
       self.instance_eval { alias n_registered_programs? n_programs_available? } # === RBT.n_registered_programs?
       self.instance_eval { alias registered_programs?   n_programs_available? } # === RBT.registered_programs?
       self.instance_eval { alias n_programs_in_total?   n_programs_available? } # === RBT.n_programs_in_total?

  require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'
  # ========================================================================= #
  # === RBT.report_how_many_cookbook_entries_exist
  # ========================================================================= #
  def self.report_how_many_cookbook_entries_exist
    e "#{Colours.rev}There are a total of #{sfancy(n_programs_available?)} registered "\
      "programs, as part of the rbt-project."
  end; self.instance_eval { alias report_how_many_programs_are_registered report_how_many_cookbook_entries_exist } # === RBT.report_how_many_programs_are_registered

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  RBT.report_how_many_cookbook_entries_exist
  pp RBT.available_programs?
  pp RBT.available_programs?(:include_aliases)
  e
  e 'How many aliases are available, in total? '+
    RBT.available_programs?(:include_aliases).size.to_s
  e 'n programs are available: '+RBT.available_programs?.size.to_s
  e
  # RBT.store_available_programs_into_a_yaml_file
end # rbtnprogramsavailable