#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/does_include.rb'
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/all_libraries.rb'

  # ========================================================================= #
  # === RBT.is_included?
  #
  # This method can be used to query whether a given input is included
  # or whether it is not - that is, if the input may be a program that
  # is registered in the RBT project.
  #
  # By default we will also include aliases, but this can be disabled,
  # such as by using the Symbol :do_not_check_for_aliases as the
  # second argument to this method.
  #
  # Remember that all aliases in use are stored in a yaml file. If you
  # want to add new aliases or change the existing ones, you may have
  # to modify that yaml file.
  #
  # In principle, we could also restrict this method here to only-aliases
  # but I found that the API reads differently than this intent.
  #
  # Usage examples for this method:
  #
  #    RBT.is_included? 'abiword' # => true
  #    RBT.is_included? 'aclocal'
  # 
  # ========================================================================= #
  def self.is_included?(
      i,
      honour_aliases = true
    )
    if i
      require 'rbt/aliases/aliases.rb'
      i = i.join(' ').strip if i.is_a? Array
      # ===================================================================== #
      # We need a String past this point.
      # ===================================================================== #
      i = i.to_s unless i.is_a? String
      i.delete!(':') if i.include? ':' # We ignore ':' tokens.
      keys = available_programs?
      case honour_aliases
      # ===================================================================== #
      # === :default
      # ===================================================================== #
      when :default # Restore the default state in this case.
        honour_aliases = true
      # ===================================================================== #
      # === :do_not_honour_aliases
      # ===================================================================== #
      when :do_not_honour_aliases,
           :do_not_check_for_aliases,
           :do_not_use_aliases,
           :ignore_aliases,
           :user_disabled_this_setting_via_the_commandline
        honour_aliases = false
      end
      if honour_aliases
        aliases = RBT::Cookbooks::Aliases.new(i)
        all_aliases = aliases.dataset
        # =================================================================== #
        # Merge the keys together next, starting with the default programs.
        # ==================================================================== #
        keys << all_aliases.keys
      end
      keys.flatten!
      return keys.include?(i)
    else
      false
    end
  end; self.instance_eval { alias really_does_include? is_included? } # === RBT.really_does_include?
       self.instance_eval { alias does_include?        is_included? } # === RBT.does_include?
       self.instance_eval { alias program_is_included? is_included? } # === RBT.program_is_included?

  # ========================================================================= #
  # === RBT.does_include_this_library?
  #
  # This will return true if we have a binary with that name.
  #
  # Usage example:
  #
  #   RBT.does_include_this_library? 'libc.so'
  #   RBT.has_this_library? 'xzcmp'
  #   RBT.is_this_library_included? 'xzcmp'
  #
  # ========================================================================= #
  def self.does_include_this_library?(i)
    all_libraries?.has_key?(i)
  end; self.instance_eval { alias has_this_library?         does_include_this_library? } # === RBT.has_this_library?
       self.instance_eval { alias is_this_library_included? does_include_this_library? } # === RBT.is_this_library_included?

end