#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rb/toplevel_methods/ftp.rb'
# =========================================================================== #
module RBT

  require 'open-uri'
  require 'rbt/colours/colours.rb'
  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/time/time.rb'

  # ========================================================================= #
  # === GNOME_FTP_URL
  # ========================================================================= #
  GNOME_FTP_URL =
    'https://download.gnome.org/sources/?C=M&O=D'

  # ========================================================================= #
  # === RBT.publish_list_of_all_programs_version
  #
  # To invoke this method, do:
  #
  #   rbt --publish-list-of-all-programs-version
  #
  # ========================================================================= #
  def self.publish_list_of_all_programs_version(
      file_programs_version = RBT.file_programs_version # This is the file 'programs_version.yml'
    )
    _ = ''.dup
    opn; e 'The file '+sfile(file_programs_version)+' will next '\
           'be FTP-uploaded/published.'
    hash = YAML.load_file(file_programs_version)
    hash.each_pair {|name_of_the_program, program_version|
      _ << name_of_the_program.ljust(28)+program_version.to_s+N
    }
    what = _
    into = "#{RBT.store_into_this_directory}programs_version.txt"
    opn; e 'Storing into `'+sfile(into)+'`.'
    RBT::SaveFile.write_what_into(what, into)
    RBT.upload_list_of_all_programs_version(into)
    # ========================================================================= #
    # Also upload a .html variant.
    # ========================================================================= #
    RBT.upload_list_of_all_programs_version(into, :html)
  end

  # ========================================================================= #
  # === RBT.upload_list_of_all_programs_version
  #
  # This will use my FTP wrapper to upload this file.
  #
  # The local .txt file is usually kept or autogenerated here:
  #
  #   /Depot/Temp/CookbookDirectory/programs_version.txt
  #
  # ========================================================================= #
  def self.upload_list_of_all_programs_version(
      this_file = RBT.store_into_this_directory+'programs_version.txt',
      type      = :txt # type can be :txt or :html
    )
    if File.exist? this_file
      case type
      when :html
        begin
          require 'rbt/toplevel_methods/create_programs_version_html_file.rb'
        rescue LoadError; end
        RBT.create_programs_version_html_file(this_file)
        this_file.sub!(/\.txt$/,'')
      end
      opn; e 'Uploading the file '+sfile(this_file)+' next.'
      begin
        require 'ftp_paradise'
      rescue LoadError
      rescue SyntaxError
      end
      if Object.const_defined? :FtpParadise
        begin
          _ = FtpParadise::Connection.new(:dont_run_yet)
          _.set_be_verbose
          _.set_mode :ascii
          _.set_data(:shevy)
          _.do_login(:shevy)
          _.upload(this_file, :be_silent)
        # ===================================================================== #
        # We have to rescue the following error because FtpParadise may be
        # unavailable - but we may still want to upload the RBT project.
        # ===================================================================== #
        rescue NameError => error
          pp error
          pp error.class
          e 'Continuing nonetheless despite the above ^^^ error.'
        end
      end
    else
      opn; e "No file exists at `#{sfile(this_file)}`."
    end
  end

  # ========================================================================= #
  # === RBT.show_the_gnome_ftp_listing
  #
  # This method will simply show the remote FTP listing of gnome,
  # on the commandline.
  #
  # The reason for the existance of this method has been to allow
  # the user to quickly scan for new gnome-specific updates.
  #
  # The URL to gnome may look like this:
  #
  #   https://download.gnome.org/sources/?C=M&O=D
  #
  # ========================================================================= #
  def self.show_the_gnome_ftp_listing(
      show_n_results = 50 # Determine how many results to show to the user.
    )
    # ======================================================================= #
    # Notify the user which URL is used next:
    # ======================================================================= #
    e "#{rev}Trying to collect data from `#{GNOME_FTP_URL}` next."
    dataset = URI.open(GNOME_FTP_URL).read.split("\n")
    hash_programs = {}
    dataset.each {|line|
      # ===================================================================== #
      # === <td class="indexcolicon"><a href="easytag/"><img src="/icons2/folder.png"
      #
      # See the following URL:
      #
      #   https://rubular.com/r/d9CvpYvkqhRZwU
      #
      # ===================================================================== #
      if line.include? '<td class="link"><a href="'
        use_this_regex =
          /<td class="link"><a href="([a-zA-Z0-9-]+)\//
        name_of_the_program = line.scan(
          use_this_regex
        ).flatten.first
      end
      hash_programs[name_of_the_program]
      # ===================================================================== #
      # === <td class="date">
      #
      # Obtain the last time-modification next.
      #
      # See the following URL:
      #
      #   https://rubular.com/r/qhpkYJS8SUoi6g
      #
      # ===================================================================== #
      if line.include? '<td class="date">'
        last_update = line.scan(
          /<td class="date">(\d{4}-[a-zA-Z]{3,5}-\d{2} \d{2}:\d{2})<\/td>/
        ).flatten.first
        if name_of_the_program
          hash_programs[name_of_the_program] = Time.parse(last_update)
        end
      end
    }
    index = 0
    e
    today_is = ::Time.now # Grab hold of today's date.
    if hash_programs.empty?
      e 'No program was found.'
    else
      hash_programs.each_pair {|key, value| index += 1
        next if index > show_n_results
        padded_index = index.to_s.rjust(4)+') '
        if (value.year  == today_is.year)  and
           (value.month == today_is.month) and
           (value.day   == today_is.day)
          colourized_date = lightgreen(value.to_s)
        else
          colourized_date = forestgreen(value.to_s) 
        end
        e royalblue(padded_index)+
          steelblue(key.ljust(30))+
          ' → '+
          colourized_date
      }
      e
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  # The next line is mostly for standalone-testing of this file here.
  RBT.upload_list_of_all_programs_version
  RBT.show_the_gnome_ftp_listing # showthegnomeftplisting
end