#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/url.rb'
# RBT.find_url_for(ARGV)
# RBT.remote_url_for?(:php)
# RBT.return_program_name_and_program_version_from_this_github_url
# =========================================================================== #
module RBT

  require 'rbt/requires/require_the_raw_cookbook.rb'
  require 'rbt/toplevel_methods/return_location_to_this_programs_yaml_file.rb'
  require 'rbt/toplevel_methods/remove_file_extension.rb'

  # ========================================================================= #
  # === RBT.return_remote_url_from_this_input
  #
  # Give the below input, this method should return a String like:
  #
  #   https://ftp.gnu.org/gnu/glibc/glibc-2.28.tar.xz
  #
  # In other words, you have to supply the arguments.
  #
  # The method has to remain quite flexible. For instance,
  # it must work correctly when the first argument is
  # "https://github.com/unicode-org/icu/releases/download/release-69-1/icu4c-69_1.tgz"
  # and the second argument is a more complex variant such as
  # "icu4c-68.2.tar.xz".
  # ========================================================================= #
  def self.return_remote_url_from_this_input(
      remote_url = 'http://ftp.gnu.org/gnu/glibc/glibc-2.29.tar.xz',
      version    = '2.28'
    )
    # ======================================================================= #
    # First, get rid of any trailing archive parts for version,
    # such as "icu4c-68.2.tar.xz". Then we will have only
    # "icu4c-68.2" left, which is much better.
    # ======================================================================= #
    version = RBT.remove_archive_from_the_end(version)
    begin
      require 'program_information'
      if version.include? '-' # In this case assume something like "icu4c-68.2"
        version = ProgramInformation.return_version(version)
      end
      program_version = ProgramInformation.return_version(remote_url)
      result = remote_url.gsub( # Do a global sub since as of September 2022.
        program_version, version
      )
      return result
    rescue LoadError; end
  end; self.instance_eval { alias guess_the_url_from_this return_remote_url_from_this_input } # === RBT.guess_the_url_from_this

  # ========================================================================= #
  # === RBT.find_url_for
  #
  # Use this method to find a specific URL quickly.
  #
  # The method will return the first URL always, also called "url1".
  #
  # The method will also try to return url2 - and hence return an Array,
  # if possible. Keep this in mind that you may have an Array as
  # result from this method.
  #
  # Usage examples:
  #
  #   RBT.find_url_for()
  #   RBT.find_url_for(:htop) # => ["http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz", "http://hisham.hm/htop/index.php?page=downloads"]
  #   RBT.url_for? :patch # => ["https://ftp.gnu.org/pub/gnu/patch/patch-2.7.6.tar.xz", "http://ftp.gnu.org/gnu/patch/?C=M;O=D"]
  #
  # ========================================================================= #
  def self.find_url_for(
      i                = ARGV,
      return_as_string = false
    )
    case return_as_string
    when :return_as_string
      return_as_string = true
    end
    i = i.first if i.is_a? Array
    i = i.to_s
    # ======================================================================= #
    # Next, we will try to use the expanded dataset, if it exists. This
    # will be MUCH faster than instantiating a new ruby object, as is
    # done in the else clause. The drawback is that the expanded dataset
    # may not always have the most-up-to-date content.
    # ======================================================================= #
    expanded_file = RBT.return_location_to_this_programs_yaml_file(i)
    if File.exist? expanded_file
      if i.end_with? '.yml'
        i = i.dup if i.frozen?
        i.sub!(/\.yml$/,'')
      end
      dataset = YAML.load_file(expanded_file)
      if dataset.has_key?(i)
        dataset = dataset[i]
      end
      url1 = dataset['url1'] # Is a String.
      url2 = dataset['url2'] # Is a String.
    else
      # ===================================================================== #
      # Else we will instantiate a new, costly Cookbook instance.
      # ===================================================================== #
      _ = RBT.raw_cookbook(i)
      url1 = _.url1? # Return the URL here.
      url2 = _.url2?
    end
    url1 = url1.to_s.strip
    url2 = url2.to_s.strip
    if return_as_string
      url1.to_s
    else
      return [url1, url2]
    end
  end; self.instance_eval { alias url1            find_url_for } # === RBT.url1
       self.instance_eval { alias location_of?    find_url_for } # === RBT.location_of?
       self.instance_eval { alias url_for?        find_url_for } # === RBT.url_for?
       self.instance_eval { alias remote_url_for? find_url_for } # === RBT.remote_url_for?

  # ========================================================================= #
  # === RBT.does_this_remote_url_exist?
  #
  # This method will make use of class Wget to query whether a remote
  # URL exists.
  #
  # To test this method, try:
  #
  #   i = 'https://github.com/linuxwacom/libwacom/releases/download/libwacom-1.6/libwacom-1.6.tar.bz2'
  #   RBT.does_this_remote_url_exist? i
  #
  # ========================================================================= #
  def self.does_this_remote_url_exist?(i)
    Wget.check_if_remote_file_exists(i)
  end

  # ========================================================================= #
  # === RBT.return_program_name_and_program_version_from_this_github_url
  #
  # This method will accept a String such as:
  #
  #   'https://github.com/commonmark/cmark/archive/0.29.0.tar.gz'
  #   'https://github.com/brndnmtthws/conky/archive/v1.11.5.tar.gz'
  #
  # It will then return something like:
  #
  #   'cmake-0.29.0'
  #   'conky-1.11.5'
  #
  # Thus, this is the program_name_and_program_version string from
  # a valid cookbook .yml file.
  #
  # Note that the method will NOT assume that the input to it
  # is valid - it only works on input-strings such as the one
  # specified above.
  # ========================================================================= #
  def self.return_program_name_and_program_version_from_this_github_url(
      i                            = 'https://github.com/commonmark/cmark/archive/0.29.0.tar.gz',
      use_this_as_the_program_name = nil
    )
    result = ''.dup
    program_version = File.basename(i.dup)
    program_version = remove_archive_from_the_end(
      program_version.delete('v') # This .delete due to 'v'.
    )
    if use_this_as_the_program_name.nil?
      use_this_as_the_program_name = i.split('/')[4]
    end
    result << "#{use_this_as_the_program_name}-#{program_version}"
    return result
  end

  # ========================================================================= #
  # === RBT.return_programs_from_this_url
  #
  # The task of this method is to return an Array of programs that can
  # be found on a remote URL.
  #
  # If we take the example below for KDE's plasma related code, then the
  # Array that will be returned will include the names of all the
  # programs that can be found at that particular remote URL.
  #
  # This then allows us to do various further checks, such as whether the
  # RBT project is tracking all these programs or not, and so forth.
  #
  # The first input argument to this method is assumed to be a remote URL,
  # as a String - so only pass an URL to the method here.
  #
  # Specific invocation example in ruby:
  #
  #   pp RBT.return_program_from_this_url('https://download.kde.org/Attic/plasma/5.16.5/'); ''
  #
  # Result will be an Array such as this one here:
  #
  #   ["powerdevil", "sddmkcm", "systemsettings", "usermanager", "xdgdesktopportalkde"]
  #
  # ========================================================================= #
  def self.return_programs_from_this_url(i)
    require 'open-uri'
    begin
      require 'program_information' unless Object.const_defined? :ProgramInformation
    rescue LoadError; end
    i = i.first if i.is_a? Array
    regex_to_use = /<tr><td valign="top">&nbsp;<\/td><td><a href="(.+?)">/ # See: http://rubular.com/r/fIcvzXfPi1
    dataset = URI.open(i).read
    scanned = dataset.scan(regex_to_use).flatten
    scanned.reject! {|entry| entry.end_with? '.sig' or entry.start_with? '/' }
    scanned.map! {|entry|
      entry.sub!(/\.xz$/,'')
      entry.sub!(/\.tar$/,'')
      program_name = ProgramInformation.return_program_name(entry)
      program_name
    }
    scanned.uniq!
    return scanned.sort
  end; self.instance_eval { alias return_program_from_this_url return_programs_from_this_url } # === RBT.return_program_from_this_url

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  if ARGV.empty?
    array = RBT.return_programs_from_this_url(
      'https://download.kde.org/Attic/plasma/5.16.5/'
    )
    # ======================================================================= #
    # The next line can be used if you want to batch-open the files locally
    # or batch-compile something.
    # ======================================================================= #
    # array.each {|entry| e 'rbt '+entry+' trad; sleep 2' }
    pp array
    # pp array.size
  else
    pp RBT.return_programs_from_this_url(ARGV); ''
  end
  RBT.return_program_name_and_program_version_from_this_github_url
  e RBT.return_remote_url_from_this_input
  e
  RBT.find_url_for(ARGV).each {|remote_url|
    e "  #{remote_url}"
  }
  e
end # rbtreturnprogramsfromthisurl https://download.kde.org/Attic/plasma/5.16.5/
# rbt_find_url_for htop