#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::UrlAction
#
# This class allows you to install something in an URL-like scheme such
# as "install/htop/2.0.1". This would install htop, version 2.0.1.
#
# Installation is not the only possibility though - you can also remove
# or query programs.
#
# Examples:
#
#   /install/htop/3.0.5
#   /remove/gaim/1.2
#   /rm/gaim/1.2
#   /rm/python/latest
#   /query/strigi/latest
#   /?
#
# As can be seen in these examples, we provide a URL action-mapper to some
# actions that can be handled. The 'latest' component is a synonym to refer
# to the latest available version in the cookbooks/recipes.
#
# Further instructions to configure can also be passed, such as:
#
#   /install/libksba/latest/--with-gnu-ld --with-pic
#
# Usage example in ruby:
#
#   RBT::UrlAction.new '/install/gaim/1.2'
#
# =========================================================================== #
# require 'rbt/url_action/url_action.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class UrlAction < Base # === RBT::UrlAction

  # ======================================================================= #
  # === DEFAULT_COLOUR
  # ======================================================================= #
  DEFAULT_COLOUR = 'green'

  # ======================================================================= #
  # === ARRAY_ALLOWED_ACTIONS
  # ======================================================================= #
  ARRAY_ALLOWED_ACTIONS = %i(
    install
    query
    remove
  )

  # ======================================================================= #
  # === RBT::UrlAction.allowed_actions?
  #
  # Query which actions are allowed for class RBT::UrlActin.
  # ======================================================================= #
  def self.allowed_actions?
    ARRAY_ALLOWED_ACTIONS
  end

  # ======================================================================= #
  # === EXTRACT_TO
  #
  # Where to extract to. Will default to /Depot/Temp/ if
  # that ENV variable was not set.
  # ======================================================================= #
  if ENV['EXTRACT_TO']
    EXTRACT_TO = ENV['EXTRACT_TO'].to_s.dup+'/'
  else
    EXTRACT_TO = '/home/Temp/' # Default for most users, I guess.
  end

  # ========================================================================= #
  # === @compile_mode
  # ========================================================================= #
  @compile_mode = nil
  
  # ========================================================================= #
  # === UrlAction.compile_mode?
  # ========================================================================= #
  def self.compile_mode?
    @compile_mode
  end

  # ========================================================================= #
  # === UrlAction.set_compile_mode
  #
  # We need to keep track whether we wish to use Easycompile or RBT.
  #
  # We do this by assigning to the @compile_mode variable. The three
  # valid settings are nil, :rbt and :easycompile.
  # ========================================================================= #
  def self.set_compile_mode(
      i = :rbt
    )
    @compile_mode = i
  end

  # ========================================================================= #
  # === initialize
  #
  # The input can also include a version-component of a given program,
  # such as "1.5.22".
  #
  # If the program version is omitted, we will default to the latest
  # version that is registered.
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @result
    # ======================================================================= #
    @result      = nil
    # ======================================================================= #
    # === @input
    # ======================================================================= #
    @input       = nil
    # ======================================================================= #
    # === :debug
    # ======================================================================= #
    enable_debug
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose
    # ======================================================================= #
    # === :use_colours
    # ======================================================================= #
    enable_colours # Whether to use colours or not.
    set_extract_to # Must initialize it once.
    RBT::UrlAction.set_compile_mode :rbt # Initialize it to this default.
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.join(' ').strip if i.is_a? Array
    i = i.to_s.dup # Must be a String.
    # ======================================================================= #
    # Eliminate leading '/' characters. We never need them really.
    # ======================================================================= #
    i[0,0] = '' if i.start_with? '/'
    if debug? and i and !i.nil? and !i.empty?
      opne "Now setting the input to `#{sfancy(i)}`."
    end
    @input = i
  end; alias set_action set_input # === set_action

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug? # Whether we will debug or whether we will not.
    @debug
  end

  # ========================================================================= #
  # === set_result
  # ========================================================================= #
  def set_result(i)
    @result = i
  end

  # ========================================================================= #
  # === report_unknown_action
  #
  # Presently, only these three actions are known:
  #
  #   - query
  #   - remove
  #   - install
  #
  # A few aliases exist to these actions, such as '_' and '?'.
  # ========================================================================= #
  def report_unknown_action(
      i = nil
    )
    i = i.to_s unless i.is_a? String # Need a String.
    unless i.empty?
      set_result 'The action called `'+i+'` is unknown.'
      e @result
    end
  end

  # ========================================================================= #
  # === extract_to?
  # ========================================================================= #
  def extract_to?
    @extract_to
  end

  # ========================================================================= #
  # === action_to_perform?
  # ========================================================================= #
  def action_to_perform?
    _ = input?
    if _.include? '/'
      _ = _.split('/').first
    end
    _ # Return it here.
  end

  # ========================================================================= #
  # === set_extract_to
  #
  # Defaults to /Depot/Temp for extracting.
  # ========================================================================= #
  def set_extract_to(i = EXTRACT_TO)
    i = EXTRACT_TO if i.nil?
    @extract_to = i
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === reset_result
  # ========================================================================= #
  def reset_result
    @result = nil
  end

  # ========================================================================= #
  # === query_package
  #
  # This method will return a listing of all versioned directories
  # found under the $PROGRAMS/Foo directory.
  # ========================================================================= #
  def query_package(this_package)
    if this_package.is_a? Array
      this_package.each {|package| query_package(package) }
    else
      opne "Querying package `#{sfancy(this_package)}` next."
      location = (programs_dir?+this_package.capitalize+'/').squeeze '/'
      if File.exist? location
        _ = Dir[location+'*'].select {|i| i =~ /\d+/ } # Select entries with a number.
        if _.empty? # Do nothing if it is empty.
        else
          reset_result
          @result << "#{N}Found these packages:"
          efancy @result
          _.each {|package|
            string = '  → '+package
            @result << string
            e string
          }
        end
      else
        @result = "  The location `#{location}` does not exist."
        opnwarn @result
      end
    end
  end

  # ========================================================================= #
  # === remove_this_program                                    (remove tag)
  #
  # Remove a package/program through this method.
  #
  # We need to be sure that a specific version is mentioned, too. If it is
  # not then we will remove all versions.
  # ========================================================================= #
  def remove_this_program(i)
    if i.is_a? Array
      i.each {|my_program|
        remove_this_program(my_program)
      }
    else
      i = programs_dir?+i.capitalize
      if File.exist? i
        opne "Removing `#{sdir(i)}` now."
        remove_directory(i) # Remove a directory here.
        possible_current_symlink = i.split('/')[0..-2].join('/')+'/Current'
        if File.symlink? possible_current_symlink
          File.delete(possible_current_symlink)
        end
      else
        @result = i+' not found. Can not remove.'
        opne @result
      end
    end
  end

  # ========================================================================= #
  # === run_everything                                  (action tag, act tag)
  #
  # This is the powerhorse of the project.
  # ========================================================================= #
  def run_everything
    _ = action_to_perform?.to_sym
    if RBT::UrlAction.allowed_actions?.include? _
      this_program = nil
      # ===================================================================== #
      # Ok, the action is allowed, thus we can continue.
      # ===================================================================== #
      which_action_to_perform = input?
      if which_action_to_perform.include? '/'
        splitted = which_action_to_perform.split('/')
        splitted.reject!(&:empty?) # Get rid of empty entries.
        which_action_to_perform = splitted.first # Grab the first entry.
        this_program = splitted[1]
      end
      case which_action_to_perform # case tag
      # ===================================================================== #
      # === query
      # ===================================================================== #
      when 'query','q','?' # ? is also treated as a query
        query_package(this_program)
      # ===================================================================== #
      # === remove
      # ===================================================================== #
      when 'remove','rm','entferne','lösche'
        remove_this_program(this_program)
      # ===================================================================== #
      # === install
      # ===================================================================== #
      when 'install','i','compile','_'
        install_this_program(this_program)
      end
    else # Else the action is not included.
      report_unknown_action _
    end
  end

  # ========================================================================= #
  # === easycompile
  #
  # Delegate to the easycompile project in order to compile something
  # from source.
  # ========================================================================= #
  def easycompile(
      i, program_version = nil
    )
    begin
      require 'easycompile' unless Object.const_defined? :Easycompile
    rescue LoadError; end
    # ======================================================================= #
    # We assume that the user wishes to use a yaml file here.
    # This is a typical operation for Easycompile.
    # ======================================================================= #
    i << '.yml' unless i.end_with? '.yml'
    if program_version
      Easycompile::Easycompile.new(i,
        extract_to:      extract_to?,
        program_version: program_version,
        prefix:          :appdir_layout
      )
    else
      Easycompile::Easycompile.new(i,
        extract_to: extract_to?,
        prefix:     :appdir_layout
      )
    end
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === install_this_program                       (install tag, compile tag)
  #
  # This will compile/install a given program.
  #
  # We also have to support installation of a specific version.
  # ========================================================================= #
  def install_this_program(i, program_version = nil)
    if i.is_a? Array
      i.each {|entry| install_this_program(entry) }
    else
      compile_mode = RBT::UrlAction.compile_mode?
      set_result('Now installing `'+sfancy(i)+'` via '+compile_mode.to_s+'.')
      opn; e result?
      case compile_mode
      when :rbt # rbt tag.
        require 'rbt/requires/require_the_installer_class.rb'
        _ = RBT::Action::Installer.new(i, :dont_run_yet)
        _.extract_to extract_to?
        _.run
      when :easycompile # Easycompile tag.
        # =================================================================== #
        # If the input includes a '-' then we assume that the user
        # wants to install a specific version.
        # =================================================================== #
        if i.include? '-'
          program_version = i.split('-')
          opn; e 'We will use version number `'+sfancy(program_version)+
                 '` if possible.'
          easycompile(i, program_version)
        else
          easycompile(i)
        end
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

  # ========================================================================= #
  # === RBT::UrlAction[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end

# =========================================================================== #
# === RBT.new_url_action
#
# A way to instantiate from the top-level module namespace.
# =========================================================================== #
def self.new_url_action(input = nil, run_already = true)
  ::RBT::UrlAction.new(input, run_already) # Delegate to class RBT::UrlAction here.
end; self.instance_eval { alias url_action new_url_action } # === RBT.url_action

end

if __FILE__ == $PROGRAM_NAME
  RBT::UrlAction.new(ARGV)
end # urlaction