#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ChangePrefix
#
# This class can change the prefix of a specific file. The primary use
# case is to modify entries such as:
#
#    "PREFIX = /usr/local"
#
# that can be found in a Makefile.
#
# Do note that the class has been designed to be a bit more flexible than
# that task alone, though. While the primary use case is still to modify
# the PREFIX variable in a Makefile, ultimately the class could be used
# to modify just about any line in any text file. It is recommended to
# keep its primary use case confined to PREFIX-like variables in general,
# so that we can use this for the RBT project.
#
# The primary use case was to be able to modify such hardcoded PREFIX
# entries, in particular for projects that only ship with a Makefile
# and no other script.
#
# Usage example:
#
#   RBT::ChangePrefix.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/change_prefix.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ChangePrefix < Base # === RBT::ChangePrefix

  # ========================================================================= #
  # === REGEX_FOR_UPPCASE_PREFIX
  # ========================================================================= #
  REGEX_FOR_UPPCASE_PREFIX = /^(PREFIX\s*\??=) ?(.+)/i

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # Designate which file is to be modified next:
    # ======================================================================= #
    set_modify_this_file(:default_file)
    # ======================================================================= #
    # === @we_have_already_modified_the_makefile
    # ======================================================================= #
    @we_have_already_modified_the_makefile = false
    # ======================================================================= #
    # === @search_for_this_pattern
    #
    # Designate which pattern we are trying to find in that file.
    #
    # Unfortunately there does not seem to be a general consensus in
    # what is allowed and what is not in a given Makefile. For example,
    # the program called 'mxk-1.10' makes use of exactly this line
    # "prefix  = /usr/local", whereas the program called 'libcli-1.9.8-4'
    # makes use of exactly this line "PREFIX = /usr/local". Thus, we
    # will have to check for these variants, which is why we will
    # ask rubular for help here.
    #
    # In June 2023 I discovered a Makefile that contains this dual-prefix:
    #
    #   prefix      ?= /usr/local
    #   PREFIX      ?= $(prefix)
    #
    # So it seems as if we also have to match against the "?" part there.
    #
    # See:
    #
    #   https://rubular.com/r/jUCLVDCCVRqZb4
    #
    # ======================================================================= #
    @search_for_this_pattern =
      REGEX_FOR_UPPCASE_PREFIX
    # ======================================================================= #
    # Infer the name and program version from return_pwd, via
    # ProgramInformation.
    # ======================================================================= #
    program_information = ProgramInformation.new(return_pwd)
    version = program_information.version?
    name    = program_information.program_name?
    @use_this_as_the_new_prefix = 
      rds(
        programs_dir?+
        name.capitalize+'/'+
        version+'/'
      )
  end

  # ========================================================================= #
  # === set_use_this_as_the_new_prefix
  #
  # This method can be used to designate another prefix that is to be
  # used - the one that will be written into the "Makefile".
  # ========================================================================= #
  def set_use_this_as_the_new_prefix(i)
    @use_this_as_the_new_prefix = i
  end

  # ========================================================================= #
  # === use_this_as_the_new_prefix?
  # ========================================================================= #
  def use_this_as_the_new_prefix?
    @use_this_as_the_new_prefix
  end; alias use_this_prefix? use_this_as_the_new_prefix? # === use_this_prefix?

  # ========================================================================= #
  # === modify_this_file?
  # ========================================================================= #
  def modify_this_file?
    @modify_this_file
  end; alias modify_which_file? modify_this_file? # === modify_which_file?
       alias of_this_makefile   modify_this_file? # === of_this_makefile
       alias main_file?         modify_this_file? # === main_file?

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Presently this class has no help option documented.'
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === change_prefix --use-the-working-directory-as-input
      # ===================================================================== #
      when /^-?-?use(_|-)?the(_|-)?working(_|-)?directory(_|-)?as(_|-)?input$/i
        _ = File.basename(return_pwd)
        program_name    = ProgramInformation.return_program_name(_)
        program_version = ProgramInformation.return_program_version(_)
        new_prefix = programs_dir?+
                     program_name.capitalize+'/'+
                     program_version+'/'
        set_use_this_as_the_new_prefix(new_prefix)
        # =================================================================== #
        # Must also designate a new Makefile.
        # =================================================================== #
        set_modify_this_file(return_pwd+'Makefile')
      # ===================================================================== #
      # === change_prefix --help
      # ===================================================================== #
      when /^-?-?help$/i,
           /^-?-?show(_|-)?help$/i
        show_help
      else
        if i.is_a?(String) and File.exist?(i)
          # ================================================================= #
          # Use this existing file in that case.
          # ================================================================= #
          set_main_file(i)
        else
          # ================================================================= #
          # Try to find a Makefile in the current location.
          # ================================================================= #
          if File.exist?('Makefile') and !File.exist?(@modify_this_file)
            set_main_file(
              File.absolute_path(return_pwd)+'Makefile'
            )
          end
        end
      end
    end
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    if i.is_a? Hash
      # ===================================================================== #
      # If a Hash is given then we will modify this Hash a bit before
      # continuing here.
      # ===================================================================== #
      if i.has_key? :use_this_makefile
        set_modify_this_file(
          i.delete(:use_this_makefile)
        )
      end
      # ===================================================================== #
      # Query for another prefix that is to be used next:
      # ===================================================================== #
      if i.has_key? :use_this_as_the_new_prefix
        new_value = i.delete(:use_this_as_the_new_prefix)
        set_use_this_as_the_new_prefix(new_value)
      end
    end
    i = [i].flatten.compact
    if i.empty?
      # ===================================================================== #
      # In that case we will attempt to use a Makefile in the current
      # directory.
      # ===================================================================== #
      i << 'Makefile'
    end
    @internal_hash[:commandline_arguments] = i
  end

  # ========================================================================= #
  # === set_modify_this_file
  #
  # This method can be used to designate where the target "Makefile"
  # is exactly.
  #
  # Do note that set_main_file() is an alias to this method here.
  # ========================================================================= #
  def set_modify_this_file(i = :default)
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :default_file
      i = '/Depot/jjj/libcli-1.9.8-4/Makefile' 
    end
    i = i.dup if i.frozen?
    unless i.include? '/'
      i = File.absolute_path(i)
    end
    i = rds(i)
    @modify_this_file = i
  end; alias set_main_file set_modify_this_file # === set_main_file

  # ========================================================================= #
  # === notify_the_user_that_a_match_was_found_in_this_file_using_that_pattern
  # ========================================================================= #
  def notify_the_user_that_a_match_was_found_in_this_file_using_that_pattern(
      _,
      pattern
    )
    opne "A match was found within the file #{sfile(_)},"
    opne "using the pattern #{steelblue(pattern)}."
  end

  # ========================================================================= #
  # === lowercase_prefix_pattern?                                (regex1 tag)
  #
  # This method will return the main regex, as-is, but with PREFIX
  # being downcased to prefix.
  # ========================================================================= #
  def lowercase_prefix_pattern?
    return /^(prefix\s*\??=) ?(.+)/i
  end; alias pattern? lowercase_prefix_pattern? # === pattern?
       alias regex1   lowercase_prefix_pattern? # === regex1

  # ========================================================================= #
  # === uppercase_prefix_pattern?                                (regex2 tag)
  # ========================================================================= #
  def uppercase_prefix_pattern?
    REGEX_FOR_UPPCASE_PREFIX
  end; alias regex2 uppercase_prefix_pattern? # === regex2

  # ========================================================================= #
  # === contine_with_this_regex_on_that_dataset
  # ========================================================================= #
  def contine_with_this_regex_on_that_dataset(
      use_this_regex,
      dataset_as_string
    )
    main_file = main_file?
    if dataset_as_string =~ use_this_regex
      # ===================================================================== #
      # First inform the user that we have, indeed, found a match.
      # ===================================================================== #
      notify_the_user_that_a_match_was_found_in_this_file_using_that_pattern(
        main_file, use_this_regex
      )
      result = ''.dup
      dataset_as_string.split("\n").each {|line|
        if line =~ use_this_regex
          use_that_for_the_inner_regex = /(prefix\s*\??=\s*)(.+)/ # See: https://rubular.com/r/SKxjzFXhjMctH4
          if line.start_with?('prefix ') and !@we_have_already_modified_the_makefile
            # ============================================================= #
            # Then use the downcased variant.
            # ============================================================= #
            line =~ use_that_for_the_inner_regex
            line = $1.to_s.dup+@use_this_as_the_new_prefix.to_s
            @we_have_already_modified_the_makefile = true
          elsif line.start_with?('PREFIX') and !@we_have_already_modified_the_makefile
            line =~ use_that_for_the_inner_regex
            line = $1.to_s.dup+@use_this_as_the_new_prefix.to_s
            @we_have_already_modified_the_makefile = true
          else
            # ============================================================= #
            # Else do nothing for now.
            # ============================================================= #
          end
          unless @we_have_already_modified_the_makefile
            opne "The new line will be: #{lightgreen(line)}"
          end
        end
        result << "#{line.rstrip}\n"
      }
      opne "Storing into the file `#{sfile(main_file)}`."
      write_what_into(result, main_file)
    else
      opne 'The pattern '+sfile(use_this_regex.to_s)+' was not found.'
    end
  end

  # ========================================================================= #
  # === do_work_on_this_file
  # ========================================================================= #
  def do_work_on_this_file(i)
    if i and File.exist?(i)
      dataset = default_readlines(i)
      # ===================================================================== #
      # Next we must determine which regex-pattern to use.
      #
      # In order to determine this, we run a simple .include?() check
      # first.
      # ===================================================================== #
      dataset_as_string = dataset.join
      if dataset_as_string.include?('prefix ')
        use_this_regex = regex1
        contine_with_this_regex_on_that_dataset(use_this_regex, dataset_as_string)
      elsif dataset_as_string.include?('PREFIX ')
        use_this_regex = regex2
        contine_with_this_regex_on_that_dataset(use_this_regex, dataset_as_string)
      else
        opne 'Neither the pattern for '+sfancy('prefix')+' nor for the pattern'
        opne sfancy('PREFIX')+' could be found in the file at `'+sfile(i)+'`.'
      end
    else
      opnn; no_file_exists_at(i)
    end
  end

  # ========================================================================= #
  # === perform_the_modification
  #
  # This is the method that will be used to modify the Makefile at hand.
  # ========================================================================= #
  def perform_the_modification
    do_work_on_this_file(main_file?)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    perform_the_modification
  end

  # ========================================================================= #
  # === RBT::ChangePrefix[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ChangePrefix.new(ARGV)
end # changeprefix /Depot/jjj/libcli-1.9.8-4/Makefile
    # changeprefix /Depot/Temp/rbt/bzip2-1.0.7/Makefile --use-the-working-directory-as-input
    # changeprefix /Depot/Temp/rbt/bzip2-1.0.7/Makefile --use-this-prefix=/Programs/Bzip2/1.0.7