#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForAvailableVersions
#
# This class, which is part of the RBT project, in the Cookbooks submodule,
# lists all available local versions of a given program.
#
# Since as of September 2011 we have a new policy, which means that we keep
# our array already ordered - the latest version comes first. That means,
# if you do "ry python v1" you will ALWAYS try to compile the latest
# version.
# =========================================================================== #
# require 'rbt/utility_scripts/check_for_available_versions.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CheckForAvailableVersions < Base # === Cookbooks::CheckForAvailableVersions

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'ruby' # Default seed.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      for_this_program = nil,
      run_already      = true,
      &block
    )
    reset
    set_seek_this_program(
      for_this_program
    )
    case run_already
    # === :dont_run_yet
    when :dont_run_yet, :do_not_run_yet
      run_already = false
    when :be_verbose
      be_verbose
      run_already = true
    end
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        be_verbose
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @seek_this_program
    # ======================================================================= #
    @seek_this_program = nil
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_quiet # by default the class is silent and less verbose.
    # ======================================================================= #
    # === @version
    # ======================================================================= #
    @version = nil # This variable is only set via a special method.
    # ======================================================================= #
    # === @results
    # ======================================================================= #
    @results = []
  end

  # ========================================================================= #
  # === seek?
  # ========================================================================= #
  def seek?
    @seek_this_program
  end; alias program? seek? # === program?
       alias input?   seek? # === input?

  # ========================================================================= #
  # === sort_results
  # ========================================================================= #
  def sort_results
    @results = @results.sort # Or we can use this: .sort_by { |key| key.scan(/\d+/).map {|x| x.to_i } }.reverse
  end

  # ========================================================================= #
  # === version?
  # ========================================================================= #
  def version?
    @version
  end; alias version            version? # === version
       alias available_versions version? # === available_version

  # ========================================================================= #
  # === set_version
  # ========================================================================= #
  def set_version(i)
    i = i.sub(/\.tar\.bz2$/,'').
          sub(/\.gz$/,'')
    @version = i
  end

  # ========================================================================= #
  # === cleanup
  #
  # We will get rid of two instance variables here.
  # ========================================================================= #
  def cleanup
    if self.instance_variable_defined? :@sanitize_cookbook_dataset
      remove_instance_variable(:@sanitize_cookbook_dataset) # Trying this.
    end
    if self.instance_variable_defined? :@data
      remove_instance_variable(:@data)
    end
  end

  # ========================================================================= #
  # === fetch_entry_at_this_position
  #
  # If the argument passed to this method, `i`, is called "first" then
  # we equal this to be the first entry listed in the array @results.
  #
  # If the argument passed is however called "last", then it will default
  # to the last entry of that array.
  #
  # We also have to ensure that input like "-3" is valid. -3 would
  # translate to "give me the version minus 3. "-1" would say "give
  # me the entry that comes last".
  # ========================================================================= #
  def fetch_entry_at_this_position(i)
    if @results.empty?
      obtain_all_entries_from_the_source_directory(seek?)
    end
    i = i.to_s.dup # Need to work on a copy here.
    case i # Handle the two keywords 'first' and 'last' here.
    when 'first','vnewest','newest'
      i = 1.to_s
    when 'last','vlatest','latest'
      i = @results.size.to_s
    end
    i[0,1] = '' if i.start_with? 'v' # Get rid of "v", should it exist.
    # ========================================================================= #
    # Handle negative input next.
    # ========================================================================= #
    if i.to_s.include? '-' # account for a negative number.
      i = (@results.size - i.to_i.abs + 1) # +1 since Jan 2012.
    end
    i = i.to_i
    i = 1 if i <= 0
    # ========================================================================= #
    # Hardcap to a proper size. @results must exist for this to work though.
    # ========================================================================= #
    i = @results.size if i > @results.size
    result = @results[i - 1]
    set_version(ProgramInformation.return_version(result))
    return result
  end; alias position_at fetch_entry_at_this_position # === position_at

  # ========================================================================= #
  # === set_seek_this_program
  #
  # This method simply sets an instance variable that will
  # designate our main program to use or search for.
  # ========================================================================= #
  def set_seek_this_program(i = DEFAULT_PROGRAM)
    i = i.join if i.is_a? Array
    i = DEFAULT_PROGRAM if i.nil?
    i = i.to_s.dup # Keep a copy.
    if i.include?('-') or i.include?('_')
      i = ProgramInformation.return_name(i)
    end
    i.chop! if i.end_with? '-' # Also get rid of trailing '-' chars.
    @seek_this_program = i
  end; alias set_program set_seek_this_program # === set_program()

  # ========================================================================= #
  # === data?
  # ========================================================================= #
  def results?
    @results # This should be changed perhaps.
  end; alias data?     results? # === data?
       alias programs? results? # === programs?

  # ========================================================================= #
  # === obtain_all_entries_from_the_source_directory
  # ========================================================================= #
  def obtain_all_entries_from_the_source_directory(
      seek_this_program = seek?
    )
    target_directory = "#{source_directory?}#{seek_this_program}/"
    @results = Dir[target_directory+'*']
    sort_results
    @results = @results.reverse # Keep it reverse-sorted.
    return @results
  end

  # ========================================================================= #
  # === try_to_find
  # ========================================================================= #
  def try_to_find(this)
    set_program(this)
    run
  end

  # ========================================================================= #
  # === feedback_matches
  #
  # This method is not needed if @be_silent is set to false.
  #
  # We will feedback the matches that we found (which were stored in
  # @results).
  # ========================================================================= #
  def feedback_matches
    if @results.empty?
      opne 'No locally available version has been found.'
    else
      e
      print rev
      cliner
      @results.each_with_index { |_, index|
        index += 1
        index  = '%02s' % (index) if @results.size > 9
        e "  #{rev}(#{simp(index.to_s)}#{rev}) #{sdir(_)}" 
      }
      cliner
      e
    end
  end; alias feedback_available_versions feedback_matches # === feedback_available_versions


  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_all_entries_from_the_source_directory
    cleanup
    feedback_matches if be_verbose? or debug?
  end; alias check_for_program run # === check_for_program()

end; end

# =========================================================================== #
# === RBT.check_for_available_versions
# =========================================================================== #
def self.check_for_available_versions(
    i           = ARGV,
    run_already = true,
    &block
  )
  RBT::Cookbooks::CheckForAvailableVersions.new(i, run_already, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  # ========================================================================= #
  # === run_some_tests
  # ========================================================================= #
  def run_some_tests
    e 'Some tests next:'
    e _.position_at '-1'
    e _.position_at '-2'
    e _.position_at '-3'
    e _.position_at 'v1'
    e _.position_at 'v2'
    e _.position_at '3'
    e _.position_at 'vlatest'
    e _.position_at 'vnewest' # Show the newest entry.
  end
  _ = RBT::Cookbooks::CheckForAvailableVersions.new(ARGV.first, :dont_run_yet)
  _.be_verbose # Be verbose.
  if ARGV[1]
    e _.fetch_entry_at_this_position(ARGV[1])
  else
    _.run
  end
end # cfav glib
    # cfav glib 3
    # cfav glib -3
    # cfav python
    # cfav glib
    # cfav gtk