#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileInTraditionalManner
#
# This class can be used to compile a specific program via the /usr/
# prefix.
#
# Usage example:
#
#   RBT::CompileInTraditionalManner.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/compile_in_traditional_manner.rb'
# RBT::CompileInTraditionalManner.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CompileInTraditionalManner < Base # === RBT::CompileInTraditionalManner

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    commandline_arguments?.each {|this_program|
      this_program = this_program.dup if this_program.frozen?
      this_program.delete!('-') if this_program.include?('-')
      # ===================================================================== #
      # First remove entries such as "/download" from URLs given like:
      #
      #   https://sourceforge.net/projects/fetchmail/files/branch_6.4/fetchmail-6.4.33.tar.xz/download
      #
      # ===================================================================== #
      if this_program.end_with?('/download')
        this_program.delete_suffix!('/download')
      end
      if this_program.start_with?('http') or
         this_program.start_with?('https') or
         this_program.start_with?('ftp')
        # =================================================================== #
        # Assume in this case that an URL was provided, such as in:
        #
        #   trad https://ftp.gnome.org/pub/gnome/sources/gedit-plugins/3.37/gedit-plugins-3.37.92.tar.xz
        #
        # We will, in this event, first run update_entry on this, before
        # shortening the variable called i.
        # =================================================================== #
        action(:update_entry, this_program) # ← Delegate to that method.
        require 'program_information'
        # =================================================================== #
        # Now simplify i:
        # =================================================================== #
        this_program = ProgramInformation.return_name(this_program)
      end
      # ===================================================================== #
      # Delegate towards class Installer next:
      # ===================================================================== #
      compile_object = action(:Installer, [this_program, :do_not_run_yet]) {{
        shall_we_check_whether_the_program_exists: :nope,
        prefix: '/usr/' # trad means --prefix=/usr/, after all.
      }}
      # ===================================================================== #
      # Check whether the input is an existing file - in this case we will
      # use it.
      # ===================================================================== #
      if this_program and File.exist?(this_program)
        program_name    = ::ProgramInformation[this_program].program_name?
        program_version = ::ProgramInformation[this_program].program_version?
        compile_object.set_program_name(program_name)
        compile_object.initialize_dataset
        compile_object.set_program_version(program_version)
        compile_object.set_program_name_and_program_version(:default)
        compile_object.set_archive_type(this_program)
        compile_object.set_base_dir return_pwd
        compile_object.set_program_full_name
        compile_object.rebuild_path
      end
      compile_object.determine_which_programs_to_compile
      compile_object.run
    }
  end

  # ========================================================================= #
  # === RBT::CompileInTraditionalManner.new[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.trad
# =========================================================================== #
def self.trad(i = ARGV, &block)
  RBT::CompileInTraditionalManner.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileInTraditionalManner.new(ARGV.first, ARGV[1..-1]) # First argument should be the name of the program.
end # trad php
    # trad https://ftp.gnome.org/pub/gnome/sources/gedit-plugins/3.37/gedit-plugins-3.37.92.tar.xz