#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CopyTheseArchives
#
# This convenience-class will copy source archives to the current working
# directory. Simply pass in the names of the programs that you wish to
# copy.
#
# I needed this functionality specifically in December 2019, to set up
# the musl c-library in a chrooted environment.
#
# Since as of February 2024 it is recommended to not use this class
# directly (even though you still can, of course), but to instead
# pass everything via RBT.actions(), as this is now the preferred way
# of handling things related to the use cases covered by RBT, e. g.
# installing software from source, and so forth.
#
# Usage example:
#
#   RBT::CopyTheseArchives.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/copy_these_archives.rb'
# RBT::CopyTheseArchives.new(ARGV)
# RBT.copy_these_archives(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CopyTheseArchives < Base # === RBT::CopyTheseArchives

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === copy_this_archive
  # ========================================================================= #
  def copy_this_archive(i)
    this_dir = return_pwd
    opne "#{rev}Now copying #{sfile(i)} #{rev}to the"
    opne "#{rev}current working directory (`#{sdir(this_dir)}#{rev}`)."
    copy_file(i, this_dir)
  end

  # ========================================================================= #
  # === do_work_on_these_files
  # ========================================================================= #
  def do_work_on_these_files(
      i = commandline_arguments?
    )
    i.each {|this_target|
      full_target_path = this_target.dup
      if RBT.respond_to? :swift_return_version_of_this_program
        version_of_the_program = return_version_of_this_program(this_target)
      else
        version_of_the_program = action(:SanitizeCookbook, this_target) { :fast }.program_version?
      end
      unless this_target.include? '/'
        unless File.file?(this_target)
          full_target_path = src_dir?+this_target
        end
      end
      # ===================================================================== #
      # this_target may now look like this:
      #
      #   /home/x/src/linux
      #
      # We will next have to obtain the most recent version of the program.
      # ===================================================================== #
      unless full_target_path.end_with? '/'
        full_target_path << '/'
      end
      # ===================================================================== #
      # Now we can attempt to find a locally existing file. We will grab the
      # first entry found, if the result is not empty.
      # ===================================================================== #
      use_this_target = full_target_path+this_target+'-'+version_of_the_program+'*'
      possible_files = Dir[use_this_target]
      unless possible_files.empty?
        copy_this_archive(possible_files.first)
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_work_on_these_files(commandline_arguments?)
  end

  # ========================================================================= #
  # === RBT::CopyTheseArchives[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.copy_these_archives
# =========================================================================== #
def self.copy_these_archives(i = ARGV)
  RBT::CopyTheseArchives.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.copy_these_archives(ARGV)
end # copythesearchives
    # rbtcopy binutils cloog gcc gmp isl linux mpc mpfr musl