#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FixPkgconfigFile
#
# The purpose of this class is to fix .pc files automatically.
#
# As input, provide an existing .pc file.
#
# See the help subsection for how to use the class.
#
# Usage example:
#
#   RBT::FixPkgconfigFile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/fix_pkgconfig_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class FixPkgconfigFile < Base # === RBT::FixPkgconfigFile

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === DEFAULT_PKGCONFIG_FILE
  # ========================================================================= #
  DEFAULT_PKGCONFIG_FILE = '/usr/lib/pkgconfig/freetype2.pc'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    if i.is_a? Hash
      if i.has_key? :use_this_as_the_new_version
        set_new_program_version(i.fetch(:use_this_as_the_new_version))
      end
      if i.has_key? :use_this_pkgconfig_file
        set_use_this_pkgconfig_file(i.fetch(:use_this_pkgconfig_file))
      end
    else
      set_pkgconfig_file(i)
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @use_this_as_program_version
    # ======================================================================= #
    @use_this_as_program_version = nil
    set_pkgconfig_file(DEFAULT_PKGCONFIG_FILE)
  end

  # ========================================================================= #
  # === set_use_this_as_program_version
  # ========================================================================= #
  def set_use_this_as_program_version(i)
    @use_this_as_program_version = i
  end; alias set_new_program_version set_use_this_as_program_version # === set_new_program_version

  # ========================================================================= #
  # === use_this_as_program_version?
  # ========================================================================= #
  def use_this_as_program_version?
    @use_this_as_program_version
  end

  # ========================================================================= #
  # === pkgconfig_file?
  # ========================================================================= #
  def pkgconfig_file?
    @pkgconfig_file
  end; alias input? pkgconfig_file? # === input?

  # ========================================================================= #
  # === return_guessed_program_name
  # ========================================================================= #
  def return_guessed_program_name(i)
    i = File.basename(i).
             delete_suffix('.pc').
             gsub(/\d/,'') # Also remove numbers.
    return i
  end

  # ========================================================================= #
  # === fix_incorrect_version_in_this_pkgconfig_file
  #
  # This method will fix an incorrect version found in the given
  # .pc file at hand.
  # ========================================================================= #
  def fix_incorrect_version_in_this_pkgconfig_file(
      i                           = pkgconfig_file?,
      use_this_as_program_version = use_this_as_program_version?
    )
    if File.exist? i
      if use_this_as_program_version
        program_version = use_this_as_program_version
      else
        # =================================================================== #
        # Tap into the Cookbooks project to get the correct program
        # version.
        # =================================================================== #
        program_name = return_guessed_program_name(i)
        program_name = action(:SanitizeCookbook, program_name) { :fast }
        program_version = program_name.program_version?
      end
      opne "#{rev}Now modifying the file #{sfile(i)} #{rev}with the "\
           "new program version"
      opne "  #{simp(program_version)}"
      regex_to_use = /Version: (\d+.\d+.\d+)/ # See: http://rubular.com/r/B06zYDqKtg
      dataset = File.read(i)
      dataset.gsub!(regex_to_use, "Version: #{program_version}")
      store_dataset(dataset, i)
    else
      opnn; no_file_exists_at(i)
    end
  end

  # ========================================================================= #
  # === set_pkgconfig_file
  # ========================================================================= #
  def set_pkgconfig_file(i = DEFAULT_PKGCONFIG_FILE)
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    unless File.exist? i
      i = DEFAULT_PKGCONFIG_FILE
    end if i.empty?
    @pkgconfig_file = i
  end; alias set_use_this_pkgconfig_file set_pkgconfig_file # === set_use_this_pkgconfig_file

  # ========================================================================= #
  # === store_dataset
  # ========================================================================= #
  def store_dataset(what, into_this_file)
    write_what_into(what, into_this_file)
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i = input?)
    case i
    when nil, :default, ''
      # ===================================================================== #
      # Default action, which is to fix incorrect versions in the
      # given pkgconfig file at hand.
      # ===================================================================== #
      fix_incorrect_version_in_this_pkgconfig_file(:freetype)
    else
      if File.exist? i
        fix_incorrect_version_in_this_pkgconfig_file(i)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::FixPkgconfigFile.new(ARGV)
end # fixpkgconfigfile