#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Gitty
#
# This class simply wraps over the "--gitty" commandline argument.
# =========================================================================== #
# require 'rbt/utility_scripts/gitty.rb'
# RBT::Gitty.new(ARGV)
# RBT.gitty(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Gitty < Base # === RBT::Gitty

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === notify_the_user_what_to_do_if_no_argument_was_supplied_to_this_class
  # ========================================================================= #
  def notify_the_user_what_to_do_if_no_argument_was_supplied_to_this_class
    opne 'Please supply an argument to this class (the program whose'
    opne "sources you want to checkout via #{steelblue('git')}#{rev})."
  end

  # ========================================================================= #
  # === let_class_installer_handle_this_program
  # ========================================================================= #
  def let_class_installer_handle_this_program(this_program)
    installer = action(:Installer, this_program) { :do_not_run_yet }
    installer.set_compile_this_program(this_program)
    installer.initialize_dataset_for(this_program)
    cd installer.base_dir? # This only works if the dataset has been set up properly before.
    # ======================================================================= #
    # Next, we will call the method download_the_latest_source_code_of_this_program()
    # which will download the latest source code of the given program at hand.
    # ======================================================================= #
    location = installer.download_the_latest_source_code_of_this_program
    opne "#{rev}The archive should now exist at: #{steelblue(location)}#{rev}"
  end

  # ========================================================================= #
  # === handle_this_remote_program
  # ========================================================================= #
  def handle_this_remote_program(this_program)
    opne "#{rev}Using the remote URL at #{steelblue(this_program)} #{rev}next."
    esystem "git clone #{this_program}"
    target = File.basename(this_program)
    if File.directory?(target)
      new_name = target+'-'+dd_mm_yyyy.to_s
      if File.directory? new_name
        opne 'Normally this class would create a new directory, at '+sdir(new_name)+'.'
        opne 'However had, an (old) directory with that name already exists, so'
        opne 'the class can not continue; the user has to decide what should'
        opne 'happen in this event.'
      else
        opne "Renaming the directory at #{sdir(target)}"\
             "#{rev} to #{steelblue(new_name)} #{rev}next."
        rename_directory(target, new_name) { :be_quiet }
        opne 'Done!'
        opne 'Last but not least, this directory will be turned into a'
        opne '.tar.xz archive.'
        require 'totarxz'
        to_tar_xz = ToTarXz.new(new_name)
        full_path = File.absolute_path(to_tar_xz.result?)
        if File.exist? full_path
          opne 'The archive can now be found at: '+sfile(full_path)+rev
        else
          opnn; no_file_exists_at(full_path)
        end
      end
    else
      opne 'No directory found at '+sdir(target)+rev+'.'
    end
  end

  # ========================================================================= #
  # === work_on_this_program
  # ========================================================================= #
  def work_on_this_program(
      this_program
    )
    # ======================================================================= #
    # First, delegate to class Installer:
    # ======================================================================= #
    if this_program.start_with?('http')
      handle_this_remote_program(this_program)
    else # else let class Installer handle the situation
      let_class_installer_handle_this_program(this_program)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    this_program = first_argument?
    if this_program
      work_on_this_program(this_program)
    else
      notify_the_user_what_to_do_if_no_argument_was_supplied_to_this_class
    end
  end

  # ========================================================================= #
  # === RBT::Gitty[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.gitty
# =========================================================================== #
def self.gitty(i = ARGV)
  RBT::Gitty.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Gitty.new(ARGV)
end # gitty libva
    # gitty https://github.com/kennylevinsen/seatd