#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemoveEmptyDirectories
#
# This class will remove empty directories. By default it will operate
# on the current working directory, which evidently allows the user to
# use this class to eliminate all empty directories from the current
# directory.
# =========================================================================== #
# require 'rbt/utility_scripts/remove_empty_directories.rb'
# RBT.remove_empty_directories
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class RemoveEmptyDirectories < Base # === RBT::RemoveEmptyDirectories

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_base_directory
  # ========================================================================= #
  def set_base_directory(i = return_pwd)
    case i
    # ======================================================================= #
    # === :current_working_directory
    # ======================================================================= #
    when :current_working_directory
      i = return_pwd
    end
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @base_directory = i
  end

  # ========================================================================= #
  # === obtain_all_directories
  # ========================================================================= #
  def obtain_all_directories
    # ======================================================================= #
    # @directories will hold the directories that will be removed.
    # They will, however had, only be removed when they are empty.
    # ======================================================================= #
    @directories = Dir["#{@base_directory}*"].select {|entry|
      File.directory? entry
    }.map {|entry| rds(entry) }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Invocation example for this method:
  #
  #   remove-empty-directories --help
  #
  # ========================================================================= #
  def show_help
    e
    e 'The argument given to this class will specify the target directory'
    e 'that is to be used. By default this will be the current working '\
      'directory.'
    e
    exit
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === remove-empty-directories --help
      # ===================================================================== #
      when /^-?-?help$/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === determine_base_directory
  # ========================================================================= #
  def determine_base_directory
    _ = first_argument?
    _ = return_pwd if _.nil?
    set_base_directory(_)
  end

  # ========================================================================= #
  # === remove_empty_directories
  #
  # This will remove empty directories.
  #
  # However had, we will not remove directories that start with a '/'
  # character and have only one '/' in total.
  # ========================================================================= #
  def remove_empty_directories(i = @directories)
    i.each {|entry|
      entry = rds(entry)
      # ===================================================================== #
      # Check whether the directory is empty or whether it is not:
      # ===================================================================== #
      if Dir.empty?(entry)
        shall_we_remove_this_directory = true
        msg_to_the_user = "The directory at `#{sdir(entry)}` is empty.".dup
        if entry.start_with?('/') and
           entry.count('/') == 1
          shall_we_remove_this_directory = false
        else
          msg_to_the_user << ' It will be removed next.'
        end
        opne msg_to_the_user
        if shall_we_remove_this_directory
          remove_directory(entry) # Only remove it if it is not something like '/run'.
        end
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    determine_base_directory
    obtain_all_directories
    remove_empty_directories
  end

end

# =========================================================================== #
# === RBT.remove_empty_directories
# =========================================================================== #
def self.remove_empty_directories(i = :current_working_directory)
  RBT::RemoveEmptyDirectories.new(i)
end
 
end

if __FILE__ == $PROGRAM_NAME 
  RBT::RemoveEmptyDirectories.new(ARGV)
end # rempty