#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemovePrograms
#
# The purpose of this class is to scan for stray programs in the
# $PROGRAMS directory.
# =========================================================================== #
# require 'rbt/utility_scripts/remove_programs/remove_programs.rb'
# RBT.remove_programs
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class RemovePrograms < Base

  attr_accessor :be_verbose

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      programs_dir           = programs_directory?,
      optional_be_verbose    = nil,
      clear_only_one_program = false
    )
    reset
    if optional_be_verbose
      set_be_verbose(optional_be_verbose)
    end
    set_programs_dir(programs_dir)
    if clear_only_one_program
      register_to_delete_directory(clear_only_one_program)
    else # This here is the default.
      scan_programs_dir # Get all entries from /Programs dir.
      report_n_dirs     # Report how many we have inside.
      enter_each_dir    # This enters each dir.
    end
    unless @array_destroy_these_directories.empty?
      action_remove_registered_directories
    end
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_destroy_these_directories
    # ======================================================================= #
    @array_destroy_these_directories = [] # These directories will be removed
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose
  end

  # ========================================================================= #
  # === set_programs_dir
  # ========================================================================= #
  def set_programs_dir(i)
    i = programs_dir? if i == :def
    @use_this_programs_dir = i # default
  end

  # ========================================================================= #
  # === register_to_delete_directory
  #
  # While running enter_each_dir we also collect info which dirs are 
  # to be deleted. We will use this info later and remove these dirs.
  # ========================================================================= #
  def register_to_delete_directory(
      i
    )
    unless @array_destroy_these_directories.include? i
      @array_destroy_these_directories << i
    end
  end

  # ========================================================================= #
  # === enter_each_dir
  #
  # Process each dir, then we register which directory to remove.
  # ========================================================================= #
  def enter_each_dir
    @directories_to_check.each { |the_dir|
      register_to_delete_directory(the_dir) if Dir.empty?(the_dir)
      change_directory(the_dir)
      efancy "Currently working on directory: #{sdir_return_pwd}"
      pointer_to_current = return_pwd+'Current'
      # Next, handle the Current symlinks.
      if File.symlink?(pointer_to_current) &&
        !File.exist?(File.readlink(pointer_to_current)) 
        e "  Deleting #{sdir_return_pwd}#{sfancy('Current')}"
        remove_file pointer_to_current
        register_to_delete_directory(return_pwd) # Register it here.
      end
      change_directory '..'
    }
  end

  # ========================================================================= #
  # === report_n_dirs
  # ========================================================================= #
  def report_n_dirs
    opne rev+'We have '+sfancy(@directories_to_check.size.to_s)+
         rev+' programs listed in '+sdir(@use_this_programs_dir)+rev+
         '.'
  end

  # ========================================================================= #
  # === scan_programs_dir
  #
  # This method retrieves all directoriess listed in our programs dir.
  # ========================================================================= #
  def scan_programs_dir
    @directories_to_check = get_dir_listing(@use_this_programs_dir, true)
  end

  # ========================================================================= #
  # === action_remove_registered_directories
  #
  # This method will delete the designated directories.
  # ========================================================================= #
  def action_remove_registered_directories
    eimp 'Last but not least we will delete some dirs now:'
    @array_destroy_these_directories.each { |dir|
      if be_verbose?
        opne "  Removing #{sdir(dir)}"
      end
      remove_directory(dir) unless dir == '/'
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    action_remove_registered_directories
  end

end

# =========================================================================== #
# === RBT.remove_programs
# =========================================================================== #
def self.remove_programs
  RBT::RemovePrograms.new
end

end 

if __FILE__ == $PROGRAM_NAME
  RBT::RemovePrograms.new
end # rcleaner