#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ShowDependenciesOf
#
# This class will simply show all dependencies of a given program.
#
# This is different to the class ShowDependenciesOn, which shows all 
# programs that depend on the given input-program.
#
# Usage examples:
#
#   sdeps_of ruby
#   sdeps_of libx11
#
# =========================================================================== #
# require 'rbt/cookbooks/show_dependencies_of.rb'
# RBT.show_dependencies_of(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ShowDependenciesOf < RBT::Base # === RBT::Cookbooks::ShowDependenciesOf

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      of_this_program = ARGV,
      run_already     = true
    )
    reset
    set_of_this_program(
      of_this_program
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_all_dependencies
    # ======================================================================= #
    @array_all_dependencies = []
  end

  # ========================================================================= #
  # === set_of_this_program
  # ========================================================================= #
  def set_of_this_program(of_this_program)
    if of_this_program.is_a? Array
      of_this_program = of_this_program.first
    end
    @of_this_program = of_this_program
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @of_this_program
  end

  # ========================================================================= #
  # === report_these_dependencies
  # ========================================================================= #
  def report_these_dependencies(deps)
    if deps.empty?
      opne "#{rev}No dependencies have been found "\
           "for `#{sfancy(program?.to_s)}#{rev}`."
    else
      opne rev+'The (direct) dependencies of `'+sfancy(program?.to_s)+rev+
           '` are:'
      e
      deps.each_with_index {|entry, index| index += 1
        index = index.to_s.rjust(deps.size.to_s.size)
        index = slateblue(index.to_s)
        e "#{rev}  (#{index.to_s}) #{steelblue(entry)}"
      }; e
    end
  end

  # ========================================================================= #
  # === sanitize_this_array
  # ========================================================================= #
  def sanitize_this_array(i)
    i.map {|entry|
      if entry.include? ' '
        entry = entry.split(' ').first.to_s.strip
      end
      entry
    }
  end

  # ========================================================================= #
  # === sort_the_big_array
  # ========================================================================= #
  def sort_the_big_array
    @array_all_dependencies.sort!
  end

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report(
      i = @array_all_dependencies
    )
    e "#{rev}All gathered dependencies of `#{sfancy(program?.to_s)}#{rev}` are:"
    e
    i.each_with_index {|entry, index| index += 1
      padded_index = royalblue(
        ("(#{index})").rjust(5)
      )
      e "#{rev} #{padded_index} #{steelblue(entry)}"
    }
    e
  end

  # ========================================================================= #
  # === gather_all_unique_dependencies_for
  # ========================================================================= #
  def gather_all_unique_dependencies_for(
      i, array_all_dependencies = @array_all_dependencies
    )
    i = sanitize_this_array(i)
    array_all_dependencies << i
    # ======================================================================= #
    # Next add more to that Array.
    # ======================================================================= #
    i.each {|entry|
      _ = action(:SanitizeCookbook, entry) { :fast }
      deps = _.required_deps_on?
      if deps # Must catch stray nils.
        array_all_dependencies << sanitize_this_array(deps)
        array_all_dependencies.flatten!
        array_all_dependencies.uniq!
      end
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = action(:SanitizeCookbook, program?) { :fast }
    deps = _.required_deps_on? # This will be an array.
    report_these_dependencies(deps)
    gather_all_unique_dependencies_for(deps)
    sort_the_big_array
    report
  end

end; end

# =========================================================================== #
# === RBT.show_dependencies_of
# =========================================================================== #
def self.show_dependencies_of(i = ARGV)
  RBT::Cookbooks::ShowDependenciesOf.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ShowDependenciesOf.new(ARGV)
end # sdof ruby
    # sdof gtk+