#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkFromToCurrent
#
# This class can symlink a given program into the corresponding
# GoboLinux-like hierarchy - that is, a target path such as
# /Programs/Foo/1.2.3/.
#
# This allows the user to make use of the appdir-compiled program in a
# FHS system.
#
# The class must be flexible enough to accomodate for different
# invocation styles.
#
# For instance, the following variants must all work:
#
#   cd /Programs/Htop/Current; rnsymc
#   rnsymc php 7.3.7
#   rnsymc php
#
# Despite its name, this class can also be used to symlink from a specific
# directory. But the primary use case is to complement class RBT::ToCurrent.
#
# Usage example:
#
#   RBT::SymlinkFromToCurrent.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_from_to_current/symlink_from_to_current.rb'
# RBT::SymlinkFromToCurrent.new(ARGV)
# =========================================================================== #
require 'rbt/compile_base/compile_base.rb'

module RBT

class SymlinkFromToCurrent < ::RBT::CompileBase # === RBT::SymlinkFromToCurrent

  # ========================================================================= #
  # === DO_REGISTER_INTO_THE_LOCAL_YAML_DATABASE
  # ========================================================================= #
  DO_REGISTER_INTO_THE_LOCAL_YAML_DATABASE = true

  # ========================================================================= #
  # === RBT::SymlinkFromToCurrent::ARRAY_ALLOWED_SUBDIRECTORIES
  #
  # Which subdirectories are allowed for this class. bin/, sbin/ and
  # lib/ are the three main directories.
  #
  # The trailing '/' can be omitted.
  # ========================================================================= #
  ARRAY_ALLOWED_SUBDIRECTORIES = %w(
    bin
    sbin
    lib
    lib64
    include
    etc
    pkgconfig
    man
    Shared
    shared
    share
    yaml
    Settings
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet,
         :dont_run_yet
      run_already = false
    end
    # ======================================================================= #
    # === Handle blocks given next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Hashes next
      # ===================================================================== #
      if yielded.is_a? Hash
        # =================================================================== #
        # === :use_opn
        # =================================================================== #
        if yielded.has_key? :use_opn
          disable_opn if (yielded[:use_opn] == false)
        end
      else
        case yielded
        # =================================================================== #
        # === :do_not_run_yet
        # =================================================================== #
        when :do_not_run_yet
          run_already = false
        # =================================================================== #
        # === :do_not_register
        # =================================================================== #
        when :do_not_register
          do_not_register_into_the_local_yaml_database
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose
    # ======================================================================= #
    # === :use_opn
    # ======================================================================= #
    do_use_opn
    # ======================================================================= #
    # === :force_symlink
    #
    # If this variable is true then we will remove entries at /usr/bin
    # before doing the symlink-operation. Use this only if you want to
    # do so, via the commandline flag --force.
    #
    # By default this is false, aka "disabled".
    # ======================================================================= #
    @internal_hash[:force_symlink] = false
    # ======================================================================= #
    # === :remove_empty_directories
    #
    # The next instance variable determines whether we will remove empty
    # directories or not. By default we will do so, but this behaviour
    # has to be modifiable as not every user may wish to have this,
    # hence why this instance variable exists.
    # ======================================================================= #
    @internal_hash[:remove_empty_directories] = true
    # ======================================================================= #
    # === :shall_we_symlink_the_headers
    # ======================================================================= #
    @internal_hash[:shall_we_symlink_the_headers] = false
    # ======================================================================= #
    # === :array_remove_these_directories
    # ======================================================================= #
    @internal_hash[:array_remove_these_directories] = []
    # ======================================================================= #
    # === :do_register_into_the_local_yaml_database
    # ======================================================================= #
    @internal_hash[:do_register_into_the_local_yaml_database] =
      DO_REGISTER_INTO_THE_LOCAL_YAML_DATABASE
    # ======================================================================= #
    # === @array_allowed_subdirectories
    # ======================================================================= #
    @array_allowed_subdirectories = ARRAY_ALLOWED_SUBDIRECTORIES
    # ======================================================================= #
    # === @will_we_symlink
    #
    # The next variable determines if we will actually symlink or whether
    # we will not.
    # ======================================================================= #
    @will_we_symlink = true
    # ======================================================================= #
    # === @symlink_pkgconfig_files
    #
    # Determine whether the .pc files will be symlinked as well or not.
    # ======================================================================= #
    @symlink_pkgconfig_files = true
  end

  # ========================================================================= #
  # === consider_removing_empty_directories
  # ========================================================================= #
  def consider_removing_empty_directories
    do_remove_empty_directories if remove_empty_directories?
  end

  # ========================================================================= #
  # === consider_symlinking_headers
  #
  # This must be conditionally - some programs will specify that we
  # symlink headers whereas others will not.
  # ========================================================================= #
  def consider_symlinking_headers
    _ = program_name?
    require 'rbt/requires/require_the_cookbook_class.rb'
    # ======================================================================= #
    # Next, load up the yaml file.
    # ======================================================================= #
    if shall_we_symlink_the_headers?
      shall_we_symlink_the_headers = shall_we_symlink_the_headers?
    else
      @dataset = action(:SanitizeCookbook, _) { :fast }
      unless shall_we_symlink_the_headers? == false
        # =================================================================== #
        # The following check is a bit odd - I did not add a comment as
        # to why I added the next line, but the check above was now
        # added at 02.01.2020, to skip a nonsensical situation.
        # =================================================================== #
        shall_we_symlink_the_headers = @dataset.shall_we_symlink_the_headers?
      end
    end
    if shall_we_symlink_the_headers
      # ===================================================================== #
      # Ok, in this case we delegate to class SymlinkHeaders.
      # ===================================================================== #
      require 'rbt/utility_scripts/symlink_headers.rb'
      RBT::SymlinkHeaders.new(_)
    end
  end

  # ========================================================================= #
  # === register_for_removal
  #
  # This method will register a directory for removal. Store the
  # absolute path here - this simplifies removing said directory..
  #
  # The check whether the directory is empty or not is done in another
  # part of this class.
  # ========================================================================= #
  def register_for_removal(i)
    i = rds(i)
    @internal_hash[:array_remove_these_directories] << i unless (i == '/')
  end

  # ========================================================================= #
  # === remove_these_directories?
  # ========================================================================= #
  def remove_these_directories?
    @internal_hash[:array_remove_these_directories]
  end

  # ========================================================================= #
  # === remove_empty_directories?
  # ========================================================================= #
  def remove_empty_directories?
    @internal_hash[:remove_empty_directories]
  end

  # ========================================================================= #
  # === do_keep_empty_directories
  # ========================================================================= #
  def do_keep_empty_directories
    @internal_hash[:remove_empty_directories] = false
  end; alias keep_empty_directories do_keep_empty_directories # === keep_empty_directories

  # ========================================================================= #
  # === keep_empty_directories?
  # ========================================================================= #
  def keep_empty_directories?
    !remove_empty_directories?
  end

  # ========================================================================= #
  # === do_remove_empty_directories
  #
  # This method will remove directories that are empty. For this, we
  # will make of the method remove() which resides in shared.rb.
  # ========================================================================= #
  def do_remove_empty_directories
    if run_simulation?
      opne 'As we are running in simulation mode, we will not remove '
      opne 'any of the following directories:'
      e
      pp remove_these_directories?
      e
    else # Removing the directories next.
      unless remove_these_directories?.empty?
        orev "Removing these #{slateblue('(empty)')} #{rev}directories next:"
        remove_these_directories?.each {|entry|
          e "  #{sdir(entry)}"
        }
        remove(remove_these_directories?)
      end
    end
  end

  # ========================================================================= #
  # === program_version?
  #
  # This method should return the program-version of the program at hand.
  # ========================================================================= #
  def program_version?
    _ = program_name?.to_s.dup
    i.chop! if _.end_with? '/'
    if _.include? '/'
      i = i.split('/').last.to_s
    end
    return _
  end

  # ========================================================================= #
  # === show_help
  #
  # To invoke this method from the commandline, try:
  #
  #   rnsymc --help
  #
  # ========================================================================= #
  def show_help
    e
    e 'The following options are available:'
    e
    eparse '  --symlink-headers # the .h header files will be '\
           'symlinked as well'
    eparse '  --bin             # symlink ONLY the bin/ subdirectory'
    eparse '  --force           # delete the target at '+
           sdir('/usr/bin/')+' before doing the symlink-operation'
    e
  end

  # ========================================================================= #
  # === force_symlink?
  # ========================================================================= #
  def force_symlink?
    @internal_hash[:force_symlink]
  end; alias force_the_symlink_operation? force_symlink? # === force_the_symlink_operation?

  require 'rbt/utility_scripts/register_program_files_into_yaml_database.rb'
  # ========================================================================= #
  # === do_register_into_yaml_database
  # ========================================================================= #
  def do_register_into_yaml_database(
      i                                             = full_path?,
      register_program_files_into_the_yaml_database = RBT.register_program_files_into_yaml_database?
    )
    if i
      unless i.include? '/'
        i = static_appdir_prefix_of?(i, program_version?)
      end
      if register_program_files_into_the_yaml_database 
        action(:RegisterProgramFilesIntoYamlDatabase, i)
      end
    end
  end

  # ========================================================================= #
  # === is_it_a_number?
  # ========================================================================= #
  def is_it_a_number?(i)
    i = i.to_s.dup.delete('.') # Ignore '.' tokens.
    return i =~ /^\d+$/
  end

  # ========================================================================= #
  # === keep_track_which_program_was_last_symlinked
  #
  # This method will save the name of the program that this class has
  # been working on, into a local file.
  #
  # The reasoning behind this behaviour is so that we can use this in
  # other classes, such as when we use :last on the commandline. The
  # class that will act on :last will then pick up the content of the
  # file that we have been using to store this information into. This
  # then allows us to always refer to the "last program that was
  # symlink_program-ed", which is quite convenient as we don't have
  # to remember the name.
  # ========================================================================= #
  def keep_track_which_program_was_last_symlinked
    what = program_name?
    into = file_last_symlinked_program?
    write_what_into(what, into)
  end; alias try_to_save_the_name_of_the_program_into_a_local_file keep_track_which_program_was_last_symlinked # === try_to_save_the_name_of_the_program_into_a_local_file

  # ========================================================================= #
  # === capitalized
  # ========================================================================= #
  def capitalized(i)
    i = i.dup
    i.capitalize! if do_capitalize?
    return i
  end

  # ========================================================================= #
  # === assumed_target_of
  # ========================================================================= #
  def assumed_target_of(i)
    target = i.dup
    case i
    when /bin$/
      target = usr_bin?
    when /share$/,
         /shared$/,
         /Shared$/
      target = share_dir?
    when /lib$/,
         /lib64$/ # lib64 could be controversial - hmmm.
      target = lib_dir?
    when /etc$/
      target = etc_dir?
    when /libexec$/
      target = '/usr/libexec/'
    end
    return target
  end

  # ========================================================================= #
  # === symlink_this_program?
  #
  # This method will return the NAME of the program at hand, in a
  # downcased manner.
  # ========================================================================= #
  def symlink_this_program?
    _ = @use_this_program.to_s.dup
    if _.include? _
      _.sub!(/#{Regexp.quote(programs_dir?)}/,'')
      # May look like this now: "Vala/0.50.3/"
      if _.include? '/'
        _ = _.split('/').first
      end
    end
    _.downcase!
    return _
  end; alias program_name?        symlink_this_program? # === program_name?
       alias input?               symlink_this_program? # === input?
       alias name_of_the_program? symlink_this_program? # === name_of_the_program?
       alias program?             symlink_this_program? # === program?

  # ========================================================================= #
  # === current_target?
  #
  # This will yield a String as a result, such as '/Programs/Htop/Current/'.
  #
  # These constitute our the symlink-targets.
  #
  # We must retain a trailing '/' here.
  # ========================================================================= #
  def current_target?
    _  = programs_directory?.to_s.dup
    if name_of_the_program?
      _ << capitalized(name_of_the_program?)
    else
      # opnwarn('No name of the program is known.')
    end
    _ << '/Current/' unless _.include? 'Current'
    _ = rds(_)
    return _
  end; alias target? current_target? # === target

  # ========================================================================= #
  # === do_symlink_the_headers
  # ========================================================================= #
  def do_symlink_the_headers
    @internal_hash[:shall_we_symlink_the_headers] = true
  end

  # ========================================================================= #
  # === will_we_symlink?
  # ========================================================================= #
  def will_we_symlink?
    @will_we_symlink
  end; alias we_will_symlink? will_we_symlink? # === we_will_symlink?

  # ========================================================================= #
  # === do_not_symlink=
  # ========================================================================= #
  def do_not_symlink=(i)
    if i == true
      do_not_symlink
    else
      @will_we_symlink = true
    end
  end

  # ========================================================================= #
  # === do_not_symlink
  # ========================================================================= #
  def do_not_symlink
    @will_we_symlink = false
  end

  # ========================================================================= #
  # === use_this_program?
  # ========================================================================= #
  def use_this_program?
    @use_this_program
  end; alias prefix?                          use_this_program? # === prefix?
       alias full_path?                       use_this_program? # === full_path?
       alias path?                            use_this_program? # === full_path?
       alias full_path_to_the_target_program? use_this_program? # === full_path_to_the_target_program?
       alias target_directory?                use_this_program? # === target_directory?

  # ========================================================================= #
  # === do_force_symlink
  # ========================================================================= #
  def do_force_symlink
    @internal_hash[:force_symlink] = true
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # All commandline options for this class can be passed through the
  # menu() method here.
  # ========================================================================= #
  def menu(
      i = hyphen_commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry)}
    else
      case i
      # ===================================================================== #
      # === rnsymc --force
      # ===================================================================== #
      when /^-?-?force$/i,
           /^-?-?f$/i
        do_force_symlink
      # ===================================================================== #
      # === symlink_program --help
      #
      # This entry point is used to display the help interface of this
      # class.
      # ===================================================================== #
      when /^-?-?help$/i,
           /^-?-?show(-|_)?help$i/
        show_help
      # ===================================================================== #
      # === rnsymc --bin/
      # === rnsymc --bin
      # ===================================================================== #
      when /^-?-?bin\/?$/i
        symlink_only_the_bin_subdirectory
      # ===================================================================== #
      # === symlink_program --symlink-headers
      #
      # This will delegate towards class SymlinkHeaders for symlinking
      # the .h header files.
      # ===================================================================== #
      when /^-?-?symlink(-|_)?headers$/
        do_symlink_the_headers
      end
    end
  end

  # ========================================================================= #
  # === only_allow_bin_as_subdirectory
  #
  # This method will enable that we only symlink the bin/ subdirectory.
  # ========================================================================= #
  def only_allow_bin_as_subdirectory
    orev "Only the subdirectory at "\
         "#{sdir('bin/')} #{rev}will be symlinked next."
    @array_allowed_subdirectories.clear
    @array_allowed_subdirectories << 'bin'
    # ======================================================================= #
    # Disable some variables too, in this event.
    # ======================================================================= #
    @internal_hash[:shall_we_symlink_the_headers] = false
    @internal_hash[:do_register_into_the_local_yaml_database] = false
    @symlink_pkgconfig_files = false
  end; alias symlink_only_the_bin_subdirectory only_allow_bin_as_subdirectory # === symlink_only_the_bin_subdirectory

  # ========================================================================= #
  # === shall_we_symlink_the_headers?
  # ========================================================================= #
  def shall_we_symlink_the_headers?
    @internal_hash[:shall_we_symlink_the_headers]
  end

  # ========================================================================= #
  # === first_argument_or_current_working_directory
  # ========================================================================= #
  def first_argument_or_current_working_directory
    _ = first?
    if (_ and _.start_with?('--')) or
       (_ and _.empty?) or
       _.nil?
      _ = return_pwd
    end
    return _
  end

  # ========================================================================= #
  # === do_not_register_into_the_local_yaml_database
  # ========================================================================= #
  def do_not_register_into_the_local_yaml_database
    @internal_hash[:do_register_into_the_local_yaml_database] = false
  end; alias do_not_register do_not_register_into_the_local_yaml_database # === do_not_register

  # ========================================================================= #
  # === do_register_into_the_local_yaml_database?
  # ========================================================================= #
  def do_register_into_the_local_yaml_database?
    @internal_hash[:do_register_into_the_local_yaml_database]
  end

  # ========================================================================= #
  # === consider_symlinking_pkgconfig_files
  #
  # This method can be used to symlink .pc files into the /usr/lib/pkgconfig
  # hierarchy.
  #
  # Since as of February 2023 this method is no longer in use, due to a
  # bug that took too long to understand. We may have to rewrite this
  # class eventually.
  # ========================================================================= #
  def consider_symlinking_pkgconfig_files(
      appdir_target = full_path_to_the_target_program?
    )
    # ======================================================================= #
    # The appdir_target variable may be incorrect, such as when we symlink
    # another version.
    # ======================================================================= #
    if appdir_target.respond_to? :split
      splitted = appdir_target.split('/')
      unless splitted.last == full_path_to_the_target_program?.split('/').last
        appdir_target = full_path_to_the_target_program?
      end
    else
      opne 'A minor problem - '+steelblue('appdir_target')+' does '\
           'not respond to the method .split.'
      opne 'We will still continue, though.'
    end
    if @symlink_pkgconfig_files and
       appdir_target.include?(programs_dir?)
      require 'rbt/utility_scripts/symlink_pkgconfig_files.rb'
      opne "Next, we will symlink all .pc files found "\
           "under `#{sdir(appdir_target)}`."
      SymlinkPkgconfigFiles.new(appdir_target)
    end
  end

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    @symlink_pkgconfig_files
  end

  # ========================================================================= #
  # === set_use_this_program
  #
  # Keep in mind that this method has to be flexible; user input may
  # look like this:
  #
  #   rnsymc php 7.3.7
  #
  # We could also enable input such as 'Ruby/2.6.3/' but right now this
  # class is not handling such input.
  # ========================================================================= #
  def set_use_this_program(
      i = nil
    )
    if i
      # ===================================================================== #
      # First, handle Arrays.
      # ===================================================================== #
      i = i.first if i.is_a? Array
      # ===================================================================== #
      # === Handle Hashes next
      #
      # If a Hash is given to this method then this class will try to
      # assume that it contains a key called :to.
      # ===================================================================== #
      if i.is_a? Hash
        i = i.delete(:to) if i.has_key? :to
      end
      commandline_arguments = commandline_arguments?
      # ===================================================================== #
      # Check for the second argument being a number or not:
      # ===================================================================== #
      if commandline_arguments and is_it_a_number?(commandline_arguments[1])
        i = "#{appdir_prefix?}#{capitalized(i)}/#{commandline_arguments[1]}/" 
      end
      case i
      when :default
        i = static_appdir_prefix_of?(name_of_the_program?, program_version?)
      else
        unless i.include? '/'
          i = appdir_prefix?+capitalized(i)+'/Current/'
        end
      end
      unless i.end_with? '/'
        i << '/'
      end
    end
    if i.is_a? String
      i = rds(i) # Get rid of double slashes, just in case.
    end
    @use_this_program = i
  end; alias set_prefix                          set_use_this_program # === set_prefix
       alias set_full_path                       set_use_this_program # === set_full_path
       alias set_full_path_to_the_target_program set_use_this_program # === set_full_path_to_the_target_program
       alias set_symlink_this_program            set_use_this_program # === set_symlink_this_program
       alias set_name_of_the_program             set_use_this_program # === set_name_of_the_program
       alias set_use_this_program                set_use_this_program # === set_use_this_program

  # ========================================================================= #
  # === inform_the_user_what_will_be_done                        (inform tag)
  # ========================================================================= #  
  def inform_the_user_what_will_be_done(
      program_name = program_name?
    )
    _ = "Working on the program #{sfancy(program_name)}#{rev}. ".dup
    if @array_allowed_subdirectories.size == 1
      _ << "#{rev}The subdirectory found at\n"
    else
      _ << "#{rev}The subdirectories found at\n" 
    end
    opnn if use_opn?; e _
    _ = "#{rev}`#{sfancy(use_this_program?)}#{rev}` will be symlinked."
    opnn if use_opn?; e _
  end

  # ========================================================================= #
  # === notify_the_user_processing_this_directory_for_symlinking_into_that_target
  # ========================================================================= #
  def notify_the_user_processing_this_directory_for_symlinking_into_that_target(
      name_of_the_subdirectory,
      target
    )
    e "#{rev}Processing #{sfancy(name_of_the_subdirectory)}"\
       "#{rev} for symlinking into `#{sdir(target)}"\
       "#{rev}` next:"
  end

  # ========================================================================= #
  # === do_symlink
  #
  # Since as of January 2020 the method will also support the --force
  # option, which will first delete the target at /usr/bin, before 
  # doing the symlink operation. Only use it if you are certain you
  # wish to do so.
  # ========================================================================= #
  def do_symlink
    will_we_symlink = will_we_symlink?
    _ = use_this_program?
    return unless _ and !_.empty?
    if _ and File.directory?(_)
      inform_the_user_what_will_be_done
      # ===================================================================== #
      # Obtain all subdirectories next (the toplevel-directories).
      # ===================================================================== #
      directories = Dir["#{_}**"].select {|entry| File.directory?(entry) }
      unless directories.empty?
        opnn if use_opn?
        e "#{rev}Symlinking from the directory #{sdir(_)} #{rev}next."
      end
      directories.each {|name_of_the_subdirectory|
        # =================================================================== #
        # === Eliminate empty subdirectories next
        #
        # If the directory in question is empty, it will be eliminated.
        #
        # This is not ideal, though, as it may also remove new directories
        # created during the postinstallation step. Hence, since as of
        # December 2022, only certain empty directores will be removed.
        # =================================================================== #
        if Dir.empty?(name_of_the_subdirectory)
          basename = File.basename(name_of_the_subdirectory)
          if ARRAY_ALLOWED_SUBDIRECTORIES.include?(basename)
            register_for_removal(name_of_the_subdirectory)
          end
        else
          # ================================================================= #
          # === Process allowed subdirectories next
          #
          # Check whether we have an allowed subdirectory or not first.
          # This class will only handle allowed subdirectories past the
          # following point.
          #
          # The Array that is processed next may look as follows:
          #
          #   ["/home/Programs/Pixman/0.42.2/Resources",
          #    "/home/Programs/Pixman/0.42.2/include",
          #    "/home/Programs/Pixman/0.42.2/lib",
          #    "/home/Programs/Pixman/0.42.2/yaml"]
          #
          # ================================================================= #
          if @array_allowed_subdirectories.any? {|entry|
              name_of_the_subdirectory.end_with?(entry)
            }
            short_name_of_the_subdirectory = File.basename(name_of_the_subdirectory)
            @will_we_symlink = true # Reset our flag here.
            if run_simulation?
              opne 'We run in simulation mode, thus we will not '
              opne "symlink #{sfile(entry)} into #{sfile(assumed_target_of(entry))}."
            else
              case short_name_of_the_subdirectory # (case tag)
              # =============================================================== #
              # === bin
              #
              # This also includes sbin/ directories.
              #
              # This functionality was removed in September 2022. It is
              # now handled by a toplevel action, via RBT.action().
              #
              # In May 2023 this was partially restored, but now handled
              # via RBT.action().
              # =============================================================== #
              when /bin$/
                action(:symlink_into_the_usr_bin_hierarchy, name_of_the_subdirectory) {{
                  force_symlink: force_symlink?
                }}
              # =============================================================== #
              # === lib
              #
              # Symlink into the lib/ subdirectory here; typically into
              # /usr/lib/.
              #
              # This entry is special in that we also have to consider
              # the /usr/lib/pkgconfig/ subdirectory - otherwise the
              # relevant .pc files may not be symlinked.
              # =============================================================== #
              when /lib$/,
                   /lib64$/ # lib64 could be controversial - hmmm.
                target = assumed_target_of(name_of_the_subdirectory)
                if be_verbose?
                  opnn if use_opn?
                  notify_the_user_processing_this_directory_for_symlinking_into_that_target(
                    name_of_the_subdirectory,
                    target
                  )
                end
                if will_we_symlink
                  symlink_from_to(name_of_the_subdirectory, target)
                  pkgconfig_subdirectory = name_of_the_subdirectory+'/pkgconfig/'
                  if File.directory?(pkgconfig_subdirectory) and
                    (all_pc_files = Dir[pkgconfig_subdirectory+'*.pc'])
                    unless all_pc_files.empty?
                      # ===================================================== #
                      # Ok, we found at the least one .pc file. Thus we can
                      # try to symlink these .pc files.
                      # ===================================================== #
                      target = '/usr/lib/pkgconfig/'
                      all_pc_files.each {|this_pc_file|
                        real_target = target+
                                      File.basename(this_pc_file)
                        symlink_this_file(
                          this_pc_file,
                          real_target
                        )
                      }
                    end
                  end
                end
              # =============================================================== #
              # === include
              #
              # This is equivalent towards a target such as '/usr/include/'.
              # =============================================================== #
              when /include$/
                target = include_dir?
                if @will_we_symlink
                  opnn if use_opn?
                  e "#{rev}Symlinking into the main "+
                    sdir('include/')+
                    "#{rev} directory at #{sdir(target)}."
                  symlink_from_to(name_of_the_subdirectory, target) { :include_directories }
                end
              # =============================================================== #
              # === share
              # =============================================================== #
              when /share$/,
                   /shared$/,
                   /Shared$/
                target = assumed_target_of(name_of_the_subdirectory)
                if be_verbose?
                  opnn if use_opn?
                  notify_the_user_processing_this_directory_for_symlinking_into_that_target(
                    name_of_the_subdirectory,
                    target
                  )
                end
                symlink_from_to(name_of_the_subdirectory, target) if @will_we_symlink
              # =============================================================== #
              # === etc
              # =============================================================== #
              when /etc$/
                target = assumed_target_of(name_of_the_subdirectory)
                if be_verbose?
                  opnn if use_opn?
                  notify_the_user_processing_this_directory_for_symlinking_into_that_target(
                    name_of_the_subdirectory,
                    target
                  )
                end
                symlink_from_to(name_of_the_subdirectory, target) if @will_we_symlink
              # =============================================================== #
              # === libexec
              # =============================================================== #
              when /libexec$/
                target = assumed_target_of(name_of_the_subdirectory)
                if be_verbose?
                  opnn if use_opn?
                  notify_the_user_processing_this_directory_for_symlinking_into_that_target(
                    name_of_the_subdirectory,
                    target
                  )
                end
                symlink_from_to(name_of_the_subdirectory, target) if @will_we_symlink
              # =============================================================== #
              # === yaml
              # =============================================================== #
              when 'yaml' # Ignore this entry.
                @will_we_symlink = false
              # =============================================================== #
              # === man
              #
              # For now, we don't have a place for man pages.
              # =============================================================== #
              when 'man'
                @will_we_symlink = false
              else
                @will_we_symlink = false
                # unless (name_of_the_subdirectory == 'yaml') or 
                #        (name_of_the_subdirectory == 'include')
                #   opne "=> #{sdir(name_of_the_subdirectory)}"\
                #        "#{swarn(' does not exist, we thus ignore it.')}"
                # end
              end
            end
          # else
          #   opnn if use_opn?; e "Unhandled subdirectory: #{sdir(name_of_the_subdirectory)}"
          end
        end
      }
    end
    consider_removing_empty_directories # ← Should come before symlinking headers files
    # ======================================================================= #
    # Register into the yaml database, before symlinking pkgconfig files.
    # ======================================================================= #
    do_register_into_yaml_database if do_register_into_the_local_yaml_database? # ← Should go before symlinking the headers.
    consider_symlinking_headers         # ← Should come before symlinking pkgconfig files
    # consider_symlinking_pkgconfig_files # ← Should come before saving the name of the program into a local file.
    keep_track_which_program_was_last_symlinked
  end; alias do_symlink_all_entries do_symlink # === do_symlink_all_entries

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    set_use_this_program(
      first_argument_or_current_working_directory
    )
    menu
    do_symlink if will_we_symlink?
  end

  # ========================================================================= #
  # === RBT::SymlinkFromToCurrent[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.symlink_this_program
#
# Easier module-method variant to use.
# =========================================================================== #
def self.symlink_this_program(i = ARGV)
  RBT::SymlinkFromToCurrent.new(i)
end; self.instance_eval { alias symlink_program symlink_this_program } # === RBT.symlink_program

end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkFromToCurrent.new(ARGV)
end # rnsymc
    # rnsymc --bin
    # rnsymc php 7.3.7