#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::WhatCouldBecomeAnAppDir
#
# This class will inform the user as to which applications could
# be "turned" into an AppDir.
#
# The general strategy for this class will be to go through the
# /usr/bin/ directory and check on each binary there.
#
# Then, if the binary is NOT registered in the RBT project, report
# to the user - but only if it really is a binary file.
#
# Otherwise, if the binary is registered, and NOT a symlink, then
# the user is told how he could compile that program in an AppDir
# fashion, showing the exact command to use here.
#
# Usage example:
#
#   RBT::WhatCouldBecomeAnAppDir.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/what_could_become_an_app_dir.rb'
# RBT.what_could_become_an_app_dir
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class WhatCouldBecomeAnAppDir < Base # === RBT::WhatCouldBecomeAnAppDir

  # ========================================================================= #
  # === UBIN
  # ========================================================================= #
  UBIN = '/usr/bin/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    #reset_the_internal_hash
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = UBIN
    opne 'Going through all binaries at '+sdir(_)+rev+' next:'
    all_files = all_files_from(_)
    all_files.each_with_index {|this_file, index| index += 1
      short_name = File.basename(this_file)
      if is_this_binary_registered?(short_name) and !is_symlink?(this_file)
        opne rev+'This binary ('+steelblue(short_name)+
             rev+') could be compiled in an AppDir manner, by issuing:'
        e
        e steelblue("  ntrad #{short_name}")
        e
      else
        if is_binary?(this_file)
          opne lightgreen(index.to_s+') ')+
               tomato('Not registered binary: ')+
               steelblue(short_name)
        end
      end
    }
  end

  # ========================================================================= #
  # === is_binary?
  # ========================================================================= #
  def is_binary?(i)
    result = `file #{i}`
    result.include?('executable, x86-64')
  end

  # ========================================================================= #
  # === RBT::WhatCouldBecomeAnAppDir[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.what_could_become_an_app_dir
# =========================================================================== #
def self.what_could_become_an_app_dir(i = ARGV)
  RBT::WhatCouldBecomeAnAppDir.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::WhatCouldBecomeAnAppDir.new(ARGV)
end # whatcouldbecomeanappdir