#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::SanitizeCookbook
#
# From holding the dataset of a specific cookbook at hand, over to
# displaying said dataset - this is the class to go for.
#
# This class builds on the data made available via
# RBT::Action::Cookbooks::RawCookbook, and turns the dataset
# of RBT::Action::Cookbooks::RawCookbook into a "full" dataset,
# that is, one that is completely sanitized, and ready
# made for storing it into a new .yml file.
#
# This behaviour is only valid if the :simple mode is used.
#
# By default, however had, another mode will be used, called :complex
# (or :expanded). This one will use the expanded dataset instead.
#
# The idea here is that we can use (and maintain) a minimal dataset
# via class RBT::Action::Cookbooks::RawCookbook, while providing a more useful
# dataset via class RBT::Action::Cookbooks::SanitizeCookbook. Note that class
# RBT::Action::Cookbooks::SanitizeCookbook has to ensure that certain values
# are set to their default (and thus correct) values. The code in
# this file here ensures precisely ensures this.
#
# The obtain() method is used specifically by class
# RBT::Action::Cookbooks::SanitizeCookbook for query-related
# methods, and queries in general.
#
# Every source archive has different components which have to be
# "assembled" properly. All entries must adhere to this
# specification as much as possible.
#
# For legacy reasons, real_short_name is an alias to short_name.
# (In May 2020 the code was rewritten, and the old real_short_name 
# was eliminated in the process.)
#
# The following numbers are important because we can invoke methods
# based on the information stored in them, at the least in the past.
# For instance, entry (8) can be assembled from entry (5) and
# entry (7).
#
# However had, since as of May 2020, this is no longer encouraged; it
# is much better to use the slightly longer names rather than numbers.
# Numbers are harder to remember, too. This was why they were
# deprecated (or discouraged) eventually.
#
# The general format is as follows:
#
#   (1) short_name                        like 'gnomemahjongg'  # This must not have any '-' tokens.
#   (2) program_name                      like 'gnome-mahjongg'
#   (3) program_version                   like '5.08'
#   (4) program_name_and_program_version  like 'gnome-mahjonng-5.08' # This one includes the version. It is short_name+'-'+program_version
#   (5) archive_type                      like: '.tar.bz2' # Yes, this must have a leading '.'
#   (6) program_full_name                 like: 'gnome-mahjonng-5.08.tar.xz', so this is program_name_and_program_version?+archive_type?
#   (7) base_dir                          like: '/home/x/src/gnomemahjongg/' # This is src_dir?+short_name?
#   (8) program_path                      like: '/home/x/src/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
#
# Usage example:
#
#   RBT::Action::Cookbooks::SanitizeCookbook.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/sanitize_cookbook/sanitize_cookbook.rb'
# RBT::Action::Cookbooks::SanitizeCookbook.new(ARGV)
# RBT.sanitize_cookbook(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class SanitizeCookbook < RBT::Action # === RBT::Action::Cookbooks::SanitizeCookbook

  require 'rbt/actions/individual_actions/cookbooks/sanitize_cookbook/sanitize_related_code.rb'

  # ========================================================================= #
  # === HOW_TO_INVOKE_THE_PROGRAM
  # ========================================================================= #
  HOW_TO_INVOKE_THE_PROGRAM = 'scookie'

  # ========================================================================= #
  # === DEFAULT_PROGRAM_VERSION
  # ========================================================================= #
  DEFAULT_PROGRAM_VERSION = '1.0.0'

  # ========================================================================= #
  # === REPLACE_PLUS_WITH_LONG_NAME
  #
  # If this constant is true then we will replace '+' with 'plus'
  # if they are part of a program name.
  # ========================================================================= #
  REPLACE_PLUS_WITH_LONG_NAME = false

  # ========================================================================= #
  # === ARRAY_ARRAY_ENTRIES                                       (array tag)
  #
  # These entries should be kept as an Array at all time. They will all
  # allow us to run .each on them.
  #
  # In "pure" ruby code, these would all be set to e. g. "xyz = []", so
  # really just an Array of Arrays.
  # ========================================================================= #
  ARRAY_ARRAY_ENTRIES = %w(
    blfs
    binaries
    gir_files
    headers
    installation_steps
    libraries
    m4_files
    pkgconfig_files
    postinstall
    pre_configure_steps
    pre_make_commands
    pre_make_install_sed
    required_deps_on
    tags
  )

  # ========================================================================= #
  # === ARRAY_NIL_ENTRIES                                           (nil tag)
  #
  # All entries that are nil by default.
  #
  # These are entries where a default value is not absolutely necessary.
  # ========================================================================= #
  ARRAY_NIL_ENTRIES = %w(
    short_description
    set_env_variables
    homepage
    meson_configure_options
    cmake_configure_options
    svn_url
    git_url
    program_name_and_program_version
    use_this_build_system
  )

  # ========================================================================= #
  # === ARRAY_TRUE_VALUES                                          (true tag)
  #
  # Only a few variables are true for now, by default - prefix,
  # run_ldconfig and has_to_be_compiled at the very least.
  # ========================================================================= #
  ARRAY_TRUE_VALUES = %w(
    is_an_active_project
    has_to_be_compiled
    may_we_modify_the_configure_options
    prefix
    run_configure
    run_ldconfig
    run_make
  )

  # ========================================================================= #
  # === ARRAY_FALSE_VALUES                                        (false tag)
  #
  # This constant collects all values that must be explicitely set to
  # false on startup.
  # ========================================================================= #
  ARRAY_FALSE_VALUES = %w(
    apply_patch
    autosymlink_lib64
    build_static
    can_be_compiled_statically
    copy_source
    do_not_symlink
    do_not_download
    echo_yes
    enable_shared
    enable_static
    ignore_errors
    keep_extracted
    localstatedir
    mirror
    modify_the_makefile
    override
    run_make_check
    sed
    symlink_glib_schema
    symlink_headers
    symlink_pkgconfig_files
    use_autoconf
    use_autogen
    use_build_directory
    use_glib_schema
    use_this_make_install_command
  )

  # ========================================================================= #
  # === ARRAY_MAIN_PROGRAM_ENTRIES
  #
  # We need the following 8 entries defined here, so that we do not
  # forget them lateron.
  #
  # These are the only entries we care about and describe the most
  # important components of a program.
  # ========================================================================= #
  ARRAY_MAIN_PROGRAM_ENTRIES = %w(
    archive_type
    base_dir
    program_full_name
    program_name
    program_path
    program_version
    real_short_name
    short_name
  )

  # ========================================================================= #
  # === ARRAY_EMPTY_VALUES
  #
  # All values that will be initialized by default to an empty string,
  # aka '', have to be registered here.
  # ========================================================================= #
  ARRAY_EMPTY_VALUES = %w(
    program_path
    program_full_name
    configure_options
    configure_base_dir
    configure_command_to_use
    extract_to
    flatpak_url
    licence
    manual_steps
    md5sum
    optional_start_message
    parameters_to_make
    preinstall
    program_compile_name
    program_version
    sha256
    use_this_build_directory
    use_this_make_command
    url1
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    case i
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
      i = []
    end
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Symbols first
      # ===================================================================== #
      if yielded.is_a? Symbol
        case yielded
        # =================================================================== #
        # === :do_not_instantiate_a_new_raw_cookbook
        # =================================================================== #
        when :do_not_instantiate_a_new_raw_cookbook
          set_simple
          do_not_instantiate_a_new_raw_cookbook
        # =================================================================== #
        # === :do_not_run_yet
        # =================================================================== #
        when :do_not_run_yet,
             :dont_run_yet
          run_already = false
        # =================================================================== #
        # === :be_quiet_and_bypass_menu
        # =================================================================== #
        when :be_quiet_and_bypass_menu,
             :be_quiet
          be_quiet
        # =================================================================== #
        # === fast_and_be_verbose
        # =================================================================== #
        when :fast_and_be_verbose
          be_verbose
          set_simple_or_complex(:complex)
        # =================================================================== #
        # === :be_fast_and_be_quiet
        # =================================================================== #
        when :be_fast_and_be_quiet,
             :complex,
             :use_expanded_dataset,
             :expanded,
             :default
          set_be_quiet
          set_simple_or_complex(:complex)
        # =================================================================== #
        # === :be_fast
        # =================================================================== #
        when :be_fast,
             :fast
          set_simple_or_complex(:complex)
        # =================================================================== #
        # === :simple
        #
        # Note that :return_flat_datastructure is an alias, since as of
        # April 2021.
        # =================================================================== #
        when :simple,
             :simple_cookbook,
             :return_flat_datastructure
          set_simple_or_complex(:simple)
        else
          # ================================================================= #
          # === Handle unknown Symbols
          #
          # This may be disabled one day - not sure yet. For now it will
          # remain, though.
          # ================================================================= #
          if yielded.is_a? Symbol
            orev "Unknown symbol called "\
                 "#{sfancy(':')}#{sfancy(yielded.to_s)} #{rev}passed."
            orev 'Showing the very important '+tomato('caller()')+
                 "#{rev} backtrace next:"
            e
            cliner; pp caller(); cliner
            e
          end
          set_simple_or_complex(yielded)
        end
      # ===================================================================== #
      # === Handle Hashes next
      # ===================================================================== #
      elsif yielded.is_a? Hash
        # =================================================================== #
        # === :make_use_of_aliases_kept_in_the_file_cookbook_aliases
        # =================================================================== #
        if yielded.has_key? :make_use_of_aliases_kept_in_the_file_cookbook_aliases
          _ = yielded[:make_use_of_aliases_kept_in_the_file_cookbook_aliases]
          set_make_use_of_aliases_kept_in_the_file_cookbook_aliases(_)
        end
        # =================================================================== #
        # === :simple_or_complex
        # =================================================================== #
        if yielded.has_key? :simple_or_complex
          set_simple_or_complex(
            yielded[:simple_or_complex]
          )
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    initialize_the_internal_hash
    # ======================================================================= #
    # === :use_colours
    # ======================================================================= #
    set_use_colours
  end

  # ========================================================================= #
  # === initialize_the_internal_hash
  # ========================================================================= #
  def initialize_the_internal_hash
    # ======================================================================= #
    # === @internal_hash
    #
    # The internal hash contains lots of entries that are important for
    # this class to work properly.
    #
    # Since as of August we inherit the "parent" Hash, so we don't
    # re-assign it here.
    # ======================================================================= #
    # @internal_hash = {}
    # ======================================================================= #
    # === :dataset
    #
    # This entry will contain the dataset that refers to the program at
    # hand. If you want to clear it then you can use the method
    # .reset_the_dataset().
    # ======================================================================= #
    @internal_hash[:dataset] = nil
    # ======================================================================= #
    # === :original_unmodified_name
    #
    # This variable will keep track of which program we are searching for.
    # ======================================================================= #
    @internal_hash[:original_unmodified_name] = nil
    # ======================================================================= #
    # === raw_cookbook
    #
    # This is initially set to nil, simply to not forget it.
    # ======================================================================= #
    @internal_hash[:raw_cookbook] = nil
    # ======================================================================= #
    # === :use_this_connective_token
    #
    # Some programs, such as artemis, have a '_' rather than a '-'. So
    # we need to be able to keep track of this as well. The default is
    # '-' though.
    # ======================================================================= #
    @internal_hash[:use_this_connective_token] = '-'
    # ======================================================================= #
    # === :wrap_result
    #
    # By default, we will word-wrap too long text lines.
    # ======================================================================= #
    @internal_hash[:wrap_result] = true
    # ======================================================================= #
    # === :replace_plus_with_long_name
    # ======================================================================= #
    @internal_hash[:replace_plus_with_long_name] = REPLACE_PLUS_WITH_LONG_NAME
    # ======================================================================= #
    # === :try_to_truncate_too_long_output
    # ======================================================================= #
    @internal_hash[:try_to_truncate_too_long_output] = true
    # ======================================================================= #
    # === :seek_this_program
    # ======================================================================= #
    @internal_hash[:seek_this_program] = nil
    # ======================================================================= #
    # === :found
    #
    # This variable keeps track as to whether the input has been found
    # or not.
    # ======================================================================= #
    @internal_hash[:found] = false
    # ======================================================================= #
    # === :autogenerate_expanded_dataset
    # ======================================================================= #
    @internal_hash[:autogenerate_expanded_dataset] = false
    # ======================================================================= #
    # === :make_use_of_aliases_kept_in_the_file_cookbook_aliases
    #
    # Next, determine whether to use cookbook-internal aliases or whether
    # we shall not.
    # ======================================================================= #
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases] = true
    # ======================================================================= #
    # === :shall_we_display_the_md5sum
    # ======================================================================= #
    @internal_hash[:shall_we_display_the_md5sum] = RBT.display_md5sum?
    # ======================================================================= #
    # === :report_how_many_programs_are_registered
    # ======================================================================= #
    @internal_hash[:report_how_many_programs_are_registered] = false
    # ======================================================================= #
    # === :array_show_these_entries
    # ======================================================================= #
    @internal_hash[:array_show_these_entries] = []
    # ======================================================================= #
    # === :may_we_instantiate_a_new_raw_cookbook
    #
    # This variable can be used to instantiate a new raw-cookbook. This
    # is the default behaviour, but sometimes we may not want to do so,
    # and instead supply our own dataset, which is why this variable
    # exists.
    # ======================================================================= #
    @internal_hash[:may_we_instantiate_a_new_raw_cookbook] = true
    # ======================================================================= #
    # === :simple_or_complex
    # ======================================================================= #
    # @internal_hash[:simple_or_complex] = :simple
    set_simple_or_complex(:default) # Set it once on startup.
  end; alias reset_the_internal_hash initialize_the_internal_hash # === reset_the_internal_hash

  # ========================================================================= #
  # === set_url1
  #
  # This method will not do any sanitize-action on its own. It will be a
  # pure setter-method.
  # ========================================================================= #
  def set_url1(i)
    set(:url1, i)
  end

  # ========================================================================= #
  # === set_url2
  # ========================================================================= #
  def set_url2(i)
    set(:url2, i)
  end

  # ========================================================================= #
  # === use_autogen
  # ========================================================================= #
  def set_use_autogen(i = true)
    set(:use_autogen, i)
  end; alias use_autogen= set_use_autogen # === use_autogen?
       alias use_autogen  set_use_autogen # === set_use_autogen

  # ========================================================================= #
  # === set_last_update
  # ========================================================================= #
  def set_last_update(i)
    set(:last_update, i)
  end

  # ========================================================================= #
  # === set_manual_steps
  # ========================================================================= #
  def set_manual_steps(i)
    set(:manual_steps, i)
  end

  # ========================================================================= #
  # === set_preinstall
  # ========================================================================= #
  def set_preinstall(i)
    set(:preinstall, i)
  end; alias preinstall= set_preinstall # === preinstall=

  # ========================================================================= #
  # === set_gir_files
  # ========================================================================= #
  def set_gir_files(i)
    set(:gir_files, i)
  end

  # ========================================================================= #
  # === reset_the_dataset
  # ========================================================================= #
  def reset_the_dataset
    # ======================================================================= #
    # === :dataset
    # ======================================================================= #
    @internal_hash[:dataset] = {}
  end

  # ========================================================================= #
  # === set                                             (set tag, setter tag)
  # ========================================================================= #
  def set(key, value)
    dataset?[key.to_sym] = value
  end; alias set_in_yaml_dataset set # === set_in_yaml_dataset
       alias set_in_dataset      set # === set_in__dataset
       alias set_in_main_hash    set
       alias update_entries      set # === update_entries
       alias update_entry        set # === update_entry
       alias []=                 set # === []=

  # ========================================================================= #
  # === set_configure_command_to_use
  #
  # The configure command will usually be configure, but it may also
  # be cmake or waf or something like that.
  # ========================================================================= #
  def set_configure_command_to_use(
      i = :configure
    )
    set(:configure_command_to_use, i)
  end; alias configure_command=    set_configure_command_to_use # === configure_command=
       alias set_configure_command set_configure_command_to_use # === set_configure_command

  # ========================================================================= #
  # === set_use_this_build_directory
  # ========================================================================= #
  def set_use_this_build_directory(i)
    set(:use_this_build_directory, i)
  end; alias use_this_build_directory= set_use_this_build_directory # === use_this_build_directory=

  # ========================================================================= #
  # === set_installation_steps
  # ========================================================================= #
  def set_installation_steps(i)
    set(:installation_steps, i)
  end

  # ========================================================================= #
  # === set_headers
  # ========================================================================= #
  def set_headers(i)
    set(:headers, i)
  end

  # ========================================================================= #
  # === set_licence
  # ========================================================================= #
  def set_licence(i)
    set(:licence, i)
  end; alias set_license set_licence # === set_license

  # ========================================================================= #
  # === set_git_url
  # ========================================================================= #
  def set_git_url(i)
    set(:git_url, i)
  end

  # ========================================================================= #
  # === set_description
  #
  # Note that we will always STORE the full description - it is only the
  # actual display that will be truncated.
  # ========================================================================= #
  def set_description(i)
    set(:description, i)
  end

  # ========================================================================= #
  # === set_pkgconfig_files
  #
  # Do note that this entry must be kept as an Array at all times.
  # ========================================================================= #
  def set_pkgconfig_files(i)
    unless i.is_a? Array
      i = [i].compact
    end
    set(:pkgconfig_files, i)
  end; alias pkgconfig_files= set_pkgconfig_files # === pkgconfig_files?

  # ========================================================================= #
  # === do_not_symlink?
  #
  # Reader method over whether we will not symlink.
  # ========================================================================= #
  def do_not_symlink?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === set_base_dir                      (base_dir tag, base dir tag, 7 tag)
  #
  # This is (7), the base_dir.
  #
  # It will be equivalent to a String such as "/home/x/src/gnomemahjongg/"
  # or "/home/x/src/file/".
  #
  # It constitutes of:
  #
  #   - src_dir?
  #   - short_name?
  #   - '/'
  #
  # The short_name? will be used as name for the archive at hand,
  # but it will always be downcased - see the .downcase variant.
  #
  # If the input is :guess_base_dir then this means that the method will
  # try to guess the base directory in use.
  # ========================================================================= #
  def set_base_dir(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    #
    # This clause can be entered via:
    #
    #   set_base_dir(:default)
    #
    # Note that the base_dir entry is not allowed to have any '.'
    # characters.
    # ======================================================================= #
    when :default,
         :guess_base_dir,
         :guess
      i = "#{src_dir?}".dup
      unless base_dir?.nil?
        i << base_dir?
        i << '/' unless i.end_with?('/')
      else
        i << "#{short_name?.to_s.downcase.delete('.')}/" # This was guess_base_dir.
      end
    end
    if i
      i = i.dup if i.frozen?
      # ===================================================================== #
      # Assume that a path starting with ./ is never correct.
      # ===================================================================== #
      if i.start_with? './'
        i[0,2] = source_dir?
      end
      # ===================================================================== #
      # === Handle + tokens
      #
      # Some directory names may include a '+' character, such as is the
      # case with gtk+. There are several ways how to deal with this, but
      # if we have a certain configuration option set, then we will
      # replace '+' with long name, which is 'plus'. I personally no 
      # longer use this - my directory names will also include a '+'
      # character as a consequence.
      # ===================================================================== #
      if i.include? '+'
        i.gsub!(/\+/, 'plus') if replace_plus_with_long_name?
      end
      # ===================================================================== #
      # === Delete _ from the name of the directory as well
      #
      # The following code was added as of January 2014.
      #
      # While programs can have names such as 'e_dbus', the directory
      # name itself is not allowed to contain any '_' token. It
      # looks visually odd otherwise.
      # ===================================================================== #
      i.delete!('_') if i.include? '_'
      # ===================================================================== #
      # We don't want any '-' in the base_dir name either, so away with them.
      # ===================================================================== #
      i.delete!('-') if i.include? '-'
      # ===================================================================== #
      # === Append /
      #
      # It is the correct way to ensure that the last char is a /. The
      # reason is: directories contain a '/' as part of their name.
      # ===================================================================== #
      i << '/' unless i.end_with? '/'
    end
    # ======================================================================= #
    # Remove double slashes
    # ======================================================================= #
    i = rds(i)
    dataset?[:base_dir] = i
  end; alias try_to_guess_the_base_dir set_base_dir # === try_to_guess_the_base_dir
       alias determine_base_directory  set_base_dir # === determine_base_directory
       alias determine_base_dir        set_base_dir # === determine_base_dir
       alias set_base_dir=             set_base_dir # === set_base_dir=
       alias base_dir=                 set_base_dir # === base_dir=

  # ========================================================================= #
  # === set_modify_the_makefile
  # ========================================================================= #
  def set_modify_the_makefile(i)
    set(:modify_the_makefile, i)
  end

  # ========================================================================= #
  # === set_seek_this_program
  #
  # This method should be used to designate which program is to be
  # sought.
  #
  # The method is allowed to modify the given input.
  #
  # The search-string should always be a String.
  #
  # Keep in mind that the input to this method may also be in the form
  # of a String such as "php.yml". In that case, we have to get rid
  # of the trailing '.yml' part.
  #
  # Additionally, do note that the input could be a binary name or a
  # library name. In these two cases, we may overrule the input provided
  # but this can be configured via some settings.
  #
  # Since as of September 2012, aliases can overrule this setting.
  # This, however had, may lead to problems, such as "make" suddenly
  # becoming "makeinfo" rather than being kept as make.
  # ========================================================================= #
  def set_seek_this_program(
      i = :default
    )
    i = i.keys.first if i.is_a? Hash
    if i.is_a? Array
      # ===================================================================== #
      # Flatten the Array first, before using only the first entry.
      # ===================================================================== #
      i = i.flatten.first
    end
    case i
    when nil, ''
      i = DEFAULT_ENTRY
    end
    i = i.to_s.dup
    set_original_unmodified_name(i)
    # ======================================================================= #
    # Clean-up the input a bit next.
    # ======================================================================= #
    if    i.end_with? '.yml'
      i.delete_suffix!('.yml')
    elsif i.end_with? '.pc'
      i.delete_suffix!('.pc')
    end
    i.downcase! # Work on a reference copy that is also downcased.
    i.delete!('-') if i.include? '-'
    i.delete!('_') if i.include? '_'
    # ======================================================================= #
    # Neither '-' nor '_' are allowed as input past this point.
    # ======================================================================= #
    if i.include? '.'
      i.sub!(/#{File.extname(i)}/,'')
    end
    # ======================================================================= #
    # === Chop off trailing '/' next
    #
    # Next, this method may chop off trailing '/' characters.
    #
    # A trailing '/' is assumed to have been a typo, since it evidently
    # can not be the name of an existing program. Nobody will call a
    # file or input program something such as 'libgweather/'.
    # ======================================================================= #
    i.chop! if i.end_with? '/'
    all_binaries  = all_binaries?
    all_libraries = all_libraries?
    i = File.basename(i) if i.include? '/'
    # ======================================================================= #
    # === Handle possible binary names next
    #
    # Here we will check for binary names of programs, but only if the input
    # has not yet been found as regular name of a registered program at
    # hand. As this is not always needed anyway, we will use various checks
    # to make sure that we really want to find a binary name.
    #
    # In order to test this, try:
    #
    #   cookbooks2 xmlif
    #   ry xmlif
    #
    # ======================================================================= #
    if !does_include?(i) and
       check_for_binary_names? and
       all_binaries.has_key?(i) and
       (i != all_binaries[i]) and
       (!i.empty?)
      new_name = all_binaries[i]
      report_binary_or_library_program_name(i, new_name, :binary)
      i = new_name
    # ======================================================================= #
    # === Check for library names
    #
    # Similar reasoning applies to this entry point, as to the one above
    # for binary names.
    # ======================================================================= #
    elsif !does_include?(i) and check_for_library_names? and
       all_libraries.has_key?(i) and (i != all_libraries[i])
      new_name = all_libraries[i]
      report_binary_or_library_program_name(i, new_name, :library)
      i = new_name
    end
    # ======================================================================= #
    # Query whether the program is included or whether it is not. If
    # it is not included, but is included if 'lib' is put before it,
    # then we will use that as name instead.
    # ======================================================================= #
    unless does_include?(i)
      if does_include?("lib#{i}")
        i.prepend('lib')
      end
    end
    if (does_include?(i, :do_not_check_for_aliases) == false) and
       does_include?(i)
      # ===================================================================== #
      # In this case we will assume it is an alias, such as 'ecm'.
      # ===================================================================== #
      available_aliases = YAML.load_file(RBT.file_cookbook_aliases)
      i = available_aliases[i]
      opne "An alias was found. It will be used now. (Name: #{steelblue(i)})."
    end
    # ======================================================================= #
    # === Next check for entries that lack a leading 'lib' entry
    #
    # If they do, and a program with the same name can be found,
    # it will be used as input instead.
    #
    # An example for this would be the input "x11" which would
    # become "libx11" due to the following code.
    # ======================================================================= #
    unless does_include?(i, :do_not_check_for_aliases)
      if does_include?("lib#{i}", :do_not_check_for_aliases)
        opne "The given input #{sfancy(i)} is not included "\
             "but #{sfancy('lib'+i)} is."
        opne 'Thus using this as the new input next.'
        i.prepend('lib')
      end
      # ===================================================================== #
      # === Check for Abbreviations if the key was not found
      # ===================================================================== #
      unless i.empty?
        # =================================================================== #
        # Ok, we did not find anything, so check for abbreviations.
        # =================================================================== #
        available_programs = available_programs?
        require 'abbrev' unless Object.const_defined? :Abbrev
        abbrevs = Abbrev.abbrev(available_programs)
        if abbrevs.has_key? i
          # ================================================================= #
          # Ok, this means that our key is included.
          # ================================================================= #
          i = abbrevs.fetch(i)
        end
      end
    end
    if is_this_program_included? i
      the_program_was_found
    end
    @internal_hash[:seek_this_program] = i
  end; alias seek_this_program           set_seek_this_program # === seek_this_program
       alias seek                        set_seek_this_program # === seek
       alias set_search_term             set_seek_this_program # === set_search_term
       alias set_search_for_this_program set_seek_this_program # === set_search_for_this_program

  # ========================================================================= #
  # === set_use_this_connective_token
  # ========================================================================= #
  def set_use_this_connective_token(i = '-')
    @internal_hash[:use_this_connective_token] = i
  end; alias use_default_connective_token_again set_use_this_connective_token # === use_default_connective_token_again
       alias set_separator_token                set_use_this_connective_token # === set_separator_token

  # ========================================================================= #
  # === is_sanitize_cookbook?
  #
  # More elegant way to state that this class is indeed SanitizeCookbook.
  # ========================================================================= #
  def is_sanitize_cookbook?
    true
  end

  # ========================================================================= #
  # === set_use_this_build_system
  #
  # This method will set the build-system that should be used for
  # the given program at hand. By default we will infer automatically,
  # with the default being set to the Symbol called :infer_automatically.
  # ========================================================================= #
  def set_use_this_build_system(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = :infer_automatically
    end
    set(:use_this_build_system, i)
  end; alias set_build_system set_use_this_build_system # === set_build_system

  # ========================================================================= #
  # === set_homepage
  # ========================================================================= #
  def set_homepage(i = nil)
    set(:homepage, i)
  end

  # ========================================================================= #
  # === set_meson_configure_options
  # ========================================================================= #
  def set_meson_configure_options(i)
    set(:meson_configure_options, i)
  end; alias meson_configure_options= set_meson_configure_options # === meson_configure_options=

  # ========================================================================= #
  # === set_cmake_configure_options
  # ========================================================================= #
  def set_cmake_configure_options(i)
    set(:cmake_configure_options, i)
  end; alias cmake_configure_options= set_cmake_configure_options # === cmake_configure_options=

  # ========================================================================= #
  # === set_original_unmodified_name
  # ========================================================================= #
  def set_original_unmodified_name(i)
    i = i.dup if i # We work on a copy, though.
    @internal_hash[:original_unmodified_name] = i
  end

  # ========================================================================= #
  # === set_program_path                                              (8 tag)
  #
  # This is (8) program_path, which may look like this:
  #
  #   '/home/x/src/gnomemahjongg/gnome-mahjonng-5.08.tar.xz'
  #   '/home/x/src/file/file-5.08.tar.xz'
  #
  # So essentially, the program_path is composed of these two elements:
  #
  #    - base_dir?
  #    - program_full_name?
  #
  # In other words, this combines (7) + (6), the two main components,
  # or shown one after the other:
  #
  #   (7) base_dir,          such as: "/home/x/src/file/"
  #   (6) program_full_name, such as: "file-5.08.tar.xz" (so
  #       this is program_name_and_program_version?+archive_type?)
  #
  # It is the most important entry, containing everything - which is
  # the major goal for the class here.
  # ========================================================================= #
  def set_program_path(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = base_dir?.to_s+
          program_full_name?
    # ======================================================================= #
    # === :try_to_rebuild_it_from_smaller_components
    #
    # This is similar to :default but will rebuild from slightly
    # smaller components.
    # ======================================================================= #
    when :try_to_rebuild_it_from_smaller_components
      archive_type = archive_type?
      if archive_type and archive_type.empty?
        archive_type = '.tar.xz'
      end
      i = base_dir?+
          program_name_and_program_version?+
          archive_type
    end
    set(:program_path, i)
  end; alias program_path= set_program_path # === program_path=

  # ========================================================================= #
  # === set_postinstall
  # ========================================================================= #
  def set_postinstall(i)
    set(:postinstall, i)
  end; alias set_postinstallation set_postinstall # === set_postinstallation
       alias postinstallation=    set_postinstall # === postinstallation=

  # ========================================================================= #
  # === set_archive_type                                 (archive tag, 4 tag)
  #
  # This method will set the 'archive_type' of a given program at hand.
  #
  # It is entry number (5), and may typically look like '.tar.xz', in
  # a program like 'gnome-mahjonng-5.08.tar.xz'.
  #
  # Typically, archives will be kept in the .tar.xz format locally, or
  # rather whatever the value of the DEFAULT_ARCHIVE_TYPE specifies.
  #
  # We need this part when we wish to assemble the program_full_name
  # setting lateron, at entry number (6), which would then look
  # like this: 'gnome-mahjonng-5.08.tar.xz'
  # ========================================================================= #
  def set_archive_type(
      i = DEFAULT_ARCHIVE_TYPE
    )
    case i
    when :infer_from_url1
      i = File.basename(url1)
    when nil,
         :default,
         'default'
      i = DEFAULT_ARCHIVE_TYPE # Safeguard.
    when :tar_xz
      i = TARXZ
    end
    i = i.to_s.dup.downcase # Keep it downcased as well, and in String format.
    # ======================================================================= #
    # === Get rid of '/'
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    # ======================================================================= #
    # Do some sanitizing next.
    # ======================================================================= #
    i.sub!(/\.source$/,'') if i.include? '.source'
    # ======================================================================= #
    # If the input includes a '-' token, then something can not be right.
    # ======================================================================= #
    if i.include? '-'
      i = i[(i.rindex('-')+1)..-1]
    # ======================================================================= #
    # Same for input containing '_'.
    # ======================================================================= #
    elsif i.include? '_'
      i = i[(i.rindex('_')+1)..-1]
    end
    if i == 'guess'
      i = File.basename(url?)
    end
    i = archive_type_of?(i) # This method must have registered all archive-types.
    set(:archive_type, i)
  end; alias set_archive        set_archive_type # === set_archive
       alias set_archive_format set_archive_type # === set_archive_format
       alias set_this_archive   set_archive_type # === set_this_archive
       alias archive_type=      set_archive_type # === archive_type=

  # ========================================================================= #
  # === set_active_project
  #
  # The default assumption is that all projects are actively maintained. If
  # this assumption is incorrect then it has to be registered in the
  # particular .yml file, via an entry such as: "active_project: no"
  # ========================================================================= #
  def set_active_project(i = true)
    set(:active_project, i)
  end

  # ========================================================================= #
  # === set_can_be_compiled_statically
  # ========================================================================= #
  def set_can_be_compiled_statically(i)
    set(:can_be_compiled_statically, i)
  end

  # ========================================================================= #
  # === set_pre_configure_steps
  # ========================================================================= #
  def set_pre_configure_steps(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_is_an_active_project
  # ========================================================================= #
  def set_is_an_active_project(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_configure_options_explained
  # ========================================================================= #
  def set_configure_options_explained(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_do_not_download
  # ========================================================================= #
  def set_do_not_download(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_echo_yes
  # ========================================================================= #
  def set_echo_yes(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_build_static
  # ========================================================================= #
  def set_build_static(i)
    set(:build_static, i)
  end

  # ========================================================================= #
  # === modify_the_makefile?
  #
  # This method will query whether RBT will modify the Makefile at hand
  # or whether it will not. Some Makefiles have hardcoded paths which
  # do not respect the given --prefix variant, not even DESTDIR, as
  # far as I know. Hence, for these .yml files, we need to have a way
  # to modify that Makefile and correct it.
  # ========================================================================= #
  def modify_the_makefile?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias modify_the_makefile modify_the_makefile? # === modify_the_makefile

  # ========================================================================= #
  # === run_make?
  # ========================================================================= #
  def run_make?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias run_make run_make? # === run_make

  # ========================================================================= #
  # === gir_files?
  # ========================================================================= #
  def gir_files?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias girfiles? gir_files? # === girfiles?
       alias gir_files gir_files? # === gir_files

  # ========================================================================= #
  # === url1?
  #
  # This method will "hold" the "most" important URL of a program, which is 
  # the entry called url1.
  #
  # Every entry of our cookbooks must have this entry available.
  #
  # Usage example to query the value of url1?:
  #
  #   puts RBT::Action::Cookbooks::SanitizeCookbook.new('htop').url1?
  #
  # ========================================================================= #
  def url1?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias main_url                   url1? # === main_url
       alias remote_url                 url1? # === remote_url
       alias main_url?                  url1? # === main_url?
       alias url1                       url1? # === url1
       alias url?                       url1? # === url?
       alias remote_url?                url1? # === remote_url?
       alias url_to_the_source_archive? url1? # === url_to_the_source_archive?

  # ========================================================================= #
  # === symlink_glib_schema?
  # ========================================================================= #
  def symlink_glib_schema?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === use_autoconf?
  # ========================================================================= #
  def use_autoconf?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === set_env_variables?
  #
  # Query the environment variables that this cookbook may set.
  # ========================================================================= #
  def set_env_variables?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias env_variables? set_env_variables? # === env_variables?
     # alias set_env_variables set_env_variables? # === set_env_variables
     # ^^^ This alias would interfere with the set_env_variables() reader method.

  # ========================================================================= #
  # === build_static?
  # ========================================================================= #
  def build_static?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias build_static build_static? # === build_static

  # ========================================================================= #
  # === has_to_be_compiled?
  # ========================================================================= #
  def has_to_be_compiled?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias has_to_be_compiled                    has_to_be_compiled? # === has_to_be_compiled
       alias does_the_program_have_to_be_compiled? has_to_be_compiled? # === does_the_program_have_to_be_compiled?

  # ========================================================================= #
  # === configure_base_dir?
  # ========================================================================= #
  def configure_base_dir?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias configure_base_directory? configure_base_dir? # === configure_base_directory?

  # ========================================================================= #
  # === configure_command_to_use?
  #
  # The specific configure command that we will use.
  # ========================================================================= #
  def configure_command_to_use?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias configure_command  configure_command_to_use? # === configure_command
       alias configure_command? configure_command_to_use? # === configure_command?

  # ========================================================================= #
  # === extra_information?
  # ========================================================================= #
  def extra_information?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === program_path?
  #
  # This method will return the 'program_path' value, through the reader
  # method obtain().
  #
  # The program_path variable has the FULL path to the archive in
  # question, for instance:
  #
  #   /home/x/src/gnomemahjonng/gnome-mahjonng-5.08.tar.xz
  #   /home/x/src/gpm/gpm-1.20.1.tar.bz2
  #   /home/x/src/gambas3/gambas3-3.10.0.tar.xz
  #   base_dir? ^^^^^^^^^ program_full_name? ^^^
  #
  # It consists of base_dir?+program_full_name?.
  #
  # Do note that the program_path value is the most important entry,
  # as ALL the other information can be derived from the information
  # stored in this variable alone.
  # ========================================================================= #
  def program_path?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias local_url?          program_path? # === local_url?
       alias full_path           program_path? # === full_path
       alias full_path?          program_path? # === full_path?
       alias location?           program_path? # === location?
       alias can_be_found_where? program_path? # === can_be_found_where?
       alias stored_where?       program_path? # === stored_where?
       alias program_path        program_path? # === program_path
       alias path?               program_path? # === path?
       alias path                program_path? # === path
       alias ppath?              program_path? # === ppath?
       alias full_program_path?  program_path? # === full_program_path?
       alias local_path?         program_path? # === local_path?
       alias local_path          program_path? # === local_path

  # ========================================================================= #
  # === svn_url?
  # ========================================================================= #
  def svn_url?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === last_update?
  #
  # The last_update tag keeps track over as to when that particular program
  # was last updated.
  # ========================================================================= #
  def last_update?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias last_update last_update? # === last_update

  # ========================================================================= #
  # === m4_files?
  # ========================================================================= #
  def m4_files?
    obtain(:m4_files)
  end; alias m4_files m4_files? # === m4_files

  # ========================================================================= #
  # === use_this_program_name_on_gobolinux?
  # ========================================================================= #
  def use_this_program_name_on_gobolinux?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === use_autogen?
  # ========================================================================= #
  def use_autogen?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias use_autogen use_autogen? # === use_autogen

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias postinstallation? postinstall? # === postinstallation?

  # ========================================================================= #
  # === description?
  #
  # In the past raw_description? was guaranteed to return the unmodified
  # description, including newlines.
  #
  # During the rewrite in April 2021, this no longer seemed necessary,
  # as the sanitization-logic was changed.
  # ========================================================================= #
  def description?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias raw_description? description? # === raw_description?
       alias raw_description  description? # === raw_description

  # ========================================================================= #
  # === pre_make_commands?
  # ========================================================================= #
  def pre_make_commands?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias pre_make_commands pre_make_commands? # === pre_make_commands

  # ========================================================================= #
  # === use_this_program_name
  # ========================================================================= #
  def use_this_program_name
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === git_url?
  # ========================================================================= #
  def git_url?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias giturl?  git_url? # === giturl?
       alias git_URL? git_url? # === git_URL?

  # ========================================================================= #
  # === pre_configure_steps?
  # ========================================================================= #
  def pre_configure_steps?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === pkgconfig_files?
  # ========================================================================= #
  def pkgconfig_files?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias pkgconfig_files pkgconfig_files? # === pkgconfig_files

  # ========================================================================= #
  # === extract_to?
  #
  # This must return a String.
  # ========================================================================= #
  def extract_to?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias will_be_extracted_towards? extract_to? # === will_be_extracted_towards?
       # alias extracted_towards?       extract_to? # === extracted_towards?

  # ========================================================================= #
  # === is_an_active_project?
  # ========================================================================= #
  def is_an_active_project?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias active_project  is_an_active_project? # === active_project
       alias active_project? is_an_active_project? # === active_project?

  # ========================================================================= #
  # === archive_type?
  #
  # Simply query the archive type here. The archive_type is something
  # such as ".tar.gz" or ".tar.bz2".
  # ========================================================================= #
  def archive_type?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias archive?      archive_type? # === archive?
       alias archive_type  archive_type? # === archive_type
       alias archive_name? archive_type? # === archive_name?

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias binaries binaries? # === binaries
       alias bin?     binaries? # === bin?

  # ========================================================================= #
  # === headers?
  # ========================================================================= #
  def headers?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias headers headers? # === headers

  # ========================================================================= #
  # === n_headers?
  # ========================================================================= #
  def n_headers?
    headers?.size
  end

  # ========================================================================= #
  # === base_dir?
  #
  # This is entry (5), a simple query method. It will give us back the
  # base directory of our program in question, such as '/home/x/src/htop/'.
  # ========================================================================= #
  def base_dir?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias base_dir base_dir? # === base_dir

  # ========================================================================= #
  # === replace_plus_with_long_name?
  # ========================================================================= #
  def replace_plus_with_long_name?
    @internal_hash[:replace_plus_with_long_name]
  end

  # ========================================================================= #
  # === mirror?
  # ========================================================================= #
  def mirror?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === do_not_download?
  # ========================================================================= #
  def do_not_download?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === apply_patch?
  #
  # Query-method as to whether we shall apply a patch or whether we
  # shall not.
  # ========================================================================= #
  def apply_patch?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === ignore_errors?
  # ========================================================================= #
  def ignore_errors?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === use_this_build_system?
  #
  # This method will query as to which build-system will be used for
  # the given program at hand.
  # ========================================================================= #
  def use_this_build_system?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias use_this_build_system    use_this_build_system? # === use_this_build_system
       alias use_which_build_system?  use_this_build_system? # === use_which_build_system?
       alias build_system?            use_this_build_system? # === build_system?
       alias build_system_to_be_used? use_this_build_system? # === build_system_to_be_used?
       alias build_system_in_use?     use_this_build_system? # === build_system_in_use?

  # ========================================================================= #
  # === cmake_configure_options?
  # ========================================================================= #
  def cmake_configure_options?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias cmake_configure_options cmake_configure_options? # === cmake_configure_options
       alias cmake_options?          cmake_configure_options? # === cmake_options?

  # ========================================================================= #
  # === use_this_build_directory?
  # ========================================================================= #
  def use_this_build_directory?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias build_dir?                   use_this_build_directory? # === build_dir?
       alias use_this_build_directory     use_this_build_directory? # === use_this_build_directory
       alias name_of_the_build_directory? use_this_build_directory? # === name_of_the_build_directory?

  # ========================================================================= #
  # === set_enable_shared
  # ========================================================================= #
  def set_enable_shared(i)
    set(to_bool(__method__.to_s.delete_prefix('set_')), i)
  end

  # ========================================================================= #
  # === set_use_build_directory
  # ========================================================================= #
  def set_use_build_directory(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias use_build_directory= set_use_build_directory # === use_build_directory=

  # ========================================================================= #
  # === set_do_not_symlink
  #
  # The alias "do_not_symlink" is an emplicit set_do_not_symlink()
  # call with the argument true. If this is changed, we would also
  # have to create a new method to specifically enable the behaviour
  # desired by do_not_symlink().
  # ========================================================================= #
  def set_do_not_symlink(i = true)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias do_not_symlink set_do_not_symlink # === do_not_symlink

  # ========================================================================= #
  # === set_libexec
  # ========================================================================= #
  def set_libexec(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias libexec= set_libexec # === libexec=

  # ========================================================================= #
  # === set_localstatedir
  # ========================================================================= #
  def set_localstatedir(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_m4_files
  # ========================================================================= #
  def set_m4_files(i)
    unless i.is_a? Array
      i = [i].compact
    end
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias m4_files= set_m4_files # === m4_files?

  # ========================================================================= #
  # === set_flatpak_url
  # ========================================================================= #
  def set_flatpak_url(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias flatpak_url= set_flatpak_url # === flatpak_url=

  # ========================================================================= #
  # === set_mirror
  # ========================================================================= #
  def set_mirror(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_symlink_glib_schema
  # ========================================================================= #
  def set_symlink_glib_schema(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_may_we_modify_the_configure_options
  # ========================================================================= #
  def set_may_we_modify_the_configure_options(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_use_autoconf
  # ========================================================================= #
  def set_use_autoconf(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_svn_url
  # ========================================================================= #
  def set_svn_url(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_wikipedia
  # ========================================================================= #
  def set_wikipedia(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end

  # ========================================================================= #
  # === set_programs_dir
  #
  # This method can be used to designate another directory for the programs
  # to be stored, other than '/Programs/'.
  # ========================================================================= #
  def set_programs_dir(
      i = '/Programs/'
    )
    @internal_hash[:programs_dir] = i
  end; alias programs_dir= set_programs_dir # === programs_dir=

  # ========================================================================= #
  # === set_extract_to
  #
  # Consistently use this method when you wish to use the extract_to
  # variable.
  #
  # This variable must ALWAYS keep the location of where to extract a
  # package to lateron.
  #
  # Values set in the corresponding .yml file are able to overrule this.
  # ========================================================================= #
  def set_extract_to(
      i = :use_extract_to_or_the_default_rbt_log_directory_otherwise
    )
    case i
    # ======================================================================= #
    # === :use_extract_to_or_the_default_rbt_log_directory_otherwise
    # ======================================================================= #
    when :use_extract_to_or_the_default_rbt_log_directory_otherwise
      if extract_to?.to_s.empty?
        i = rbt_log_directory?
      else
        i = extract_to? # This is for e. g. "linux.yml" which has an extract_to: entry.
      end
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         nil
      i = rbt_log_directory?
    end
    i = i.dup # We don't want to keep a tainted reference.
    # ======================================================================= #
    # Ensure that we have a proper default value if the input was ''.
    # ======================================================================= #
    i = extract_to? if i.to_s.empty?
    # ======================================================================= #
    # Try to get an ENV variable here, if ENV is part of @extract_to.
    # ======================================================================= #
    if i.include? 'ENV'
      match = i.gsub(/ENV\['/,'').gsub(/'\]/,'') # contains the ENV variable
      i = ENV[match].dup
    end
    # ======================================================================= #
    # append / if the last character is not '/'
    # ======================================================================= #
    i << '/' unless i.end_with? '/'
    set(:extract_to, i)
  end; alias extract_to=                        set_extract_to # === extract_to=
       alias sync_extract_to_onto_the_main_hash set_extract_to # === sync_extract_to_onto_the_main_hash

  # ========================================================================= #
  # === set_env_variables
  # ========================================================================= #
  def set_env_variables(i)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias set_env_variable set_env_variables # === set_env_variable

  # ========================================================================= #
  # === set_program_name                                     (entry number 2)
  #
  # This is entry number (2), the setter for the program name.
  #
  # The program name of a program might be something such as this:
  #
  #   gnome-mahjonng
  #   file
  #
  # A '-' is usually required as separator for a proper "program_name"
  # entry, about at the least in 99.5% of the programs out there in
  # the wild. They usually include a '-' as part of the program name.
  #
  # Note that there are some weird exceptions, such as
  # "boost_1_59_0".
  #
  # Consistently use this and ONLY this method when modifying the
  # 'program_name' entry in the main dataset.
  #
  # Some other programs require to have a '_' as part of the program name.
  #
  # This can lead to problems such as for those programs that have a _ as
  # part of their name. A good example is the name "e_dbus", which
  # includes includes a '_' token as part of its name.
  #
  # In Jan 2014, I realized that eliminating '_' is too much of a hassle,
  # so I disabled it in this method. I will retain the explanation above
  # though, in the event that future programmers - or future me - may
  # encounter similar problems.
  # ========================================================================= #
  def set_program_name(
      i = :default
    )
    if i.is_a? Array
      i = i.first
    end
    case i
    when :default
      i = return_sanitize_url1(url1?.to_s.dup)
    when 'URL_REPLACE_UNDERSCORE'
      i = url1?.to_s.dup
    when :assemble, :guess # Then we assemble it for these two conditions.
      i = assemble_program_name # In these cases, we will try to guess the name.
    when 'FIRST_UNDERSCORE_IS_HYPHEN'
      e 'UNHANDLED - Reonsider.'
      exit
    end
    i = i.to_s.dup
    i = remove_archive_at_the_end(i)
    # ======================================================================= #
    # A program name never has a '/'.
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    i = remove_file_extension(i) if i.include? '.' # ← Added this as of Nov 2013.
    set(:program_name, i)
  end; alias set_program   set_program_name # === set_program
       alias program_name= set_program_name # === program_name=

  # ========================================================================= #
  # === set_binaries
  # ========================================================================= #
  def set_binaries(i)
    set(:binaries, i)
  end

  # ========================================================================= #
  # === set_libraries
  # ========================================================================= #
  def set_libraries(i)
    set(:libraries, i)
  end

  # ========================================================================= #
  # === set_tags
  # ========================================================================= #
  def set_tags(i)
    set(:tags, i)
  end

  # ========================================================================= #
  # === set_configure_base_dir
  # ========================================================================= #
  def set_configure_base_dir(i)
    set(:configure_base_dir, i)
  end

  # ========================================================================= #
  # === set_has_to_be_compiled
  # ========================================================================= #
  def set_has_to_be_compiled(i = true)
    set(:has_to_be_compiled, i)
  end

  # ========================================================================= #
  # === set_blfs
  # ========================================================================= #
  def set_blfs(i)
    set(:blfs, i)
  end

  # ========================================================================= #
  # === set_skip_configure
  # ========================================================================= #
  def set_skip_configure(i)
    set_run_configure(!i)
  end

  # ========================================================================= #
  # === set_run_configure
  # ========================================================================= #
  def set_run_configure(i)
    set(:run_configure, i)
  end

  # ========================================================================= #
  # === set_autosymlink_lib64
  # ========================================================================= #
  def set_autosymlink_lib64(i)
    set(:autosymlink_lib64, i)
  end; alias set_autosymlink_lib set_autosymlink_lib64 # === set_autosymlink_lib

  # ========================================================================= #
  # === set_use_this_make_install_command
  # ========================================================================= #
  def set_use_this_make_install_command(i)
    set(:use_this_make_install_command, i)
  end; alias use_this_make_install_command= set_use_this_make_install_command # === use_this_make_install_command=

  # ========================================================================= #
  # === set_run_make
  # ========================================================================= #
  def set_run_make(i)
    set(:run_make, i)
  end

  # ========================================================================= #
  # === set_use_this_make_command
  # ========================================================================= #
  def set_use_this_make_command(i)
    set(:use_this_make_command, i)
  end; alias use_this_make_command= set_use_this_make_command # === use_this_make_command=

  # ========================================================================= #
  # === set_sed
  # ========================================================================= #
  def set_sed(i)
    set(:sed, i)
  end; alias sed= set_sed # === sed=

  # ========================================================================= #
  # === set_symlink_headers
  #
  # If we want to symlink the headers, this is the setter-method that should
  # be called.
  # ========================================================================= #
  def set_symlink_headers(i = true)
    set(:symlink_headers, i)
  end; alias symlink_headers= set_symlink_headers # === symlink_headers=

  # ========================================================================= #
  # === set_use_glib_schema
  # ========================================================================= #
  def set_use_glib_schema(i = nil)
    if i
      set(:use_glib_schema, i)
    end
  end; alias use_glib_schema= set_use_glib_schema # === use_glib_schema=

  # ========================================================================= #
  # === set_archive_size
  #
  # The archive size, that is, how large the archive is - in bytes.
  #
  # The archive size will default to 0 on startup or when no argument
  # is passed to it.
  # ========================================================================= #
  def set_archive_size(i = 0)
    set(:archive_size, i)
  end; alias set_size set_archive_size # === set_size

  # ========================================================================= #
  # === set_apply_patch
  # ========================================================================= #
  def set_apply_patch(i = nil)
    set(:apply_patch, i)
  end

  # ========================================================================= #
  # === set_program_full_name
  #
  # This is entry (6) program_full_name, a combination of the two
  # other values called program_name_and_program_version?+archive_type?.
  #
  # The program_full_name variable thus looks like this:
  #
  #   gnome-mahjonng-5.08.tar.xz
  #
  # It consists of program_name_and_program_version?+archive_type?
  # ========================================================================= #
  def set_program_full_name(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = "#{program_name_and_program_version?}"\
          "#{archive_type?}"
    end
    set(:program_full_name, i)
  end; alias build_program_full_name set_program_full_name # === build_program_full_name

  # ========================================================================= #
  # === set_required_deps_on
  #
  # Set the required dependencies for a program through this method.
  #
  # Note that this method will not do any sanitizing - you have to sanitize
  # the input on your own prior to calling this method.
  # ========================================================================= #
  def set_required_deps_on(i)
    set(:required_deps_on, i)
  end

  # ========================================================================= #
  # === set_configure_options
  # ========================================================================= #
  def set_configure_options(i)
    # ======================================================================= #
    # Since as of September 2017, the input is no longer allowed to have
    # any newlines. This is explicitely mentioned here in the event that
    # we may have to revert this decision one day.
    # ======================================================================= #
    if i and i.include?(N)
      i = i.dup if i.frozen?
      i.delete!(N)
      i.squeeze!(' ') # And also get rid of '  ' variants.
    end
    set(:configure_options, i)
  end; alias configure_options= set_configure_options # === configure_options=

  # ========================================================================= #
  # === set_compiler_to_use
  #
  # Do note that this method is normally not used by default; it primarily
  # exists so that downstream code in RBT::Compile can keep track of
  # which compiler is used, e. g. :gcc or :clang and similar variants.
  # ========================================================================= #
  def set_compiler_to_use(i = :gcc)
    set(:compiler_to_use, i)
  end; alias compiler_to_use= set_compiler_to_use # === compiler_to_use=

  # ========================================================================= #
  # === set_parameters_to_make
  # ========================================================================= #
  def set_parameters_to_make(i)
    set(:parameters_to_make, i)
  end; alias set_make_options set_parameters_to_make # === set_make_options

  # ========================================================================= #
  # === set_make_use_of_aliases_kept_in_the_file_cookbook_aliases
  # ========================================================================= #
  def set_make_use_of_aliases_kept_in_the_file_cookbook_aliases(i)
    case i
    # ======================================================================= #
    # === :nope
    # ======================================================================= #
    when :nope,
         :user_disabled_this_setting_via_the_commandline
      i = false
    end
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases] = i
  end

  # ========================================================================= #
  # === set_autoconf
  # ========================================================================= #
  def set_autoconf(i)
    set(:use_autoconf, i)
  end; alias use_autoconf= set_autoconf # === use_autoconf=

  # ========================================================================= #
  # === set_program_compile_name
  # ========================================================================= #
  def set_program_compile_name(i)
    set(:program_compile_name, i)
  end

  # ========================================================================= #
  # === set_pre_make_commands
  # ========================================================================= #
  def set_pre_make_commands(i)
    set(:pre_make_commands, i)
  end

  # ========================================================================= #
  # === set_md5sum
  # ========================================================================= #
  def set_md5sum(i)
    set(:md5sum, i)
  end

  # ========================================================================= #
  # === set_keep_extracted
  # ========================================================================= #
  def set_keep_extracted(i)
    set(:keep_extracted, i)
  end; alias keep_extracted= set_keep_extracted # === keep_extracted=

  # ========================================================================= #
  # === set_short_description
  #
  # The short_description entry should not end with a newline.
  # ========================================================================= #
  def set_short_description(i = nil)
    i.chomp! if i # We do not want trailing newlines.
    set(:short_description, i)
  end; alias set_short_desc set_short_description # === set_short_desc

  # ========================================================================= #
  # === set_extra_information
  # ========================================================================= #
  def set_extra_information(i)
    set(:extra_information, i)
  end

  # ========================================================================= #
  # === set_internal_commandline_arguments
  # ========================================================================= #
  def set_internal_commandline_arguments(
      i = ARGV
    )
    # ======================================================================= #
    # === :internal_commandline_arguments
    #
    # Keep track of the commandline arguments in the main Hash of this
    # class.
    # ======================================================================= #
    @internal_hash[:internal_commandline_arguments] = [i].flatten.compact
  end

  # ========================================================================= #
  # === report
  #
  # This is just a general-purpose method that will output some values.
  #
  # It will output useful information about a program.
  #
  # For a full report system, use the specialized class
  # DisplayCookbookDataset.
  # ========================================================================= #
  def report
    program_name = seek_which_program?
    return if program_name.to_s.empty?
    array_show_these_entries?.clear
    append_to_array_show_these_entries(:mode_in_use)
    # ======================================================================= #
    # === short_description
    # ======================================================================= #
    unless short_description?.empty?
      append_to_array_show_these_entries(:short_description)
    end if short_description?
    # ======================================================================= #
    # Add a bunch of symbols to the main array here.
    # ======================================================================= #
    append_to_array_show_these_entries(%i( description ))
    unless extra_information?.empty?
      append_to_array_show_these_entries(:extra_information)
    end if extra_information?
    append_to_array_show_these_entries(%i(
      program_path
      base_dir
      program_full_name
      program_name_and_program_version
      program_version
      archive_type
      short_name
      program_name
      url1
      url2
    ))
    # ======================================================================= #
    # === Show the main homepage if it exists
    # ======================================================================= #
    if homepage?
      append_to_array_show_these_entries(:homepage)
    end
    if flatpak_url? and !flatpak_url?.empty?
      append_to_array_show_these_entries(:flatpak_url)
    end
    # ======================================================================= #
    # Unless the pkgconfig_files Array is empty, we will display the
    # .pc files of the given program.
    # ======================================================================= #
    if pkgconfig_files?
      unless pkgconfig_files?.empty?
        append_to_array_show_these_entries(:pkgconfig_files)
      end
    end
    # ======================================================================= #
    # === Display installation_steps if they are not empty
    # ======================================================================= #
    if installation_steps? and !installation_steps?.empty?
      append_to_array_show_these_entries(:installation_steps)
    end
    # ======================================================================= #
    # === Display the configure command to use next
    # ======================================================================= #
    if configure_command_to_use?
      unless configure_command_to_use?.empty?
        unless configure_command_to_use? == 'configure'
          append_to_array_show_these_entries(:configure_command_to_use)
        end
      end
    end
    # ======================================================================= #
    # === Add the .gir-files, if they exist
    # ======================================================================= #
    if gir_files?
      unless gir_files?.empty?
        append_to_array_show_these_entries(:gir_files)
      end
    end
    if program_compile_name? and !program_compile_name?.empty?
      append_to_array_show_these_entries(:program_compile_name)
    end
    # ======================================================================= #
    # === Add the headers, if they exist
    # ======================================================================= #
    if headers?
      unless headers?.empty?
        append_to_array_show_these_entries(:headers)
      end
    end
    if binaries?
      append_to_array_show_these_entries(:binaries) unless binaries?.empty?
    end
    if libraries?
      append_to_array_show_these_entries(:libraries) unless libraries?.empty?
    end
    append_to_array_show_these_entries(%i( prefix ))
    if configure_base_dir? and !configure_base_dir?.to_s.empty?
      append_to_array_show_these_entries(:configure_base_dir)
    end
    append_to_array_show_these_entries(
      %i(
        extract_to
        configure_options
        meson_configure_options
        md5sum
        file_size
        keep_extracted
      )
    )
    if build_static? == true
      append_to_array_show_these_entries(:build_static)
    end
    unless use_this_make_command?.to_s == 'make'
      append_to_array_show_these_entries(:use_this_make_command)
    end
    if sed? and sed?.is_a?(Array) and !sed?.empty?
      append_to_array_show_these_entries(:sed)
    end
    append_to_array_show_these_entries(:do_not_download) if do_not_download?
    append_to_array_show_these_entries(:apply_patch)     if apply_patch?
    if deps?
      append_to_array_show_these_entries(:required_deps_on) unless deps?.empty?
    end
    if make_options?
      unless make_options?.empty? # Only show the make_options if they are not empty.
        append_to_array_show_these_entries(:make_options)
      end
    end
    append_to_array_show_these_entries(:enable_shared) # if enable_shared?
    if preinstall?
      append_to_array_show_these_entries(:preinstall) unless preinstall?.empty?
    end
    if postinstall?
      append_to_array_show_these_entries(:postinstall) unless postinstall?.empty?
    end
    append_to_array_show_these_entries(:tags)                unless tags?.empty?
    append_to_array_show_these_entries(:svn_url)             if svn_url? and !svn_url?.empty?
    append_to_array_show_these_entries(:git_url)             if git_url? and !git_url?.empty?
    append_to_array_show_these_entries(:pre_make_commands)   unless pre_make_commands?.empty?
    append_to_array_show_these_entries(:use_build_directory) if use_build_directory?
    append_to_array_show_these_entries(:manual_steps)        if manual_steps? and !manual_steps?.empty?
    append_to_array_show_these_entries(:licence) # ← Mandatory entry.
    # ======================================================================= #
    # === use_this_build_system will always be shown
    # ======================================================================= #
    append_to_array_show_these_entries(:use_this_build_system) # ← Mandatory entry.
    # ======================================================================= #
    # === Use autogen will always be shown.
    # ======================================================================= #
    append_to_array_show_these_entries(:use_autogen)
    append_to_array_show_these_entries(:has_to_be_compiled)
    append_to_array_show_these_entries(:last_update)
    append_to_array_show_these_entries(:do_not_symlink) if do_not_symlink?
    append_to_array_show_these_entries(:md5sum)         if show_md5sum?
    append_to_array_show_these_entries(:use_this_make_install_command) if use_this_make_install_command?
    # ======================================================================= #
    # Show whether we will run "configure" or whether we will not:
    # ======================================================================= #
    append_to_array_show_these_entries(:run_configure)  if run_configure?
    # ======================================================================= #
    # Display the BLFS entry fairly close to the bottom.
    # ======================================================================= #
    append_to_array_show_these_entries(:blfs)           unless blfs?.empty?
    # ======================================================================= #
    # Next, display it with a coloured "border".
    # ======================================================================= #
    use_this_colour_or_no_colour = :lightblue
    unless use_colours?
      use_this_colour_or_no_colour = :do_not_use_colours
    end
    array_show_these_entries?.flatten!
    array_display_these_entries = array_show_these_entries?
    cliner(use_this_colour_or_no_colour) {
      e "#{rev}Feedbacking all about the program "\
        "`#{mediumslateblue(program_name)}#{rev}`, "\
        "#{rev}taken from the file"
      e "`#{sfile(which_file?)}#{rev}`."
      e
      # ===================================================================== #
      # Next, use a dedicated method to display all the individual
      # components:
      # ===================================================================== #
      show_this_dataset_containing_which_entries_should_be_displayed(
        array_display_these_entries
      )
      # ===================================================================== #
      # Show important URLs next.
      # ===================================================================== #
      if urls?.first
        e; e '  Important URLs:'
        _ = urls?.map {|entry| entry.to_s.chomp }.reject {|x| x.empty?}
        if _
          e; _.each_with_index {|line, index|
            e "#{grey('    (')}#{seagreen(index+1).to_s}#{grey(') ')}#{grey(line)}"
          }; e
        end
      end
    }
    e
  end; alias feedback                   report # === feedback
       alias feedback_colourized_result report # === report (report tag)
       alias print_hash                 report # === print_hash
       alias display_all                report # === display_all
       alias full_report                report # === full_report

  # ========================================================================= #
  # === show_this_dataset_containing_which_entries_should_be_displayed (display tag)
  # ========================================================================= #
  def show_this_dataset_containing_which_entries_should_be_displayed(
      i = array_show_these_entries?
    )
    # ======================================================================= #
    # Determine the threshold value to use for this display.
    # ======================================================================= #
    threshold_value = 68
    left_pad = '  '
    ljust_value = 43
    i.each {|this_entry|
      case this_entry # case tag
      # ===================================================================== #
      # === extra_information
      # ===================================================================== #
      when :extra_information
        # =================================================================== #
        # === Handle extra_information
        #
        # Next, limit the size for extra_description. We do not want to show
        # too much here.
        # =================================================================== #
        result = send_this_method_and_sanitize_arrays("#{this_entry}?").to_s
        if self.class.honour_threshold_value?
          if (result.size > threshold_value)
            result = result[0, result.index("\n")].delete('"')+' [...]'
          end
        end
        result.chomp!
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkslateblue(result.lstrip)
      # ===================================================================== #
      # === description
      #
      # Handle the description entry here.
      # ===================================================================== #
      when :description
        result = description?.chomp.rstrip
        if (result.size > threshold_value)
          result = result.tr("\n",' ')[0..(threshold_value-2)]+' [...]'
        end
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          limegreen(result.lstrip)+rev
      # ===================================================================== #
      # === program_version
      #
      # Feedback the program-version at hand.
      # ===================================================================== #
      when :program_version
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          springgreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === :use_this_build_system
      # ===================================================================== #
      when :use_this_build_system
        result = send_this_method_and_sanitize_arrays("#{this_entry}?").to_s.dup
        result.prepend(':') if result == 'infer_automatically'
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumpurple(result)
      # ===================================================================== #
      # === homepage
      #
      # Colourize the homepage properly.
      # ===================================================================== #
      when :homepage
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkseagreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === flatpak_url
      # ===================================================================== #
      when :flatpak_url
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumseagreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)+rev
      # ===================================================================== #
      # === .gir files
      #
      # This entry point can be used to display the .gir files, if they
      # exist.
      # ===================================================================== #
      when :gir_files
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkseagreen(
            send_this_method_and_sanitize_arrays("#{this_entry}?")
          )
      # ===================================================================== #
      # === extract_to
      # ===================================================================== #
      when :extract_to
        result = send_this_method_and_sanitize_arrays("#{this_entry}?").to_s
        unless result.empty?
          e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
            cornflowerblue(result)
        end
      # ===================================================================== #
      # === program_name
      # ===================================================================== #
      when :program_name
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumslateblue(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === blfs
      # ===================================================================== #
      when :blfs
        result = send_this_method_and_sanitize_arrays("#{this_entry}?")
        result = result.flatten.first if result.is_a? Array
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkseagreen(result.to_s.delete('[]')) # .to_s to avoid nil values.
      # ===================================================================== #
      # === short_name
      #
      # Is steelblue.
      # ===================================================================== #
      when :short_name
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          steelblue(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === use_autogen
      # ===================================================================== #
      when :use_autogen
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          cadetblue(
            yes_or_no(
              send_this_method_and_sanitize_arrays("#{this_entry}?").to_s
            )
          )
      # ===================================================================== #
      # === headers
      #
      # This entry point can be used to display the headers.
      # ===================================================================== #
      when :headers
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkseagreen(
            send_this_method_and_sanitize_arrays(this_entry.to_s+'?', 5) # ← Limit to 5 by default.
          )
      # ===================================================================== #
      # === binaries
      #
      # This entry point can be used to display the binaries.
      # ===================================================================== #
      when :binaries
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(
            send_this_method_and_sanitize_arrays(this_entry.to_s+'?', 5) # ← Limit to 5 by default.
          )
      # ===================================================================== #
      # === libraries
      #
      # This entry point can be used to display the libraries.
      # ===================================================================== #
      when :libraries
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(
            send_this_method_and_sanitize_arrays(this_entry.to_s+'?', 5) # ← Limit to 5 by default.
          )
      # ===================================================================== #
      # === sed
      # ===================================================================== #
      when :sed
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumturquoise(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === short_description
      # ===================================================================== #
      when :short_description
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumturquoise(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === run_configure
      # ===================================================================== #
      when :run_configure
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          olivedrab(
            yes_or_no(send_this_method_and_sanitize_arrays("#{this_entry}?")).to_s
          )
      # ===================================================================== #
      # === has_to_be_compiled
      # ===================================================================== #
      when :has_to_be_compiled
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          steelblue(
            yes_or_no(send_this_method_and_sanitize_arrays("#{this_entry}?")).to_s
          )
      # ===================================================================== #
      # === skip_configure
      # ===================================================================== #
      when :skip_configure
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumorchid(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === archive_size
      # ===================================================================== #
      when :archive_size,
           :file_size,
           :program_size # colourize program_size.
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumorchid(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === program_compile_name
      # ===================================================================== #
      when :program_compile_name
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          mediumslateblue(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === program_path
      # ===================================================================== #
      when :program_path
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          orange(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === configure_options
      # ===================================================================== #
      when :configure_options
        unless configure_options?.empty?
          e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
            mediumseagreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)+rev
        end
      # ===================================================================== #
      # === meson_configure_options
      # ===================================================================== #
      when :meson_configure_options
        if meson_configure_options?
          unless meson_configure_options.empty?
            e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
              mediumseagreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)+rev
          end
        end
      # ===================================================================== #
      # === do_not_symlink
      # ===================================================================== #
      when :do_not_symlink
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          lightslategray(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === keep_extracted
      # ===================================================================== #
      when :keep_extracted
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
         seagreen(
           yes_or_no(
             send_this_method_and_sanitize_arrays("#{this_entry}?").to_s
           )
         )
      # ===================================================================== #
      # === prefix
      # ===================================================================== #
      when :prefix
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          darkseagreen(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === url1
      # ===================================================================== #
      when :url1,:url2,:url3,:url4,:url5
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sandybrown(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === pkgconfig_files
      # ===================================================================== #
      when :pkgconfig_files
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          deepskyblue(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === md5sum
      # ===================================================================== #
      when :md5sum
        if show_md5sum?
          e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
            sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?"))
        end
      # ===================================================================== #
      # === remote_url
      # ===================================================================== #
      when :remote_url
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === program_full_name
      # ===================================================================== #
      when :program_full_name
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === base_dir
      # ===================================================================== #
      when :base_dir
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === archive_type
      # ===================================================================== #
      when :archive_type
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?"))
      # ===================================================================== #
      # === last_update
      # ===================================================================== #
      when :last_update
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          cornflowerblue(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      # ===================================================================== #
      # === mode
      # ===================================================================== #
      when :mode, :mode_in_use
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          cornflowerblue(send_this_method_and_sanitize_arrays("#{this_entry}?").to_s)
      else # else tag
           # ================================================================ #
           # Note that sfancy() is the default colour used here for the
           # else clause.
           # ================================================================ #
        e "#{rev}#{left_pad}#{this_entry}: ".ljust(ljust_value)+
          sfancy(send_this_method_and_sanitize_arrays("#{this_entry}?")).to_s
      end
    }
  end; alias show_this_array show_this_dataset_containing_which_entries_should_be_displayed # === show_this_array

  # ========================================================================= #
  # === find_this_program                                          (find tag)
  #
  # This method will also keep track which program we are trying to
  # find.
  #
  # Note that .find() is an alias to this method.
  # ========================================================================= #
  def find_this_program(
      i = first?
    )
    determine_which_program_is_sought(i)               # (1) seek this new program
    if may_we_instantiate_a_new_raw_cookbook?
      instantiate_raw_cookbook(i)                      # (2) finally, instantiate a new raw cookbook
    end
    populate_the_dataset_variable                      # (4) initialize the main hash properly
    if raw_cookbook?
      if in_simple_mode?                               # (5) in this case, sync a lot of stuff.
        # =================================================================== #
        # We use this method call because the program we are trying to find
        # may have been modified, such as from "x11" to "libx11".
        # =================================================================== #
        find_in_the_raw_cookbook(seek_this_program?)
        ensure_that_this_hash_has_only_valid_keys(
          raw_cookbook?.dataset?
        )
        # =================================================================== #
        # :archive_type can be set early because we can scan for the real
        # value lateron anyway.
        # =================================================================== #
        dataset?[:archive_type] = '.tar.xz'              # (6) archive_type
        # =================================================================== #
        # In this case we have to do additional parsing. However had, when
        # the .yml file contains a set-entry called program_name_and_program_version
        # then this entry will be used rather than using url1.
        # =================================================================== #
        _ = program_name_and_program_version?
        if _ and _.is_a?(String) and !_.empty?
          _ = ::RBT.sanitize_url1_entry(_, dataset?.merge(url1: raw_cookbook?.url1?)).dup
          # ================================================================= #
          # In this case program_name_and_program_version exists, so we will
          # set these parts first.
          # ================================================================= #
          set_program_name_and_program_version(_)
          deduce_everything_from_the_program_name_and_program_version_downwards
          set_program_full_name(:default)
          set_short_name(:default)
          set_base_dir(:default)
          set_program_path(:default)
          set_program_version(:infer_from_the_program_name_and_program_version)
        else
          parse_url1_and_set_all_associated_values
        end
        check_for_outdated_entries(dataset?)
        sanitize_everything                              # (7) sanitize everything
        determine_all_relevant_values_for_the_dataset_variable
      else
        # =================================================================== #
        # else we are in complex-mode.
        # =================================================================== #
        copy_the_data_from_the_raw_cookbook_onto_the_main_hash
      end
    end
  end; alias assign_and_run                                               find_this_program # === assign_and_run
       alias assign                                                       find_this_program # === assign
       alias initialize_yaml_dataset                                      find_this_program # === initialize_yaml_dataset
       alias update_with_this_program                                     find_this_program # === update_with_this_program
       alias update                                                       find_this_program # === update
       alias query                                                        find_this_program # === query
       alias set_dataset                                                  find_this_program # === set_dataset
       alias find                                                         find_this_program # === find
       alias find_this_program_and_then_set_the_internal_cookbook_dataset find_this_program # === find_this_program_and_then_set_the_internal_cookbook_dataset

  # ========================================================================= #
  # === copy_the_data_from_the_raw_cookbook_onto_the_main_hash
  # ========================================================================= #
  def copy_the_data_from_the_raw_cookbook_onto_the_main_hash
    hash = dataset?
    raw_cookbook_dataset?.each_pair {|key, value|
      hash[key] = value # Only copy, do not modify anything.
    }
  end

  # ========================================================================= #
  # === find_in_the_raw_cookbook
  # ========================================================================= #
  def find_in_the_raw_cookbook(
      i            = search_for_this_program?,
      raw_cookbook = raw_cookbook?
    )
    if debug?
      opne "#{rev}Loading the dataset from the file "\
           "`#{sfile(raw_cookbook?.file?)}#{rev}`."
    end
    if raw_cookbook
      result = raw_cookbook.find(i)
      return result
    end
  end; alias load_yaml_dataset_from_this_file find_in_the_raw_cookbook # === load_yaml_dataset_from_this_file

  # ========================================================================= #
  # === @honour_threshold_value
  # ========================================================================= #
  @honour_threshold_value = true

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.do_not_honour_threshold_value
  # ========================================================================= #
  def self.do_not_honour_threshold_value
    @honour_threshold_value = false
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.honour_threshold_value?
  # ========================================================================= #
  def self.honour_threshold_value?
    @honour_threshold_value
  end

  # ========================================================================= #
  # === populate_the_dataset_variable
  # ========================================================================= #
  def populate_the_dataset_variable(
      i = simple_or_complex?
    )
    initialize_the_main_hash_with_default_values # First, fill the dataset with default values.
    case i
    # ======================================================================= #
    # === :simple
    #
    # If we make use of the simple dataset then we do not have to sync the
    # value back onto RawCookbook.
    # ======================================================================= #
    when :simple
      # ===================================================================== #
      # In this case we have to do sanitizing stuff.
      # ===================================================================== #
      merge_in_the_dataset_derived_from_the_raw_cookbook # must come after we instantiated the raw cookbook
    # ======================================================================= #
    # === :complex
    #
    # In this case we can simply query from the raw-cookbook dataset.
    # ======================================================================= #
    when :complex
      raw_cookbook?.set_complex # Pass the info to the raw-cookbook as well.
      allowed_cookbook_entries?.each {|this_entry|
        _ = ("#{this_entry.to_s}?").to_sym
        if raw_cookbook?.respond_to? _
          dataset?[this_entry.to_sym] = raw_cookbook?.send(_)
        else
          # e '  Notice: the object does not respond to '+steelblue(_)+'.'
        end
      }
      determine_all_relevant_values_for_the_dataset_variable
    end
  end

  # ========================================================================= #
  # === array_show_these_entries?
  # ========================================================================= #
  def array_show_these_entries?
    @internal_hash[:array_show_these_entries]
  end

  # ========================================================================= #
  # === append_to_array_show_these_entries
  #
  # Use this method when you wish to append to the main Array keeping
  # track of which entries are to be shown.
  # ========================================================================= #
  def append_to_array_show_these_entries(i)
    array_show_these_entries? << i
  end

  # ========================================================================= #
  # === find_this_program_and_then_set_the_internal_cookbook_dataset_then_determine_everything
  # ========================================================================= #
  def find_this_program_and_then_set_the_internal_cookbook_dataset_then_determine_everything(
      compile_this_program = first_argument?
    )
    if does_include?(compile_this_program)
      the_program_was_found # The program was found - so continue.
      find_this_program_and_then_set_the_internal_cookbook_dataset(
        compile_this_program
      )
      determine_everything
    else
      this_program_was_not_found(compile_this_program)
    end
  end; alias metafind  find_this_program_and_then_set_the_internal_cookbook_dataset_then_determine_everything # === metafind
       alias meta_find find_this_program_and_then_set_the_internal_cookbook_dataset_then_determine_everything # === meta_find

  # ========================================================================= #
  # === parse_url1_and_set_all_associated_values
  #
  # We will make various changes depending on the url1 entry.
  #
  # If this entry exists, various data points will be set on the main
  # dataset of this class.
  # ========================================================================= #
  def parse_url1_and_set_all_associated_values(
      url1 = raw_cookbook?.url1?
    )
    # ======================================================================= #
    # url1 may look like this:
    #
    #  https://www.python.org/ftp/python/3.9.4/Python-3.9.4.tgz
    #
    # ======================================================================= #
    if url1
      # ===================================================================== #
      # Fix .js files.
      # ===================================================================== #
      if url1.end_with? '.js'
        set_archive('.js')
      elsif url1.end_with? '.gem'
        set_archive(url1)
      end
      if url1.include? '%2B' # This means '+'.
        url1.gsub!(/%2B/,'+')
      end
      if url1.include? '-trunk'
        url1 = remove_file_extension(url1)
      end
      if url1.include? '-release.tar'
        url1 = chop_off_archive(url1)
        url1.sub!(/-release/,'')
      end
      # ===================================================================== #
      # === Handle github URLs next
      #
      # Next, check if it is a github url and if it may not have a
      # "-" and if it consists only of numbers; in this case we know
      # we have to determine the program name in a different manner.
      # ===================================================================== #
      if is_github_url?(url1) and
         !File.basename(url1).include?('-')  and
         (url1 =~ /\d{0,2}\.\d{0,2}\.?\d{0,2}/)
         # ================================================================== #
         # In this case we will re-write this URL; and also modify the
         # variable called url1.
         #
         # The current content of url1 may be a String such as:
         #
         #   'https://github.com/opencv/opencv/archive/3.4.5.tar.gz'
         #
         # ================================================================== #
         use_this_regex =
           /\/v?(\d{0,2}\.\d{0,2}\.?\d{0,2})\./ # See: https://rubular.com/r/dvCdVJWbtUhJaJ
         # ================================================================== #
         # We must use the unmodified input-name here because that name may
         # still include '-' characters, which may be important for some
         # programs. Unfortunately this is not always available, such as
         # for:
         #
         #   https://github.com/kdave/btrfs-progs/archive/refs/tags/v5.11.1.tar.gz
         #
         # Thus, we will default to original_unmodified_name?, but we
         # will try to determine whether the "/"-splitted third part
         # contains a '-' character. If so then this name is used
         # instead. The explanation can be seen by the text
         # given above.
         # ================================================================== #
         replacement_name = original_unmodified_name?.to_s.dup
         if url1.include? '/'
           splitted = url1.split('/')
           if splitted[4] and splitted[4].include?('-')
             replacement_name = splitted[4]
           end
         end
         url1.sub!(
           use_this_regex,
           '/'+replacement_name+'-\1.'
         )
      end
    end
    # ======================================================================= #
    # Note that url1 may also be invalid, such as by having this as
    # content:
    #
    #   cmake@3.22.4
    #
    # In this case we will simply rewrite this towards cmake-3.22.4.
    # ======================================================================= #
    if url1 and url1.include?('@')
      url1.tr!('@','-')
    end
    if url1
      url1 = remove_archive_at_the_end(File.basename(url1))
    end
    program_information = ProgramInformation.new(url1)
    set_short_name(program_information.short_name?)                                   # (1) short_name, such as 'gnomemahjongg'
    set_program_name(program_information.program_name?)                               # (2) program_name, such as 'gnome-mahjongg'
    set_program_version(program_information.program_version?)                         # (3) program_version
    set_program_name_and_program_version("#{program_name?}-#{program_version?}")      # (4) program_name_and_program_version
    set_program_full_name(program_name_and_program_version?+archive_type?)            # (6) program_full_name, such as 'gnome-mahjonng-5.08.tar.xz'
    set_base_dir(:default)                                                            # (7) base_dir
    _ = base_dir?.to_s+
        program_name?.to_s+'-'+
        program_version?.to_s+
        archive_type?.to_s
    set_program_path(_)                                                               # (8) program_path
    unless File.exist? _
      try_to_find_another_local_file(_)
      set_program_path
    end
  end; alias guess_entries_based_on_the_first_url parse_url1_and_set_all_associated_values # === guess_entries_based_on_the_first_url

  # ========================================================================= #
  # === reassemble_the_program_path
  #
  # This method will always ultimate attempt to set the program path, if
  # possible.
  # ========================================================================= #
  def reassemble_the_program_path(
      i = return_default_program_path
    )
    assemble_the_default_program_name_and_program_version # Assign to something such as: "gnome-mahjongg-0.1.3"
    set_program_path(i)
  end; alias assemble_program_path   reassemble_the_program_path # === assemble_program_path
       alias reassemble_program_path reassemble_the_program_path # === reassemble_program_path

  # ========================================================================= #
  # === initialize_the_main_hash_with_default_values
  #
  # This method is necessary so that our main hash contains all
  # necessary values initially.
  # ========================================================================= #
  def initialize_the_main_hash_with_default_values
    reset_the_dataset # This will make @internal_hash[:dataset] an empty Hash.
    # ======================================================================= #
    # Add the emptry-entries to the dataset next:
    # ======================================================================= #
    add_the_empty_entries_to_the_dataset_variable
    add_the_nil_entries_to_the_dataset_variable
    add_the_array_entries_to_the_dataset_variable
    add_the_true_entries_to_the_dataset_variable
    add_the_false_entries_to_the_dataset_variable
    # ======================================================================= #
    # Last but not least, add the remaining entries to the dataset variable.
    # ======================================================================= #
    add_the_remaining_entries_to_the_dataset_variable
  end; alias populate_the_dataset_variable_with_default_values initialize_the_main_hash_with_default_values # === populate_the_dataset_variable_with_default_values

  # ========================================================================= #
  # === report_binary_or_library_program_name
  #
  # Only call this method when we have found an alias.
  # ========================================================================= #
  def report_binary_or_library_program_name(
      i,
      new_program_name,
      binary_or_library = 'binary'
    )
    if i and !i.empty?
      case binary_or_library
      # ===================================================================== #
      # === :binary
      # ===================================================================== #
      when :binary
        binary_or_library = 'binary'
      # ===================================================================== #
      # === :library
      # ===================================================================== #
      when :library
        binary_or_library = 'library'
      end
      opne '`'+simp(i)+rev+'` was found as name of a '+
           simp(binary_or_library)+rev+' file.'
      opne 'We will now assume that you want to get this '\
           'whole package instead and will'
      opne 'thus set the name to this program now, which belongs to '\
           'the package `'+sfancy(new_program_name)+rev+'`.'+N
    end
  end

  # ========================================================================= #
  # === be_complex
  # ========================================================================= #
  def be_complex
    @internal_hash[:simple_or_complex] = :complex
  end

  # ========================================================================= #
  # === clear_yaml_dataset
  # ========================================================================= #
  def clear_yaml_dataset
    cookbook_dataset?.clear
  end

  # ========================================================================= #
  # === do_extract
  #
  # You can use this method, if you have class Extracter available,
  # to force class Extracter to extract this program.
  # ========================================================================= #
  def do_extract(
      i = program_path?
    )
    begin
      require 'extracter' unless Object.const_defined? :Extracter
    rescue LoadError; end
    if Object.const_defined? :Extracter
      # ======================================================================= #
      # Ok, extracter is available so we can continue.
      # ======================================================================= #
      Extracter.new(i?).extracted_path? # Specify the path.
    else
      opne 'class Extracter is not available. Consider installing it via:'
      e
      opne orange('  gem install extracter')
      e
    end
  end

  # ========================================================================= #
  # === assume_default_archive_type
  # ========================================================================= #
  def assume_default_archive_type(
      i = DEFAULT_ARCHIVE_TYPE
    )
    set_archive_type(i)
  end

  # ========================================================================= #
  # === feedback_version
  #
  # This will feedback the program-version.
  # ========================================================================= #
  def feedback_version(shall_we_exit = true)
    opne "The current RBT-Version is: #{RBT.version?}"
    exit if shall_we_exit
  end

  # ========================================================================= #
  # === determine_the_program_name_and_the_program_version_from_url1
  #
  # Do note that this method will respect program_name_and_program_version
  # entries.
  # ========================================================================= #
  def determine_the_program_name_and_the_program_version_from_url1(
      url1 = url1?
    )
    i = remove_archive_from_the_end(
      File.basename(url1)
    ).dup
    _ = program_name_and_program_version?
    if _ and _.is_a?(String) and !_.empty?
      # ===================================================================== #
      # This clause is entered when the .yml file at hand has both an
      # url1 entry but also a specific program_name_and_program_version
      # entry. In this case we simply grab the
      # program_name_and_program_version entry instead.
      # ===================================================================== #
      i = _
    end
    if i.include?(token?)
      completely_infer_the_program_name_and_the_program_version_from_this_input(i)
    elsif i.include?(opposite_token?)
      set_separator_token(opposite_token?)
      completely_infer_the_program_name_and_the_program_version_from_this_input(i)
    else
      if is_a_github_url? and (i =~ /^v\d+/)
        require 'rbt/toplevel_methods/return_program_name_and_program_version_from_this_github_url.rb'
        i = RBT.return_program_name_and_program_version_from_this_github_url(url1?)
      end
      # ===================================================================== #
      # Alright - we have not found a separator token, such as '-' or
      # '_'. What to do in such a case?
      # One strategy is to notify the user about it, but this does not
      # really resolve the issue at hand. Since as of 27.05.2020
      # I decided to "fake-simulate" the proper program name and program
      # version based on ProgramInformation. That also means that the
      # old notification was disabled, but it will be kept in the source
      # code here, in the event that we may wish to re-attach it one day.
      # ===================================================================== #
      completely_infer_the_program_name_and_the_program_version_from_this_input(
        ProgramInformation.return_name(i)+
        '-'+
        ProgramInformation.return_version(i)
      )
      # opne 'No separator token found: '+steelblue(token?.to_s)
      # opne tomato('The URL was: ')+steelblue(url1)
    end
  end; alias determine_the_program_name_and_the_program_version_from_url1_while_respecting_existing_values determine_the_program_name_and_the_program_version_from_url1 # === determine_the_program_name_and_the_program_version_from_url1_while_respecting_existing_values

  # ========================================================================= #
  # === assemble_the_default_program_name_and_program_version
  #
  # This method will assemble the program_name (6) variable, which
  # is derived directly from:
  #
  #  program_name? and '-' and program_version?
  #
  # For example it could look like this:
  #
  #   "gnome-mahjongg-5.08"
  #
  # ========================================================================= #
  def assemble_the_default_program_name_and_program_version(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = program_name?+
          token?+
          program_version?
    end
    set_program_name_and_program_version(i)
  end; alias assemble_program_name                   assemble_the_default_program_name_and_program_version # === assemble_program_name
       alias update_program_name                     assemble_the_default_program_name_and_program_version # === update_program_name
       alias attach_program_name_and_program_version assemble_the_default_program_name_and_program_version # === attach_program_name_and_program_version
       alias assemble_default_program_name           assemble_the_default_program_name_and_program_version # === assemble_default_program_name

  # ========================================================================= #
  # === rebuild_program_full_name
  # ========================================================================= #
  def rebuild_program_full_name
    set_program_full_name(:default)
  end; alias update_program_full_name rebuild_program_full_name # === update_program_full_name

  # ========================================================================= #
  # === update_appdir_prefix
  # ========================================================================= #
  def update_appdir_prefix
    set_prefix(:appdir)
  end

  # ========================================================================= #
  # === clear_configure_options
  # ========================================================================= #
  def clear_configure_options
    set_configure_options nil
  end; alias clear clear_configure_options # === clear

  # ========================================================================= #
  # === this_program_was_not_found
  # ========================================================================= #
  def this_program_was_not_found(i)
    unless i.to_s.empty?
      orev "The program #{steelblue(i.to_s)} #{rev}does "\
           "#{tomato('not')} #{rev}appear to be registered."
    end
  end

  # ========================================================================= #
  # === quiet_down
  #
  # This method will cause the cookbook to be silent and not report
  # how many programs are registered.
  # ========================================================================= #
  def quiet_down
    set_be_silent # This is defined in the parent class.
    @internal_hash[:report_how_many_programs_are_registered] = false
  end

  # ========================================================================= #
  # === feedback_available_cookbooks
  #
  # Feedback which cookbooks are available.
  # ========================================================================= #
  def feedback_available_cookbooks(i = available_programs?)
    e 'These cookbooks are available:'; print '  '
    e i.join(', ')
  end

  # ========================================================================= #
  # === feedback_description
  #
  # Use this method if you wish to feedback the description of a given
  # program at hand.
  # ========================================================================= #
  def feedback_description
    e description?
  end

  # ========================================================================= #
  # === determine_which_program_is_sought
  # ========================================================================= #
  def determine_which_program_is_sought(
      _ = first_argument?
    )
    set_seek_this_program(_)
  end

  # ========================================================================= #
  # === set_sha256
  # ========================================================================= #
  def set_sha256(i)
    set(:sha256, i)
  end

  # ========================================================================= #
  # === preinstall?
  # ========================================================================= #
  def preinstall?
    obtain(__method__.to_s.delete('?').to_sym)
  end

  # ========================================================================= #
  # === sha256?
  # ========================================================================= #
  def sha256?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias sha256sum? sha256? # === sha256sum?

  # ========================================================================= #
  # === set_symlink_pkgconfig_files
  # ========================================================================= #
  def set_symlink_pkgconfig_files(i = true)
    set(__method__.to_s.delete_prefix('set_').to_sym, i)
  end; alias symlink_pkgconfig_files= set_symlink_pkgconfig_files # === symlink_pkgconfig_files=

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias symlink_pkgconfig_files symlink_pkgconfig_files? # === symlink_pkgconfig_files

  # ========================================================================= #
  # === add_the_remaining_entries_to_the_dataset_variable
  #
  # This will add the remaining entries into the main dataset.
  #
  # These entries will default to an empty String. If you need another
  # default for any of the missing entry then you should modify the
  # corresponding value in the file 'constants.rb' and specify to which
  # group it belongs.
  # ========================================================================= #
  def add_the_remaining_entries_to_the_dataset_variable
    registered_cookbook_entries?.each {|key|
      dataset?[key.to_sym] unless has_key? key.to_sym
    }
  end

  # ========================================================================= #
  # === add_the_false_entries_to_the_dataset_variable             (false tag)
  #
  # Here we will set all false values into our main dataset.
  #
  # We do this only if we do not yet have such a value in our dataset
  # though.
  # ========================================================================= #  
  def add_the_false_entries_to_the_dataset_variable(i = ARRAY_FALSE_VALUES)
    # ======================================================================= #
    # Set the false values to false next.
    # ======================================================================= #
    i.each {|this_false_value|
      if has_key? this_false_value
        # =================================================================== #
        # If that key already exists in the dataset then we have to ensure
        # that it will be a boolean value.
        # =================================================================== #
        sanitize_boolean_value(this_false_value)
      else
        set(this_false_value, false)
      end
    }
  end

  # ========================================================================= #
  # === add_the_true_entries_to_the_dataset_variable
  # ========================================================================= #
  def add_the_true_entries_to_the_dataset_variable(
      i = ARRAY_TRUE_VALUES
    )
    # ======================================================================= #
    # Set the false values to false next.
    # ======================================================================= #
    i.each {|this_false_value|
      if has_key? this_false_value
        # =================================================================== #
        # If that key already exists in the dataset then we have to ensure
        # that it will be a boolean value.
        # =================================================================== #
        sanitize_then_set_boolean_value(this_false_value)
      else
        set(this_false_value, true)
      end
    }
  end

  # ========================================================================= #
  # === symlink_headers?
  # ========================================================================= #
  def symlink_headers?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias symlink_headers               symlink_headers? # === symlink_headers
       alias shall_we_symlink_the_headers? symlink_headers? # === shall_we_symlink_the_headers?

  # ========================================================================= #
  # === add_the_empty_entries_to_the_dataset_variable             (empty tag)
  #
  # We must ensure that the empty values are also properly set to ""
  # on startup, in a mandatory fashion.
  # ========================================================================= #
  def add_the_empty_entries_to_the_dataset_variable(
      i = ARRAY_EMPTY_VALUES
    )
    i.each {|entry|
      # ===================================================================== #
      # Store the keys as a Symbol.
      # ===================================================================== #
      dataset?[entry.to_sym] = ''
    }
  end

  # ========================================================================= #
  # === add_the_array_entries_to_the_dataset_variable             (array tag)
  #
  # This method will ensure that all specified Array entries are part
  # of the internal yaml dataset when a new class is instantiated.
  # ========================================================================= #
  def add_the_array_entries_to_the_dataset_variable(
      i = ARRAY_ARRAY_ENTRIES
    )
    i.each {|entry|
      set(entry.to_sym, [])
    }
  end

  # ========================================================================= #
  # === add_the_nil_entries_to_the_dataset_variable                 (nil tag)
  #
  # This will set entries, such as "homepage", to nil initially.
  # ========================================================================= #
  def add_the_nil_entries_to_the_dataset_variable(
      i = ARRAY_NIL_ENTRIES
    )
    i.each {|entry|
      dataset?[entry.to_sym] = nil
    }
  end

  # ========================================================================= #
  # === update_kde_applications
  # ========================================================================= #
  def update_kde_applications
    unless RBT::Cookbooks.const_defined? :UpdateKdeApplications
      require 'rbt/check_for_updates/update_kde_applications.rb'
    end
    UpdateKdeApplications.new
  end

  begin
    require 'md5sum' # Used in this file here.
  rescue LoadError; end
  require 'rbt/toplevel_methods/md5sum.rb'
  # ========================================================================= #
  # === md5sum?
  #
  # Conditionally use md5sum.
  #
  # This method must accomodate three different possibilities:
  #
  #   (1) We have generated a .yml file in a special directory. We
  #       will preferentially use that one, if it exists. This requires
  #       of the user to have generated these values already. If the
  #       user did so then we will use these values rather than
  #       calculate them anew again. The advantage of this way is 
  #       that it will significantly speed up the process.
  #
  #   (2) IF we do not have such a file as described in (1), then we will
  #       first try to use the ruby Md5sum class, specifically its Md5sum[]
  #       class method. Note that the ruby Md5sum class is fairly slow for
  #       large programs, which is why option (1) is the default since as
  #       of April 2016.
  #
  #   (3) The other two options (1) and (2) not being available, we will
  #       try to use the unix coreutils "md5sum" instead, which is a 
  #       binary, and should be pretty fast.
  #
  # Note that as of May 2016, we also make available a way to
  # AVOID calculating the md5sum at all. Not everyone will need
  # the md5sum information after all.
  # ========================================================================= #
  def md5sum?(
      use_this_path = program_path?
    )
    # ======================================================================= #
    # === Try to use yaml files first
    #
    # The expanded directory on my system will be here:
    #
    #   /Depot/Temp/CookbookDirectory/expanded_cookbooks/
    #
    # ======================================================================= #
    if File.directory?(RBT.base_dir_to_store_expanded_cookbooks?) and
       File.exist?(use_this_path.to_s) and
       use_this_path.end_with?('.yml')
       begin
         dataset = YAML.load_file(use_this_path)
         return dataset['md5sum'].to_s
       rescue ArgumentError => error
         opne 'An error happened for the path `'+sfile(use_this_path)+
              '`. We will exit as a consequence.'
         pp error
         exit
       end
    # ======================================================================= #
    # === Use the pure ruby way next
    # ======================================================================= #
    elsif Object.const_defined? :Md5sum
      Md5sum[use_this_path].md5sum? # Calculate it anew via Md5sum[].
    # ======================================================================= #
    # === Try to use md5sum binary directly next
    #
    # This entry point depends on a binary that is part of the
    # coreutils package.
    # ======================================================================= #
    elsif (result = `md5sum bogus 2>&1`) and
          !result.include?('command not found')
      if File.directory? use_this_path # A workaround for now.
        return '0'
      else
        # =================================================================== #
        # Delegate to the system tool here, part of coreutils.
        # =================================================================== #
        result = `md5sum #{use_this_path}`.split(' ').first
        result.strip! if result
        return result
      end
    else
      '0' # Return 0 then. But that should not ever happen.
    end
  end; alias md5sum md5sum? # === md5sum

  # ========================================================================= #
  # === try_to_find_another_local_file
  # ========================================================================= #
  def try_to_find_another_local_file(i)
    without_file_extension = remove_file_extension_from(i)
    array = action(:return_alternative_archive, without_file_extension)
    if array and !array.empty?
      first_element = array.first
      if File.exist? first_element
        set_archive(first_element)
      end
    end
  end

  # ========================================================================= #
  # === determine_program_name_and_program_version
  #
  # This method will automatically determine the program name and
  # the program version, based on two other settings in the
  # main cookbook-dataset. This means that this method will
  # ONLY succeed if these two settings have been determined
  # PRIOR to invoking this method here.
  # ========================================================================= #
  def determine_program_name_and_program_version
    set_program_name_and_program_version(
      :default
    )
  end

  # ========================================================================= #
  # === check_for_outdated_entries
  #
  # This method will check for outdated/obsolete entries and notify the user
  # about this. Tha way we can get rid of such obsolete entries.
  # ========================================================================= #
  def check_for_outdated_entries(
      _ = dataset?
    )
    use_this_key = 'github'
    if !_.has_key?(use_this_key) and _.has_key?(use_this_key.to_sym)
      use_this_key = use_this_key.to_sym
    end
    if _.has_key?(use_this_key) and
      !_[use_this_key].nil? and
      !_[use_this_key].empty?
      opne crimson('This file contains an obsolete entry called ')+
           royalblue('github')
      opne crimson('This entry could be replaced with the '\
           'entry called ')+
           steelblue('git_url')+
           crimson('.')
      opne crimson('It should be possible to get rid of this entry in ')+
           crimson('the yaml file ')
      e
      e "  #{sfile(yaml_file?)}"
      e
    end
  end

  # ========================================================================= #
  # === return_proper_type
  #
  # This will return the proper default time, such as [] or '' and so forth.
  # ========================================================================= #
  def return_proper_type_for(i = :required_deps_on)
    file = ::RBT.file_specification_of_registered_cookbook_entries
    if File.exist? file
      _ = YAML.load_file(file)
      if _.include? i.to_s
        assumed_type = _[i.to_s]
        case assumed_type
        when 'Array'
          return []
        else
          e tomato('Unhandled type: ')+
            steelblue(assumed_type.to_s)
        end
      end
    end
    return nil
  end

  # ========================================================================= #
  # === deduce_everything_from_the_program_name_and_program_version_downwards
  #
  # This method will deduce - and set - everything starting from the
  # program_name_and_program_version entry downwards.
  #
  # It will NOT set other aggregate methods, such as set_program_path().
  # The reason for this is that this method here shall work downwards
  # what it can deduce on its own, without depending on other methods.
  # ========================================================================= #
  def deduce_everything_from_the_program_name_and_program_version_downwards(
      i = program_name_and_program_version?
    )
    program_information = ProgramInformation.new(i)
    set_short_name(program_information.short_name?)
    set_program_name(program_information.program_name?)
    set_program_version(program_information.program_version?)
  end

  # ========================================================================= #
  # === sort_by_archive_priority
  #
  # This method will sort the archive format based on priorities.
  # ========================================================================= #
  def sort_by_archive_priority(i)
    array_priority = %w(
      .xz
      .bz2
      .gz
      .lz
      .gem
      .js
    )
    begin
      selection = i.select {|entry|
        entry.include?('.') and is_an_archive?(entry)
      }
      i = selection.sort_by {|entry|
        extname = File.extname(entry)
        if array_priority.include? extname
          entry = array_priority.index(extname)
        end
        entry
      }
    rescue ArgumentError => error
      opne 'An error occurred in the method '+
           slateblue(__method__.to_s)+'. (Line: '+royalblue(__LINE__)+')'
      pp i
      pp error
      exit
    end
    return i
  end

  # ========================================================================= #
  # === the_program_is_included
  #
  # This method will set-toggle that the program we are looking for,
  # is included; aka that it was found.
  # ========================================================================= #
  def the_program_is_included
    @internal_hash[:found] = true
  end; alias the_program_was_found the_program_is_included # === the_program_was_found

  # ========================================================================= #
  # === infer_the_short_name
  #
  # This will simply assume the default short name.
  # ========================================================================= #
  def infer_the_short_name
    set_short_name(:default)
  end

  # ========================================================================= #
  # === output_main_dataset
  # ========================================================================= # 
  def output_main_dataset
    do_not_use_colours
    pp yaml_dataset?
  end

  # ========================================================================= #
  # === completely_infer_the_program_name_and_the_program_version_from_this_input
  #
  # This will honour macros in program_name_and_program_version.
  # ========================================================================= #
  def completely_infer_the_program_name_and_the_program_version_from_this_input(i)
    set_program_name_and_program_version(
      return_sanitized_program_name_and_program_version(
        i
      )
    )
    set_program_name(
      ProgramInformation.return_name(program_name_and_program_version?).dup
    )
    set_program_version(
      ProgramInformation.return_version(program_name_and_program_version?).dup
    )
  end

  # ========================================================================= #
  # === enable_debug
  # ========================================================================= #
  def enable_debug
    @internal_hash[:debug] = true
  end

  # ========================================================================= #
  # === use_this_connective_token?
  # ========================================================================= #
  def use_this_connective_token?
    @internal_hash[:use_this_connective_token]
  end; alias connective_token? use_this_connective_token? # === connective_token?
       alias token?            use_this_connective_token? # === token?

  # ========================================================================= #
  # === do_use_this_program_version
  #
  # Use this method specifically to set (and use) another program version.
  #
  # The difference towards set_program_version is that we will also
  # re-assemble the default program path.
  # ========================================================================= #
  def do_use_this_program_version(i = '1.52.1')
    set_program_version(i)
    reassemble_program_path
  end; alias use_this_program_version do_use_this_program_version # === use_this_program_version
       alias do_use_this_version=     do_use_this_program_version # === do_use_this_version=

  # ========================================================================= #
  # === guess_short_name
  #
  # This method can be used to guess the short name of a program.
  # ========================================================================= #
  def guess_short_name(
      i = program_name?
    )
    _ = ::ProgramInformation.return_short_name(i)
    set_short_name(_)
  end

  # ========================================================================= #
  # === try_to_guess_the_program_version
  # ========================================================================= #
  def try_to_guess_the_program_version(
      i = program_name?
    )
    _program_name, program_version = ProgramInformation.new(i).return_name_and_version
    set_program_version(program_version)
  end

  # ========================================================================= #
  # === append_this_to_the_configure_options
  #
  # This method can be used to append something to the (stored)
  # configure options in use.
  # ========================================================================= #
  def append_this_to_the_configure_options(i = '')
    _ = configure_options?.to_s
    _ << " #{i}"
    set_configure_options(_)
  end

  # ========================================================================= #
  # === determine_the_full_program_path
  #
  # This refers to program_path, which may look like
  # '/home/x/src/gnomemahjongg/gnome-mahjonng-5.08.tar.xz'.
  #
  # In other words, it combines base_dir?+program_full_name?.
  #
  # In June 2020 this was changed, to more adjust towards the needs
  # of class RBT::Compile.
  # ========================================================================= #
  def determine_the_full_program_path
    set_program_path(:try_to_rebuild_it_from_smaller_components)
  end; alias determine_the_program_path determine_the_full_program_path # === determine_the_program_path
       alias rebuild_program_path       determine_the_full_program_path # === rebuild_program_path

  # ========================================================================= #
  # === determine_everything                                  (determine tag)
  #
  # This method will essentially determine all the other dependent
  # variables, and set them onto the main (internal) dataset - the
  # cookbook dataset.
  # ========================================================================= #
  def determine_everything
    determine_the_program_name_and_the_program_version_from_url1_while_respecting_existing_values
    determine_the_base_dir
    determine_the_archive_type
    determine_the_full_program_name
    determine_the_full_program_path
    determine_the_file_size # ← can happen only after the program path was determined.
  end

  # ========================================================================= #
  # === determine_the_archive_type
  # ========================================================================= #
  def determine_the_archive_type
    set_archive_type(
      dataset[:archive_type]
    )
  end

  # ========================================================================= #
  # === determine_the_file_size
  # ========================================================================= #
  def determine_the_file_size
    result = 0
    _ = program_path?
    if File.exist? _
      result = File.size(_)
    end
    set_archive_size(result)
  end

  # ========================================================================= #
  # === determine_the_full_program_name
  #
  # program_full_name is like 'gnome-mahjonng-5.08.tar.xz', so
  # this is program_name_and_program_version?+archive_type?
  # ========================================================================= #
  def determine_the_full_program_name
    set_program_full_name(:default)
  end

  # ========================================================================= #
  # === determine_the_base_dir
  #
  # This is like '/home/x/src/gnomemahjongg/', so combining
  # src_dir?+short_name?. We will use :default, though, as
  # argument to set_base_dir() since that already does the
  # above for us.
  # ========================================================================= #
  def determine_the_base_dir
    if short_name?.nil?
      if program_name?
        set_short_name(program_name?)
      end
    end
    set_base_dir(:default)
  end

  # ========================================================================= #
  # === update_yaml_dataset
  # ========================================================================= #
  def update_yaml_dataset(i)
    dataset?.update(i)
  end

  # ========================================================================= #
  # === is_another_archive_used_locally?
  #
  # This method depends on program_path? having been properly set BEFORE 
  # the method is invoked.
  #
  # The method will return true or false.
  # ========================================================================= #
  def is_another_archive_used_locally?(
      i = archive_type?
    )
    result = false
    program_path = program_path?
    if program_path
      unless File.exist?(program_path)
        target = base_dir?+program_name_and_program_version?+'*'
        entries = Dir[target]
        if entries.nil? or 
           entries.first.nil? or
           entries.first.include?(archive_type?)
          e 'An error happened. The target was:'
          e
          e tomato(target)
          e
          e 'The program at hand was assumed to be '+steelblue(i)+' and'
          e 'the program_path was assumed to be: '+lightgreen(program_path.to_s)
          if is_on_roebe? and !target.end_with?('-*')
            # =============================================================== #
            # This here looks like so: '/home/x/src/-*'
            # palemoon may show this error.
            # =============================================================== #
            exit
          end
        else
          result = true
        end
      end
    end
    return result
  end

  # ========================================================================= #
  # === do_change_to_appdir_layout
  #
  # You can use this method to change to the AppDir layout.
  # ========================================================================= #
  def do_change_to_appdir_layout
    sanitize_prefix(:return_appdir_path)
  end

  # ========================================================================= #
  # === substitute_macros
  #
  # Do "macro" expansion through this method.
  #
  # This method will sanitize some special variables, which are assumes
  # to be "macros" - that is, to be substituted with another value.
  #
  # Right now this is only used for kernel variable, PROGRAM_NAME,
  # PNAME, NAME, KERNEL_V and PROGRAM_VERSION but maybe someone else
  # wants to do more macro processing, so we will keep this bundled
  # here together.
  # ========================================================================= #
  def substitute_macros(input)
    if input.is_a? Array
      input.map {|entry| substitute_macros(entry) } # Recursive call.
    else
      i = input.to_s.dup # Work on a new copy here.
      if i.include? 'PROGRAM_VERSION'
        i.gsub!(/PROGRAM_VERSION/, program_version?)
      end
      if i.include? 'PNAME'
        i.gsub!(/PNAME/,           short_name?)
      elsif i.include? 'NAME'
        i.gsub!(/NAME/,            short_name?)
      elsif i.include? 'PROGRAM_NAME'
        i.gsub!(/PROGRAM_NAME/,    short_name?)
      end
      if ENV['KERNEL_V'] && i.include?('KERNEL_V')
        i.gsub!(/#{i}/, ENV['KERNEL_V'])
      end
      i
    end
  end

  # ========================================================================= #
  # === try_to_guess_the_program_path_based_on_the_program_name_and_program_version
  #
  # This will build program_path, which is:
  #
  # '/home/x/src/gnomemahjongg/gnome-mahjonng-5.08.tar.xz'
  #
  # aka base_dir?+program_full_name?
  # ========================================================================= #
  def try_to_guess_the_program_path_based_on_the_program_name_and_program_version(
      _ = base_dir?+
          program_full_name?
    )
    if File.exist? _
      set_program_path(_)
    end
  end

  # ========================================================================= #
  # === determine_the_program_name_and_the_program_version
  #
  # This method will determine:
  #
  #   (1) the "program_name"
  #   (2) the "program_version"
  #
  # setting.
  #
  # We will not use .downcase because some program names may include
  # upcased characters, such as "libX11".
  # ========================================================================= #
  def determine_the_program_name_and_the_program_version(
      i = dataset?[:url1]
    )
    if ihash?.has_key?(:program_name)
      # ===================================================================== #
      # If we already have set a program_name value, and this one includes
      # a '-' character, then we assume here that the original cookbook
      # file already contains a "foobar-1.0" entry, which we will simply
      # use next.
      # ===================================================================== #
      if ihash?[:program_name].include? '-'
        i = ihash?[:program_name]
      end
    end
    # ======================================================================= #
    # First, if the input contains a '/' then we split towards the last
    # part.
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    i = remove_archive_at_the_end(i) # Get rid of e. g. '.tar.xz'
    if i.include? '-'
      program_name    = ProgramInformation.return_name(i)
      program_version = ProgramInformation.return_version(i)
    else
      program_name    = i.dup
      program_version = DEFAULT_PROGRAM_VERSION
    end
    set_program_name(program_name)
    set_program_version(program_version)
    set_program_name_and_program_version(:default)
    set_short_name(program_name)
  end

  # ========================================================================= #
  # === decompose_program_name
  #
  # This method will set short_name and program_version, by making use of
  # the content stored in program_name? by default.
  # ========================================================================= #
  def decompose_program_name(
      i = program_name?
    )
    _ = ::ProgramInformation.return_short_name_and_program_version(i)
    set_short_name(_.short_name?, :propagate_towards_real_short_name)
    set_program_version(_.program_version?)
  end

  # ========================================================================= #
  # === ensure_that_this_hash_has_only_valid_keys
  #
  # The input to this method must be a Hash.
  #
  # This method will ensure that the given Hash has only valid keys.
  #
  # In the past, we could use it to convert it into Symbols but
  # this is no longer in use. We now make no further modifications
  # to the original dataset.
  #
  # If all keys are valid then we won't have to notify the user about
  # it at all.
  # ========================================================================= #
  def ensure_that_this_hash_has_only_valid_keys(
      i = raw_cookbook.hash?
    )
    registered_cookbook_entries = registered_cookbook_entries?
    # ======================================================================= #
    # Check whether all is fine with the raw-dataset.
    # ======================================================================= #
    if i.nil?
      cliner
      opne sfancy('nil')+' was passed into the method '+
           orange('ensure_that_this_hash_has_only_valid_keys()')
      opne steelblue('This can not be right. Program name was: ')+
           sfancy(program_name?.to_s)+'.'
      pp caller()
      pp self
      cliner
    else
      i.each_pair {|key, value|
        key = key.to_s # We will only deal with Strings past this point.
        case key
        # =================================================================== #
        # === Add some ad-hoc exceptions next
        # =================================================================== #
        when 'program_full_name','md5sum', # Ad-hoc exceptions.
             'program_name',
             'program_path',
             'program_version',
             'set_env_variables'
          next
        end
        unless registered_cookbook_entries.include?(key)
          seek_this_yaml_file = "#{seek_this_program?}"
          seek_this_yaml_file << '.yml' unless seek_this_yaml_file.end_with? '.yml'
          opne "#{swarn('Warning')}#{rev} - the cookbook-file for `"\
               "#{sfancy(seek_this_yaml_file)}#{rev}`"
          opne "(path: #{sfile(raw_cookbook?.file?)}#{rev})"
          opne "#{rev}contains a key that was #{swarn('NOT')} #{rev}"\
               "registered. The name of that"
          opne "#{rev}(invalid / ignored) key is: `#{sfancy(key.to_s)}#{rev}`."
          opne "#{rev}This is defined in the file "\
               "#{Colours.sfile('registered_cookbook_entries.yml')}"
          # ================================================================= #
          # Next, also try to find a name that is similar.
          # ================================================================= #
          if registered_cookbook_entries.any? {|entry|
              entry.include? key.to_s
            }
            select_this_entry = registered_cookbook_entries.select {|entry|
              entry.include? key.to_s
            }.first
            opne "#{rev}Did you mean this entry instead? "\
                 "#{sfancy(select_this_entry)}"  
          end
          opne 'This is currently discouraged.'
          # exit
        end
      }
    end
  end

  # ========================================================================= #
  # === chop_off_revision_number
  #
  # This method will check if a program version ends with a String such
  # as "-r8". If so then this part will be chopped off.
  #
  # Usually this trailing part denotes an internal revision rather than
  # an external program version.
  # ========================================================================= #
  def chop_off_revision_number
    _ = program_version?
    use_this_regex = /(-r\d{1,2})$/
    if _ =~ use_this_regex
      _.sub!(use_this_regex,'')
      set_program_version(_)
    end
  end

  # ========================================================================= #
  # === try_to_guess_the_archive_type
  #
  # This method will try to guess the archive type, e. g. .tar.xz or
  # .tar.gz of a given program at hand, based on which files may
  # exist locally already.
  #
  # Note that program_name may look like 'gnome-mahjongg' and
  # base_dir may look like '/home/x/src/gnomemahjongg/'.
  # ========================================================================= #
  def try_to_guess_the_archive_type
    target = "#{base_dir?}#{program_name?}*"
    files = Dir[target]
    unless files.empty?
      first = files.first
      # And use the first variant here.
      set_archive(return_archive_type(first))
    end # else we won't set it
  end

  # ========================================================================= #
  # === assemble_program_full_name
  #
  # This method will build-up a valid "program_full_name" setting.
  #
  # In order to achieve this, it will make use of the method called
  # return_default_program_full_name, which itself is a combination
  # of program_name?+archive_type?.
  # ========================================================================= #
  def assemble_program_full_name(
      i = return_default_program_full_name
    )
    set_program_full_name(i)
  end; alias reassemble_program_full_name assemble_program_full_name # === reassemble_program_full_name
       alias determine_program_full_name  assemble_program_full_name # === determine_program_full_name

  # ========================================================================= #
  # === has_key?
  #
  # This query method will delegate towards the yaml dataset, whether
  # it has a given key or whether it does not.
  #
  # The key should ideally be a Symbol - keep this in mind.
  #
  # Since as of April 2021 there will be no modifications made; the input
  # argument will be the key. Prior to that we converted it to a String
  # via .to_s.
  # ========================================================================= #
  def has_key?(i = :blfs)
    dataset?.has_key?(i)
  end

  # ========================================================================= #
  # === is_a_game?
  #
  # This method will simply return true if the program in question is
  # a game, and false if it is not a game. That way we can quickly
  # find out whether the input is a game or whether it is not.
  #
  # Usage example:
  #
  #   ProgramInformation.return_name @program_name
  #   ProgramInformation.return_name 'supertux-0.1.1'
  #
  # ========================================================================= #
  def is_a_game?
    return true if tags?.to_s.include? 'game'
    return false # Else return false here.
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # To invoke this particular method, try:
  #
  #   cookbook --help
  #
  # To invoke individual entries, do:
  #
  #   cookbook nprograms?
  #   cookbook --overview
  #   cookbook --version
  #
  # ========================================================================= #
  def show_help
    left_padding = '  ' # A bit of left-padding here.
    n_space_characters = 50
    how_to_invoke_the_program = HOW_TO_INVOKE_THE_PROGRAM
    opne 'This class can feedback information about any '\
         'specific program.'
    e
    # ======================================================================= #
    # === Common usage examples
    # ======================================================================= #
    e 'Common usage examples:'
    e
    e left_padding+how_to_invoke_the_program+' ruby'
    e left_padding+how_to_invoke_the_program+' htop'
    e left_padding+how_to_invoke_the_program+' m4'
    e
    # ======================================================================= #
    # === Advanced Options
    # ======================================================================= #
    e 'Advanced options:'
    e
    e properly_pad_and_colourize_this_help_option('nprograms?',
        'show how many programs are registered in the cookbooks-collection'
      )
    e properly_pad_and_colourize_this_help_option('--aliases?',
        'show all available aliases to the registered cookbooks-programs'
      )
    e properly_pad_and_colourize_this_help_option('--available_programs?',
        'show all available programs'
      )
    e properly_pad_and_colourize_this_help_option('--batch-download-all-source-archives',
        'batch-download all source archives (optionally select for specific tags)'
      )
    e properly_pad_and_colourize_this_help_option('--overview',
        'give us an overview over all programs in the utility_scripts directory,'
      )
    e (' ' * n_space_characters)+'# of the cookbooks project'
    e properly_pad_and_colourize_this_help_option('--format=[name]',
        'feedback the format type of this entry,'
      )
    e (' ' * n_space_characters)+'# such as via '+simp('cookbook --format=postinstall')
    e properly_pad_and_colourize_this_help_option('--expand',
        'to expand the short yaml dataset into'
      )
    e properly_pad_and_colourize_this_help_option('--expand=PROGRAM_NAME',
        'to expand an individual program, such as python or ruby'
      )
    # ======================================================================= #
    # === --n-registered-binaries?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--n-registered-binaries?',
        'to report how many registered binaries are available'
      )
    # ======================================================================= #
    # === --show-all-about=PROGRAM_NAME
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--show-all-about=PROGRAM_NAME',
        'to show all about the given program at hand'
      )
    e properly_pad_and_colourize_this_help_option('--permanently-disable-colours',
        'to permanently disable all colour output by the Cookbooks project'
      )
    e properly_pad_and_colourize_this_help_option('--update-kde-plasma',
        'batch-update (that is, download) all KDE5 components of KDE Plasma'
      )
    e (' ' * n_space_characters)+'# a long dataset; --initialize is an alias to this action'
    e properly_pad_and_colourize_this_help_option('--create-programs-version-html-file',
        'create and upload a .html file containing all '\
        'program_name-program_versions'
      )
    e (' ' * n_space_characters)+'# entries. This is primarily meant for my home system.'
    # ======================================================================= #
    # === --allowed_tags?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--allowed_tags?',
       'show a list of all allowed tags in the '+simp('tags: subsection')+
       ' of a cookbook-yaml file'
      )
    # ======================================================================= #
    # === --generate_version_file
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--generate_version_file',
        'to generate a programs-version yaml file'
      )
    # ======================================================================= #
    # === --use-psych
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--use-psych',
        'to use psych for the yaml files'
      )
    # ======================================================================= #
    # === --use-syck
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--use-syck',
        'to use syck for the yaml files'
      )
    # ======================================================================= #
    # === --create-html-page-for-these-programs=
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--create-html-page-for-these-programs=',
        'to generate a .html page for the given programs; separate '\
        'the input via'
      )
    e (' ' * n_space_characters)+'# "'+sfancy(',')+'" such as in '+
      simp('=ruby,python,php')
    # ======================================================================= #
    # === --n_binaries?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--n_binaries?',
        'tell us how many binaries/executables are tracked '\
        'in all cookbook-yaml files'
      )
    e properly_pad_and_colourize_this_help_option('--publish',
        'publish the list of all program versions of the cookbooks project'
      )
    e properly_pad_and_colourize_this_help_option('--total-size?',
        'show the total size of all locally available archives'
      )
    e properly_pad_and_colourize_this_help_option('--tutorial',
        'do show the short tutorial how the Cookbooks project may be used'
      )
    e properly_pad_and_colourize_this_help_option('--update-tags',
        'do update the tags defined in the various cookbooks'
      )
    e properly_pad_and_colourize_this_help_option('--validate-entries',
        'validate all Cookbook entries'
      )
    e properly_pad_and_colourize_this_help_option('--version',
        'feedback the current version of the Cookbooks gem'
      )
    e properly_pad_and_colourize_this_help_option('--set_source_dir=/tmp',
        'permanently set a new source dir (aka where all '\
        'archives are kept locally)'
      )
    e properly_pad_and_colourize_this_help_option('--set_temp_dir=/tmp',
        'permanently change the yaml file to set a '\
        'new temp directory'
      )
    e
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.e
  # ========================================================================= #
  def self.e(i = '')
    RBT.e(i)
  end

  # ========================================================================= #
  # === show_tags
  # ========================================================================= #
  def show_tags
    e return_tags
  end

  # ========================================================================= #
  # === query_for
  #
  # This method is a more general query-method for certain data stored
  # in the Cookbook class.
  #
  # For example: the input String 'Compiling this program:' will yield
  # the name of the program that the cookbook class currently holds.
  #
  # The primary use case is for the Flexbox class.
  # ========================================================================= #
  def query_for(i)
    _ = i.to_s.dup
    _.chop! if _.end_with? ':'
    case _
    # ======================================================================= #
    # === "Homepage"
    # ======================================================================= #
    when /^Homepage$/i
      homepage_url?
    # ======================================================================= #
    # === program_name_and_program_version?
    # ======================================================================= #
    when /^program(-|_| )?name(-|_| )?and(-|_| )?program(-|_| )?version\??$/i
      program_name_and_program_version?
    # ======================================================================= #
    # === extract_to?
    # ======================================================================= #
    when /^extract(-|_| )?to\??$/i
      extract_to?
    # ======================================================================= #
    # === "Build system to be used:"
    # ======================================================================= #
    when /^Build(-|_| )?system(-|_| )?to(-|_| )?be(-|_| )?used$/i
      use_this_build_system?
    # ======================================================================= #
    # === "configure options"
    # ======================================================================= #
    when /^configure(-|_| )?options\??$/i
      configure_options?
    # ======================================================================= #
    # === "URL to the source archive"
    # ======================================================================= #
    when /^URL(-|_| )?to(-|_| )?the(-|_| )?source(-|_| )?archive$/i
      url_to_the_source_archive?
    # ======================================================================= #
    # === "Keep the archive extracted"
    # ======================================================================= #
    when /^Keep(-|_| )?the(-|_| )?archive(-|_| )?extracted$/i
      keep_the_archive_extracted?
    # ======================================================================= #
    # === "Program path"
    # ======================================================================= #
    when /^Program(-|_| )?path$/i
      program_path?
    # ======================================================================= #
    # === "Program version"
    # ======================================================================= #
    when /^Program(-|_| )?version$/i
      program_version?
    # ======================================================================= #
    # === "Compiling this program"
    # ======================================================================= #
    when /^Compiling(-|_| )?this(-|_| )?program$/i,
         /^Installing(-|_| )?this(-|_| )?program$/i,
         /^Extracting(-|_| )?this(-|_| )?program$/i
      short_name?
    # ======================================================================= #
    # === "Prefix"
    # ======================================================================= #
    when /^Prefix$/i
      prefix?
    else
      e crimson("Unhandled input to #{__method__}: #{_}")
    end
  end

  # ========================================================================= #
  # === show_n_programs_available
  # ========================================================================= #
  def show_n_programs_available
    e return_n_programs_available
  end

  # ========================================================================= #
  # === do_show_full_description
  #
  # This method will allow us to show the full description of a given
  # program.
  # ========================================================================= #
  def do_show_full_description
    # ======================================================================= #
    # The following method-call will toggle the proper behaviour.
    # ======================================================================= #
    RBT::Action::Cookbooks::SanitizeCookbook.do_not_honour_threshold_value
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(
      namespace:   namespace?,
      use_colours: RBT.use_colours?
    )
  end

  # ========================================================================= #
  # === send_this_method_and_sanitize_arrays
  #
  # The second argument can be used to "limit the result", e. g. to
  # show only n elements (from an Array, typically).
  #
  # It is used in particular to not display too many elements, as these
  # may overflow on the screen (commandline output) - but the user
  # can override this behaviour, via --do-not-truncate.
  # ========================================================================= #
  def send_this_method_and_sanitize_arrays(
      i, limit_the_result = false
    )
    result = send(i)
    # ======================================================================= #
    # The following clause will be entered if there are too many entries
    # for the given field - in particular header-files, binaries and
    # libraries.
    # ======================================================================= #
    if @internal_hash[:try_to_truncate_too_long_output] and
       limit_the_result and
       (result.size > limit_the_result) # Check for more elements here.
      # ===================================================================== #
      # === Handle "headers?" next
      # ===================================================================== #
      if i.to_s.include?('headers?')
        if limit_the_result
          append_this = ' ... → TRUNCATED '+
                        (result.size - limit_the_result).to_s+
                        ' more .h header files'
          result = result[0 .. (limit_the_result - 1)]
          result << append_this
        end
      # ===================================================================== #
      # === Handle "libraries?" next
      # ===================================================================== #
      elsif i.to_s.include?('libraries?')
        if limit_the_result
          append_this = ' ... → TRUNCATED '+
                        (result.size - limit_the_result).to_s+
                        ' more library files'
          result = result[0 .. (limit_the_result - 1)]
          result << append_this
        end
      # ===================================================================== #
      # === Handle "binaries?" next
      # ===================================================================== #
      elsif i.to_s.include?('binaries?')
        if limit_the_result
          append_this = ' ... → TRUNCATED '+
                        (result.size - limit_the_result).to_s+
                        ' more binaries/executables'
          result = result[0 .. (limit_the_result - 1)]
          result << append_this
        end
      end
    end
    if result.is_a? Array
      result = '['+result.join(',')+']' unless result.compact.empty?
    end
    return result
  end

  # ========================================================================= #
  # === do_not_make_use_of_aliases_kept_in_the_file_cookbook_aliases
  # ========================================================================= #
  def do_not_make_use_of_aliases_kept_in_the_file_cookbook_aliases
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases] = false
  end

  # ========================================================================= #
  # === calculate_archive_size
  #
  # This method can be used to determine the size of the target archive
  # in question. The corresponding entry will be set in the main Hash,
  # called 'archive_size'.
  #
  # In order for this to work, we need the content of program_path defined
  # before, however. Whenever we set program_path, we can re-calculate
  # the filesize anew.
  # ========================================================================= #
  def calculate_archive_size(
      i = program_path?
    )
    set_archive_size(0) # First set it to 0 always, as default.
    if found_result? and File.exist?(i.to_s)
      size = File.size?(i.to_s)
      set_archive_size(size)
    end
  end

  # ========================================================================= #
  # === properly_pad_and_colourize_this_help_option
  # ========================================================================= #
  def properly_pad_and_colourize_this_help_option(
      key_to_invoke_the_method,
      text_that_is_to_be_displayed
    )
    left_padding = '  ' # A bit of left-padding here.
    n_left_pad_value  = 38
    n_left_pad_value -= key_to_invoke_the_method.size
    left_padding+HOW_TO_INVOKE_THE_PROGRAM+
                 skyblue(' '+key_to_invoke_the_method)+
                 (' ' * n_left_pad_value)+' # '+
                 text_that_is_to_be_displayed
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.feedback_all_available_formats
  #
  # To invoke this method from the commandline, try:
  #
  #   rbt --feedback_all_available_formats
  #
  # ========================================================================= #
  def self.feedback_all_available_formats(
      i = ::RBT.return_file_specification_of_registered_cookbook_entries
    )
    dataset = YAML.load_file(i)
    dataset.each_pair {|key, value|
      if Object.const_defined? :Colours
        separator = ::RBT.slateblue(': ')
      else
        separator = simp(': ')
      end
      ::RBT.e(
        key.ljust(30)+separator+value.rjust(10)
      )
    }
  end

  # ========================================================================= #
  # === n_programs_available?
  # ========================================================================= #
  def n_programs_available?
    RBT.n_programs?
  end

  # ========================================================================= #
  # === dataset_of_the_program?
  #
  # Note that specifically the method hash? should return the FULL,
  # sanitized dataset, because other programs may rely on that
  # dataset, and because this information may then easily be
  # dumped into a local .yml file as well.
  # ========================================================================= #
  def dataset_of_the_program?
    @internal_hash[:dataset]
  end; alias dataset?          dataset_of_the_program? # === dataset?
       alias dataset           dataset_of_the_program? # === dataset
       alias internal_dataset? dataset_of_the_program? # === internal_dataset?
       alias hash?             dataset_of_the_program? # === hash?
       alias return_hash       dataset_of_the_program? # === return_hash
       alias main_dataset?     dataset_of_the_program? # === main_dataset?
       alias yaml_dataset?     dataset_of_the_program? # === yaml_dataset?
       alias return_dataset    dataset_of_the_program? # === return_dataset
       alias cookbook_dataset? dataset_of_the_program? # === cookbook_dataset?
       alias cookbook?         dataset_of_the_program? # === cookbook?
       alias return_dataset    dataset_of_the_program? # === return_dataset

  # ========================================================================= #
  # === complex_mode?
  # ========================================================================= #
  def complex_mode?
    @internal_hash[:simple_or_complex] == :complex
  end

  # ========================================================================= #
  # === in_simple_mode?
  # ========================================================================= #
  def in_simple_mode?
    @internal_hash[:simple_or_complex] == :simple
  end

  # ========================================================================= #
  # === return_default_program_full_name
  #
  # program_full_name (entry 6) is something like:
  #
  #   'gnome-mahjonng-5.08.tar.xz'
  #
  # so this is program_name_and_program_version?+archive_type?
  # ========================================================================= #
  def return_default_program_full_name(
      i = program_name_and_program_version?.to_s+
          archive_type?.to_s
    )
    return i
  end; alias return_program_full_name return_default_program_full_name # === return_program_full_name

  # ========================================================================= #
  # === found?
  #
  # Query method to determine whether the search term (the program
  # we are seeking) is included or whether it is not.
  #
  # Returns a boolean.
  # ========================================================================= #
  def found?
    @internal_hash[:found]
  end; alias program_is_included? found? # === program_is_included?
       alias found_result?        found? # === found_result?
       alias program_was_found?   found? # === program_was_found?

  # ========================================================================= #
  # === return_middle_part_of_the_program_version
  # ========================================================================= #
  def return_middle_part_of_the_program_version(
      program_version = program_version?
    )
    result = program_version.dup
    if result.include? '.'
      result = result.split('.')[1]
    end
    return result
  end

  # ========================================================================= #
  # === has_a_dependency_on_meson?
  #
  # This method will return a boolean value, indicating whether the
  # given program at hand has a dependency on the build system called
  # meson or whether it has not.
  # ========================================================================= #
  def has_a_dependency_on_meson?
    required_deps_on? and required_deps_on?.include?('meson')
  end

  # ========================================================================= #
  # === skip_configure?
  #
  # Query-method over as to whether we will skip configure or whether
  # we will not.
  # ========================================================================= #
  def skip_configure?
    !run_configure?
  end; alias skip_configure skip_configure? # === skip_configure

  # ========================================================================= #
  # === wrap_result?
  # ========================================================================= #
  def wrap_result?
    @internal_hash[:wrap_result]
  end; alias use_word_wrap? wrap_result? # === use_word_wrap?

  # ========================================================================= #
  # === main_seven?
  #
  # This method will return the main 7 entries that are the most important
  # ones for any given program at hand.
  # ========================================================================= #
  def main_seven?(
      use_padded_output = true
    )
    _ = ''.dup
    one   = 'short_name: '.dup
    two   = 'program_version: '.dup
    three = 'archive_type: '.dup
    four  = 'base_dir: '.dup
    five  = 'program_name: '.dup
    six   = 'program_full_name: '.dup
    seven = 'program_path: '.dup
    # ======================================================================= #
    # Next make sure to pad the output, if the user requested this.
    # ======================================================================= #
    if use_padded_output
      n_ljust = 19
      one   = one.ljust(n_ljust)
      two   = two.ljust(n_ljust)
      three = three.ljust(n_ljust)
      four  = four.ljust(n_ljust)
      five  = five.ljust(n_ljust)
      six   = six.ljust(n_ljust)
      seven = seven.ljust(n_ljust)
    end
    _ << "#{one}#{short_name?}#{N}"
    _ << "#{two}#{program_version?}#{N}"
    _ << "#{three}#{archive_type?}#{N}"
    _ << "#{four}#{base_dir?}#{N}"
    _ << "#{five}#{program_name?}#{N}"
    _ << "#{six}#{program_full_name?}#{N}"
    _ << "#{seven}#{program_path?}#{N}"
    return _
  end; alias all_seven        main_seven? # === all_seven
       alias return_the_seven main_seven? # === return_the_seven

  # ========================================================================= #
  # === meson_or_configure_options?
  #
  # This method will either return the meson-configure options or the
  # GNU configure options. The checks are in order, so first we will
  # check for meson, and then default to configure if meson is NOT
  # used.
  # ========================================================================= #
  def meson_or_configure_options?
    if is_the_build_system_meson?
      meson_configure_options?
    else
      configure_options?
    end
  end; alias configure_options_or_meson_options? meson_or_configure_options? # === configure_options_or_meson_options?

  # ========================================================================= #
  # === return_first_underscore_as_hyphen_from_this_url
  # ========================================================================= #
  def return_first_underscore_as_hyphen_from_this_url(
      i = url1?
    )
    new_url = File.basename(i).dup # Obtain the URL here.
    new_url.sub!(/_/,'-') # This replaces only the first '_' found.
    new_url = remove_archive_from_the_end(new_url)
    return new_url
  end

  # ========================================================================= #
  # === return_default_program_path
  #
  # This is entry number (8), program_path, which will look like:
  #
  #   '/home/x/src/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
  #   '/home/x/src/gnomemahjongg/' + 'gnome-mahjonng-5.08.tar.xz'
  #
  # ========================================================================= #
  def return_default_program_path
    base_dir?+
    program_full_name?
  end; alias return_program_path return_default_program_path # === return_program_path

  # ========================================================================= #
  # === return_sanitized_program_name_and_program_version
  #
  # This method will also get rid of archives.
  #
  # A full program_name_and_program_version would be something
  # like:
  #
  #    'gnome-mahjonng-5.08'
  #
  #     (2) program_name                      like 'gnome-mahjongg'
  #
  # ========================================================================= #
  def return_sanitized_program_name_and_program_version(
      i   = program_name_and_program_version?.dup,
      url = url1?.dup
    )
    case i # case tag
    # ======================================================================= #
    # === USE_URL_DOWNCASE
    #
    # Use the URL as program name but downcase it.
    # ======================================================================= #
    when 'USE_URL_DOWNCASE',
         'USE_DOWNCASED'
      i = File.basename(url.downcase)
    # ======================================================================= #
    # === FIRST_UNDERSCORE_AS_HYPHEN
    #
    # This will convert input such as 'boost_1_61_0' into 'boost-1_61_0'.
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_AS_HYPHEN',
         'FIRST_UNDERSCORE_IS_HYPHEN',
         'USE_URL_REPLACE_UNDERSCORE'
      i = return_first_underscore_as_hyphen_from_this_url(url)
    # ======================================================================= #
    # === PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET
    #
    # This entry point has been specifically added for the program "brotli",
    # but also other programs that would fall into that scheme.
    #
    # Brotli has an URL like this:
    #
    #   https://github.com/google/brotli/archive/v1.0.7/brotli-v1.0.7.tar.gz
    #
    # The annoying part is the very last 'v'. If you extract this archive
    # then the resulting directory name is "brotli-1.0.7", so the folks
    # who packaged this archive evidently are very clueless. But even
    # aside from this, the issue is that the RBT project may have to guess
    # and infer the correct program version, which should not include the
    # 'v' character anyway. The following when-entry ensures precisely
    # this: all alphabet-characters will be dropped from the program
    # version.
    #
    # This entry point has been added on 30.11.2019.
    #
    # Note that PROGRAM_VERSION is not exactly the same, since it will
    # ONLY modify the program version; it will NOT change the name of
    # the program at hand.
    #
    # Keep in mind that the input may be incomplete, such as in the
    # String "ninja-PROGRAM_VERSION".
    # ======================================================================= #
    when /^PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET$/,
         /^PROGRAM_VERSION$/,
         /-PROGRAM_VERSION$/
      program_version = program_version?
      if program_version.nil? or program_version.empty?
        _ = remove_archive_from_the_end(File.basename(url1?.dup))
        _.delete!('a-z')
        _[0,1] = '' if _.start_with? '-'
        program_version = _
      end
      if    i.include? 'PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET'
        use_this_as_program_short_name = program_short_name?
        if use_this_as_program_short_name.nil? or
           use_this_as_program_short_name.empty?
          use_this_as_program_short_name = ProgramInformation.return_short_name(
            remove_archive_from_the_end(
              File.basename(url1?.dup)
            )
          )
        end
        i.sub!(/PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET/,
          use_this_as_program_short_name+'-'+program_version)
      elsif i.include? 'PROGRAM_VERSION'
        i.sub!(/PROGRAM_VERSION/, program_version)
      end
      i.squeeze!('.')
    # ======================================================================= #
    # === USE_URL_LAST_HYPHEN_IS_DOT
    #
    # This entry point will convert the last hyphen, aka '-', into a '.'
    # dot. The input may be 'ImageMagick-7.0.8-27' and it will then
    # become 'ImageMagick-7.0.8.27'.
    # ======================================================================= #
    when /^USE_URL_LAST_HYPHEN_IS_DOT$/
      _ = File.basename(url1?)
      i = remove_archive_from_the_end(_)
      i = i.reverse.sub(/-/,'.').reverse
    # ======================================================================= #
    # === FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS
    #
    # This is due to boost again - apparnetly the boost developers make
    # use of a very strange packaging schemata.
    #
    # So this entry point will convert input such as 'boost_1_61_0' into
    # 'boost-1.61.0'.
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS'
      i = return_first_underscore_as_hyphen_and_other_undscores_as_dots(url)
    # ======================================================================= #
    # === FIRST_DOT_AS_HYPHEN
    #
    # The following case is for input such as "sendmail.5.2.18"
    # which should become "sendmail-5.2.18".
    #
    # We only manipulate the last part though, so if we have input that
    # contains a '/', we assume that this is a path and split it up.
    # This is the reason why we will use File.basename()
    # ======================================================================= #
    when 'FIRST_DOT_AS_HYPHEN'
      i = return_first_dot_as_hyphen_from_this_url(url)
    # ======================================================================= #
    # === URL_REPLACE_UNDERSCORE
    #
    # This entry point handles the case when the input has '_' characters
    # in the String. If this is the case then all '_' will be replaced
    # via '-' characters.
    # ======================================================================= #
    when 'URL_REPLACE_UNDERSCORE',
         'URL_CONVERT_UNDERSCORE',
         'USE_URL_SANITIZED',
         'REPLACE_UNDERSCORES',
         'USE_URL_MIDDLE_HYPHENS'
      i = return_url_replace_underscore(url)
    # ======================================================================= #
    # === USE_URL
    #
    # Simply use the URL as-is, for the program name.
    # ======================================================================= #
    when 'USE_URL'
      i = url
    else
      # ===================================================================== #
      # Handle github-specific links next.
      # ===================================================================== #
      if i and
         is_github_url? and
         i.start_with?('v-') and
         (i =~ /v-\d{0,2}.\d{1,2}.\d{1,2}/) # See: http://rubular.com/r/tWF7ikLZiw
        # =================================================================== #
        # Ok, we now know that the input is something like 'v-1.11.0'.
        # We must thus sanitize several parts of the dataset.
        # =================================================================== #
        if short_name? == 'v' # ← This must evidently be replaced then.
          set_short_name(
            search_for_this_program?.dup #, :also_set_other_names
          )
          set_program_name
          set_base_dir
          i = program_name?
        end
      end
    end
    # ======================================================================= #
    # Get rid of any .tar.xz or similar archive end here.
    # ======================================================================= #
    i = remove_archive_from_the_end(i)
    return i # We must return something.
  end

  # ========================================================================= #
  # === return_url_replace_underscore
  # ========================================================================= #
  def return_url_replace_underscore(i = url?)
    _ = File.basename(i).dup
    if _.include? '_'
      _.tr!('_','-') # Replace all '_' with '-'.
    elsif _.include? '.' # Else replace just the first '.'.
      _.sub!('.','-')
    end
    _ = remove_file_extension(_)
    return _
  end

  # ========================================================================= #
  # === return_first_dot_as_hyphen_from_this_url
  # ========================================================================= #
  def return_first_dot_as_hyphen_from_this_url(i = url1?)
    new_url = File.basename(i).dup # Obtain the URL here.
    new_url.sub!(/\./,'-') # This replaces only the first '.' found.
    return new_url
  end

  # ========================================================================= #
  # === internal_commandline_arguments?
  # ========================================================================= #
  def internal_commandline_arguments?
    @internal_hash[:internal_commandline_arguments]
  end

  # ========================================================================= #
  # === shall_we_display_the_md5sum?
  #
  # Query whether we shall display the md5sum or whether we shall not.
  # ========================================================================= #
  def shall_we_display_the_md5sum?
    dataset?[:shall_we_display_the_md5sum]
  end; alias display_md5sum? shall_we_display_the_md5sum? # === display_md5sum?

  # ========================================================================= #
  # === check_for_binary_names?
  # ========================================================================= #
  def check_for_binary_names?
    CHECK_FOR_BINARY_NAMES
  end

  # ========================================================================= #
  # === is_a_boolean_configuration_value?
  #
  # Query method to determine whether the input is a boolean configuration
  # value. For example, the String "use_glib_schema", would be a boolean
  # configuration value and thus this method would return true.
  # ========================================================================= #
  def is_a_boolean_configuration_value?(i)
    ( ARRAY_FALSE_VALUES + ARRAY_TRUE_VALUES ).flatten.include? i
  end

  # ========================================================================= #
  # === check_for_library_names?
  # ========================================================================= #
  def check_for_library_names?
    CHECK_FOR_LIBRARY_NAMES
  end

  # ========================================================================= #
  # === entry_exists?
  # ========================================================================= #
  def entry_exists?
    File.exist? location?
  end

  # ========================================================================= #
  # === first_url?
  #
  # Returns the first url. Should be used if you want to access URLs.
  # ========================================================================= #
  def first_url?
    dataset?.remote_urls.empty? ? '' : dataset?.remote_urls.first  
  end

  # ========================================================================= #
  # === return_sanitize_url1
  #
  # This can be considered almost equaly to the old "sanitize_url1" method.
  # ========================================================================= #
  def return_sanitize_url1(
      i = url1?
    )
    i = i.dup
    # ======================================================================= #
    # === Handle "llvm-10.0.0.src.tar.xz" and similar cases next
    # ======================================================================= #
    if i
      i.sub!(/\.src\./,'')          if i.include?  '.src.' # For doxygen.yml
      i.sub!(/-src/,'')             if i.include?  '-src' # For zatacka.yml
      i.delete_suffix!('/download') if i.end_with? '/download'
    end
    return i
  end

  # ========================================================================= #
  # === raw_cookbook?
  # ========================================================================= #
  def raw_cookbook?
    @internal_hash[:raw_cookbook]
  end

  # ========================================================================= #
  # === raw_cookbook_dataset?
  # ========================================================================= #
  def raw_cookbook_dataset?
    @internal_hash[:raw_cookbook] and raw_cookbook?.dataset?
  end; alias raw_dataset? raw_cookbook_dataset? # === raw_dataset?

  # ========================================================================= #
  # === make_use_of_aliases_kept_in_the_file_cookbook_aliases?
  #
  # This method can be used to query whether we will make use of aliases
  # kept in the cookbook_aliases.yml file or whether we will not.
  # ========================================================================= #
  def make_use_of_aliases_kept_in_the_file_cookbook_aliases?
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases]
  end; alias make_use_of_cookbook_aliases? make_use_of_aliases_kept_in_the_file_cookbook_aliases? # === make_use_of_cookbook_aliases?

  # ========================================================================= #
  # === extracted_towards?
  #
  # Determine where the archive will be extracted towards. This will
  # also specifically include the name of the program at hand and
  # the corresponding version, such as 'gnome-mahjonng-5.08'.
  # ========================================================================= #
  def extracted_towards?
    # ======================================================================= #
    # We include the program name and the program version.
    # ======================================================================= #
    result = "#{extract_to?}#{program_name_and_program_version?}"
    unless result.end_with? '/'
      result = result.dup if result.frozen?
      result << '/'
    end
    return result
  end; alias extract_towards? extracted_towards? # === extract_towards?

  # ========================================================================= #
  # === return_default_location_of_this_cookbook
  #
  # This will use the default location based on the search_term.
  # ========================================================================= #
  def return_default_location_of_this_cookbook(
      i = search_which_program?
    )
    "#{individual_cookbooks_directory?}#{i}.yml"
  end; alias return_default_location return_default_location_of_this_cookbook # == return_default_location
       alias yaml_file?              return_default_location_of_this_cookbook # == yaml_file?

  # ========================================================================= #
  # === raw_yaml_file?
  # ========================================================================= #
  def raw_yaml_file?
    raw_cookbook?.yaml_file?
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    raw_cookbook?.file?
  end; alias file               file? # === file
       alias which_file?        file? # === which_file?
       alias raw_cookbook_file? file? # === raw_cookbook_file?

  # ========================================================================= #
  # === does_include?
  # ========================================================================= #
  def does_include?(
      this_program,
      honour_aliases = make_use_of_aliases_kept_in_the_file_cookbook_aliases?
    )
    # ======================================================================= #
    # Delegate towards the toplevel-method next.
    # ======================================================================= #
    ::RBT.does_include?(
      this_program, honour_aliases
    )
  end; alias is_included? does_include? # === is_included?

  # ========================================================================= #
  # === return_first_underscore_as_hyphen_and_other_undscores_as_dots
  #
  # This method exists specifically for input such as 'boost_1_61_0'.
  # ========================================================================= #
  def return_first_underscore_as_hyphen_and_other_undscores_as_dots(i = url?)
    i = return_first_underscore_as_hyphen_from_this_url(i).dup
    i.tr!('_','.')
    return i
  end

  # ========================================================================= #
  # === opposite_token?
  #
  # This method will return the "opposite" token, e. g. '_' is the
  # "opposite to '-' and vice versa.
  # ========================================================================= #
  def opposite_token?(
      i = @internal_hash[:use_this_connective_token]
    )
    case i
    when '-'
      '_'
    when '_'
      '-'
    end
  end

  # ========================================================================= #
  # === simple_or_complex?
  # ========================================================================= #
  def simple_or_complex?
    @internal_hash[:simple_or_complex]
  end; alias mode?        simple_or_complex? # === mode?
       alias mode_in_use? simple_or_complex? # === mode_in_use?
       alias type?        simple_or_complex? # === type?

  # ========================================================================= #
  # === seek_this_program?                                         (seek tag)
  #
  # This method will tell us as for which program we will search.
  #
  # Do note that this is NOT the same as program_name? or short_name?,
  # because the latter two may contain upcased characters, such as
  # for "libX11", whereas the variable here is ALWAYS downcased.
  # ========================================================================= #
  def seek_this_program?
    @internal_hash[:seek_this_program]
  end; alias seek_which_program       seek_this_program? # === seek_which_program
       alias seek_which_program?      seek_this_program? # === seek_which_program?
       alias which_program?           seek_this_program? # === which_program?
       alias search_for_this_program? seek_this_program? # === search_for_this_program?
       alias search_which_program?    seek_this_program? # === search_which_program?

  # ========================================================================= #
  # === return_program_name_from_the_raw_cookbook
  # ========================================================================= #
  def return_program_name_from_the_raw_cookbook
    raw_cookbook?.program_name?
  end

  # ========================================================================= #
  # === is_the_build_system_meson?
  #
  # Query-method to determine whether the build system makes use of meson
  # or whether it does not.
  # ========================================================================= #
  def is_the_build_system_meson?
    use_this_build_system? == 'meson'
  end

  # ========================================================================= #
  # === is_the_build_system_cmake?
  # ========================================================================= #
  def is_the_build_system_cmake?
    use_this_build_system? == 'cmake'
  end

  # ========================================================================= #
  # === return_program_version_from_the_raw_cookbook
  # ========================================================================= #
  def return_program_version_from_the_raw_cookbook
    raw_cookbook?.program_version?
  end

  # ========================================================================= #
  # === original_unmodified_name?
  # ========================================================================= #
  def original_unmodified_name?
    @internal_hash[:original_unmodified_name]
  end

  # ========================================================================= #
  # === urls?
  #
  # This method is an "aggregate-method"; it will return all available
  # URLs in Array format.
  #
  # We only support 5 URLs at max right now, though.
  #
  # Since as of July 2012 this method will not return empty values,
  # as they are useless anyway.
  # ========================================================================= #
  def urls?
    # ======================================================================= #
    # Build our Array that will hold all 5 URLs.
    # ======================================================================= #
    array = %w( url1 url2 url3 url4 url5 )
    _ = dataset?.values_at(*array) # We require the * here.
    _.compact!
    _.reject!(&:empty?) # We don't want empty entries, hence we filter them away.
    return _
  end; alias remote_urls  urls? # === remote_urls
       alias remote_urls? urls? # === remote_urls?

  # ========================================================================= #
  # === return_version_from_url1
  #
  # This will return the inferred version from the url1 entry.
  # ========================================================================= #
  def return_version_from_url1(
      use_this_URL = url1?
    )
    _ = remove_archive_at_the_end(
      File.basename(use_this_URL)
    )
    program_information_return_version(_)
  end

  # ========================================================================= #
  # === licence?
  #
  # This is the query method over licence.
  #
  # Do note that while the specification allows for both a String value,
  # or an Array value alternatively, it only really makes sense to
  # assume that tag to be a String.
  #
  # The reason why an Array is used as an alternative is twofold:
  #
  #   (1) That way we can also respect programs that have more than
  #       one licence.
  #   (2) It may sometimes be more convenient to use an Array in the
  #       corresponding .yml file.
  #
  # ========================================================================= #
  def licence?
    _ = obtain(__method__.to_s.delete('?').to_sym).to_s
    _ = _.first if _.is_a? Array
    return _
  end; alias license? licence? # === license?

  # ========================================================================= #
  # === has_tags?
  #
  # This method will return true or false depending on whether the given
  # program at hand has any tags registered at all, or whether it has not.
  # ========================================================================= #
  def has_tags?
    return false if tags?.empty?
    return true
  end

  # ========================================================================= #
  # === meson_configure_options?
  # ========================================================================= #
  def meson_configure_options?
    obtain __method__.to_s.delete('?').to_sym
  end; alias meson_configure_options meson_configure_options? # === meson_configure_options

  # ========================================================================= #
  # === autosymlink_lib64
  # ========================================================================= #
  def autosymlink_lib64?
    obtain __method__.to_s.delete('?').to_sym
  end; alias autosymlink_lib64 autosymlink_lib64? # === autosymlink_lib64

  # ========================================================================= #
  # === use_this_make_command?
  # ========================================================================= #
  def use_this_make_command?
    obtain __method__.to_s.delete('?').to_sym
  end; alias use_this_make_command   use_this_make_command? # === use_this_make_command
       alias use_which_make_command? use_this_make_command? # === use_which_make_command?

  # ========================================================================= #
  # === parameters_to_make?
  # ========================================================================= #
  def parameters_to_make?
    obtain __method__.to_s.delete('?').to_sym
  end; alias make_options  parameters_to_make? # === make_options
       alias make_options? parameters_to_make? # === make_options?

  # ========================================================================= #
  # === configure_options?
  #
  # Convenience method to query the configuration options available.
  # ========================================================================= #
  def configure_options?
    obtain __method__.to_s.delete('?').to_sym
  end; alias configuration_options  configure_options? # === configuration_options
       alias configuration_options? configure_options? # === configuration_options?
       alias configure_options      configure_options? # === configure_options

  # ========================================================================= #
  # === use_this_make_install_command?
  #
  # Which variant of "make install" we will use, if we will use a
  # different variant that is.
  # ========================================================================= #
  def use_this_make_install_command?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === may_we_modify_the_configure_options?
  # ========================================================================= #
  def may_we_modify_the_configure_options?
    obtain __method__.to_s.delete('?').to_sym
  end; alias may_we_modify_the_configure_options may_we_modify_the_configure_options? # === may_we_modify_the_configure_options

  # ========================================================================= #
  # === sed?
  # ========================================================================= #
  def sed?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === recommended_deps_on?
  # ========================================================================= #
  def recommended_deps_on?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === description?                                        (description tag)
  #
  # The description value of a program will be returned via this method.
  # ========================================================================= #
  def description?(
      wrap_result = wrap_result?
    )
    result = obtain(:description)
    case wrap_result
    # ======================================================================= #
    # === :retain_newlines
    # ======================================================================= #
    when :retain_newlines
      wrap_result = false
    end
    if wrap_result
      if use_word_wrap?
        result = word_wrap(result)
      end if result # Do not work on nil-values here.
    end
    return result
  end; alias desc?           description? # === desc?
       alias description_of? description? # === description_of?

  # ========================================================================= #
  # === n_libraries?
  # ========================================================================= #
  def n_libraries?
    libraries?.size
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    first_argument?
  end

  # ========================================================================= #
  # === installation_steps?
  # ========================================================================= #
  def installation_steps?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === url3?
  # ========================================================================= #
  def url3?
    obtain __method__.to_s.delete('?').to_sym
  end; alias url3 url3? # === url3

  # ========================================================================= #
  # === program_compile_name?
  #
  # This will be guaranteed to be a String.
  # ========================================================================= #
  def program_compile_name?
    obtain(__method__.to_s.delete('?').to_sym).to_s
  end

  # ========================================================================= #
  # === program_name_and_program_version?
  #
  # Note that the connective token in use is usually '-', but there are
  # programs such as 'artemis_v16.0.11.jar' that have no '-' at all
  # whatsoever. To also honour such programs, we have to use this
  # connective token - it will be re-defined towards '_' specifically
  # for programs that have no '-' token, such as artemis.
  # ========================================================================= #
  def program_name_and_program_version?
    obtain __method__.to_s.delete('?').to_sym
  end; alias program_name_and_program_version        program_name_and_program_version? # === program_name_and_program_version
       alias program_name_and_version?               program_name_and_program_version? # === program_name_and_version?
       alias program_name_and_the_program_version?   program_name_and_program_version? # === program_name_and_the_program_version?
       alias return_program_name_and_program_version program_name_and_program_version? # === return_program_name_and_program_version

  # ========================================================================= #
  # === tags?
  # ========================================================================= #
  def tags?
    obtain __method__.to_s.delete('?').to_sym
  end; alias return_tags tags? # === return_tags
       alias tags        tags? # === tags

  # ========================================================================= #
  # === short_description?
  #
  # This method will obtain the short description of a given program.
  #
  # Not every program will have a short description, but it is recommended
  # that all programs SHOULD have a short description.
  # ========================================================================= #
  def short_description?
    obtain __method__.to_s.delete('?').to_sym
  end; alias short_desc?              short_description? # === short_desc?
       alias return_short_description short_description? # === return_short_description

  # ========================================================================= #
  # === short_name?
  #
  # Reader method for short_name.
  #
  # This method will query the short name of the program we are trying
  # to find.
  #
  # For example, this method will return the program name without the
  # version, such as:
  #
  #  "ruby" (which is part of the program_name "ruby-2.2.0")
  #
  # This method must always return a String, even if it is an empty
  # String.
  #
  # Characters such as '-' are NOT allowed to be part of the
  # short_name variable. The corresponding setter-method will
  # make sure of this.
  # ========================================================================= #
  def short_name?
    obtain __method__.to_s.delete('?').to_sym
  end; alias real_short_name?    short_name? # === real_short_name?
       alias rshort?             short_name? # === rshort?
       alias program_short_name? short_name? # === program_short_name?
       alias short_program_name? short_name? # === short_program_name?
       alias shortname?          short_name? # === shortname?
       alias short_name          short_name? # === short_name
       alias sname?              short_name? # === sname?
       alias name                short_name? # === name
       alias name?               short_name? # === name?
       # alias program?          short_name? # === program? # This is already occupied.

  # ========================================================================= #
  # === program_version?
  #
  # This method must always return a String.
  #
  # We use it to query the program version of the specific program at hand.
  #
  # For instance, if the program name is "python-2.4" then this
  # method will return the version string part, i.e. "2.4".
  # ========================================================================= #
  def program_version?
    obtain __method__.to_s.delete('?').to_sym
  end; alias version                program_version? # === version An alias to the above ^^^.
       alias version?               program_version? # === version?
       alias pversion?              program_version? # === pversion?
       alias program_version        program_version? # === program_version
       alias return_program_version program_version? # === return_program_version

  # ========================================================================= #
  # === enable_shared?
  # ========================================================================= #
  def enable_shared?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === use_glib_schema?
  # ========================================================================= #
  def use_glib_schema?
    obtain __method__.to_s.delete('?').to_sym
  end; alias use_glib_schema use_glib_schema? # === use_glib_schema

  # ========================================================================= #
  # === optional_start_message
  # ========================================================================= #
  def optional_start_message?
    obtain __method__.to_s.delete('?').to_sym
  end; alias optional_start_message optional_start_message? # === optional_start_message

  # ========================================================================= #
  # === pre_make_install_sed?
  # ========================================================================= #
  def pre_make_install_sed?
    obtain __method__.to_s.delete('?').to_sym
  end; alias pre_make_install_sed pre_make_install_sed? # === pre_make_install_sed?

  # ========================================================================= #
  # === libraries?
  #
  # Return the given libraries that will be installed by the given program
  # at hand.
  # ========================================================================= #
  def libraries?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @internal_hash[:debug]
  end

  # ========================================================================= #
  # === last_url?
  #
  # Returns the last url. Should be used if you want to access URLs.
  # ========================================================================= #
  def last_url?
    dataset?.remote_urls.empty? ? '' : dataset?.remote_urls.last  
  end

  # ========================================================================= #
  # === program_name?
  #
  # This method is a reader-method over e. g. 'gnome-mahjongg'.
  #
  # It does NOT have the version of the program included, which is
  # a different behaviour enabled during May 2020.
  # ========================================================================= #
  def program_name?
    obtain __method__.to_s.delete('?').to_sym
  end; alias name_and_version?                 program_name? # === name_and_version?
       alias pname?                            program_name? # === pname?
       alias program_name                      program_name? # === program_name

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    obtain __method__.to_s.delete('?').to_sym
  end; alias return_homepage  homepage? # === return_homepage
       alias remote_homepage? homepage? # === remote_homepage?
       alias homepage         homepage? # === homepage
       alias homepage_url?    homepage? # === homepage_url?

  # ========================================================================= #
  # === run_configure
  # ========================================================================= #
  def run_configure?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === return_appdir_prefix
  #
  # This method will return an App-Dir like path of the given program at
  # hand, such as "ffmpeg" or "php".
  #
  # It is typically called from within the method called .sanitize_the_prefix().
  # In other words, whenever we need to sanitize the prefix passed this
  # method will be called.
  #
  # This method requires that short_name? and program_version? were
  # already defined prior to its invocation.
  #
  # The appdir components would be like this:
  #
  #   /home//Programs/Ruby/2.4.1/
  #   programs_dir?+short_name?.capitalize+'/'+program_version?
  #
  # ========================================================================= #
  def return_appdir_prefix(
      programs_dir_to_use    = programs_dir?, 
      short_name_to_use      = short_name?.capitalize,
      program_version_to_use = program_version?
    )
    _ = programs_dir_to_use.dup
    if are_we_on_gobolinux?
      _ << program_compile_name?.dup
    else
      _ << short_name_to_use.dup
    end
    _ << "/#{program_version_to_use.dup}/"
    return _
  end; alias return_appdir_path return_appdir_prefix # === return_appdir_path
       alias build_appdir_path  return_appdir_prefix # === build_appdir_path

  # ========================================================================= #
  # === can_be_compiled_statically?
  # ========================================================================= #
  def can_be_compiled_statically?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === compiler_to_use?
  # ========================================================================= #
  def compiler_to_use?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === libexec?
  # ========================================================================= #
  def libexec?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === required_deps_on?
  #
  # This method will list the required dependencies of a given program and
  # returns them, at least those that are defined in the
  # 'required_deps_on:' entry of the respective yaml file.
  #
  # Please keep in mind that our dependencies must be kept in Array form.
  # ========================================================================= #
  def required_deps_on?
    obtain __method__.to_s.delete('?').to_sym
  end; alias program_dependencies_on   required_deps_on? # === program_dependencies_on
       alias deps_on?                  required_deps_on? # === deps_on?
       alias dependencies_on?          required_deps_on? # === dependencies_on?
       alias return_dependencies       required_deps_on? # === return_dependencies
       alias required_deps?            required_deps_on? # === required_deps?
       alias dependencies?             required_deps_on? # === dependencies?
       alias required_dependencies_on? required_deps_on? # === required_dependencies_on?
       alias deps?                     required_deps_on? # === deps?

  # ========================================================================= #
  # === enable_static?
  # ========================================================================= #
  def enable_static?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === localstatedir?
  # ========================================================================= #
  def localstatedir?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === keep_extracted?
  # ========================================================================= #
  def keep_extracted?
    obtain __method__.to_s.delete('?').to_sym
  end; alias keep_extracted              keep_extracted? # === keep_extracted
       alias keep_the_archive_extracted? keep_extracted? # === keep_the_archive_extracted?

  # ========================================================================= #
  # === flatpak_url?
  # ========================================================================= #
  def flatpak_url?
    obtain __method__.to_s.delete('?').to_sym
  end; alias flatpak_url flatpak_url? # === flatpak_url

  # ========================================================================= #
  # === program_full_name?
  #
  # This entry will have everything except the leading PATH.
  #
  # For instance:
  #
  #  "gpm-1.20.1.tar.xz"
  #
  # ========================================================================= #
  def program_full_name?
    obtain __method__.to_s.delete('?').to_sym
  end; alias program_full_name  program_full_name? # === program_full_name
       alias full_program_name? program_full_name? # === full_program_name?

  # ========================================================================= #
  # === blfs?                                                      (blfs tag)
  #
  # The "Beyond linux from scratch" variable will be returned here,
  # should it exist.
  # ========================================================================= #
  def blfs?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === use_build_directory?
  #
  # Query whether a build directory will be used for the given program
  # at hand.
  # ========================================================================= #
  def use_build_directory?
    obtain __method__.to_s.delete('?').to_sym
  end; alias use_build_dir?         use_build_directory? # === use_build_dir?
       alias use_build_directory    use_build_directory? # === use_build_directory
       alias use_a_build_directory  use_build_directory? # === use_a_build_directory
       alias use_a_build_directory? use_build_directory? # === use_a_build_directory?

  # ========================================================================= #
  # === manual_steps?
  # ========================================================================= #
  def manual_steps?
    obtain __method__.to_s.delete('?').to_sym
  end

  # ========================================================================= #
  # === prefix?
  #
  # Query the prefix in use by our sought program.
  # ========================================================================= #
  def prefix?
    obtain __method__.to_s.delete('?').to_sym
  end; alias prefix         prefix? # === prefix
       alias prefix_to_use? prefix? # === prefix_to_use?

  # ========================================================================= #
  # === obtain                                            (obtain tag, o tag)
  #
  # This getter-method can be used to obtain elements from the main yaml
  # dataset, which is stored in:
  #
  #   @internal_hash[:dataset]
  #
  # The main reason why a method is used here is because the keys can
  # be either Symbols or Strings. If we ever wish to change to another
  # format, we can do so thanks to the flexibility of this setup.
  # ========================================================================= #
  def obtain(this_key)
    result = nil
    # ======================================================================= #
    # Next, we need to decide whether we use a Symbol, or a String.
    # ======================================================================= #
    unless this_key.is_a? Symbol
      this_key = this_key.to_sym if complex_mode?
    end
    dataset = dataset?
    if dataset
      if dataset.has_key? this_key
        result = dataset[this_key] # [] is better than .fetch() here.
      # elsif raw_cookbook_dataset?.has_key?(what.to_s)
      #  result = raw_cookbook_dataset?[what.to_s]
      #else
        # e rev+'DEBUG: TODO, remove me -> No key called '+
        # steelblue(this_key.to_s)+' (class: '+this_key.class.to_s+
        # ') could be found.'
      end
    end
    return result
  end; alias o            obtain # === o (o tag)
       alias get_value_of obtain # === get_value_of

  # ========================================================================= #
  # === set_short_name
  #
  # This is entry number (1), the setter for the short name of the program.
  # It will set the short_name entry in the main cookbook dataset.
  #
  # The 'short_name' variable is NOT allowed to include the '-' character.
  #
  # Valid input to this method could be something like:
  #
  #   "xfce4-dev-tools"
  #
  # We must delete all '-' if they occur.
  # ========================================================================= #
  def set_short_name(
      i = program_name? # 1 tag
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = program_name?.dup
    end
    i = i.to_s.dup # ← This must come after checking for Symbols.
    i.downcase!
    i.delete!('-') if i.include? '-'
    # ======================================================================= #
    # If the input includes .tar, check it further.
    # ======================================================================= #
    if i and i.include?('.tar')
      i = ::ProgramInformation.return_short_name(i)
    end
    # ======================================================================= #
    # === Eliminate '/' if they are found.
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    dataset?[:short_name] = i
  end; alias set_real_short_name       set_short_name # === set_real_short_name
       alias determine_real_short_name set_short_name # === determine_real_short_name
       alias set_program_short_name    set_short_name # === set_program_short_name
       alias set_real_program_name=    set_short_name # === set_real_program_name=

  # ========================================================================= #
  # === archive_size?
  #
  # This method keeps track of the size of the program tarball in question.
  #
  # For this to work, we should have initialized the program size at 0.
  # ========================================================================= #
  def archive_size?
    obtain(__method__.to_s.delete('?').to_sym)
  end; alias size          archive_size? # === size
       alias size?         archive_size? # === size?
       alias program_size  archive_size? # === program_size
       alias program_size? archive_size? # === program_size?
       alias archive_size  archive_size? # === archive_size
       alias file_size?    archive_size? # === file_size?
       alias file_size     archive_size? # === file_size

  # ========================================================================= #
  # === set_simple_or_complex
  # ========================================================================= #
  def set_simple_or_complex(
      i = :default
    )
    # ======================================================================= #
    # === @simple_or_complex
    #
    # This can be :simple or :complex. The value for :default will be
    # :complex. 
    # ======================================================================= #
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         nil
      i = :complex
    # ======================================================================= #
    # === :use_expanded_dataset
    # ======================================================================= #
    when :use_expanded_dataset,
         :expanded,
         :fast,
         :complex
      i = :complex
    # ======================================================================= #
    # === :fast_and_be_verbose
    # ======================================================================= #
    when :fast_and_be_verbose
      i = :complex
      set_be_verbose
    # ======================================================================= #
    # === :simple
    # ======================================================================= #
    when :simple,
         :simple_cookbook
      i = :simple
    # ======================================================================= #
    # === :expanded
    # ======================================================================= #
    else
      pp caller()
      e 'Unknown: '+i.to_s+' ('+i.class.to_s+')  - please fix this.'
      exit
    end
    @internal_hash[:simple_or_complex] = i
  end

  # ========================================================================= #
  # === set_simple
  # ========================================================================= #
  def set_simple
    set_simple_or_complex(:simple)
  end

  # ========================================================================= #
  # === meson_or_cmake_or_configure_options?
  #
  # This is similar to the method called .meson_or_configure_options?()
  # but we will also check whether cmake is in use.
  #
  # The optional argument must be a Symbol. If provided then this
  # method will simply return the corresponding configure-options.
  # ========================================================================= #
  def meson_or_cmake_or_configure_options?(
      optional_use_this_build_system = nil
    )
    if optional_use_this_build_system
      case optional_use_this_build_system
      # ===================================================================== #
      # === :meson
      # ===================================================================== #
      when :meson
        meson_configure_options?
      # ===================================================================== #
      # === :cmake
      # ===================================================================== #
      when :cmake
        cmake_configure_options?
      # ===================================================================== #
      # === :configure
      # ===================================================================== #
      when :configure
        configure_options?
      end
    else
      if is_the_build_system_meson?
        meson_configure_options?
      elsif is_the_build_system_cmake?
        cmake_configure_options?
      else
        configure_options?
      end
    end
  end

  # ========================================================================= #
  # === set_complex_program_version
  #
  # The purpose of this method is to allow the user to overrule the
  # respective settings, at run-time.
  #
  # That way, you can change e. g. "glib-2.24" to "glib-2.25".
  # ========================================================================= #
  def set_complex_program_version(
      i,
      be_verbose = true
    )
    case be_verbose
    # ======================================================================= #
    # === :be_quiet
    # ======================================================================= #
    when :be_quiet, :be_silent
      be_verbose = false
    end
    i = i.dup if i.frozen?
    # ======================================================================= #
    # Add a '=' to make it easier to know what we want to modify. This
    # will NOT happen if the input includes a '@', though.
    # ======================================================================= #
    i.prepend '=' unless i.include?('=') or i.include?('@')
    # ======================================================================= #
    # Next, we fetch a specific version. We assume this to be the last part
    # of the command given, which must include a '=' character. Since as
    # of September 2022 a '@' is permissive as well.
    # ======================================================================= #
    if i.include? '='
      new_version = i.split('=').last
    elsif i.include? '@'
      new_version = i.split('@').last
    end
    # ======================================================================= #
    # Chop off archive-extension since we don't need these.
    # ======================================================================= #
    new_version = remove_file_extension(new_version)
    if be_verbose
      opne "Setting a new version: #{simp(new_version)}"
    end
    set_program_version(new_version, :propagate_the_change) # The second argument will propagate the change, by cascading downwards.
    update_prefix :bypass_check
  end

  # ========================================================================= #
  # === set_program_name_and_program_version
  #
  # This is entry number (4), combining program_name and program_version.
  # ========================================================================= #
  def set_program_name_and_program_version(
      i = :default
    )
    case i
    # === :default
    when :default
      i = "#{program_name?}#{connective_token?}#{program_version?}"
    end
    set(:program_name_and_program_version, i)
  end; alias update_program_name_and_program_version set_program_name_and_program_version # === update_program_name_and_program_version

  # ========================================================================= #
  # === update_program_path
  # ========================================================================= #
  def update_program_path
    set_program_path(:default)
  end; alias assemble_full_path      update_program_path # === assemble_full_path
       alias update_the_program_path update_program_path # === update_the_program_path

  # ========================================================================= #
  # === set_internal_prefix
  #
  # Use this method to set towards a specific prefix, such as '/usr/'.
  #
  # This is then the option that will be passed into scripts such as
  # 'configure' or cmake-based systems.
  #
  # This setter method is NOT allowed to modify the given input.
  # ========================================================================= #
  def set_internal_prefix(
      i            = '/usr/',
      bypass_check = :no_longer_available # false
    )
    if i.is_a? Symbol
      i = '/usr/'
    end
    @internal_hash[:dataset][:prefix] = i
    # Or use set(:prefix, i).
  end; alias set_prefix     set_internal_prefix # === set_prefix
       alias prefix=        set_internal_prefix # === prefix=
       alias prefix_to_use= set_internal_prefix # === prefix_to_use=
       alias update_prefix  set_internal_prefix # === update_prefix

  # ========================================================================= #
  # === set_program_version                                     (version tag)
  #
  # This is entry number (3), the program version of our program at hand.
  #
  # An example for this would be '5.08', for instance.
  #
  # As of January 2019, this method will also eliminate all '/' from the
  # input, if found. A proper program version does not have this
  # character.
  # ========================================================================= #  
  def set_program_version(
      i                    = program_version?,
      propagate_the_change = true
    )
    case i
    # ======================================================================= #
    # === INFER_THE_VERSION
    # ======================================================================= #
    when /INFER_THE_VERSION/ # Match this one exactly.
      _ = url1? # This may be: https://github.com/FluidSynth/fluidsynth/archive/refs/tags/v2.3.1.tar.gz
      if _.include?('/')
        _ = File.basename(_)
      end
      i = remove_archive_at_the_end(_).delete('v')
    # ======================================================================= #
    # === :infer_from_the_program_name_and_program_version
    # ======================================================================= #
    when :infer_from_the_program_name_and_program_version
      i = ProgramInformation.return_version(
        program_name_and_program_version?
      )
    end
    i = i.to_s.dup # Work on a copy.
    i.delete!('/')        if i.include? '/'
    i.gsub!(/\.src/,'')   if i.include? '.src'
    i.gsub!(/_src/,'')    if i.include? '_src'
    i.sub!(/-source$/,'') if i.include? '-source'
    i.sub!(/-Source/,'')  if i.include? '-Source'
    i = chop_off_archive(i)
    i = remove_file_suffix(i) # Added this as of January 2014.
    if i.include? '#' # This is an illegal character as of April 2014.
      i = i[0, i.index('#')]
    end
    set(:program_version, i)
    if propagate_the_change
      update_program_name_and_program_version
      update_the_program_path
    end
  end; alias set_program_version= set_program_version # === set_program_version=
       alias set_version          set_program_version # === set_version
       alias use_this_version=    set_program_version # === use_this_version=

  # ========================================================================= #
  # === do_not_autogenerate
  # ========================================================================= #
  def do_not_autogenerate
    @internal_hash[:autogenerate_expanded_dataset] = false
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments_with_leading_hyphens
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    elsif i.is_a? Hash
      # ===================================================================== #
      # === Handle Hashes passed to menu() here
      # ===================================================================== #
      i.each_pair {|key, value|
        case key
        # =================================================================== #
        # === :autogenerate_expanded_dataset
        # =================================================================== #
        when :autogenerate_expanded_dataset
          case value
          when true
            @internal_hash[:autogenerate_expanded_dataset] = true
          when false
            @internal_hash[:autogenerate_expanded_dataset] = false
          end
        end 
      }
    else
      case i
      # ===================================================================== #
      # === --no-colours
      # ===================================================================== #
      when /^-?-?no(-|_)?colou?rs$/i
        disable_colours
      # ===================================================================== #
      # === --simple
      #
      # --raw is an alias to this entry point.
      # ===================================================================== #
      when /^-?-?simple$/i,
           /^-?-?raw$/i
        set_simple_or_complex(:simple)
      end
    end
  end

  # ========================================================================= #
  # === determine_all_relevant_values_for_the_dataset_variable
  #
  # This method will copy some important values from the raw-cookbook
  # to the main @dataset variable.
  # ========================================================================= #
  def determine_all_relevant_values_for_the_dataset_variable
    # ======================================================================= #
    # Some variables are still missing, such as program_path. The
    # specification does not include these, but we need them for
    # class RBT::Compile, so add these next.
    #
    # Perhaps in the future we may also add these variables, but for
    # the time being, these variables have to be defined next.
    # ======================================================================= #
    dataset?[:program_path] = base_dir?.to_s+
                              program_name_and_program_version?.to_s+
                              archive_type?.to_s
    if dataset?[:program_path].nil?
      dataset?[:program_path] = raw_cookbook?.program_path?
    end
    if dataset?[:program_version].nil?
      dataset?[:program_version]   = return_program_version_from_the_raw_cookbook
    end
    if dataset?[:program_full_name].nil?
      dataset?[:program_full_name] = return_default_program_full_name
    end
    if dataset?[:program_name].nil?
      dataset?[:program_name]      = return_program_name_from_the_raw_cookbook
    end
    if dataset?[:program_name_and_program_version].nil?
      dataset?[:program_name_and_program_version] = raw_cookbook?.program_name_and_program_version?
    end
    if dataset?[:keep_extracted].nil?
      set_keep_extracted(raw_cookbook?.keep_extracted?)
    end
  end

  # ========================================================================= #
  # === set_raw_cookbook
  # ========================================================================= #
  def set_raw_cookbook(i)
    @internal_hash[:raw_cookbook] = i
  end

  # ========================================================================= #
  # === instantiate_raw_cookbook                                    (raw tag)
  # ========================================================================= #
  def instantiate_raw_cookbook(
      i = seek_this_program?
    )
    # ===================================================================== #
    # We also have to sync the variable @simple_or_complex down onto the
    # raw-cookbooks loader.
    # ===================================================================== #
    # @internal_hash[:raw_cookbook] = RBT.raw_cookbook(i) { simple_or_complex? }
    @internal_hash[:raw_cookbook] = action(
      :raw_cookbook,
      i,
      {
        autogenerate_expanded_dataset: @internal_hash[:autogenerate_expanded_dataset]
      }
    ) { simple_or_complex? }
  end; alias initialize_raw_cookbook instantiate_raw_cookbook # === initialize_raw_cookbook

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.return_dataset
  #
  # This method will return the whole dataset for a given program at
  # hand (if it is registered). You can then use this dataset in
  # other programs.
  #
  # As input it is preferred to provide a symbol, such as :htop or
  # :lua; a String should work fine as well, though.
  #
  # Returns: a Hash
  #
  # Usage examples:
  #
  #   htop_dataset = RBT::Action::Cookbooks::SanitizeCookbook.return_dataset(:htop)
  #   ruby_dataset = RBT::Action::Cookbooks::SanitizeCookbook.return_dataset(:ruby)
  #
  # ========================================================================= #
  def self.return_dataset(
      of_this_program = :htop
    )
    dataset = RBT::Action::Cookbooks::SanitizeCookbook.new(of_this_program).yaml_dataset?
    return dataset
  end; self.instance_eval { alias return_cookbook_dataset_for return_dataset } # === RBT::Cookbooks::Cookbook.return_cookbook_dataset_for
       self.instance_eval { alias cookbook_dataset_for        return_dataset } # === RBT::Cookbooks::Cookbook.cookbook_dataset_for

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.does_include?
  # ========================================================================= #
  def self.does_include?(
      this_program,
      optional_value = :default
    )
    RBT.does_include?(this_program, optional_value)
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.main_program_entries?
  # ========================================================================= #
  def self.main_program_entries?
    ARRAY_MAIN_PROGRAM_ENTRIES
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.opnn
  # ========================================================================= #
  def self.opnn(i = NAMESPACE)
    if i.is_a? String
      i = { namespace: i }
    end
    Opn.opn(i)
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook.feedback_format_specification_of
  #
  # This is the method that will tell us whether an entry in the cookbooks
  # settings is an Array or a String or a Hash or a Boolean value.
  #
  # Usage example:
  #
  #   RBT::Action::Cookbooks::SanitizeCookbook.feedback_format_specification_of :headers
  #
  # From the commandline:
  #
  #   rbt --format-specification-of=extract_to
  #   rbt --format-specification-of=extract_to
  #
  # ========================================================================= #
  def self.feedback_format_specification_of(
      i = 'extract_to'
    )
    i = i.to_s unless i.is_a? String # We need a String here.
    # ======================================================================= #
    # First, load the YAML file that has our dataset.
    # ======================================================================= #
    dataset = YAML.load_file(
      ::RBT.return_file_specification_of_registered_cookbook_entries # We need this method.
    )
    if dataset.has_key? i
      format_type = dataset[i]
      RBT.opnn; e "The format type for the key `#{Colours.simp(i)}` is:"
      e
      e "  #{Colours.sfancy(format_type)}"
      e
    else
      RBT.opnn; e "No format called `#{Colours.simp(i)}` is known."
    end
  end

  # ========================================================================= #
  # === may_we_instantiate_a_new_raw_cookbook?
  # ========================================================================= #
  def may_we_instantiate_a_new_raw_cookbook?
    @internal_hash[:may_we_instantiate_a_new_raw_cookbook]
  end

  # ========================================================================= #
  # === do_not_instantiate_a_new_raw_cookbook
  # ========================================================================= #
  def do_not_instantiate_a_new_raw_cookbook
    @internal_hash[:may_we_instantiate_a_new_raw_cookbook] = false
  end

  # ========================================================================= #
  # === merge_in_the_dataset_derived_from_the_raw_cookbook
  # ========================================================================= #
  def merge_in_the_dataset_derived_from_the_raw_cookbook(
      dataset = raw_cookbook_dataset?
    )
    merge_in_this_dataset(dataset)
  end

  # ========================================================================= #
  # === merge_in_this_dataset
  # ========================================================================= #
  def merge_in_this_dataset(dataset)
    # ======================================================================= #
    # Next "copy" the dataset onto the main Hash.
    # ======================================================================= #
    if dataset
      dataset.each_pair {|key, value|
        key = key.to_sym
        @internal_hash[:dataset][key] = value
      }
    end
  end

  # ========================================================================= #
  # === do_convert_the_dataset
  #
  # This method will convert the raw-dataset into the sanitized dataset.
  # ========================================================================= #
  def do_convert_the_dataset(
      url1 = :default,
      url2 = :default
    )
    set_archive_type
    parse_url1_and_set_all_associated_values(url1)
    check_for_outdated_entries(dataset?)
    sanitize_everything(url1, url2)
    determine_all_relevant_values_for_the_dataset_variable
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    find_this_program
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::SanitizeCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT::Actions::Cookbooks.new
#
# Invocation examples:
#
#   ruby     = RBT::Actions::Cookbooks.new 'ruby'
#   zip      = RBT::Actions::Cookbooks.new 'zip'
#   xxf86dga = RBT::Actions::Cookbooks.new 'xxf86dga'
#
# =========================================================================== #
def self.new(i = 'htop')
  RBT::Action::Cookbooks::SanitizeCookbook.new(i)
end; self.instance_eval { alias []                 new } # === RBT::Cookbooks[]
     self.instance_eval { alias load_this_cookbook new } # === RBT::Cookbooks.load_this_cookbook
     self.instance_eval { alias cookbook           new } # === RBT::Cookbooks.cookbook

end

end

# =========================================================================== #
# === RBT.return_cookbook_for
#
# This is an easier toplevel method to create a new cookbook-instance.
# Use with care, though.
# =========================================================================== #
def self.return_cookbook_for(
    i = :lua,
    &block
  )
  if block_given?
    return RBT::Action::Cookbooks::SanitizeCookbook.new(i, &block)
  else
    return RBT::Action::Cookbooks::SanitizeCookbook.new(i) { :fast }
  end
end; self.instance_eval { alias load_this_cookbook return_cookbook_for } # === RBT.load_this_cookbook
     self.instance_eval { alias cookbook           return_cookbook_for } # === RBT.cookbook
     self.instance_eval { alias cookbook_for       return_cookbook_for } # === RBT.cookbook_for

# =========================================================================== #
# === RBT.return_description_of
#
# This method will return the description of the given program at hand.
#
# Usage example:
#
#   x = RBT.return_description_of(:ruby)
#
# =========================================================================== #
def self.return_description_of(i)
  i = i.first if i.is_a? Array
  _ = RBT::Action::Cookbooks::SanitizeCookbook.new(i)
  return _.description?.to_s
end; self.instance_eval { alias return_description_of? return_description_of } # === RBT.return_description_of?

# =========================================================================== #
# === RBT.feedback_description_of_via_sanitize_cookbook
#
# Delegate towards the method .feedback_description_of().
#
# Invocation example:
#
#   RBT.feedback_description_of(:htop)
#
# =========================================================================== #
def self.feedback_description_of_via_sanitize_cookbook(i)
  i = i.first if i.is_a? Array
  _ = RBT::Action::Cookbooks::SanitizeCookbook.new(i)
  _.feedback_description
end; self.instance_eval { alias description_of? feedback_description_of_via_sanitize_cookbook } # === RBT.description_of?

# =========================================================================== #
# === RBT.return_cookbook_dataset_for
#
# This will return the dataset for a given program, in a fast manner.
# =========================================================================== #
def self.return_cookbook_dataset_for(i)
  RBT::Action::Cookbooks::SanitizeCookbook.return_dataset(i) { :fast }
end; self.instance_eval { alias cookbook_dataset_for return_cookbook_dataset_for } # === RBT.cookbook_dataset_for

# =========================================================================== #
# === RBT.return_configure_options_of_this_program
#
# This method will return the GNU configure options of a program - that
# is options such as "--enable-static" that may be passed to "./configure"
# on the commandline.
#
# Usage example:
#
#   RBT.return_configure_options_of_this_program('php') # => "--enable-bcmath" and so forth
#
# =========================================================================== #
def self.return_configure_options_of_this_program(i)
  action(:sanitize_cookbook, i).configure_options?
end

# =========================================================================== #
# === RBT.sanitize_cookbook
# =========================================================================== #
def self.sanitize_cookbook(
    i            = ARGV,
    options_hash = {},
    &block
  )
  sanitize_cookbook = RBT::Action::Cookbooks::SanitizeCookbook.new(
    i,
    :do_not_run_yet,
    &block
  )
  sanitize_cookbook.menu(options_hash) unless options_hash.empty?
  sanitize_cookbook.run
  return sanitize_cookbook
end

# =========================================================================== #
# === RBT::Cookbook
#
# This will also "alias" methods such as RBT::Cookbook[].
# =========================================================================== #
Cookbook = RBT::Action::Cookbooks::SanitizeCookbook

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours/autoinclude'
  _ = RBT::Action::Cookbooks::SanitizeCookbook.new(ARGV) { :simple } # { :fast }
  e "#{Colours.rev}The available tags are:"
  e
  pp _.tags?
  e
  e "#{Colours.rev}Short description: "+
    Colours::HtmlColoursMethods.steelblue(_.short_description?)
  e
  e "#{Colours.rev}The dataset is:"
  e
  pp _.dataset?
  e
  _.report
  # e 'The mode is: '+_.mode?.to_s
end # scookie1 0install
    # scookie1 htop
    # sanitizecookbook webkitgtk
    # sanitizecookbook zsync
    # sanitizecookbook modemmanager
    # sanitizecookbook libinput
    # sanitizecookbook poppler
    # sanitizecookbook 0install
