#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/menu.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  # ======================================================================= #
  # === menu                                                     (menu tag)
  # ======================================================================= #
  def menu(
      i = real_commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    elsif i.is_a? Hash
      # ======================================================================= #
      # === Handle Hashes next
      #
      # Such a Hash may look like this:
      #
      #   { :prefix=>"/usr/" }
      #
      # ======================================================================= #
      if i.has_key? :prefix
        set_user_prefix(
          i.delete(:prefix)
        )
      end
    else
      case i # (case tag, cas tag)
      # ===================================================================== #
      # === Create an AppDir program
      #
      # This entry point allows you to create an AppDir program in
      # the respective $PROGRAMS hierarchy - which is usually at
      # /Programs/.
      #
      # Invocation example:
      #
      #   rbt --create-program=foobar-1.0
      #
      # ===================================================================== #
      when /^-?-?create(-|_)?program=(.+)$/i
        create_program($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Uninstall a specific AppDir                       (uninstall tag)
      #
      # This option allows you to uninstall a specific AppDir again.
      #
      # All clean-up steps will be run.
      #
      # Invocation example:
      #
      #   rbt evince --uninstall-appdir
      #
      # ===================================================================== #
      when /^-?-?uninstall(_|-| )?appdir$/i,
           /^-?-?uninstall$/i
        uninstall_this_appdir
        exit_program
      # ===================================================================== #
      # === Package this appdir
      #
      # This entry point allows us to package an existing AppDir.
      #
      # Invocation examples:
      #
      #   ry --package-this-app-dir=htop
      #   ry --package-this-appdir=htop
      #   ry --package-appdir=htop
      #
      # ===================================================================== #
      when /^-?-?package(_|-| )?this(_|-| )?app(_|-| )?dir=(.+)$/i, # $4
           /^-?-?package(_|-| )?this(_|-| )?appdir=(.+)$/i,       # $3
           /^-?-?package(_|-| )?appdir=(.+)$/i                  # $2
        _ = $2.to_s.dup
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        package_this_appdir(_)
      # ===================================================================== #
      # === Check out the latest source code of the given program
      #
      # This variant is similar to the --git-download option, but
      # we will specify which program to download directly, so the
      # "=" is a mandatory component. Thus, we need one less argument
      # to pass here.
      #
      # This entry point is mostly helpful for when we want to be
      # lazy and efficient.
      #
      # Invocation examples:
      #
      #   rbt --git-clone=htop
      #   rbt --git-clone=libxmlb
      #
      # ===================================================================== #
      when /^-?-?git(-|_| )?clone=(.+)$/i, # === $2
           /^-?-?from(-|_| )?git=(.+)$/i   # === $2
        download_the_latest_source_code_of_this_program($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Use program URL
      #
      # Use url. Overrules the program_name of the cookbook yaml file.
      #
      #   ry ffmpeg use_url
      #   ry ffmpeg --use_url
      #
      # ===================================================================== #
      when /^-?-?us?e?(-|_| )?url$/i,
           /^-?-?u(-|_| )?url$/i
        use_url
      # ===================================================================== #
      # === Easily compile the KDE5 foundation module
      #
      # This entry point allows you to compile all of the KDE5 foundation
      # programs. The foundation starts with "extracmakemodules" and so
      # forth.
      #
      # Invocation examples:
      #
      #   rbt --compile-kde-framework
      #   rbt --compile-kde-frameworks
      #   rbt --compile-kde-foundation
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?kde(-|_| )?frameworks?$/i,
           /^-?-?compile(-|_| )?kde(-|_| )?foundation$/i,
           /^-?-?compile(-|_| )?kde(-|_| )?base$/i,
           /^-?-?kde(-|_| )?frameworks$/i,# ← This alias may change at a later time.
           /^-?-?kde(-|_| )?foundation$/i,
           /^-?-?kde(-|_| )?base$/i,
           /^-?-?kde(-|_| )?base$/i
        orev "All KDE5 foundation programs will be compiled next."
        set_compile_these_programs(
          RBT.return_kde5_foundation
        )
      # ===================================================================== #
      # === Update all KDE5 porting aids components
      #
      # This entry point allows us to update the KDE porting aids from
      # within RBT itself.
      #
      # Invocation example:
      #
      #   rbt --update_porting_aids
      #   rbt --updateportingaids
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?porting(-|_| )?aids$/i
        update_kde_porting_aids
        exit_program
      # ===================================================================== #
      # === Batch compile the given input
      #
      # If you need to batch-compile several programs, then this is
      # one way to do so. This used to be called "chain compiling"
      # but is now called batch-compiling.
      #
      # Invocation examples:
      #
      #   rbt --batch-compile=mate
      #   rbt --batch-compile=python,php,perl,ruby
      #   rbt --chained=xfce
      #   rbt --chained=mate
      #   rbt --batch-compile=kde5_porting_aids
      #   rbt --chained=kde5_porting_aids
      #   rbt --batch-compile=kde5_foundation
      #   rbt --chained-compiling=kde5_foundation
      #
      # ===================================================================== #
      when /^-?-?batch-?compile=(.+)$/i, # URL: http://rubular.com/r/uItIs454yH
           /^-?-?chained=(.+)$/i,
           /^-?-?batch=(.+)$/i,                              # === $1
           /^-?-?chained(-|_| )?compiling=(.+)$/i,           # === $2
           /^-?-?do(-|_| )?chained(-|_| )?compiling=(.+)$/i # === $3
        match = $1.to_s.dup
        match = $2.to_s.dup if $2
        match = $3.to_s.dup if $3
        if match and match.include? ','
          these_programs = match.split(',')
        else
          these_programs = try_to_batch_compile_this_stack(match)
        end
        if these_programs
          orev "We will batch-compile these "\
               "#{simp(these_programs.size.to_s)} #{rev}programs next."
          e; these_programs.each {|install_this_program|
            e sfancy("  #{install_this_program}")
          }; e
          set_compile_these_programs(these_programs)
        else
          orev "No result found for #{orange(match)}#{rev}."
          exit_program
        end
      # ===================================================================== #
      # === Permanently disable the colours
      #
      # If you wish to permanently disable the colours for the RBT project,
      # use this entry point here.
      #
      # Invocation example:
      #
      #   rbt --permanently-disable-colours
      #
      # ===================================================================== #
      when /^-?-?permanentl?y?(-|_)?disable(-|_)?colou?rs$/,
           /^-?-?permanentl?y?(-|_)?no(-|_)?colours$/
        do_permanently_disable_the_colours
        exit_program
      # ===================================================================== #
      # === Permanently enable the colours
      #
      # If you wish to permanently enable the colours for the RBT project,
      # use this entry point here. This is specifically enabled to allow
      # us to reset after having done the entry point above, e. g. at
      # rbt --permanently-disable-colours.
      #
      # Invocation example:
      #
      #   rbt --permanently-enable-colours
      #
      # ===================================================================== #
      when /^-?-?permanentl?y?(-|_)?enable(-|_)?colou?rs$/,
           /^-?-?permanentl?y?(-|_)?colours$/
        do_permanently_enable_the_colours
        exit_program
      # ===================================================================== #
      # === Permanently disable ccache
      #
      # Use this entry point if you do not want to use ccache.
      #
      # Invocation example:
      #
      #   rbt --permanently-disable-ccache
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?disable(-|_| )?ccache$/,
           /^-?-?do(-|_| )?permanently(-|_| )?disable(-|_| )?ccache$/
        do_permanently_disable_ccache
        exit_program
      # ===================================================================== #
      # === Permanently set programs dir
      #
      # This compiles gcc with "language rich" configure options.
      #
      # Invocation example:
      #
      #   ry gcc --ntrad --language-rich
      #
      # ===================================================================== #
      when /^-?-?language(_|-| )?rich$/i
        try_to_use_this_profile(i.to_s.delete('-_'))
      # ===================================================================== #
      # === Permanently enable ccache
      #
      # Use this entry point if you wish to make use of ccache permanently.
      #
      # Invocation example:
      #
      #   rbt --permanently-enable-ccache
      #   ry --permanently-use-ccache
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?enable(-|_| )?ccache$/,
           /^-?-?permanently(-|_| )?use(-|_| )?ccache$/,
           /^-?-?do(-|_| )?permanently(-|_| )?enable(-|_| )?ccache$/,
           /^-?-?enable(-|_| )?ccache$/
        do_permanently_enable_ccache
        exit_program
      # ===================================================================== #
      # === Show a specific compile chain
      #
      # This entry point allows you to e. g. display which members
      # belong to the compile-chain plasma.
      #
      # Invocation examples:
      #
      #   rbt --view=kde5_plasma
      #   rbt --show-chain=plasma
      #   rbt --compile-chain-of=kde5_plasma
      #   rbt --show-compile-chain-of=kde5_plasma
      #   rbt --show-compile-chain-of=mate
      #   rbt --show-compile-chain=mate
      #   rbt --show-compile-chain=lxqt
      #
      # ===================================================================== #
      when /^-?-?view=(.+)$/i,                                 # ← Aim for $1.
           /^-?-?show(-|_)?chain=(.+)$/i,                      # ← Aim for $2.
           /^-?-?compile(-|_)?chain(-|_)?of=(.+)$/i,           # ← Aim for $3.
           /^-?-?show(-|_)?compile(-|_)?chain(-|_)?of=(.+)$/i, # ← Aim for $4.
           /^-?-?show(-|_)?compile(-|_)?chain=(.+)$/i          # ← Aim for $3.
        if $4
          of_this_program = $4.to_s.dup
        elsif $3
          of_this_program = $3.to_s.dup
        elsif $2
          of_this_program = $2.to_s.dup
        elsif $1
          of_this_program = $1.to_s.dup
        end
        show_compile_chain(of_this_program)
        exit_program
      # ===================================================================== #
      # === Install the dependencies of the given program at hand
      #
      # This entry point allows the user to install the dependencies of
      # the given program at hand. This will NOT check for any other
      # dependencies, so it will not be a so-called
      # "transverse-installation" that includes checks for such add-ons.
      #
      # The entry point will only install the dependencies that are
      # directly noted down in the "required_deps_on" part of the
      # relevant .yml file, so in some ways it is simple for the
      # user as well.
      #
      # Note that the program itself will NOT be installed through
      # this entry point.
      #
      # Invocation examples:
      #
      #   rbt rails --install-its-dependencies
      #   rbt zee --install-its-dependencies
      #
      # ===================================================================== #
      when /^-?-?install(-|_| )?its(-|_| )?dependencies$/i
        install_the_dependencies_of_this_program
      # ===================================================================== #
      # === Show all available (registered) gems to the user
      #
      # This entry point allows the user to display, onto the commandline,
      # which gems are registered in the RBT project.
      #
      # Invocation examples:
      #
      #   rbt --registered-gems
      #   rbt --available-gems
      #   rbt --ruby?
      #
      # ===================================================================== #
      when /^-?-?registered(-|_| )?gems$/i,
           /^-?-?show(-|_| )?registered(-|_| )?gems$/i,
           /^-?-?available(-|_| )?gems$/i,
           /^-?-?rubygems$/i,
           /^-?-?ruby\?$/i
        first_commandline_argument = first_commandline_argument?
        case first_commandline_argument
        when 'ruby','rubygems'
        else
          show_the_available_gems
          exit_program
        end
      # ===================================================================== #
      # === Clear the ENV (environment) variable
      #
      # This entry point allows us to clear the environment that ruby
      # uses, aka ENV.
      #
      # Invocation example:
      #
      #   ry gtk --clear-env --ntrad
      #
      # ===================================================================== #
      when /^-?-?clear(_|-| )?env$/
        clear_env_variables_but_ensure_the_path_variable_works
      # ===================================================================== #
      # === Compile certain applications from the KDE stack, based on a range
      #
      # This entry point allows the user to compile several programs
      # that belong to the KDE stack. A number has to be specified
      # here.
      #
      # For example:
      #
      #   ry --kde1-first=10
      #
      # This command would take the programs listed under KDE5 foundation
      # ("kde1") and try to compile the first 10 of these programs.
      #
      # Usage examples:
      #
      #   ry --kde1-first=10
      #   ry --kde1-first=20
      #   ry --kde2-first=20
      #   ry --kde2-first=30
      #
      # ===================================================================== #
      when /^-?-?kde\d{1,2}(-|_| )?first=(\d+)$/i
        try_to_compile_these_kde_programs(i)
      # ===================================================================== #
      # === Disable the use of md5sum calculation
      #
      # Invocation example:
      #
      #   ry --no-md5sum
      #
      # ===================================================================== #
      when /^-?-?no(_|-| )?md5sum$/i,
           /^-?-?disable(_|-| )?md5sum(_|-| )?calculation$/i
        disable_md5sum_calculation
      # ===================================================================== #
      # === /usr/local/ prefix
      #
      # This is a convenient feature mostly, in that we can quickly
      # compile into the /usr/local/ prefix.
      #
      # Usage examples:
      #
      #   ry gcc ULOCAL
      #   ry htop ULOCAL
      #
      # ===================================================================== #
      when /^ULOCAL$/,
           /^-?-?opt(_|-| )?prefix$/
        set_user_prefix(i)
      # ===================================================================== #
      # === Feedback the prefix in use
      #
      # If you wish to determine the --prefix that will be used for
      # the given program at hand, use this entry point here.
      #
      # Invocation example:
      #
      #   rbt htop --prefix?
      #
      # ===================================================================== #
      when *ARRAY_HELP_PREFIX_IN_USE
        feedback_prefix_in_use
      # ===================================================================== #
      # === Feedback multiple versions
      #
      # Invocation example:
      #
      #   ry --multiple-versions
      #
      # ===================================================================== #
      when /^-?-?multiple-?versions$/,
           /^-?-?find-?duplicates$/,
           'mversions'
        RBT.find_multiple_versions
        exit_program
      # ===================================================================== #
      # === Run the post-installation step at once
      #
      # This entry point allows us to specifically run the various
      # postinstallation steps.
      #
      # Invocation example:
      #
      #   rbt htop --do-postinstall-step
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?postinstall(_|-| )?step$/i
        consider_performing_postinstallation_actions
        exit_program
      # ===================================================================== #
      # === Enable the -O3 optimization
      #
      # This will enable -O3 optimizations via GCC. I am unsure whether
      # it will work on clang/LLVM.
      #
      # Invocation example:
      #
      #   ry gnomedesktop --opt -O3
      #   ry gnomedesktop --opt --speedy
      #
      # ===================================================================== #
      when /^-?-O3/i,
           /^-?-?superspeed$/,
           /^-?-?speed$/,
           /^-?-?speedy$/,
	         /^-?-?fast$/
        new_value = old_cflags?
        if new_value.include? '-O2'
          new_value.sub!(/-O2/,'-O3')
        else
          new_value.prepend('-O3 ')
        end
        ENV['CFLAGS'] = new_value
        orev 'Using -O3 optimization next. The new CFLAGS are: '
        e
        orev "  #{steelblue(ENV['CFLAGS'])}"
        e
      # ===================================================================== #
      # === Generate the pdf tutorial, then upload it
      #
      # Invocation example:
      #
      #   rbt --generate_pdf_tutorial_then_upload_it
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?pdf(_|-| )?tutorial(_|-| )?then(_|-| )?upload(_|-| )?it$/i
        ::RBT.generate_pdf_tutorial_then_upload_it
        exit_program
      # ===================================================================== #
      # === Publish a list of all programs versions
      #
      # Invocation example:
      #
      #   rbt --publish-list-of-all-programs-version
      #
      # ===================================================================== #
      when /^-?-?publish(_|-| )?list(_|-| )?of(_|-| )?/i # This is copied 1:1 from the Cookbooks project.
        RBT.publish_list_of_all_programs_version
        exit_program
      # ===================================================================== #
      # === Feedback Programs Directory
      #
      # This entry point shows us where the /Programs/ directory is at.
      #
      # Invocation example:
      #
      #   ry --programs_dir?
      #
      # ===================================================================== #
      when /^-?-?programs(_|-| )?dir\??$/,'pdir?'
        feedback_programs_dir
        exit_program
      # ===================================================================== #
      # === rbt --repeat_last_configure_option
      # ===================================================================== #
      when /^-?-?repeat(_|-| )?last(_|-| )?configure(_|-| )?option$/i,
           /^-?-?repeat$/
        esystem(return_last_configure_command)
        exit_program
      # ===================================================================== #
      # === Batch-download all source archives
      #
      # This entry point allows us to batch-download all source archives
      # onto the local filesystem.
      #
      # Invocation example:
      #
      #   rbt --batch-download-all-source-archives
      #   rbt --download-all-source-archives
      #
      # ===================================================================== #
      when /^-?-?batch(_|-| )?download(_|-| )?all(_|-| )?source(_|-| )?archives$/i,
           /^-?-?download(_|-| )?all(_|-| )?source(_|-| )?archives$/i
        batch_download_all_source_archives
        exit_program
      # ===================================================================== #
      # === Extract the linux kernel tarball only
      #
      # This entry point can be used to extract the linux kernel only,
      # to the specified target specified via extract_to: in the file
      # called 'linux.yml'.
      #
      # Invocation examples:
      #
      #   ry --extract-linux
      #   ry --extract-the-linux-kernel
      #
      # ===================================================================== #
      when /^-?-?extract(-|_| )?linux$/i,
           /^-?-?extract(-|_| )?the(-|_| )?linux(-|_| )?kernel(-|_| )?archive$/i,
           /^-?-?extract(-|_| )?the(-|_| )?linux-kernel$/i
        extract_the_linux_kernel_archive
        exit_program
      # ===================================================================== #
      # === Start the sinatra-interface of RBT
      #
      # Invocation example:
      #
      #   rbt --sinatra
      #
      # ===================================================================== #
      when /^-?-?sinatra$/i
        do_start_the_sinatra_interface
        exit_program
      # ===================================================================== #
      # === Create a specific shellscript (shell recipe)
      #
      # Create a specific (single) shellscript via this entry point here.
      #
      # Invocation examples:
      #
      #   ry sed        --generate_shellscript
      #   ry python     --standalone
      #   ry gnunetgtk  --create_shellscript
      #   ry wxwidget   --shellscript
      #   ry php        cshell
      #   ry htop       generate_shellscript
      #
      # ===================================================================== #
      when *ARRAY_HELP_CREATE_SHELLSCRIPT_OPTIONS
        unless yaml_dataset?
          this_program = compile_these_programs?.first
          assign_to_the_main_dataset(
            initialize_dataset_for(this_program)
          )
        end
        create_shellscript(
          dataset?,
          :save_into_file,
          :display_result
        )
        exit_program
      # ===================================================================== #
      # === Use instructions to GNU configure from a local file
      #
      # This entry point allows us to use configure-options stored in
      # a local file. This is useful because other parts of the RBT
      # suite can generate this file, which we can then easily use.
      #
      # For example, if the xorg-server does not work then copy/pasting
      # through the mouse may be harder. That is one possible use
      # case for this option.
      #
      # Invocation example:
      #
      #   ry gcc --use-instructions-from-this-file=/Depot/foobar.md
      #   ry php --trad --use-this-file=/Depot/j/foo.md
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?instructions(-|_| )?from(-|_| )?this(-|_| )?file=(.+)$/i, # $5
           /^-?-?use(-|_| )?this(-|_| )?file=(.+)$/i # $3
        _ = $3.to_s.dup if $3
        _ = $5.to_s.dup if $5
        consider_using_this_file_for_the_configure_options_that_are_to_be_used(_)
      # ===================================================================== #
      # === Compile all LXQT-desktop components
      #
      # This entry point allows us to compile all LXQT-components.
      #
      # Invocation example:
      #
      #   ry --compile-all-lxqt-components
      #   ry --batch-compile-lxqt
      #   ry --lxqt
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?all(_|-| )?lxqt(_|-| )?components$/i,
           /^-?-?compile(_|-| )?lxqt$/i,
           /^-?-?batch(_|-| )?compile(_|-| )?lxqt$/i,
           /^-?-?lxqt$/i
        compile_all_lxqt_components
      # ===================================================================== #
      # === Convert a Gobolinux-like Package into the traditional structure:
      #
      # This entry point allows us to take a program structure such as
      # /Programs/Ruby/2.5.1/ and "convert" it into the "/usr/"
      # hierarchy.
      #
      # Usage examples:
      #
      #   ry php convert
      #   ry htop convert
      #   ry ruby --convert-to-traditional
      #
      # ===================================================================== #
      when /^-?-?convert(_|-| )?to(_|-| )?traditional$/i,
           'convert','conv','convert_package',
           'cpackage',
           'to_usr',
           'dumpp',
           'tousr',
           'converttotrad',
           'totrad',
           'cpa',
           'ctt',
           'to_traditional',
           'ctotrad'
        convert_to_traditional
        exit_program
      # ===================================================================== #
      # === rbt LAST_DOWNLOADED
      #
      # The input is normally downcased - but I will keep the upcased
      # variant as main entry point; it seems easer to remember.
      # ===================================================================== #
      when *ARRAY_HELP_LAST_DOWNLOADED_OPTIONS
        # =================================================================== #
        # The format of the following file may be like this here:
        #   /home/x/src/redmine/redmine-3.2.0.tar.gz # 2015-12-31 23:48:48 +0100
        # =================================================================== #
        file = RBT.last_downloaded_file
        if File.exist? file
          # ================================================================= #
          # We grab the name of the program found in the above file.
          # ================================================================= #
          i = program_information_return_program_name(
            File.basename(File.readlines(file).first)
          )
          i = [i] # We expect it to be an Array lateron.
          clear_commandline_options
        else
          ewarn 'No file at `'+sfile(file)+'` exists. Exiting now.'
          exit_program
        end
      # ===================================================================== #
      # === Compile every component up until KDE konsole
      #
      # This entry point can be used to compile all components belonging
      # to the KDE stack, up to (and including) the KDE konsole.
      #
      # Invocation example:
      #
      #   ry --compile-all-up-until-konsole
      #   ry --compile-towards-konsole
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?up(-|_| )?until(-|_| )?konsole$/i,
           /^-?-?compile(-|_| )?towards(-|_| )?konsole$/i
        compile_all_up_until_konsole
        exit_program
      # ===================================================================== #
      # === Enable cmake
      #
      # We deliberately enable cmake here, via this commandline flag.
      #
      # Usage example:
      #
      #   ry htop cmake
      #
      # ===================================================================== #
      when /^-?-?cmake$/i,
           /^-?-?use(_|-| )?ccmake$/
        do_use_cmake
      # ===================================================================== #
      # === Shortcut for a specific program version
      #
      # This entry point allows us to pick a specific version more easily.
      #
      # See the following usage example.
      #
      # Usage example:
      #
      #   ry python --2
      #
      # ===================================================================== #
      when /^--(\d+)$/
        this_program = compile_these_programs?.first
        directory = source_directory?+this_program+'/'
        available_programs = Dir[directory+'*']
        this_version = $1.to_s.dup
        selection = available_programs.select {|entry|
          entry =~ /-#{this_version}/
        }
        if selection.empty?
          orev "Not found any local program version for "\
               "the input `#{sfancy(this_version)}#{rev}`."
        else
          _ = selection.first
          this_version = program_information_return_version_of(_)
          set_program_version(this_version)
        end
      # ===================================================================== #
      # === Do not use sed at all
      #
      # This entry point will disable sed.
      #
      # Usage examples:
      #
      #   rbt htop --disable-sed
      #   rbt htop --disable-sed-modifications
      #   rbt extracmakemodules --disable-sed-modifications trad
      #
      # ===================================================================== #
      when /^-?-?disable(-|_| )?sed$/,
           /^-?-?disable(-|_| )?sed(-|_| )?modifications$/,
           /^-?-?do(-|_| )?not(-|_| )?use(-|_| )?sed$/,
           /^-?-?dont(-|_| )?use(-|_| )?sed$/
        do_not_use_sed(:be_verbose)
      # ===================================================================== #
      # === Apply only the sed operation next
      #
      # Invocation examples:
      #
      #   rbt htop --only-sed
      #   rbt krb5 --only-sed
      #
      # ===================================================================== #
      when /^-?-?only(-|_| )?sed$/i
        determine_the_configure_base_directory_to_use
        extract_program
        cd_to_the_configure_base_directory_to_be_used
        check_for_sed_action
        exit_program
      # ===================================================================== #
      # === Use a white background
      #
      # Invocation example:
      #
      #   ry htop --whitebg
      #
      # ===================================================================== #
      when /^-?-?background-?white$/,'--white-background',
           /^-?-?whitebg$/,
           /^-?-?do_?use_?a_?white_?background/
        do_use_a_white_background
      # ===================================================================== #
      # === Copy the custom .yml for the file into the current working
      #     directory
      #
      # This entry point allows the user to quickly copy the current
      # .yml file into the current working directory.
      #
      # Usage examples:
      #
      #   ry php --generate-custom-yaml
      #   ry ruby --generate_yaml
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?custom(_|-| )?yaml$/i,
           /^-?-?generate(_|-| )?yaml$/i,
           /^-?-?copy(_|-| )?yaml(_|-| )?file$/i
        copy_this_yaml_file(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Honour a specific licence and only compile if it matches that licence
      #
      # This entry point allows the user to accept only programs with a
      # certain licence. So if the licence does NOT match the one given
      # on the commandline then RBT::Action::Installer will refuse to compile
      # it.
      #
      # Usage examples:
      #
      #   rbt php --honour-this-licence=GPLv3
      #   rbt php --check-for-licence=GPLv3
      #   rbt php --check-licence=GPLv3
      #   rbt php --only-this-licence=GPLv3
      #   rbt php --only-this-licence=gplv3
      #   rbt xfce4panelprofiles --licence=gplv2
      #   rbt exo --licence=gplv2
      #   rbt exo --licence=gplv3
      #   rbt plasmapa --licence=gplv2
      #
      # ===================================================================== #
      when /^-?-?honour(_|-| )?this(_|-| )?licence=(.+)$/i, # === $3
           /^-?-?check(_|-| )?for(_|-| )?licence=(.+)$/i,   # === $3
           /^-?-?only(_|-| )?this(_|-| )?licence=(.+)$/i,   # === $3
           /^-?-?check(_|-| )?licence=(.+)$/i,              # === $2
           /^-?-?licence=(.+)$/i                            # === $1
        _ = $1
        _ = $2 if $2
        _ = $3 if $3
        @internal_hash[:check_for_this_licence] = RBT.sanitize_this_licence(_)
      # ===================================================================== #
      # === rbt --allowed_entries?
      #
      # Invocation example:
      #
      #   rbt --allowed-cookbook-entries
      #
      # ===================================================================== #
      when '--allowed_entries?','allowed_entries?',
           '--allowed_entries',
           /^-?-?allowed(_|-| )?cookbook(_|-| )?entries/i
        require 'rbt/checks_and_validations/allowed_cookbook_entries.rb'
        RBT::Cookbooks::AllowedCookbookEntries.new
        exit_program
      # ===================================================================== #
      # === Querying whether RBT will symlink .gir files
      #
      # Invocation example:
      #
      #   rbt upower --ntrad --do-not-symlink-gir-file
      #   rbt gnomedesktop --ntrad --do-not-symlink-gir-files
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?symlink(_|-| )?gir(_|-| )?files?$/i
        orev "We will not symlink #{steelblue('.gir')}#{rev} files."
        @internal_hash[:symlink_gir_files] = false
      # ===================================================================== #
      # === rbt PWD
      # ===================================================================== #
      when /^PWD$/i
        _ = File.basename(return_pwd)
        set_compile_this_program(_)
        set_compile_these_programs(_)
      # ===================================================================== #
      # === Feedback extended description about a program
      #
      # Feedback extended description to the user.
      #
      # ry htop desc2
      # ===================================================================== #
      when /^-?-?description2$/,
           'desc2'
        feedback_extra_description
        feedback_description
        exit_program
      # ===================================================================== #
      # === Update library names
      #
      # Invocation example:
      #
      #   ry --uln
      #
      # ===================================================================== #
      when /^-?-?update(_|-| )?library(_|-| )?names/i,
           /^-?-?uln/i,
           'registered_libraries'
        update_library_names
        exit_program
      # ===================================================================== #
      # === Feedback every case option of the main menu here
      #
      # Invocation example:
      #
      #   ry htop case_options?
      #
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_EVERY_CASE_OPTION_OPTIONS
        feedback_all_case_options_of_the_main_menu
        exit_program
      # ===================================================================== #
      # === Use a specific build directory
      #
      # Sometimes you may want to use another build directory.
      #
      # This options here ensures that you can do so.
      #
      # Usage example:
      #
      #   ry htop --use-this-build-dir=/Depot/j
      #   ry htop --build-dir=/Depot/j
      #   ry python --build-dir=/Depot/jjj/
      #
      # ===================================================================== #
      when /^-?-?use-this-build-dir=(.+)/,
           /^-?-?use-this-build-directory=(.+)/,
           /^-?-?build-dir=(.+)/
        do_use_a_build_directory :be_verbose
        this_build_directory = $1.to_s.dup
        use_this_build_directory(this_build_directory, :report_back)
      # ===================================================================== #
      # === Feedback all Libaries
      #
      # This will feedback all libraries that are installed by the given
      # program at hand. This of course requires of you to actually pass
      # in the name of the program.
      #
      # Invocation example:
      #
      #   ry htop --all-libraries?
      #
      # ===================================================================== #
      when /^-?-?all(-|_| )?libraries\??$/i,
           /^-?-?libraries\??$/i
        feedback_all_libraries
        exit_program
      # ===================================================================== #
      # === Report how many programs are registered
      #
      # Invocation example:
      #
      #   ry --report-n-programs
      #
      # ===================================================================== #
      when /^-?-?report(-|_| )?n(-|_| )?programs$/i,
           /^-?-?n(-|_| )?programs\??$/i
        report_n_programs
        exit_program
      # ===================================================================== #
      # === Show all programs that dependent on this program
      #
      # This entry point allows the user to show, for example, all
      # programs that have "cmake" as its dependency. In fact: this
      # was the primary use case for adding this functionality. I
      # needed to show all cmake-based programs that are registered
      # in the RBT project.
      #
      # Invocation examples:
      #
      #   rbt --show-programs-with-this-as-dependency=cmake
      #   rbt --dependencies-on=kio
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?programs(_|-| )?with(_|-| )?this(_|-| )?as(_|-| )?dependency=(.+)$/i, # === $6
           /^-?-?dependencies(_|-| )?on=(.+)$/i # === $2
        _ = $2.to_s.dup
        _ = $6.to_s.dup if $6
        show_programs_with_this_as_dependency(_)
        exit_program
      # ===================================================================== #
      # === Feedback program URL
      #
      # Invocation example:
      #
      #   ry libxrandr url2
      #
      # ===================================================================== #
      when 'url2'
        RBT::Cookbooks::Url.new(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Compile LLVM with support for clang
      #
      # This entry point exists so that we can compile llvm with clang
      # support. In my opinion LLVM without clang makes only little
      # sense on a full-scale linux system these days, so having clang
      # available by default is a big advantage. (I don't know why they
      # don't bundle the distribution together.)
      #
      # Invocation examples:
      #
      #   ry llvm --include-clang-for-compilation
      #   ry llvm --compile-clang
      #   ry llvm --include-clang-for-compilation --ntrad
      #
      # ===================================================================== #
      when /-?-?include(-|_| )?clang(-|_| )?for(-|_| )?compilation$/i,
           /-?-?compile(-|_| )?clang$/i
        do_include_clang_for_the_llvm_compilation
      # ===================================================================== #
      # === Set permanently extract to
      #
      # This entry point allows us to permanently redefine the extract_to
      # target.
      #
      # In other words, we will permanently determine where to we will
      # extract a given archive to.
      #
      # Usage example:
      #
      #   rbt --permanently_extract_to=pwd
      #   rbt --permanently_extract_to=/Depot
      #   rbt --permanently_extract_to=/tmp
      #   rbt --permanently-set-temp-dir=/user/bioinf4/Temp/
      #   rbt --permanently-set-temp-dir-to=/user/bioinf4/Temp/
      #   rbt --use-this-temp-dir=/Depot/
      #   rbt --use-temp=/Depot
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?extract(-|_| )?to=(.+)$/,                    # This is for $3
           /^-?-?permanently(-|_| )?set(-|_| )?temp(-|_| )?dir=(.+)$/,           # This is for $4
           /^-?-?permanently(-|_| )?set(-|_| )?temp(-|_| )?dir(-|_| )?to=(.+)$/, # This is for $5
           /^-?-?use(-|_| )?temp=(.+)$/                                          # This is for $2
        _ = $2.to_s.dup if $2
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        _ = $5.to_s.dup if $5
        # =================================================================== #
        # We also silently ensure that this directory exists, if it does
        # not yet exist that is.
        # =================================================================== #
        mkdir(_) { :be_quiet } unless File.directory?(_)
        mkdir(_+'rbt/') { :be_quiet } unless File.directory?(_+'rbt/')
        set_permanently_extract_to(_)
        exit_program
      # ===================================================================== #
      # === Permanently extract into the home directory
      #
      # This entry point allows us to extract into the home directory. That
      # way we should be able to easily compile programs in a restricted
      # environment. 
      #
      # Invocation examples:
      #
      #   ry --permanently-extract-to-home
      #   ry --permanently-extract-to-home-dir
      #   ry --extract-to-home
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_)?extract(-|_)?to(-|_)?home$/i,
           /^-?-?permanently(-|_)?extract(-|_)?to(-|_)?home(-|_)?dir$/i,
           /^-?-?extract(-|_)?to(-|_)?home$/i
        set_permanently_extract_to(:home_directory)
        exit_program
      # ===================================================================== #
      # === Compile all of KDE
      #
      # This entry point allows us to compile all of KDE.
      #
      # Invocation example:
      #
      #   rbt --all-of-kde
      #   rbt --compile-all-of-kde
      #
      # ===================================================================== #
      when /^-?-?all(_|-| )?of(_|-| )?kde$/i,
           /^-?-?compile(_|-| )?all(_|-| )?of(_|-| )?kde$/i
        compile_all_of_kde
        exit_program
      # ===================================================================== #
      # === Report the status of the KDE components on this computer
      #
      # This entry point will provide some more information about
      # the KDE components available on the given computer system
      # at hand.
      #
      # Invocation example:
      #
      #   rbt --kde-status
      #
      # ===================================================================== #
      when /^-?-?kde(_|-| )?status$/i,
           /^-?-?report(_|-| )?the(_|-| )?kde(_|-| )?status$/i
        report_the_kde_status
        exit_program
      # ===================================================================== #
      # === rbt --create-simple-version-file
      # ===================================================================== #
      when /^-?-?create(-|_| )?simple(-|_| )?version(-|_| )?file/
        require 'rbt/toplevel_methods/toplevel_methods.rb'
        RBT.create_simple_version_file
        exit_program
      # ===================================================================== #
      # === Show the last updated programs
      #
      # This entry point will simply show the last updated programs.
      #
      # Invocation examples:
      #
      #   ry --last-update
      #
      # ===================================================================== #
      when /^-?-?last(-|_| )?update?$/i
        require 'rbt/toplevel_methods/toplevel_methods.rb'
        e
        e steelblue(RBT.return_string_showing_the_last_updated_programs)+rev
        e
      # ===================================================================== #
      # === Query whether we are on a GoboLinux system
      #
      # This entry point can be used to determine whether the RBT scripts
      # are running on GoboLinux or whether they are not.
      #
      # Invocation examples:
      #
      #   ry --is-on-gobolinux?
      #   ry --are-we-on-gobolinux?
      #
      # ===================================================================== #
      when /^-?-?is(_|-| )?on(_|-| )?gobolinux\?$/i,
           /^-?-?are(_|-| )?we(_|-| )?on(_|-| )?gobolinux?$/i
        report_whether_we_are_on_gobolinux
        exit_program
      # ===================================================================== #
      # === Symlink all upcased programs to downcased variants
      #
      # This entry point will add a symlink for an existing, upcased
      # program directory. For example, /Programs/ZLib will receive
      # a symlink called /Programs/Zlib.
      #
      # Invocation example:
      #
      #   rbt --symlink-upcased-programs
      #
      # ===================================================================== #
      when /^-?-?symlink(_|-| )?all(_|-| )?upcased(_|-| )?programs(_|-| )?to(_|-| )?downcased(_|-| )?variants$/i,
           /^-?-?symlink(_|-| )?upcased(_|-| )?programs$/i,
           /^-?-?symlink(_|-| )?all(_|-| )?programs$/i,
           /^-?-?gobolinux(_|-| )?compatibility$/i,
           /^-?-?downcase(_|-| )?programs$/i,
           /^-?-?downcase-programs(_|-| )?directory$/i
        symlink_all_upcased_programs_to_downcased_variants
      # ===================================================================== #
      # === Upload a ruby binary
      #
      # Use this entry point to upload a ruby binary to a remote FTP site.
      #
      # Usage example:
      #
      #   rbt --upload-ruby-binary
      #
      # ===================================================================== #
      when /(-|_)?(-|_)?upload(-|_)?ruby(-|_)?binary/
        do_upload_ruby_binary
        exit_program
      # ===================================================================== #
      # === Do not use autogen
      #
      # This entry point here can be used to disable the use of autogen.
      #
      # This may sometimes be necessary if you do not want to run the
      # autogen.sh script for example, despite the yaml file associated
      # with the program defaulting to "do use autogen".
      #
      # Note that by default, RBT will not run autogen for most programs -
      # the default for "use autogen" is simply set to false. It is only
      # a few programs that may need a regeneration through autogen
      # anyway.
      #
      # Invocation examples:
      #
      #   ry gtk+ --disable-autogen
      #   ry gtk+ disable-autogen
      #   ry matesystemmonitor disable-autogen
      #
      # ===================================================================== #
      when *ARRAY_HELP_DISABLE_AUTOGEN
        disable_autogen
      # ===================================================================== #
      # === Symlink into the /usr/include/ hierarchy
      #
      # Invocation example:
      #
      #   rbt mpfr ntrad --symlink-headers
      #
      # ===================================================================== #
      when /^-?-?symlink(_|-| )?headers$/,
           /^-?-?symlink(_|-| )?h$/,
           'sheaders'
        verbose_do_symlink_headers_into_usr_include_hierarchy
      # ===================================================================== #
      # === Querying the Configuration Options
      #
      # This entry point allows the user to obtain some feedback
      # about the configuration options used by a given program.
      #
      # This may be the options passed to GNU configure, but also
      # options to cmake or meson - whatever the target program
      # uses.
      #
      # Usage example:
      #
      #   ry glibc qco
      #
      # ===================================================================== #
      when /^query(-|_| )?configuration(-|_| )?options$/,
           /^-?-?qco$/i
        query_configuration_options
      # ===================================================================== #
      # === Generate a large .html file containing all programs version
      #
      # Invocation example:
      #
      #   rbt --create-html-file
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?html(-|_| )?file$/i,
           /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?html(-|_| )?file$/i,
           /^-?-?program(-|_| )?versions$/i
        action(:create_programs_version_html_file)
        exit_programming
      # ===================================================================== #
      # === Query to which program a binary belongs to
      #
      # This entry point allows us to query to which program a given
      # locally existing binary belongs to. That way you can e. g.
      # query the /usr/bin/ hierarchy for programs, and the class here
      # will report to which program it belongs to.
      #
      # Invocation examples:
      #
      #   rbt --query-programs
      #   rbt --query-programs?
      #   rbt --attribute?
      # 
      # ===================================================================== #
      when /^-?-?query(-|_| )?programs$/i,
           /^-?-?query(-|_| )?programs\??$/i,
           /^-?-?attribute\??$/i
        require 'rbt/utility_scripts/query_binary_to_package.rb'
        RBT.query_binary_to_package('/usr/bin/')
      # ===================================================================== #
      # == Set to verbose output
      #
      # Invocation example:
      #
      #   ry htop --verbose
      #
      # ===================================================================== #
      when '--verbose','-verbose','verbose','verb','set_verbose','be_verbose',
           'beverbose','setverbose'
        set_be_verbose :report_to_the_user # Second true so we report to the user.
      # ===================================================================== #
      # === Verify the md5sum
      #
      # Examples for md5:
      #
      #   ry gcc3   md5
      #   ry python md
      #   ry ruby   --md
      #
      # ===================================================================== #
      when 'md5',/^-?-?md$/,'mdcheck'
        verbose_md5_checking
        exit_program
      # ===================================================================== #
      # === Run in simulation mode
      #
      # Use this to run a simulation.
      #
      # Invocation example:
      #
      #   ry htop simulation
      #
      # ===================================================================== #
      when *ARRAY_HELP_SIMULATION_MODE_OPTIONS
        enable_simulation('Running in simulation mode. No real '\
                          'action will be performed.')
      # ===================================================================== #
      # === Symlink all the programs
      #
      # If you ever want to symlink all entries under /Programs, this
      # is the way to go.
      #
      # Symlink all programs:
      #
      #   ry htop symlinkall
      #   ry x traverse
      #   ry symlink_every_program
      # 
      # ===================================================================== #
      when *ARRAY_SYMLINK_ALL_PROGRAMS_OPTIONS
        do_symlink_all_programs_then_exit
      # ===================================================================== #
      # === Combine --ntrad and --dontsymlink into the single option --backup
      #
      # This entry point allows the user to just use --backup, rather than
      # combine two options (--ntrad and --dontsymlink). Thus, this is
      # merely a convenience entry point, to allow the user to type
      # less. This functionality was added at 24.11.2020.
      #
      # The name "--backup" is not a great name, though, so be aware that
      # the name may one day change.
      #
      # Usage examples:
      #
      #   ry libx11 --backup
      #   ry metacity --lazyntrad
      #
      # ===================================================================== #
      when /^-?-?backup$/i,
           /^-?-?lazy(_|-| )?ntrad$/i
        do_compile_in_a_non_traditional_way
        do_not_symlink_in_general_and_also_do_not_symlink_header_files
      # ===================================================================== #
      # === rbt --total-size?
      # ===================================================================== #
      when /^-?-?total(-|_| )?size\??$/,
           /^-?-?report(-|_| )?total(-|_| )?size(-|_| )?of(-|_| )?all(-|_| )?archives$/
        require 'rbt/toplevel_methods/report_total_size_of_all_archives.rb'
        RBT.report_total_size_of_all_archives
        exit_program
      # ===================================================================== #
      # === Toggle the debug value
      #
      # This method will toggle the debug value, then query it.
      #
      # Invocation example:
      #
      #   ry htop debug
      #
      # ===================================================================== #
      when *ARRAY_HELP_TOGGLE_DEBUG_VALUE_OPTIONS
        toggle_the_debug_value_then_query_the_debug_value
        exit_program
      # ===================================================================== #
      # === Copy archives to the current working directory
      #
      # This entry point allows us to copy e. g. a directory, such as the
      # archive of ruby (at a local location like "/home/x/src/ruby/",
      # into the current working directory.
      #
      # The basic idea behind this functionality is to be able to quickly
      # copy a select subset of source archives, which we can then
      # transfer via FTP or a local USB device.
      #
      # Invocation examples:
      #
      #   rbt --copy-these-archives=ruby,php,python
      #   rbt --copy-archives=kde5_plasma
      #   rbt --copy-archives=:kde5_plasma
      #
      # ===================================================================== #
      when /^-?-?copy(-|_| )?these(-|_| )?archives=(.+)$/i,
           /^-?-?copy(-|_| )?archives=(.+)$/i
        these_programs = $2.to_s.dup
        these_programs = $3.to_s.dup if $3 # ← Valid for the first regex-entry above.
        copy_these_archives(these_programs)
        exit_program
      # ===================================================================== #
      # === Use a specific prefix
      #
      # This entry point allows us to quickly use a specific prefix.
      #
      # Invocation example:
      #
      #   ry coreutils --use-this-prefix=/Depot/Chroot/usr/
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?this(_|-| )?prefix=(.+)$/i # $3
        set_user_prefix($3.to_s.dup)
      # ===================================================================== #
      # === Installed files
      #
      # This will tell us which files are to be installed. For this
      # to work, we will load the .yml file.
      #
      # Usage example:
      #
      #   ry htop --show_which_files_will_be_installed
      #
      # ===================================================================== #
      when '--installed-files?',
           /^-?-?show(_|-| )?which(_|-| )?files(_|-| )?will(_|-| )?be(_|-| )?installed$/,
           'ifiles',
           '--installed-files',
           '--installedfiles'
        show_which_files_will_be_installed
        exit_program
      # ===================================================================== #
      # === Replace Symlinks
      #
      # Use this to replace the symlinks, using RBT::ReplaceSymlinks.
      # Then we will exit.
      #
      # Usage example:
      #
      #   ry --replace_symlinks
      #
      # ===================================================================== #
      when *ARRAY_HELP_REPLACE_SYMLINKS_OPTIONS
        replace_symlinks
        exit_program
      # ======================================================================= #
      # === Batch-install all registered python-addons
      #
      # This entry point allows us to batch-install the registered python
      # components. This functionality exists mostly due to convenience.
      #
      # Invocation examples:
      #
      #   rbt --compile-python-addons
      #   rbt --enable-python-addons
      #   rbt --python-addons
      #   rbt --all-of-python
      #   rbt --install-python-addons
      #
      # ======================================================================= #
      when /^-?-?compile(-|_| )?python(-|_| )?addons$/i,
           /^-?-?python(-|_| )?addons$/i, # ry --python-addons
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?python$/i,
           /^-?-?compile(-|_)?python(-|_)?addons$/,
           /^-?-?all(-|_)?of(-|_)?python$/,
           /^-?-?install(-|_)?python(-|_)addons?$/
        do_ignore_the_errors
        compile_the_python_addons
      # ===================================================================== #
      # === Enable python3
      #
      # Invocation example:
      #
      #   rbt --enable_python3
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?python3$/
        enable_python3
      # ===================================================================== #
      # === Download the Source of a given program at hand
      #
      # Do not build, just fetch the source. This is sort of equivalent to
      # applying "wget".
      #
      # Usage example:
      #
      #   ry htop --dload
      #   ry ruby dload
      #
      # ===================================================================== #
      when *ARRAY_HELP_DOWNLOAD_THE_SOURCE_OPTIONS
        ensure_that_the_cookbook_dataset_exists(first_argument?)
        # =================================================================== #
        # First check whether we may actually download the remote archive
        # or not.
        # =================================================================== #
        if may_we_download?
          _ = cookbook_dataset_base_dir?
          mkdir _ unless File.exist? _
          default_download_then_exit
        else
          orev "We are not allowed to download "\
               "#{teal(compile_which_program?.to_s)}."
          orev "The method #{royalblue('may_we_download?')}"\
               "#{rev} determined this, via a configuration setting."
          exit_program
        end
      # ===================================================================== #
      # === Compile a base system for Linux
      #
      # This entry point taps into class RBT::CompileBaseSystem,
      # which will compile the programs for a typical Linux base
      # system, into an AppDir prefix.
      #
      # Invocation examples:
      #
      #   rbt --compile-base-system
      #   rbt --compile-basic-system
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?base(_|-| )?system$/,
           /^-?-?compile(_|-| )?basic(_|-| )?system$/,
           /^-?-?compile(_|-| )?basic(_|-| )?subsystem$/,
           /^-?-?compile(_|-| )?basic(_|-| )?linux(_|-| )?core$/,
           /^-?-?compile(_|-| )?basic(_|-| )?linux$/
        require 'rbt/utility_scripts/compile_base_system.rb'
        RBT::CompileBaseSystem.report_which_programs_may_be_compiled
        RBT::CompileBaseSystem.new
        exit_program
      # ===================================================================== #
      # === Display the total file size of all locally available files
      #
      # Invocation example:
      #
      #   rbt --total-size-of-locally-available-files
      #
      # ===================================================================== #
      when /^-?-?total(-|_| )?size(-|_| )?of(-|_| )?locally(-|_| )?available(-|_| )?files$/
        report_total_size_of_locally_available_files
        exit_program
      # ===================================================================== #
      # === What will be installed
      #
      # Usage example:
      #
      #   ry htop --installs_what?
      #
      # ===================================================================== #
      when /^-?-?installs(_|-| )?what\??$/i,
           'iwhat?'
        feedback_all_libraries_and_all_binaries
        exit_program
      # ===================================================================== #
      # === Run a specific postinstallation action, then exit
      #
      # This entry point allows the user to run a particular
      # postinstall action only.
      #
      # Usage example:
      #
      #   rbt --postinstall-for=docbookxsl
      #   rbt --postinstall-for=littleutils
      #
      # ===================================================================== #
      when /^-?-?postinstall(_|-| )?for=(.+)$/i
        this_program = $2.to_s.dup
        orev "Initializing the dataset for the program "\
             "#{steelblue(this_program)}."
        initialize_dataset_for(this_program)
        do_postinstall_actions(true)
        # postinstall?.each {|entry|
        #   e steelblue(entry)
        #   system(entry)
        # }
        exit_program
      # ===================================================================== #
      # === Compile mouse-components
      #
      # This entry point can be used to compile all mouse-relevant parts
      # of xorg. It is mostly just a quick hackish way to get mouse
      # support in, after the xorg-server was compiled/installed.
      #
      # Invocation example:
      #
      #   rbt --mouse
      #
      # ===================================================================== #
      when /^-?-?mouse$/i
        compile_mouse_components
      # ===================================================================== #
      # === Set another prefix
      #
      # This variant will make use of a regex capture group.
      #
      # Invocation example:
      #
      #   rbt xfce4panel --prefix=:xfce
      #   rbt xfce4panel :xfce
      #   rbt htop --use-this-appdir=/Programs/Bla/1.2.3
      #
      # ===================================================================== #
      when /^-?-?prefix=(.+)/, # for $1
           /^(:.+)/,           # for $1
           /^-?-?use(-|_)?this(-|_)?appdir=(.+)$/i # for $3
        _ = $1.to_s.dup
        _ = $3.to_s.dup if $3
        set_user_prefix(_)
      # ===================================================================== #
      # === Do not make install
      #
      # This entry point will skip the "make install" step.
      #
      # Invocation examples:
      #
      #   ry gcc dontmakeinstall
      #   ry gcc --skip-make-install
      #
      # ===================================================================== #
      when /^-?-?dont(-|_| )?make(-|_| )?install$/i,
           /^-?-?skip(-|_| )?make(-|_| )?install$/i
        dont_make_install
      # ===================================================================== #
      # === Do not run a GNU configure script
      #
      # Skip running a configure script. This may be necessary for
      # entries that solely rely on "make" and "make install".
      #
      # Usage example:
      #
      #   ry htop --skip_configure
      #
      # ===================================================================== #
      when /^-?-?skip(_|-| )?configure$/i,
           'skipconfigure','sconfigure','skip','skipconf',
           'dontconfigure','skipc',
           'sc',
           'dconf',
           'dcon',
           'dconfigure',
           /^-?-?do(-|_| )?not(-|_| )?run(-|_| )?configure$/i,
           /^-?-?do(-|_| )?not(-|_| )?run(-|_| )?configure(-|_| )?script$/i,
           /^-?-?skip(-|_| )?configure$/i
        do_not_run_configure
      # ===================================================================== #
      # === Install only headers
      #
      # Use this to install only the headers of a particular program.
      #
      # Usage example:
      #
      #   ry gmp --only-headers
      #
      # ===================================================================== #
      when /^-?-?install(_|-| )?only(_|-| )?headers$/,
           /^-?-?only(_|-| )?headers$/,
           /^-?-?do(_|-| )?install(_|-| )?only(_|-| )?the(_|-| )?headers$/,
           'headers',
           'header'
        do_install_only_the_headers
      # ===================================================================== #
      # === Repackage archives
      #
      # This entry point can be used to repackage an archive file.
      #
      # Repackaging means to, e. g., change the archive format from
      # .tar.gz to .tar.xz.
      #
      # Usage example:
      #
      #   rbt --repackage=poppler-0.73.0.tar.gz
      #
      # ===================================================================== #
      when /^-?-?repackage=(.+)$/
        _ = $1.to_s.dup
        if File.exist? _
          repackage_this(_)
        else
          opnn; no_file_exists_at(_)
        end
      # ===================================================================== #
      # === Feedback program version
      #
      # Use this to feedback the specific program version, that is,
      # the version that is specified in the specific yaml file
      # belonging to a program.
      #
      # Invocation examples:
      #
      #   ry glibc version?
      #   ry gcc version?
      #
      # =>   5.3.0
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_PROGRAM_VERSION_OPTIONS
        feedback_program_version
        exit_program
      # ===================================================================== #
      # === Use the working directory as input for everything
      #
      # This entry point allows us to simply use information gathered by
      # the current working directory as input.
      #
      # An example for such a directory would be:
      #
      #   /Depot/j/libxkbcommon-0.10.0/
      #
      # Invocation example:
      #
      #   ry --use-current-working-directory-as-input
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?current(-|_| )?working(-|_| )?directory(-|_| )?as(-|_| )?input$/
        # =================================================================== #
        # The variable _ may look like this: "/Depot/j/libxkbcommon-0.10.0/"
        # =================================================================== #
        _ = File.basename(return_pwd)
        do_skip_extracting
        do_not_download
        set_extract_to File.dirname(return_pwd)
        set_configure_base_directory(return_pwd)
        program_information = ProgramInformation.new(_)
        @internal_hash[:program_version] = program_information.version?
        @internal_hash[:program_name]    = program_information.program_name?
        @internal_hash[:short_name]      = program_information.short_name?
        @internal_hash[:base_dir]        = rds(return_pwd)
        set_compile_this_program(@internal_hash[:short_name])
        set_compile_these_programs(@internal_hash[:short_name])
        @internal_hash[:cd_into_the_rbt_log_directory] = false
        cd @internal_hash[:configure_base_directory]
      # ===================================================================== #
      # === rbt --use-syck
      #
      # This entry point allows the user to permanently use syck rather than
      # psych, for yaml files. Almost nobody will need this these days,
      # but for legacy reason this entry point will be available.
      # ===================================================================== #
      when /^-?-?use(-|_| )?syck$/i,
           /^-?-?permanently(-|_| )?use(-|_| )?syck$/i
        do_permanently_use_syck
        exit_program
      # ===================================================================== #
      # === Querying whether porg is available on the given computer system
      #
      # Invocation example:
      #
      #   rbt --is_porg_available?
      #
      # ===================================================================== #
      when /^-?-?is(_|-| )?porg(_|-| )?available\??$/i
        report_whether_porg_is_available
        exit_program
      # ===================================================================== #
      # === Enable porg
      #
      # Use this specifically to enable porg for the specific program
      # at hand.
      # ===================================================================== #
      when /-?-?enable(-|_)?porg/,
           /-?-?use(-|_)?porg/,
           /-?-?do(-|_)?use(-|_)?porg/,
           /-?-?enable(-|_)?porg/,
           /-?-?use(-|_)?paco/,
           /--paco$/,
           /--porg$/
        verbose_enable_porg
        exit_program
      # ===================================================================== #
      # === Disable porg
      #
      # Through this entry point you can disable porg. You can also do so
      # via the configuration files, but an ad-hoc commandline way like
      # this one here also exists. 
      # ===================================================================== #
      when /^-?-?disable(-|_)?porg$/i,
           /^-?-?no(-|_)?porg$/i
        disable_porg
      # ===================================================================== #
      # === Querying whether RBT will symlink .gir files
      #
      # Invocation example:
      #
      #   rbt --symlink-gir-files?
      #
      # ===================================================================== #
      when /^-?-?symlink(-|_| )?gir(-|_| )?files\?$/i
        e symlink_gir_files?
      # ===================================================================== #
      # === Output how many processors the local computer system has
      #
      # Use this entry point if you wish to find out how many
      # processors/core the target machine has.
      #
      # Invocation examples:
      #
      #   rbt --n-processors?
      #   rbt --n-cores?
      #   Installer --n-cores?
      #
      # ===================================================================== #
      when /^-?-?n(_|-| )?processors\??$/i,
           /^-?-?n(_|-| )?cores\??$/i,
           /^-?-?n(-|_| )?CPU\??$/i
        try_to_output_how_many_processors_this_computer_has
        exit_program
      # ===================================================================== #
      # === Feedback all available licenses
      #
      # Use this to feedback all available licenses.
      #
      # Usage examples:
      #
      #   ry --all_licenses
      #   ry gcc license lgpl
      #
      # ===================================================================== #
      when *ARRAY_HELP_VIEW_AVAILABLE_LICENSES_OPTIONS
        feedback_licenses
        exit_program
      # ===================================================================== #
      # === Show missing use-this-build-system programs
      #
      # This entry point allows the user to show which registered programs
      # in the RBT suite are missing a build-system definition. 
      #
      # Invocation examples:
      #
      #   rbt --missing-use-this-build-system
      #
      # ===================================================================== #
      when /^-?-?missing(_|-| )?use(_|-| )?this(_|-| )?build(_|-| )?system$/,
           /^-?-?missing(_|-| )?build(_|-| )?system$/
        show_missing_use_this_build_system_entry
        exit_program
      # ===================================================================== #
      # === Show how to compile KDE
      #
      # This entry point will give some help as to how to compile KDE.
      #
      # Invocation examples:
      #
      #   rbt --how-to-compile-kde?
      #   rbt --show-how-to-compile-kde
      #
      # ===================================================================== #
      when /^-?-?how(_|-| )?to(_|-| )?compile(_|-| )?kde\??$/,
           /^-?-?show(_|-| )?how(_|-| )?to(_|-| )?compile(_|-| )?kde\??$/
        show_how_to_compile_kde
        exit_program
      # ===================================================================== #
      # === Show compile time statistics
      #
      # This entry point will display on the commandline some
      # compile-statistics, that is how long it took to compile
      # this or that particular program at hand.
      #
      # Usage examples:
      #
      #   rbt --show-compile-times
      #   rbt --show-compile-time-statistics
      #   rbt --compile-time?
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?compile(-|_| )?times$/,
           /^-?-?show(-|_| )?compile(-|_| )?time(-|_| )?statistics/,
           /^-?-?compile(-|_| )?times\??/,
           /^-?-?compile(-|_| )?stats$/,
           /^-?-?compile(-|_| )?time\??$/,
           /^-?-?compile(-|_| )?statistics\??$/i
        show_compile_time_statistics
        exit_program
      # ===================================================================== #
      # === rbt matplotlib --main-yaml
      #
      # This is essentially the same as using :simple for the cookbook
      # dataset.
      #
      # Invocation example:
      #
      #   newrbt python --main-yaml
      #
      # ===================================================================== #
      when /^-?-?main(-|_| )?yaml$/i
        set_simple_or_complex(:simple)
      # ===================================================================== #
      # === Do not pass any parameters to make
      #
      # This will specifically set things to false.
      #
      # Usage example:
      #
      #   ry patchelf --no-parameters-to-make --opt
      #
      # ===================================================================== #
      when /^-?-?no(_|-| )?parameters(_|-| )?to(_|-| )?make$/i
        @internal_hash[:parameters_to_make] = false
        orev 'No parameters will be passed to "'+
             steelblue('make')+rev+'".'
      # ===================================================================== #
      # === Use traditional compilation (/usr/ as the prefix)
      #
      # This entry point allows us to compile in a traditional way,
      # via /usr/ as the main prefix.
      #
      # Invocation example:
      #
      #   rbt php --trad
      #
      # ===================================================================== #
      when /^-?-?trad$/i
        ensure_that_a_cookbook_object_has_been_instantiated
        orev 'Enabling /usr/ as prefix.'
        set_prefix('/usr/')
      # ===================================================================== #
      # === rbt --expand
      # ===================================================================== #
      when /^-?-?expand$/i,
           /^-?-?expand(-|_)?dataset$/i,
           /^-?-?initialize$/
        action(:ExpandCookbooks)
        exit_program
      # ===================================================================== #
      # === Report BLFS entries
      #
      # This entry point can be used to report all entries that have a
      # BLFS tag.
      #
      # Invocation example:
      #
      #   rbt --blfs2
      #
      # ===================================================================== #
      when /^report(-|_)?all(-|_)?blfs(-|_)?entries/,
           /^-?-?blfs2$/i
        report_all_blfs_entries
        exit_program
      # ===================================================================== #
      # === Generate the shell script containing all program versions
      #
      # Usage example:
      #
      #   ry --all_available_programs?
      #
      # ===================================================================== #
      when /^-?-?generate(-|_| )?the(-|_| )?shell(-|_| )?scripts$/i
        require 'rbt/shell/create_shell_script_containing_these_program_versions.rb'
        RBT::CreateShellScriptContainingTheseProgramsVersions.new('--everything')
        exit_program
      # ===================================================================== #
      # === Show all configure options
      #
      # This will show a beautiful summary of the configure options.
      #
      # Invocation example:s
      #
      #   ry htop allconfigureoptions?
      #   ry htop --so-pretty
      #
      # ===================================================================== #
      when /-?-?all(_|-| )?configure(_|-| )?options\??/,
           '--beautify','--configure_help','do_beautify',
           'dobeautify',
           /-?-?pretty/,
           '--beauty','--so-pretty'
        all_configure_options?
      # ===================================================================== #
      # === Get compile-instructions from a locally existing file
      #
      # This entry point can be used to use the content of a file for
      # determining which programs should be compiled. That way you
      # may have a bit more flexibility, such as when another ruby
      # script generates this file, which can then be used by RBT.
      #
      # In fact, this is the original use case why this method has been
      # created in the first place.
      #
      # The file can handle .yml files too, where we will use YAML.load_file()
      # to load the content, rather than File.read() or File.readlines().
      #
      # Invocation examples:
      #
      #   rbt --upgrade-from-this-file=/Depot/jjj/these_programs_can_be_upgraded.yml
      #   rbt --upgrade-from-this-file=:default
      #   rbt --upgrade-from-this-file=:def
      #
      # ===================================================================== #
      when /^-?-?upgrade(-|_| )?from(-|_| )?this(-|_| )?file=(.+)$/i # See: http://rubular.com/r/HjEJRwWc4f
        this_file = $4.to_s.dup
        do_upgrade_from_this_file(this_file)
      # ===================================================================== #
      # === Speed up compilation via make -jNUMBER
      #
      # This will automatically use the "-j" option that equals the
      # number of processors that the current computer has.
      #
      # Invocation example:
      #
      #   ry flac ntrad --speed-up-the-compilation
      #
      # ===================================================================== #
      when /^-?-?speed(_|-| )?up(_|-| )?the(_|-| )?compilation$/i,
           /^-?-?speed(_|-| )?up(_|-| )?compilation$/
        speed_up_the_compilation
      # ===================================================================== #
      # === Show the last compiled programs
      #
      # Usage example:
      #
      #   rbt --last-5-compiled?
      #
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_LAST_COMPILED_OPTIONS,
           /^lastcompiled\?=$/,
           /^-?-?last-compiled\?=$/,
           /^-?-?last(-\d+-)compiled\?$/
        feedback_last_compiled(i)
        exit_program
      # ===================================================================== #
      # === Do not use maximum speed
      #
      # Entry point:
      #
      #   ry lame --do-not-use-maximum-speed
      #   ry xvidcore --ntrad --donot-use-maximum-speed
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?maximum(_|-| )?speed$/i,
           /^-?-?no(_|-| )?auto(_|-| )?speedup$/i,
           /^-?-?dont(_|-| )?use(_|-| )?maximum(_|-| )?speed$/i
        verbose_disable_maximum_speed_for_compilation
      # ===================================================================== #
      # === Use a simpler entry-point for the basic-gcc profile/decorator
      # ===================================================================== #
      when /^-?-?basic(-|_)?gcc$/i,
           /^-?-?gcc(-|_)?profile1$/i
        try_to_use_this_profile(:basic_gcc)
      # ===================================================================== #
      # === Show important status information
      #
      # Invocation examples:
      #
      #   rbt --report-status
      #   rbt --report
      #
      # ===================================================================== #
      when /^-?-?report(-|_| )?status$/i,
           /^-?-?report$/i
        report_status_information
      # ===================================================================== #
      # === Where to store log files
      #
      # Use this entry point to report where RBT will store logfiles.
      #
      # Invocation examples:
      #
      #   rbt log_dir?
      #   rbt --log_dir?
      #
      # ===================================================================== #
      when *ARRAY_HELP_LOG_DIR_OPTIONS
        report_where_we_store_then_exit
      # ===================================================================== #
      # === Inclusion check
      #
      # Invocation examples:
      #
      #   ry php  --included?
      #   ry php2 --included?
      #   ry ruby --included?
      #
      # ===================================================================== #
      when *ARRAY_HELP_INCLUDED_OPTIONS
        report_whether_this_program_is_included_or_whether_it_is_not(program?)
        exit_program
      # ===================================================================== #
      # === Check for inclusion
      #
      # Check whether our entries in $SRC are included in our cookbooks too.
      #
      # Hence, you can use a dummy name like "x" for this:
      #
      #   ry x inclusions?
      #   ry htop check_for_inclusion
      #
      # ===================================================================== #
      when /^-?-?check(_|-| )?for(_|-| )?inclusion$/i,
           'inclusions','inclusions?',
           'cinc','inc?','cure','not_included?',
           'cinc?','all_not_included?'
        check_for_inclusion
        exit_program
      # ===================================================================== #
      # === Show everything about a registered program
      #
      # This entry point will show, on the commandline, the content
      # of the queried program. This allows you to quickly show
      # the .yml file that is associated with a given program at
      # hand.
      #
      # Usage example:
      #
      #   rbt --show-all-about=ruby
      #
      # ===================================================================== #
      when /^-?-?show(-|_)?all(-|_)?about=(.+)$/
        show_all_about($3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Enable stow
      #
      # Enable stow via this option.
      #
      # Invocation example:
      #
      #   ry --enable-stow
      #
      # ===================================================================== #
      when /-?-?enable-?stow/
        enable_stow
        exit_program
      # ===================================================================== #
      # === Use an environment variable
      #
      # This method will use a specific environment variable as the
      # new prefix to be used.
      #
      # Do note that class RBT::CompileViaEnvironmentVariableAsPrefix can
      # also be used as an alternative.
      #
      # In order to use this entry point, try:
      #
      #   ry htop --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4screensaver --use-this-environment-variable=XFCE_PREFIX
      #   ry libxfce4util --use-this-environment-variable=XFCE_PREFIX
      #   ry xfconf --use-this-environment-variable=XFCE_PREFIX
      #   ry exo --use-this-environment-variable=XFCE_PREFIX
      #   ry garcon --use-this-environment-variable=XFCE_PREFIX
      #   ry libxfce4ui --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4panel --use-this-environment-variable=XFCE_PREFIX
      #   ry thunar --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4appfinder --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4session --use-this-environment-variable=XFCE_PREFIX
      #   ry xfdesktop --use-this-environment-variable=XFCE_PREFIX
      #   ry xfceengine --use-this-environment-variable=XFCE_PREFIX
      #   ry xfwm4 --use-this-environment-variable=XFCE_PREFIX
      #   ry xfmedia --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4devtools --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4settings --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4cpufreqplugin --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4panelprofiles --use-this-environment-variable=XFCE_PREFIX
      #   ry matedesktop  --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmatemixer --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmateweather --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matemenus --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matepanel --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matenetbook --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matemedia --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry marco --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmatekbd --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matesensorsapplet --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matesettingsdaemon --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry caja --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry cajaextensions --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry engrampa --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry eom --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry mateutils --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry pluma --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry mateterminal --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matecontrolcenter --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #
      # Or:
      #
      #   env_compile_via_prefix libxfce4util
      #   env_compile_via_prefix xfconf
      #   env_compile_via_prefix exo
      #   env_compile_via_prefix garcon
      #   env_compile_via_prefix libxfce4ui
      #   env_compile_via_prefix xfce4panel
      #   env_compile_via_prefix thunar
      #   env_compile_via_prefix xfce4appfinder
      #   env_compile_via_prefix xfce4session
      #   env_compile_via_prefix xfdesktop
      #   env_compile_via_prefix xfceengine
      #   env_compile_via_prefix xfwm4
      #   env_compile_via_prefix xfmedia
      #   env_compile_via_prefix xfce4devtools
      #   env_compile_via_prefix xfce4settings
      #   env_compile_via_prefix xfce4cpufreqplugin
      #   env_compile_via_prefix xfce4panelprofiles
      #   env_compile_via_prefix xfce4screensaver
      #   env_compile_via_prefix xfce4notifyd
      #   env_compile_via_prefix xfce4screenshooter
      #   env_compile_via_prefix xfce4systemloadplugin
      #   env_compile_via_prefix xfce4whiskermenuplugin
      #   env_compile_via_prefix xfce4taskmanager
      #
      # or
      #
      #   env_compile_via_prefix matedesktop MATE
      #   env_compile_via_prefix libmatemixer MATE
      #   env_compile_via_prefix libmateweather MATE
      #   env_compile_via_prefix matemenus MATE
      #   env_compile_via_prefix matepanel MATE
      #   env_compile_via_prefix matenetbook MATE
      #   env_compile_via_prefix matemedia MATE
      #   env_compile_via_prefix marco MATE
      #   env_compile_via_prefix libmatekbd MATE
      #   env_compile_via_prefix matesensorsapplet MATE
      #   env_compile_via_prefix matesettingsdaemon MATE
      #   env_compile_via_prefix matecontrolcenter MATE
      #   env_compile_via_prefix caja MATE
      #   env_compile_via_prefix cajaextensions MATE
      #   env_compile_via_prefix engrampa MATE
      #   env_compile_via_prefix eom MATE
      #   env_compile_via_prefix mateutils MATE
      #   env_compile_via_prefix pluma MATE
      #   env_compile_via_prefix mateterminal MATE
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?this(-|_)?environment(-|_)?variable=/,
           /^-?-?use(-|_)?environment(-|_)?variable=/
        use_this_environment_variable(i)
      # ===================================================================== #
      # === Show the commandline-tutorial for the RBT project
      #
      # Invocation example:
      #
      #   rbt --tutorial
      #
      # ===================================================================== #
      when /^-?-?tutorial$/i
        require 'rbt/toplevel_methods/tutorial.rb'
        RBT.show_tutorial
      # ===================================================================== #
      # === Report the total size of all archives
      #
      # Invocation example:
      #
      #   rbt --report_total_size_of_all_archives
      #
      # ===================================================================== #
      when /^-?-?report(_|-| )?total(_|-| )?size(_|-| )?of(_|-| )?all(_|-| )?archives$/i
        require 'rbt/toplevel_methods/report_total_size_of_all_archives.rb'
        RBT.report_total_size_of_all_archives
      # ===================================================================== #
      # === Finding all Archives
      #
      # This entry point can be used to find all local archives.
      #
      # Finding all Archives:
      #
      #   ry --archives?
      #
      # ===================================================================== #
      when /^-?-?find_?all_?archive_?types/,
           'farchives',
           /^-?-?archives\??/
        require 'rbt/cookbooks/find_all_archive_types.rb'
        _ = RBT::Cookbooks::FindAllArchiveTypes.new(:do_not_run_yet)
        _.be_silent
        _.run_and_report
        exit_program
      # ===================================================================== #
      # === Make use of system() rather than IO.popen()
      #
      # Invocation example:
      #
      #   ry glade trad --use-simple-system
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?simple(-|_| )?system$/i,
           /^-?-?use(-|_| )?simple(-|_| )?system(-|_| )?calls$/i,
           /^-?-?use(-|_| )?plain(-|_| )?system$/i,
           /^-?-?use(-|_| )?system$/i
        orev 'Making use of system() calls rather than IO.popen() from now on.'
        @internal_hash[:use_which_system_command] = :system
      # ===================================================================== #
      # === Do not compile
      #
      # The following instruction will run configure, but not make.
      #
      # Invocation examples:
      #
      #   ry gcc dont_compile
      #   ry htop dc
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_NOT_COMPILE_OPTIONS
        do_not_compile
        do_not_run_configure
        skip_postinstall_actions
      # ===================================================================== #
      # === Generate a slack-desc file
      #
      # Usage examples:
      #
      #   ry htop --gsc
      #   ry ruby slack-desc-file
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_SLACK_DESC_FILE
        generate_slack_desc_file_for_this_program(program?)
        exit_program
      # ===================================================================== #
      # === Find all archives
      #
      # Invocation example:
      #
      #   ry --archives?
      #
      # ===================================================================== #
      when *ARRAY_HELP_FIND_ALL_ARCHIVES
        find_all_archive_types
        exit_program
      # ===================================================================== #
      # === Remove all temporary directories
      #
      # This entry point will get rid of all directories found at the
      # /Depot/Temp/ hierarchy - or whever else you have defined your
      # temp hierarchy to reside.
      #
      # Why is this entry point useful? Well, sometimes this directory
      # may be a bit messy, so this entry point allows you a simple
      # way to "start over". We only remove directories because these
      # may include stray object files or cmake-cache files and so on.
      #
      # Invocation example:
      #
      #   ry --remove-all-temp-directories
      #
      # ===================================================================== #
      when /^-?-?remove(_|-| )?all(_|-| )?temp-directories$/,
           /^-?-?purge(_|-| )?old(_|-| )?directories$/,
           /^-?-?purge(_|-| )?old(_|-| )?dirs$/,
           /^-?-?purge(_|-| )?base(_|-| )?directory$/,
           /^-?-?purge(_|-| )?base(_|-| )?dir$/,
           /^-?-?clear(_|-| )?temp(_|-| )?dir$/,
           /^-?-?purge(_|-| )?directories$/,
           /^-?-?purge(_|-| )?dirs$/,
           /^-?-?purge(_|-| )?dir$/,
           /^-?-?purge(_|-| )?base(_|-| )?dirs$/,
           /^-?-?purge(_|-| )?temp(_|-| )?dir$/,
           /^-?-?purge$/
        remove_all_temp_directories
        exit_program
      # ===================================================================== #
      # === Use /usr/lib64 as libdir target
      #
      # This entry point allows the user to make use of /usr/lib64/
      # as the libdir-prefix in use.
      #
      # Usage example:
      #
      #   ry libgpgerror --libdir64
      #
      # ===================================================================== #
      when /^-?-?libdir64$/i,
           /^-?-?dir64$/i,
           /^-?-?lib64$/i,
           /^-?-?64dir$/i,
           /^-?-?lib2$/i
        do_use_libdir64
      # ===================================================================== #
      # === Compile every program
      #
      # Here we will try to compile every program that is part of our
      # database.
      #
      # Invocation example:
      #
      #   ry --compile_all
      #
      # ===================================================================== # 
      when /^-?-?compile(_|-| )?all$/,
           /^-?-?compile(_|-| )?all(_|-| )?programs$/,
           'all',
           'rall'
        compile_all_programs(:do_exit_afterwards) # Argument true for "shall we exit".
      # ===================================================================== #
      # === Do not keep the archive
      #
      # After extracting and compiling, we will get rid of the archive
      # again. We will get rid of the archive independent over whether
      # the installation process was a success or whether it was not,
      # mind you. 
      #
      # Usage examples:
      #
      #   ry htop --remove-archive
      #   ry htop dontkeeparchive
      #
      # ===================================================================== #
      when *ARRAY_HELP_DONT_KEEP_EXTRACTED_OPTIONS
        set_dont_keep_archive # Remove archive(s) after a successful build.
      # ===================================================================== #
      # === Download a remote program into a given directory at hand
      #
      # This entry point allows us to download a remote program into
      # a local directory.
      #
      # Invocation example:
      #
      #   rbt htop --download-into-this-directory=/opt/bla2/
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?into(-|_| )?this(-|_| )?directory=(.+)$/
        _ = $4.to_s.dup
        url = url1?
        download_this_remote_url_into_that_local_directory(url, _)
      # ===================================================================== #
      # === Show the .pc files installed by the program at hand
      #
      # If you want to find out which .pc files are installed by the
      # program at hand, this entry point allows you to do so.
      #
      # Issue something similar to the following usage examples:
      #
      #   ry glib --pkgconfig-files?
      #   ry samba --pkgconfig-files?
      #   ry poppler pkgconfig_files?
      #
      # ===================================================================== #
      when /^-?-?pkgconfig(_|-| )?files\??/,
           /^-?-?show(_|-| )?pkgconfig(_|-| )?files/
        show_pc_files_installed_by_this_program(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Purge FHS binaries of a specific program
      #
      # This will delete all binaries under /usr/bin/ hierarchy of the
      # given program at hand.
      #
      # It currently depends on all registered programs in the yaml file.
      # In principle we could also work on existing bin/ files in the
      # AppDir hierarchy, but for now, let's stick to the yaml file instead.
      #
      # Invocation examples:
      #
      #   ry littleutils --purge-fhs-binaries
      #   ry --purge-traditional-binaries-from=xscreensaver
      #   ry --purge-traditional-binaries-of=gnupg
      #   ry --purge-traditional-binaries-of=flac
      #
      # ===================================================================== #
      when /^-?-?purge-?fhs-?binaries$/i,
           /^-?-?purge-?traditional-?binaries-?from=(.+)/i,
           /^-?-?purge-?fhs-?binaries-?from=(.+)/i,
           /^-?-?purge-?fhs-?binaries=(.+)/i,
           /^-?-?fhs-?purge=(.+)/i,
           /^-?-?purge-?binaries-?of=(.+)/i,
           /^-?-?purge(-|_)?traditional(-|_)?binaries(-|_)?of=(.+)$/i
        _ = nil
        _ = $1 if $1
        _ = $4 if $4 # This is for the last entry.
        _ = program_name? if _.nil?
        verbose_purge_fhs_binaries_of_this_program(_.to_s.dup)
        exit_program
      # ===================================================================== #
      # === No man pages
      #
      # Use this subsection to get rid of all manpages in the /Programs/
      # hierarchy. Invoke this only when you are sure to not need these
      # man-pages.
      #
      # Usage example:
      #
      #   rbt --no-man-pages
      #
      # ===================================================================== #
      when *ARRAY_HELP_PURGE_ALL_MAN_ENTRIES
        purge_all_man_files_in_the_programs_hierarchy
        exit_program
      # ===================================================================== #
      # === Feedback everything
      #
      # Feedback EVERY available information.
      #
      # This is the most important option for feedback. It chains the
      # existing feedback options together.
      #
      # Usage example:
      #
      #   ry ruby --feedback_all
      #
      # ===================================================================== #  
      when 'ud','fall','uv','vu','info','lal','aall','al','aa','info?',
           '?',/^-?-?feedback(-|_)?all$/,'allinfo','everything',
           'feedback_everything','feedbackeverything'
        feedback_all # Wrapper.
        exit_program
      # ===================================================================== #
      # === Purge all binaries of the given program at hand
      #
      # This entry point will remove all binaries of the given input
      # program at hand under /usr/bin/.
      #
      # Invocation examples:
      #
      #   rbt poppler --purge-its-binaries
      #   rbt vim --purge-binaries
      #
      # ===================================================================== #
      when /^-?-?purge(-|_| )?its(-|_| )?binaries$/,
           /^-?-?purge(-|_| )?binaries$/
        ensure_that_the_cookbook_dataset_is_there
        purge_these_binaries(binaries?)
        exit_program
      # ===================================================================== #
      # === Pin a registered program
      #
      # This entry point allows us to pin down a certain program. A
      # pin-downed program can not be installed anymore. Users can
      # unpin this operation though.
      #
      # Invocation example:
      #
      #   rbt pin htop
      #   rbt --pin htop
      #
      # ===================================================================== #
      when /^-?-?pin (.+)$/i
        this_program = $1.to_s.dup
        pin_this_program(this_program)
        exit_program
      # ===================================================================== #
      # === Query where we will extract to
      #
      # This entry point can be used to determine where to we will extract
      # archives.
      #
      # Invocation example:
      #
      #   rbt --extract-to?
      #
      # ===================================================================== #
      when /^-?-?extract(-|_)?to\??$/i
        report_where_archives_will_be_extracted_into
        exit_program
      # ===================================================================== #
      # === Query whether we are on windows
      #
      # This entry point can be quickly used to determine whether we are
      # on windows or whether we are not. The reason this entry point
      # exists is mostly for debugging, so that we can determine whether
      # the windows-operating system is properly detected.
      #
      # Invocation example:
      #
      #   rbt --is-on-windows?
      #
      # ===================================================================== #
      when /^-?-?is(-|_| )?on(-|_| )?windows\??$/i
        e "#{rev}Are we on windows? "\
          "#{steelblue(verbose_truth(is_on_windows?))}"
        exit_program
      # ===================================================================== #
      # === Show the dependencies of a given program
      # 
      # Tell which dependencies a given program has:
      #
      #   ry sed --dependencies
      #   ry gcc --dependencies
      #   ry xsm fdep
      #
      # ===================================================================== #
      when 'abhängigkeiten',
           /^-?-?dependencies$/,
           'depe','deps','dep',
           'fdep',
           'd',
           'deps?',
           'dep?',
           'dependencies?',
           /^-?-?feedback(-|_| )?dependencies(-|_| )?of(-|_| )?this(-|_| )?program$/
        feedback_dependencies_of_this_program
        exit_program
      # ===================================================================== #
      # === Update the KDE applications
      #
      # This entry point allows us to update the KDE applications.
      #
      # Usage example:
      #
      #   rbt --update-kde-apps
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?kde(-|_| )?apps$/i
        update_kde_apps
      # ===================================================================== #
      # === Do not use any prefix option
      #
      # By default, RBT::Action::Installer tries to use a --prefix option
      # when# possible, no matter the build system at hand.
      #
      # This may not be always wanted, though, such as when a user just
      # wants to use the default value if --prefix is omitted, without
      # that user knowing what that default value is. The user should
      # not HAVE to know, either - thus this option exists.
      #
      # Invocation example:
      #
      #   ry htop --noprefix
      #
      # ===================================================================== #
      when /^-?-?no(_|-| )?prefix$/i,
           /^-?-?noprefix$/i,
           /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?a(_|-| )?prefix$/i
        do_not_use_a_prefix
      # ===================================================================== #
      # === Guessing the build type
      #
      # This entry point allows us to guess the build type of a target
      # program at hand. It may be easier to try to compile the
      # program, though.
      #
      # Invocation examples:
      #
      #   rbt --guess-build-type
      #   rbt --build-type
      #   rbt m4 --build-type
      #
      # ===================================================================== #
      when /^-?-?build(-|_| )?type$/i,
           /^-?-?guess(-|_| )?build(-|_| )?type$/i
        verbose_guess_the_build_type
        exit_program
      # ===================================================================== #
      # === Use /opt as prefix
      #
      # This entry point allows us to use /opt as the prefix value.
      #
      # Invocation example:
      #
      #   ry python --prefix-opt
      #
      # ===================================================================== #
      when /^-?-?prefix(-|_| )?opt$/i
        set_user_prefix('/opt/')
      # ===================================================================== #
      # === Enable a shortcut for cmake, via --release
      #
      # This entry point allows a user to pass -DCMAKE_BUILD_TYPE=Release
      # to a cmake-based project, by just typing --release.
      #
      # Invocation example:
      #
      #   ry initng --release
      #
      # ===================================================================== #
      when /^--release$/i
        append_these_configure_options(:release)
      # ===================================================================== #
      # === Append any configure option to the GNU configure run
      #
      # This entry point can be used to append any configure option to
      # the configure script.
      #
      # Invocation example:
      #
      #   ry emacs --ntrad --without-x
      #
      # ===================================================================== #
      when /^-?-?append(-|_| )?these(-|_| )?configure(-|_| )?options=(.+)$/i # === $4
        append_these_configure_options($4.to_s)
      # ===================================================================== #
      # === Enable shared compilation
      #
      # This entry point allows a user to enable shared compilation.
      #
      # It is typically equivalent to adding "--enable-shared" on
      # the configure commandline invocation.
      #
      # Invocation example:
      #
      #   ry htop ntrad --enable-shared
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?shared$/i
        append_enable_shared_to_the_configure_options
      # ===================================================================== #
      # === Show the allowed tags for the cookbooks-submodule
      #
      # Usage example:
      #
      #   rbt --allowed_tags?
      #
      # ===================================================================== #
      when /^-?-?allowed(-|_| )?tags\??/,
           /^-?-?registered(-|_| )?tags\??/,
           /^valid(-|_| )?tags\??/,
           /^-?-?valid(-|_| )?tags\??/
        load_and_display_the_allowed_tags
        exit_program
      # ===================================================================== #
      # === Compile all the protos next
      #
      # Invocation examples:
      #
      #   ry --compile-all-protos
      #   ry protos
      #   ry --protos
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?all(_|-| )?protos$/i, # ry --compile-all-protos
           /^-?-?protos$/,
           /^-?-?all(_|-| )?protos$/,
           /^-?-?compile(_|-| )?protos$/,
           'proto'
        register_all_protos_for_compilation
      # ===================================================================== #
      # === Query which headers are part of this package (different variant)
      #
      # This is similar to the variant below, but the usage is different.
      #
      # Invocation example:
      #
      #   ry --headers-of=ruby
      #
      # ===================================================================== #
      when /^-?-?headers(-|_| )?of=(.+)$/i # === $2
        _ = $2.to_s.dup
        if cookbook_dataset?.nil?
          initialize_dataset_for(_)
        end
        show_available_headers_of_this_program(_)
        exit_program
      # ===================================================================== #
      # === Query which headers are part of this package
      #
      # Use this if you wish to give back all .h files of a specific
      # program at hand. For this to work, of course the program in
      # question needs to install any .h files in the first place; and
      # we will also have had these .h files registered in the first
      # place.
      #
      # Invocation example:
      #
      #   ry glibc --headers?
      #
      # ===================================================================== #
      when /^-?-?headers\??$/,
           /^-?-?show(_|-| )?available(_|-| )?headers(_|-| )?of(_|-| )?this(_|-| )?program$/
        show_available_headers_of_this_program
        exit_program
      # ===================================================================== #
      # === Automatically determining a few things
      #
      # Invocation example:
      #
      #   rbt --auto
      #
      # ===================================================================== #
      when /^-?-?auto$/i
        try_to_automatically_setup_the_rbt_project
        exit_program
      # ===================================================================== #
      # === Prioritize on the traditional /usr/bin/ $PATH
      #
      # Invocation example:
      #
      #   ry fluxbox --traditional-path
      #
      # ===================================================================== #
      when /^-?-?traditional(_|-| )?path$/
        use_traditional_env_path
      # ===================================================================== #
      # === Update libraries of a given program
      #
      # This entry point allows us to automatically update the
      # libraries of a compiled, AppDir-installed program.
      #
      # Invocation examples:
      #
      #   rbt --update-libraries-of=poppler
      #   rbt --update-libraries-for=poppler
      #   rbt --autoupdate=opencv
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?libraries(-|_| )?of=(.+)$/i,
           /^-?-?update(-|_| )?libraries(-|_| )?for=(.+)$/i
        this_program = $1.to_s.dup # Default.
        this_program = $3.to_s.dup if $3 # Only for $3.
        update_libraries_of_this_program(this_program)
      # ===================================================================== #
      # === Designate another source location that is to be used
      #
      # This entry point allows us to use another source location,
      # e. g. if we have an USB stick and data on this stick then
      # this entry point may be very helpful.
      #
      # Invocation example:
      #
      #   ry gdb ntrad --use-this-source-location=/Depot/j/gdb-8.2.1.tar.xz
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?this(-|_| )?source(-|_| )?location=(.+)$/i # $4
        _ = $4.to_s.dup
        if File.exist? _
          if cookbook_dataset?
            cookbook_dataset?.set_program_path(_)
            disallow_changing_the_cookbook_dataset
          end
          orev "Making use of the target location at `#{slateblue(_)}#{rev}`."
          set_program_path(_)
        else
          opne swarn('No target exists at ')+sfile(_)+swarn('. Exiting now.')
          exit_program
        end
      # ===================================================================== #
      # === Upload binaries
      #
      # Invocation example:
      #
      #   rbt --upload-binaries
      #
      # ===================================================================== #
      when /^-?-?upload(-|_)?binaries$/
        rev 'We will upload all files found in the respective '\
             'bin/ directory'
        orev 'of this program ('+simp(program?)+"#{rev})."
        do_upload_binaries
      # ===================================================================== #
      # === Specify to use configure as the build-system
      #
      # This entry point allows us to overrule the build-system that is to
      # be used - and specifically use configure.
      #
      # Invocation example:
      #
      #   ry htop --use-configure
      #   ry zlib -ntrad --use-gnu-configure
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?configure$/i,
           /^-?-?use(_|-| )?autoconfigure$/i,
           /^-?-?use(_|-| )?configure(_|-| )?system$/i,
           /^-?-?use(_|-| )?gnu(_|-| )?configure$/i
        orev "Using #{royalblue('configure')} #{rev}as build system."
        use_configure_as_build_system
      # ===================================================================== #
      # === Create a standalone .yml file
      #
      # Invocation example:
      #
      #   rbt ruby --create_standalone_yml
      #   rbt ruby syml
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?standalone(-|_| )?yaml(-|_| )?file$/i,
           /^-?-?create(-|_| )?standalone(-|_| )?yml$/i,
           'syml','generate_standalone',
           /^-?-?csyf/ # 'yaml' ← can not be used.
        create_standalone_yaml_file
        exit_program
      # ===================================================================== #
      # === rbt --return_n_programs_available
      #
      # This entry point will return (or rather, display) how many
      # programs are available.
      # ===================================================================== #
      when /^-?-?return(-|_| )?n(-|_| )?programs(-|_| )?available/
        e return_n_programs_available
        exit_program
      # ===================================================================== #
      # === rbt --path_to_source_directory=
      # ===================================================================== #
      when /^-?-?path(-|_| )?to(-|_| )?source(-|_| )?directory=(.+)$/, # for $4
           /^-?-?set(-|_| )?archive(-|_| )?directory=(.+)$/,         # for $3
           /^-?-?source(-|_| )?directory=(.+)$/                    # for $2
        # =================================================================== #
        # Usage examples:
        #   cookbooks --path_to_source_directory=/Depot/Files
        #   cookbooks --set-archive-directory=/opt2
        #   cookbooks --source_directory=/Depot/Files
        # =================================================================== #
        if $4
          _ = $4.to_s.dup
        elsif $3
          _ = $3.to_s.dup
        else
          _ = $2.to_s.dup
        end # ^^^ we have to work through the special $ variables.
        RBT.permanently_set_path_to_this_source_directory(_)
        exit_program
      # ===================================================================== #
      # === rbt --create-html-page-for-these-programs=ruby,python,tcl,filelight
      #
      # This entry point allows us to create HTML pages for some programs
      # quickly.
      # ===================================================================== #
      when /^-?-?create(-|_| )?html(-|_| )?page(-|_| )?for(-|_| )?these(-|_| )?programs=(.+)/
        these_programs = $6.to_s.dup
        if these_programs.include? ','
          these_programs = these_programs.split(',').flatten
        end
        require 'rbt/toplevel_methods/html.rb'
        RBT.create_html_page_for_these_programs(these_programs)
        exit_program
      # ===================================================================== #
      # === rbt --update-kde-applications
      # ===================================================================== #
      when /^-?-?update(-|_| )?kde(-|_| )?applications$/i,
           /^-?-?download(-|_| )?kde(-|_| )?applications$/,
           /^-?-?kde(-|_| )?apps$/
        update_kde_applications
        exit_program
      # ===================================================================== #
      # === rbt --correct_dependencies?
      # ===================================================================== #
      when /^-?-?correct(-|_| )?dependencies\??$/,
           'DEPS','--cdeps?'
        RBT::Cookbooks::CheckForCorrectDependencies[]
        exit_program
      # ===================================================================== #
      # === rbt --format=postinstall
      # ===================================================================== #
      when /^format=(.+)$/,
           /^f=(.+)/
        feedback_format_specification_of($1.to_s.dup)
        exit_program
      # ===================================================================== #
      # === rbt --nprograms
      # ===================================================================== #
      when /^-?-?n(-|_| )?programs\??/,
           'registered_programs?',
           'nregistered?'
        show_n_programs_available
        exit_program
      # ===================================================================== #
      # === rbt --size
      # ===================================================================== #
      when /^-?-?size$/ # cookbook --size "php,ruby,perl,python"
        if i.include? '=' # Assume input such as this one here: "--size=php,ruby,perl,python"
          report_size_of_these_programs(i)
        end
        exit_program
      # ===================================================================== #
      # === rbt --set_temp_dir=/tmp
      # ===================================================================== #
      when /-?-?set(-|_| )?temp(-|_| )?dir=(.+)/
        _ = $3.to_s.dup
        orev 'We will permanently set to a new temp '\
             'directory next ('+simp(_)+rev+')'
        RBT.permanently_set_temp_directory(_)
        exit_program
      # ===================================================================== #
      # === rbt --last_option?
      # ===================================================================== #
      when /^-?-?last(-|_| )?option\?/,'show_last_configuration_options',
           'lastoptions?',
           'lastoptions',
           'loptions'
        require 'rbt/toplevel_methods/show_last_configure_option.rb'
        RBT::Cookbooks.show_last_configure_option
        exit_program
      # ===================================================================== #
      # === rbt --tutorial
      # ===================================================================== #
      when /^-?-?tutorial/
        require 'rbt/toplevel_methods/tutorial.rb'
        RBT.show_tutorial
        exit_program
      # ===================================================================== #
      # === rbt --do-not-truncate
      # ===================================================================== #
      when /^-?-?do(-|_| )?not(-|_| )?truncate$/i
        @internal_hash[:try_to_truncate_too_long_output] = false
      # ===================================================================== #
      # === rbt --show-all-about=htop
      # ===================================================================== #
      when /^-?-?show(-|_| )?all(-|_| )?about=(.+)/i,
           /^-?-?show(-|_| )?all=(.+)/i,
           /^-?-?show(-|_| )?file=(.+)/i
        show_this_program = $2.to_s.dup # This is valid for two entry points ^^^.
        show_this_program = $3.to_s.dup if $3
        show_all_about(show_this_program)
        exit_program
      # ===================================================================== #
      # === rbt --permanently-disable-colours
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?disable(-|_| )?colours/
        RBT.permanently_disable_colours
        exit_program
      # ===================================================================== #
      # === rbt --batch-validate
      # ===================================================================== #
      when /^-?-?batch(-|_| )?validate$/,
           /^-?-?validate(-|_| )?entries$/,
           /^-?-?validate$/
        batch_validate_the_cookbooks
        exit_program
      # ===================================================================== #
      # === rbt --use-psych
      # ===================================================================== #
      when /^-?-?use(-|_| )?psych$/i,
           /^-?-?permanently(-|_| )?use(-|_| )?psych$/i
        do_permanently_use_psych
        exit_program
      # ===================================================================== #
      # === rbt --publish-list-of-all-programs-version
      # ===================================================================== #
      when /^publish(-|_| )?list(-|_| )?of(-|_| )?/i,
           /^-?-?publish/
        publish_list_of_all_programs_version # in file base/misc.rb
        exit_program
      # ===================================================================== #
      # === rbt --scripts?
      # ===================================================================== #
      when /^-?-?scripts\??$/
        show_available_utility_scripts
      # ===================================================================== #
      # === rbt --feedback_all_available_formats
      # ===================================================================== #
      when /^-?-?feedback(-|_| )?all(-|_| )?available(-|_| )?formats/
        RBT::Cookbooks::SanitizeCookbook.feedback_all_available_formats
        exit_program
      # ===================================================================== #
      # === rbt --check-for-gnome-updates
      # ===================================================================== #
      when /^-?-?check(-|_| )?for(-|_| )?gnome(-|_| )?updates$/,
           /^-?-?gnome$/
        require 'rbt/utility_scripts/check_for_gnome_updates.rb'
        RBT::Cookbooks::CheckForGnomeUpdates.new
        exit_program
      # ===================================================================== #
      # === scookie htop --array
      # ===================================================================== #
      when /-?-?array$/
        output_main_dataset
        exit_program
      # ===================================================================== #
      # === rbt --debug
      # ===================================================================== #
      when /-?-?debug$/
        enable_debug
      # ===================================================================== #
      # === rbt --show-available-utility-scripts
      # ===================================================================== #
      when /-?-?show-?available-?utility-?scripts$/,
           /-?-?show-?available-?scripts$/
        RBT::Cookbooks.show_available_utility_scripts
        exit_program
      # ===================================================================== #
      # === Do not use ldconfig
      #
      # We will not use ldconfig, but we will continue regardless.
      #
      # Usage example:
      #
      #   rbt htop --do-not-use-ldconfig
      #   rbt htop noldconfig
      #   rbt htop do_not_use_ldconfig
      #
      # ===================================================================== #
      when /^-?-?no(-|_| )?ldconfig/,
           /^-?-?do(-|_| )?not(-|_| )?use(-|_| )?ldconfig$/i,
           /^-?-?dont(-|_| )?run(-|_| )?ldconfig$/,
           /^-?-?do(-|_| )?not(-|_| )?run(-|_| )?ldconfig$/,
           /^-?-?dont(-|_| )?ldconfig$/,
           /^-?-?no(-|_| )?ld$/
        do_not_use_ldconfig
      # ===================================================================== #
      # === Search for tags
      #
      # Do a search for specific tags via the entry point here.
      #
      # Invocation examples:
      #
      #   ry --tags pdf
      #   ry tags? database
      #   ry scons --tags
      #
      # ===================================================================== #
      when /^-?-?tags?\??$/
        search_for_tag(compile_which_program?)
      # ===================================================================== #
      # === Do not use any compile time flags
      #
      # Sometimes you may not wish to use any FLAG variables such as
      # CFLAGS or LDFLAGS. This option here allows you to ignore these
      # FLAGS globally. 
      #
      # Invocation example:
      #
      #   rbt htop --no-compile-time-flags
      #   rbt strace --no-compile-time-flags ntrad
      #   rbt ruby no-compile-time-flags
      #
      # ===================================================================== #
      when /^-?-?no(_|-| )?compile(_|-| )?time(_|-| )?flags$/,
           /^-?-?plain$/,
           /^-?-?buildtype=plain$/
        do_not_use_any_compile_time_flags
      # ===================================================================== #
      # === Query whether we will use autogen
      #
      # Use this entry point if you wish to find out whether we will use
      # autogen or whether we will not.
      #
      # Invocation examples:
      #
      #   ry matesystemmonitor --use-autogen?
      #   ry php --use-autogen?
      #
      # ===================================================================== #
      when /^-?-?uses?(_|-| )?autogen\??/
        verbose_report_will_we_use_autogen
        exit_program
      # ===================================================================== #
      # === Compile both traditional and non-traditional
      #
      # Use this to compile both traditional and non-traditional.
      #
      # Usage examples:
      #
      #   ry htop trad_ntrad
      #   ry php --dual-compile
      #
      # ===================================================================== #
      when /^-?-?trad(_|-| )?ntrad$/i,
           /^-?-?dual(_|-| )?compile$/i,
           /^-?-?trad(_|-| )?then(_|-| )?ntrad$/i,
           /^-?-?trad(_|-| )?and(_|-| )?ntrad$/i,
           /^-?-?trad(_|-| )?and(_|-| )?nontrad$/i
        do_compile_in_both_traditional_and_non_traditional_manner
      # ===================================================================== #
      # === Compile in a traditional way                           (trad tag)
      #
      # We use traditional compilation here by changing the compile mode
      # (traditional or non-gobolinux way) here.
      #
      # Usage example:
      #
      #   ry make trad
      #   ry php --trad
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_COMPILE_IN_A_TRADITIONAL_WAY_OPTIONS
        do_compile_in_a_traditional_way
      # ===================================================================== #
      # === Query whether libtool files will be removed
      #
      # This entry point allows us to query whether we will delete or
      # keep libtool .la files.
      #
      # Invocation example:
      #
      #   ry --delete_libtool_files?
      #
      # ===================================================================== #
      when /^-?-?delete(-|_| )?libtool(-|_| )?files\??$/i
        orev 'Will we remove libtool files: '+
              teal(
                verbose_truth(delete_libtool_files?)
              )
      # ===================================================================== #
      # === Compile all KDE5 plasma components
      #
      # This entry point is a shortcut to batch-compile all KDE5
      # plasma components.
      #
      # Invocation examples:
      #
      #   rbt --compile-plasma
      #   rbt --compile-all-kde5-plasma-components
      #   rbt --kde-plasma
      #   rbt --compile-plasma-desktop
      #   rbt --compile-all-of-plasma
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?plasma$/,
           /^-?-?compile(-|_| )?all(-|_| )?kde5(-|_| )?plasma(-|_| )?components$/,
           /^-?-?compile(-|_| )?kde(-|_| )?plasma$/,
           /^-?-?compile(-|_| )?plasma(-|_| )?desktop$/,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?plasma$/,
           /^-?-?kde(-|_| )?plasma$/,
           /^-?-?all(-|_| )?of(-|_| )?plasma$/
        compile_all_kde5_plasma_components
      # ===================================================================== #
      # == Search for a specific tag
      #
      # This entry point allows us to query for a specific registered tag.
      #
      # Invocation example:
      #
      #   rbt --tags=xorg
      #   rbt --tags=xorg_app
      #
      # ===================================================================== #
      when /^-?-?tags=(.+)$/i
        search_for_tag($1.to_s.dup)
      # ===================================================================== #
      # === Compile atk-core and atk-bridge
      #
      # Invocation examples:
      #
      #   ry --atk-core-bridge
      #   ry --atk-bridge-core
      #
      # ===================================================================== #
      when /^-?-?atk(_|-| )?core(_|-| )?bridge$/i,
           /^-?-?atk(_|-| )?bridge(_|-| )?core$/i
        set_compile_these_programs_then_batch_compile_them(
          %w( atspi2atk atspi2core )
        )
      # ===================================================================== #
      # === Feedback extra description of a program
      #
      # Feedback extra description for a given program here, by making use
      # of class SanitizedCookbookDataset.
      #
      # Usage example:
      #
      #   ry php edesc
      #
      # ===================================================================== #
      when /^extra(-|_)?description\??$/i,
           'extra_description',
           'extradescription',
           'edesc'
        feedback_extra_description
        exit_program
      # ===================================================================== #
      # === Set source directory
      #
      # Set the source dir here.
      #
      # Usage example:
      #
      #   ry htop --source_dir=/Depot/j
      #
      # ===================================================================== #
      when /^-?-?source(-|_| )?dir$/,
           'source_dir',
           'set_source_dir'
        opnn; set_source_dir(first_arg)
      # ===================================================================== #
      # === Compile all locally available versions of a source archive
      #
      # This entry point allows us to compile all locally available
      # versions of a program, if we have stored it as an archive,
      # such as a .tar.xz file or something like that.
      #
      # The advantage, or "big win" here, is that we can batch-compile
      # all these programs into the corresponding /Programs/NAME/VERSION/
      # directory.
      #
      # Invocation examples:
      #
      #   ry libsigc++ --compile-all-available-local-versions
      #   ry libsigc++ --all-available-local-versions
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?available(-|_| )?local(-|_| )?versions$/i,
           /^-?-?all(-|_| )?available(-|_| )?local(-|_| )?versions$/i
        do_compile_all_available_local_version_of_this_program(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Enable bootstrap for compilation
      #
      # Invocation example:
      #
      #   ntrad binutils --enable-bootstrap
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?bootstrap$/i
        append_this_to_the_configure_options(:enable_bootstrap)
      # ===================================================================== #
      # === Symlink the lib/ subdirectory into /usr/lib/
      #
      # This entry point allows us to symlink the compiled library-files,
      # in particular the .so files, into the /usr/lib/ hierarchy.
      #
      # This will only be honoured when the program is compiled in an
      # AppDir-like fashion, e. g. under /Programs/Foobar/1.1.0/.
      #
      # Invocation examples:
      #
      #   ry gtk+ --symlink-into-usr-lib-dir
      #   ry gtk+ --symlink-into-lib-dir
      #   ry gtk+ --symlink-lib-dir
      #   ry gtk+ --symlink-libs
      #
      # ===================================================================== #
      when /^-?-?symlink(-|_| )?into(-|_| )?usr(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?into(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?libs$/i,
           /^-?-?symlink(-|_| )?into(-|_| )?usr(-|_| )?lib$/i
        do_symlink_appdir_lib_into_usr_lib
      # ===================================================================== #
      # === Install only the bin/ components
      #
      # This entry point can be used to install only the bin/ components
      # of a given program.
      #
      # The reasoning for this option is that we sometimes may only wish
      # to install all entries in the bin/ subdirectory, for one reason
      # or the other.
      #
      # Usage examples:
      #
      #   ry htop do_install_only_binaries
      #   ry xpr  --onlybinaries
      #
      # ===================================================================== #
      when /^-?-?only(_|-| )?binaries$/i,
           /^-?-?do(_|-| )?install(_|-| )?only(_|-| )?binaries$/i,
           /^-?-?only(_|-| )?bin$/i
        do_install_only_binaries
      # ===================================================================== #
      # === Upload to a remote site
      #
      # ry htop upload_to
      # ===================================================================== #
      when 'upload_to','uploadto','upload','uto'
        upload_to(i)
        exit_program
      # ===================================================================== #
      # === Do not show the classname
      #
      # If we don't want to display the classname, use this.
      #
      # Trying this:
      #
      #   ry php noclass fall
      #
      # ===================================================================== #
      when 'noclassname','noclass'
        @internal_hash[:do_show_the_classname] = :be_silent
      # ===================================================================== #
      # === set_pkgconfig_symlink
      #
      # Invocation example:
      #
      #   ry htop --pkgconfig-symlink=yes
      #
      # ===================================================================== #
      when /^-?-?pkgconfig-?symlink=(.+)$/
        set_pkgconfig_symlink($1.to_s.dup) { :be_verbose }
      # ===================================================================== #
      # === Use old configure options
      #
      # This entry point allows you to re-use configure options used by
      # an existing program. This works only for a few programs though
      # such as gcc and perhaps nano.
      #
      # Invocation examples:
      #
      #   ry gcc --reuse-old-version --ntrad
      #   ry gcc --use-this-version=7.1.0 --use-old-configure-options
      #   ry gcc --copy-old-version
      #   ry gcc --copy-old-version --ntrad
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?old(_|-| )?configure(_|-| )?options$/i,
           /^-?-?copy(_|-| )?old(_|-| )?version$/i,
           /^-?-?reuse(_|-| )?old(_|-| )?configure(_|-| )?option$/i,
           /^-?-?reuse(_|-| )?configure(_|-| )?options$/i,
           /^-?-?use(_|-| )?old(_|-| )?config(_|-| )?version$/i,
           /^-?-?use(_|-| )?old(_|-| )?configure(_|-| )?version$/i,
           /^-?-?re(_|-| )?use(_|-| )?gcc(_|-| )?configure$/i,
           /^-?-?re(_|-| )?use(_|-| )?old(_|-| )?gcc(_|-| )?version$/i,
           /^-?-?re(_|-| )?use(_|-| )?old(_|-| )?configure(_|-| )?options$/i,
           /^-?-?re(_|-| )?use(_|-| )?old$/i,
           /^-?-?reuse(_|-| )?options$/i,
           /^-?-?old$/i,
           /^-?-?reuse(_|-| )?old(_|-| )?version$/i
        reuse_old_configure_options
      # ===================================================================== #
      # === Copy the linux kernel .h header files
      #
      # This entry point allows the user to copy the linux-kernel header
      # files to the corresponding versioned AppDir location.
      #
      # Invocation examples:
      #
      #   ry --install-kernel-headers
      #   ry --install-linux-headers
      #   ry --extract-kernel
      #
      # ===================================================================== #
      when /^-?-?install(-|_| )?kernel(-|_| )?headers$/i,
           /^-?-?install(-|_| )?linux(-|_| )?headers?$/i,
           /^-?-?kernel(-|_| )?headers$/i,
           /^-?-?install(-|_| )?kernel$/i,
           /^-?-?linux(-|_| )?headers$/i,
           /^-?-?extract(-|_| )?kernel$/i,
           /^-?-?compile(-|_| )?headers$/i,
           /^-?-?install(-|_| )?headers$/i,
           /^-?-?install(-|_| )?kernel(-|_| )?sources$/i
        install_the_linux_kernel_header_files_via_appdir_prefix
        exit_program
      # ===================================================================== #
      # === Disabling colours
      #
      # Use this method if you wish to disable colours on a per-project
      # (per compile-run) basis. This means that the complete RBT suite
      # will not make use of any colours.
      #
      # Colours can be disabled like in this way:
      #
      #   ry nocolours
      #   ry --nocolours
      #   ry htop --nocolours
      #
      # ===================================================================== #
      when *OPTIONS_NO_COLOURS # This constant is defined in bl $RBT/compile/compile.rb
        disable_colours
      # ===================================================================== #
      # === Compile minimal GCC but with lots of languages enabled
      #
      # Invocation example:
      #
      #   ry --gcc_minimal_but_lots_of_languages_enabled
      #
      # ===================================================================== #
      when /^-?-?gcc(-|_)?minimal(-|_)?but(-|_)?lots(-|_)?of(-|_)?languages(-|_)?enabled$/
        try_to_use_this_profile(i)
      # ===================================================================== #
      # === Show BLFS Information
      #
      # This will show BLFS information.
      #
      # Invocation example:
      #
      #   ry binutils --lfs
      #   ry binutils lfs
      #
      # ===================================================================== #
      when /^-?-?lfs$/,
           'blfs',
           'blfs?',
           'show_information_about_blfs'
        show_information_about_blfs
        exit_program
      # ===================================================================== #
      # === Use autogen
      #
      # Use this entry point to enable autogen.
      #
      # Invocation example:
      #
      #   ry gtk+ use-autogen
      #
      # ===================================================================== #
      when *ARRAY_HELP_USE_AUTOGEN
        enable_autogen
      # ===================================================================== #
      # === Query whether the internal sed wrapper will be used
      #
      # Invocation example:
      #
      #   rbt --use-the-internal-sed-wrapper?
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?the(-|_)?internal(-|_)?sed(-|_)?wrapper\??$/,
           'usetheinternalsedwrapper?'
        verbose_query_whether_we_will_use_the_internal_sed_wrapper(:and_then_exit)
      # ===================================================================== #
      # === Use random version
      #
      # The user can pick to use a random version. This means that we will
      # obtain all matches found in the specific directory, and randomly
      # select one of them.
      #
      # Usage example:
      #
      #   ry gcc --use-random-version
      #
      # ===================================================================== #
      when /-?-?use(-|_| )?random(-|_| )?version/,'--userandomversion'
        do_use_random_version
      # ===================================================================== #
      # === Package RBT Scripts
      #
      # Package all the RBT Scripts together, into one tarball/archive.
      #
      # Invocation examples:
      #
      #   ry --package_rbt_scripts
      #   ry --prs
      #   ry --make_release
      #
      # ===================================================================== # 
      when *ARRAY_HELP_PACKAGE_RBT_OPTIONS
        package_rbt_scripts :then_exit
      # ===================================================================== #
      # === Obtain available Packages
      #
      # Get all available packages as a filtered result.
      #
      #   ry htop --packages?
      #
      # ===================================================================== #
      when /^-?-?packages\??$/i,
          'package',
          'available?',
          'available',
          'pgk?',
          'pkg?'
        get_packages
        exit_program
      # ===================================================================== #
      # === Use different configure options
      #
      # We must allow the user to use a different set of configure options.
      # This entry point ensures that.
      #
      # The user will be able to pass in the configure options specified
      # in a commandline-flag.
      #
      # Invocation examples:
      #
      #   ry gcc --use-these-configure-options="--enable-shared --enable-bootstrap --enable-languages=c,c++,lto,objc,fortran --enable-threads=posix --enable-checking=release --enable-objc-gc --with-system-zlib --enable-libstdcxx-dual-abi --with-default-libstdcxx-abi=gcc4-compatible --disable-libunwind-exceptions --enable-__cxa_atexit --enable-libssp --enable-lto --disable-install-libiberty --with-gnu-ld --verbose --with-arch-directory=amd64 --disable-gtktest --disable-multilib --target=x86_64-slackware-linux --build=x86_64-slackware-linux --host=x86_64-slackware-linux"
      #   ry poppler --use-these-configure-options="--enable-xpdf-headers"
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?these(-|_| )?configure(-|_| )?options=(.+)$/
        @internal_hash[:configure_options] = $4.to_s.dup
      # ===================================================================== #
      # === Enable pkgconfig-symlinks
      #
      # This option exists in order to allow, on the commandline, to
      # enable the symlink-action of .pc files into /usr/lib/pkgconfig.
      #
      # This is only useful if you use an AppDir compilation step of
      # course.
      #
      # Note that the regex below is anchored, so to avoid conflicts
      # with instructions such as --symlink-pkgconfig-files.
      #
      # Invocation examples:
      #
      #   ry htop --pkgconfig-symlink
      #   ry libxi --symlink-pkgconfig-files
      #
      # ===================================================================== #
      when /^-?-?pkgconfig(_|-| )?symlink$/,
           /^-?-?symlink(_|-| )?pkgconfig(_|-| )?files$/
        verbose_enable_pkgconfig_symlink
      # ===================================================================== #
      # === Query who the main maintainers are for RBT right now
      #
      # Invocation example:
      #
      #   rbt --maintainer?
      #
      # ===================================================================== #
      when /^-?-?maintainers?\??$/i
        e "#{rev}You may be able to contact the current lead maintainer(s) via:"
        e
        e "  #{sfancy(RBT.configuration?.email)}"
        e
      # ===================================================================== #
      # === Show components belonging to a particular compile chain
      #
      # This entry point allows us to display the components of a
      # particular (meta)program. So for example, if we pass xfce
      # to the method below then all components of xfce will be
      # displayed, together with their index/position value, on the
      # commandline.
      #
      # Invocation example:
      #
      #   rbt --show-components=xfce
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?components=(.+)$/i # === $2
        show_components($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Feedback the program description
      #
      # Feedback the program description here:
      #
      #   rbt gcc --desc
      #   rbt htop desc?
      #
      # ===================================================================== #
      when /^-?-?descr?$/,
           'de',
           'des',
           /^-?-?descri?p?t?i?o?n?\??$/,
           'beschreibung',
           'desc?',
           'd?',
           'feedback_description',
           'desc1'
        feedback_program_description
        exit_program
      # ===================================================================== #
      # === Show all locally available programs
      #
      # Invocation example:
      #
      #   rbt --locally-available-programs?
      #
      # ===================================================================== #
      when /^-?-?locally(-|_)?available(-|_)?programs\??$/
        show_all_locally_available_programs
        exit_program
      # ===================================================================== #
      # === rbt --format=preinstall
      # === rbt --f=enable_shared
      # ===================================================================== #
      when /^-?-?format=(.+)/,
           /^-?-?f=(.+)/
        feedback_format_specification_of($1.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Create snapcraft file
      #
      # This entry point allows you to create a snapcraft for a particular
      # program, such as PHP.
      #
      # Invocation example:
      #
      #   cookbooks --create-snapcraft-for=php
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?snapcraft(-|_| )?for=(.+)$/
        require 'rbt/utility_scripts/create_snapcraft_file.rb'
        program_name = $3.to_s.dup
        if program_name.include? ','
          program_name = program_name.split(',')
        end
        RBT.create_snapcraft_file_for(program_name)
        exit_program
      # ===================================================================== #
      # === Show the available aliases
      #
      # Invocation example:
      #
      #   rbt --aliases?
      #
      # ===================================================================== #
      when /^-?-?aliases\??/
        show_available_aliases
        exit_program
      # ===================================================================== #
      # === Compile with the listed dependencies as well
      #
      # This entry point allows us to compile all the specified
      # dependencies in a .yml file, prior to the main program
      # at hand.
      #
      # Note that this will always compile everything listed there,
      # even if it is already installed. We may have to change this
      # behaviour in the future, which will then be mentioned here,
      # as well - but for the time being, this is the documented
      # behaviour.
      #
      # Invocation examples:
      #
      #   ry kbreakout --compile-dependencies-as-well
      #   ry python --with-deps
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?dependencies(_|-| )?as(_|-| )?well$/,
           /^-?-?with(_|-| )?deps$/
        compile_the_main_program_including_the_dependencies
      # ===================================================================== #
      # === Fix the libtool file
      #
      # This entry point allows us to fix a faulty .la file, based
      # from reading this information from a file.
      #
      # Invocation examples:
      #
      #   rbt --fix-libtool-from-file
      #   rbt --fix-it
      #   rbt --fixit
      #   rbt --patch-it
      #
      # ===================================================================== #
      when /^-?-?fix(-|_| )?libtool(-|_| )?from(-|_| )?file$/i,
           /^-?-?fix(-|_| )?it$/i,
           /^-?-?patch(-|_| )?it$/i
        fix_libtool_la_file_from_saved_file
      # ===================================================================== #
      # === Ensure that the required directories exist
      #
      # Invocation example:
      #
      #   rbt --ensure-that-the-required-directories-exist
      #
      # ===================================================================== #
      when /^-?-?ensure(-|_| )?that(-|_| )?the(-|_| )?required(-|_| )?directories(-|_| )?exist$/i
        RBT.ensure_that_the_required_directories_exist { :be_verbose }
        exit_program
      # ===================================================================== #
      # === Set a specific source directory
      #
      # This entry point allows us to use a specific source archive
      # directory.
      #
      # Invocation example:
      #
      #   rbt --set-source-archive=/Mount/USB1/last_backup/src/
      #
      # ===================================================================== #
      when /^-?-?set(-|_| )?source(-|_| )?archive=(.+)$/i
        opnn; set_source_dir($3.to_s.dup)
      # ===================================================================== #
      # === Query the existance of a binary name
      #
      # Query whether a binary program exists or not of the given name.
      #
      # Invocation examples:
      #
      #   ry file binary?
      #   ry esetroot binary?
      #   ry xgc binary?
      #   ry xgc --binary?
      #
      # ===================================================================== #
      when *ARRAY_HELP_QUERY_THE_EXISTANCE_OF_A_BINARY_NAME
        binary?(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Show binary duplicates
      #
      # Invocation examples:
      #
      #   rbt gettext --show-binary-duplicates
      #   rbt gtk+ --show-binary-duplicates
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?binary(-|_| )?duplicates$/,
           /^-?-?duplicates$/ # ← This variant is experimental; may be changed one day.
        try_to_show_all_binary_duplicates_of_this_program(
          compile_which_program?
        )
        exit_program
      # ===================================================================== #
      # === Check for binary duplicates under /usr/bin/
      #
      # Via the following invocation you can check for binary duplicates
      # in the /usr/bin/ hierarchy.
      #
      # Invocation examples:
      #
      #   rbt --binary-duplicates?
      #   rbt --duplicates?
      #
      # ===================================================================== #
      when *ARRAY_CHECK_FOR_BINARY_DUPLICATES_OPTIONS
        do_check_for_binary_duplicates
        exit_program
      # ===================================================================== #
      # === Use default chroot prefix being /Depot/Chroot
      #
      # This is a convenience method mostly, which allows me to save
      # typing a manual prefix for a chrooted environment.
      #
      # Invocation examples:
      #
      #   ry htop --chroot
      #   ry coreutils --chroot
      #   ry glibc 2.23 --do-not-run-ldconfig --do-not-symlink --chroot
      #
      # ===================================================================== #
      when /-?-?chroot$/,
           /-?-?chroot(-|_)?prefix/,
           'CHROOT'
        use_default_chroot_as_prefix
      # ===================================================================== #
      # === Feedback all binaries
      #
      # Query for all binaries of any given program and show them when it
      # was found.
      #
      # This obviously requires of you to provide the name of a program.
      #
      # See the following use cases.
      #
      # Usage examples:
      #
      #   ry htop     --all_binaries?
      #   ry automake all_binaries?
      #   ry coreutils binaries?
      #   ry xz feedback_all_binaries
      #
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_ALL_BINARIES_OPTIONS
        show_all_binaries_of_this_program
        exit_program
      # ===================================================================== #
      # === Enable shared and no static compilation
      #
      # This entry point combines --enable-shared and --disable-static
      # into one option.
      #
      # Invocation example:
      #
      #   rbt libxmu --shared-no-static ntrad
      #
      # ===================================================================== #
      when /^-?-?shared(-|_| )?no(-|_| )?static$/i
        enable_shared_disable_static
      # ===================================================================== #
      # === Compile into your home directory
      #
      # This option allows you to compile into your user's home directory.
      # This does NOT use an AppDir layout.
      #
      # Invocation examples:
      #
      #   ry htop --into-home-dir
      #   ry xwd --homedir
      #   ry pango --compile-into-home-dir
      #   ry lzma --static --into-the-home-directory
      #
      # ===================================================================== #
      when 'HOME_DIR',
           /^-?-?tohomedir$/,
           /^-?-?home$/,
           /^-?-?into(-|_| )?home(-|_| )?dir$/,
           /^-?-?compile(-|_| )?into(-|_| )?the(-|_| )?home(-|_| )?dir$/,
           /^-?-?use(-|_| )?home$/i,
           /^-?-?home(-|_| )?dir$/,
           /^-?-?compile(-|_| )?into(-|_| )?home(-|_| )?dir$/,
           /^-?-?home(-|_| )?directory$/,
           /^-?-?homedir$/i,
           /^-?-?into(-|_| )?home(-|_| )?directory$/i,
           /^-?-?into(-|_| )?the(-|_| )?home(-|_| )?directory$/i
        compile_into_the_home_directory
      # ===================================================================== #
      # === Specify to use another build-system
      #
      # This entry point allows us to overrule the build-system that is
      # to be used.
      #
      # Invocation example:
      #
      #   ry htop --use-this-build-system=configure
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?this(_|-| )?build(_|-| )?system=(.+)$/i
        this = $4.to_s.dup
        orev "Using #{royalblue(this)} #{rev}as build system."
        set_use_this_build_system(this)
      # ===================================================================== #
      # === Compile all registered games
      #
      # This entry point allows us to compile all registered games.
      #
      # It is recommend to get all dependencies compiled first, e. g.
      # allegro, SDL, and so forth, as the code is very "dumb" - it
      # will not check much at all.
      #
      # Invocation example:
      #
      #   rbt --compile-all-games
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?games$/i,
           /^-?-?compile(-|_| )?all(-|_| )?the(-|_| )?games$/i,
           /^-?-?compile(-|_| )?games$/i
        compile_all_games
      # ===================================================================== #
      # === Use a profile/decorator for the configure options
      #
      # This entry point allows us to use pre-defined configure options for
      # some programs. It thus gives us some more flexibility when working
      # with different computer systems, under different environments.
      #
      # Invocation examples:
      #
      #   ry gcc --profile=gcc_lfs_pass1
      #   ry qt --use-profile=qt trad
      #   ry binutils --use-profile=binutils_lfs trad
      #
      # ===================================================================== #
      when /^-?-?profile=(.+)$/i,
           /^-?-?use(-|_| )?profile=(.+)$/i
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        try_to_use_this_profile(_)
      # ===================================================================== #
      # === Generate a MacHomebrew Formula
      #
      # This should allow you to generate a Mac homebrew formula.
      #
      # Usage examples:
      #
      #   ry m4 machomebrew
      #   ry php ghs
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?machomebrew(_|-| )?formula$/i,
           /^-?-?mac(_|-| )?home(_|-| )?brew$/,
           /^-?-?generate(_|-| )?homebrew(_|-| )?script$/i,
           /^-?-?ghs$/i
        generate_machomebrew_formula
        exit_program
      # ===================================================================== #
      # === Use /tools as prefix
      #
      # This entry point allows us to use /tools as the prefix value,
      # which is the default for building a LFS/BLFS system.
      #
      # Invocation example:
      #
      #   rbt bison --tools-prefix
      #
      # ===================================================================== #
      when /^-?-?tools(-|_| )?prefix$/i
        set_user_prefix('/tools/')
      # ===================================================================== #
      # === Do not use Opn
      #
      # This means that we will not use Opn to show the names of the
      # class in use.
      #
      # Trigger it like so:
      #
      #   rbt php --no_opn
      #   rbt htop dont_show_names
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?opn$/i,
           /-?-?no(-|_)?opn/,
           /-?-?disable(-|_)?opn/,
           'noopn',
           'do_not_output_the_name',
           /no_?names/,
           /do_?not_?show_?names/,
           /dont_?show_?names/,
           'short'
        disable_opn # Defined in the file "opn.rb".
      # ===================================================================== #
      # === Show which build-system will be prioritized
      #
      # This entry point allows us to query which build-systems are
      # our priorities, e. g. whether we will invoke meson before
      # configure or vice versa, in the event that the given
      # package has more than one build-systems in use.
      #
      # Invocation examples:
      #
      #   rbt --priorities?
      #   rbt --prioritize?
      #   rbt --build-priorities?
      #
      # ===================================================================== #
      when /^-?-?priorities\??$/i,
           /^-?-?prioritize\??$/i,
           /^-?-?build-priorities\??$/i,
           /^-?-?build(_|-| )?system(_|-| )?priorities\??$/i
        show_build_systems_priorities
        exit_program
      # ===================================================================== #
      # === Expand the cookbooks
      #
      # Invocation examples:
      #
      #   rbt --expand
      #   rbt --create-expanded-directory
      #
      # ===================================================================== #
      when /^-?-?expand$/,
           /^-?-?expand(-|_)?cookbooks$/,
           /^-?-?create(-|_)?expanded(-|_)?directory$/,
           /^-?-?expand(-|_)?cookbook(-|_)?dataset$/
        do_expand_the_cookbooks
        exit_program
      # ===================================================================== #
      # === Feedback all binaries of a program, on a single line
      #
      # This will show all associated binaries to the given program at
      # hand.
      #
      # Invocation example:
      #
      #   rbt binutils --binaries-as-one-line?
      #
      # ===================================================================== #
      when /^-?-?binaries(-|_| )?as(-|_| )?one(-|_| )?line\??$/i
        show_all_binaries_of_this_program(:default, :on_a_single_line)
        exit_program
      # ===================================================================== #
      # === Show all entries that have a BLFS entry
      #
      # Use this entry point if you wish to show all cookbook entries that
      # have a BLFS entry.
      #
      # Invocation examples:
      #
      #   rbt --show-programs-with-a-blfs-entry
      #   rbt --all-blfs
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?programs(_|-| )?with(_|-| )?a(_|-| )?blfs(_|-| )?entry$/,
           /^-?-?show(_|-| )?all(_|-| )?blfs(_|-| )?entries$/,
           /^-?-?all(_|-| )?blfs$/
        show_programs_with_a_blfs_entry
        exit_program
      # ===================================================================== #
      # === Make use of /usr/libdir/ as the libdir target
      #
      # Usage example:
      #
      #   rbt nettle trad --libdir
      #
      # ===================================================================== #
      when /^-?-?libdir$/i
        do_use_libdir
      # ===================================================================== #
      # === Generate new cookbook
      #
      # Invocation examples:
      #
      #   rbt --generate_new_cookbook
      #   rbt htop --generate_new
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?new(_|-| )?cookbook$/i,
           /^-?-?generate(_|-| )?new$/i
        generate_new_cookbook
      # ===================================================================== #
      # === Set up a LFS tools-like directory
      #
      # This entry point can be used for the initial steps of LFS, but it
      # could also be used to help bootstrap a new GCC/GLIBC combination.
      #
      # Invocation example:
      #
      #   rbt --lfs-tools
      #
      # ===================================================================== #
      when /^-?-?lfs(-|_| )?tools$/i,
           /^-?-?compile(-|_| )?the(-|_| )?initial(-|_| )?lfs(-|_| )?toolchain$/i
        compile_the_initial_lfs_toolchain
        exit_program
      # ===================================================================== #
      # === Enable everything for compilation
      #
      # This entry point is an interesting one. Given a typical GNU 
      # configure based system, this option allows us to enable ALL 
      # of the various --enable* and --with* options.
      #
      # Why was this entry point added in July 2020? I sometimes had a
      # need to just quickly compile the "most feature rich" program and
      # move on. So I did not want to lose any time for fine tuning
      # at that moment in time. Hence this option was created.
      #
      # Keep in mind that sometimes this may not work, as some
      # options can be mutually exclusive.
      #
      # Also note that when this entry point is used, it will
      # clear all old configure options as-is.
      #
      # Usage example:
      #
      #   rbt ncurses ntrad --enable-everything --dontsymlink
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?everything$/i
        enable_everything_for_the_current_compilation_run
      # ===================================================================== #
      # === Strip the binaries of the given program at hand
      #
      # This option is similar to the "Strip the binaries as-is" variant,
      # but the difference is that we will run "strip" only on the
      # selected binaries of the passed program. Note that this also
      # requires that your working directory is where these files
      # are as well.
      #
      # Invocation example:
      #
      #   rbt --strip-binaries-of=htop
      #
      # ===================================================================== #
      when /^-?-?strip(-|_)?binaries(-|_)?of=(.+)$/i
        _ = $3.to_s.dup
        strip_the_binaries_of_this_program(_)
      # ===================================================================== #
      # === Install a flatpak application via the official flatpak-URL
      #
      # This entry point can be used to install a remote flatpak
      # application from the official flatpak store.
      #
      # Invocation examples:
      #
      #   rbt --install-the-flatpak-for=firefox
      #   rbt --flatpak-for=firefox
      #
      # ===================================================================== #
      when /^-?-?install(-|_| )?the(-|_| )?flatpak(-|_| )?for=(.+)$/i, # === $4
           /^-?-?flatpak(-|_| )?for=(.+)$/i,                           # === $2
           /^-?-?flaty=(.+)$/i                                         # === $1
        _ = $1
        _ = $2 if $2
        _ = $4 if $4
        install_this_flatpak(_.to_s.dup)
      # ===================================================================== #
      # === Deliberately use meson
      #
      # This entry point allows you to use meson.
      #
      # Invocation examples:
      #
      #   rbt htop --use-meson
      #   rbt glib --use-meson
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?meson$/
        do_use_meson_and_ninja
      # ===================================================================== #
      # === Compile only C and C++ language support for GCC
      #
      # Usage examples:
      #
      #   rbt gcc --only-c-and-c++
      #   rbt gcc --only-c-and-cpp
      #
      # ===================================================================== #
      when /^-?-?only(-|_| )?c(-|_| )?and(-|_| )?c\+\+$/i,
           /^-?-?only(-|_| )?c(-|_| )?and(-|_| )?cpp$/i
        compile_gcc_with_only_c_and_cpp
      # ===================================================================== #
      # === Show the expanded dependencies of a given program
      # 
      # This entry point will show the expanded dependencies of
      # a given program.
      #
      # Invocation example:
      #
      #   rbt sed --expanded-dependencies
      #
      # ===================================================================== #
      when /^-?-?expanded(-|_| )?dependencies$/
        feedback_expanded_dependencies_of_this_program
        exit_program
      # ===================================================================== #
      # === Set another prefix
      #
      # ry htop --prefix=$J
      # ===================================================================== #
      when /^--prefix/,
           'set_prefix'
        set_user_prefix(i)
      # ===================================================================== #
      # === Build a Linux from Scratch system
      #
      # This entry point will delegate towards the class that will do a
      # LFS build for us.
      #
      # Invocation example:
      #
      #   ry --lfs
      #
      # ===================================================================== #
      when /^-?-?lfs$/i,
           /^linux(_|-| )?from(_|-| )?scratch$/i,'tolfs'
        linux_from_scratch
        exit_program
      # ===================================================================== #
      # === Quickly compile a GCC version based on other configure options
      #
      # This entry point allows us to quickly compile another gcc version.
      #
      # Invocation example:
      #
      #   ry --compile-gcc=8.1.0
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?gcc=(.+)$/i # === $2
        version_to_use = $2.to_s.dup
        gcc = load_yaml_dataset_for_this_program(:gcc)
        gcc.use_this_version = version_to_use
        gcc.rebuild_path
        if File.exist? gcc.path?
          orev 'Now compiling version '+sfancy(version_to_use)+
               "#{rev} of the program called #{royalblue('gcc')}."
          compile_object = RBT::Action::Installer.new(:gcc) { :do_not_run_yet }
          compile_object.use_this_as_cookbook = gcc
          compile_object.ntrad
          compile_object.do_not_initialize_a_new_cookbook_dataset
          compile_object.run
        end
        exit_program
      # ===================================================================== #
      # === Query the headers of the system
      #
      # This entry point is just a wrapper towards class
      # RBT::QueryHeaderToPackage.
      #
      # Invocation examples:
      #
      #   rbt --query-headers
      #   rbt --query-headers=/usr/local/include/
      #
      # ===================================================================== #
      when /^-?-?query(-|_| )?headers$/i,
           /^-?-?query(-|_| )?headers=(.+)$/i
        _ = $2.to_s.dup
        _.delete!('-')
        if _.empty?
          _ = :default
        end
        require 'rbt/utility_scripts/query_header_to_package.rb'
        RBT::QueryHeaderToPackage.new(_)
        exit_program
      # ===================================================================== #
      # === Create an expanded cookbook dataset for a program
      #
      # This entry point allows us to use RBT to create a .yml file
      # holding all relevant information to describe a program that
      # is to be compiled/installed.
      #
      # Invocation examples:
      #
      #   rbt --create-yaml-file-for=subversion
      #   rbt --create-yaml-file-for=ruby
      #
      # ===================================================================== #
      when /^-?-?create(_|-| )?yaml(_|-| )?file(_|-| )?for=(.+)/
        create_yaml_file_for_this_program($4.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Show the last configure options
      #
      # Note that this entry point depends on show_last_options(). Since
      # as of January 2018, another way exists, based on a .yml file
      # that keeps track of the last configure option in use.
      #
      # Usage examples:
      #
      #   ry ffmpeg last_options?
      #   ry htop last_options?
      #
      # ===================================================================== #
      when *ARRAY_HELP_LAST_CONFIGURE_OPTIONS
        show_last_configure_options_then_exit
      # ===================================================================== #
      # === Show everything about a program based on the current working
      #     directory as input
      #
      # Invocation example:
      #
      #   rbt --show-all-about-this-dir
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?all(-|_| )?about(-|_| )?this(-|_| )?dir$/i,
           /^-?-?download(-|_| )?as(-|_| )?name$/ # === rbt --download-as-name
        show_all_about_this_dir(return_pwd)
      # ===================================================================== #
      # === Show the available profiles
      #
      # This entry point allows us to show the available profiles.
      #
      # Invocation example:
      #
      #   rbt --available-profiles?
      #
      # ===================================================================== #
      when /^-?-?available(-|_| )?profiles\??$/i,
           /^-?-?show(-|_| )?available(-|_| )?profiles$/i,
           /^-?-?profiles\??$/i
        show_available_profiles
        exit_program
      # ===================================================================== #
      # === Generate a LFS-like homepage
      #
      # This will generate a LFS-like homepage, where the individual programs
      # are written down into standalone .html pages.
      #
      # In other words, this is then a web-interface.
      #
      # The Cookbooks submodule solves this problem.
      #
      # Usage examples:
      #
      #   rbt htop generate_homepage
      #   rbt --generate-homepage
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_HOMEPAGE_OPTIONS
        generate_homepage
        exit_program
      # ===================================================================== #
      # === Compile all KDE5 porting aids components
      #
      # The KDE5 porting aids include components such as "kdelibs4support",
      # "khtml", "kjs", "kjsembed", "kmediaplayer" or "kross".
      #
      # Invocation examples:
      #
      #   rbt --porting-aids
      #   rbt --compile-kde-porting-aids
      #   rbt --compile-porting-aids
      #
      # ===================================================================== #
      when /^-?-?porting(-|_| )?aids$/,
           /^-?-?kde(-|_| )?porting(-|_| )?aids$/,
           /^-?-?compile(-|_| )?kde(-|_| )?porting(-|_| )?aids$/,
           /^-?-?compile(-|_| )?porting(-|_| )?aids$/
        set_compile_these_programs(return_porting_aids)
      # ===================================================================== #
      # === Check for binary duplicates in the current working directory
      #
      # Usage examples:
      #
      #   rbt --check-for-binary-duplicates
      #   rbt --check-for-duplicates
      #   rbt --duplicates-in-pwd?
      #
      # ===================================================================== #
      when /^-?-?check(-|_| )?for(-|_| )?binary(-|_| )?duplicates$/i,
           /^-?-?check(-|_| )?for(-|_| )?duplicates$/i,
           /^-?-?duplicates(-|_| )?in(-|_| )?pwd\??$/i
        check_for_binary_duplicates_in_this_directory(return_pwd)
      # ===================================================================== #
      # === Display the program dataset as json file and store it
      #
      # This entry point allows the user to display the cookbook-dataset
      # in json format, and store it into a file.
      #
      # Note that this presently depends on the expanded cookbook dataset.
      #
      # Invocation examples:
      #
      #   rbt htop --json
      #   rbt htop --use-json
      #
      # ===================================================================== #
      when /^-?-?json$/,
           /^-?-?use(-|_| )?json$/
        show_and_store_json_for
        exit_program
      # ===================================================================== #
      # === rbt --nocolours
      # ===================================================================== #
      when /^-?-?no(-|_| )?colours/,
           /^-?-?nocol/,
           /^-?-?disable(-|_| )?colours/,
           'NOCOL',
           'NOCOLOURS'
        disable_colours
      # ===================================================================== #
      # === rbt --set_source_dir=/opt
      # ===================================================================== #
      when /^-?-?set(-|_| )?source(-|_| )?dir=(.+)$/,
           /^-?-?set(-|_| )?src(-|_| )?dir=(.+)$/,
           /^-?-?permanently(-|_| )?set(-|_| )?base(-|_| )?directory(-|_| )?to=(.+)$/
        _ = $3.to_s.dup
        _ = $5.to_s.dup if $5 # This is for the variant on the third line, see: http://rubular.com/r/6E6v45rB7u
        opnn; RBT.permanently_set_source_directory(_) { :be_verbose }
        exit_program
      # ===================================================================== #
      # === Download the remote BLFS page and display it
      #
      # This entry point allows the user to grab the remote content of the
      # BLFS page at hand (about a given program) and then display its
      # content on the commandline.
      #
      # Invocation examples:
      #
      #   ry php --paste-blfs-page
      #   ry ruby --parseblfs
      #   ry ruby --pasteblfs
      #   ry kerberos --show-blfs
      #
      # ===================================================================== #
      when /^-?-?paste(-|_| )?blfs(-|_| )?page$/i,
           /^-?-?blfs(-|_| )?page$/i,
           /^-?-?paste(-|_| )?blfs$/i,
           /^-?-?parse(-|_| )?blfs$/i,
           /^-?-?show(-|_| )?blfs$/i
        try_to_show_the_BLFS_content_of_this_program
        exit_program
      # ===================================================================== #
      # === Set extract to
      #
      # Specify our target directory for extracting an archive. Special
      # shortcuts, such as pwd for "current working directry", also
      # exist.
      #
      # Usage examples:
      #
      #   ry htop --extract_to=pwd
      #   ry htop --base-dir=/tmp/
      #   ry htop --base-dir=/opt/
      #   ry htop --extract-to=/opt/test
      #   ry mesa --use-this-as-temp-dir=/opt/foo
      #   ry htop ntrad --use-this-temp-dir=/opt/foobar_create_it
      #
      # ===================================================================== #
      when /^-?-?extract(_|-| )?to=(.*)$/,                                      # === $2
           /^-?-?temp(_|-| )?dir=(.*)$/,                                        # === $2
           /^-?-?use(_|-| )?temp(_|-| )?dir=(.*)$/,                             # === $3
           /^-?-?base(_|-| )?dir=(.+)$/,                                        # === $2
           /^-?-?use(_|-| )?this(_|-| )?temp(_|-| )?directory=(.*)$/,           # === $4
           /^-?-?use(_|-| )?this(_|-| )?temp(_|-| )?dir=(.*)$/,                 # === $4
           /^-?-?use(_|-| )?this(_|-| )?as(_|-| )?temp(_|-| )?dir=(.*)$/,       # === $5
           /^-?-?use(_|-| )?this(_|-| )?as(_|-| )?temp(_|-| )?directory=(.*)$/, # === $4
           /^-?e=(.*)$/                                                         # === $1
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        _ = $5.to_s.dup if $5
        set_extract_to_this_directory_as_specified_by_the_user(
          _
        )
      # ===================================================================== #
      # === Show whether RBT uses ccache or whether it does not
      #
      # This entry point just queries whether RBT will be making use of
      # ccache or whether it will not. The trailing '?' is mandatory.
      #
      # Invocation example:
      #
      #   rbt --use-ccache?
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?ccache\?$/
        show_whether_we_are_using_ccache_or_whether_we_are_not
      # ===================================================================== #
      # === Disable ccache
      #
      # This entry point will disable ccache, for the current compilation
      # run.
      #
      # Invocation examples:
      #
      #   rbt ruby --ntrad --disable-ccache
      #   rbt --disable-ccache
      #   rbt --do-not-use-ccache
      #   rbt --no-ccache
      #   rbt --ccache-
      #
      # ===================================================================== #
      when /^-?-?disable(-|_| )?ccache$/i,
           /^-?-?do(-|_| )?not(-|_| )?use(-|_| )?ccache$/i,
           /^-?-?no(-|_| )?ccache$/i,
           /^-?-?ccache-$/i,
           /^-?-?dont(-|_| )?use(-|_| )?ccache$/i
        verbose_disable_ccache
      # ===================================================================== #
      # === Tell people where they can find the .sh files
      #
      # Invokation example:
      #
      #   rbt --shell-completion?
      #
      # ===================================================================== #
      when /^-?-?shell(-|_| )?completion\??$/i
        e 'The .sh file for bash completion can be found here:'
        e
        e sfile("  #{project_base_dir?}shell/bash_completion.sh")
        e
        exit_program
      # ===================================================================== #
      # === Use a random, extracted directory
      #
      # This entry point allows us to use a random, extracted directory.
      #
      # Invocation examples:
      #
      #   ry htop --random-extracted-dir
      #   ry php --random-extracted-dir
      #   ry php --random-extracted-dir
      #
      # ===================================================================== #
      when /^-?-?random(_|-| )?extracted(_|-| )?dir$/i,
           /^-?-?random(_|-| )?dir$/i,
           /^-?-?gibberish$/i
        use_a_random_extracted_directory
        exit_program
      # ===================================================================== #
      # === Use a profile/decorator for the configure options
      #
      # This entry point allows us to use the LFS/BLFS glibc variant.
      #
      # Invocation examples:
      #
      #   ry glibc --use-glibc1-profile
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?glibc1(-|_| )?profile$/i,
           /^-?-?glibc1(-|_| )?profile$/i
        try_to_use_this_profile(:glibc1)
      # ===================================================================== #
      # === Show the flexbox only
      #
      # This entry point can be used to only show the flexbox, and then
      # exit.
      #
      # Invocation examples:
      #
      #   rbt ruby --show-flexbox-only
      #   rbt php --show-infobox-only
      #   rbt m4 --show-infobox-only
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?flexbox(_|-| )?only$/i,
           /^-?-?show(_|-| )?infobox(_|-| )?only$/i
        show_flexbox(self)
        exit_program
      # ===================================================================== #
      # === Query whether we will use stow
      #
      # Invoke this entry point like so:
      #
      #   ry --stow?
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_USE_STOW_OPTIONS
        feedback_whether_we_will_use_stow
        exit_program
      # ===================================================================== #
      # === Show how to use batch-compile
      #
      # Invocation example:
      #
      #   ry --batch-compile?
      #
      # ===================================================================== #
      when /-?-?batch-?compile\??$/,
           /-?-?show(_|-| )?how(_|-| )?to(_|-| )?batch(_|-| )?compile/,
           /--batch-?help/
        show_how_to_batch_compile
        exit_program
      # ===================================================================== #
      # === Show the GoboLinux instructions on the commandline
      #
      # This entry point allows us to show the GoboLinux instructions,
      # as far as RBT is concerned.
      #
      # Invocation example:
      #
      #   rbt --show-gobo-instructions
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?gobo(_|-| )?instructions$/
        show_gobolinux_instructions
      # ===================================================================== #
      # === Remove all broken symlinks via class RBT::RemoveSymlinks
      #
      # Use this entry point if you simply wish to invoke class
      # RBT::RemoveSymlinks.
      #
      # Invocation example:
      #
      #   rbt --remove_symlinks
      #
      # ===================================================================== #
      when /^-?-?remove(-|_| )?symlinks$/,
           /^-?-?remove(-|_| )?broken$/i
        action(:remove_symlinks)
        exit_program
      # ===================================================================== #
      # === rbt --download-all-source-archives
      # ===================================================================== #
      when /^-?-?download(-|_| )?all(-|_| )?source(-|_| )?archives$/i,
           /^-?-?download(-|_| )?source(-|_| )?archives$/i,
           /^-?-?download(-|_| )?sources$/i,
           /^-?-?grab(-|_| )?source(-|_| )?archives$/i
        action(:download_all_source_archives)
        exit_program
      # ===================================================================== #
      # === rbt --base_directory?
      # ===================================================================== #
      when /^-?-?base(-|_| )?directory\??$/,
           /^-?-?archive(-|_| )?directory\??$/
        action(:report_where_the_source_directory_can_be_found)
        exit_program
      # ===================================================================== #
      # === rbt --scanu
      # ===================================================================== # 
      when /^-?-?scanu/
        action(:missing_last_update_entries)
        exit_program
      # ===================================================================== #
      # === rbt --n_binaries?
      # ===================================================================== #
      when /^-?-?n(-|_| )?binaries\??$/,
           /^-?-?n(-|_| )?registered(-|_| )?binaries?\??$/,
           /^-?-?report(-|_| )?how(-|_| )?many(-|_| )?binaries(-|_| )?are(-|_| )?registered$/i
        action(:report_how_many_binaries_are_registered)
        exit_program
      # ===================================================================== #
      # === Create the log-directory
      #
      # This entry point will create the log-directory, that is, the
      # directory that will be used by RBT::Action::Installer to store
      # some files.
      #
      # Invocation examples:
      #
      #   rbt --create-log-dir
      #   rbt --create-log-directory
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?log(-|_| )?dir$/i,
           /^-?-?create(-|_| )?log(-|_| )?directory$/i
        create_log_directory
      # ===================================================================== #
      # === Generate global yaml database
      #
      # To generate the global yaml database, do this:
      #
      #   ry --gdatabase
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?global(_|-| )?database$/,
           /^-?-?database$/,
           /^-?-?create-database$/,
           /^-?-?gdatabase$/
        generate_global_yaml_database
        exit_program
      # ===================================================================== #
      # === Batch-install the registered ruby addons
      #
      # This entry point allows us to batch-install the registered ruby
      # addons. It exists mostly due to convenience.
      #
      # Invocation examples:
      #
      #   ry --enable-ruby-addons
      #   ry --ruby-addons
      #   ry --compile-ruby-addons
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?ruby(-|_| )?addons$/i,
           /^-?-?ruby(-|_| )?addons$/i,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?ruby$/i,
           /^-?-?compile(-|_| )?ruby(-|_| )?addons$/,
           /^-?-?install(-|_| )?ruby(-|_| )?addons$/i,
           /^-?-?install(-|_| )?my(-|_| )?ruby(-|_| )?addons$/i
        install_the_ruby_addons
      # ===================================================================== #
      # === Use a specific, locally existing file for compilation
      #
      # The following entry point allows us to make use of a locally
      # existing file and compile that.
      #
      # Invocation examples:
      #
      #   ry --this_file=/home/x/src/php/php-7.3.4.tar.xz
      #   ry --this-file=/home/x/src/kalarm/kalarm-21.08.0.tar.xz
      #
      # ===================================================================== #
      when /^-?-?this(_|-| )?file=(.+)$/i
        _ = $2.to_s.dup
        set_compile_these_programs(_)
      # ===================================================================== #
      # === Check if source directories exist
      #
      # This entry point, a "clean-up step", can be used to find out
      # whether all source directories exist or whether they do not.
      #
      # Invocation example:
      #
      #   ry --check-if-source-dirs-exist
      #
      # ===================================================================== #
      when /-?-?check(-|_| )?if(-|_| )?source(-|_| )?dirs(-|_| )?exist$/
        check_if_source_directories_exist
        exit_program
      # ===================================================================== #
      # === Show the required dependencies via the Flexbox
      #
      # This entry point can be used by the user to show the dependencies
      # of the given program via the flexbox as well.
      #
      # Invocation example:
      #
      #   ry rack --show-required-dependencies
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?required-dependencies$/i
        orev 'Will show the required dependencies in the Fluxbox'
        RBT.configuration?.always_show_dependencies = true
      # ===================================================================== #
      # === Use glib schemata
      #
      # This option allows us to install an AppDir program with the
      # additional glib-schemata step, thus overruling any other
      # setting from the cookbook-yaml file at hand.
      #
      # Invocation example:
      #
      #   rbt anjuta ntrad --use-glib-schemata
      #
      # ===================================================================== #
      when /^-?-?use-?glib-?scheme$/i,
           /^-?-?use-?glib-?schemata$/i,
           /^-?-?glib-?scheme$/i,
           /^-?schemata$/i
        verbose_enable_glib_schema
      # ===================================================================== #
      # === This entry point allows us to set up a new RBT configuration
      #
      # Invocation example:
      #
      #   ry --generate-rbt-config
      #   ry --generate-config
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?rbt(_|-| )?configuration$/i,
           /^-?-?generate(_|-| )?rbt(_|-| )?config$/i,
           /^-?-?generate(_|-| )?config$/i
        require 'rbt/utility_scripts/generate_rbt_configuration.rb'
        RBT::GenerateRbtConfiguration.new
      # ===================================================================== #
      # === Skip the extracting stage
      #
      # Skip extracting the source archive, via a configuration flag.
      #
      # Invocation examples for this entry point:
      #
      #   ry php skip_extracting
      #   ry gtk2 --skipextract
      #
      # ===================================================================== #
      when *ARRAY_HELP_SKIP_EXTRACTING_OPTIONS
        do_not_extract
      # ===================================================================== #
      # === Show which compiler we will use
      #
      # This entry point will simply display, on the commandline, which
      # compiler is used.
      #
      # Invocation examples:
      #
      #   ry --use-which-compiler?
      #   ry --compiler?
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?which(_|-| )?compiler\??$/i,
           /^-?-?compiler\??$/i
        show_which_compiler_will_be_used
        exit_program
      # ===================================================================== #
      # === Permanently remove a global flag
      #
      # Invocation example:
      #
      #   rbt --remove-global-flag=--disable-static
      #
      # ===================================================================== #
      when /-?-?remove(-|_)?global(-|_)?flag=(-?-?.+)$/
        permanently_remove_this_global_flag($3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Download a remote program into the default directory
      #
      # This entry point allows us to quickly download a remote program
      # into a local directory.
      #
      # Invocation examples:
      #
      #   rbt --download-into-proper-directory htop
      #   rbt --download=htop
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?into(-|_| )?proper(-|_| )?directory$/i,
           /^-?-?download=(.+)$/i
        if i.include? '='
          match = $1.to_s.dup
          initialize_dataset_for(match)
        end
        download_this_remote_url_into_that_local_directory(
          url1?, base_dir?
        )
      # ===================================================================== #
      # === Show which programs will be compiled for a base system on Linux
      #
      # Invocation examples:
      #
      #   rbt --compile-base-system?
      #   rbt --compile-basic-system?
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?base(_|-| )?system\?$/,
           /^-?-?compile(_|-| )?basic(_|-| )?system\?$/,
           /^-?-?compile(_|-| )?basic(_|-| )?subsystem\?$/,
           /^-?-?compile(_|-| )?basic(_|-| )?linux(_|-| )?core\?$/,
           /^-?-?compile(_|-| )?basic(_|-| )?linux\?$/
        require 'rbt/utility_scripts/compile_base_system.rb'
        RBT::CompileBaseSystem.report_which_programs_will_be_compiled
        exit_program
      # ===================================================================== #
      # === Invoke the manual_steps value, should it exist 
      #
      # This entry point allows the user to invoke the manual_steps entry,
      # should it exist. A program that has such an entry is "bzip2",
      # for instance. This way we can allow installation of the program
      # based on the manual_steps entries.
      #
      # Invocation example:
      #
      #   ry bzip2 --manual-steps
      #
      # ===================================================================== #
      when /^-?-?manual(-|_)?steps$/i,
           /^-?-?ad(-|_)?hoc$/i
        ensure_that_the_cookbook_dataset_exists(program_name?)
        _ = cookbook_dataset_manual_steps?.to_s.lstrip
        # =================================================================== #
        # Next sanitize it a bit, such as by removing '#' comment lines.
        # =================================================================== #
        _ = _.split("\n").reject {|entry|
          entry.strip.empty? or entry.strip.start_with?('#')
        }
        new_directory = extract_this_archive(
          program_path?,
          return_pwd
        )+program_name_and_program_version?.to_s+'/'
        if File.directory? new_directory
          cd new_directory
          if _.empty?
            orev 'No manual_steps entries were found.'
          else
            orev 'Now running all '+steelblue(_.size.to_s)+
                 rev+' entries (working directory '\
                 'is: '+sdir(return_pwd)+
                 rev+'):'
            e
            _.each {|entry|
              e tomato(entry)
              case entry
              when /^cd /
                cd(entry.split(' ').last)
              else
                system entry
              end
            }
            e
          end
        else
          opne rev+'No directory exists at `'+sdir(new_directory)+rev+'`.'
        end
        exit_program
      # ===================================================================== #
      # === Compile the xorg libraries
      #
      # This entry point allows the user to easily compile all
      # xorg-libraries.
      #
      # Invocation example:
      #
      #   rbt --compile-xorg-libraries
      #   rbt --compile-xorg-base
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xorg(-|_| )?libraries$/i,
           /^-?-?compile(-|_| )?xorg(-|_| )?base$/i
        compile_xorg_libraries
      # ===================================================================== #
      # === Feedback the available formats
      #
      # This entry point will simply show the available (registered)
      # formats.
      #
      # Invocation examples:
      #
      #   rbt --formats?
      #   rbt --feedback_all_available_formats
      #
      # ===================================================================== #
      when /^-?-?formats?\??$/i,
           /^-?-?feedback(-|_)?all(-|_)?available(-|_)?formats$/
        feedback_all_available_formats
        exit_program
      # ===================================================================== #
      # === Show the colours in use
      #
      # Invocation example:
      #
      #   rbt --colours?
      #
      # ===================================================================== #
      when /^-?-?colours\?$/i
        show_the_colours_in_use
        exit_program
      # ===================================================================== #
      # === Show which programs have a dependency on a particular program
      #
      # This entry point enables the user to determine which programs
      # have a dependency on meson. Once that has been established
      # these programs will be shown on the commandline.
      #
      # Usage example:
      #
      #   rbt --dependency-on=meson
      #
      # ===================================================================== #
      when /^-?-?dependency(-|_)?on=(.+)$/i # === $2
        _ = $2.to_s.dup
        show_all_programs_that_have_this_as_dependency(_)
        exit_program
      # ===================================================================== #
      # === Generate the available programs versions
      #
      # This entry point will generate the big .md file that can be used
      # to display which versions are tracked by the RBT project.
      #
      # This .md file is displayed on the homepage of the RBT gem.
      #
      # Invocation example:
      #
      #   rbt --bigfile
      #
      # ===================================================================== #
      when /^-?-?bigfile$/i,
           /^-?-?save(_|-| )?the(_|-| )?available(_|-| )?programs(_|-| )?versions$/i
        action(:SaveTheAvailableProgramsVersions)
        exit_program
      # ===================================================================== #
      # === Do not append no-error
      #
      # This entry point prevents adding the no-error shell script to
      # configure.
      #
      # Invocation example:
      #
      #   ry cmake --do-not-append-no-error
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?append(_|-| )?no(_|-| )?error$/i
        orev 'Not appending no-error to the configure line.'
        @internal_hash[:append_shell_error_code] = false
      # ===================================================================== #
      # === Scan for missing last update entries in the cookbooks entries
      #
      # This entry point allows us to scan for missing "last_update"
      # entries in the various cookbooks .yml files.
      #
      # Invocation example:
      #
      #   ry --scan
      #
      # ===================================================================== #
      when /^-?-?scan$/i
        require 'rbt/checks_and_validations/scan_for_missing_last_update_entries.rb'
        RBT.scan_for_missing_last_update_entries
        exit_program
      # ===================================================================== #
      # === Do not show any output from class Extracter, when extracting
      #     an archive
      #
      # This entry point allows the user to suppress output from class
      # Extracter. This may be useful sometimes because class Extracter
      # is quite verbose/spammy.
      #
      # Usage example:
      #
      #   Installer libdrm --silent-extracter --ntrad
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?show(_|-| )?output(_|-| )?from(_|-| )?the(_|-| )?extracter(_|-| )?class$/i,
           /^-?-?silent(_|-| )?extracter(_|-| )?class$/i,
           /^-?-?silent(_|-| )?extracter$/i
        @internal_hash[:show_output_from_the_extracter_class] = false
      # ===================================================================== #
      # === Do not use a build directory
      #
      # This setting allows the user to specifically NOT use a build
      # directory. Sometimes this may be necessary to have, and
      # overrule via the commandline - hence why this entry point
      # exists.
      #
      # Invocation example:
      #
      #   installer zsync --do-not-use-a-build-directory
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?a(_|-| )?build(_|-| )?directory$/i
        do_not_use_a_build_directory { :be_verbose }
      # ===================================================================== #
      # === Copy the expanded directory to its designated (default) location
      #
      # This entry point can be used to quickly copy the existing set of
      # yaml files (.yml) to their assumed target location. Note that this
      # will overwrite existing .yml files, so do not invoke this if you
      # do not desire that behaviour.
      #
      # The reason this functionality was added in February 2022 was so
      # that we can quickly "bootstrap" compiling new programs from
      # source, on a freshly installed (linux) system.
      #
      # Invocation example:
      #
      #   rbt --copy-expanded-directory
      #
      # ===================================================================== #
      when /^-?-?copy(_|-| )?expanded(_|-| )?directory?$/i,
           /^-?-?copy(_|-| )?expanded(_|-| )?directories$/i
        copy_expanded_directory
        exit_program
      # ===================================================================== #
      # === Sleep before continuing
      #
      # Delay also works as alias to it (since Oct 2011).
      #
      # Usage example:
      #
      #   ry htop sleep5
      #   ry htop --sleep5 --ntrad
      
      # ===================================================================== #
      when /^-?-?sleep(\d+)$/,
           /^-?-?delay(\d+)$/,
           /-sl(\d+)$/
        n_seconds = $1
        orev "Sleeping for `#{teal(n_seconds.to_s)}#{rev}` "\
             "seconds before continuing."
        set_delay_before_continuing(n_seconds)
      # ===================================================================== #
      # === Copy the source tarball into the current working directory
      #
      # This entry point can be used to copy the source archive to the
      # current working directory.
      #
      # Invocation examples:
      #
      #   ry --copy-source=php
      #   ry --copy-source=ruby
      #   ry --copy-this-archive=bison
      #
      # ===================================================================== #
      when /^-?-?copy(-|_| )?source=(.+)$/i,             # === $2
           /^-?-?copy(-|_| )?this(-|_| )?archive=(.+)$/i # === $3
        _ = $2.to_s.dup
        _ = $3.to_s.dup if $3
        copy_source(_)
        exit_program
      # ===================================================================== #
      # === --add_global_flag
      # ===================================================================== #
      when /^-?-?add(-|_)?global(-|_)?flag (-?-?.+)$/
        add_this_global_flag($3)
      # ===================================================================== #
      # === --remove_global_flag
      # ===================================================================== #
      when /^-?-?remove(-|_)?global(-|_)?flag (-?-?.+)$/
        remove_this_global_flag($3)
      # ===================================================================== #
      # === --update-entry
      # ===================================================================== #
      when /^-?-?update-?entry (http.+)$/
        update_this_entry($1.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Compile a simple GCC but with lots of languages enabled
      #
      # Invocation examples:
      #
      #   ry gcc --use-profile-minimal-gcc-but-lots-of-languages
      #   ry gcc --minimal-gcc
      #   ry gcc --basic-profile
      #   ry gcc --minimal
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?profile(-|_| )?minimal(-|_| )?gcc(-|_| )?but(-|_| )?lots(-|_| )?of(-|_| )?languages$/i,
           /^-?-?use(-|_| )?profile(-|_| )?minimal(-|_| )?gcc(-|_| )?but(-|_| )?lots(-|_| )?of(-|_| )?languages(-|_| )?enabled$/i,
           /^-?-?minimal(-|_| )?gcc$/i,
           /^-?-?minimal$/i,
           /^-?-?basic(-|_| )?profile$/i,
           /^-?-?gcc1$/i
        try_to_use_this_profile(:basic_gcc_but_with_lots_of_languages_enabled)
      # ===================================================================== #
      # === Backup Program
      #
      # This can be used to backup a single (individual) program.
      #
      # Invocation example:
      #
      #   ry htop --backup-program
      #
      # ===================================================================== #
      when /^-?-?backup(_|-| )?programs?$/,
           'backup-programs',
           'programs_backup',
           'packup',
           'backup'
        backup_program
        exit_program
      # ===================================================================== #
      # === GoboLinux Dependency File
      #
      # This will create a GoboLinux dependency file.
      #
      # Invocation example:
      #
      #   ry sox  --create_dependencies_file
      #   ry htop create_dependencies_file
      #   ry php --create_dependencies_file
      #
      # ===================================================================== #
      when /-?-?create(_|-| )?dependencies(_|-| )?file$/,
           /-?-?dependency(_|-| )?file$/
        create_dependencies_file
        exit_program
      # ===================================================================== #
      # === Show some instructions for GoboLinux
      #
      # Invocation examples:
      #
      #   rbt --gobolinux-instructions
      #   rbt --show-instructions-for=gobolinux
      #
      # ===================================================================== #
      when /^-?-?gobolinux(-|_| )?instructions$/i,
           /^-?-?show(-|_| )?instructions(-|_| )?for=(.+)$/i
        show_instructions_for_gobolinux
        exit_program
      # ===================================================================== #
      # === Query whether we will use colours
      #
      # This entry point allows you to query whether colours will be
      # used in class RBT::Compile for the current compilation run.
      #
      # Invocation example:
      #
      #   ry --use-colours?
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?colours\??$/
        report_whether_colours_will_be_used
        exit_program
      # ===================================================================== #
      # === Query the debug value
      #
      # Query the debug value:
      #
      #   ry --debug?
      #
      # ===================================================================== #
      when /^-?-?query(_|-| )?debug(_|-| )?value$/,/^-?-?debug\??$/
        query_debug_value
        exit_program
      # ===================================================================== #
      # === Use a specific compile-version
      #
      # This allows us to use another version at compile-time.
      #
      # This only works if we compile in an app-dir like variant.
      # ===================================================================== #
      when /^-?-?compile-?version=(.+)/,
           /^-?-?use_?this_?compile_?version=(.+)/
        use_this_compile_version($1.to_s.dup)
      # ===================================================================== #
      # === Scan the Source Archive
      #
      # Scan source archive:
      #
      #   ry --scan_source_archive
      #
      # ===================================================================== #
      when *ARRAY_HELP_SCAN_SOURCE_ARCHIVE_OPTIONS
        scan_source_archive
        exit_program
      # ===================================================================== #
      # === Add a packager-user
      #
      # Add a packager-user this way:
      #
      #   ry php add_package_user
      #
      # ===================================================================== #
      when /^-?-?add(_|-| )?package(_|-| )?user$/i,
           'adduser','packageuser','useradd'
        add_package_user
        exit_program
      # ===================================================================== #
      # === Clear the global flags
      #
      # Invocation example:
      #
      #   rbt --clear-global-flags
      #
      # ===================================================================== #
      when /^-?-?clear(_|-| )?global(_|-| )?flags$/i
        clear_global_flags
        exit_program
      # ===================================================================== #
      # === Do not use the sed wrapper                              (sed tag)
      #
      # If you do not want to apply the internal sed-wrapper, then
      # this is the proper entry point.
      #
      # Invocation examples:
      #
      #   ry libmad --disable-sed
      #   ry cogl --use-simple-sed
      #
      # ===================================================================== #
      when /^-?-?disable(-|_)?internal(-|_)?sed$/,
           /^-?-?use(-|_)?simple(-|_)?sed$/,
           /^-?-?use(-|_)?oldschool(-|_)?sed$/,
           /^-?-?use(-|_)?traditional(-|_)?sed$/,
           /^-?-?oldschool(-|_)?sed$/,
           /^-?-?disable(-|_)?sed(-|_)?wrapper$/
        verbose_disable_sed
      # ===================================================================== #
      # === Keep .la files
      #
      # This allows us to keep .la files, even if the configuration setting
      # specifies otherwises.
      #
      # Invocation example:
      #
      #   ry pcre ntrad --keep-la-files
      #
      # ===================================================================== #
      when /^-?-?keep(-|_)?la(-|_)?files$/i,
           /^-?-?do(-|_)?keep(-|_)?la(-|_)?files$/i
        do_keep_la_files
      # ===================================================================== #
      # === Query whether to use a specific build directory
      #
      # This entry point allows us to determine whether a given program
      # will use a build directory.
      #
      # Usage examples:
      #
      #   ry htop --usebuilddir?
      #   ry libsysstat usebuilddir?
      #
      # ===================================================================== #
      when /^-?-?usebuilddir\??$/i
        report_whether_a_build_directory_will_be_used_for_this_program
        exit_program
      # ===================================================================== #
      # === Run preinstall action from the commandline
      #
      # Sometimes you may wish to run a pre-install action from the
      # commandline, rather than have to rely on the .yml file
      # (which may not even include this pre-install action).
      #
      # The entry point here allows you to do so, and supply your
      # own pre-install actions. Individual entries can be
      # separated via a ',' character.
      #
      # Invocation examples:
      #
      #   rbt libcaca --preinstall="sh bootstrap"
      #   rbt libcaca --prerun="sh bootstrap"
      #   rbt libcaca --prerun="sh bootstrap" --trad
      #
      # ===================================================================== #
      when /^-?-?preinstall="?(.+)"?$/,
           /^-?-?prerun="?(.+)"?$/
        set_preinstall_action($1.to_s.dup)
      # ===================================================================== #
      # === Do not strip the binaries
      #
      # Invocation examples:
      #
      #   ry htop ntrad donotstripbinaries
      #   ry awk ntrad donotstripbinaries
      #
      # ===================================================================== #
      when /^-?-?do(-|_)?not(-|_)?strip(-|_)?the(-|_)?binaries$/,
           /^-?-?do(-|_)?not(-|_)?strip(-|_)?binaries$/,
           /^-?-?do(-|_)?not(-|_)?strip$/,
           /^-?-?no(-|_)?strip$/
        verbose_do_not_strip_the_binaries
      # ===================================================================== #
      # === Feedback whether we will strip the produced binaries or not
      #
      # This is a simple query-method, to determine whether RBT will
      # run the "strip" command on the resulting binaries or whether
      # it will not.
      #
      # Invocation examples:
      #
      #   rbt --use-strip?
      #   rbt --will-we-strip?
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?strip\??$/i,
           /^-?-?strip\??$/i,
           /^-?-?will(-|_)?we(-|_)?use(-|_)?strip\??$/i,
           /^-?-?will(-|_)?we(-|_)?strip\??$/i
        feedback_whether_we_will_strip_or_whether_we_will_not
        exit_program
      # ===================================================================== #
      # === Update registered binaries
      #
      # Update our list of Registered Binaries (the same as running regbin)
      #
      # Invocation example:
      #
      #   ry --update-registered-binaries
      #
      # ===================================================================== #
      when *ARRAY_HELP_UPDATE_BINARY_NAMES_OPTIONS
        update_binary_names :then_exit
      # ===================================================================== #
      # === Register the Cookbook tags
      #
      # This method will create all registered tags (associated with the
      # cookbook entries).
      #
      # Invocation example:
      #
      #   ry --register-tags
      #
      # ===================================================================== #
      when /register(_|-| )?tags/
        action(:create_registered_tags)
        exit_program
      # ===================================================================== #
      # === Query whether the cookbook will modify env variables
      #
      # This option exists to give users means to query, from the
      # commandline, whether the target cookbook-yaml file will make
      # use of special ENV variables.
      #
      # For example, the program called "mesa" may require the
      # manipulation of the ENV variable called "GLL_DRV" being
      # set, before compilation should start.
      #
      # Invocation example:
      #
      #   ry mesa --modify_env_variables?
      #   ry gcc --modify_env_variables?
      #
      # ===================================================================== #
      when /^-?-?modify(_|-| )?env(_|-| )?variables\??$/
        feedback_whether_the_cookbook_will_modify_env_variables
        exit_program
      # ===================================================================== #
      # === Copy header (.h) files to the current working directory
      #
      # This entry point copies .h files that are registered for
      # a given program at hand, into the current working directory.
      #
      # This entry point is a bit different from the variant that is
      # used via --copy-headers-from=, in that we will ONLY copy
      # registered .h files that are available under an AppDir-like
      # prefix. The other method can also use .h files that reside
      # under the /usr/include/ hierarchy.
      #
      # Invocation examples:
      #
      #   ry glibc --copy-headers
      #   ry glibc --copy-headers-into-pwd
      #   ry glibc --copy-headers-to-pwd
      #
      # ===================================================================== #
      when /^-?-?copy(-|_| )?headers$/i,
           /^-?-?copy(-|_| )?headers(-|_| )?into(-|_| )?pwd$/i,
           /^-?-?copy(-|_| )?headers(-|_| )?to(-|_| )?pwd$/
        copy_headers_from(appdir_prefix?+'include/')
        exit_program
      # ===================================================================== #
      # === Copy the linux kernel .h header files into the home dir
      #
      # This entry point allows the user to copy the linux-kernel header
      # files to the home directory.
      #
      # Invocation example:
      #
      #   ry --install-kernel-headers-into-the-home-dir
      #
      # ===================================================================== #
      when /^-?-?install(-|_| )?kernel(-|_| )?headers(-|_| )?into(-|_| )?the(-|_| )?home(-|_| )?dir$/i,
           /^-?-?install(-|_| )?linux(-|_| )?headers(-|_| )?into(-|_| )?the(-|_| )?home(-|_| )?dir$/i
        install_the_linux_kernel_header_files_via_this_prefix(home_dir?)
        exit_program
      # ===================================================================== #
      # === Create a gentoo Ebuild Recipe
      #
      # This entry point allows us to build a gentoo-specific ebuild
      # file. This is an experimental feature as of the year 2018.
      #
      # For the format specification you can look at this remote wiki:
      #
      #   https://wiki.gentoo.org/wiki/Ebuild
      #
      # Invoke it in such a way:
      #
      #   ry ruby --create_ebuild_recipe
      #   ry htop create_ebuild_recipe
      #   ry python ebuild
      #
      # ===================================================================== #
      when /^-?-?create(_|-| )?ebuild(_|-| )?recipe$/,
           'make_ebuild',
           'ebuild',
           'gentoo',
           'gent'
        create_ebuild_recipe
        exit_program
      # ===================================================================== #
      # === Register all programs
      #
      # Usage example:
      #
      #   ry --register_all
      #
      # ===================================================================== #
      when /-?-?register(_|-| )?all$/,
           'regall'
        register_all_programs
        exit_program
      # ===================================================================== #
      # === Specify to use cmake as build-system
      #
      # Invocation example:
      #
      #   ry htop --use-cmake
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?cmake$/i
        set_use_this_build_system(:cmake)
      # ===================================================================== #
      # === Append -march=native onto the CFLAGS
      #
      # Invocation example:
      #
      #   ry sed --march=native -O3 --static --opt
      #
      # ===================================================================== #
      when /^-?-?march=native$/,
           /^-?-?native/i
        _ = '-march=native'
        orev "Appending #{steelblue(_)} #{rev}onto the CFLAGS next."
        append_onto_the_cflags(_)
      # ===================================================================== #
      # === rbt --feedback_expanded_dependencies_of_this_program
      # ===================================================================== #
      when /^-?-?feedback(-|_| )?expanded(-|_| )?dependencies(-|_| )?of(-|_| )?this(-|_| )?program$/i
        feedback_expanded_dependencies_of_this_program
        exit_program
      # ===================================================================== #
      # === Open a specific yaml file via the editor
      #
      # This entry point allows you to open a specific yaml file.
      #
      # Invocation examples:
      #
      #   rbt --edit-file=nano
      #   rbt --edit-file=ruby
      #   rbt --edit=nano
      #
      # ===================================================================== #
      when /-?-?edit-?file=(.+)/,
           /-?-?edit=(.+)/
        edit_this_yaml_file_via_the_main_editor($1.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Compile libstdc++, from GCC
      #
      # Invocation examples:
      #
      #   rbt --compile-libstdc
      #   rbt --compile-libstdc++
      #   rbt --install-libstdc
      #   rbt --install-libstdc++
      #   rbt --c++headers
      #   rbt --libstdc++
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?libstdc$/i,
           /^-?-?compile(-|_| )?libstdc\+\+$/i,
           /^-?-?install(-|_| )?libstdc$/i,
           /^-?-?install(-|_| )?libstdc\+\+$/i,
           /^-?-?c\+?\+?headers$/i,
           /^-?-?libstdc\+\+$/i
        do_compile_libstdc_plus_plus
        exit_program
      # ===================================================================== #
      # === Show the last configure option for a given program
      #
      # Invocation example:
      #
      #   ry attica --show-last-configure-option
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?last(_|-| )?configure(_|-| )?option$/i,
           /^-?-?show(_|-| )?last(_|-| )?configure(_|-| )?option(_|-| )?for(_|-| )?this(_|-| )?program$/i,
           /^-?-?was(_|-| )?compiled(_|-| )?via\??$/
        show_the_last_configure_option_for_this_program(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Delegate to RBT::Cookbooks::Cookbook
      #
      # Invocation example:
      #
      #   rbt --scookie=php
      #
      # ===================================================================== #
      when /^-?-?scookie=(.+)$/i
        instantiate_a_new_cookbook_for($1.to_s.dup).report
        exit_program
      # ===================================================================== #
      # === Delegate to RBT::Cookbooks::Cookbook
      #
      # Invocation example:
      #
      #   ry calligra --scookie
      #
      # ===================================================================== #
      when /^-?-?scookie$/,
           /^-?-?sanitize(-|_| )?cookbook(-|_| )?dataset$/
        instantiate_a_new_cookbook_for(program?).report
        exit_program
      # ===================================================================== #
      # === Check for updates of all registered gems
      #
      # This entry point allows us to batch-update all registered gems in
      # the RBT project.
      #
      # A working internet connection is required for this functionality,
      # obviously.
      #
      # Invocation examples:
      #
      #   rbt --update-all-gems
      #   rbt --update-ruby-gems
      #
      # ===================================================================== #
      when /^-?-?update(-|_)?all(-|_)?gems$/i,
           /^-?-?update(-|_)?gems$/,
           /^-?-?update(-|_)?ruby(-|_)?gems$/,
           /^-?-?update(-|_)?all(-|_)?ruby(-|_)?gems$/i,
           /^-?-?upgrade(-|_)?gems$/i,
           /^-?-?update(-|_)?ruby$/i,
           /^-?-?update(-|_)?ruby(-|_)?addons$/i
        update_all_gems
      # ===================================================================== #
      # === Simply run "waf"
      #
      # Invocation example:
      #
      #   rbt --run-waf
      #
      # ===================================================================== #
      when /^-?-?run(-|_| )?waf$/i
        run_waf
        exit_program
      # ===================================================================== #
      # === Permanently use clang as the main compiler in use
      #
      # This entry point allows the user to permanently use clang as the
      # main compiler in use, rather than, say, GCC.
      #
      # Invocation example:
      #
      #   rbt --permanently-use-clang
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?use(-|_| )?clang$/i,
           /^-?-?do(-|_| )?permanently(-|_| )?use(-|_| )?clang$/i
        do_permanently_use_clang
        exit_program
      # ===================================================================== #
      # === Parse a specific remote LFS/BLFS page and display it
      #
      # This entry point allows the user to parse a remote LFS/BLFS page
      # and display the content on the commandline.
      #
      # Invocation example:
      #
      #   rbt --parse-blfs-page-for=llvm
      # 
      # ===================================================================== #
      when /^-?-?parse(-|_)?b?lfs(-|_)?page(-|_)?for=(.+)$/i, # === $4
           /^-?-?parse(-|_)?b?lfs(-|_)?page(-|_)?of=(.+)$/i   # === $4
        try_to_show_the_BLFS_content_of_this_program($4.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Remove all /home/Programs/*Settings/ directories
      #
      # This is an ad-hoc entry point, where we will simply kill all
      # /Settings/ directories under /home/Programs/. Only invoke
      # this is if you are absolutely certain that you need this
      # functionality.
      #
      # Invocation example:
      #
      #   ry --purge-all-settings-directories
      #
      # ===================================================================== #
      when /^-?-?purge(-|_| )?all(-|_| )?settings(-|_| )?directories$/i
        purge_all_settings_directories
        exit_program
      # ===================================================================== #
      # === Enable the use of the GCC compiler
      #
      # This option will enable the use of the GCC compiler.
      #
      # Invocation example:
      #
      #   ry htop --use-gcc
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?gcc$/i
        do_use_the_gcc_compiler :and_notify_the_user
      # ===================================================================== #
      # === Resymlink all programs
      #
      # This entry point will resymlink all programs under the /Programs
      # hierarchy again, or respectively, wherever you have set your
      # $PROGRAMS value at.
      #
      # Invocation examples:
      #
      #   rbt --resymlink
      #   rbt --resymlink-all-programs
      #   rbt --symlink-all-under-programs-hierarchy
      #
      # ===================================================================== #
      when /^-?-?resymlink$/i,
           /^-?-?do(_|-| )?symlink(_|-| )?all(_|-| )?programs$/i,
           /^-?-?resymlink(-|_| )?all(-|_| )?programs$/i,
           /^-?-?symlink(_|-| )?all(_|-| )?under(_|-| )?programs(_|-| )?hierarchy$/i
        do_symlink_all_programs_then_exit
      # ===================================================================== #
      # === Symlink a single program.
      #
      # Symlink a specific (single) program from within this file.
      #
      # This requires an AppDir like setup, such as within the /Programs/
      # hierarchy.
      #
      # Usage examples:
      #
      #   ry lua --symlink
      #   ry xload --symlink
      #   ry htop --symlink
      #
      # ===================================================================== #
      when /^-?-?symlink$/,
           /^-?-?advanced(_|-| )?symlink$/,
           'asymlink'
        do_invoke_symlink_program(
          compile_which_program?,
          program_version? # bl $RBT/utility_scripts/symlink_program/symlink_program.rb
        )
        exit_program
      # ===================================================================== #
      # === Remove all Symlinks in the /home/Programs/ hierarchy
      #
      # Use this entry point if you wish to eliminate all symlinks for
      # the /Programs/ hierarchy.
      #
      # Invocation examples:
      #
      #   ry --remove_symlinks
      #   ry remove_symlinks
      #   ry CLEAR_SYMLINKS
      #
      # ===================================================================== #
      when *ARRAY_HELP_REMOVE_ALL_SYMLINKS_OPTIONS
        remove_all_symlinks
        exit_program
      # ===================================================================== #
      # === Use BLFS instructions
      #
      # This entry point allows the user to use the BLFS instruction for
      # a given program directly. It will make use of system().
      #
      # The idea behind this feature is that you may be at a computer
      # without a working xorg-server, but you still wish to compile
      # some programs quickly, so it may be more convenient to use
      # these battle-tested instructions as-is, via the commandline.
      #
      # Invocation example:
      #
      #   ry kerberos --use-blfs-instructions
      #   ry binutils --lfs-instructions
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?blfs(-|_)?instructions$/i,
           /^-?-?lfs(-|_)?instructions$/i,
           /^-?-?use(-|_)?lfs(-|_)?instructions$/i
        do_use_the_blfs_instructions_for_compiling_this_program
        exit_program
      # ===================================================================== #
      # === Show the content of the yaml file
      #
      # This entry point can be used to simply display the full yaml
      # file here. That is, the content of a program's recipe.
      #
      # Invocation examples:
      #
      #   ry htop show_all
      #   ry db show_all
      #   ry libmad show-content
      #   ry libmad --show-content
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_ALL_OPTIONS
        show_all
        exit_program
      # ===================================================================== #
      # === Enable --with-newlib
      #
      # Invocation example:
      #
      #   ry gcc --trad --newlib
      #
      # ===================================================================== #
      when /^-?-?newlib$/i
        append_these_configure_options('--with-newlib')
      # ===================================================================== #
      # === Obtain a random program and compile it
      #
      # This may be useful if you just want to check whether compiling
      # any of the registered programs works, without caring too much
      # which entry it is. RBT will randomly pick a program and
      # use that for compilation.
      #
      # Invocation examples:
      #
      #   ry --random
      #   ry --random-program
      #
      # ===================================================================== #
      when 'RANDOM',
           /^-?-?random$/,
           /^-?-?return(_|-| )?random(_|-| )?program$/,
           /^-?-?random(_|-| )?program$/i
        _ = return_random_program
        orev "Compiling the program `#{sfancy(_)}#{rev}` next."
        set_compile_these_programs(_) # ← This will also invoke set_compile_this_program(_)
      # ===================================================================== #
      # === rbt --path_to_source_directory?
      # ===================================================================== #
      when /^-?-?path(-|_| )?to(-|_| )?source(-|_| )?directory\??$/,
           /^-?-?path\??$/
        show_path_to_the_source_directory_yaml_file
        exit_program
      # ===================================================================== #
      # === Extract a local archive
      #
      # This entry point allows us to quickly extract a source archive
      # that exists locally. As argument to this invocation, specify
      # the location to the archive at hand.
      #
      # Invocation examples:
      #
      #   rbt --extract-this=bison-3.2.4.tar.xz
      #   rbt --extract-this=/home/x/src/php/php-8.0.3.tar.xz
      #
      # ===================================================================== #
      when /^-?-?extract(-|_)?this=(.+)$/i
        require 'rbt/toplevel_methods/extract.rb'
        RBT.extract($2.to_s.dup)
      # ===================================================================== #
      # === Query where the source directory is
      #
      # This entry point allows us to query where the source directory
      # on the local computer system resides at.
      #
      # Invocation examples:
      #
      #   rbt --source-dir?
      #   rbt --source-archive?
      #
      # ===================================================================== #
      when *REGEXES_TO_SHOW_THE_SOURCE_DIRECTORY
        e sdir(source_directory?)
      # ===================================================================== #
      # === Query the PATH in use
      #
      # This entry point will display the current PATH in use.
      #
      # Invocation example:
      #
      #   ry --path?
      #
      # ===================================================================== #
      when /^-?-?path\?$/i
        e "#{rev}The path in use is:"
        e
        e slateblue(ENV['PATH'])
        e
      # ===================================================================== #
      # === Update the various tags
      #
      # Invocation example:
      #
      #   rbt --update-tags
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?tags$/
        update_tags
        exit_program
      # ===================================================================== #
      # === Use a specific version based on numbers-input only
      #
      # This entry point allows us to respond to input such as "1.5.2".
      #
      # This is assumed to be a shortcut for --use-this-version=1.5.2.
      #
      # Usage examples:
      #
      #   ry caja 1.21.4
      #   ry caja 1.21.4 --trad
      #
      # ===================================================================== #
      when /^(\d{0,2}\.\d{0,2}\.?\d{0,2}?)$/
        use_a_specific_program_version($1.to_s.dup)
      # ===================================================================== #
      # === Offer simplified bzip2 installation
      #
      # This entry point allows the user to quickly compile bzip2,
      # no matter what.
      #
      # Invocation example:
      #
      #   ry --simplified-bzip
      #
      # ===================================================================== #
      when /^-?-?simplified(-|_)?bzip2?$/i,
           /^-?-?simple(-|_)?bzip2?$/i,
           /^-?-?simple(-|_)?bzip2?(-|_)?installation$/i
        do_perform_simplified_bzip_installation
        exit_program
      # ===================================================================== #
      # === Query the manual_steps value, should it exist 
      #
      # This entry point allows the user to query the manual_steps entry,
      # should it exist. A program that has such an entry is "bzip2",
      # for instance.
      #
      # Note that for this entry step the trailing "?" is mandatory, in
      # order to help distinguish it for the corresponding entry that
      # lacks a trailing "?".
      #
      # Invocation example:
      #
      #   ry bzip2 --manual-steps?
      #
      # ===================================================================== #
      when /^-?-?manual(-|_)?steps\?$/i
        ensure_that_the_cookbook_dataset_exists(program_name?)
        _ = cookbook_dataset_manual_steps?.lstrip
        if _ and !_.empty?
          orev 'The program '+steelblue(program_name?)+
               rev+' has this manual_steps entry:'
          e
          e tomato(_)
          e
        else
          orev 'The program '+steelblue(program_name?)+
               rev+' does not have any manual_steps: entry defined.'
        end
        exit_program
      # ===================================================================== #
      # === Check for available program versions
      #
      # This entry point will check for the available program versions
      # of the given program at hand.
      #
      # Usage examples:
      #
      #   ry libsigc++ --available_versions
      #   ry bluefish --available_versions
      #   ry gcc ?
      #
      # ===================================================================== #
      when *ARRAY_HELP_AVAILABLE_PROGRAM_VERSIONS
        check_for_available_versions
        exit_program
      # ===================================================================== #
      # === Show the minimal GCC configure options
      #
      # This entry point will show the minimal GCC configure options that
      # can be used for when you wish to configure a GCC from scratch.
      #
      # The rationale behind this option is so that we are able to quickly
      # compile GCC without necessarily needing the RBT scripts for this -
      # aka oldschool shell scripts and so forth.
      #
      # Invocation example:
      #
      #   ry --gcc-minimal?
      #
      # ===================================================================== #
      when /^-?-?gcc(_|-| )?minimal\??$/i
        show_minimal_gcc_configure_options
        exit_program
      # ===================================================================== #
      # === Show the last compiled program
      #
      # Use this method if you wish to show the last compiled program.
      #
      # Invocation example:
      #
      #   ry --last_compile?
      #
      # ===================================================================== #
      when *ARRAY_HELP_LAST_COMPILED_OPTIONS
        show_last_compiled_program :then_exit
      # ===================================================================== #
      # === Update binaries of a given program
      #
      # This entry point allows us to automatically update the
      # binaries of a compiled, AppDir-installed program.
      #
      # Invocation examples:
      #
      #   rbt --update-binaries-of=poppler
      #   rbt --update-binaries-for=poppler
      #   rbt --autoupdate=opencv
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?binaries(-|_| )?of=(.+)$/i,
           /^-?-?update(-|_| )?binaries(-|_| )?for=(.+)$/i
        this_program = $1.to_s.dup # Default.
        this_program = $3.to_s.dup if $3 # Only for $3.
        update_binaries_of_this_program(this_program)
      # ===================================================================== #
      # === Report where log files will be stored
      #
      # Invocation example:
      #
      #   ry --report_where_we_will_store_logs
      #
      # ===================================================================== #
      when /^-?-?report(-|_| )?where(-|_| )?we(-|_| )?will(-|_| )?store(-|_| )?logs$/i,
           /^-?-?store(-|_| )?where\??$/i,
           /^-?-?log(-|_| )?dir\??$/i
        report_where_we_will_store_logs
        exit_program
      # ===================================================================== #
      # === Show last update
      #
      # Show the last updated programs in a table format, on the commandline.
      #
      # RBT will, by default, show all available programs.
      #
      # Invocation example:
      #
      #   ry htop last_update?
      #   ry --last_update?
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_LAST_UPDATE_OPTIONS
        show_last_updated
      # ===================================================================== #
      # === Feedback the make options
      #
      # Use this entry point to feedback all possible make options.
      #
      # Invocation example:
      #
      #   ry gcc make_opts
      #
      # ===================================================================== #
      when *ARRAY_FEEDBACK_MAKE_OPTIONS
        feedback_make_options
        exit_program
      # ===================================================================== #
      # === Stop before the 'make' stage/step
      #
      # This entry point allows us to stop at, or rather right before, the
      # 'make' stage.
      #
      # Invocation example:
      #
      #   rbt htop --stop-at-make-stage
      #
      # ===================================================================== #
      when /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make$/i,
           /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?step$/i
        stop_at_make_stage
      # ===================================================================== #
      # === Show the last configure options stored
      #
      # This entry point allows us to query the last configure options
      # stored (which will be stored in a .md file).
      #
      # Invocation example:
      #
      #   rbt --last-configure-options?
      #
      # ===================================================================== #
      when /^-?-?last(_|-| )?configure(_|-| )?options\??$/
        _ = RBT.file_last_configure_options
        if File.exist? _
          e orange(File.read(_).strip)
        else
          opnn; no_file_exists_at(_)
        end
      # ===================================================================== #
      # === rbt --update-kde-framework
      # ===================================================================== #
      when /^-?-?update(-|_| )?kde(-|_| )?framework$/i
        update_kde_framework
        exit_program
      # ===================================================================== #
      # === rbt --update-kde-plasma
      # ===================================================================== #
      when /^-?-?update(-|_| )?kde(-|_| )?plasma$/i
        update_kde_plasma
        exit_program
      # ===================================================================== #
      # === Use a random name for the temp-directory
      #
      # This entry point can be used to make use of a random name for
      # the temp-directory, that is, onto which directory to extract
      # to.
      #
      # Invocation examples:
      #
      #   ry php --random-temp-dir
      #   ry php --random-dir
      #   ry php --random-temp
      #   ry php --rdir
      #
      # ===================================================================== #
      when /^-?-?random(_|-| )?temp(_|-| )?dir$/i,
           /^-?-?random(_|-| )?dir$/i,
           /^-?-?random(_|-| )?temp$/i,
           /^-?-?random(_|-| )?base(_|-| )?dir$/i,
           /^-?-?rdir$/i
        set_extract_to_this_directory_as_specified_by_the_user(:random)
      # ===================================================================== #
      # === Ad-hoc install palemoon
      #
      # This entry point is mostly for my home system, so that I can
      # quickly install palemoon.
      #
      # Invocation example:
      #
      #   rbt --install-palemoon
      #
      # ===================================================================== #
      when /^-?-?install(-|_)?palemoon$/i
        install_palemoon
        exit_program
      # ===================================================================== #
      # === Do not remove empty directories
      #
      # This entry point exists so that we can keep empty directories
      # in the respective /Programs/PROGRAM_NAME/PROGRAM_VERSION/
      # subdirectory.
      #
      # Invocation examples:
      #
      #   rbt feh --ntrad --do-not-remove-empty-directories
      #   rbt htop --ntrad --do-not-remove-empty-directories
      #
      # ===================================================================== #
      when /^-?-?do(-|_| )?not(-|_| )?remove(-|_| )?empty(-|_| )?directories$/
        opne slateblue('class RBT::SymlinkProgram')+' will '+
             crimson('not')+' remove empty directories.'
        symlink_program?.do_keep_empty_directories
      # ===================================================================== #
      # === Perform an intelligent bootstrap-operation
      #
      # This entry point has been added in November 2020, as means to
      # more easily batch-install software on a given computer system.
      #
      # The idea is to query the external gem called environment_information,
      # and then batch-install the missing, or incomplete, programs.
      #
      # Usage examples:
      #
      #   rbt --intelligent-bootstrap
      #   rbt --smart
      #   rbt --what-is-missing
      #
      # ===================================================================== #
      when /^-?-?intelligent(_|-| )?bootstrap$/i,
           /^-?-?what(_|-| )?is(_|-| )?missing\??$/i,
           /^-?-?bootstrap$/i,
           /^-?-?lazy$/i,
           /^-?-?smart$/i,
           /^-?-?compile(_|-| )?missing\??$/i
        do_perform_an_intelligent_bootstrap
        exit_program
      # ===================================================================== #
      # === Extract a program only
      #
      # Extract a given program only, without continuing past this stage.
      #
      # Invocation examples:
      #
      #   ry php eonly
      #   ry htop --extract-only
      #   ry ruby --extract-onl
      #
      # ===================================================================== #
      when /^-?-?extract(-|_| )?only?$/i,
           /^-?-?eonly$/i,
           /^-?-?extract$/i,
           'eonly',
           'eo'
        verbose_do_extract_only
      # ===================================================================== #
      # === Determine whether clang is the main compiler in use
      #
      # This entry point allows us to determine whether clang is in use, or
      # another compiler such as GCC.
      #
      # Invocation example:
      #
      #   ry --use-clang?
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?clang\?$/i
        orev "Will RBT make use of clang right now? "\
             "#{sfancy(verbose_truth(is_clang_used_as_the_main_compiler?))}"
        exit_program
      # ===================================================================== #
      # === Compile/Install all programs that have a specific tag
      #
      # This entry point allows us to e. g. install all of python,
      # that is, every entry that has "python" listed under their
      # tags, or all of "pdf". A requirement for this to work is
      # that the particular .yml file at hand has the particular
      # tag registered.
      #
      # Invocation examples:
      #
      #   rbt --compile-tags=pdf
      #   rbt --compile-tags=python
      #   rbt --compile_all_of=science
      #   rbt --compile_these_tags=science
      #   rbt --tag_compile=science
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?tags=(.+)$/i,            # === $2
           /^-?-?compile(_|-| )?all(_|-| )?of=(.+)/,     # === $3
           /^-?-?compile(_|-| )?these(_|-| )?tags=(.+)/, # === $3
           /^-?-?tag(_|-| )?compile=(.+)/                # === $2
        use_this_tag = $3.to_s.dup
        use_this_tag = $2.to_s.dup if use_this_tag.empty?
        compile_these_tags(use_this_tag)
      # ===================================================================== #
      # === Compile the various xcb-components
      #
      # This entry point exists primarily due to convenience, so that the 
      # user does not have to remember the specific order how the 
      # xcb-components have to be compiled.
      #
      # Invocation examples:
      #
      #   rbt --compile-xcb-components
      #   rbt --compile-the-xcb-components
      #   rbt --compile-all-the-xcb
      #   rbt --compile-xcb
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xcb(-|_| )?components$/i,
           /^-?-?compile(-|_| )?the(-|_| )?xcb(-|_| )?components$/i,
           /^-?-?compile(-|_| )?xcb$/i,
           /^-?-?compile(-|_| )?the(-|_| )?xcb(-|_| )?components$/i,
           /^-?-?compile(-|_| )?all(-|_| )?the(-|_| )?xcb$/i,
           /^-?-?all(-|_| )?of(-|_| )?xcb$/,
           /^-?-?all(-|_| )?xcb$/
        compile_xcb_components
      # ===================================================================== #
      # === Feedback the URL of a specific program
      #
      # This entry point allows us to make use of bin/rbt in order to
      # find out where the (remote) URL of a registered program may
      # be.
      #
      # Invocation examples:
      #
      #   ry --show-url=readline
      #   ry --show-url=libsigc++
      #
      # ===================================================================== #
      when /^-?-?show(-|_)?url=(.+)$/
        feedback_url($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Download the source archive of a given program, then exit
      #
      # Usage example:
      #
      #   ry ruby --downloadhere
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?here$/i,
           /^-?-?dhere$/i
        ensure_cookbooks_object_is_there
        default_download_then_exit(return_pwd)
      # ===================================================================== #
      # === Ignore errors
      #
      # This entry point allows us to ignore any errors encountered, in
      # particular the check whether we can continue or not.
      #
      # Invocation examples:
      #
      #   ry htop --dont-stop-on-errors
      #   ry relion --ignore-errors --ntrad
      #   ry libxslt --no-errors --ntrad
      #
      # ===================================================================== #
      when /^-?-?dont(_|-| )?stop(_|-| )?on(_|-| )?errors$/i,
           /^-?-?ignore(_|-| )?errors$/i,
           /^-?-?no(_|-| )?errors$/i
        orev 'We will not stop upon errors encountered.'
        do_ignore_errors
      # ===================================================================== #
      # === Copy header (.h) files from a certain path
      #
      # This entry point allows us to e. g. copy the .h header files from,
      # say, glibc into the current working directory.
      #
      # Invocation example:
      #
      #   rbt glibc --copy-headers-from=/usr/include
      #
      # ===================================================================== #
      when /^-?-?copy(-|_| )?headers(-|_| )?from=(.+)$/
        this_directory = $3.to_s.dup
        copy_headers_from(this_directory)
        exit_program
      # ===================================================================== #
      # === Strip the binaries as-is
      #
      # This option here allows us to call "strip" at once. The entry point
      # in particular will invoke "strip" on the current working directory,
      # which should be a bin/ subdirectory or a bin/-like subdirectory.
      #
      # Invocation example:
      #
      #   rbt --strip-binaries
      #
      # ===================================================================== #
      when /^-?-?strip(-|_| )?binaries$/
        run_the_strip_command_on_this_input(:pwd)
      # ===================================================================== #
      # === Compile 32 bit programs
      #
      # Invocation example:
      #
      #   rbt make --opt -m32
      #
      # ===================================================================== #
      when /^-m32$/i
        append_these_configure_options('-m32')
      # ===================================================================== #
      # === Compile an existing archive
      #
      # This entry point allows the user to specifically compile a local
      # archive, e. g. such as "ffmpeg-4.2.2.tar.xz".
      #
      # RBT::Action::Installer will make some guesses, in order to infer
      # more information from that archive.
      #
      # Invocation example:
      #
      #   rbt --use-this-file-as-input=/Depot/j/ffmpeg-4.2.2.tar.xz
      #   rbt --use-this-file-as-input=/home/x/src/ruby/ruby-2.7.2.tar.xz
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?this(_|-| )?file(_|-| )?as(_|-| )?input=(.+)$/i # === $5
        _ = $5.to_s.dup
        infer_information_from_this_archive(_)
      # ===================================================================== #
      # === Use GoboLinux SymlinkProgram
      #
      # The RBT scripts provide a class called RBT::SymlinkProgram, which
      # will symlink a program that was installed under the /Programs
      # hierarchy. However had, this class is not 100% the same as the
      # GoboLinux SymlinkProgram script. If you want to invoke the
      # GoboLinux SymlinkProgram rather than the RBT::SymlinkProgram,
      # then this entry point here is allowing you to do precisely that -
      # to invoke the SymlinkProgram script from GoboLinux rather than
      # the integrated class called RBT::SymlinkProgram.
      #
      # Usage example:
      #
      #   rbt --use-gobolinux-symlink-program
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?gobolinux(-|_| )?symlink(-|_| )?program$/i,
           /^-?-?do(-|_| )?use(-|_| )?gobolinux(-|_| )?symlink(-|_| )?program$/i
        do_use_gobolinux_symlinkprogram
      # ===================================================================== #
      # === Create a Slackware Package
      #
      # You can create a slackware package from the current directory
      # through this entry point.
      #
      # Invocation examples:
      #
      #   rbt --create-slackware-package
      #   rbt php --create-slackware-package
      #   Installer ruby --create-slackware-package
      #
      # ===================================================================== #
      when /^-?-?create(_|-| )?slackware(_|-| )?package$/i
        do_create_a_slackware_package(return_pwd)
        exit_program
      # ===================================================================== #
      # === Display the oldest programs in the various cookbook yaml files
      #
      # This entry point allows the user to show which programs are the
      # oldest. That way we could remove ancient, inactive entries.
      #
      # Usage example:
      #
      #   rbt --oldest-programs
      #
      # ===================================================================== #
      when /^-?-?oldest(-|_| )?programs$/i
        display_the_oldest_programs
        exit_program
      # ===================================================================== #
      # === Download the documentation of the RBT project for local use
      #
      # This entry point allows the user to download the remotely available
      # documentation of RBT locally. Perhaps the user does not have a
      # working browser on the current setup, or is in a restricted environment
      # such as campus/university facility. Or the xorg-server no longer
      # works, which actually was the primary reason this entry point was
      # added. It's hard to google for solutions if the xorg-server is down. :)
      #
      # Invocation example:
      #
      #   ry --download-documentation
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?documentation$/i,
           /^-?-?download(-|_| )?docu$/i
         download_the_remote_documentation_of_the_RBT_project
      # ======================================================================= #
      # === Create a new SQL database from scratch
      #
      # This entry point allows the user to generate a new SQL database
      # containing all registered programs.
      #
      # Invocation example:
      #
      #   installer --sql
      #
      # ======================================================================= #
      when /^-?-?SQL$/i
        require 'rbt/sql/generate_sql_table.rb'
        RBT::GenerateSqlTable.new
      # ======================================================================= #
      # === Create a new program-versions-URL file
      #
      # Invocation example:
      #
      #   installer --create_program_version_url_file
      #
      # ======================================================================= #
      when /^-?-?create(-|_| )?program(-|_| )?version(-|_| )?url(-|_| )?file$/
        action(:create_program_version_url_file)
        exit_program
      # ===================================================================== #
      # === Compile the KDE Applications
      #
      # This entry point allows us to compile KDE applications.
      #
      # Usage example:
      #
      #   rbt --compile-kde-apps
      #   rbt --kdeapps
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?kde(-|_| )?apps$/i,
           /^-?-?compile(-|_| )?kde(-|_| )?applications$/i,
           /^-?-?kde(-|_| )?apps$/i
        compile_kde_apps
      # ===================================================================== #
      # === rbt --update-kde-porting-aids
      # ===================================================================== #
      when /^-?-?update(-|_| )?kde(-|_| )?porting(-|_| )?aids$/i
        update_kde_porting_aids
        exit_program
      # ===================================================================== #
      # === Show n last compiled programs
      #
      # This entry point allows us to show the last n compiled programs.
      #
      # Invocation example:
      #
      #   ry --last-compiled=20
      #   ry --last-compiled-programs=10
      #
      # ===================================================================== #
      when /^-?-?last(-|_| )?compiled=(\d+)$/i,                # $2
           /^-?-?last(-|_| )?compiled(-|_| )?programs=(\d+)$/i # $3
        n_programs = $2.to_s.to_i
        n_programs = $3.to_s.to_i if $3
        feedback_last_compiled(n_programs)
        exit_program
      # ===================================================================== #
      # === Show the last compiled programs
      #
      # This entry point can be used to show the last-compiled programs,
      # all of them.
      #
      # Invocation example:
      #
      #   ry --last-compilations?
      #
      # ===================================================================== #
      when /^-?-?last(-|_| )?compilations\??$/i
        feedback_last_compiled(:all_of_them)
        exit_program
      # ===================================================================== #
      # === Show individual configure-profiles
      #
      # This entry point allows the user to display individual profiles.
      #
      # Invocation examples:
      #
      #   ry --show-this-profile=gcc
      #   ry --show-these-profiles=gcc
      #   ry --profiles-of?=gcc
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?this(-|_| )?profile=(.+)$/i,   # $3
           /^-?-?show(-|_| )?these(-|_| )?profiles=(.+)$/i, # $3
           /^-?-?profiles(-|_| )?of\??=(.+)$/i              # $2
        _ = $2
        _ = $3 if $3
        show_all_profiles_fitting_to_this_program(_.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Show help options                                      (help tag)
      #
      # This entry point allows the user to get some help about how to
      # use this project properly.
      #
      # Invocation example:
      #
      #   ry --help
      #
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      # ===================================================================== #
      # === Stop before the 'make install' stage
      #
      # This entry point allows us to stop at, or rather right before,
      # the 'make install' stage.
      #
      # Invocation example:
      #
      #   rbt htop --stop-at-make-install-stage
      #
      # ===================================================================== #
      when /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?install(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?install(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?install$/i,
           /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?install(-|_| )?step$/i
        stop_at_make_install_stage
      # ===================================================================== #
      # === Check all tags
      #
      # This entry point will check all tags under the /Programs/
      # hierarchy.
      #
      # Invocation example:
      #
      #   ry --setup-tags
      #
      # ===================================================================== #
      when *ARRAY_HELP_CHECK_ALL_TAGS_OPTIONS
        check_all_tags_under_programs_directory
        exit_program
      # ===================================================================== #
      # === Use slackware makepkg
      #
      # The RBT.run_makepkg_command can be used to install a slackware
      # package.
      #
      # Invocation example:
      #
      #   ry --makepkg
      #
      # ===================================================================== #
      when /^-?-?makepkg$/
        RBT.run_makepkg_command(:from_the_current_working_directory) # bl $RBT/toplevel_methods/run_makepkg_command.rb
        exit_program
      # ===================================================================== #
      # === Generate a slack-desc file
      #
      # This entry point allows us to use RBT to create a slack-desc file
      # for a slackware package.
      #
      # Invocation examples:
      #
      #   rbt --create-slackdesc-file-for=php
      #   rbt --create-slackdesc-file-for=htop
      #
      # ===================================================================== #
      when /^-?-?create(_|-| )?slackdesc(_|-| )?file(_|-| )?for=(.+)$/i # $4
        this_program = $4.to_s.dup
        create_a_slackdesc_file_for_this_program(this_program)
        exit_program
      # ===================================================================== #
      # === Generate all slackware files
      #
      # This will create a bunch of .desc files into the target
      # directory at rbt_log_directory?+"slackware_desc_files/".
      #
      # Invocation examples:
      #
      #   ry --all-slack-files
      #   ry all-slack-files
      #   ry --create-all-slackdesc-files
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_ALL_SLACK_DESC_FILES_OPTIONS
        generate_slack_desc_files_for_every_program
        exit_program
      # ===================================================================== #
      # == Do not rename the konsole tab.
      #
      # If you do not want to rename the KDE konsole tab, use this entry
      # point here.
      #
      # Invocation example:
      #
      #   ry pango trad dont-rename-tab
      #   ry libtool --trad --dont-rename-konsole
      #   ry php --do-not-rename-tab --mirror
      #
      # ===================================================================== #
      when /^-?-?do(-|_| )?no?t(-|_| )?rename(-|_| )?tab$/i,
           /^-?-?do(-|_| )?no?t(-|_| )?rename(-|_| )?konsole$/i,
           /^-?-?dont(-|_| )?change(-|_| )?tab(-|_| )?title$/i
        do_not_rename_the_kde_konsole_tab
      # ===================================================================== #
      # === Check out the latest source code of the given program
      #
      # This entry point allows us to check out the latest source
      # code of the given program.
      #
      # Invocation examples:
      #
      #   rbt binutils --check-source
      #   rbt binutils --git
      #   rbt mpv --git-download
      #   rbt htop --git-download
      #   rbt ardour --git-download
      #
      # ===================================================================== #
      when /^-?-?check(-|_| )?source$/i,
           /^-?-?git(-|_| )?download$/i,
           /^-?-?git(-|_| )?checkout$/i,
           /^-?-?download(-|_| )?via(-|_| )?git$/i,
           /--git$/i, # Important here to NOT use leading ^-?-?.
           /^-?-?dl$/i,
           /^-?-?use(-|_| )?git$/i,
           /^-?-?latest(-|_| )?git(-|_| )?source$/i,
           /^-?-?git(-|_| )?it$/i,
           /^-?-?latest(-|_| )?and(-|_| )?greatest$/i,
           /^-?-?the(-|_| )?latest(-|_| )?and(-|_| )?greatest$/i,
           /^-?-?get(-|_| )?the(-|_| )?latest$/i,
           /^-?-?the(-|_| )?latest$/i,
           /^-?-?gitty$/i,
           /^-?-?ditty$/i
        if missing_cookbooks_dataset?
          set_compile_this_program(first_argument?)
          initialize_dataset_for(first_argument?)
        end
        if commandline_arguments?.size > 1
          cd base_dir? # This only works if the dataset has been set up properly before.
          download_the_latest_source_code_of_this_program
          exit_program
        end
      # ===================================================================== #
      # === Use a specific version                              (version tag)
      #
      # Note that we will ignore trailing archive information such as
      # ".tar.xz" and similar.
      #
      # Usage examples:
      #
      #   ry htop --version=2.6.30.1
      #   ry gcc  --version=4.9.3.tar.xz
      #   ry gtk2 --compile-version=2.22.111
      #   ry bluefish --version=1.0.7
      #   ry bluefish --use-this-version=1.0.7
      #   ry networkmanager ntrad --use-this-version=NetworkManager-1.12.6.tar.xz
      #   ry mesa trad --use-this-version=mesa-18.2.6
      #
      # ===================================================================== #
      when /^-?-?version=$/,
           /^-?-?use(-|_| )?version=$/,
           /^-?-?version(-|_| )?number=$/,
           /^-?-?use(-|_| )?version=(.+)$/,
           /^-?-?version(-|_| )?number$/,
           /^-?-?version(-|_| )?number /,
           /^-?e=/,
           /^-?-?v=/,
           /^-?-?use(-|_| )?version=$/,
           /^-?-?use(-|_| )?this(_|-| )?version=/,
           /^-?-?cversion=(.+)/,
           /^-?-?compile(-|_| )?version=(.+)$/
        use_a_specific_program_version(i) # ← This method will handle the case.
      # ===================================================================== #
      # === Use a specific value for "make -jN" where N is a number
      #
      # This entry point allows us to quickly make use of the -j flag
      # for the program called "make".
      #
      # It is recommended to set -j to the number of cores the given
      # CPU at hand supports.
      #
      # To use this, try something like:
      #
      #   rbt lame -j5
      #   installer libtool -j8
      #
      # ===================================================================== #
      when /^-j\d{1,2}$/
        set_parameters_to_make(i)
      # ===================================================================== #
      # === Feedback all programs that do use a build directory
      #
      # This entry point simply reports only those programs that make
      # use of a build directory.
      #
      # Invocation examples:
      #
      #   ry --show-all-programs-that-make-use-of-a-build-directory
      #   ry --all-build-directory
      #   ry --all-build-directories
      #   ry --build-directories?
      #   ry --build-directories
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?all(_|-| )?programs(_|-| )?that(_|-| )?make(_|-| )?use(_|-| )?of(_|-| )?a(_|-| )?build(_|-| )?directory$/i,
           /^-?-?all(_|-| )?build(_|-| )?directory$/i,
           /^-?-?all(_|-| )?build(_|-| )?directories$/i,
           /^-?-?build(_|-| )?directories\??$/i
        report_all_programs_that_make_use_of_a_build_directory
        exit_program
      # ===================================================================== #
      # === Feedback the default values of the internal hash
      #
      # If you ever need a way to query the values of the internal
      # hash, you can do so through this entry point here.
      #
      # Invocation example:
      #
      #   ry --feedback-the-default-values-of-the-internal-hash
      #
      # ===================================================================== #
      when /^-?-?feedback(-|_)?default(-|_)?values(-|_)?of(-|_)?the(-|_)?internal(-|_)?hash$/i,
           /^-?-?feedback(-|_)?the(-|_)?default(-|_)?values(-|_)?of(-|_)?the(-|_)?internal(-|_)?hash$/i
        feedback_default_values_of_the_internal_hash
        exit_program
      # ===================================================================== #
      # === Purge FHS libraries of a specific program
      #
      # This entry point will get rid of all registered libraries of a
      # target program. Use with care!
      #
      # It will remove all the libraries belonging to a particular
      # program at the /usr/lib/ hierarchy.
      #
      # Invocation example:
      #
      #   ry --fhs-purge-libraries=libgc
      #
      # ===================================================================== #
      when /^-?-?purge-?fhs-?libraries$/i,
           /^-?-?purge-?traditional-?libraries-?from=(.+)/,
           /^-?-?purge-?fhs-?libraries-?from=(.+)/,
           /^-?-?purge-?fhs-?libraries=(.+)/,
           /^-?-?fhs-?library-?purge=(.+)/,
           /^-?-?fhs-?purge-libraries=(.+)/,
           /^-?-?purge(-|_)?traditional(-|_)?libraries(-|_)?of=(.+)$/
        _ = nil
        _ = $1 if $1
        _ = $4 if $4
        _ = program_name? if _.nil?
        verbose_purge_fhs_libraries_of_this_program(_.to_s.dup)
        exit
      # ===================================================================== #
      # === Do not display the Flexbox
      #
      # Invocation example:
      #
      #   rbt sed --no-flexbox --ntrad
      #   rbt sed --no-infobox --ntrad
      #
      # ===================================================================== #
      when /-?-?do(_|-| )?not(_|-| )?display(_|-| )?the(_|-| )?flexbox$/i,
           /-?-?no(_|-| )?flexbox$/i,
           /-?-?do(_|-| )?not(_|-| )?display(_|-| )?the(_|-| )?infobox$/i,
           /-?-?no(_|-| )?infobox$/i
        orev 'No flexbox will be shown.'
        do_not_display_the_flexbox
      # ===================================================================== #
      # === Whether we use porg or whether we dont
      #
      # This entry point can be used to determine whether porg will be
      # used by the RBT suite.
      #
      # Invocation example:
      #
      #   rbt --use-porg?
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?porg\??$/
        report_whether_we_will_use_porg
        exit_program
      # ===================================================================== #
      # === Query all KDE versions
      #
      # This entry point allows the user to show which KDE versions are
      # in use.
      #
      # Invocation example:
      #
      #   ry --kde-version
      #
      # ===================================================================== #
      when /^-?-?kde(_|-| )?version\??$/i
        try_to_require_the_environment_information_gem
        if Object.const_defined? :EnvironmentInformation
          EnvironmentInformation.new('--kdeversions?')
        end
      # ===================================================================== #
      # === Batch-update all rubygems registered in a .yml file
      #
      # This entry point allows you to batch-download all rubygems
      # that have been updated, compared to your local dataset.
      #
      # If you also need to batch-install these gems then you have
      # to use the RBT project for this.
      #
      # Invocation example:
      #
      #   rbt --update_all_ruby_gems
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?all(-|_| )?ruby(-|_| )?gems$/,
           /^-?-?update(-|_| )?ruby(-|_| )?gems$/
        action(:update_all_ruby_gems)
        exit
      # ===================================================================== #
      # === Install all registered rubygems
      #
      # If you wish to install all registered rubygems, then this
      # entry point may be useful to you.
      #
      # In order for this to work, the RBT project has to tap into the
      # cookbooks project, so make sure that the latter is also installed.
      #
      # Invocation examples:
      #
      #   rbt --install-all-rubygems
      #   rbt --install-all-gems
      #
      # ===================================================================== #
      when /^-?-?install(-|_)?all(-|_)?rubygems$/i,
           /^-?-?install(-|_)?all(-|_)?gems$/i,             
           /^-?-?batch(-|_)?install(-|_)?all(-|_)?gems$/i,
           /^-?-?install(-|_)?all(-|_)?registered(-|_)?gems$/i,
           /^-?-?install(-|_)?rubygems$/i
        update_these_rubygems
      # ===================================================================== #
      # === Compile all of gstreamer
      #
      # As I was tired of compiling gstreamer manually, this entry point
      # was added to allow for all of the gstreamer components to be
      # compiled one after the other.
      #
      # Invocation examples:
      #
      #   rbt --compile-gstreamer
      #   rbt --compile-all-of-gstreamer
      #   rbt --do-gstreamer
      #   rbt --all-of-gstreamer
      #   rbt --gstreamer1
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?gstreamer$/i,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?gstreamer$/i,
           /^-?-?do(-|_| )?gstreamer$/i,
           /^-?-?all(-|_| )?of(-|_| )?gstreamer$/i, # rbt --all-of-gstreamer
           /^-?-?gstreamer1$/i
        compile_all_of_gstreamer
      # ===================================================================== #
      # === Generate all shell scripts
      #
      # This will create a file which bash can use, containing all
      # available shellscripts.
      #
      # Invoke the following as in:
      #
      #   ry --bash
      #
      # ===================================================================== #
      when 'to_bash',
           '2bash',
           'tobash',
           'generate_all',
           'generateall',
           'gall',
           'generate_bash',
           'shell_stuff',
           'allshell',
           'generate_shellscripts',
           'cbs',
           'generate_shell_script',
           'allshellscripts',
           'allsh',
           'toshellscripts',
           /create_?shell_?scripts/,
           'generateshellscripts',
           'shellscripts',
           'def',
           /^-?-?bash$/ # def is special; 'bash' can not be used.
        unless first_argument? == 'bash'
          create_shell_scripts
          exit_program
        end
      # ===================================================================== #
      # === Start the gtk-notebook
      #
      # This entry point shall load the ruby-gtk3 widget that ties together
      # all other, smaller ruby-gtk3 widgets.
      #
      # Invocation examples:
      #
      #   rbt --gtk-notebook
      #   rbt --gtk
      #
      # ===================================================================== #
      when /^-?-?notebook$/i,
           /^-?-?gtk(-|_| )?notebook$/i,
           /^-?-?gtk$/i
        unless first_argument? == 'gtk'
          require 'rbt/gui/gtk3/notebook/notebook.rb'
          RBT::GUI::Gtk::Notebook.run
        end
      # ===================================================================== #
      # === Start the ruby-gtk3 compile widget 
      #
      # This entry-point can be used to start the ruby-gtk3 compile widget.
      #
      # Invocation examples:
      #
      #   rbt --GUI
      #   rbt --gtk-compile-program
      #
      # ===================================================================== #
      when /^-?-?GUI$/i,
           /^-?-?gtk(-|_| )?compile(-|_| )?program/
        require 'rbt/gui/gtk3/compile_program/compile_program.rb'
        ::RBT::GUI::Gtk::CompileProgram.run
      # ===================================================================== #
      # === Query the valid /System/Tags in use
      #
      # Here you can query as to which /System/Tags are valid. And
      # which ones are not.
      #
      # Invocation examples:
      #
      #   ry --valid-sys-tags?
      #   ry --available-tags?
      #
      # ===================================================================== #
      when /^-?-?valid-?sys-?tags\??$/,
           /^-?-?allowed(-|_)?tags$/,
           /^-?-?available(-|_)?tags\??$/,
           /^-?-?allowed(-|_)?tags\??$/
        show_valid_system_tags
        exit
      # ======================================================================= #
      # === Compile/Install all registered perl-addons
      #
      # This entry point allows the user to batch-install the registered
      # perl components.
      #
      # Invocation example:
      #
      #   installer --compile-perl-addons
      #
      # ======================================================================= #
      when /^-?-?compile(-|_)?perl(-|_)?addons$/,
           /^-?-?perl(-|_)?addons$/,
           /^-?-?enable(-|_| )?perl(-|_| )?addons$/i,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?perl$/i,
           /^-?-?do(-|_| )?compile(-|_| )?the(-|_| )?perl(-|_| )?addons$/i,
           /^-?-?perl(-|_| )?inst$/i,
           /^-?-?all(-|_| )?of(-|_| )?perl$/i
        do_compile_the_perl_addons
      # ===================================================================== #
      # === Use /root as prefix
      #
      # This entry point allows us to use /root/ as the prefix value.
      # Note that this is distinct to --rootprefix which uses /.
      #
      # Invocation examples:
      #
      #   ry bison --root
      #   ry libtool --root
      #
      # ===================================================================== #
      when /^-?-?root$/i,
           /^-?-?root(_|-| )?prefix$/i
        set_user_prefix('/root/')
      # ===================================================================== #
      # === Use the / root prefix as target for --prefix=
      #
      # This entry point allows the user to target / as the main prefix
      # to use for the current compilation run.
      #
      # Usage example:
      #
      #   ry coreutils --static --realrootprefix
      #
      # ===================================================================== #
      when /^-?-?real(_|-| )?root(_|-| )?prefix$/i
        set_user_prefix('/')
      # ===================================================================== #
      # === Show the intro for RBT.
      #
      # This entry point will provide a very few useful hints to a new
      # user of the rbt suite.
      #
      # We have to be careful with the input, as "gobject-introspection"
      # includes the word "intro" as well.
      #
      # Invocation example:
      #
      #   rbt --intro
      # 
      # ===================================================================== #
      when /^-?-?intro$/,
           /^-?-?show(_|-| )?intro$/,
           /^-?-?show(_|-| )?rbt(_|-| )?intro$/
        show_rbt_intro
        exit_program
      # ===================================================================== #
      # === Compile the first file found in the current working directory
      #
      # This entry point allows the user to simply compile the first
      # file found in the current directory.
      #
      # Invocation example:
      #
      #   rbt --compile-the-first-entry-in-the-current-working-directory
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?the(_|-| )?first(_|-| )?entry(_|-| )?in(_|-| )?the(_|-| )?current(_|-| )?working(_|-| )?directory$/i
        _ = Dir[original_start_directory?.to_s+'*'].select {|entry| File.file?(entry) }
        first = _.first
        orev 'Trying to compile '+steelblue(first)+"#{rev} next."
        first = program_version_of?(first)
        require 'rbt/utility_scripts/compile_versioned_program/compile_versioned_program.rb'
        RBT::CompileVersionedProgram.new(first) {{ use_this_directory: original_start_directory? }}
        exit_program
      # ===================================================================== #
      # === Show the base directory
      #
      # Show the base directory of a given program.
      #
      # Invocation example:
      #
      #   ry htop base_dir?
      #
      # ===================================================================== #
      when /^-?-?base(_|-| )?dir\??$/,
           /^basedir\??/
        show_where_the_base_directory_can_be_found
        exit_program
      # ===================================================================== #
      # === Show the CFLAGS in use
      #
      # Invocation examples:
      #
      #   rbt --show-cflags
      #   rbt --cflags?
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?cflags$/i,
           /^-?-?cflags\?$/i
        consider_reporting_the_cflags_we_will_use
        exit_program
      # ===================================================================== #
      # === Feedback all registered binaries
      #
      # This entry point allows us to feedback all registered binaries.
      #
      # Invocation examples:
      #
      #   rbt --all-binaries?
      #   rbt --show-binaries
      #
      # ===================================================================== #
      when /^-?-?all(-|_| )?binaries\??$/,
           /^-?-?registered(-|_| )?binaries\??$/,
           /^-?-?show(-|_| )?binaries\??$/
        feedback_all_registered_binaries_of_all_programs
        exit_program
      # ===================================================================== #
      # === Start the ruby-gtk3 statistics widget 
      #
      # This entry-point can be used to start the ruby-gtk3 statistics widget.
      #
      # Invocation example:
      #
      #   ry --gtk-statistics
      #
      # ===================================================================== #
      when /^-?-?gtk(-|_| )?statistics/
        require 'rbt/gui/universal_widgets/statistics/statistics.rb'
        RBT::GUI::UniversalWidgets::Statistics.new('--gtk3')
      # ===================================================================== #
      # === rbt --update-mate-desktop
      #
      # This entry point can be used to compile all of the mate-desktop.
      # ===================================================================== #
      when /^-?-?update(-|_| )?mate(-|_| )?desktop$/i,
           /^-?-?update(-|_| )?mate$/i,
           /^-?-?check(-|_| )?for(-|_| )?mate(-|_| )?updates$/i,
           /^-?-?check(-|_| )?for(-|_| )?mate(-|_| )?desktop(-|_| )?updates$/i,
           /^-?-?check(-|_| )?for(-|_| )?mate(-|_| )?desktop$/i,
           /^--mate(-|_| )?desktop$/i # This variant conflicts with "mate-desktop", hence now the leading --.
        update_mate_desktop_components
        exit
      # ===================================================================== #
      # === Create all program versions 
      #
      # Invocation example:
      #
      #   rbt --create-program-versions
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?program(-|_| )?versions/
        action(:CreateProgramVersionUrlFile)
        exit_program
      # ===================================================================== #
      # === Show statistics about the whole RBT project
      #
      # This will show some statistics about the RBT project and the
      # dataset it contains.
      #
      # Invocation examples:
      #
      #   rbt --stats?
      #   rbt --stats
      #
      # ===================================================================== #
      when /^-?-?stats\??$/i
        show_statistics_about_the_rbt_project
        exit_program
      # ===================================================================== #
      # === Skip the postinstall action
      #
      # This allows you to ignore the postinstall instructions.
      #
      # Usage example:
      #
      #   ry htop --no-postinstall
      #
      # ===================================================================== #
      when /^-?-?skip(_|-| )?postinstall(_|-| )?actions$/i,
           '-P',
           /^-?-?no(_|-| )?postinstall$/i
        orev 'We will skip any potential postinstallation actions.'
        skip_postinstall_actions
      # ===================================================================== #
      # === Show postinstall
      #
      # This entry point allows you to simply show the postinstall
      # dataset for the given program at hand.
      #
      # Usage examples:
      #
      #   ry docbookxml postinstall?
      #   ry perl postinstall?
      #   ry libraw1394 postinstall?
      #   ry littleutils --postinstall?
      #
      # ===================================================================== #
      when /^-?-?postinstall?/,
           /^-?-?show(_|-| )?postinstall/i,
           'pinstall?',
           'p?',
           /^-?-?display(_|-| )?postinstall$/,
           /^-?-?postinstallation(_|-| )?steps\??$/i
        display_postinstall_dataset
        exit_program
      # ===================================================================== #
      # === Generate a big shell script
      #
      # This entry point can be used to quickly generate the big
      # shell script file that contains all programs versions as
      # UPPERCASED variable, with an appended 'V'. So for example,
      # BASHV would have a value of 5.0 or so, depending on the
      # current version of bash registered in the rbt project.
      #
      # Invocation example:
      #
      #   ry --shellscript
      #
      # ===================================================================== #
      when /^-?-?shellscript$/i
        action(:generate_a_big_shell_script)
      # ===================================================================== #
      # === Show all Installed Programs, via a ruby-gtk3 widget
      #
      # Invocation example:
      #
      #   rbt --installed-programs
      #
      # ===================================================================== #
      when /^-?-?installed(-|_| )?programs$/i,
           /^-?-?gtk(-|_| )?installed(-|_| )?programs$/i
        require 'rbt/gui/gtk3/installed_programs/installed_programs.rb'
        RBT::GUI::Gtk::InstalledPrograms.run
      # ===================================================================== #
      # === Start the main libui widget
      #
      # This entry point allows the user to start the libui-bindings for
      # the SQL-centric compilation GUI.
      #
      # Invocation example:
      #
      #   rbt --libui
      #
      # ===================================================================== #
      when /^-?-?libui$/i
        require 'rbt/gui/libui/compile_via_sql_database/compile_via_sql_database.rb'
        RBT::GUI::LibUI::CompileProgram.new
      # ===================================================================== #
      # === Show all available programs
      #
      # This entry point allows the user to display all available programs,
      # that is, all programs that are registered within the RBT project.
      #
      # Usage examples:
      #
      #   rbt --available_programs?
      #   installer --available_programs?
      #
      # ===================================================================== #
      when /^-?-?available(-|_| )?programs\??/i
        show_all_available_programs
        exit
      # ===================================================================== #
      # === Enable the use of the clang compiler
      #
      # This option will enable the (LLVM) clang compiler, in a verbose
      # manner, for the current compilation run.
      #
      # Invocation example:
      #
      #   ry htop --use-clang
      #
      # ===================================================================== #
      when /-?-?use(-|_| )?clang$/i,
           /-?-?do(-|_| )?use(-|_| )?clang$/i,
           /-?-?clang$/i
        do_use_the_clang_compiler :and_notify_the_user
      # ===================================================================== #
      # === Permanently set a specific source directory
      #
      # This entry point allows us to permanently specify any source
      # directory on the local filesystem.
      #
      # Invocation examples:
      #
      #   rbt --permanently-set-source-archive=/Mount/USB1/last_backup/src/
      #   rbt --permanently-set-this-source-directory=/opt/rbt/source_archives
      #   rbt --set-source-dir=/opt/rbt/source_archives
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?set(-|_| )?base(-|_| )?directory(-|_| )?to=(.+)$/i,     # for $5
           /^-?-?permanently(-|_| )?set(-|_| )?source(-|_| )?archive=(.+)$/i,               # for $4
           /^-?-?permanently(-|_| )?set(-|_| )?source(-|_| )?dir(-|_| )?to=(.+)$/i,         # for $5
           /^-?-?permanently(-|_| )?set(-|_| )?source(-|_| )?dir=(.+)$/i,                   # for $4
           /^-?-?permanently(-|_| )?source(-|_| )?directory=(.+)$/i,                        # for $3
           /^-?-?permanently(-|_| )?set(-|_| )?this(-|_| )?source(-|_| )?directory=(.+)$/i, # for $5
           /^-?-?set(-|_| )?source(-|_| )?dir=(.+)$/i,                                      # for $3
           /^-?-?set(-|_| )?src(-|_| )?dir=(.+)$/i,                                         # for $3
           /^-?-?source(-|_| )?dir=(.+)$/i                                                  # for $2
        _ = $2.to_s.dup
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        _ = $5.to_s.dup if $5 # This is for the variant on the first line, see: http://rubular.com/r/6E6v45rB7u
        opnn; RBT.permanently_set_source_directory(_) { :be_verbose }
      # ===================================================================== #
      # === Permanently set programs dir
      #
      # This entry point can be used to permanently set a new PROGRAMS
      # directory.
      #
      # Invocation example:
      #
      #   rbt --permanently_set_programs_dir=/pkg
      #   rbt --use-this-as-programs-dir=/pkg
      #   
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?set(-|_| )?programs(-|_| )?dir=(.+)$/i, # $4
           /^-?-?use(-|_| )?this(-|_| )?as(-|_| )?programs(-|_| )?dir=(.+)$/i # $5
        _ = $4.to_s.dup
        _ = $5.to_s.dup if $5
        permanently_set_programs_dir(_)
      # ===================================================================== #
      # === Permanently enable GoboLinux layout and behaviour from the
      #     commandline
      #
      # Use this method if you wish to let RBT assume that we are
      # running on gobolinux, without changing any ENV variable.
      #
      # Invocation example:
      #
      #   rbt --permanently-enable-is-on-gobolinux
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?enable(-|_| )?is(-|_| )?on(-|_| )?gobolinux$/i
        permanently_enable_is_on_gobolinux(:be_verbose)
        exit_program
      # ===================================================================== #
      # == Permanently do not rename the konsole tab.
      #
      # If you never want to rename the KDE konsole tab, use this entry
      # point here.
      #
      # Invocation example:
      #
      #   ry pango --permanently-do-not-rename-tab
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_| )?do(-|_| )?not(-|_| )?rename(-|_| )?tab$/i
        permanently_do_not_rename_the_kde_konsole_tab
        exit_program # After it is done, we can exit.
      # ===================================================================== #
      # === Permanently disable porg
      #
      # This is as disable_porg(), but we will permanently disable porg.
      #
      # Invocation example:
      #
      #   rbt --permanently-disable-porg
      #
      # ===================================================================== #
      when /-?-?permanently(-|_)?disable(-|_)?porg$/i
        permanently_disable_porg
        exit_program
      # ===================================================================== #
      # === Enable the GoboLinux layout and behaviour from the commandline
      #
      # Note that this entry point enables the GoboLinux layout only
      # for the CURRENT invocation, that is, for the program that you
      # desire to install/compile.
      #
      # If you want to enable it permanently, then either change an
      # environment variable, or use the entry point called
      # --permanently-enable-is-on-gobolinux.
      #
      # Invocation examples:
      #
      #   rbt --enable-is-on-gobolinux
      #   rbt htop ntrad --enable-is-on-gobolinux
      #   ry atk --ntrad --is-on-gobolinux
      #
      # ===================================================================== #
      when /^-?-?is(_|-| )?on(_|-| )?gobolinux$/,
           /^-?-?enable(_|-| )?is(_|-| )?on(_|-| )?gobolinux$/,
           /^-?-?do(_|-| )?we(_|-| )?create(_|-| )?the(_|-| )?architecture(_|-| )?file$/
        enable_is_on_gobolinux
      # ===================================================================== #
      # === Generate shell completion
      #
      # Generate the general tab completions here. Right now this
      # will only work for bash-completions.
      #
      # Invocation example:
      #
      #   rbt --generate-shell-completion
      #   rbt --gsc
      #   rbt --shell
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_SHELL_COMPLETION_OPTIONS
        generate_shell_completion(:all)
        exit_program
      # ===================================================================== #
      # === Mirror-copy
      #
      # Combine --ntrad with --dont-symlink.
      #
      # This may at a later time become a new standard, or part of a
      # new compile chain.
      #
      # Invocation example:
      #
      #   ry glib --mirror
      #   ry atk --mirror
      #
      # ===================================================================== #
      when /^-?-?mirror$/i
        do_not_symlink_in_general_and_also_do_not_symlink_header_files
        do_compile_in_a_non_traditional_way
      # ===================================================================== #
      # === Do not symlink
      #
      # Use this option if you do not wish to automatically symlink
      # the compiled/installed program. This will also disable the
      # instruction "symlink_header".
      #
      # Invocation example:
      #
      #   ry htop dontsymlink
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_NOT_SYMLINK_OPTIONS
        do_not_symlink_in_general_and_also_do_not_symlink_header_files
      # ===================================================================== #
      # == Non-Traditional compilation                            (ntrad tag)
      #
      # This entry point allows the user to compile the given program in
      # an AppDir variant - an AppDir prefix. This would be equivalent
      # to a --prefix value of, for example, /Programs/Ruby/2.6.2/.
      #
      # Invocation examples:
      #
      #   rbt awk gstyle
      #   rbt ruby --app_dir
      #   rbt php --ntrad
      #   Installer libxml2 --ntrad
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_COMPILE_IN_A_NON_TRADITIONAL_WAY_OPTIONS
        do_compile_in_a_non_traditional_way
      # ======================================================================= #
      # === Copy the expanded cookbooks
      #
      # This entry point allows us the user to quickly copy the expanded
      # cookbooks that are distributed via the RBT project. This is not
      # terribly useful for most users, but it may be helpful in the
      # event that the user has a slow computer (so generating the
      # expanded dataset would take a lot of time) or when you just want
      # to compile from scratch.
      #
      # Invocation example:
      #
      #   rbt --copy-expanded-cookbooks
      #
      # ======================================================================= #
      when /^-?-?copy(-|_| )?expanded(-|_| )?cookbooks$/i
        copy_expanded_cookbooks
        exit_program
      # ===================================================================== #
      # === Create the programs URL file
      #
      # This entry point allows us to create the local programs-url file.
      #
      # Invocation examples:
      #
      #   rbt --create-programs-url-file
      #   rbt --create-cookbooks-url-file
      #   rbt --generate-url
      #   rbt --create-programs-version-file
      #   rbt --programs-version-file
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?programs(-|_| )?url(-|_| )?file$/,
           /^-?-?create(-|_| )?programs(-|_| )?url$/,
           /^-?-?create(-|_| )?programs(-|_| )?file$/,
           /^-?-?programs(-|_| )?url(-|_| )?file$/,
           /^-?-?programs(-|_| )?url$/,
           /^-?-?generate(-|_| )?programs(-|_| )?url$/,
           /^-?-?create(-|_| )?cookbooks(-|_| )?url(-|_| )?file$/,
           /^-?-?generate(-|_| )?url$/,
           /^-?-?create(-|_| )?programs?(-|_| )?url(-|_| )?file$/,
           /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?file$/i,
           /^-?-?programs(-|_| )?version$/i,
           /^-?-?programs(-|_| )?version(-|_| )?file$/,
           /^-?-?create(-|_| )?program(-|_| )?version$/,
           /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?html(-|_| )?file$/,
           /^-?-?create(-|_| )?programs(-|_| )?version$/i
        create_programs_url_file
        exit_program
      # ===================================================================== #
      # === Mouse and keyboard and similar input devices for Xorg
      #
      # This entry point is quite similar to --post-xorg-server
      # but simpler. We'll mostly just compile some input-devices,
      # through a hardcoded Array as entry.
      #
      # Invocation examples:
      #
      #   rbt --compile-mouse-and-keyboard
      #   installer --xorg-input
      #
      # ===================================================================== #
      when /^-?-?mouse(-|_| )?and(-|_| )?keyboard$/i,
           /^-?-?mouse(-|_| )?keyboard$/i,
           /^-?-?meyboard$/i,
           /^-?-?compile(-|_| )?mouse(-|_| )?and(-|_| )?keyboard$/i,
           /^-?-?compile(-|_| )?mouse(-|_| )?and(-|_| )?keyboard(-|_| )?xorg(-|_| )?server(-|_| )?components$/i,
           /^-?-?input(-|_| )?drivers$/i,
           /^-?-?input(-|_| )?devices$/i,
           /^-?-?xorg(-|_| )?input(-|_| )?devices$/i,
           /^-?-?xorg(-|_| )?input$/i,
           /^-?-?keyboard(-|_| )?mouse$/i,
           /^-?-?compile(-|_| )?mouse(-|_| )?keyboard$/i,
           /^-?-?keyboard$/i,
           /^-?-?compile(-|_| )?keyboard$/i,
           /^-?-?compile(-|_| )?mouse$/i,
           /^-?-?keyboard(-|_| )?and(-|_| )?mouse$/i
        compile_mouse_and_keyboard_xorg_server_components
      # ===================================================================== #
      # === Show the verbose CFLAGS in use
      #
      # Usage example:
      #
      #   installer --verbose_cflags_in_use?
      #
      # ===================================================================== #
      when /^-?-?verbose(_|-| )?cflags(_|-| )?in(_|-| )?use\??$/i
        e
        e "  #{gold(verbose_cflags_in_use?)}"
        e
      # ===================================================================== #
      # === Show which build-system will be prioritized
      #
      # Invocation example:
      #
      #  ry --configure_has_a_higher_priority_than_meson?
      #
      # ===================================================================== #
      when /^-?-?configure(_|-| )?has(_|-| )?a(_|-| )?higher(_|-| )?priority(_|-| )?than(_|-| )?meson\??$/i
        e configure_has_a_higher_priority_than_meson?
        exit_program
      # ===================================================================== #
      # === Fix all Shebangs
      #
      # Try to fix the shebangs of the RBT project:
      #
      #   ry x shebangfixer
      #
      # ===================================================================== #
      when 'shebang',
           'shebangfixer',
           /^-?-?try_?to_?fix_?all_?shebangs/
        try_to_fix_all_shebangs
        exit_program
      # ===================================================================== #
      # === Compile all programs that belong to a certain tag
      #
      # This entry point allows us to compile all programs that belong
      # to a certain tag, such as "gnome" or "kde5".
      #
      # Invocation examples:
      #
      #   rbt --compile-this-tag=gnome
      #   rbt --compile-this-tag=kde
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?this(-|_| )?tag=(.+)$/
        _ = $3.to_s.dup
        compile_all_programs_that_belong_to_this_tag(_)
      # ===================================================================== #
      # === Show the highest programs
      #
      # Show the highest programs, that is, the biggest in size, available
      # locally.
      #
      # Invocation example:
      #
      #   rbt --show_highest
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_HIGHEST_PROGRAMS_OPTIONS
        show_n_largest_local_programs(150)
      # ===================================================================== #
      # === Show the highest n programs
      #
      # Invocation example:
      #
      #   rbt --highest=50
      #
      # ===================================================================== #
      when /^-?-?highest=(.+)/i
        show_n_largest_local_programs($1.to_s.dup)
      # ===================================================================== #
      # === Query the host system
      #
      # This entry point allows us to query the host-system of the computer.
      #
      # Invocation example:
      #
      #   rbt --host-system
      #
      # ===================================================================== #
      when /^-?-?host(-|_| )?system$/i,
           /^-?-?host(-|_| )?cpu$/i,
           /^-?-?report(-|_| )?host(-|_| )?cpu$/i
        action(:report_host_cpu)
      # ======================================================================= #
      # === Do not use autoconf
      #
      # This entry point allows the user to skip using autoconf. This is
      # only ever useful for the few programs that depend on autoconf,
      # so users will only very rarely need this entry point.
      #
      # Usage example:
      #
      #   rbt bridgeutils --dont-autoconf
      #
      # ======================================================================= #
      when /^-?-?dont(-|_| )?autoconf$/,
           /do(-|_| )?not(-|_| )?use(-|_| )?autoconf$/
        do_not_use_autoconf
      # ======================================================================= #
      # === Post xorg-server components
      #
      # This entry point can be used to install components of the
      # xorg-server after the xorg-server has been compiled/installed.
      #
      # Invocation example:
      #
      #   installer --post-xorg-server
      #
      # ======================================================================= #
      when /^-?-?post(-|_| )?xorg(-|_| )?server$/i,
           /^-?-?post(-|_| )?xorg$/i,
           /^-?-?compile(-|_| )?post(-|_| )?xorg(-|_| )?server$/i
        compile_post_xorg_server_components
      # ======================================================================= #
      # === Edit/Open the yaml file in your editor
      #
      # This entry point allows the user to open the .yml file at hand
      # in his/her editor.
      #
      # Invocation example:
      #
      #   inst --edit
      #
      # ======================================================================= #
      when /^-?-?edit$/i,
           /-?-?open$/i,
           /-?-?modify$/i
        edit_the_yaml_file_via_the_main_editor
        exit_program
      # ===================================================================== #
      # === Compile into the user's home directory, via an AppDir variant
      #
      # This option allows you to compile into your user's home directory,
      # in an AppDir variant.
      #
      # Invocation example:
      #
      #   ry htop --appdir-into-home-dir
      #
      # ===================================================================== #
      when /^-?-?appdir(_|-| )?into(_|-| )?home(_|-| )?dir$/,
           /^-?-?appdir(_|-| )?at(_|-| )?home$/
        compile_into_the_home_directory_as_appdir
      # ===================================================================== #
      # === Combining base? and url? query.
      #
      # Invocation example:
      #
      #   ry ruby --baseurl?
      #   ry pygtk baseurl?
      #
      # ===================================================================== #
      when /urlbase\??/,
           'base?url?',
           'url?base?',
           /baseurl\??/,
           'basurl?',
           /burl/
        feedback_url
        feedback_local_location
        exit_program
      # ===================================================================== #
      # === rbt --show_config
      # ===================================================================== #
      when /^-?-?show_?config/,
           'sconfig'
        show_configuration_options
      # ===================================================================== #
      # === Show the Basename
      #
      # This entry point will show where a particular program can be
      # found locally.
      #
      # Invocation example:
      #
      #   ry gcc basename?
      #
      # ===================================================================== #
      when 'basedir2?','basedirr','bdir','base?','base','localbase?',
           'bdir?',
           'b?','whereis?','whereis',
           /^-?-?basename\??$/i
        feedback_local_base
        exit_program
      # ===================================================================== #
      # === Generate PDF tutorial
      #
      # Invocation example:
      #
      #   ry --pdf-generation
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_PDF_TUTORIAL_OPTIONS
        verbose_generate_the_pdf_tutorial
        exit_program
      # ===================================================================== #
      # === Show the last 20 compiled programs
      #
      # Invocation example:
      #
      #   rbt --last-compiled
      #
      # ===================================================================== #
      when /^-?-?last(-|_| )?compiled$/,
           /^-?-?last(-|_| )?programs$/
        feedback_last_compiled(20)
        exit_program # ^^^ This variant uses hardcoded 20 programs.
      # ===================================================================== #
      # === Show which colours we will use
      #
      # Invocation examples:
      #
      #   ry --show-which-colours-will-be-used
      #   ry htop colours?
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_WHICH_COLOURS_WILL_BE_USED
        show_which_colours_will_be_used
        exit_program
      # ===================================================================== #
      # === Designate which programs to compile
      #
      # This entry point can be used to assign which programs are to be
      # compiled. This is most useful if you wish to compile more than
      # one program.
      #
      # Commandline invocation example:
      #
      #   rbt --compile-these-programs=make,sed,awk,coreutils,utillinux,gawk,
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?these(-|_| )?programs=(.+)/
        compile_these_programs = $3.to_s.dup
        compile_these_programs.chop! if compile_these_programs.end_with? ','
        if compile_these_programs.include? ','
          compile_these_programs = compile_these_programs.split(',')
        end
        set_compile_these_programs(compile_these_programs)
      # ===================================================================== #
      # === Use a random build directory
      #
      # This entry point allows us to use a build directory that has
      # a random name. The idea is that this random name will not
      # interfere with any existing directory.
      #
      # Invocation examples:
      #
      #   ry htop --use-random-build-directory
      #   ry conky --use-random-build-directory
      #   ry conky --use-random-build-directory --ntrad
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?random(_|-| )?build(_|-| )?directory$/
        do_use_random_build_directory
      # ===================================================================== #
      # === rbt --overview-utility-scripts
      # ===================================================================== #
      when /^-?-?overview(_|-| )?utility(_|-| )?scripts/i
        require 'rbt/toplevel_methods/show_available_utility_scripts.rb'
        show_overview_over_available_utility_scripts
        exit_program
      # ===================================================================== #
      # === Feedback the Program URL of several programs at the same time
      #
      # Note that this will apply .uniq, as I think this makes sense
      # to assume.
      #
      # Usage example:
      #
      #   ry --url-for-these-programs=php,ruby,python,m4,php
      #
      # ===================================================================== #
      when /-?-?url(-|_| )?for(-|_| )?these(-|_| )?programs=(.+)$/i # === $4
        _ = $4.dup
        _ = _.split(',') if _.include?(',')
        _.uniq! if _.is_a? Array
        show_the_URL_for_these_programs(_, :initialize_the_dataset_anew)
        exit_program
      # ===================================================================== #
      # === Allowed cookbook entries
      #
      # Inform the user which cookbook entries are allowed.
      #
      # Invocation example:
      #
      #   rbt --allowed-cookbook-entries
      #
      # ===================================================================== #
      when /^-?-?allowed(-|_| )?cookbook(-|_| )?entries$/,
           'ace',
           'allowed?'
        action(:AllowedCookbookEntries)
        exit_program
      # ===================================================================== #
      # === rbt --vca
      # ===================================================================== #
      when /^-?-?vca$/
        action(:ValidateCookbookAliases)
        exit
      # ===================================================================== #
      # === Expand the cookbooks on the local computer system
      #
      # Invocation example:
      #
      #   rbt --expand
      #
      # ===================================================================== #
      when /^-?-?expand=(.+)$/,
           /^-?-?expand-?dataset=(.+)$/
        action(:ExpandCookbooks, $1.to_s.dup)
        exit
      # ===================================================================== #
      # === rbt --show-last-updated-programs
      #
      # Show the last n updated programs, in a table-layout. This
      # presently defaults to show ALL the available programs.
      # ===================================================================== #
      when /^-?-?show(-|_| )?last(-|_| )?updated(-|_| )?programs/,
           /^-?-?last(-|_| )?updated\??/, # cookbooks --last-updated?
           'slu'
        action(:ShowLastUpdated, "-n #{n_programs_available?}")
        exit
      # ===================================================================== #
      # === Use a GoboLinux Recipe
      #
      # Invocation example:
      #
      #   ry --load_data_from_this_gobolinux_recipe=return_pwd
      #
      # ===================================================================== #
      when /^-?-?load(_|-| )?data(_|-| )?from(_|-| )?this(_|-| )?gobolinux(_|-| )?recipe=(.+)/
        load_data_from_this_gobolinux_recipe($6)
      # ===================================================================== #
      # === Generate all GoboLinux recipes
      #
      # This entry point will generate all GoboLinux recipes.
      #
      # Invocation example:
      #
      #   ry --generate-all-gobolinux-recipes
      #
      # ===================================================================== #
      when /^-?-?generate(_|-| )?all(_|-| )?gobolinux(_|-| )?recipes$/,
           /^-?-?generate(_|-| )?gobolinux(_|-| )?recipes$/,
           /^-?-?create(_|-| )?all(_|-| )?gobolinux(_|-| )?recipes$/,
           /^-?-?create(_|-| )?gobolinux(_|-| )?recipes$/
        generate_all_gobolinux_recipes
        exit_program
      # ===================================================================== #
      # === Report the registered cookbook entries/programs
      #
      # Show how many programs are registered.
      #
      # Usage examples:
      #
      #   ry --nprograms
      #   ry --reportregisteredprograms
      #
      # ===================================================================== #
      when *ARRAY_HELP_REPORT_N_PROGRAMS_OPTIONS
        report_n_programs
        exit_program
      # ===================================================================== #
      # === Check whether a specific program is registered
      #
      # This is similar to the other inclusion check, but uses a slightly
      # different syntax.
      #
      # Invocation examples:
      #
      #   rbt --registered?=awk
      #   rbt --registered?=gawk
      #
      # ===================================================================== #
      when /^-?-?registered\??=(.+)$/i
        i = $1.to_s.dup
        opnn
        case RBT.is_this_program_included?(i, :be_quiet)
        when true
          e "#{rev}Yes, the program #{sfancy(i)}#{rev} is available "\
            "and registered in the RBT project."
        when false
          e "#{rev}No, the program #{sfancy(i)}#{rev} is NOT registered in the RBT project."
        end 
        exit_program
      # ===================================================================== #
      # === Remove incorrect yaml/ entries under the /home/Programs hierarchy
      #
      # This is mosly a convenience entry point. Faulty yaml/ directories
      # under the /home/Programs/ directory will be removed.
      #
      # Do note that another target directory can be given to the class
      # doing the action.
      #
      # Invocation examples:
      #
      #   rbt --purge-incorrect-yaml-directories-in-the-programs-hierarchy
      #   rbt --purge-flawed-yaml-directories-in-the-programs-hierarchy
      #   rbt --purge-flawed-yaml
      #
      # ===================================================================== #
      when /^-?-?purge(-|_| )?incorrect(-|_| )?yaml(-|_| )?directories(-|_| )?in(-|_| )?the(-|_| )?programs(-|_| )?hierarchy$/i,
           /^-?-?purge(-|_| )?flawed(-|_| )?yaml(-|_| )?directories(-|_| )?in(-|_| )?the(-|_| )?programs(-|_| )?hierarchy$/i,
           /^-?-?purge(-|_| )?flawed(-|_| )?yaml$/i,
           /^-?-?no(-|_| )?yaml$/i
        action(:PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy)
        exit_program
      # ===================================================================== #
      # === Check for gnome updates
      # ===================================================================== #
      when /^-?-?gnome$/,
           /^-?-?check(_|-| )?for(_|-| )?gnome(_|-| )?updates$/
        action(:CheckForGnomeUpdates)
        exit_program
      # ===================================================================== #
      # === Feedback compile strategies used
      #
      # Compile-strategies refers to essential build-related strategies
      # such as GNU configure, cmake and meson, and how many projects
      # make use of these respectively.
      #
      # Invocation examples:
      #
      #   ry --compile_strategies?
      #   ry --build-statistics
      #   ry --build-stats
      #   ry --statistics2
      #
      # ===================================================================== #
      when /^-?-?compile(_|-| )?strategies\??$/i,
           /^-?-?build(_|-| )?statistics\??$/i,
           /^-?-?build(_|-| )?stats?\??$/i,
           /^-?-?mega(_|-| )?stats?\??$/i,
           /^-?-?statistics\??$/i,
           /^-?-?overview\??$/i,
           /^-?-?feedback(_|-| )?statistics?\??$/i,
           /^-?-?statistics2$/i
        do_extensively_report_the_build_statistics
      # ===================================================================== #
      # === Compile all of XFCE
      #
      # This entry point allows us to compile all XFCE components in
      # one go.
      #
      # Invocation examples:
      #
      #   rbt --compile-xfce
      #   rbt --superxfce
      #   installer --superxfce
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xfce$/i,
           /^-?-?super(-|_| )?xfce$/i,
           /^-?-?all(-|_| )?of(-|_| )?xfce$/i
        compile_all_of_xfce
      # ===================================================================== #
      # === Compile all of XFCE into a standalone directory
      #
      # This entry point allows us to compile all of XFCE into a
      # standalone directory, such as /Programs/Xfce/4.12.0.
      #
      # I considered this sufficiently useful to add this functionality.
      # That way we could easily and quickly relocate XFCE between
      # different computers.
      #
      # Invocation examples:
      #
      #   rbt --compile-xfce-components-into-one-standalone-directory
      #   rbt --compile-xfce-components-into-a-standalone-directory
      #   rbt --compile-xfce-components-into-standalone-directory
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?one(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?a-standalone(-|_| )?directory$/i,
           /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?components(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?into(-|_| )?standalone(-|_| )?dir$/i,
           /^-?-?xfce(-|_| )?standalone$/i
        compile_xfce_components_into_one_standalone_directory
        exit_program
      # ===================================================================== #
      # === Show the full XFCE compilechain
      #
      # This entry point can be used to show the full compilechain for
      # XFCE.
      #
      # Invocation examples:
      #
      #   rbt --show-xfce-compilechain
      #   rbt --xfcecomponents?
      #
      # ===================================================================== #
      when /^-?-?show(-|_)?xfce(-|_)?compilechain$/i,
           /^-?-?xfce(-|_)?components\??$/i,
           /^-?-?xfce\??$/i
        show_compile_chain(:xfce)
        exit_program
      # ===================================================================== #
      # === Enable --without-headers
      #
      # Invocation example:
      #
      #   rbt gcc --trad --newlib --without-headers
      #
      # ===================================================================== #
      when /^-?-?no(-|_| )?headers$/i
        append_these_configure_options('--without-headers')
      # ===================================================================== #
      # === Ensure that all directories for RBT exist
      #
      # Invocation example:
      #
      #   rbt --ensure-all-directories-exist
      #
      # ===================================================================== #
      when /^-?-?ensure(-|_| )?all(-|_| )?directories(-|_| )?exist$/i,
           /^-?-?ensure(-|_| )?that(-|_| )?all(-|_| )?directories(-|_| )?exist$/i
        orev "Next trying to ensure that all relevant directories exist,"
        orev "as far as the RBT project is concerned."
        if File.directory? log_dir?
          orev 'The directory at '+sdir(log_dir?)+rev+' already exists.'
        else
          ensure_that_all_directories_for_rbt_exist
        end
        exit_program
      # ===================================================================== #
      # === Use default chroot prefix + /usr/, at /Depot/Chroot/usr/
      #
      # This entry point is a modification of --chroot.
      #
      # Invocation examples:
      #
      #   rbt bison --uchroot
      #   rbt flex --uchroot
      #   rbt nasm --uchroot
      #   rbt sed --uchroot
      #
      # ===================================================================== #
      when /^-?-?uchroot$/i
        set_user_prefix('/Depot/Chroot/usr/')
      # ===================================================================== #
      # === Compile all of mate
      #
      # This entry point allows us to compile all components of the
      # mate-desktop.
      #
      # Invocation examples:
      #
      #   rbt --compile-all-of-mate
      #   rbt --compile-mate-desktop
      #   rbt --all-of-mate
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?mate$/i,
           /^-?-?compile(-|_| )?mate(-|_| )?desktop$/i,
           /^-?-?compile(-|_| )?mate$/i,
           /^-?-?all(-|_| )?of(-|_| )?mate$/i
        do_compile_all_of_mate
      # ===================================================================== #
      # === Compile all of Mate-Desktop into a standalone directory
      #
      # This entry point allows us to compile all of MATE into a
      # standalone directory, such as /Programs/Mate/1.20.
      #
      # Invocation examples:
      #
      #   rbt --compile-mate-desktop-components-into-one-standalone-directory
      #   rbt --compile-mate-desktop-components-into-a-standalone-directory
      #   rbt --compile-mate-desktop-components-into-standalone-directory
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?one(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?a-standalone(-|_| )?directory$/,
           /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?components(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?into(-|_| )?standalone(-|_| )?dir$/,
           /^-?-?mate(-|_| )?prefix$/,
           /^-?-?mate(-|_| )?standalone$/
        compile_mate_desktop_components_into_one_standalone_directory
        exit_program
      # ===================================================================== #
      # === Show the configuration options for a given program
      #
      # Invocation example:
      #
      #   rbt gcc --show-configuration-options
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?configuration(-|_| )?options$/i
        action(:ShowConfigurationOptions, compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Show the last updated programs
      #
      # Show when some programs were last updated.
      #
      # Invocation example:
      #
      #   rbt --slu
      #
      # ===================================================================== #
      when *ARRAY_SHOW_LAST_UPDATED_OPTIONS
        show_the_n_last_updated_programs(875)
        exit_program
      # ===================================================================== #
      # === Feedback how many cookbook aliases exist
      #
      # This entry point allows the user to quickly determine how many
      # cookbook-related aliases exist.
      #
      # Invocation example:
      #
      #   ry --n-aliases?
      #
      # ===================================================================== #
      when /^-?-?n(-|_| )?aliases\??$/
        _ = RBT.file_cookbook_aliases?
        if File.exist? _
          report_how_many_registered_cookbook_aliases_are_available
        else
          no_file_exists_at(_)
        end
        exit_program
      # ===================================================================== #
      # === Validate the cookbook entries
      #
      # Invocation example:
      #
      #   rbt --validate-entries
      #
      # ===================================================================== #
      when /^-?-?validate(_|-| )?entries$/
        action(:validate_cookbook_aliases)
        exit_program
      # ===================================================================== #
      # === Query to which program a particular binary belongs to
      #
      # This entry point allows us to find out to which particular
      # program a specific binary belongs to.
      #
      # Invocation examples:
      #
      #   rbt --this-binary?=shar
      #   rbt --binary=env
      #
      # ===================================================================== #
      when /^-?-?this(-|_| )?binary\?=(.+)$/i, # For $2
           /^-?-?binary=(.+)$/i                # For $1
        _ = $1.to_s.dup # Default.
        _ = $2.to_s.dup if $2
        action(:BinaryNameBelongsToWhichProgram, _)
        exit_program
      # ===================================================================== #
      # === Show all about a specific program
      #
      # This entry point will show all about a particular program.
      #
      # Invocation example:
      #
      #   rbt --show-all-about=htop
      #
      # ===================================================================== #
      when /^-?-?show(_|-| )?all(_|-| )?about=(.+)/i,
           /^-?-?show(_|-| )?all=(.+)/i,
           /^-?-?show(_|-| )?file=(.+)/i
        show_this_program = $2.to_s.dup # This is valid for two entry points ^^^.
        show_this_program = $3.to_s.dup if $3
        action(:ShowAllAbout, show_this_program)
      # ===================================================================== #
      # === ry --generate-homepage
      # ===================================================================== #
      when /^-?-?generate(-|_| )?homepage$/,
           /^-?-?gen(-|_| )?homepage$/,
           /^-?-?homepage$/
        action(:GenerateHomepage)
        exit
      # =================================================================== #
      # === ry htop --ntrad
      # =================================================================== #
      when /^-?-?ntrad/,
           /^-?-?appdir$/i # === ry htop --appdir
        do_change_the_prefix_towards_an_appdir_prefix
      # ===================================================================== #
      # === Use another directory as the new /Programs/ dir
      # ===================================================================== #
      when /^-?-?use-?this-?programs-?dir=(.+)/
        use_this_as_programs_dir($1.to_s.dup) # See: http://rubular.com/r/HR72sazzMi
      # ===================================================================== #
      # === Convert to non-traditional
      #
      # Convert a FHS-conforming program back into a non-traditional
      # hierarchy:
      #
      #   ry htop to_programs
      #
      # ===================================================================== #
      when 'to_programs','toprograms',
           'convert_to_non_traditional',
           'tobin',
           'to_fhs',
           'toubin'
        convert_to_non_traditional
        exit_program
      # ===================================================================== #
      # === Make use of CreateProgram with the current directory as input
      #
      # This entry point allows us to make use of the current working
      # directory as the input-argument.
      #
      # Invocation examples:
      #
      #   rbt --create-program
      #   rbt --rcp
      #
      # ===================================================================== #
      when /^-?-?create(-|_)?program$/i,
           /^-?-?rcp$/i
        _ = File.basename(return_pwd)
        RBT.create_program(_)
        exit_program
      # ===================================================================== #
      # === Query the path to the windows cookbook file
      #
      # Invocation example:
      #
      #   rbt --windows-cookbook?
      #
      # ===================================================================== #
      when /^-?-?windows(-|_| )?cookbook\??$/i
        e 'The windows_cookbook.yml file resides at:'
        e
        e "  #{sfile(RBT::FILE_WINDOWS_COOKBOOK)}"
        e
      # ===================================================================== #
      # === Show the project's base directory here
      #
      # Invocation example:
      #
      #   ry --project-base-directory?
      #
      # ===================================================================== #
      when /^-?-?project(-|_| )?base(-|_| )?directory\??$/
        e "#{steelblue(project_base_directory?)}#{rev}"
        exit_program
      # ===================================================================== #
      # === Do not use any special environment flag at all
      #
      # Sometimes you really do not want to use ENV, that is, the
      # environment. The entry point here allows you to not only
      # empty ENV, but to also not set any new ENV flags.
      #
      # Invocation examples:
      #
      #   ry htop --do-not-use-special-flags
      #   ry nspr --clear-envs ntrad
      #   ry nss  --clear-envs ntrad
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?special(_|-| )?flags$/,
           /^-?-?clear(_|-| )?envs$/
        verbose_do_not_use_special_flags
      # ===================================================================== #
      # === Validate Cookbook Aliases
      #
      # This entry point can be used to validate the cookbook aliases.
      #
      # Invocation examples:
      #
      #   rbt vca
      #   rbt --vca
      #
      # ===================================================================== #
      when /^validate_?cookbook_?aliases$/,
           /^-?-?vca/,
           /^-?-?cookbook(-|_| )?aliases$/,
           'calias'
        validate_the_cookbook_aliases
        exit_program
      # ===================================================================== #
      # === Report the XFCE version at hand
      #
      # This entry point allows the user to report the current XFCE
      # version in use.
      #
      # Invocation example:
      #
      #   ry --report-xfce-version
      #   ry --xfce-version
      #
      # ===================================================================== #
      when /^-?-?report(-|_)?xfce(-|_)?version$/i,
           /^-?-?xfce(-|_)?version\??$/i
        action(:report_xfce_version)
        exit_program
      # ===================================================================== #
      # === ry --display_barplot_statistics
      # ===================================================================== #
      when /-?-?display(-|_| )?barplot(-|_| )?statistics$/i
        display_barplot_statistics
        exit
      # ===================================================================== #
      # === rbt --return_compile_time_statistics
      # ===================================================================== #
      when /^-?-?return(-|_| )?compile(-|_| )?time(-|_| )?statistics$/
        e lightblue('Compile time statistics:')
        e steelblue(return_compile_time_statistics.body?)
      # ===================================================================== #
      # === Do not make use of cookbook aliases
      #
      # This entry point here exists to prevent the use of cookbook
      # aliases. Normally, the aliases are useful, since they allow
      # us to use abbreviations, and easier-to-type names for commonly
      # used programs. However had, sometimes we distinctly do NOT want
      # to use a specific alias, such as when we wish to compile "sdlttf"
      # and the cookbook alias automatically picks "sdlttff" for us
      # instead.
      #
      # Invocation examples:
      #
      #   ry sdlttf --do-not-use-cookbook-aliases
      #   ry sdlttf --do-not-use-cookbook-alias
      #
      # ===================================================================== #
      when /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?cookbook(_|-| )?aliases$/i,
           /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?cookbook(_|-| )?alias$/i,
           /^-?-?do(_|-| )?not(_|-| )?make(_|-| )?use(_|-| )?of(_|-| )?cookbook(_|-| )?aliases$/i,
           /^-?-?no(_|-| )?alias$/
        do_not_use_cookbook_aliases(:be_verbose)
      # ===================================================================== #
      # === Feedback the Program URL
      #
      # Feedback the url of a given program
      #
      # Usage examples:
      #
      #   ry libxrandr url
      #   ry ruby url?
      #   ry python --url?
      #   ry libsigc++ --show-url
      #   ry ruby --show-url
      #
      # ===================================================================== #
      when /^-?-?url\??$/i,
           'ulr',
           'ur',
           'u',
           /urla$/i,
           /feedback(-|_| )?url/,
           /^-?-?show(-|_| )?url$/i
        feedback_url
        exit_program
      # ===================================================================== #
      # === Feedback how many binaries are registered
      #
      # This entry point tells us how many binaries are registered in
      # total. Binaries in this context refers to programs that reside
      # under the corresponding bin/ hierarchy.
      #
      # However had, if a program has been assigned to the main dataset
      # already then the behaviour of the method will be different -
      # in this case the method will report how many binaries are
      # part of that program at hand.
      #
      # Invocation examples:
      #
      #   rbt --n-binaries?
      #   rbt glib --n-binaries?
      #
      # ===================================================================== #
      when /^-?-?n(_|-| )?binaries\??$/
        either_report_the_total_amount_of_binaries_or_how_many_binaries_this_program_will_install
        exit_program
      # ===================================================================== #
      # === Update binaries and libraries of a given program automatically
      #
      # Invocation example:
      #
      #   rbt --autoupdate=htop
      #   rbt --autoupdate=geany
      #
      # ===================================================================== #
      when /^-?-?autoupdate=(.+)$/i
        this_program = $1.to_s.dup # Default.
        update_binaries_of_this_program(this_program)
        update_libraries_of_this_program(this_program)
        exit_program
      # ===================================================================== #
      # === Output All URLs
      #
      # This entry point will output all URLs.
      #
      # Invocation examples:
      #
      #   rbt --all_urls
      #   rbt all_urls
      #   rbt all_urls?
      #   Installer --all-urls
      #  
      # ===================================================================== #
      when *ARRAY_HELP_OUTPUT_ALL_URLS_OPTIONS
        output_all_urls_then_exit
      # ===================================================================== #
      # === Report the RBT version in use
      #
      # This part will tell us which specific version of RBT we
      # are currently using.
      #
      # Invocation examples:
      #
      #   ry --version
      #   ry VERSION
      #   ry --version
      #
      # ===================================================================== #
      when *ARRAY_HELP_VERSION_OPTIONS,
           /^VERSION/
        first = first?
        if first and !first.start_with?('--')
          initialize_for(first.to_sym)
          e steelblue(first+': ')+
            royalblue(cookbook_program_version?)
        else
          report_rbt_version #:then_exit
        end
        exit_program
      # ===================================================================== #
      # === Show the chained programs
      #
      # This entry point exists mostly to quickly test this functionality.
      #
      # Invocation example:
      #
      #   ry --return-chained-programs
      #
      # ===================================================================== #
      when /^-?-?return(_|-| )?chained(_|-| )?programs\??$/i
        return_chained_programs
        exit_program
      # ===================================================================== #
      # === Show which programs can be compiled statically
      #
      # This entry point allows the user to show which programs can be
      # compiled statically, in principle.
      #
      # Invocation examples:
      #
      #   ry --which-programs-can-be-compiled-statically?
      #   ry --programs-that-can-be-compiled-statically?
      #
      # ===================================================================== #
      when /^-?-?which(_|-| )?programs(_|-| )?can(_|-| )?be(_|-| )?compiled(_|-| )?statically\??$/i,
           /^-?-?statically(_|-| )?compilable(_|-| )?programs\??$/i,
           /^-?-?programs(_|-| )?that(_|-| )?can(_|-| )?be(_|-| )?compiled(_|-| )?statically\??$/i,
           /^-?-?statically\??$/i,
           /^-?-?all(_|-| )?static\??$/i
        orev 'Trying to determine which programs can be compiled statically.'
        orev 'Please be patient ...'
        report_which_programs_can_be_compiled_statically
        exit_program
      # ===================================================================== #
      # === Static compilation
      #
      # If you wish to compile something statically, use this entry point
      # here.
      #
      # Invocation examples:
      #
      #   ry m4 static
      #   ry make static
      #   ry make static ntrad
      #
      # ===================================================================== #
      when *ARRAY_HELP_ENABLE_STATIC
        verbose_enable_static
      # ===================================================================== #
      # === Permanently add a global flag
      #
      # Invocation example:
      #
      #   rbt --add-global-flag=--disable-static
      #
      # ===================================================================== #
      when /^-?-?add(-|_)?global(-|_)?flag=(-?-?.+)/
        permanently_add_this_global_flag($3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Disable static compilation
      #
      # This entry point exists so that users can disable static
      # compilation, e. g. when the corresponding .yml file has
      # a "build_static: t" entry by default.
      #
      # Invocation example:
      #
      #   rbt wget --disable-static-compilation
      #
      # ===================================================================== #
      when /^-?-?disable(-|_| )?static(-|_| )?compilation$/i,
           /^-?-?dont(-|_| )?compile(-|_| )?statically$/i,
           /^-?-?not(-|_| )?static$/i,
           /^-?-?no(-|_| )?static$/i
        disable_static_compilation(:be_verbose)
      # ===================================================================== #
      # === Show how many statically compiled binaries exist on this
      #     computer system
      #
      # Invocation example:
      #
      #   ry --static-overview?
      #
      # ===================================================================== #
      when /^-?-?static(_|-| )?overview\??$/i
        action(:StaticOverviewOfTheAvailableBinariesOnThisComputerSystem)
        exit_program
      # ===================================================================== #
      # === Show all available tags that are registered in the RBT suite
      #
      # Invocation example:
      #
      #   rbt --available-tags
      #
      # ===================================================================== #
      when /^-?-?available(-|_| )?tags$/i
        opnn; action(:show_available_tags)
        exit_program
      # ===================================================================== #
      # === Remove .la files
      #
      # This entry point allows you to get rid of the .la files associated
      # with the given program. Currently it will only be done for programs
      # that are installed in an AppDir-like manner, but this behaviour
      # may change at a later time.
      #
      # Invocation example:
      #
      #   ry pcre2 ntrad --remove-la-files
      #
      # ===================================================================== #
      when /^-?-?remove(_|-| )?la(_|-| )?files$/
        if is_installed_in_an_appdir_like_fashion?
          orev "Any .la files found to be part of the program "\
               "#{sfancy(program_name?)} will be deleted."
          do_remove_la_files_after_compilation
        end
      # ===================================================================== #
      # === Prepare gmp, mpc and mpfr for GCC compilation
      #
      # This will simply copy the three tarballs, extract them and
      # re-name the extracted directories, to make fit for compilation
      # of GCC.
      #
      # Invocation examples:
      #
      #   ry --prepare-for-gcc
      #   ry --prepare-gcc
      #   ry --copy-gcc-base
      #
      # ===================================================================== #
      when /^-?-?prepare(-|_| )?for(-|_| )?gcc$/i,
           /^-?-?prepare(-|_| )?gcc$/i,
           /^-?-?copy(-|_| )?gcc(-|_| )?base$/i
        prepare_for_gcc
        exit_program
      # ===================================================================== #
      # === Feedback whether we are currently compiling anything
      #
      # Invocation example:
      #
      #   rbt --currently_compiling?
      #
      # ===================================================================== #
      when /-?-?currently(_|-| )?compiling\??/,
           /-?-?what(_|-| )?are(_|-| )?we(_|-| )?currently(_|-| )?compiling\??/,
           /-?-?currently(_|-| )?compiled\??/
        feedback_what_we_are_currently_compiling?
        exit_program
      # ===================================================================== #
      # === Use a specific, local version
      #
      # Note that this variant does not yet work.
      #
      # Invocation example:
      #
      #   ry --use-this-program=gtkmm-3.24.7
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?this(-|_| )?program=(.+)$/ # === $3
        _ = $3.to_s.dup
        program_name    = action(:program_name, _)
        program_version = action(:program_version, _)
        orev "Using this program name: #{program_name} and "\
             "this program version: #{program_version}"
        if cookbook_dataset.nil?
          initialize_cookbook_for(program_name)
        end
        # set_use_this_version(program_version)
        set_program_name_and_program_version(_)
        set_compile_this_program(program_name)
      # ===================================================================== #
      # === Show downloadable programs
      #
      # This entry point will display all programs that are "downloadable",
      # in a condensed manner. This simply refers to the registered
      # programs really.
      #
      # Invocation examples:
      #
      #   rbt --show-downloadable-programs
      #   installer --show-downloadable-programs
      #
      # ===================================================================== #
      when /^-?-?show(-|_| )?downloadable(-|_| )?programs$/i,
           /^-?-?downloadable(-|_| )?programs\??$/i
        show_downloadable_programs
        exit_program
      # ===================================================================== #
      # === Download via svn
      #
      # If you wish to download via svn, then you can use this entry point
      # here.
      #
      # Invocation examples:
      #
      #   ry dosbox --svn
      #   ry dosbox svn
      #
      # ===================================================================== #
      when /^-?-?svn$/,
           'use_svn',
           'svn_download',
           'svndownload',
           'svndown',
           'download_via_svn'
        if first?.to_s == 'svn'
          set_compile_this_program(first?.to_s)
        else
          download_using_svn
          exit_program
        end
      # ===================================================================== #
      # === Show the Recipes directory
      #
      # Invocation example:
      #
      #   ry --recipes_dir?
      #
      # ===================================================================== #
      when /^-?-?recipes(_|-| )?dir\??/i
        show_the_recipes_directory
        exit_program
      # ===================================================================== #
      # === Reset the status.yml file next
      #
      # Invocation example:
      #
      #   ry --reset-the-status-file
      #
      # ===================================================================== #
      when /^-?-?reset(_|-| )?the(_|-| )?status(_|-| )?file$/
        orev 'Reseting the file '+sfile(RBT.file_status)+"#{rev} next."
        reset_the_status_file
        exit_program
      # ===================================================================== #
      # === Be completely quiet
      #
      # This directive will tell us to be completely silent.
      #
      # Invocation example:
      #
      #   ry htop --be_so_quiet
      #
      # ===================================================================== #
      when /^-?-?be(_|-| )?completely(_|-| )?quiet$/,
           /^-?-?be(_|-| )?very(_|-| )?quiet$/,
           /^-?-?be(_|-| )?so(_|-| )?quiet$/
        be_completely_quiet
      # ===================================================================== #
      # === Store all abbreviations in a file
      #
      # Invocation example:
      #
      #   rbt --store-abbreviations
      #
      # ===================================================================== #
      when /^-?-?store(-|_| )?abbreviations$/,
           /^-?-?store$/i,
           /^-?-?generate(-|_| )?abbreviations$/i,
           /^-?-?generate(-|_| )?aliases$/i,
           /^-?-?abbreviations(-|_| )?to(-|_| )?the(-|_| )?programs$/i,
           /^-?-?create(-|_| )?abbreviations$/i,
           /^-?-?abbrevs$/i,
           /^-?-?abb(-|_| )?reviations$/i,
           /^-?-?store(-|_| )?abreviations$/i
        action(:store_abbreviations)
        exit_program
      # ===================================================================== #
      # === Do not use any configure options
      #
      # This entry point allows you to completely ignore all configure
      # options specified in the particular .yml file.
      #
      # In September 2020 --pristine was added as a shortcut. Do note
      # that --pristine may mean more than merely "do not use the
      # configure options", so in the future, this may change. But
      # for now, --pristine is simply a shorter alias. 
      #
      # Invocation examples:
      #
      #   rbt htop --ntad --dont-use-configure-options
      #   rbt glib --user-home-prefix --use-meson --no-configure-options
      #   rbt ruby --do-not-use-any-configure-options
      #
      # ===================================================================== #
      when /^-?-?dont(_|-| )?use(_|-| )?configure(_|-| )?options?$/i,
           /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?configure(_|-| )?options$/i,
           /^-?-?dont(_|-| )?use(_|-| )?configure(_|-| )?options$/i,
           /^-?-?no(_|-| )?configure(_|-| )?options$/i,
           /^-?-?no(_|-| )?config$/i,
           /^-?-?do(_|-| )?not(_|-| )?use(_|-| )?any(_|-| )?configure(_|-| )?options$/i,
           /^-?-?dont(_|-| )?use(_|-| )?any(_|-| )?configure(_|-| )?options$/i,
	        /^-?-?clear(_|-| )?configure(_|-| )?options$/i,
	        /^-?-?ignore(_|-| )?configure(_|-| )?options$/i,
	        /^-?-?barebones$/i,
	        /^-?-?pristine$/i,
	        /^-?-?virgin$/i, # (virgin tag)
	        /^-?-?dont(_|-| )?use(_|-| )?configure/i
        do_not_use_any_configure_option { :be_verbose }
        # =================================================================== #
        # The next line was added on 24.11.2019. Don't use meson
        # configure options either.
        # =================================================================== #
        @internal_hash[:use_meson_configure_options] = false
      # ===================================================================== #
      # === Use /opt/ as prefix
      #
      # This entry point allows us to use /opt/ as the prefix. It is mostly
      # a convenience entry point.
      #
      # Invocation examples:
      #
      #   ry llvm --use-opt-as-prefix
      #   ry zlib --opt
      #
      # ===================================================================== #
      when /^-?-?use(_|-| )?opt(_|-| )?as(_|-| )?prefix$/,
           /^-?-?opt(_|-| )?prefix$/,
           /^-?-?opt$/
        use_opt_as_prefix
      # ===================================================================== #
      # === Last compiled program
      #
      # Invocation example:
      #
      #   ry LAST
      #
      # ===================================================================== #
      when *ARRAY_HELP_LAST_COMPILED_PROGRAM_OPTIONS
        set_compile_these_programs(last_compiled_program?)
      # ===================================================================== #
      # === Show help options
      #
      # Show the help options here, then exit.
      #
      #   ry --help
      #
      # ===================================================================== #
      when *ARRAY_HELP_OPTIONS
        display_standard_help_options
        exit_program
      # ===================================================================== #
      # === Show the temp directory
      #
      # This entry point allows us to show the temp-directory on the
      # commandline.
      #
      # The resulting output is equivalent to:
      #
      #   RBT.config_directory?+'temp_dir.yml'
      #
      # Invocation example:
      #
      #   ry --temp_dir?
      #
      # ===================================================================== #
      when *ARRAY_HELP_TEMP_DIR
        show_temp_directory
        exit_program
      # ===================================================================== #
      # === Feedback configure options
      #
      # This part will show the configure options for a given program.
      #
      # Feedback the configure option for a specific program at hand:
      #
      #   ry htop configure?
      #   ry gcc  configure?
      #
      # ===================================================================== #
      when *ARRAY_FEEDBACK_CONFIGURE_OPTIONS,
           /configure_?option\?$/i,
           'showconfig?'
        feedback_configure_options
        exit_program
      # ===================================================================== #
      # === Use /opt/toolchain/ as prefix
      #
      # This entry point allows the user to use the /opt/toolchain/
      # as prefix.
      #
      # Invocation examples:
      #
      #   rbt binutils --toolchain
      #
      # ===================================================================== #
      when /^-?-?toolchain$/i
        prefix_to_be_used = '/opt/toolchain/'
        orev "Using the prefix #{steelblue(prefix_to_be_used)}#{rev}"
        set_user_prefix(prefix_to_be_used)
        do_not_use_any_configure_options
      # ===================================================================== #
      # === Feedback local location
      #
      # This entry point will show the local location of the target
      # program (archive) at hand.
      #
      # Usage examples:
      #
      #   ry gcc    --local_url
      #   ry aalib  local_url
      #   ry libx11 lurl
      #
      # ===================================================================== #
      when /^-?-?local(-|_| )?url$/i,
           'local','lurl','localurl?','lurl?',
           'location?','location','fulllocation?','fulllocation',
           'where?',
           'feedback_local_location'
        feedback_local_location
        exit_program
      # ===================================================================== #
      # === Suggest the content to a .yml file
      #
      # This entry point allows us to show the content of an assumed
      # .yml file via the commandline. That way we should be more
      # rapidly able to create new .yml files or update existing
      # ones with new content. 
      #
      # Invocation example:
      #
      #   rbt --suggest-cookbook-for=glib
      #
      # ===================================================================== #
      when /^-?-?suggest(_|-| )?cookbook(_|-| )?for=(.+)$/i
        action(:suggest_cookbook_for, $3.to_s.dup)
      # ===================================================================== #
      # === Remove all empty directories from the current working directory
      #
      # Invocation example:
      #
      #   ry --remove_empty_directories
      #
      # ===================================================================== #
      when /^-?-?remove(_|-| )?empty(_|-| )?directories$/
        remove_empty_directories
      # ===================================================================== #
      # === Query whether this program can be compiled statically
      #
      # This entry point allows us to query whether the given program at
      # hand can be compiled statically or whether it can not.
      #
      # Invocation example:
      #
      #   ryt tar --can-be-compiled-statically?
      #
      # ===================================================================== #
      when /^-?-?can(-|_| )?be(-|_| )?compiled(-|_| )?statically\??$/i
        verbose_query_whether_this_program_can_be_compiled_statically
        exit_program
      # ===================================================================== #
      # === Show the installation_procedure.yml file
      #
      # Invocation example:
      #
      #   ry --installation-procedure
      #
      # ===================================================================== #
      when /-?-?installation(-|_| )?procedure$/
        show_installation_procedure_file
        exit_program
      # ======================================================================= #
      # === Do use autoconf
      #
      # This entry point allows the user to use/invoke autoconf.
      #
      # Usage examples:
      #
      #   rbt htop --run-autoconf
      #   rbt tiff --ntrad --use-autoconf
      #   installer bridgeutils --ntrad --use-autoconf
      #
      # ======================================================================= #
      when /^-?-?run(-|_| )?autoconf$/,
           /^-?-?do(-|_| )?use(-|_| )?autoconf$/,
           /^-?-?use(-|_| )?autoconf$/,
           'autoconf+'
        do_use_autoconf
      # ===================================================================== #
      # === Show every available programs
      #
      # Feedback all available programs:
      #
      #   ry all_programs
      #   ry --all_available_programs?
      #
      # ===================================================================== #
      when *ARRAY_HELP_ALL_AVAILABLE_PROGRAMS
        all_available_programs?
        exit_program
      # ===================================================================== #
      # === Generate a gobolinux recipe
      #
      # Use this entry point to generate a GoboLinux recipe.
      #
      # You can find all the GoboLinux recipes here:
      #
      #   https://recipes.gobolinux.org/r/
      #
      # Invocation examples:
      #
      #   ry zenity  --gobolinux-recipe
      #   ry LibDRM  --gobolinux-recipe
      #   ry zziplib --gobolinux-recipe
      #   ry ruby    --gobolinux-recipe
      #   ry php     --gobolinux-recipe
      #   ry khtml   --generate-gobolinux-description-file
      #
      # ===================================================================== #
      when /^-?-?gobolinux(_|-| )?recipe$/,
           /^-?-?create(_|-| )?gobo(_|-| )?recipe$/, # See: http://rubular.com/r/8vWvfd2izB
           /^-?-?grecipe$/i,
           /^-?-?gobolinux(_|-| )?package$/i,
           /^-?-?generate(_|-| )?gobolinux(_|-| )?description(_|-| )?file$/i,
           /^-?-?generate(_|-| )?gobolinux(_|-| )?description$/i,
           /^-?-?to(_|-| )?gobo$/i,
           /^-?-?tog$/i,
           /^-?-?gobo(_|-| )?recipe$/i,
           /^-?-?gobo(_|-| )?scripts$/i,
           /^-?-?to(_|-| )?gobo?$/i, # ry ruby --to_gobo
           /^-?-?generate(_|-| )?gobolinux(_|-| )?recipe$/i,
           /^-?-?ggr$/i
        if program_name?.nil? and cookbook_dataset?.nil?
          set_compile_this_program(first_argument?)
          initialize_cookbook_dataset_for(program_name?)
        end
        generate_gobolinux_recipe(program_name?)
        exit_program
      # ===================================================================== #
      # === Create a program_version_url file for a specific program
      #
      # Invocation examples:
      #
      #   rbt --create-file-for=manpages
      #   rbt --create-file-for=ruby
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?file(-|_| )?for=(.+)$/i
        action(:CreateProgramVersionUrlFile, $3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Use build directory
      #
      # Instruct RBT to use a build directory.
      #
      # Usage examples:
      #
      #   ry htop build
      #   ry automake usebuilddir
      #   ry automake use_build_dir ntrad
      #   ry modules  use_build_dir
      #   ry yakuake --use-build-dir
      #   ry yakuake --trad --use-build-dir
      #
      # ===================================================================== #
      when *ARRAY_HELP_USE_BUILD_DIRECTORY_OPTIONS
        do_use_a_build_directory
      # ===================================================================== #
      # === Feedback various versions
      #
      # This entry point will tell us which version is in use for the
      # RBT project.
      #
      # Invocation examples:
      #
      #   ry feedback_version
      #   ry --feedback-version
      #   ry --versions-in-use?
      #
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_VERSIONS_OPTIONS
        feedback_various_versions :do_not_use_opn
        exit_program
      # ===================================================================== #
      # === Show libraries that are not available on this system
      #
      # This entry point allows us to display all library files that
      # may be missing on this host.
      #
      # Invocation example:
      #
      #   ry --not-found
      #
      # ===================================================================== #
      when /^-?-?not(-|_| )?found$/i
        report_which_libraries_are_missing
        exit_program
      # ===================================================================== #
      # === Extract the source archive here
      #
      # Extract a source archive into the current working directory.
      #
      # Invocation example:
      #
      #   ry gimp --extract-here
      #
      # ===================================================================== #
      when /^-?-?extract(-|_| )?here$/i
        unless cookbook_dataset_has_been_initialized?
          initialize_cookbook_dataset_for(first_argument?)
        end
        _ = program_path?
        if File.exist? _
          orev 'Extracting from '+sfile(_)+"#{rev} next."
          extract_what_to(_, return_pwd)
        else
          opnn; no_file_exists_at(_)
        end
        exit_program
      # ===================================================================== #
      # === Ad-hoc silence "LC_ALL: cannot change locale" warnings
      #
      # Invocation example:
      #
      #   ry libunistring ntrad --lcall
      #
      # ===================================================================== #
      when /^-?-?LC_?ALL$/i
        ENV['LC_ALL'] = 'C'
      # ===================================================================== #
      # === Keep the archive extracted
      #
      # The user can use this entry point if he/she dodoes not want
      # to delete the extracted archive again.
      #
      # Keep our source archive extracted:
      #
      #   ry htop ke
      #
      # ===================================================================== #
      when *ARRAY_HELP_KEEP_EXTRACTED_OPTIONS
        do_set_keep_extracted
      # ===================================================================== #
      # === Check for the latest program
      #
      # Check for the latest program:
      #
      #   SM --latest?
      #   SM --latest
      #   SM --check-for-latest
      #
      # ===================================================================== #
      when /^-?-?latest\??$/,
           'not_latest?',
           'not_latest','sanitize',
           'clean',
           /^-?-?check(_|-| )?for(_|-| )?latest$/
        check_for_latest
        exit_program
      # ===================================================================== #
      # === Feedback the main configuration
      #
      # This entry point can be used to show the main configuration of
      # the RBT project.
      #
      # Invocation example:
      #
      #   SM --main-config?
      #
      # ===================================================================== #
      when *ARRAY_HELP_CONFIG_OPTIONS
        feedback_main_config
        exit_program
      # ===================================================================== #
      # === Create the programs-version yaml file
      #
      # Invocation example:
      #
      #   SM --create-programs-version-yaml-file
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?yaml(-|_| )?file$/i,
           /^-?-?generate(-|_| )?file$/i,
           /^-?-?generate(-|_| )?programs(-|_| )?version$/i
        orev "Next generating the #{sfile('programs_version.yml')} #{rev}file."
        action(:generate_version_file)
        exit_program
      # ===================================================================== #
      # === SM --generate-version-file
      #
      # This variant is a bit less verbose then the one ^^^ above.
      # ===================================================================== #
      when /^-?-?generate(-|_| )?version(-|_| )?file/,
           /^-?-?create(-|_| )?version(-|_| )?file/,
           /^-?-?gversionfile/
        action(:generate_version_file)
        exit_program
      # ===================================================================== #
      # === Show all registered keys
      #
      # Show which keys are registered.
      #
      # Invocation examples:
      # 
      #   SM --allregistered?
      #   SM --show_registered_keys
      #   SM allregistered?
      #   SM  show_registered_keys
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_REGISTERED_KEYS
        show_registered_keys
        exit_program
      else # else tag
        # =================================================================== #
        # else no given argument was found, so we will handle this event
        # next.
        # =================================================================== #
        if i # check that we still have some kind of input here
          # ================================================================= #
          # Get the variant without any numbers or '-' hyphens next. We
          # will apply .to_s here, just to not have to worry about Symbols.
          # ================================================================= #
          entry_without_numbers_and_hyphen = i.to_s.delete('-').gsub(/[0-9]/,'')
          # ================================================================= #
          # We may still have input such as "--plasma1". The next code
          # block will account for that situation.
          #
          # Invocation examples for this:
          #
          #   ry --plasma1
          #   ry --plasma2
          #   ry --plasma3
          #
          # ================================================================= #
          if ARRAY_SPECIAL_COMPONENT_CONTAINING_MANY_SUBCOMPONENTS.include?(entry_without_numbers_and_hyphen) and
             (i =~ /\d+$/) # ← This line was added April 2021 to distinguish between "-lxqt" and "-lxqt1".
            # ================================================================= #
            # The next code should return a single entry.
            # ================================================================= #
            entry = try_to_return_a_special_compile_component(i.delete('-'))
            determine_which_programs_to_compile(entry)
          elsif i.start_with?('--')
            warning_argument_was_not_found(i)
          end
        end
      end
    end
  end

end; end; end
